/*
  @(#)gti_whip.c	1.77
*/

/****************************************************************************/
/* (C) Copyright 1998-2003 by GlobeSpan Semiconductor Inc                   */
/*                                                                          */
/* File name: GTI_WHIP.c                          Created: April 06, 1998   */
/* Author: Dmitry Katz                                                      */
/*                                                                          */
/* Description: This file implements Serial Communication Protocol. The     */
/*              Protocol implemented here is responsible for communication  */
/*              errors detection, retransmittion, hot line exchange, e.t.c. */
/****************************************************************************/

#include "gti.h"
#include "gti_rel.h"
#include "gtiextn.h"

#ifdef GTI_DEBUG
    #include "p3hw.h"
    #include "global.h"
    #include "extern.h"
#endif

#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC))
    #include "gti_fw_rel.h"
#endif

#ifdef GTI_PROD_DMT
    #include "gti_eoc.h"
#endif

#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC))
    #include "nucleus.h"
    #include "dsl_typedef.h"
    extern SDSLConfigParams    DSLConfigParams[];
#endif

/* Global Definitions                                                       */
/* ------------------                                                       */
#ifdef GTI_PROD_DMT
    RXEOC WhipEocBuffer;
    GTIEOC_CMD WhipEocCmd;
#endif

#ifdef GTI_PROD_DMT
    void WhipFillEocExtBuffer(GTIEOC_CMD *pEoc);
    void GTIEocServiceCall(void);
#endif

/* Host Interface Protocol function prototypes                              */
/* -------------------------------------------                              */
#ifdef GTI_ISOS
    GTI_WORD wsp_length;
#endif


#ifndef GTI_USE_INTERNAL_TARGET
    void WhipInitSM(void);
    void WhipPrtclSendControlMsg (int Opcode, GTI_BYTE id);
#ifdef GTI_ISOS
    extern void Gti_WhipTx(GTI_BYTE GTI_ADDRESS_QUAL *array, short length);
    void Gti_WhipRx(GTI_BYTE * ptrSymbArray, GTI_WORD nLength);
#else
    void Gti_WhipTx(GTI_BYTE GTI_ADDRESS_QUAL *array, short length);
#endif

    void WhipRetransmitLastMsg(GTI_BYTE id);
#endif

void WhipSendMsg(GTI_BYTE opcode, GTI_BYTE type, void *pMsgBody, GTI_BYTE length);

#ifdef GTI_DEBUG
    extern void Gti_WhipRxNotify(GTI_BYTE *, GTI_WORD);
#else
    void Gti_WhipRxNotify(GTI_BYTE *, GTI_WORD);
#endif

int  WhipDispatchMsg(GTI_XCVR_INFO GTI_GLOBAL_QUAL *pXcvrPtr, GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
void WhipDebugTest(int nLength, GTI_BYTE *pArray);

/* Host Interface Dispatcher function prototypes                            */
/* -------------------------------------------                              */
int WhipDispatchHotLine        (GTI_BYTE *pOutMsg);
int WhipDispatchRegistration   (GTI_BYTE *pOutMsg);
int WhipReadConstellation      (GTI_BYTE *pOutMsg);

int WhipDispatchBreakpntInfo   (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);

#ifdef GTI_TRACE
int WhipDispatchReadTrace      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchReadTimeStamp  (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
#endif

int WhipDispatchSystemStatus   (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchReadWriteMemory(GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchExecCmd        (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchInitXcvr       (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchEocCommand     (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchAnyStatus      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchActConfig      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchBertTest       (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipSetGenericParameter    (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipDispatchRandomReadMem  (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipCustomMessage          (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
#ifdef GTI_EXTENDED_TRACE
int WhipSetReportCoefTrig      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipSetReportCoefData      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
int WhipGetReportCoefData      (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
#endif
#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
int WhipLocalBustTest          (GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
#endif
#if (defined (GTI_DSPOCTANE)  && (defined(GTI_USE_DMAXFR)|| defined(GTI_USE_XDPXFR) ) )
int WhipDispatchLbusTrace      (GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg);
#endif

/*       Internal helper functions                                          */
/* -----------------------------------                                      */
#ifdef GTI_DEBUG
    extern int WhipInternalDispatcher(GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
    extern int SetInternalBreakpoints(GTI_BYTE *pData, GTI_BYTE *pDataOut);
#endif

#ifdef GTI_DEBUG
extern int p_WhipReadWriteHostMemory(char bRead, GTI_BYTE *pMsg, GTI_BYTE *pOutMsg);
#endif

#if defined(GTI_DSPQUAD)
GTI_WORD AccessDmaExternalRam(GTI_BYTE bRead, GTI_WORD wSeg, GTI_WORD wOfs,
                              GTI_WORD wSize, GTI_BYTE *pBuffer);
#endif

GTI_WORD WhipSStatusXCVR(GTI_WORD nStatId, void *pBuffer,
                         GTI_WORD nLength, GTI_BYTE nChanID);

void ReadDspBlock(  GTI_XCVR_INFO *pSys,
                    GTI_BYTE bMemType, GTI_BYTE bAddress,
                    GTI_BYTE GTI_ADDRESS_QUAL *pBlock,
                    GTI_WORD nNumber, GTI_BYTE bChannel);

void WriteDspBlock( GTI_XCVR_INFO *pSys,
                    GTI_BYTE bMemType,
                    GTI_BYTE bAddress,
                    GTI_BYTE GTI_ADDRESS_QUAL *pBlock,
                    GTI_WORD nNumber, GTI_BYTE bChannel);


/* KJP */
extern 
GTI_WORD ExecuteXCVR_USB (GTI_XCVR_INFO *pXcvrInfo,
                           GTI_WORD ipuReady,
                           GTI_WORD nextState);

extern
GTI_WORD GTISetParamXCVR_USB(GTI_XCVR_INFO *pXcvrInfo, struct PARAM_XCVR_ARRAY *ParamTbl);
/* KJP */


#ifndef GTI_USE_INTERNAL_TARGET
/****************************************************************************/
/* Procedure: WhipInitSM                                  Created: 04.06.98 */
/* ----------------------------                                             */
/* Description: Intializes the WHIP state machine                           */
/* -------------------------------------------------------------------------*/
void WhipInitSM(void)
{
    Whip_state = L2_STATE_IDLE;
    Whip_rcv_bufidx = 0;
    Whip_chksum = 0;
    Whip_xmt_total_idx = 0;
    Whip_xmt_sentb_idx = 0;
}
#endif

/****************************************************************************/
/* Procedure: Gti_WhipInit                                Created: 04.06.98 */
/* ----------------------------                                             */
/* Description: The procedure initializes all the HIP variables.            */
/* -------------------------------------------------------------------------*/
void
Gti_WhipInit(void)
{
#ifdef GTI_TRACE
    unsigned int i, k;
#endif

#ifdef GTI_PROD_DMT
    WhipEocEnabled = GTI_FALSE;
#endif

    WhipMessageID = 0;
    WhipCurrentSessionID = 0;

#ifndef GTI_USE_INTERNAL_TARGET

    WhipInitSM();
    WhipMessagesReceived = 0;
    WhipLastSOMidx = 0;
    WhipRexmtCounter = 0;
#endif

    WhipGtiXcvrPtr = GTIXcvr;
    WhipActiveXcvr = 0;
    memcpy(WhipTargetName, GTI_PRODUCT, sizeof(WhipTargetName));

#ifdef GTI_TRACE
    for(i=0; i<L2_MAX_BREAKPOINTS; i++)
    {
        Whip_BrkPntData[i].bState = 0;
        Whip_BrkPntData[i].wType = 0xffff;
        Whip_BrkPntData[i].bPage = 0;
        Whip_BrkPntData[i].wAddr = 0;
        Whip_BrkPntData[i].wOffs = 0;

        for(k=0; k < sizeof(Whip_BrkPntData[i].aCode); k++)
        {
            Whip_BrkPntData[i].aCode[k] = 0;
        }
    }
#endif
}

void Gti_WhipAcceptMsg(GTI_BYTE *InpMsg)
{
    int nMsgLength;
    GTI_BYTE OutMsg[L2_MAX_MSG_LENGTH + 1];

    if((InpMsg[L2_MSG_BODY_OFS - 1] & 0xff) == L2_REGISTRATION_MSG_OPC)
    {
         WhipCurrentSessionID = InpMsg[L2_HEADER_SID_OFS];
    }
    else
    if((InpMsg[L2_MSG_BODY_OFS - 1] & 0xff) == L2_TRAFFIC_DEBUG_OPC)
    {
        WhipDebugTest(InpMsg[L2_HEADER_LEN_OFS] - 3, &InpMsg[L2_MSG_BODY_OFS]);
        return;
    }

    nMsgLength = WhipDispatchMsg(WhipGtiXcvrPtr,
        &InpMsg[L2_MSG_BODY_OFS - 1], &OutMsg[L2_MSG_BODY_OFS - 1]);

    if(nMsgLength >= 0)
    {
        WhipSendMsg(OutMsg[L2_MSG_BODY_OFS - 1], 0,
            &OutMsg[L2_MSG_BODY_OFS], (GTI_BYTE)nMsgLength);
    }
}

/****************************************************************************/
/* Procedure: Gti_WhipRxHandler                          Created: 04.06.98  */
/* ----------------------------                                             */
/* Description: The function analizes the received message. If the message  */
/*               not valid - NACK is sent to the other side. If the message */
/*               valid and the ACK is required, the ACK is sent to the oth  */
/*               er side.                                                   */
/* ------------------------------------------------------------------------ */
/* void WhipAnalyzeMsg (void)                                               */
void Gti_WhipRxHandler(GTI_BYTE *pBuffer, GTI_WORD wLength)
{
#if ( (!defined(GTI_USE_INTERNAL_TARGET)) && (!defined(GTI_HOSTVSC)) )
    int nMsgLength, nBadMsg;

    GTI_BYTE InpMsg[L2_MAX_MSG_LENGTH + 1];  /* Create Local message buffer in our stack */

    WhipMessagesReceived++;

    nMsgLength = (int)(wLength - 5);

    if (nMsgLength <= 0)
        return;

    wLength = 0;
    wsp_length = 0;

    memcpy(InpMsg, pBuffer, L2_MAX_MSG_LENGTH);

    nBadMsg = GTI_FALSE;

    /* First check the validity of  the message */

    /* last byte in a buffer is a checksum, valid checksum can be 0 or DLE  */
    if((InpMsg[L2_MSG_BODY_OFS + nMsgLength - 1] != 0) &&
       (InpMsg[L2_MSG_BODY_OFS + nMsgLength - 1] != L2_DLE))
        nBadMsg = GTI_TRUE;

    if (InpMsg[L2_HEADER_LEN_OFS] != nMsgLength)
         nBadMsg = GTI_TRUE;

    switch (InpMsg[L2_MSG_BODY_OFS - 1] & 0xff)
    {
    case L2_ACK_OPC:
        return;

    case L2_NACK_OPC:
        if(!nBadMsg)
            WhipRetransmitLastMsg(InpMsg[L2_HEADER_MID_OFS]);
        return;

    default:
        if(nBadMsg)
        {
            WhipPrtclSendControlMsg(L2_NACK_OPC, InpMsg[L2_HEADER_MID_OFS]);
            return;
        }
        else
        {
            /* A message is received. Now send ACK if  needed               */
            if((InpMsg[L2_HEADER_TYP_OFS] >> 0x04) == 1)
                WhipPrtclSendControlMsg(L2_ACK_OPC, InpMsg[L2_HEADER_MID_OFS]);

            Gti_WhipAcceptMsg(InpMsg);
        }
        return;
    }

#else

    Gti_WhipAcceptMsg(pBuffer); /* Accept a message without control */

#endif
}

/****************************************************************************/
/* Procedure: WhipDispatchMsg                             Created: 04.06.98 */
/* --------------------------------                                         */
/* Description: This function dispatches messages according to their opcode */
/*              and performs appropriate actions.                           */
/* ------------------------------------------------------------------------ */
int WhipDispatchMsg(GTI_XCVR_INFO GTI_GLOBAL_QUAL *pXcvrPtr,
        GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    WhipGtiXcvrPtr = pXcvrPtr;

    switch(pInpMsg[L2_MSG_OPCODE_OFS] & 0xff)
    {
    case L2_REGISTRATION_MSG_OPC:
        return WhipDispatchRegistration(pOutMsg);

    case L2_HOT_LINE_MSG_OPC:
        return WhipDispatchHotLine(pOutMsg);

    case L2_READMEMORY_OPC:
    case L2_WRITEMEMORY_OPC:
    case L2_WRITEMEMORY_BDCST_OPC:
        return WhipDispatchReadWriteMemory(pInpMsg, pOutMsg);

    case L2_SYSTEM_STATUS_OPC:
        return WhipDispatchSystemStatus(pInpMsg, pOutMsg);

    case L2_EXEC_COMMAND_OPC:
        return WhipDispatchExecCmd(pInpMsg, pOutMsg);

    case L2_INIT_XCVR_OPC:
        return WhipDispatchInitXcvr(pInpMsg, pOutMsg);

#ifdef GTI_TRACE
#if (!defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR) )
    case L2_READ_TRACE_BUFFER_OPC:
        return WhipDispatchReadTrace(pInpMsg, pOutMsg);

    case L2_READ_TIMESTAMP_OPC:
        return WhipDispatchReadTimeStamp(pInpMsg, pOutMsg);
#endif
#endif

    case L2_BREAKPOINT_INFO_OPC:
        return WhipDispatchBreakpntInfo(pInpMsg, pOutMsg);

#if (defined (GTI_DSPOCTANE)  && (defined(GTI_USE_DMAXFR)|| defined(GTI_USE_XDPXFR) ) )
    case L2_READ_TRACE_BUFFER_OPC:
    case L2_LBUS_TRACE_OPC:
        return WhipDispatchLbusTrace(pInpMsg, pOutMsg);
#endif

    case L2_RANDOM_MEMREAD_OPC:
        return WhipDispatchRandomReadMem(pInpMsg, pOutMsg);

    case L2_READ_CONSTELLATION_OPC:
        return WhipReadConstellation(pOutMsg);

    case L2_EOC_COMMAND_OPC:
        return WhipDispatchEocCommand(pInpMsg, pOutMsg);

    case L2_ANY_STATUS_OPC:
        return WhipDispatchAnyStatus(pInpMsg, pOutMsg);

    case L2_ACTIVE_CONFIG_OPC:
        return WhipDispatchActConfig(pInpMsg, pOutMsg);

    case L2_BERT_TEST_OPC:
        return WhipDispatchBertTest(pInpMsg, pOutMsg);

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    case L2_TEST_LOCAL_BUS_OPC:
        return WhipLocalBustTest(pInpMsg, pOutMsg);
#endif
    case L2_SET_GENERIC_PARAM_OPC:
        return WhipSetGenericParameter(pInpMsg, pOutMsg);

#ifdef GTI_EXTENDED_TRACE
    case L2_SET_REP_COEF_TRIG_OPC:
        return WhipSetReportCoefTrig(pInpMsg, pOutMsg);

    case L2_SET_REP_COEF_DATA_OPC:
        return WhipSetReportCoefData(pInpMsg, pOutMsg);

    case L2_GET_REP_COEF_DATA_OPC:
        return WhipGetReportCoefData(pInpMsg, pOutMsg);
#endif

    case L2_CUSTOM_MSG_OPC:
        return WhipCustomMessage(pInpMsg, pOutMsg);

    default:
#ifdef GTI_DEBUG
        return WhipInternalDispatcher(pInpMsg, pOutMsg);
#else
        return -1;
#endif
    }
}

/******************************************************************************/
/* Procedure: WhipDebugTest                                 Created: 12.24.99 */
/* --------------------------------                                           */
/* Description: The routine serves for debug purpose only. The received mess  */
/*              is analized and reflected to the PC side.                     */
/* -------------------------------------------------------------------------- */
void WhipDebugTest(int nLength, GTI_BYTE *pArray)
{
    int nSendLoop;

    nSendLoop = pArray[0];

    while(nSendLoop--)
    {
        WhipSendMsg(L2_TRAFFIC_DEBUG_OPC, 0, pArray, (GTI_BYTE)nLength);
    }
}

#ifndef GTI_DSPOCTANE
/******************************************************************************/
/* Procedure: ReadDspBlock                                  Created: 09.14.98 */
/* --------------------------------                                           */
/* Description: The procedure reads block from the DSP memory to a specified  */
/*              buffer. This is a high level function to read any DSP location*/
/*              and it uses all the low level drivers calls.                  */
/* -------------------------------------------------------------------------- */
void ReadDspBlock(GTI_XCVR_INFO *pSys,
                 GTI_BYTE bMemType, GTI_BYTE bAddress,
                 GTI_BYTE GTI_ADDRESS_QUAL *pBlock,
                 GTI_WORD nNumber, GTI_BYTE bChannel)
{
    int i;
    GTI_WORD result;
    GTI_BYTE wordSize, bitSize;
    GTI_BYTE lowout, medout, highout;

    wordSize = Gti_GetWordSize(bMemType);
    bitSize  = Gti_GetWordBitSize(bMemType);

    (void)bChannel;

#if( defined(GTI_CRAM_16BIT) )
    if(wordSize == GTI_SPECIAL_CRAM) /* a special case, means: Size = 2, memtype -= 20 */
    {
        bMemType -= 0x20;
        wordSize = 2;
    }
#endif

    /* Configure for Reading Internal Register */
    CONFIGURE_DSP(pSys, bChannel, GTI_READ_BLOCK_MODE);
    WRITE_DSP_ADDR(pSys, bChannel, bAddress, bMemType | GTI_READ_DSP);

    switch(wordSize)
    {
    case 6:
    case 4:
    case 2:
        READ_DSP_BLOCK(pSys, bChannel, GTI_DATAL_DSP, nNumber, pBlock);
        break;
    case 3:
        READ_DSP_EXT_BLOCK(pSys, bChannel, GTI_EXT_DATA_DSP, nNumber, pBlock);
        break;
    default:
        break;
    }

    for(i=0; (i + wordSize) <= nNumber; i += wordSize)
    {
        switch(bitSize)
        {
        case 20:
            lowout =  pBlock[i+0];
            medout =  pBlock[i+1];
            highout = pBlock[i+2];

#if (!defined(GTI_DSPMULTI))
            if(bMemType == GTI_BSRAM) /* special case */
            {
                pBlock[i+0] = (GTI_BYTE)(((medout  << 0x02) & 0xfc) | ((lowout >>  0x06) & 0x03));
                pBlock[i+1] = (GTI_BYTE)(((highout << 0x02) & 0xfc) | ((medout >>  0x06) & 0x03));
                pBlock[i+2] = (GTI_BYTE)(((lowout  << 0x02) & 0x0c) | ((highout >> 0x06) & 0x03));
            }
            else
            {
                pBlock[i+0] = (GTI_BYTE)(((lowout >> 0x04) & 0x0F) | ((medout  << 0x04) & 0xF0));
                pBlock[i+1] = (GTI_BYTE)(((medout >> 0x04) & 0x0F) | ((highout << 0x04) & 0xF0));
                pBlock[i+2] = (GTI_BYTE)((highout >> 0x04) & 0x0F);
            }
#else
            pBlock[i+0] = (GTI_BYTE)(((lowout >> 0x04) & 0x0F) | ((medout  << 0x04) & 0xF0));
            pBlock[i+1] = (GTI_BYTE)(((medout >> 0x04) & 0x0F) | ((highout << 0x04) & 0xF0));
            pBlock[i+2] = (GTI_BYTE)((highout >> 0x04) & 0x0F);
#endif
            break;

        case 12:
            result = (GTI_WORD)(MAKEWORD(pBlock[i+0], pBlock[i+1]));
            result = (GTI_WORD)(result >> 0x04);
            pBlock[i+0] = (GTI_BYTE) (result & 0x00ff);
            pBlock[i+1] = (GTI_BYTE) ((result >> 0x08) & 0x00ff);
            break;

        case 0:
        default:
            return ;
        }
    }

    (void)pSys; /* kill the warning */
    return ;
}

/******************************************************************************/
/* Procedure: WriteDspBlock                                 Created: 09.14.98 */
/* --------------------------------                                           */
/* Description: The procedure writes block from a specified buffer to the DSP */
/*              memory. This is a high level function to write any DSP        */
/*              location and it uses all the low level drivers calls.         */
/* -------------------------------------------------------------------------- */
void WriteDspBlock(GTI_XCVR_INFO *pSys,
                  GTI_BYTE bMemType,
                  GTI_BYTE bAddress,
                  GTI_BYTE GTI_ADDRESS_QUAL *pBlock,
                  GTI_WORD nNumber, GTI_BYTE bChannel)
{
    int i;
    GTI_BYTE wordSize;
    GTI_BYTE lowout,medout,highout;
    GTI_WORD wMode;

    bChannel = bChannel;
    wordSize = Gti_GetWordSize(bMemType);

#if( defined(GTI_CRAM_16BIT) )
    if(wordSize == GTI_SPECIAL_CRAM) /* a special case, means: Size = 2, memtype -= 20 */
    {
        bMemType -= 0x20;
        wordSize = 2;
    }
#endif

    CONFIGURE_DSP(pSys, bChannel, GTI_DMA);
    WRITE_DSP_ADDR(pSys, bChannel, bAddress, bMemType);

    if(wordSize == 3)
        wMode = GTI_EXT_DATA_DSP;
    else
        wMode = GTI_DATAL_DSP;

    if(bMemType == GTI_BSRAM) /* special case */
    {
        for(i=0; (i + wordSize) <= nNumber; i += wordSize)
        {
            lowout = (GTI_BYTE)(((pBlock[i+2] >> 0x06) & 0x03) | ((pBlock[i] << 0x02) &  0xc0));
            medout = (GTI_BYTE)(((pBlock[i] >> 0x06) & 0x03)  | ((pBlock[i+1] << 0x02) & ~0x03));
            highout= (GTI_BYTE)(((pBlock[i+1] >> 0x06) & 0x03)  | ((pBlock[i+2] << 0x02) & ~0x03));

            pBlock[i+0] = lowout;
            pBlock[i+1] = medout;
            pBlock[i+2] = highout;
        }
    }

    WRITE_DSP_BLOCK(pSys, bChannel, wMode, nNumber, pBlock);

    (void)pSys;
    return ;
}

#endif

GTI_WORD WhipSStatusXCVR(GTI_WORD nStatId, void *pBuffer,
                         GTI_WORD nLength, GTI_BYTE nChanID)
{
    GTI_WORD wLength;
#ifdef GTI_BIG_ENDIAN
    GTI_WORD i, *pWordPtr = (GTI_WORD *)pBuffer;
#endif

#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC))

    GTI_WORD    dspState=0;

    /*
    ** If WHIP is disabled, return zeros for the GTI_COUNTERS.
    */
    if((DSLConfigParams[0].bWhipEnable == GTI_FALSE) &&
       ((nStatId == GTI_COUNTERS) || (nStatId == GTI_FAILURES)))
    {
        wLength = 0;
    }
    else
    {
        if((nStatId == GTI_COUNTERS) || (nStatId == GTI_FAILURES))
        {
            /* Check to see the if we are in DATA_MODE */
            (void)Gti_SStatusXCVR(WhipGtiXcvrPtr, GTI_OP_STATE, &dspState, sizeof(dspState), nChanID);
            if(dspState != GTI_DATA_OP)
            {
                wLength = 0;
            }
            else
            {
                wLength  = Gti_SStatusXCVR(WhipGtiXcvrPtr, nStatId, pBuffer, nLength, nChanID);
            }
        }
        else
        {
            wLength  = Gti_SStatusXCVR(WhipGtiXcvrPtr, nStatId, pBuffer, nLength, nChanID);
        }
    }
#else /* !(Viking && Embedded Host) */
    wLength  = Gti_SStatusXCVR(WhipGtiXcvrPtr, nStatId, pBuffer, nLength, nChanID);
#endif



#ifdef GTI_BIG_ENDIAN
    switch(nStatId)
    {
    case GTI_FW_REL:
    case GTI_LBUS_FW_VERSION:
    case GTI_INTERNAL_SAS:
    case GTI_BIT_LOADING_UP:
    case GTI_BIT_LOADING_DN:
    case GTI_LCL_SNR_MARGIN:
        break;
    default:
        for(i=0; i < wLength/sizeof(GTI_WORD); i++)
            SWAP_WORD(&(pWordPtr[i]), &(pWordPtr[i]));
        break;
    }
#endif

    return wLength;
}

/****************************************************************************/
/* Procedure: WhipDispatchRegistration                    Created: 04.06.98 */
/* --------------------------------                                         */
/* Description: This function dispatches registration                       */
/* ------------------------------------------------------------------------ */
int WhipDispatchRegistration(GTI_BYTE *pOutMsg)
{
    pOutMsg[L2_MSG_OPCODE_OFS] = L2_REGISTRATION_MSG_OPC;
    pOutMsg[1] = 0;

    /* fill the Target name, version and Platform name into the message    */
    memcpy(&pOutMsg[2], WhipTargetName, 6);
#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC))
    memcpy(&pOutMsg[8], pGtiFwRel, 6);
#else
    memcpy(&pOutMsg[8], RELEASE, 6);
#endif

    (void)WhipSStatusXCVR(GTI_DSP_VERSION, &pOutMsg[14], 6, 0);
    memcpy(&pOutMsg[20], GTI_DSP_TYPE_STRING, 6);

    pOutMsg[26] = GTI_TOTAL_NUM_OF_XCVRS;
    pOutMsg[27] = WhipActiveXcvr;
    pOutMsg[28] = GTI_TOTAL_NUM_OF_XCVRS;
    pOutMsg[29] = 0;

    return 30;
}

/****************************************************************************/
/* Procedure: WhipDispatchHotLine                         Created: 04.06.98 */
/* --------------------------------                                         */
/* Description: This function replies to the hotline message                */
/* ------------------------------------------------------------------------ */
int
WhipDispatchHotLine(GTI_BYTE *pOutMsg)
{
    pOutMsg[0] = L2_HOT_LINE_MSG_OPC;
    return 0;
}

/****************************************************************************/
/* Procedure: WhipDispatchInitXcvr                        Created: 04.06.98 */
/* --------------------------------                                         */
/* Description: This function resets the DSP chip                           */
/* ------------------------------------------------------------------------ */
int
WhipDispatchInitXcvr(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    pOutMsg[L2_MSG_OPCODE_OFS] = L2_INIT_XCVR_OPC;

    switch(pInpMsg[L2_INITXCVR_TYPE_OFS])
    {
    case INIT_XCVR_RESET_DSP:
        ResetDSP(WhipGtiXcvrPtr, GTI_ALL_XCVR_CORES);      /* Reset Star function */
        Gti_InitXCVR(WhipGtiXcvrPtr);
        return 0;

#if (defined(GTI_DSPQUAD) || defined (GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case INIT_XCVR_RESET_CORE_DSP:
        Gti_InitXCVR(WhipGtiXcvrPtr);
        return 0;

    case INIT_XCVR_RESET_CHIP_DSP:
        Gti_InitCHIP(WhipGtiXcvrPtr->pChipInfo);
        return 0;
#endif

    case INIT_XCVR_INT_DSP:

        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_ON)
            InitSTARInterrupt(WhipGtiXcvrPtr, GTI_TRUE);
        else
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_OFF)
            InitSTARInterrupt(WhipGtiXcvrPtr, GTI_FALSE);
        else
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_CLR)
            ClearSTARInterrupt(WhipGtiXcvrPtr, GTI_ALL_XCVR_CORES);
        return 0;

#if (defined(GTI_DSPFRAMER) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
    case INIT_XCVR_INT_FRMR:
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_ON)
        {
#ifdef GTI_PROD_H2B1Q
            InitFRAMERInterrupt(WhipGtiXcvrPtr, GTI_RXCHB_RXCHA_TXCHB_TXCHA_INT);
#endif
#ifdef GTI_PROD_HDSL2
            InitFRAMERInterrupt(WhipGtiXcvrPtr, GTI_FRM_ENABLE_INT);
#endif
#ifdef GTI_PROD_MSDSL
            InitFRAMERInterrupt(WhipGtiXcvrPtr, GTI_RX_1_TX_1_INT);
#endif
#ifdef GTI_PROD_N2B1Q
            InitFRAMERInterrupt(WhipGtiXcvrPtr, GTI_FRM_ENABLE_INT);
#endif
        }
        else
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_OFF)
            InitFRAMERInterrupt(WhipGtiXcvrPtr, GTI_FRM_DISABLE_INT);
        else
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_CLR)
            ClearFRAMERInterrupt(WhipGtiXcvrPtr, GTI_FRAMER_INT_CLR);
        return 0;
#endif
#ifdef GTI_DEBUG
    case INIT_XCVR_INT_HOST:
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_ON)
        {
            InitInt0();
#ifndef GTI_DSPQUAD  /* Trinity does not use "B" interrupt */
            InitInt1();
#endif
        }
        else
        if(pInpMsg[L2_INITXCVR_ACTN_OFS] == INIT_XCVR_INT_OFF)
        {
            DisableInt0();
            DisableInt1();
        }
        return 0;
#endif /* GTI_DEBUG */
    default:
        return -1;
    }
}

/******************************************************************************/
/* Procedure: WhipDispatchSystemStatus                      Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: This function analyzes Status message and fills up the reply. */
/* -------------------------------------------------------------------------- */
int
WhipDispatchSystemStatus(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_BYTE status, length;
    GTI_BYTE bStatBuff[L2_MAX_MSGBODY_LENGTH];
    int i, limit;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_SYSTEM_STATUS_OPC;

    limit = L2_MAX_MSGBODY_LENGTH;

    pOutMsg[L2_SSTATUS_CHNL_OFS] = pInpMsg[L2_SSTATUS_CHNL_OFS];
    pOutMsg[L2_SSTATUS_NUMB_OFS] = 0;
    i = 0;
    while((limit > 0) && (pOutMsg[L2_SSTATUS_NUMB_OFS] < pInpMsg[L2_SSTATUS_NUMB_OFS]))
    {
        status = pInpMsg[L2_SSTATUS_LIST_OFS + pOutMsg[L2_SSTATUS_NUMB_OFS]];

        length = (GTI_BYTE)WhipSStatusXCVR((GTI_WORD)status, bStatBuff,
            (GTI_WORD)limit, pOutMsg[1]);

        pOutMsg[L2_SSTATUS_LIST_OFS + i] = status;
        pOutMsg[L2_SSTATUS_LIST_OFS + i + 1] = length;
        memcpy(&pOutMsg[L2_SSTATUS_LIST_OFS + i + 2], bStatBuff, length);

        limit -= length + 2;
        pOutMsg[L2_SSTATUS_NUMB_OFS]++;
        i += length + 2;
    }

    return i + 2;
}

/******************************************************************************/
/* Procedure: WhipDispatchAnyStatus                         Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: This function analizes Status message and fills up the reply. */
/*              The function fills a buffer for one status only.              */
/* -------------------------------------------------------------------------- */
int WhipDispatchAnyStatus(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{

#ifdef GTI_USE_GLOBAL_BUFFER_FOR_LARGE_STATUS
    GTI_BYTE Buffer[1024]; /* need extra memory for VDSL SQ Status */
#else
    GTI_BYTE Buffer[512];
#endif

    GTI_WORD wFirst, wTotal;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_ANY_STATUS_OPC;

    pOutMsg[L2_TW_ANYSTAT_CHNL_OFS] = pInpMsg[L2_WT_ANYSTAT_CHNL_OFS];
    pOutMsg[L2_TW_ANYSTAT_FST_OFS] = pInpMsg[L2_WT_ANYSTAT_FST_OFS];
    pOutMsg[L2_TW_ANYSTAT_FST_OFS + 1] = pInpMsg[L2_WT_ANYSTAT_FST_OFS + 1];
    pOutMsg[L2_TW_ANYSTAT_STID_OFS] = pInpMsg[L2_WT_ANYSTAT_STID_OFS];

    wFirst = (GTI_WORD)(MAKEWORD(pInpMsg[L2_WT_ANYSTAT_FST_OFS],
        pInpMsg[L2_WT_ANYSTAT_FST_OFS+1]));

    wTotal = WhipSStatusXCVR( pInpMsg[L2_WT_ANYSTAT_STID_OFS], Buffer,
        sizeof(Buffer), pInpMsg[L2_WT_ANYSTAT_CHNL_OFS]);

    if(wTotal > 0)
    {
        pOutMsg[L2_TW_ANYSTAT_LEN_OFS] = pInpMsg[L2_WT_ANYSTAT_LEN_OFS];

        if(pOutMsg[L2_TW_ANYSTAT_LEN_OFS] > (wTotal - wFirst))
            pOutMsg[L2_TW_ANYSTAT_LEN_OFS] = (GTI_BYTE)(wTotal -  wFirst);

        memcpy(&pOutMsg[L2_TW_ANYSTAT_BUF_OFS],
            &Buffer[wFirst],  pOutMsg[L2_TW_ANYSTAT_LEN_OFS]);

        SWAP_WORD(&pOutMsg[L2_TW_ANYSTAT_TOTL_OFS], &wTotal);

        return L2_TW_ANYSTAT_BUF_OFS + pOutMsg[L2_TW_ANYSTAT_LEN_OFS];
    }
    else
        return -1;
}

#ifdef GTI_EXTENDED_TRACE
/****************************************************************************/
/* Procedure: WhipSetReportCoefTrig                       Created: 09.14.98 */
/* --------------------------------                                         */
/* Description: This function sets Report Coefficient Triggers Data Base    */
/*              and raises a flag that work should be started               */
/* -----------------------------------------------------------------------  */
int WhipSetReportCoefTrig(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    int i,  k, m, nLoopsRead;
    GTI_RCF_ADDRESS_DB *pAddrDb;
    GTI_RCF_DATA_DB *pDataDB;
    GTI_WORD wData;
    GTI_BYTE bBytes;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_SET_REP_COEF_TRIG_OPC;

    nLoopsRead = pInpMsg[L2_TW_RC_LOOP_NUMBER];
    WhipGtiXcvrPtr->pChipInfo->pSystInfo->ExtTrace.Rcf_LoopsRead = pInpMsg[L2_TW_RC_LOOP_NUMBER];

    while(nLoopsRead)
    {
        for(i = 0; i < GTI_RC_MAX_OVERLAYED_PLOTS; i++)
        {
            pAddrDb = &(WhipGtiXcvrPtr->pChipInfo->pSystInfo->ExtTrace.AddrDb[i]);
            pDataDB = &(WhipGtiXcvrPtr->pChipInfo->pSystInfo->ExtTrace.DataDB[i]);

            if ((pAddrDb->address[i].mem_type >= GTI_CRAM0) &&
                (pAddrDb->address[i].mem_type <= GTI_CRAM5))
            {
                CONFIGURE_DSP(WhipGtiXcvrPtr, pAddrDb->chip_id, GTI_NO_AI);

                for(k=0; k < pAddrDb->size; k++)
                {
                    WRITE_DSP_ADDR(WhipGtiXcvrPtr, pAddrDb->chip_id, 0x53, GTI_IREGA);
                    WRITE_DSP_WORD(WhipGtiXcvrPtr, pAddrDb->chip_id, 0xff, 0xff);

                    WRITE_DSP_ADDR(WhipGtiXcvrPtr, pAddrDb->chip_id, 0x53, GTI_IREGA);
                    WRITE_DSP_WORD(WhipGtiXcvrPtr, pAddrDb->chip_id,
                        pAddrDb->address[k].address, (pAddrDb->address[k].mem_type - GTI_CRAM0));

                    for(m=0; m < 3; m++)
                    {
                        WRITE_DSP_ADDR(WhipGtiXcvrPtr, pAddrDb->chip_id, 0x10,  GTI_IREGA|GTI_READ_DSP);
                        wData = READ_DSP_WORD(WhipGtiXcvrPtr, pAddrDb->chip_id);

                        if(wData & 0x8000)
                        {
                            WRITE_DSP_ADDR(WhipGtiXcvrPtr, pAddrDb->chip_id, 0x11,  GTI_IREGA|GTI_READ_DSP);
                            pDataDB->data[k] = READ_DSP_WORD(WhipGtiXcvrPtr, pAddrDb->chip_id);
                            pDataDB->fail[k] = GTI_TRUE;
                            break;
                        }
                    }

                    if(m >= 3)
                    {
                        pDataDB->data[k] = 0;
                        pDataDB->fail[k] = GTI_FALSE;
                    }
                }
            }
            else
            {
                for(k=0; k < pAddrDb->size; k++)
                {
                    bBytes = Gti_GetWordBitSize(pAddrDb->address[k].mem_type);
                    bBytes = (bBytes == 20) ? 3 : bBytes / 8;

                    pDataDB->data[k] = 0;
                    pDataDB->fail[k] = GTI_TRUE;

                    ReadDspBlock(
                        WhipGtiXcvrPtr,
                        pAddrDb->address[k].mem_type,
                        pAddrDb->address[k].address,
                        (GTI_BYTE GTI_ADDRESS_QUAL *)&(pDataDB->data[k]),
                        bBytes,
                        pAddrDb->chip_id);
                }
            }
        }

        nLoopsRead--;

    }

    return 0;
}

/****************************************************************************/
/* Procedure: WhipSetReportCoefData                       Created: 09.14.98 */
/* --------------------------------                                         */
/* Description: This function sets Report Coefficient Data base and starts  */
/*              the feature execution.                                      */
/* ------------------------------------------------------------------------ */
int WhipSetReportCoefData(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    int i, k;
    GTI_WORD wBeg, wEnd;
    GTI_RCF_ADDRESS_DB *pAddrDb;
    int nPlot;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_SET_REP_COEF_DATA_OPC;

    pOutMsg[L2_TW_RC_PLOT_CHIP] = pInpMsg[L2_TW_RC_PLOT_CHIP];
    pOutMsg[L2_TW_RC_BEG_IDX] = pInpMsg[L2_TW_RC_BEG_IDX];
    pOutMsg[L2_TW_RC_BEG_IDX+1] = pInpMsg[L2_TW_RC_BEG_IDX+1];
    pOutMsg[L2_TW_RC_END_IDX] = pInpMsg[L2_TW_RC_END_IDX];
    pOutMsg[L2_TW_RC_END_IDX+1] = pInpMsg[L2_TW_RC_END_IDX+1];

    wBeg = MAKEWORD(pInpMsg[L2_TW_RC_BEG_IDX], pInpMsg[L2_TW_RC_BEG_IDX + 1]);
    wEnd = MAKEWORD(pInpMsg[L2_TW_RC_END_IDX], pInpMsg[L2_TW_RC_END_IDX + 1]);

    nPlot = pInpMsg[L2_TW_RC_PLOT_CHIP] & 0x0f;

    if(nPlot >= GTI_RC_MAX_OVERLAYED_PLOTS)
        return -1;

    pAddrDb = &(WhipGtiXcvrPtr->pChipInfo->pSystInfo->ExtTrace.AddrDb[nPlot]);

    pAddrDb->chip_id = (GTI_BYTE)(((pInpMsg[L2_TW_RC_PLOT_CHIP]) >> 4) & 0x0f);
    pAddrDb->size = (GTI_WORD)(wEnd + 1);

    for(i = wBeg; i < GTI_RC_MAX_COEFFICIENT_LOOP; i++)
    {
        pAddrDb->address[i].address = 0;
        pAddrDb->address[i].mem_type = 0;
    }

    for(k = L2_TW_RC_END_IDX + 2, i=0;
        (i < GTI_RC_MAX_COEFFICIENT_LOOP) && (i <= (wEnd - wBeg));
        i++, k+=2)
    {
        pAddrDb->address[i + wBeg].mem_type = pInpMsg[k];
        pAddrDb->address[i + wBeg].address = pInpMsg[k+1];
    }

    return L2_TW_RC_END_IDX + 1;
}

/****************************************************************************/
/* Procedure: WhipGetReportCoefData                       Created: 09.14.98 */
/* --------------------------------                                         */
/* Description: This function starts sends Report Coefficient Data base     */
/*              info to the WHIP unit                                       */
/* -------------------------------------------------------------------------*/
int WhipGetReportCoefData(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_WORD i, k, wBeg, wEnd, wPlot;
    GTI_RCF_DATA_DB *pDataDB;
    GTI_DWORD dwData;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_GET_REP_COEF_DATA_OPC;

    wBeg = MAKEWORD(pInpMsg[L2_TW_RC_BEG_IDX], pInpMsg[L2_TW_RC_BEG_IDX + 1]);
    wEnd = MAKEWORD(pInpMsg[L2_TW_RC_END_IDX], pInpMsg[L2_TW_RC_END_IDX + 1]);
    wPlot = (GTI_WORD)(pInpMsg[L2_TW_RC_PLOT_CHIP] & 0x0f);

    pDataDB = &(WhipGtiXcvrPtr->pChipInfo->pSystInfo->ExtTrace.DataDB[wPlot]);

    pOutMsg[L2_TW_RC_PLOT_CHIP] = pInpMsg[L2_TW_RC_PLOT_CHIP];
    pOutMsg[L2_TW_RC_BEG_IDX] = pInpMsg[L2_TW_RC_BEG_IDX];
    pOutMsg[L2_TW_RC_BEG_IDX+1] = pInpMsg[L2_TW_RC_BEG_IDX+1];
    pOutMsg[L2_TW_RC_END_IDX] = pInpMsg[L2_TW_RC_END_IDX];
    pOutMsg[L2_TW_RC_END_IDX+1] = pInpMsg[L2_TW_RC_END_IDX+1];
    pOutMsg[L2_TW_RC_LOOP_IDX] = pInpMsg[L2_TW_RC_LOOP_IDX];
    pOutMsg[L2_TW_RC_LOOP_IDX+1] = pInpMsg[L2_TW_RC_LOOP_IDX+1];

    k = L2_TW_RC_DATA_IDX;

    for(i=0; i < wEnd - wBeg; i++)
    {
        dwData = pDataDB->data[wBeg + i];

        pOutMsg[k++] = pDataDB->fail[wBeg + i];
        pOutMsg[k++] = (GTI_BYTE)((dwData) & 0xff);
        pOutMsg[k++] = (GTI_BYTE)((dwData >> 8) & 0xff);
        pOutMsg[k++] = (GTI_BYTE)((dwData >> 16) & 0xff);
        pOutMsg[k++] = (GTI_BYTE)((dwData >> 24) & 0xff);
    }

    return (int)(L2_TW_RC_DATA_IDX + 5 * (wEnd - wBeg));

}
#endif


/***************************************************************************/
/* Procedure: WhipSetGenericParameter                    Created: 05.26.98 */
/* ----------------------------------                                      */
/* Description: This function sets any generic parameter according to the  */
/*              provided type.                                             */
/* ----------------------------------------------------------------------- */
int WhipSetGenericParameter(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
#if (defined(GTI_PROD_SHDSL) && defined(GTI_DEBUG))
    GTI_BYTE EOCBuffer[10];
#endif

#if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE)
     GTI_BYTE HW;
#endif
#ifdef GTI_PROD_DMT
     GTI_BYTE tbuf[33];
#endif
     GTI_WORD i;

#if !defined(GTI_PROD_SHDSL)
     GTI_WORD j;
#endif

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_SET_GENERIC_PARAM_OPC;

    pOutMsg[L2_TW_SET_GEN_TYP_OFS] = pInpMsg[L2_TW_SET_GEN_TYP_OFS];
    pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_OK;
    pOutMsg[L2_TW_SET_GEN_RES_OFS + 1] = 0;

    switch(pInpMsg[L2_TW_SET_GEN_TYP_OFS])
    {
#if defined(GTI_PROD_DMT) && defined(GTI_USE_GLOBAL_BUFFER_FOR_LARGE_STATUS) && defined(GTI_DEBUG)
        case L2_START_PARAMETRIC_TEST:
        /****************************/
        {
            GTI_WORD wParam[8];

            for( i=0; i<(int)(sizeof(wParam)/sizeof(GTI_WORD)); i++)
            {
                wParam[i] = MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + (i * 3) + 1],
                            pInpMsg[L2_WT_SET_GEN_BUF_OFS + (i * 3) + 2]);
            }

            if(Gti_StartParametricTest(WhipGtiXcvrPtr, wParam) != GTI_OK)
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
          }
        break;
#endif /* DMT and Large Global Buffer */

#if defined (GTI_PROD_MSDSL) || defined (GTI_PROD_M2B1Q) || defined (GTI_PROD_HDSL2)
    case L2_FRAMER_PARAM_TYPE:
        for(i=0; i<pInpMsg[L2_WT_SET_GEN_MAX_OFS]; i++)
        {
            if(Gti_SetParamFramer(WhipGtiXcvrPtr,
                (GTI_WORD) pInpMsg[L2_WT_SET_GEN_BUF_OFS + (i * 3)],
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1 + (i * 3)],
                pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2 + (i * 3)]) ) != GTI_OK)
            {
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
            }
        }
        break;


#elif defined(GTI_PROD_SHDSL)

#if !defined(GTI_USE_XDPXFR)

#if defined(GTI_DEBUG)

    /* Only support in "HOST" mode */

    case L2_XCVR_LATENCY_TYPE:
            /************************/
            if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 2)
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
            else
            {
                HIPRqstDelay = MAKEWORD( pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                    pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]);
                HIPIntDelay = pInpMsg[L2_WT_SET_GEN_BUF_OFS + 4];

                Gti_UpdateTrace(HIPIntDelay, HIPRqstDelay);

            }
            break;


    case L2_RELOAD_DEBUG_POINTERS_TYPE:
        /*********************************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 1)
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        else
        {
            switch( pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1])
            {
            case FLASH_MEM:
                InitPointers(&(WhipGtiXcvrPtr->CmdInfo), GTI_PAGE_A);
                WhipGtiXcvrPtr->WhichMEM = FLASH_MEM;
                break;

            case RAM_MEM:
                ReadRAMPointers(WhipGtiXcvrPtr, GTI_PAGE_A);
                WhipGtiXcvrPtr->WhichMEM = RAM_MEM;
                break;
            default:
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_PARAMETER_NOT_EXISTS;
                break;
            }
        }
        break;

   case L2_EOC_PARAM_TYPE:
        /*********************/

        for(i=0; i< pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1]; i++)
        {
            EOCBuffer[i] = pInpMsg[( L2_WT_SET_GEN_BUF_OFS + 4) + (3*i)];
        }
        if(EOC2Xcvr(WhipGtiXcvrPtr, pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1], EOCBuffer) != GTI_OK)
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;

        break;
#endif /* GTI_DEBUG */

    case L2_SEND_TOM_DATA_TYPE:
        /*************************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 4)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
            Gti_SendTOMData( WhipGtiXcvrPtr,
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]),
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 4],
                pInpMsg[L2_WT_SET_GEN_BUF_OFS + 5]),
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 7],
                pInpMsg[L2_WT_SET_GEN_BUF_OFS + 8]),
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 10],
                pInpMsg[L2_WT_SET_GEN_BUF_OFS + 11]) );
        }
        break;

    case L2_TXCLAV_MODE_TYPE:
        /***********************/
        if(Gti_TxCLAVMode(WhipGtiXcvrPtr,
            MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]) ) != GTI_OK )
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
        }
        break;

    case L2_OH_RECEIVE_TYPE:
        /**********************/
        i = OH2Host( WhipGtiXcvrPtr);
        i=i;
        break;

    case L2_OH_PARAM_TYPE:
        /********************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 1)
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        else
            OH2Xcvr(WhipGtiXcvrPtr, pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1] );
        break;


#endif  /* !GTI_USE_XDPXFR (Host Mode Only) */


#ifdef GTI_TRACE
    case L2_RESET_HISTORY_TYPE:
        /*************************/
        Gti_ResetTraceBuffer();
        break;

    case L2_ENABLE_TRACE_TYPE:
        /************************/
        Gti_EnableTraceOutput(WhipGtiXcvrPtr,
            MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]) );
        break;
#endif

#elif defined (GTI_PROD_H2B1Q)
    case L2_EOC_PARAM_TYPE:
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 1)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
            if(LOSD2Xcvr(WhipGtiXcvrPtr, pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1]) == GTI_FAIL)
                 pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
        }
        break;
    case L2_PARAM_INTERFACE:

        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 5)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
        if(ParamInterface(WhipGtiXcvrPtr,
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 4],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 7],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 10],
            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 13] != GTI_OK)
            {
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
            }
        }
        break;

    case L2_MASTER_LP_TMG_SEL:

        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 1)
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        else
        {
            ParamMasterLoopSelection(WhipGtiXcvrPtr,
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                    pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]));
        }
        break;
#elif defined (GTI_PROD_DMT)

#if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE)
    case L2_SHUT_OFF_XCVR:
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] >= 4)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
            HW = pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1];

            if( Gti_ShutDwnXcvr(WhipGtiXcvrPtr, HW ) != GTI_OK )
            {
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
            }
        }
        break;
#endif /* Quad or Octane Only */

    case L2_SELECT_BIN_USAGE:
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] <= 4)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
            for( i=0, j=1; i<32; i+=2, j+=6)
            {
                tbuf[i] = pInpMsg[L2_WT_SET_GEN_BUF_OFS + j];
                tbuf[i+1] = pInpMsg[L2_WT_SET_GEN_BUF_OFS + j + 3];
            }
            if( Gti_SelectBinUsage( WhipGtiXcvrPtr, &tbuf[0] ) != GTI_OK )
            {
                pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
            }
        }
        break;
    #ifdef GTI_DEBUG
    case L2_AUTO_RETRAIN:
        /************************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 1)
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        else
        {
            /* For internal developement only - Enable/Disable AutoRetrain */
                if( WhipGtiXcvrPtr->IrEnabled == GTI_FALSE )
                {
                  HostRetrainMode =
                  MAKEWORD( pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                            pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]);
                }
                else
                {
               pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_SET_GEN_PARAMETER_FAILED;
                }
        }
        break;
    #endif /* GTI_DEBUG */

    #ifdef GTI_TRACE
    case L2_RESET_HISTORY_TYPE:
        /*************************/
        Gti_ResetTraceBuffer();
        break;
    #endif

#if ( defined(GTI_INSTALL_ANNEX_CO) )
    case L2_SET_PERFERED_MODE:
        /*************************/
        (void)Gti_GetPreferredMode( WhipGtiXcvrPtr,
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                    pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]));
        break;

    case L2_SET_INSTALL_SELECTION:
        /*************************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] != 2)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {

           (void)Gti_InstallSelection( WhipGtiXcvrPtr,
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1],
                    pInpMsg[L2_WT_SET_GEN_BUF_OFS + 2]),
                MAKEWORD(pInpMsg[L2_WT_SET_GEN_BUF_OFS + 4],
                    pInpMsg[L2_WT_SET_GEN_BUF_OFS + 5]));
        }
        break;
#endif /* Install Mode */

#if defined(GTI_PAGE_F_BUILT) && defined(GTI_BUILD_CP)
    case L2_SET_RINGING:
        /*************************/
        if(pInpMsg[L2_WT_SET_GEN_MAX_OFS] >= 4)
        {
            pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_INCOMPATIBLE_NUMBER;
        }
        else
        {
            i = pInpMsg[L2_WT_SET_GEN_BUF_OFS + 1];
            Gti_IndicateEvent(WhipGtiXcvrPtr, i );
        }
        break;
#endif

#else /* !DMT */
    case L2_FRAMER_PARAM_TYPE:
        pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_PARAMETER_NOT_EXISTS;
        break;
#endif

    default:
        pOutMsg[L2_TW_SET_GEN_RES_OFS] = L2_GEN_PARAMETER_NOT_EXISTS;
        break;
    }
    return L2_TW_SET_GEN_RES_OFS + 1;
}


/***************************************************************************/
/* Procedure: WhipDispatchExecCmd                        Created: 05.26.98 */
/* --------------------------------                                        */
/* Description: This function analizes Execute Command message and returns */
/*              result of the immidiate evaluation to the target.          */
/* ----------------------------------------------------------------------- */
int WhipDispatchExecCmd(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
#ifdef GTI_DEBUG
    extern void p_WHIP_bckg_operations(void);
#endif

    GTI_WORD wProjectType, wResult = GTI_OK;
    struct PARAM_XCVR_ARRAY ParamTbl;
    int i, k;

    wProjectType = 0;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_EXEC_COMMAND_OPC;
    pOutMsg[L2_TW_EXEC_ACTION_OFS] = L2_EXECUTE_COMMAND_REPLY;
    pOutMsg[L2_TW_EXEC_RESULT_OFS] = GTI_OK;

    ParamTbl.length = pInpMsg[L2_WT_EXEC_PAR_MAX_OFS];

    /* if length passed is zero, the command should be stopped */
    if(ParamTbl.length == 0)
    {
        if(WhipGtiXcvrPtr->ExecFlags.active)
        {
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
            WhipGtiXcvrPtr->CmdInfo.state = GTI_CANCELLED_SAS;
#endif
            pOutMsg[L2_TW_EXEC_RESULT_OFS] = GTI_CANCELLED_SAS;
        }
        if(WhipGtiXcvrPtr->IrEnabled == GTI_FALSE)
            WhipGtiXcvrPtr->ExecFlags.active = GTI_FALSE;

        return L2_TW_EXEC_RESULT_OFS;
    }
    else
    {
        /* Start Command execute */
        if(WhipGtiXcvrPtr->IrEnabled == GTI_FALSE)
            WhipGtiXcvrPtr->ExecFlags.active  = GTI_TRUE;
    }

    for(i=0, k=0; (i < ParamTbl.length) && (i < GTI_NUM_OF_CMD_PARAMETERS); k+=2, i++)
    {
        ParamTbl.item[i] = (GTI_WORD)(MAKEWORD(pInpMsg[L2_WT_EXEC_PAR_LST_OFS + k],
            pInpMsg[L2_WT_EXEC_PAR_LST_OFS + k + 1]));
    }

#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC))
    if(DSLConfigParams[0].bWhipEnable == GTI_TRUE)
    {
//         wResult = Gti_SetParamXCVR(WhipGtiXcvrPtr, &ParamTbl);
/* KJP */
       wResult = GTISetParamXCVR_USB(WhipGtiXcvrPtr, &ParamTbl);
    }
#else
//    wResult = Gti_SetParamXCVR(WhipGtiXcvrPtr, &ParamTbl);
    wResult = GTISetParamXCVR_USB(WhipGtiXcvrPtr, &ParamTbl);
/* KJP */
#endif



    /* do not proceed if the parameters are not OK  */
    if(wResult != GTI_OK)
        WhipGtiXcvrPtr->ExecFlags.active = GTI_FALSE;
    else
    {
#if defined(GTI_PROD_DMT) && defined(GTI_DEBUG)
        /* Copy Parameters - enabled and startup command  */
        if( (ParamTbl.item[GTI_ACTION_ITEM] == GTI_STARTUP_REQ) &&
            ( HostRetrainMode != 0 ) )
        {
          for(i =0; i<GTI_NUM_OF_CMD_PARAMETERS; i++ ){
            HostRetrainParams[WhipActiveXcvr].item[i] = ParamTbl.item[i];
            HostRetrainParams[WhipActiveXcvr].length = ParamTbl.length;
          }
        }
#endif
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
        /* Just Exec XCVR in the interrupt case */
        if(WhipGtiXcvrPtr->IrEnabled == GTI_TRUE)
            ExecuteXCVR_USB(WhipGtiXcvrPtr,0,0);
/* KJP */
//            (void)Gti_ExecuteXCVR(WhipGtiXcvrPtr);
        else
            Gti_BackgroundExecXCVR();
#endif
    }

#if defined(GTI_PROD_DMT)
    wProjectType = WhipGtiXcvrPtr->wAnnexSlctd;
#endif

    pOutMsg[L2_TW_EXEC_RESULT_OFS] = LOBYTE(wResult);
    pOutMsg[L2_TW_EXEC_RESULT_OFS + 1] = LOBYTE(wProjectType);

    /* send the result of the command execution operation                   */
    return L2_TW_EXEC_RESULT_OFS + 1;
}

/****************************************************************************/
/* Procedure: Gti_BackgroundExecXCVR                     Created: 04.06.98  */
/* --------------------------------                                         */
/* Description: Call this procedure from any background task, loop or timer */
/*              interrupt. The flag is set to false when the XCVR command   */
/*              is over and SAS is not active any more.                     */
/* ------------------------------------------------------------------------ */
void Gti_BackgroundExecXCVR(void)
{
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

#if defined(GTI_PROD_DMT)
    GTI_WORD wOpState, k;
    struct PARAM_XCVR_ARRAY ParamTbl;
#endif
    GTI_WORD i;

    for(i=0; i < GTI_TOTAL_NUM_OF_XCVRS; i++)
    {
        if(GTIXcvr[i].ExecFlags.active != GTI_FALSE)
        {
            /* when iterrupts are enabled - always set flag to zero */
            if(GTIXcvr[i].IrEnabled == GTI_TRUE)
            {
                GTIXcvr[i].ExecFlags.active = GTI_FALSE;
                (void)Gti_ExecuteXCVR(&GTIXcvr[i]);
            }
            else
            if(Gti_ExecuteXCVR(&GTIXcvr[i]) != GTI_ACTIVE)
                GTIXcvr[i].ExecFlags.active = GTI_FALSE;
        }

#ifdef GTI_DEBUG_WHIP
#ifdef GTI_PROD_DMT
        if( (GTIXcvr[i].ExecFlags.active == GTI_FALSE) &&
            (GTIXcvr[i].IrEnabled  == GTI_FALSE))
        {
            if(Gti_SStatusXCVR(&GTIXcvr[i], GTI_OP_STATE, &wOpState, sizeof(wOpState), 0))
            {
                if(wOpState == GTI_FAST_RETRAIN_OP)
                {
                    for(k=0; k < GTI_NUM_OF_CMD_PARAMETERS; k++)
                        ParamTbl.item[k] = 0;

                    ParamTbl.length = GTI_NUM_OF_CMD_PARAMETERS;
                    ParamTbl.item[GTI_ACTION_ITEM] = GTI_FAST_RETRAIN_REQ;
                    ParamTbl.item[GTI_MODE_ITEM] = GET_HOST_DB_B(&GTIXcvr[i], GTI_MODE);

                    if(ParamTbl.item[GTI_MODE_ITEM] == GTI_ATU_C)
                        ParamTbl.length = GTI_NUM_OF_ATU_C_CMD_PARAMETERS;
                    else
                        ParamTbl.length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;

                    if(Gti_SetParamXCVR(&GTIXcvr[i], &ParamTbl) == GTI_OK)
                        GTIXcvr[i].ExecFlags.active = GTI_TRUE;
                }
                else
                if(wOpState == GTI_PROF_MNGT_OP)
                {
                    for(k=0; k < GTI_NUM_OF_CMD_PARAMETERS; k++)
                        ParamTbl.item[k] = 0;

                    ParamTbl.length = GTI_NUM_OF_CMD_PARAMETERS;
                    ParamTbl.item[GTI_ACTION_ITEM] = GTI_PROF_MNGT_REQ;
                    ParamTbl.item[GTI_MODE_ITEM] = GET_HOST_DB_B(&GTIXcvr[i], GTI_MODE);

                    if(ParamTbl.item[GTI_MODE_ITEM] == GTI_ATU_C)
                        ParamTbl.length = GTI_NUM_OF_ATU_C_CMD_PARAMETERS;
                    else
                        ParamTbl.length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;

                    if(Gti_SetParamXCVR(&GTIXcvr[i], &ParamTbl) == GTI_OK)
                        GTIXcvr[i].ExecFlags.active = GTI_TRUE;
                }
                else
                if(wOpState == GTI_BIT_SWAPPING_OP)
                {
                    for(k=0; k < GTI_NUM_OF_CMD_PARAMETERS; k++)
                        ParamTbl.item[k] = 0;

                    ParamTbl.length = GTI_NUM_OF_CMD_PARAMETERS;
                    ParamTbl.item[GTI_ACTION_ITEM] = GTI_BIT_SWAPPING_REQ;
                    ParamTbl.item[GTI_ANNEX_TYPE_ITEM] = Gti_GetCurrentAnnexType((&GTIXcvr[i]));
                    ParamTbl.item[GTI_MODE_ITEM] = GET_HOST_DB_B(&GTIXcvr[i], GTI_MODE);

                    if(ParamTbl.item[GTI_MODE_ITEM] == GTI_ATU_C)
                        ParamTbl.length = GTI_NUM_OF_ATU_C_CMD_PARAMETERS;
                    else
                        ParamTbl.length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;

                    if(Gti_SetParamXCVR(&GTIXcvr[i], &ParamTbl) == GTI_OK)
                        GTIXcvr[i].ExecFlags.active = GTI_TRUE;
                }
            }
        }
#endif
#endif
    }
#endif /* !DMA */
}
#if defined(GTI_DSPQUAD)

GTI_WORD AccessDmaExternalRam(GTI_BYTE bRead, GTI_WORD wSeg, GTI_WORD wOfs,
                              GTI_WORD wSize, GTI_BYTE *pBuffer)
{
    GTI_WORD i, wWindow, wBapr, wOffs, wResult;
    GTI_WORD wAddress;
    GTI_BYTE bImage[2];

    GTI_BYTE DmaAccessTbl[] =
    {
        0x89, 0x05,                 /* -A0- CFR  */
        0x00, 0x00,                 /* -A1- BAPR */
        0x00, 0x00,                 /* -A2- EBR  */
        GTI_DMA_window, GTI_ISRAM,  /* -A3- ISAR */
        0x01, 0x00,                 /* -A4- DLR  */
        0x00, 0x00,                 /* -A5- CCR  */
        0x00, 0x00                  /* -A6- XSAR */
    };

    if(Gti_IsCXfrActive(WhipGtiXcvrPtr))
        return 0;

    wBapr = (GTI_WORD)((wSeg >> 0x04) | (wOfs >> 0x08));
    wOffs =(GTI_WORD)(wOfs & 0xff);

    CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_NO_AI);
    WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA_window, GTI_READ_DSP | GTI_ISRAM);
    wWindow = READ_DSP_WORD(WhipGtiXcvrPtr, GTI_DSP_A);

    if(bRead)
    {
        for(i = 0, wAddress = wOffs; i < wSize; i += 2, wAddress += 2)
        {
            DmaAccessTbl[02] = (GTI_BYTE)(wBapr & 0xff);
            DmaAccessTbl[03] = (GTI_BYTE)(wBapr >> 0x08);
            DmaAccessTbl[10] = 0x01;
            DmaAccessTbl[12] = (GTI_BYTE)(wAddress & 0xff);
            DmaAccessTbl[13] = (GTI_BYTE)(wAddress >> 0x08);

            /* configure DMA */
            CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA);
            WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, 0xa0, GTI_IREGA);
            WRITE_DSP_BLOCK(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DATAL_DSP, sizeof(DmaAccessTbl), DmaAccessTbl);

            /* configure ISRAM */
            CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_NO_AI);
            WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA_window, GTI_ISRAM|GTI_READ_DSP);

            wResult = READ_DSP_WORD(WhipGtiXcvrPtr, GTI_DSP_A);

            pBuffer[i] = LOBYTE(wResult);
            pBuffer[i+1] = HIBYTE(wResult);
        }
    }
    else
    {
        for(i = 0, wAddress = wOffs; i < wSize; i += 2, wAddress += 2)
        {
            if((i + 1) >= wSize)
                (void)AccessDmaExternalRam(GTI_TRUE, wSeg, (GTI_WORD)(wOfs + i), 1, bImage);
            else
                bImage[1] = pBuffer[i+1];

            /* configure ISRAM */
            CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_NO_AI);
            WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA_window, GTI_ISRAM);
            WRITE_DSP_WORD(WhipGtiXcvrPtr, GTI_DSP_A, pBuffer[i], bImage[1]);

            /* configure DMA */
            DmaAccessTbl[02] = (GTI_BYTE)(wBapr & 0xff);
            DmaAccessTbl[03] = (GTI_BYTE)(wBapr >> 0x08);
            DmaAccessTbl[10] = 0x11;
            DmaAccessTbl[12] = (GTI_BYTE)(wAddress & 0xff);
            DmaAccessTbl[13] = (GTI_BYTE)(wAddress >> 0x08);

            CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA);
            WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, 0xa0, GTI_IREGA);
            WRITE_DSP_BLOCK(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DATAL_DSP, sizeof(DmaAccessTbl), DmaAccessTbl);
        }
    }

    CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_A, GTI_NO_AI);
    WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_A, GTI_DMA_window, GTI_ISRAM);
    WRITE_DSP_WORD(WhipGtiXcvrPtr, GTI_DSP_A, LOBYTE(wWindow), HIBYTE(wWindow));

    return wSize;
}
#endif

/****************************************************************************/
/* Procedure: WhipDispatchReadWriteMemory                Created: 04.06.98  */
/* --------------------------------                                         */
/* Description: This function analizes Read and Write memory messages.      */
/*              Reply memory is always Read Memory message with actual data.*/
/* ------------------------------------------------------------------------ */
int WhipDispatchReadWriteMemory(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_WORD mem_id, mem_type, offset, star_id, need_ack, i;
    GTI_WORD segment;
    GTI_BYTE length;
    GTI_BYTE bPage;

#ifdef GTI_TRACE
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    GTI_BYTE GTI_ADDRESS_QUAL *pMemory;
#endif
#endif

#ifndef GTI_DSPTURBO
    GTI_BYTE bCs_B = 0;
#endif

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_READMEMORY_OPC;

    mem_id = MAKEWORD(pInpMsg[L2_MEMORY_RW_MID_OFS],
        pInpMsg[L2_MEMORY_RW_MID_OFS + 1]);

    offset = MAKEWORD(pInpMsg[L2_MEMORY_RW_OFF_OFS],
        pInpMsg[L2_MEMORY_RW_OFF_OFS + 1]);

    segment = MAKEWORD(pInpMsg[L2_MEMORY_RW_SEG_OFS],
        pInpMsg[L2_MEMORY_RW_SEG_OFS + 1]);
    /*
    #if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING)
    segment = MAKEWORD(pInpMsg[L2_MEMORY_RW_SEG_OFS],
    pInpMsg[L2_MEMORY_RW_SEG_OFS + 1]);
    #endif
    */

    mem_type = (GTI_WORD)(WHIP_GET_MEM_TYPE(mem_id));
    star_id  = (GTI_WORD)(WHIP_GET_CHIP_ID(mem_id));
    need_ack = (GTI_WORD)(WHIP_GET_NEEDACK(mem_id));
    length   = pInpMsg[L2_MEMORY_RW_LEN_OFS];

    star_id |= (segment & 0xf) << 3;

    memcpy((&pOutMsg[L2_MEMORY_RW_MID_OFS]),
        (&pInpMsg[L2_MEMORY_RW_MID_OFS]), (L2_MEMORY_RW_LEN_OFS - 1));

    /* Dispatch HOST memory in a separate Target processor oriented routine */
    if((mem_type >> 8) != 0)
    {
        switch(mem_type)
        {
        case EXTERNAL_LBRAM:
        case EXTERNAL_RSRAM:

            if(pInpMsg[L2_MSG_OPCODE_OFS] != L2_READMEMORY_OPC)
            {
#if defined(GTI_DSPQUAD)
                length = (GTI_BYTE)AccessDmaExternalRam(GTI_FALSE,
                    segment, offset, length, &pInpMsg[L2_MEMORY_RW_DAT_OFS]);
#elif defined(GTI_USE_XDPXFR)
                length = (GTI_BYTE)Gti_ExtMemoryAccess(WhipGtiXcvrPtr,
                    mem_type, (GTI_BYTE)(HIBYTE(segment) >> 0x04), offset,
                    length, &pInpMsg[L2_MEMORY_RW_DAT_OFS]);
#endif
            }

            if(need_ack)
                pOutMsg[L2_MEMORY_RW_LEN_OFS] = (GTI_BYTE)(length);
            else
                pOutMsg[L2_MEMORY_RW_LEN_OFS] = 0;

            if(length && need_ack)
            {
#if defined(GTI_DSPQUAD)
                length = (GTI_BYTE)AccessDmaExternalRam(GTI_TRUE,
                    segment, offset, length, &pOutMsg[L2_MEMORY_RW_DAT_OFS]);
#elif defined(GTI_USE_XDPXFR)
                length = (GTI_BYTE)Gti_ExtMemoryAccess(WhipGtiXcvrPtr,
                    mem_type, (GTI_BYTE)((HIBYTE(segment) >> 0x04) | GTI_READ_DSP), offset,
                    length, &pOutMsg[L2_MEMORY_RW_DAT_OFS]);
#endif
            }
            if(length == 0)
                return -1;
            else
                return L2_MEMORY_RW_DAT_OFS + pOutMsg[L2_MEMORY_RW_LEN_OFS];

        case HREG_DUMP_TYPE:
#ifndef GTI_DSPTURBO
            if(star_id == GTI_DSP_DMA)
                bCs_B = 0;
            else
            {
                bCs_B = (GTI_BYTE)WhipGtiXcvrPtr->dsp_B_cs;
            }
#endif

#ifdef GTI_DSPQUAD
            if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_BDCST_OPC)
                bCs_B = GTI_DSP_ABCD_MSK;
#endif
            if( (pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC) ||
                (pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_BDCST_OPC))
            {
                for(i=0; i<length; i++)
                {
#ifdef GTI_DSPTURBO
                    /* The IO driver should do the MASK!!! */
                    WRITE_DSP_BYTE(WhipGtiXcvrPtr, star_id,
                        (GTI_BYTE)(offset + i), pInpMsg[L2_MEMORY_RW_DAT_OFS + i]);
#else
                    WRITE_DSP_REG(WhipGtiXcvrPtr,
                        (GTI_BYTE)(offset + i) | bCs_B, pInpMsg[L2_MEMORY_RW_DAT_OFS + i]);
#endif
                }
            }
            if(need_ack)
            {
                for(i=0; i<length; i++)
                {
#ifdef GTI_DSPTURBO
                    /* The IO driver should do the MASK!!! */
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i] = READ_DSP_BYTE(WhipGtiXcvrPtr, star_id,
                        (GTI_BYTE)(offset + i) );
#else
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i] = READ_DSP_REG(WhipGtiXcvrPtr,
                        (GTI_BYTE)(offset + i) | bCs_B);
#endif
                }
            }
            break;

        case XCVR_HOST_DUMP_TYPE:

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

            if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC)
            {
                memcpy(&(WhipGtiXcvrPtr->paramsDB[offset]),
                    &pInpMsg[L2_MEMORY_RW_DAT_OFS], length);
            }

            if(need_ack)
            {
                if(length > L2_MAX_MSGBODY_LENGTH)
                    length = L2_MAX_MSGBODY_LENGTH;

                memcpy(&pOutMsg[L2_MEMORY_RW_DAT_OFS],
                    &(WhipGtiXcvrPtr->paramsDB[offset]), length);
            }
#else
            length = 0;
#endif
            break;
#if defined(GTI_DSPTURBO) || defined(GTI_DSPMONACO)
        case VREG0_DUMP_TYPE:
        case VREG1_DUMP_TYPE:
            segment  = (offset&0xff);
            if( mem_type == VREG0_DUMP_TYPE )
            {
                segment |= ((GTI_VREG0 - GTI_VREG_ADJUST) << 0x08);
            }
            else
            {
                segment |= ((GTI_VREG1 - GTI_VREG_ADJUST) << 0x08);
            }

            CONFIGURE_DSP( WhipGtiXcvrPtr, GTI_DSP_TOP, GTI_NO_AI);

            if( (pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC) ||
                (pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_BDCST_OPC))
            {
                /* Write Memory */
                for(i=0; i < length; i+= 4)   /* 32-bits register */
                {
                    WRITE_DSP_REG(WhipGtiXcvrPtr,  0x0c, LOBYTE(segment) );  /* Low Address SAP for Top */
                    WRITE_DSP_REG(WhipGtiXcvrPtr,  0x0d, HIBYTE(segment));

                    WRITE_DSP_REG( WhipGtiXcvrPtr, 0x1a, pInpMsg[L2_MEMORY_RW_DAT_OFS+i]);
                    WRITE_DSP_REG( WhipGtiXcvrPtr, 0x1b, pInpMsg[L2_MEMORY_RW_DAT_OFS+i+1]);
                    WRITE_DSP_WORD( WhipGtiXcvrPtr, GTI_DSP_TOP, pInpMsg[L2_MEMORY_RW_DAT_OFS+i+2], pInpMsg[L2_MEMORY_RW_DAT_OFS+i+3]);
                    segment++;
                }
            }
            if(need_ack)
            {
                segment = (offset & 0xff);
                if( mem_type == VREG0_DUMP_TYPE )
                {
                    segment |= ((GTI_VREG0-GTI_VREG_ADJUST)<<0x08);
                }
                else
                {
                    segment |= ((GTI_VREG1-GTI_VREG_ADJUST)<<0x08);
                }
                for(i=0; i<length; i+= 4)
                {
                    WRITE_DSP_REG(WhipGtiXcvrPtr,  0x0c, LOBYTE(segment) );  /* Low Address SAP for Top */
                    WRITE_DSP_REG(WhipGtiXcvrPtr,  0x0d, (HIBYTE(segment)|GTI_READ_DSP));
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i] = READ_DSP_REG(WhipGtiXcvrPtr,0x1a);
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i+1] = READ_DSP_REG(WhipGtiXcvrPtr,0x1b);
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i+2] = READ_DSP_REG(WhipGtiXcvrPtr,0x1e);
                    pOutMsg[L2_MEMORY_RW_DAT_OFS + i+3] = READ_DSP_REG(WhipGtiXcvrPtr,0x1f);
                    segment++;
                }
            }

            break;
#endif /* Turbo and monaco */

        case IPU_CODE_DUMP_TYPE:
        case FDP_CODE_DUMP_TYPE:
        case PCU_CODE_DUMP_TYPE:
        case AVM_CODE_DUMP_TYPE:
        case BE_CODE_DUMP_TYPE:
        case TR_CODE_DUMP_TYPE:
        case XDP_CODE_DUMP_TYPE:

            bPage = (GTI_BYTE)(segment >> 12);

#ifdef GTI_USE_XDPXFR

            if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC)
            {
                if(!Gti_AccessProjectCode(WhipGtiXcvrPtr, 0, bPage, mem_type,
                    offset, length, (GTI_BYTE *)(&pInpMsg[L2_MEMORY_RW_DAT_OFS])))
                    return -1;
            }
            if(need_ack)
            {
                if(length > L2_MAX_MSGBODY_LENGTH)
                    length = L2_MAX_MSGBODY_LENGTH;

                if(!Gti_AccessProjectCode(WhipGtiXcvrPtr, GTI_READ_DSP, bPage, mem_type,
                    offset, length, (GTI_BYTE *)(&pOutMsg[L2_MEMORY_RW_DAT_OFS])))
                    return -1;
            }
#endif

#ifdef GTI_TRACE
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

            if((bPage < 0xf) )
                Gti_InitPointers(&(WhipGtiXcvrPtr->CmdInfo), bPage);

            pOutMsg[L2_MEMORY_RW_SEG_OFS] = LOBYTE(WhipGtiXcvrPtr->CmdInfo.ProjectIDPnt << 12);
            pOutMsg[L2_MEMORY_RW_SEG_OFS+1] = HIBYTE(WhipGtiXcvrPtr->CmdInfo.ProjectIDPnt << 12);

            switch(mem_type)
            {
            case IPU_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntIPU + (offset << (IPU_CODE_DUMP_WORD_LENGTH/16));
                break;
            case FDP_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntFDP + (offset << (FDP_CODE_DUMP_WORD_LENGTH/16));
                break;
            case PCU_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntPCU + (offset << (PCU_CODE_DUMP_WORD_LENGTH/16));
                break;
            case AVM_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntAVM + (offset << (AVM_CODE_DUMP_WORD_LENGTH/16));
                break;
            case BE_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntBE + (offset << (BE_CODE_DUMP_WORD_LENGTH/16));
                break;
            case TR_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntTR + (offset << (TR_CODE_DUMP_WORD_LENGTH/16));
                break;
            case XDP_CODE_DUMP_TYPE:
                pMemory = WhipGtiXcvrPtr->CmdInfo.PntXDP + (offset << (XDP_CODE_DUMP_WORD_LENGTH/16));
                break;
            }

            if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC )
            {
                memcpy(pMemory,&pInpMsg[L2_MEMORY_RW_DAT_OFS],length);
            }

            if(need_ack)
            {
                if(length > L2_MAX_MSGBODY_LENGTH)
                    length = L2_MAX_MSGBODY_LENGTH;

                memcpy(&pOutMsg[L2_MEMORY_RW_DAT_OFS], pMemory, length);
            }

#endif /* Host mode */
#endif /* Trace mode */
            break;
            default:
#ifdef GTI_DEBUG
                return p_WhipReadWriteHostMemory(
                    (pInpMsg[L2_MSG_OPCODE_OFS] == L2_READMEMORY_OPC), pInpMsg, pOutMsg);
#else
                break;
#endif
        }
    }
    else
    {
        if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_READMEMORY_OPC)
        {
            need_ack = 1;
        }
        else
        if(pInpMsg[L2_MSG_OPCODE_OFS] == L2_WRITEMEMORY_OPC)
        {
            WriteDspBlock(WhipGtiXcvrPtr,
                (GTI_BYTE)mem_type, (GTI_BYTE)offset,
                (GTI_BYTE *)(&pInpMsg[L2_MEMORY_RW_DAT_OFS]),
                (GTI_WORD)length, (GTI_BYTE)star_id);
        }

        if(need_ack)
        {
            ReadDspBlock(WhipGtiXcvrPtr,
                (GTI_BYTE)mem_type, (GTI_BYTE)offset,
                (GTI_BYTE *)(&pOutMsg[L2_MEMORY_RW_DAT_OFS]),
                (GTI_WORD)length, (GTI_BYTE)star_id);
        }
    }

    if(need_ack)
        pOutMsg[L2_MEMORY_RW_LEN_OFS] = (GTI_BYTE)length;
    else
        pOutMsg[L2_MEMORY_RW_LEN_OFS] = 0;

    return L2_MEMORY_RW_DAT_OFS + pOutMsg[L2_MEMORY_RW_LEN_OFS] - 1;
}

/****************************************************************************/
/* Procedure: WhipDispatchBreakpntInfo                Created: 04.06.98     */
/* --------------------------------                                         */
/* Description: This functionsets and reads Breakpoint Info from/to target. */
/* ------------------------------------------------------------------------ */
int WhipDispatchBreakpntInfo(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
#ifdef GTI_TRACE
    int i;

    GTI_DWORD dwAddr;
    GTI_WORD wOffset, wFdpOff;
    GTI_BYTE bIpuDataArray[4] =
    {
        0xd8, 0x00, 0x00, 0x04,
    };

    GTI_BYTE bFdpDataArray[8] =
    {
        0x00, 0x00, 0x06, 0xec, 0x08, 0x00, 0x04, 0xe0,
    };

    GTI_BYTE bPcuDataArray[4] =
    {
        0xda, 0x00, 0x00, 0xc0,
    };

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_BREAKPOINT_INFO_OPC;

    pOutMsg[L2_BRKPNTI_TYP_OFS] = pInpMsg[L2_BRKPNTI_TYP_OFS];

    for(i=0; i < L2_MAX_BREAKPOINTS; i++)
        pOutMsg[L2_BRKPNTI_STA_OFS + i] = L2_BRKPNT_STATE_ILLEGAL;

    if((pInpMsg[L2_BRKPNTI_TYP_OFS] >> 0x04) == L2_SAS_BREAKPOINTS)
    {
        if(pInpMsg[L2_BRKPNTI_TYP_OFS] & 0x0f)
        {

            if(SASStop_enable)
            {
                if(!Gti_SetSasBreakpoint(WhipGtiXcvrPtr, SASStop_line, SASStop_baud, 2, &SASStop_hip))
                return -1;
            }

            SASStop_baud =  pInpMsg[L2_BRKPNTI_INF_OFS + 3];
#ifndef GTI_USE_XDPXFR
            /* Host mode                                */
            /* _baud - not used                         */
            /* _hip - set by user(WHIP)                 */
            SASStop_hip =  pInpMsg[L2_BRKPNTI_INF_OFS + 3];
#endif
            SASStop_line = MAKEWORD(pInpMsg[L2_BRKPNTI_INF_OFS], pInpMsg[L2_BRKPNTI_INF_OFS + 1]);
            SASStop_enable = pInpMsg[L2_BRKPNTI_STA_OFS];


            if(SASStop_enable == 1)
            {
                if(!Gti_SetSasBreakpoint(WhipGtiXcvrPtr, SASStop_line, SASStop_baud, 1, &SASStop_hip))
                    return -1;
            }
        }

        pOutMsg[L2_BRKPNTI_STA_OFS] = SASStop_enable;
        pOutMsg[L2_BRKPNTI_INF_OFS] = LOBYTE(SASStop_line);
        pOutMsg[L2_BRKPNTI_INF_OFS + 1] = HIBYTE(SASStop_line);
        pOutMsg[L2_BRKPNTI_INF_OFS + 3] = SASStop_baud;
    }
    else
    {
        for(i=0; i < L2_MAX_BREAKPOINTS; i++)
        {
            if(pInpMsg[L2_BRKPNTI_TYP_OFS] & 0x0f)
            {
                wOffset = pInpMsg[L2_BRKPNTI_INF_OFS + 3 + (i * 4)];

                /* first clear the breakpoint if it was previously set */
                if((Whip_BrkPntData[i].bState & 0x01) == GTI_TRUE)
                {
                    if(Whip_BrkPntData[i].wType == L2_IPU_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_IPU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bIpuDataArray));

                    }
                    else if(Whip_BrkPntData[i].wType == L2_FDP_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_FDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bFdpDataArray));
                    }
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
                    else if(Whip_BrkPntData[i].wType == L2_XDP_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_XDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bIpuDataArray));
                    }
#endif /* Host mode */
                    else
                    if(Whip_BrkPntData[i].wType == L2_PCU_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_PCU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bPcuDataArray));
                    }

                    Whip_BrkPntData[i].bState = GTI_FALSE | 0x80;
                }
                /* fill the structure */
                Whip_BrkPntData[i].bState = pInpMsg[L2_BRKPNTI_STA_OFS + i] | 0x80;

                Whip_BrkPntData[i].wAddr = MAKEWORD(pInpMsg[L2_BRKPNTI_INF_OFS + (i * 4)],
                    pInpMsg[L2_BRKPNTI_INF_OFS + 1 + (i * 4)]);

                Whip_BrkPntData[i].bPage = pInpMsg[L2_BRKPNTI_INF_OFS + 3 + (i * 4)];

                Whip_BrkPntData[i].wOffs = MAKEWORD(pInpMsg[L2_BRKPNTI_AUX_OFS + 0 + (i * 4)],
                    pInpMsg[L2_BRKPNTI_AUX_OFS + 1 + (i * 4)]);

                Whip_BrkPntData[i].wType = MAKEWORD(pInpMsg[L2_BRKPNTI_AUX_OFS + 2 + (i * 4)],
                    pInpMsg[L2_BRKPNTI_AUX_OFS + 3 + (i * 4)]);

                /* set breakpoint if it has to be set */
                if(pInpMsg[L2_BRKPNTI_STA_OFS + i] == GTI_TRUE)
                {
                    if(Whip_BrkPntData[i].wType == L2_IPU_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, GTI_READ_DSP, L2_IPU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bIpuDataArray));

                        bIpuDataArray[1] = (GTI_BYTE)Whip_BrkPntData[i].wOffs;

                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_IPU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, bIpuDataArray, sizeof(bIpuDataArray));
                    }
                    else if(Whip_BrkPntData[i].wType == L2_FDP_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, GTI_READ_DSP, L2_FDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bFdpDataArray));

                        wFdpOff = (GTI_WORD)((Whip_BrkPntData[i].wOffs << 6) | 0x08);

                        bFdpDataArray[0] = LOBYTE(wFdpOff);
                        bFdpDataArray[1] = HIBYTE(wFdpOff);

                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_FDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, bFdpDataArray, sizeof(bFdpDataArray));
                    }
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
                    else if(Whip_BrkPntData[i].wType == L2_XDP_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, GTI_READ_DSP, L2_XDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bIpuDataArray));

                        bIpuDataArray[1] = (GTI_BYTE)Whip_BrkPntData[i].wOffs;

                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_XDP_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, bIpuDataArray, sizeof(bIpuDataArray));
                    }
#endif /* host mode */
                    else
                    if(Whip_BrkPntData[i].wType == L2_PCU_BREAKPOINTS)
                    {
                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, GTI_READ_DSP, L2_PCU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, Whip_BrkPntData[i].aCode, sizeof(bPcuDataArray));

                        bPcuDataArray[1] = (GTI_BYTE)Whip_BrkPntData[i].wOffs; /* insert the goto $ */

                        Gti_SetDspBreakpoint(WhipGtiXcvrPtr, 0, L2_PCU_BREAKPOINTS, wOffset,
                            Whip_BrkPntData[i].wAddr, bPcuDataArray, sizeof(bPcuDataArray));
                    }
                }
            }

            /* return the updated structure */
            pOutMsg[L2_BRKPNTI_STA_OFS + i] = Whip_BrkPntData[i].bState | 0x80;

            pOutMsg[L2_BRKPNTI_INF_OFS + 0 + (i * 4)] = LOBYTE(Whip_BrkPntData[i].wAddr);
            pOutMsg[L2_BRKPNTI_INF_OFS + 1 + (i * 4)] = HIBYTE(Whip_BrkPntData[i].wAddr);
            pOutMsg[L2_BRKPNTI_INF_OFS + 2 + (i * 4)] = 0;
            pOutMsg[L2_BRKPNTI_INF_OFS + 3 + (i * 4)] = Whip_BrkPntData[i].bPage;

            pOutMsg[L2_BRKPNTI_AUX_OFS + 0 + (i * 4)] = LOBYTE(Whip_BrkPntData[i].wOffs);
            pOutMsg[L2_BRKPNTI_AUX_OFS + 1 + (i * 4)] = HIBYTE(Whip_BrkPntData[i].wOffs);
            pOutMsg[L2_BRKPNTI_AUX_OFS + 2 + (i * 4)] = LOBYTE(Whip_BrkPntData[i].wType);
            pOutMsg[L2_BRKPNTI_AUX_OFS + 3 + (i * 4)] = HIBYTE(Whip_BrkPntData[i].wType);
        }
    }

    return L2_BRKPNTI_INF_OFS + (34 * L2_MAX_BREAKPOINTS);
#else
    (void)pInpMsg;
    (void)pOutMsg;
    return -1;
#endif
}

#if (defined (GTI_DSPOCTANE)  && (defined(GTI_USE_DMAXFR)|| defined(GTI_USE_XDPXFR) ) )

/****************************************************************************/
/* Procedure: WhipDispatchLbusTrace                   Created: 04.06.98     */
/* --------------------------------                                         */
/* Description: This function fills trace info into the output buffer       */
/* ------------------------------------------------------------------------ */
int WhipDispatchLbusTrace(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_WORD start, wStartAddr, max, size, type;
    GTI_BYTE wTraceData[4];

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_LBUS_TRACE_OPC;

    start = MAKEWORD(pInpMsg[L2_TRACE_START_OFS], pInpMsg[L2_TRACE_START_OFS+1]);
    type = MAKEWORD(pInpMsg[L2_TRACE_TYPE_OFS], pInpMsg[L2_TRACE_TYPE_OFS+1]);

    pOutMsg[L2_TRACE_START_OFS] = pInpMsg[L2_TRACE_START_OFS];
    pOutMsg[L2_TRACE_START_OFS+1] = pInpMsg[L2_TRACE_START_OFS+1];


    if(type == 1)
        ReadDspBlock(WhipGtiXcvrPtr,
            GTI_XDPSRAM0, 0x19, wTraceData, sizeof(wTraceData), GTI_DSP_TOP);
    else
    {
        ReadDspBlock(WhipGtiXcvrPtr,
            GTI_XDPSRAM0, 0x09, wTraceData, sizeof(wTraceData), GTI_DSP_A);
    }

    if((wTraceData[2] == 0) && (wTraceData[3] == 0))
        return -1;

    max = MAKEWORD(wTraceData[0], wTraceData[1]);
    wStartAddr = (GTI_WORD)(start * 2);

    if((wStartAddr + (L2_MAX_LBUS_TRACE_LINES * 2)) >= max)
        size = (GTI_WORD)((max - wStartAddr) * 2);
    else
        size = L2_MAX_LBUS_TRACE_LINES * 4;

    if(!Gti_ExtMemoryAccess(WhipGtiXcvrPtr, EXTERNAL_LBRAM, (GTI_BYTE)
        (GTI_READ_DSP | wTraceData[2]), wStartAddr, size, &pOutMsg[L2_TRACE_BUF_OFS]))
        return -1;

    pOutMsg[L2_TRACE_NUM_OFS] = (GTI_BYTE)(size / 4);
    return L2_TRACE_BUF_OFS + size;
}
#endif

#ifdef GTI_TRACE
/****************************************************************************/
/* Procedure: WhipDispatchReadTrace                   Created: 04.06.98     */
/* --------------------------------                                         */
/* Description: This function fills trace info into the output buffer       */
/* ------------------------------------------------------------------------ */
int WhipDispatchReadTrace(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_WORD i, start, size;
    GTI_WORD traceBuffer[L2_MAX_TRACE_LINES][3];

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_READ_TRACE_BUFFER_OPC;

    size = L2_TRACE_BUF_OFS;

    start = MAKEWORD(pInpMsg[L2_TRACE_START_OFS], pInpMsg[L2_TRACE_START_OFS+1]);

    pOutMsg[L2_TRACE_START_OFS] = pInpMsg[L2_TRACE_START_OFS];
    pOutMsg[L2_TRACE_START_OFS+1] = pInpMsg[L2_TRACE_START_OFS+1];

    for(i = 0; (start < HistPnt) && (i < L2_MAX_TRACE_LINES); i++)
    {
        SWAP_WORD(&(traceBuffer[i][0]), &(History[0][start]));
        SWAP_WORD(&(traceBuffer[i][1]), &(History[1][start]));
        SWAP_WORD(&(traceBuffer[i][2]), &(History[2][start]));
        start++;
        size += sizeof(GTI_WORD) * 3;
    }

    memcpy((&pOutMsg[L2_TRACE_BUF_OFS]), (&traceBuffer[0][0]),
        (size - L2_TRACE_BUF_OFS));

    pOutMsg[L2_TRACE_NUM_OFS] = (GTI_BYTE)i;

    return size;
}

/****************************************************************************/
/* Procedure: WhipDispatchReadTimeStamp               Created: 04.06.98     */
/* --------------------------------                                         */
/* Description: This function fills timestamp info into the output buffer   */
/* ------------------------------------------------------------------------ */
int WhipDispatchReadTimeStamp(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_WORD i, k, start, size;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_READ_TIMESTAMP_OPC;

    size = L2_TRACE_BUF_OFS;

    start = MAKEWORD(pInpMsg[L2_TRACE_START_OFS], pInpMsg[L2_TRACE_START_OFS+1]);
    pOutMsg[L2_TRACE_START_OFS] = pInpMsg[L2_TRACE_START_OFS];
    pOutMsg[L2_TRACE_START_OFS+1] = pInpMsg[L2_TRACE_START_OFS+1];

    for(i=0, k=0; (start < HistPnt) && (i < L2_MAX_TIMESTAMP_LINES); i++, k+=2)
    {
        /*
        pOutMsg[L2_TRACE_BUF_OFS + k] = LOBYTE(HistTimeStamp[start]),
        pOutMsg[L2_TRACE_BUF_OFS + k + 1] = HIBYTE(HistTimeStamp[start]);
        */
        start++;
        size += sizeof(GTI_WORD);
    }

    pOutMsg[L2_TRACE_NUM_OFS] = (GTI_BYTE)i;

    return size;
}
#endif

int WhipCustomMessage(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    int i;
    struct PARAM_XCVR_ARRAY ParamTbl;
    GTI_WORD wResult;
    GTI_BYTE bParam1, bParam2, bParam3, bParam4;
#ifdef GS_INCLUDE_API_CODE
    GTI_XCVR_INFO       *pGtiXcvrPtr;
    GS_XCVR_INFO *       pXcvrInfo;
    GS_OCTANE_XCVR_INFO *pOctaneXcvrInfo;
#endif

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_CUSTOM_MSG_OPC;
    pOutMsg[L2_MSG_OPCODE_OFS+1] = GTI_FAIL;

    bParam1 = pInpMsg[1];
    bParam2 = pInpMsg[2];
    bParam3 = pInpMsg[3];
    bParam4 = pInpMsg[4];

    ParamTbl.length = GTI_NUM_OF_CMD_PARAMETERS;

#ifdef GS_INCLUDE_API_CODE
    if(GS_SetXcvrDefault(bParam4, &ParamTbl) == GS_FAIL)
        return 1;

    ParamTbl.item[GS_DMT_ACTION_ITEM] = bParam1;
#else
    ParamTbl.item[GTI_ACTION_ITEM] = bParam1;
#endif

    wResult = GTI_OK;

    for(i=bParam2; (i < bParam3) && (wResult == GTI_OK); i++)
    {
#ifdef GS_INCLUDE_API_CODE
        pXcvrInfo = (GS_XCVR_INFO*)Com_SystemXcvrIdToXcvrInfo(pSystInfo, i);

        if (pXcvrInfo != GS_NULL)
        {
            pOctaneXcvrInfo = (GS_OCTANE_XCVR_INFO*)pXcvrInfo->pChipXcvrInfo;
            pGtiXcvrPtr = &(pOctaneXcvrInfo->sGtiXcvrInfo);
            wResult = Gti_SetParamXCVR(pGtiXcvrPtr , &ParamTbl, 0);
        }
#else
        wResult = Gti_SetParamXCVR(&GTIXcvr[i], &ParamTbl);
#endif
    }

    pOutMsg[L2_MSG_OPCODE_OFS+1] = (GTI_BYTE)wResult;

    (void)bParam4;
    return 1;
}

/****************************************************************************/
/* Procedure: WhipReadConstellation                   Created: 04.06.98     */
/* --------------------------------                                         */
/* Description: This function reads constellation data and sends a reply    */
/* ------------------------------------------------------------------------ */
int WhipReadConstellation(GTI_BYTE *pOutMsg)
{
#ifdef GTI_PROD_DMT
    GTI_DWORD dwData, dwX, dwY;
    GTI_WORD nWordLen;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_READ_CONSTELLATION_OPC;

    nWordLen = (GTI_WORD)Gti_GetWordSize(GTI_FDPSRAM0);

    dwData = GTI_TRUE;

    WriteDspBlock(WhipGtiXcvrPtr,
        GTI_FDPSRAM0, 0x89, (GTI_BYTE *)&dwData, nWordLen, GTI_DSP_A);

    ReadDspBlock(WhipGtiXcvrPtr,
        GTI_FDPSRAM0, 0x84, (GTI_BYTE *)&pOutMsg[L2_CONSTELL_BIN_OFS], nWordLen, GTI_DSP_A);
    ReadDspBlock(WhipGtiXcvrPtr,
        GTI_FDPSRAM0, 0x87, (GTI_BYTE *)&dwX, nWordLen, GTI_DSP_A);
    ReadDspBlock(WhipGtiXcvrPtr,
        GTI_FDPSRAM0, 0x88, (GTI_BYTE *)&dwY, nWordLen, GTI_DSP_A);

    SWAP_DWORD(&pOutMsg[L2_CONSTELL_X_OFS], &dwX);
    SWAP_DWORD(&pOutMsg[L2_CONSTELL_Y_OFS], &dwY);

    dwData = GTI_FALSE;
    WriteDspBlock(WhipGtiXcvrPtr,
        GTI_FDPSRAM0, 0x89, (GTI_BYTE *)&dwData, nWordLen, GTI_DSP_A);

    return L2_CONSTELL_Y_OFS + 4;

#else
    pOutMsg = pOutMsg;
    return -1;
#endif
}

void WhipResetEocDB(TXEOC *pDB)
{
     pDB->AutoLen = 0;
     pDB->EocLen = 0;
     pDB->EocCmd = 0;
#ifdef GTI_PAGE_I_BUILT
     pDB->EocOHCmd[0] = 0;
     pDB->EocOHCmd[1] = 0;
     pDB->EocOHCmd[2] = 0;
#endif
}

/****************************************************************************/
/* Procedure: WhipDispatchEocCommand                   Created: 04.06.98    */
/* --------------------------------                                         */
/* Description: This function calls EOC routine and sends it's reply.       */
/* ------------------------------------------------------------------------ */
int WhipDispatchEocCommand(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{

#if (defined(GTI_PROD_HDSL2) || defined (GTI_PROD_H2B1Q) || defined(GTI_PROD_HDSL4W) || \
     defined(GTI_PROD_N2B1Q) || defined(GTI_PROD_MSDSL))
#ifdef GTI_DEBUG
    struct EOC_BUFFER EOCtmp;
#endif
#endif
    int nLength;
#ifdef GTI_PROD_DMT
    RXEOC *pEoc;
#endif

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_EOC_COMMAND_OPC;

    switch(pInpMsg[L2_WT_EOCCMD_COD] & 0x0f)
    {
    case WHIP_DISABLE_EOC:
        WhipEocEnabled = GTI_FALSE;
        return -1;
#ifdef GTI_PROD_DMT
    case WHIP_XMTDATA_EOC:

        WhipEocCmd.tx.EocCmd = pInpMsg[L2_WT_EOCCMD_CMD];

        if(WhipEocCmd.tx.EocCmd == 0) /* DATA only*/
        {
            WhipEocCmd.tx.EocLen = 0;
            WhipEocCmd.tx.AutoLen = pInpMsg[L2_WT_EOCCMD_LEN];

            memcpy(WhipEocCmd.tx.AutoData, &pInpMsg[L2_WT_EOCCMD_DAT],
                WhipEocCmd.tx.AutoLen);
        }
        else /* non DATA only */
        {
            WhipEocCmd.tx.EocLen = pInpMsg[L2_WT_EOCCMD_LEN];
            WhipEocCmd.tx.AutoLen = 0;

            if((pInpMsg[L2_WT_EOCCMD_COD] & 0xf0) == 0xe0) /* BIS */
            {
#ifdef GTI_PAGE_I_BUILT
                WhipEocCmd.tx.EocOHCmd[0] = pInpMsg[L2_WT_EOCCMD_DAT];
                WhipEocCmd.tx.EocOHCmd[1] = pInpMsg[L2_WT_EOCCMD_DAT+1];
                WhipEocCmd.tx.EocOHCmd[2] = pInpMsg[L2_WT_EOCCMD_DAT+2];
#endif

                memcpy(WhipEocCmd.tx.EocData, &pInpMsg[L2_WT_EOCCMD_DAT+3],
                    WhipEocCmd.tx.EocLen);
            }
            else
            {
                memcpy(WhipEocCmd.tx.EocData, &pInpMsg[L2_WT_EOCCMD_DAT],
                    WhipEocCmd.tx.EocLen);
            }
        }
        break;

    case WHIP_POLLDATA_EOC:
        WhipEocCmd.tx.AutoLen = 0;
        break;
#endif

#if (defined(GTI_PROD_HDSL2) || defined (GTI_PROD_H2B1Q) || defined(GTI_PROD_HDSL4W) || \
     defined(GTI_PROD_N2B1Q))

#ifdef GTI_DEBUG
    case WHIP_XMTDATAMODE_EOC:
        pOutMsg[L2_TW_EOCCMD_COD] = WHIP_POLLDATAMODE_EOC;
        pOutMsg[L2_TW_EOCDAT_LEN] = 0;
        pOutMsg[L2_TW_EOCDAT_NXT]  = 0;

        if(pInpMsg[L2_WT_EOCCMD_COD] == 0xff)
        {
            TxEOCMsgBufferPtr = 0xffff;
            EOCtmp.EOCByte1 = 0xff;
            EOCtmp.EOCByte2 = 0x07;
            EOCtmp.EOCByte3 = 0x55;
        }
        else
        {
            TxEOCMsgBufferPtr = pMsg->Msg.EocCommand.length;
            EOCtmp.bId = pMsg->Msg.EocCommand.begin;
            memcpy(TxEOCMsgBuffer, &(pOutMsg[L2_TW_EOCDAT_DAT]), TxEOCMsgBufferPtr);

            EOCtmp.EOCByte1 = TxEOCMsgBuffer[--TxEOCMsgBufferPtr];
            EOCtmp.EOCByte2 = TxEOCMsgBuffer[--TxEOCMsgBufferPtr];
            EOCtmp.EOCByte3 = TxEOCMsgBuffer[--TxEOCMsgBufferPtr];
        }

        EOC2Xcvr(WhipGtiXcvrPtr, &EOCtmp);
        nLength = L2_TW_EOCDAT_DAT;
        break;

    case WHIP_POLLDATAMODE_EOC:
        pOutMsg[L2_TW_EOCDAT_LEN] = 0;
        pOutMsg[L2_TW_EOCCMD_COD] = WHIP_POLLDATAMODE_EOC;
        if(RxEOCMsgBufferPtr)
        {
            for(nLength = pMsg->Msg.EocCommand.begin + (pInpMsg[L2_WT_EOCCMD_LEN] << 0x08);
               (nLength < RxEOCMsgBufferPtr) && (pOutMsg[L2_TW_EOCDAT_LEN] < WHIP_MAX_EOC_DATA_LEN);
                nLength++)
            {
                pOutMsg[L2_TW_EOCDAT_DAT + pOutMsg[L2_TW_EOCDAT_LEN]++] = RxEOCMsgBuffer[nLength][0];
                pOutMsg[L2_TW_EOCDAT_DAT + pOutMsg[L2_TW_EOCDAT_LEN]++] = RxEOCMsgBuffer[nLength][1];
                pOutMsg[L2_TW_EOCDAT_DAT + pOutMsg[L2_TW_EOCDAT_LEN]++] = RxEOCMsgBuffer[nLength][2];
            }
            pOutMsg[L2_TW_EOCDAT_NXT] = nLength;
            if(nLength >= RxEOCMsgBufferPtr)
                RxEOCMsgBufferPtr = 0;
        }
        else
        {
            pOutMsg[L2_TW_EOCDAT_NXT] = 0;
        }
        nLength = L2_WT_EOCCMD_DAT + pOutMsg[L2_TW_EOCDAT_LEN];
        break;
#endif
#endif

    default:
        nLength = -1;
        break;
    }

#ifdef GTI_PROD_DMT
    if(WhipEocEnabled == GTI_FALSE)
    {
        WhipEocEnabled = GTI_TRUE;
    }

    if((WhipEocBuffer.EocCmd != 0) || (WhipEocBuffer.AutoLen > 0))
    {
        pOutMsg[L2_TW_EOCCMD_COD] = WHIP_BUFFER_VALID_EOC;
        nLength = L2_TW_EOCCMD_DAT + sizeof(RXEOC);
        memcpy(&pOutMsg[L2_TW_EOCCMD_DAT], &WhipEocBuffer, sizeof(RXEOC));
        WhipResetEocDB(&WhipEocBuffer);
    }
    else
    {
        pEoc = (RXEOC *)&pOutMsg[L2_TW_EOCCMD_DAT];
        WhipResetEocDB(pEoc);
        pEoc->Status = WhipEocBuffer.Status;
        pOutMsg[L2_TW_EOCCMD_COD] = WHIP_BUFFER_EMPTY_EOC;
        nLength = 3;
    }
#endif

    return nLength;
}

#ifdef GTI_PROD_DMT
/******************************************************************************/
/* Procedure: GTIEocServiceCall                             Created: 12.24.99 */
/* --------------------------------                                           */
/* Description: The routine can be called either from the IPU interrupt or    */
/*              by any polling mechanizm. The routine calls the EOC driver    */
/*              and if there is some valid information stores it in a buffer. */
/* -------------------------------------------------------------------------- */
void GTIEocServiceCall(void)
{
    extern void GTIEocProc(GTI_XCVR_INFO *GtiSysPtr, GTIEOC_CMD *eocPtr);

    /* Call EOC Proceesing to process EOC Information (T1.413) */
    GTIEocProc(WhipGtiXcvrPtr, &WhipEocCmd);

    WhipResetEocDB(&(WhipEocCmd.tx));

#if (defined(GTI_DSPVIKING) && defined(GTI_HOSTVSC) && defined(WHIP_ON_SERIAL))
#ifdef DEBUG_DSL
     {
        unsigned int i;

        for(i = 0; i < WhipEocCmd.rx.AutoLen; i ++)
        {
           NU_Printf("Autonomous data on EOC Channel: %x.\n", WhipEocCmd.rx.AutoData[i]);
        }
    }
#endif /* DEBUG_DSL */
    WhipEocCmd.rx.AutoLen = 0;
#else /* !(viking && Embedded host) */

    if (WhipEocCmd.tx.Status != GTI_EOC_COMPLETE)
    {
       ;
    }

    WhipFillEocExtBuffer(&WhipEocCmd);
#endif /* !(Viking && Embedded host) */
}

/******************************************************************************/
/* Procedure: WhipFillEocExtBuffer                          Created: 12.24.99 */
/* --------------------------------                                           */
/* Description: The routine fills the external Whip EOC buffer that stores    */
/*              last valid EOC information for future WHIP inquiry messages.  */
/* -------------------------------------------------------------------------- */
void WhipFillEocExtBuffer(GTIEOC_CMD *pEoc)
{
    WhipEocBuffer.Status = pEoc->rx.Status;

    if(pEoc->rx.EocCmd != 0)
    {
        WhipEocBuffer.EocCmd = pEoc->rx.EocCmd;
        WhipEocBuffer.EocLen = pEoc->rx.EocLen;
        /*
        WhipEocBuffer.EocOHCmd[0] = pEoc->rx.EocOHCmd[0];
        WhipEocBuffer.EocOHCmd[1] = pEoc->rx.EocOHCmd[1];
        WhipEocBuffer.EocOHCmd[2] = pEoc->rx.EocOHCmd[2];
        */

        if(pEoc->rx.EocLen > 0) /* Copy Eoc Data into external buffer */
        {
            memcpy(WhipEocBuffer.EocData, pEoc->rx.EocData,
                sizeof(WhipEocBuffer.EocData));
        }
    }

    if(pEoc->rx.AutoLen > 0) /* Copy Auto Data into external buffer */
    {
        WhipEocBuffer.AutoLen = pEoc->rx.AutoLen;
        memcpy(WhipEocBuffer.AutoData, pEoc->rx.AutoData,
            sizeof(WhipEocBuffer.AutoData));
    }
}
#endif

/******************************************************************************/
/* Procedure: WhipDispatchRandomReadMem                     Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: This function reads random addresses  and fills up the reply. */
/* -------------------------------------------------------------------------- */
int WhipDispatchRandomReadMem(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    int i, k, address;
    unsigned char channel;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_RANDOM_MEMREAD_OPC;

    channel = pInpMsg[L2_RNDREAD_CHNL_OFS];

    pOutMsg[L2_RNDREAD_CHNL_OFS] = pInpMsg[L2_RNDREAD_CHNL_OFS];
    pOutMsg[L2_RNDREAD_MEMT_OFS] = pInpMsg[L2_RNDREAD_MEMT_OFS];
    pOutMsg[L2_RNDREAD_NUMB_OFS] = pInpMsg[L2_RNDREAD_NUMB_OFS];

    /* Fill the reply structure in the following way    */
    /* 0 - address                                      */
    /* 1 - LS byte of the data                          */
    /* 2 - MS byte of the data                          */

    for(k=0, i=0; i < pInpMsg[L2_RNDREAD_NUMB_OFS]; i++)
    {
        address = pInpMsg[L2_RNDREAD_DATA_OFS + i];
        pOutMsg[L2_RNDREAD_DATA_OFS + k++] = (GTI_BYTE)address;

        ReadDspBlock(WhipGtiXcvrPtr, pInpMsg[L2_RNDREAD_MEMT_OFS],
            (GTI_BYTE)address,
            (GTI_BYTE *)&(pOutMsg[L2_RNDREAD_DATA_OFS + k]), 2, channel);
        k+= 2;
    }

    return L2_RNDREAD_DATA_OFS + pInpMsg[L2_RNDREAD_NUMB_OFS] * 3;
}

/******************************************************************************/
/* Procedure: WhipDispatchBertTest                          Created: 11.01.99 */
/* --------------------------------                                           */
/* Description: The routine will provide Bert test results for the Host       */
/*              Interface Program. If the State is not the Data mode,         */
/*              the result is zero                                            */
/* -------------------------------------------------------------------------- */
int WhipDispatchBertTest(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
#ifdef GTI_PROD_DMT
    GTI_WORD wActConfig[GTI_ACTUAL_CONFIGURATION_LEN*sizeof(GTI_BYTE)];
#endif

    GTI_WORD wBitRate, wOpState, wBertError[2];

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_BERT_TEST_OPC;

    if(Gti_SStatusXCVR(WhipGtiXcvrPtr, GTI_OP_STATE,
        &wOpState, sizeof(wOpState), pInpMsg[L2_BERTEST_CHNL_OFS]) == 0)
        return -1;

    if(Gti_SStatusXCVR(WhipGtiXcvrPtr, GTI_BERT_ERROR,
        wBertError, sizeof(wBertError), pInpMsg[L2_BERTEST_CHNL_OFS]) == 0)
        return -1;

#ifdef GTI_PROD_DMT
    if(Gti_SStatusXCVR(WhipGtiXcvrPtr, GTI_ACTUAL_CONFIGURATION,
        wActConfig, sizeof(wActConfig), pInpMsg[L2_BERTEST_CHNL_OFS]) == 0)
         return -1;

    if((wOpState == GTI_DL_TEST_OP) || (wOpState == GTI_LL_TEST_OP))
    {
        wBitRate = 13;
    }
    else
    {
        if(Gti_GetCurrentMode(WhipGtiXcvrPtr) == GTI_CO)
            wBitRate = wActConfig[GTI_LS0_BYTES_ACTUAL];
        else
            wBitRate = wActConfig[GTI_AS0_BYTES_ACTUAL];
    }

    if(wBertError[0] >= 0x7fff)
        wBertError[0] = 0xff00;

#else /* for non DMT products read the bitrate status directly */

    if(SStatusXCVR(WhipGtiXcvrPtr, GTI_ACTUAL_BITRATE_STATUS,
        &wBitRate, sizeof(wBitRate), pInpMsg[L2_BERTEST_CHNL_OFS]) == 0)
         return -1;

    if(wBertError[0] >= 0xff00)
        wBertError[0] = 0xff00;

#endif

    if(wBertError[1] != 0)
        pOutMsg[L2_BERTEST_SYNC_OFS] = GTI_TRUE;
    else
        pOutMsg[L2_BERTEST_SYNC_OFS] = GTI_FALSE;

    pOutMsg[L2_BERTEST_BITR_OFS] = LOBYTE(wBitRate);
    pOutMsg[L2_BERTEST_BITR_OFS + 1] = HIBYTE(wBitRate);

    pOutMsg[L2_BERTEST_ERR_OFS] = LOBYTE(wBertError[0]);
    pOutMsg[L2_BERTEST_ERR_OFS + 1] = HIBYTE(wBertError[0]);

    return L2_BERTEST_ERR_OFS + 2;
}

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
/******************************************************************************/
/* Procedure: WhipLocalBustTest                             Created: 12.24.99 */
/* --------------------------------                                           */
/* Description: The routine will call the local bus tester function that will */
/*              perform full or partial local bus testnseiver is set.         */
/* -------------------------------------------------------------------------- */
int WhipLocalBustTest(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    GTI_DWORD dwStart, dwEnd, dwCurAddr;
    GTI_WORD wChkSum;

    pOutMsg[L2_MSG_OPCODE_OFS] = L2_TEST_LOCAL_BUS_OPC;

    dwCurAddr = 0;
    wChkSum = 0;
    SWAP_DWORD(&dwStart, &pInpMsg[L2_LBUSTEST_BEG_OFS]);
    SWAP_DWORD(&dwEnd, &pInpMsg[L2_LBUSTEST_END_OFS]);

    pOutMsg[L2_TW_LBUSTEST_TYP_OFS] = pInpMsg[L2_WT_LBUSTEST_TYP_OFS];

    if(pInpMsg[L2_WT_LBUSTEST_TYP_OFS] == LB_TEST_TYPE)
    {
        dwStart = 0;

#if defined(GTI_USE_XDPXFR)
        if(dwEnd != 0)
        {
            dwEnd = 0x3ffff;
        }
#endif

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
        pOutMsg[L2_LBUSTEST_RES_OFS] = (GTI_BYTE)Gti_LocalBusTest(WhipGtiXcvrPtr, dwStart, dwEnd, &dwCurAddr);
#endif
    }
    else
    if(pInpMsg[L2_WT_LBUSTEST_TYP_OFS] == LB_LOAD_BUS_ADV)
    {
        if(dwStart == 0)
        {
#if defined(GTI_USE_XDPXFR)
            Gti_LocalBusActivate(WhipGtiXcvrPtr, GTI_FALSE);

            Gti_LocalBusDspLoadInit(WhipGtiXcvrPtr);
#endif

#if defined(GTI_USE_DMAXFR)
            Gti_SetLocalBusAnnexType(WhipGtiXcvrPtr->pChipInfo->pSystInfo,
                WhipGtiXcvrPtr->pChipInfo->wLbusID,
                (int)pInpMsg[L2_WT_LBUSTEST_COD_OFS]);

            Gti_ResetDMAEngine(WhipGtiXcvrPtr);
#endif
        }

#if defined(GTI_USE_XDPXFR)
        if(dwStart == 0x03)
        {
            Gti_LocalBusActivate(WhipGtiXcvrPtr, GTI_TRUE);
            pOutMsg[L2_LBUSTEST_RES_OFS] = GTI_COMPLETE_PASS;
        }
        else
#endif
        {
            pOutMsg[L2_LBUSTEST_RES_OFS] =
                (GTI_BYTE)Gti_LocalBusDspLoad(WhipGtiXcvrPtr, (GTI_WORD)dwStart,
                &pInpMsg[L2_WT_LBUSTEST_DAT_OFS], (GTI_WORD)(dwEnd), &wChkSum);
        }

        dwEnd = wChkSum;
        dwCurAddr = dwStart;
    }
    else
    if(pInpMsg[L2_WT_LBUSTEST_TYP_OFS] == LB_CHKSUM_TYPE)
    {
        pOutMsg[L2_LBUSTEST_RES_OFS] =
            (GTI_BYTE)Gti_LocalBusChkSum(WhipGtiXcvrPtr, (GTI_BYTE)dwEnd, &wChkSum, 0, 0);

#ifdef GTI_USE_DMAXFR
        dwEnd = Gti_GetDSPLoadedCodeLength(WhipGtiXcvrPtr, 0);
#endif
        dwCurAddr = wChkSum;
    }
#ifdef GTI_USE_DMAXFR
    else
    if(pInpMsg[L2_WT_LBUSTEST_TYP_OFS] == LB_LOAD_BUS)
    {
        if(dwStart == 0)
            dwStart = GTI_LB_CONFIG_ANNEX_A;

        Gti_SetLocalBusAnnexType(WhipGtiXcvrPtr->pChipInfo->pSystInfo,
            WhipGtiXcvrPtr->pChipInfo->wLbusID,
            (int)dwStart);

        pOutMsg[L2_LBUSTEST_RES_OFS] =
            (GTI_BYTE)Gti_InitLBUS(WhipGtiXcvrPtr->pChipInfo->pSystInfo,
            WhipGtiXcvrPtr->pChipInfo->wLbusID);

        dwEnd = (GTI_DWORD)GTI_DXL_TOTAL_LENGTH * 2;
    }
#endif
    SWAP_DWORD(&pOutMsg[L2_LBUSTEST_BEG_OFS], &dwCurAddr);
    SWAP_DWORD(&pOutMsg[L2_LBUSTEST_END_OFS], &dwEnd);

    return L2_TW_LBUSTEST_TYP_OFS;
}
#endif

/******************************************************************************/
/* Procedure: WhipDispatchActConfig                         Created: 12.24.99 */
/* --------------------------------                                           */
/* Description: The routine will provide Active configuration info for the    */
/*              Host Interface Program. If the Active Transeiver is set to    */
/*              any number greater than -1, the newtranseiver is chosen.      */
/* -------------------------------------------------------------------------- */
int WhipDispatchActConfig(GTI_BYTE *pInpMsg, GTI_BYTE *pOutMsg)
{
    pOutMsg[L2_MSG_OPCODE_OFS] = L2_ACTIVE_CONFIG_OPC;

    if((pInpMsg[L2_WT_ACTCONF_XCVR_OFS] != 0xff) &&
        (pInpMsg[L2_WT_ACTCONF_XCVR_OFS] < GTI_TOTAL_NUM_OF_XCVRS))
    {
        WhipActiveXcvr = pInpMsg[L2_WT_ACTCONF_XCVR_OFS];
        WhipGtiXcvrPtr = &GTIXcvr[WhipActiveXcvr];

#ifdef GTI_DSPQUAD
        switch(WhipGtiXcvrPtr->dsp_B_cs)
        {
        case GTI_CORE_0_MSK:
            WRITE_DSP_REG(WhipGtiXcvrPtr, 0x7b, 0x00);
            break;
        case GTI_CORE_1_MSK:
            WRITE_DSP_REG(WhipGtiXcvrPtr, 0x7b, 0x40);
            break;
        case GTI_CORE_2_MSK:
            WRITE_DSP_REG(WhipGtiXcvrPtr, 0x7b, 0x80);
            break;
        case GTI_CORE_3_MSK:
            WRITE_DSP_REG(WhipGtiXcvrPtr, 0x7b, 0xc0);
            break;
        default:
            break;
        }
#endif

#ifdef GTI_DSPOCTANE
        CONFIGURE_DSP(WhipGtiXcvrPtr, GTI_DSP_TOP,  GTI_NO_AI);
        WRITE_DSP_ADDR(WhipGtiXcvrPtr, GTI_DSP_TOP, 0x93, GTI_IREGA);
        WRITE_DSP_WORD(WhipGtiXcvrPtr, GTI_DSP_TOP, WhipGtiXcvrPtr->xcvrId, 0x00);
#endif
    }

    pOutMsg[L2_TW_ACTCONF_XCVR_OFS] = WhipActiveXcvr;
    pOutMsg[L2_WT_ACTCONF_MAX_OFS] = GTI_TOTAL_NUM_OF_XCVRS;
    pOutMsg[L2_WT_ACTCONF_FUT1_OFS] = GTI_NUM_CORES_PER_XCVR;
    pOutMsg[L2_WT_ACTCONF_FUT2_OFS] = Gti_GetActiveProjectId(WhipGtiXcvrPtr);

    if(pInpMsg[L2_WT_ACTCONF_TARG_OFS] != 0)
        memcpy(WhipTargetName, &pInpMsg[L2_WT_ACTCONF_TARG_OFS], 6);

    return L2_WT_ACTCONF_FUT2_OFS;
}

/******************************************************************************/
/* Procedure: WhipSendMsg                                   Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: Thie function builds and send a  message to the periphiral    */
/*              master side.                                                  */
/* -------------------------------------------------------------------------- */
void
WhipSendMsg( GTI_BYTE  opcode,                   /* control opcode       */
             GTI_BYTE  type,                     /* message framing type */
             void      *pMsgBody,
             GTI_BYTE  length
)
{
#if ( (!defined(GTI_USE_INTERNAL_TARGET)) && (!defined(GTI_HOSTVSC)) )
    int i, realLength;
    GTI_BYTE outBuffer[520], *outBufPtr, checkSum = 0;

    if(length <= L2_MAX_MSG_LENGTH - 1)
    {
        outBufPtr = outBuffer;       /* initialize the atart ptr to Buffer */

        *outBufPtr++ = L2_DLE;       /* encode our message Start of Msg    */
        *outBufPtr++ = L2_STX;

        *outBufPtr = (GTI_BYTE)(length + 1); /* INFO length, includes opcode       */
        if(*outBufPtr++ == L2_DLE)
            *outBufPtr++  = L2_DLE;

        *outBufPtr = WhipMessageID++;   /* Assign msg identifiction number */

        if (*outBufPtr++ == L2_DLE)
            *outBufPtr++  = L2_DLE;

        *outBufPtr = WhipCurrentSessionID;
        if (*outBufPtr++ == L2_DLE)
            *outBufPtr++  = L2_DLE;

        *outBufPtr = (GTI_BYTE)(type << 0x04);

        if (*outBufPtr++ == L2_DLE)
            *outBufPtr++  = L2_DLE;

        *outBufPtr++ = opcode;        /* message opcode, used for message */
        if(opcode == L2_DLE)          /* dispatch at the destination side */
            *outBufPtr++ = L2_DLE;

        for (i=0; i<length; i++)      /* build a message, inserting DLE   */
        {
            *outBufPtr++ = ((GTI_BYTE *)pMsgBody)[i];

            if ( ((GTI_BYTE *)pMsgBody)[i] == L2_DLE)
                *outBufPtr++ = L2_DLE;
        }

        realLength = outBufPtr - outBuffer;

        for (i=2; i<realLength; i++)
            checkSum ^= outBuffer[i];

        *outBufPtr++ = checkSum;
        if(checkSum == L2_DLE)
            *outBufPtr++ = L2_DLE;

        *outBufPtr++ = L2_DLE;
        *outBufPtr++ = L2_ETX;

        Gti_WhipTx(outBuffer, (short)(outBufPtr - outBuffer));
    }
#else
    int i;
    GTI_BYTE outBuffer[520], *outBufPtr;

    outBufPtr = outBuffer;

    *outBufPtr++ = (GTI_BYTE)(length + 1); /* INFO length, includes opcode       */
    *outBufPtr++ = WhipMessageID++;   /* Assign msg identifiction number */
    *outBufPtr++ = WhipCurrentSessionID;
    *outBufPtr++ = 0;
    *outBufPtr++ = opcode;

    for (i=0; i < length; i++)      /* build a message, inserting DLE   */
        *outBufPtr++ = ((GTI_BYTE *)pMsgBody)[i];

  //FIXMEKJP  Gti_WhipTx(outBuffer, (short)(length + 5));
  /* KJP Compiler error because this is not defined for this #define */

#endif
}

#ifndef GTI_USE_INTERNAL_TARGET

/******************************************************************************/
/* Procedure: WhipPrtclSendControlMsg                       Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: Thie function builds and send a control message to the periph */
/*              iral master side. The message can be ACK or NACK              */
/*              and contanes the message ID.                                  */
/* -------------------------------------------------------------------------- */
void
WhipPrtclSendControlMsg (
        int Opcode,                                         /* control opcode */
        GTI_BYTE id                                         /* message id     */
)
{
    GTI_BYTE outBuffer[20], *outBufPtr = outBuffer, checkSum = 0;

#if (!(defined(GTI_HOSTVSC) && defined(GTI_DSPVIKING)) )
    *outBufPtr++ = L2_DLE;
    *outBufPtr++ = L2_STX;
#endif

    *outBufPtr++ = 1;                             /* INFO length: only opcode */
    checkSum ^= 1;

    *outBufPtr = id;
    checkSum ^= id;

#if (!(defined(GTI_HOSTVSC) && defined(GTI_DSPVIKING)) )
    if (*outBufPtr++ == L2_DLE)
    {
        *outBufPtr++  = L2_DLE;
        checkSum ^= L2_DLE;
    }
#endif

    /* destination address of the message and the type  */
    *outBufPtr = 0;
    checkSum ^= *outBufPtr++;

    *outBufPtr++ = 0;      /* session ID            */

    *outBufPtr++ = (GTI_BYTE)Opcode; /* opcode of the message */
    checkSum ^= (GTI_BYTE)Opcode;

    *outBufPtr++ = checkSum; /* check sum of the message */
#if (!(defined(GTI_HOSTVSC) && defined(GTI_DSPVIKING)) )
    if(checkSum == L2_DLE)
        *outBufPtr++ = L2_DLE;
#endif

    *outBufPtr++ = L2_DLE;
    *outBufPtr++ = L2_ETX;

    Gti_WhipTx(outBuffer, (short)(outBufPtr - outBuffer));
}
/* End of p_PrtclSendControlMsg                                               */
/******************************************************************************/

/******************************************************************************/
/* Procedure: WhipRetransmitLastMsg                         Created: 04.06.98 */
/* --------------------------------                                           */
/* Description: The procedure will try to send last message if it is still    */
/*              stored in the buffer. The retransmittion can be performed only*/
/*              MAX_RETRANSMITTION_TIMES.                                     */
/* -------------------------------------------------------------------------- */
void WhipRetransmitLastMsg(GTI_BYTE id)
{
    GTI_BYTE GTI_ADDRESS_QUAL *pLastMsg;

    if(WhipRexmtCounter > L2_MAX_RETRIES)
        return;

    pLastMsg = (GTI_BYTE GTI_ADDRESS_QUAL *)&(Whip_xmt_buffer[WhipLastSOMidx + 2]);

    /* Check if the stored message is our ID messaage */
    if(pLastMsg[L2_HEADER_MID_OFS] != id)
        return;

    Gti_WhipTx((GTI_BYTE GTI_ADDRESS_QUAL *)pLastMsg - 2,
                       (short)(Whip_xmt_total_idx - WhipLastSOMidx) );
}
/* End of RetransmitLastMsg                                                   */
/******************************************************************************/

#ifndef GTI_DEBUG
/******************************************************************************/
/* Procedure: Gti_WhipRx                                    Created: 04.06.00 */
/* --------------------------------                                           */
/* Description:  This function should be called in the interrupt service      */
/*               routine with the byte received from the serial port. The     */
/*               function accepts two parameters: a pointer to the array of   */
/*               byte that we need to process and the number of bytes.        */
/* -------------------------------------------------------------------------- */
void Gti_WhipRx(GTI_BYTE * ptrSymbArray, GTI_WORD nLength)
{
    GTI_BYTE bSymbol;
    GTI_WORD nIndex;

    for(nIndex = 0; nIndex < nLength; nIndex++)
    {
        bSymbol = ptrSymbArray[nIndex];

        switch(Whip_state)
        {
        case L2_STATE_ACTIVE:  /* receiving a message */
            switch(bSymbol)
            {
            case L2_DLE:    /* DLE is a control sequence */
                Whip_state = L2_STATE_DLE_STOP;
                break;

            default:  /* any other character goes into the buffer */
                if(Whip_rcv_bufidx < L2_MAX_MSG_LENGTH) /* check buffer overflow */
                {
                    Whip_rcv_buffer[Whip_rcv_bufidx++] = bSymbol;
                    Whip_chksum ^= bSymbol;
                }
                else
                    WhipInitSM();
                break;
            }
            break;

        case L2_STATE_DLE_START:  /* waiting for a start of transmission */
            if(bSymbol == L2_STX)
                Whip_state = L2_STATE_ACTIVE;
            else
                WhipInitSM();
            break;

        case L2_STATE_IDLE:             /* wait for a DLE */
            if(bSymbol == L2_DLE)
                Whip_state = L2_STATE_DLE_START;
            break;

        case L2_STATE_DLE_STOP:         /* waiting for a DLE or STX */
            switch(bSymbol)
            {
            case L2_ETX:

                /* copy the current checksum over the checksum in the buffer */
                /* It will be checked to see if it is 0 or L2_DLE.           */
                Whip_rcv_buffer[Whip_rcv_bufidx-1] = Whip_chksum;

                /* Notify WHIP task about a new message */
                Gti_WhipRxNotify(Whip_rcv_buffer, Whip_rcv_bufidx);
                /*-------------------------------------------*/
                WhipInitSM();  /* re-initialize state machine */
                break;

            case L2_DLE:
                /* two DLEs so put one DLE into the buffer, no checksum */
                if(Whip_rcv_bufidx < L2_MAX_MSG_LENGTH)
                {
                    Whip_rcv_buffer[Whip_rcv_bufidx++] = bSymbol;
                    Whip_state = L2_STATE_ACTIVE;
                }
                else
                    WhipInitSM();
                break;

            default:
                WhipInitSM();
                break;
            }
            break;

        default:
            WhipInitSM();
            break;
        }
    }
}

#if (!(defined(GTI_HOSTVSC) && defined(GTI_DSPVIKING)) )
/*****************************************************************************/
/* Procedure: Gti_WhipRxNotify                             Created: 04.06.00 */
/* --------------------------------                                          */
/* Description: The procedure shoud notify the WHIP task about a new message */
/*              Create your own body of notification here.                   */
/* ------------------------------------------------------------------------- */
void Gti_WhipRxNotify(GTI_BYTE *ptrMsg, GTI_WORD wLength)
{
   (void)ptrMsg;
   (void)wLength;
#ifdef GTI_ISOS
   Gti_WhipRxHandler( ptrMsg, wLength );
#endif /* ISOS */
}
#endif /* Viking Embeded Host */

#endif /* !GTI_DEBUG */
#endif /* !USE_INTERNAL_TARGET */


