/*+I*************************************************************************
 * TriplePoint Debug Interface Definition
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *
 * This source code may be used by anyone provided the TriplePoint copyright
 * is maintained in any derivitive works.
 *-I*************************************************************************/
/****************************************************************************
@doc INTERNAL TpiDebug TpiDebug_h

@module TpiDebug.h |

    This module defines a set of debug macros that can be used to instrument
    C/C++ source code.  The macros defined here are only compiled in the code
    when compiler defines DEBUG, _DEBUG, or DBG != 0.  This allows you to
    add as much instrumentation to your code as you like, and then have it
    all compiled out for the final release.

@head3 Contents |
@index class,mfunc,func,msg,mdata,struct,enum | TpiDebug_h

@end
 ****************************************************************************/

#ifndef _TPIDEBUG_H
#define _TPIDEBUG_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

//#define MYDEBUG 
/* NT builds define DBG=0 or DBG=1, so we use that throughout */
#if (DBG != 0) || defined(DEBUG) || defined(_DEBUG)
# undef  DBG
# define DBG 1
#else
# undef  DBG
# define DBG 0
#endif

/* In case these aren't defined in the current environment. */
# if !defined(IN)
#  define IN
#  define OUT
# endif

#if defined(macintosh)
#   define __cdecl
# ifdef USE_DRIVERLIB
#   define DbgPrintMac  SysDebugStr
#  else
#   define DbgPrintMac  DebugStr
# endif
#elif defined(linux)
#   define __cdecl
#endif

/****************************************************************************/
#if DBG /* When (DBG != 0) we enable all the debug macros and functions. */

/* Generic debug filtering flags */
# define DBG_ERROR_ON       0x00000001L /* Should never happen */
# define DBG_WARNING_ON     0x00000002L /* Unusual event */
# define DBG_NOTICE_ON      0x00000004L /* Interesting information */
# define DBG_TRACE_ON       0x00000008L /* Function tracing */
# define DBG_PARAM_ON       0x00000010L /* Display interface parameters */
# define DBG_INFO_ON        0x00000020L /* Display informational messages */

# define DBG_DATA_ON        0x00000040L /* Data tracing */
# define DBG_SPINLOCK_ON    0x00000080L /* SpinLock tracing */
# define DBG_LOG_ON         0x00000100L /* Log to circular memory queue */
# define DBG_FILE_ON        0x00000200L /* Output to file (if supported) */
# define DBG_MESSAGE_ON     0x00000400L /* Output to MessageBox (if supported) */
# define DBG_MEMORY_ON      0x00001000L /* Trace memory allocation and free */
# define DBG_TERSE_ON       0x00002000L /* Display minimum information */
# define DBG_VERBOSE_ON     0x00004000L /* Display maximum information */
# define DBG_BREAK_ON       0x00008000L /* Breakpoint on error */
# define DBG_ALL_ON         0xFFFFFFFFL /* Enable all debug */

/* Application specific filtering flags */
# define DBG_GEN_SYS        0x00000001L /* Generic subsystem */
# define DBG_HW_SYS         0x00000002L /* Hardware subsystem */
# define DBG_ISR_SYS        0x00000004L /* Interrupt handler */
# define DBG_IOCTL_SYS      0x00000008L /* Ioctl interface */
# define DBG_TX_SYS         0x00000010L /* Transmit handler */
# define DBG_RX_SYS         0x00000020L /* Receive handler */
# define DBG_CALL_SYS       0x00000040L /* Call management */
# define DBG_INIT_SYS       0x00000080L /* Initialization handler */
# define DBG_HALT_SYS       0x00000100L /* Termination handler */
# define DBG_RESET_SYS      0x00000200L /* Reset handler */
# define DBG_TRAP_SYS       0x00000400L /* Trap handler */
# define DBG_REG_SYS        0x00000800L /* Registry interfaces */
# define DBG_ALL_SYS        0xFFFFFFFFL /* Enable all sub-systems */

# ifndef   DBG_ENABLE_DEBUGGER  /* Enable output to debugger */
#  if !defined(UNDER_CE)
#   define DBG_ENABLE_DEBUGGER  1
#  else
#   define DBG_ENABLE_DEBUGGER  0
#  endif
# endif /* DBG_ENABLE_DEBUGGER */

# ifndef   DBG_ENABLE_FILE      /* Enable output to file */
#  if defined(UNDER_CE)
#   define DBG_ENABLE_FILE      DBG_FILE_ON
#  else
#   define DBG_ENABLE_FILE      0
#  endif
# endif /* DBG_ENABLE_FILE */

# ifndef   DBG_ENABLE_MESSAGE   /* Enable output to MessageBox */
#  if defined(WIN32) && (defined(_EXE_) || defined(_DLL_))
#   define DBG_ENABLE_MESSAGE   DBG_MESSAGE_ON
#  else
#   define DBG_ENABLE_MESSAGE   0
#  endif
# endif

# ifndef  DBG_SYSTEMS
#  define DBG_SYSTEMS       DBG_ALL_SYS
# endif

# ifndef  DBG_DEFAULTS
#  define DBG_DEFAULTS      (DBG_BREAK_ON | DBG_ERROR_ON | DBG_WARNING_ON | DBG_ENABLE_FILE | DBG_ENABLE_MESSAGE)
# endif

# ifndef  DBG_MESSAGES
#  define DBG_MESSAGES      (DBG_BREAK_ON | DBG_ERROR_ON)
# endif

# ifndef  DBG_UNUSED
#  define DBG_UNUSED        {__FUNC__ ? __MASK__=__MASK__ : 0;}
# endif

# ifdef  _NTDDK_
#  define PROCESSOR_BREAKPOINT  DbgBreakPoint()
# elif defined(_M_IX86) || defined(_M_I86)
#  define PROCESSOR_BREAKPOINT  __asm{int 3}
# elif defined(UNDER_CE)
#  define PROCESSOR_BREAKPOINT
# elif defined(macintosh)
extern void DbgBreakPoint(void);
#  define PROCESSOR_BREAKPOINT  DbgBreakPoint()
# elif defined(linux)
#  if defined(__KERNEL__)
#   define KDB_BREAKPOINT       asm volatile ("int $129");
//KKA Commented #   define PROCESSOR_BREAKPOINT KDB_BREAKPOINT
#   define PROCESSOR_BREAKPOINT 
#  else /* __KERNEL__ */
//KKA Commented #   define PROCESSOR_BREAKPOINT abort()
#   define PROCESSOR_BREAKPOINT 
#  endif /* __KERNEL__ */
# else
#  define PROCESSOR_BREAKPOINT
#  pragma message("WARNING: You need to define PROCESSOR_BREAKPOINT for this processor!")
# endif

# ifndef  DBG_TRAP
#  define DBG_TRAP          PROCESSOR_BREAKPOINT
# endif

# ifndef  DBG_BREAKPOINT
#  define DBG_BREAKPOINT    PROCESSOR_BREAKPOINT
# endif

# ifndef  DBG_FILE_NAME
#  define DBG_FILE_NAME     "\\TEMP\\%s.txt"   /* %s replaced with DBG_NAME */
# endif

# ifndef  DBG_BUFFER_LEN
#  define DBG_BUFFER_LEN    1024            /* Max debug output string length. */
# endif

# ifndef  DBG_LOG_SIZE
#  define DBG_LOG_SIZE      60              /* Maximum number of bytes per entry. */
# endif

# ifndef  DBG_LOG_ENTRIES
#  define DBG_LOG_ENTRIES   512             /* Maximum number of FIFO log entries. */
# endif

typedef struct DBG_STRUCT *PDBG_STRUCT;
typedef void (* DBG_FUNC_TYPE)(PDBG_STRUCT);

typedef struct DBG_LOG
{
    unsigned short          Flags;
    unsigned short          NumBytes;
    unsigned char           Buffer[DBG_LOG_SIZE];

}   DBG_LOG, * PDBG_LOG;

/*
// Define an access macro for reading each debug field.
// Make sure you keep the access macros in sync with the DBG_FIELDS macro.
*/
# define DBG_FLAGS(A)       (A)->DbgFlags
# define DBG_ID(A)          (A)->DbgId
# define DBG_NAME(A)        (A)->DbgName
# define DBG_NAME_BUFFER(A) (A)->DbgNameBuffer
# define DBG_SYSTEM(A)      (A)->DbgSystem
# define DBG_LEVEL(A)       (A)->DbgLevel
# define DBG_LPARAM(A)      (A)->DbgParam
# define DBG_PRINT_FUNC(A)  (A)->DbgPrintFunc
# define DBG_ALT_FUNC(A)    (A)->DbgAltFunc
# define DBG_ALT_FLAGS(A)   (A)->DbgAltFlags

/*
// Always declare these debug fields at the top of your context specific data
// structure so your structure can be cast to a DBG_STRUCT when debugging.
// The global context structure pointed to by gDbgInfo can be used in those
// routines that don't have access to your context specific debug structure.
*/
# define DBG_FIELDS \
    unsigned long           DbgFlags;\
    unsigned long           DbgId;\
    char *                  DbgName;\
    unsigned long           DbgSystem;\
    unsigned long           DbgLevel;\
    void *                  DbgParam;\
    DBG_FUNC_TYPE           DbgPrintFunc;\
    DBG_FUNC_TYPE           DbgAltFunc;\
    unsigned long           DbgAltFlags;

# define DBG_FIELD_NAME_BUFFER(L) \
    char                    DbgNameBuffer[L];

/*
// This is the global debug structure which can be used when you have not
// created a context specific debug structure.  You can use pass gDbgInfo
// to any of the
*/
typedef struct DBG_STRUCT
{
    DBG_FIELDS              /* Declare debug fields at top of structure. */

    /*
    // The global gDbgInfo structure also contains the other buffers and
    // variables used by the debug library and macros functions.  These
    // extra fields are only accessed via the gDbgInfo pointer and are
    // not part of the context specific data structure.
    // These fields should only be used by the debug libary and macros.
    */

# define DBG_PUT_MASK(M)    ((gDbgInfo->TmpMask = (M)) != 0)
# define DBG_PUT_INFO(A)    (gDbgInfo->TmpInfo = (PDBG_STRUCT)(A))
# define DBG_GET_MASK()     gDbgInfo->TmpMask
# define DBG_GET_INFO()     gDbgInfo->TmpInfo
    struct DBG_STRUCT *     TmpInfo;    /* context saved here before output */
    unsigned long           TmpMask;    /* mask saved here before output */

    const char *            Systems[33];    /* 32 bits plus one for zero */

    unsigned long           BufferIndex;
    unsigned char           Buffer[DBG_BUFFER_LEN];
# if defined(UNDER_CE) || defined(UNICODE)
    unsigned short          BufferW[DBG_BUFFER_LEN];
# endif

    unsigned long           LogIndex;
    struct DBG_LOG          LogBuffer[DBG_LOG_ENTRIES];

}   DBG_STRUCT;

extern PDBG_STRUCT          gDbgInfo;       /* Always a global debug context */
# define DbgInfo            gDbgInfo        /* For compatability */

extern void DbgOpenFile(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void DbgCloseFile(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void DbgInit(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void DbgHalt(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void DbgSystemDefine(
    IN const char *     name,
    IN unsigned long    mask
    );

extern const char * DbgSystemName(
    IN unsigned long    mask
    );

#if !defined(_DBGNT_) && !defined(NDIS_WIN)
#if defined(linux)
extern unsigned long __cdecl DbgPrint(
    IN const char *     format,
    ...
    ) __attribute((format (printf, 1, 2)));
#else
extern unsigned long __cdecl DbgPrint(
    IN char *           format,
    ...
    );
#endif
#endif /* _DBGNT_ */

extern void DbgMessageOut(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void DbgPrintOut(
    IN PDBG_STRUCT      pDbgInfo
    );

extern void __cdecl DbgPrintf(
    IN const char *     format,
    ...
    );

extern void DbgPrintData(
    IN PDBG_STRUCT      pDbgInfo,
    IN unsigned char *  data,
    IN unsigned long    numBytes,
    IN unsigned long    offset
    );

extern void DbgQueueData(
    IN unsigned char *  data,
    IN unsigned long    numBytes,
    IN unsigned short   flags
    );

extern char * DbgDataToString(
    IN unsigned char *  pData,
    IN unsigned long    offset,
    IN unsigned long    length,
    IN char *           pBuffer
    );

# define _DBG_ENTER_STR ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
# define _DBG_LEAVE_STR "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"

# define DBG_DEF_SYSTEM(N,M) DbgSystemDefine(N,M)
# define DBG_SYSNAME()       DbgSystemName(__MASK__)

# ifndef  DBG_INIT_SYSTEMS
#  define DBG_INIT_SYSTEMS() {\
                            DBG_DEF_SYSTEM("", DBG_GEN_SYS);\
                            DBG_DEF_SYSTEM("HW:", DBG_HW_SYS);\
                            DBG_DEF_SYSTEM("ISR:", DBG_ISR_SYS);\
                            DBG_DEF_SYSTEM("IOCTL:", DBG_IOCTL_SYS);\
                            DBG_DEF_SYSTEM("TX:", DBG_TX_SYS);\
                            DBG_DEF_SYSTEM("RX:", DBG_RX_SYS);\
                            DBG_DEF_SYSTEM("CALL:", DBG_CALL_SYS);\
                            DBG_DEF_SYSTEM("INIT:", DBG_INIT_SYS);\
                            DBG_DEF_SYSTEM("HALT:", DBG_HALT_SYS);\
                            DBG_DEF_SYSTEM("RESET:", DBG_RESET_SYS);\
                            DBG_DEF_SYSTEM("TRAP:", DBG_TRAP_SYS);}
# endif

# ifndef  DBG_ALTERNATE
#  define DBG_ALTERNATE(A,M,F) {\
                            DBG_ALT_FLAGS(A) = DBG_ENABLE_MESSAGE ? \
                                               (M | DBG_MESSAGE_ON):\
                                               (M & ~DBG_MESSAGE_ON);\
                            DBG_ALT_FUNC(A)  = F;}
# endif

# ifndef  DBG_INIT
#  define DBG_INIT(A,N,I,M,S) {\
                            DBG_INIT_SYSTEMS();\
                            DBG_NAME(A)       = N;\
                            DBG_ID(A)         = I;\
                            DBG_FLAGS(A)      = DBG_ENABLE_FILE ? \
                                                ((M) | DBG_FILE_ON):\
                                                ((M) & ~DBG_FILE_ON);\
                            DBG_SYSTEM(A)     = S;\
                            DBG_PRINT_FUNC(A) = DbgPrintOut;\
                            DBG_LEVEL(A)      = 0;\
                            DBG_LPARAM(A)     = 0;\
                            DBG_ALTERNATE(A, DBG_MESSAGES, DbgMessageOut);\
                            DbgInit((PDBG_STRUCT)(A));\
                            DBG_PREFIX(A,"DBG_INIT");\
                            DBG_PRINT(A,("DebugFlags @0x%lX =0x%lX\n",\
                                         (unsigned long)(A), DBG_FLAGS(A)));\
                            }
# endif

# ifndef  DBG_HALT
#  define DBG_HALT(A)       {\
                            DBG_PREFIX(A,"DBG_HALT");\
                            DBG_PRINT(A,("DebugFlags @0x%lX =0x%lX\n",\
                                         (unsigned long)(A), DBG_FLAGS(A)));\
                            DbgHalt((PDBG_STRUCT)(A));\
                            }
# endif

# define DBG_FUNC(F)        static const char __FUNC__[] = F; \
                            static unsigned long __MASK__ = DBG_GEN_SYS;
# define DBG_FUNCX(F,M)     static const char __FUNC__[] = F;\
                            static unsigned long __MASK__ = M;

# ifndef  DBG_PRINT
#  define DBG_PRINT(A,S)    {DBG_PUT_INFO(A); printk S;}
# endif

# ifndef  DBG_MODULE
#  if defined(_NTDDK_) && defined(NT_INCLUDED) && !defined(BINARY_COMPATIBLE)
#   define _DBG_PROC()      KeGetCurrentProcessorNumber()
#   define DBG_MODULE(A)    DBG_PRINT(A,("%ld:%s:%ld:",_DBG_PROC(),DBG_NAME(A),DBG_ID(A)))
#  else
#   define DBG_MODULE(A)    DBG_PRINT(A,("%s:%ld:",DBG_NAME(A),DBG_ID(A)))
# endif
# endif

# ifndef  DBG_PREFIX
#  define DBG_PREFIX(A,S)   {DBG_MODULE(A);\
                             DBG_PRINT(A,("%s:%s ",S,__FUNC__));}
# endif

# define _DBG_ENTER(A)      {DBG_MODULE(A);\
                             DBG_PRINT(A,("%.*s:%s",(int)(++DBG_LEVEL(A)),_DBG_ENTER_STR,__FUNC__));}
# define _DBG_LEAVE(A)      {DBG_MODULE(A);\
                             DBG_PRINT(A,("%.*s:%s",(int)DBG_LEVEL(A)--,_DBG_LEAVE_STR,__FUNC__));}

//# define _DBG_ON(A,M)       ((DBG_FLAGS(A) & (M)) && (DBG_PUT_MASK(M)) && (__MASK__ == __MASK__))
# define _DBG_ON(A,M)       ((DBG_FLAGS(A) & (M)))

# define _DBG_SYS(A,M)      (_DBG_ON(A,M) && (DBG_SYSTEM(A) & __MASK__))

# define DBG_DISPLAY(A,S)   {if (_DBG_ON(A,0)) \
                                {DBG_MODULE(A);DBG_PRINT(A,S);}}

# define DBG_MESSAGE(A,S)   {if (_DBG_ON(A,DBG_MESSAGE_ON)) \
                                {DBG_MODULE(A);DBG_PRINT(A,S);}}

# define DBG_BREAK(A)       {if (_DBG_ON(A,DBG_BREAK_ON)) \
                                {DBG_PREFIX(A,"BREAK");DBG_PRINT(A,("LINE=%d\n",__LINE__));\
                                 DBG_BREAKPOINT;}}

# define DBG_ERROR(A,S)     {if (_DBG_ON(A,DBG_ERROR_ON)) \
                                {DBG_PREFIX(A,"ERROR");DBG_PRINT(A,S);DBG_TRAP;}}

# define DBG_WARNING(A,S)   {if (_DBG_ON(A,DBG_WARNING_ON)) \
                                {DBG_PREFIX(A,"WARNING");DBG_PRINT(A,S);}}

# define DBG_NOTICE(A,S)    {if (_DBG_SYS(A,DBG_NOTICE_ON)) \
                                {DBG_PREFIX(A,"NOTICE");DBG_PRINT(A,S);}}

# define DBG_FILTER(A,M,S)  {if (_DBG_SYS(A,M)) \
                                {DBG_PREFIX(A,DBG_SYSNAME());DBG_PRINT(A,S);}}

# define DBG_DATA(A,M,B,N,O) {if (_DBG_SYS(A,(M)|DBG_DATA_ON))\
                                {DbgPrintData((PDBG_STRUCT)(A),B,N,O);}}

# define DBG_ENTER(A)       {if (_DBG_SYS(A,DBG_TRACE_ON)) \
                                 {_DBG_ENTER(A);DBG_PRINT(A,("\n"));}}

# define DBG_LEAVE(A)       {if (_DBG_SYS(A,DBG_TRACE_ON)) \
                                 {_DBG_LEAVE(A);DBG_PRINT(A,("\n"));}}

# define DBG_RETURN(A,V)    {if (_DBG_SYS(A,DBG_TRACE_ON)) \
                                 {_DBG_LEAVE(A);DBG_PRINT(A,("(0x%lX)\n",(long)(V)));}}

# define DBG_RETURNF(A,S)   {if (_DBG_SYS(A,DBG_TRACE_ON)) \
                                {DBG_PRINT(A,("  Return -- "));DBG_PRINT(A,S);DBG_PRINT(A,("\n"));\
                                 _DBG_LEAVE(A);DBG_PRINT(A,("\n"));}}

# define DBG_PARAM(A,S)     {if (_DBG_SYS(A,DBG_TRACE_ON|DBG_PARAM_ON)) \
                                {DBG_PRINT(A,S);}}

# define DBG_PARAMF(A,N,S)  {if (_DBG_SYS(A,DBG_PARAM_ON|DBG_TRACE_ON)) \
                                {DBG_PRINT(A,("  %s -- ",N));DBG_PRINT(A,S);DBG_PRINT(A,("\n"));}}

# define DBG_TRACE(A)       {if (_DBG_SYS(A,DBG_TRACE_ON)) \
                                {DBG_PREFIX(A,DBG_SYSNAME());DBG_PRINT(A,("LINE=%d\n",__LINE__));}}

# define DBG_INFO(A,S)      {if (_DBG_SYS(A,(DBG_TRACE_ON|DBG_INFO_ON))) \
                                {DBG_PREFIX(A,DBG_SYSNAME());DBG_PRINT(A,S);}}

# define DBG_ASSERT(C)  if (!(C)) { \
                            DBG_PRINT(gDbgInfo,("ASSERT(%s) -- %s#%d (%s)\n", \
                                      #C, __FILE__, __LINE__, __FUNC__)); \
                            DBG_TRAP;}
# if !defined(ASSERT)
#  define ASSERT(C)     if (!(C)) { \
                            DBG_PRINT(gDbgInfo,("ASSERT(%s) -- %s#%d\n", \
                                      #C, __FILE__, __LINE__)); \
                            DBG_TRAP;}
# else
#  undef ASSERT
#  define ASSERT        DBG_ASSERT
# endif

# define DBG_STATIC

/****************************************************************************/
#else /* !DBG -- When (DBG == 0) we disable all the debug macros. */
/****************************************************************************/

# define DBG_TRACE_ON       0
# define DBG_PARAM_ON       0
# define DBG_ISR_SYS        0
# define DBG_IOCTL_SYS      0

typedef struct DBG_STRUCT *PDBG_STRUCT;
typedef void (* DBG_FUNC_TYPE)(PDBG_STRUCT);

//# define DBG_FIELDS
# define DBG_FIELDS \
    unsigned long           DbgFlags;\
    unsigned long           DbgId;\
    char *                  DbgName;\
    unsigned long           DbgSystem;\
    unsigned long           DbgLevel;\
    void *                  DbgParam;\
    DBG_FUNC_TYPE           DbgPrintFunc;\
    DBG_FUNC_TYPE           DbgAltFunc;\
    unsigned long           DbgAltFlags;

# define DBG_FLAGS(A)           0
# define DBG_ID(A)              0
# define DBG_NAME(A)            ""
# define DBG_NAME_BUFFER(A)
# define DBG_FIELD_NAME_BUFFER(L)
# define DBG_SYSTEM(A)          0
# define DBG_LEVEL(A)           0
# define DBG_LPARAM(A)          0
# define DBG_PRINT_FUNC(A)
# define DBG_ALT_FUNC(A)
# define DBG_ALT_FLAGS(A)       0
# define DBG_PUT_MASK(M)
# define DBG_PUT_INFO(A)

# define DBG_DEF_SYSTEM(N,M)
# define DBG_SYSNAME()          ""
# define DBG_INIT_SYSTEMS()
# define DBG_INIT(A,N,I,M,S)
# define DBG_INITX(A,M,F)
# define DBG_HALT(A)
# define DBG_FUNC(F)
# define DBG_FUNCX(F,M)
# define DBG_PRINT(A,S)
# define DBG_UNUSED

# define DBG_DISPLAY(A,S)
# define DBG_MESSAGE(A,S)
# define DBG_BREAK(A)
# define DBG_ERROR(A,S)
# define DBG_WARNING(A,S)
# define DBG_NOTICE(A,S)
# define DBG_FILTER(A,M,S)
# define DBG_DATA(A,F,B,N,O)
# define DBG_ENTER(A)
# define DBG_LEAVE(A)
# define DBG_RETURN(A,V)
# define DBG_RETURNF(A,S)
# define DBG_PARAM(A,S)
# define DBG_PARAMF(A,N,S)
# define DBG_TRACE(A)
# define DBG_INFO(A,S)
# define DBG_ASSERT(C)
#if !defined(ASSERT)
# define ASSERT(C)
#endif
# define DBG_STATIC         static

#endif /* DBG */
/****************************************************************************/

/****************************************************************************
@doc INTERNAL TpiDebug TpiDebug_h

@func void | DBG_ASSERT |

    Use this macro to conditionally report a fatal error if the condition
    specified is NOT true.

@parm BOOLEAN_EXPRESSION | Expression | Any valid if (Expression).

@ex <tab> | DBG_ASSERT(Actual == Expected);

@end
 ****************************************************************************/

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _TPIDEBUG_H */

