/*+I*************************************************************************
 * GlobeSpan NDIS Abstraction Layer Definitions
 *
 *-I*************************************************************************/
#ifndef _GPSNULL_H_
#define _GPSNULL_H_

#include "common.h"
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/version.h>
#include <linux/usb.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/netdevice.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/wait.h>
//#include "usbphy.h"

#if	defined(linux)
#ifndef	SPINLOCK_DEFINED
#define	SPINLOCK_DEFINED	1
typedef	spinlock_t		KSPIN_LOCK;
typedef	struct urb		URB;
typedef	struct urb		*PURB;
typedef	unsigned long	KIRQL;
#endif	/* SPINLOCK_DEFINED */

#endif	/* linux */

//
// This is the type of an NDIS OID value.
//

typedef ULONG NDIS_OID, *PNDIS_OID;

typedef PVOID       NDIS_HANDLE, *PNDIS_HANDLE;
typedef ULONG       NDIS_STATUS, *PNDIS_STATUS;
typedef ULONG       NDIS_PHYSICAL_ADDRESS, *PNDIS_PHYSICAL_ADDRESS;
#define OPTIONAL

#define NDIS_STATUS_SUCCESS                     0x00000000
#define NDIS_STATUS_PENDING                     0x00000001
#define NDIS_STATUS_FAILURE                     0xFFFFFFFF
#define NDIS_STATUS_MEDIA_CONNECT               ((NDIS_STATUS)0x4001000BL)
#define NDIS_STATUS_MEDIA_DISCONNECT            ((NDIS_STATUS)0x4001000CL)
#define NDIS_STATUS_UNSUPPORTED_MEDIA           ((NDIS_STATUS)0xC0010019L)
#define NDIS_STATUS_RESOURCES                   ((NDIS_STATUS)0xC000009AL)
#define NDIS_STATUS_NOT_SUPPORTED               ((NDIS_STATUS)0xC00000BBL)
#define NDIS_STATUS_BUFFER_TOO_SHORT            ((NDIS_STATUS)0xC0010016L)
#define NDIS_STATUS_INVALID_LENGTH              ((NDIS_STATUS)0xC0010014L)
#define NDIS_STATUS_TAPI_INVALCALLPARAMS          ((NDIS_STATUS)0xC001200EL)
#define NDIS_STATUS_INVALID_DATA                ((NDIS_STATUS)0xC0010015L)

#define NDIS_PACKET_TYPE_DIRECTED               0x00000001
#define NDIS_PACKET_TYPE_MULTICAST              0x00000002
#define NDIS_PACKET_TYPE_ALL_MULTICAST          0x00000004
#define NDIS_PACKET_TYPE_BROADCAST              0x00000008
#define NDIS_PACKET_TYPE_PROMISCUOUS            0x00000020

//
// Determine if an argument is present by testing the value of the pointer
// to the argument value.
//

#define ARGUMENT_PRESENT(ArgumentPointer)    (\
    (PVOID)(ArgumentPointer) != (PVOID)(NULL) )

// begin_winnt begin_ntminiport
//
// Calculate the byte offset of a field in a structure of type type.
//

#define FIELD_OFFSET(type, field)    ((ULONG)(PULONG)&(((type *)0)->field))


//
// Calculate the address of the base of the structure given its type, and an
// address of a field within the structure.
//

#define CONTAINING_RECORD(address, type, field) ((type *)( \
                                                  (PCHAR)(address) - \
                                                  (PCHAR)(&((type *)0)->field)))

//
// Check whether an address is multicast.
//
#define ETH_IS_MULTICAST(Address) \
    (BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x01))

//
// Check whether an address is broadcast.
//
#define ETH_IS_BROADCAST(Address) \
    ((((PUCHAR)(Address))[0] == ((UCHAR)0xff)) && \
     (((PUCHAR)(Address))[1] == ((UCHAR)0xff)))

#ifndef ETH_LENGTH_OF_ADDRESS
#   define ETH_LENGTH_OF_ADDRESS    6
#endif

//
// NDIS Spin Locks
//
typedef struct _NDIS_SPIN_LOCK
{
#if defined(linux)
    spinlock_t      slock;
    unsigned long   flags;
#elif defined(MAC_OS)
    ULONG           slock;
#endif /* OS */
} NDIS_SPIN_LOCK, *PNDIS_SPIN_LOCK;

VOID NdisAllocateSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    );

VOID NdisFreeSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    );

VOID NdisAcquireSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    );

VOID NdisReleaseSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    );

void  KeInitializeSpinLock( KSPIN_LOCK  *pLock  );

void	KeAcquireSpinLock( KSPIN_LOCK *pLock, KIRQL *pIrql);

void  KeReleaseSpinLock( KSPIN_LOCK  *pLock, KIRQL  Irql );


//
// NDIS Buffers
//
#define NDIS_BUFFER_TO_SPAN_PAGES(_Buffer)      1

typedef struct _NDIS_BUFFER *PNDIS_BUFFER;

typedef struct _NDIS_BUFFER_POOL
{
#if DBG
#   define                      NDIS_BUFFER_POOL_TAG 0xBEEF0001
    ULONG                       Tag;
#endif
    ULONG                       PoolSize;
    ULONG                       DescriptorSize;
    ULONG                       NumberOfDescriptors;
    NDIS_SPIN_LOCK              SpinLock;
    PNDIS_BUFFER                BufferList;
} NDIS_BUFFER_POOL, *PNDIS_BUFFER_POOL;

typedef struct _NDIS_BUFFER {
#if DBG
#   define                      NDIS_BUFFER_TAG 0xBEEF0002
    ULONG                       Tag;
#endif
    struct _NDIS_BUFFER *       Next;
    USHORT                      Size;
    PVOID                       Address;
    ULONG                       ByteCount;
    ULONG                       ByteOffset;
    PNDIS_BUFFER_POOL           Pool;
} NDIS_BUFFER;

VOID NdisQueryBuffer(
    IN  PNDIS_BUFFER            Buffer,
    OUT PVOID *                 VirtualAddress OPTIONAL,
    OUT PUINT                   Length
    );

VOID NdisGetNextBuffer(
    IN  PNDIS_BUFFER            CurrentBuffer,
    OUT PNDIS_BUFFER *          NextBuffer
    );

VOID NdisQueryBufferOffset(
    IN  PNDIS_BUFFER            Buffer,
    OUT PUINT                   Offset,
    OUT PUINT                   Length
    );

VOID NdisAdjustBufferLength(
    IN  PNDIS_BUFFER            Buffer,
    IN  UINT                    Length
    );

VOID NdisAllocateBufferPool(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            PoolHandle,
    IN  UINT                    NumberOfDescriptors
    );

VOID NdisFreeBufferPool(
    IN  NDIS_HANDLE             PoolHandle
    );

VOID NdisAllocateBuffer(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_BUFFER *          Buffer,
    IN  NDIS_HANDLE             PoolHandle,
    IN  PVOID                   VirtualAddress,
    IN  UINT                    Length
    );

VOID NdisFreeBuffer(
    IN PNDIS_BUFFER             Buffer
    );

//
// NDIS Packets
//
typedef struct _NDIS_PACKET *PNDIS_PACKET;

typedef struct _NDIS_PACKET_POOL
{
#if DBG
#   define                      NDIS_PACKET_POOL_TAG 0xBEEF0003
    ULONG                       Tag;
#endif
    ULONG                       PoolSize;
    ULONG                       DescriptorSize;
    ULONG                       NumberOfDescriptors;
    NDIS_SPIN_LOCK              SpinLock;
    PNDIS_PACKET                PacketList;
} NDIS_PACKET_POOL, *PNDIS_PACKET_POOL;

typedef struct _NDIS_PACKET_PRIVATE
{
#if DBG
#   define                      NDIS_PACKET_TAG 0xBEEF0004
    ULONG                       Tag;
#endif
    PNDIS_PACKET_POOL           Pool;   // So we know where to free it back to

    // if Head is NULL the chain is empty; Tail doesn't have to be NULL also
    PNDIS_BUFFER                Head;   // first buffer in the chain
    PNDIS_BUFFER                Tail;   // last buffer in the chain

    ULONG                       TotalLength;// Total amount of data in the packet.
    ULONG                       Count;  // Number of buffers chained to the packet
    ULONG                       Flags;
    ULONG                       PhysicalCount;
    BOOLEAN                     ValidCounts;
    NDIS_STATUS                 Status;
} NDIS_PACKET_PRIVATE, *PNDIS_PACKET_PRIVATE;

typedef ULONG *ULONG_PTR;
#define PROTOCOL_RESERVED_SIZE_IN_PACKET    (4 * sizeof(PVOID))

typedef struct _NDIS_PACKET 
{
    NDIS_PACKET_PRIVATE  Private;
    union {
        struct {
             UCHAR       MiniportReserved[2*sizeof(PVOID)];
             UCHAR       WrapperReserved[2*sizeof(PVOID)];
        };
        struct {
             UCHAR       MiniportReservedEx[3*sizeof(PVOID)];
             UCHAR       WrapperReservedEx[sizeof(PVOID)];
        };
        struct {
             UCHAR       MacReserved[4*sizeof(PVOID)];
        };
    };
    ULONG_PTR            Reserved[2];
    UCHAR                ProtocolReserved[1];
} NDIS_PACKET, **PPNDIS_PACKET;

#define NDIS_SET_PACKET_MEDIA_SPECIFIC_INFO(_Packet,                    \
                                            _MediaSpecificInfo,         \
                                            _SizeMediaSpecificInfo)     \
{                                                                       \
}   // UNSUPPORTED

#define NDIS_GET_PACKET_MEDIA_SPECIFIC_INFO(_Packet,                    \
                                            _pMediaSpecificInfo,        \
                                            _pSizeMediaSpecificInfo)    \
{                                                                       \
        *(_pMediaSpecificInfo) = NULL;                                  \
        *(_pSizeMediaSpecificInfo) = 0;                                 \
}   // UNSUPPORTED

VOID NdisAllocatePacketPool(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            PoolHandle,
    IN  UINT                    NumberOfDescriptors,
    IN  UINT                    ProtocolReservedLength
    );

VOID NdisFreePacketPool(
    IN  NDIS_HANDLE             PoolHandle
    );

VOID NdisFreePacket(
    IN  PNDIS_PACKET            Packet
    );

VOID NdisAllocatePacket(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_PACKET *          Packet,
    IN  NDIS_HANDLE             PoolHandle
    );

VOID NdisRecalculatePacketCounts(
    IN OUT PNDIS_PACKET         Packet
    );

VOID NdisReinitializePacket(
    IN OUT PNDIS_PACKET         Packet
    );

VOID NdisQueryPacket(
    IN  PNDIS_PACKET            Packet,
    OUT PUINT                   PhysicalBufferCount OPTIONAL,
    OUT PUINT                   BufferCount OPTIONAL,
    OUT PNDIS_BUFFER *          FirstBuffer OPTIONAL,
    OUT PUINT                   TotalPacketLength OPTIONAL
    );

VOID NdisChainBufferAtBack(
    IN OUT PNDIS_PACKET         Packet,
    IN OUT PNDIS_BUFFER         Buffer
    );

VOID NdisUnchainBufferAtBack(
    IN OUT PNDIS_PACKET         Packet,
    OUT PNDIS_BUFFER *          Buffer
    );

VOID NdisChainBufferAtFront(
    IN OUT PNDIS_PACKET         Packet,
    IN OUT PNDIS_BUFFER         Buffer
    );

VOID NdisUnchainBufferAtFront(
    IN OUT PNDIS_PACKET         Packet,
    OUT PNDIS_BUFFER *          Buffer
    );

//
// NDIS Timers
//
typedef VOID (*PNDIS_TIMER_FUNCTION)(
    IN  PVOID                   SystemSpecific1,
    IN  PVOID                   FunctionContext,
    IN  PVOID                   SystemSpecific2,
    IN  PVOID                   SystemSpecific3
    );

typedef struct _NDIS_MINIPORT_TIMER
{
#if DBG
#   define                      NDIS_TIMER_TAG 0xBEEF0005
    ULONG                       Tag;
#endif
    PNDIS_TIMER_FUNCTION        MiniportTimerFunction;
    PVOID                       MiniportTimerContext;
    PVOID                       MiniportAdapterHandle;
#if defined(linux)
    struct timer_list           TimerInfo;
    UCHAR                       LinuxTimerInfo[5*sizeof(PVOID)];
#endif /* OS */
} NDIS_MINIPORT_TIMER, *PNDIS_MINIPORT_TIMER;

VOID NdisMSetTimer(
    IN  PNDIS_MINIPORT_TIMER    Timer,
    IN  UINT                    MillisecondsToDelay
    );

VOID NdisMInitializeTimer(
    IN  OUT PNDIS_MINIPORT_TIMER Timer,
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PNDIS_TIMER_FUNCTION    TimerFunction,
    IN  PVOID                   FunctionContext
    );

VOID NdisMCancelTimer(
    IN  PNDIS_MINIPORT_TIMER    Timer,
    OUT PBOOLEAN                TimerCancelled
    );

//
// NDIS Work Items
//
typedef struct _NDIS_WORK_ITEM *PNDIS_WORK_ITEM;
typedef VOID    (*NDIS_PROC)(PNDIS_WORK_ITEM, PVOID);

typedef struct _NDIS_WORK_ITEM
{
    PVOID           Context;
    NDIS_PROC       Routine;
} NDIS_WORK_ITEM;

VOID NdisInitializeWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem,
    IN  NDIS_PROC               Routine,
    IN  PVOID                   Context
    );

NDIS_STATUS NdisScheduleWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem
    );

//
// NDIS Memory
//
#define NDIS_MEMORY_TAG 0xBEEF0006

NDIS_STATUS NdisAllocateMemoryWithTag(
    OUT PVOID *                 VirtualAddress,
    IN  UINT                    Length,
    IN  ULONG                   Tag
    );

VOID NdisFreeMemory(
    IN  PVOID                   VirtualAddress,
    IN  UINT                    Length,
    IN  UINT                    MemoryFlags
    );

//
// NDIS Events
//
typedef wait_queue_head_t NDIS_EVENT, *PNDIS_EVENT;

VOID NdisInitializeEvent(
    IN  PNDIS_EVENT             Event
    );
    
VOID NdisSetEvent(
    IN PNDIS_EVENT  Event
    );
              
VOID NdisResetEvent(
    IN  PNDIS_EVENT             Event
    );

BOOLEAN NdisWaitEvent(
    IN  PNDIS_EVENT             Event,
    IN  UINT                    msToWait
    );

//
// NDIS Strings
//
typedef struct NDIS_STRING
{
    USHORT                      MaximumLength;
    USHORT                      Length;
    PUCHAR                      Buffer;
} NDIS_STRING, *PNDIS_STRING;

#define NdisFreeString(NdisString) if (NdisString.Buffer) \
                                    {NdisFreeMemory(NdisString.Buffer, \
                                                    NdisString.MaximumLength);\
                                    NdisString.Buffer = NULL;\
                                    NdisString.MaximumLength = 0;\
                                    NdisString.Length = 0;}

//
// NDIS Files
//
VOID NdisOpenFile(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            FileHandle,
    OUT PUINT                   FileLength,
    IN  PNDIS_STRING            FileName,
    IN  NDIS_PHYSICAL_ADDRESS   HighestAcceptableAddress
    );

VOID NdisCloseFile(
    IN  NDIS_HANDLE             FileHandle
    );

VOID NdisMapFile(
    OUT PNDIS_STATUS            Status,
    OUT PVOID *                 MappedBuffer,
    IN  NDIS_HANDLE             FileHandle
    );

VOID NdisUnmapFile(
    IN  NDIS_HANDLE             FileHandle
    );

//
// Ndis Indications
//
void NdisMSendComplete(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PNDIS_PACKET            Packet,
    IN  NDIS_STATUS             Status,
	IN	void *					plan
    );

void NdisMEthIndicateReceive(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  NDIS_HANDLE             MiniportReceiveContext,
    IN  PVOID                   HeaderBuffer,
    IN  UINT                    HeaderBufferSize,
    IN  PVOID                   LookaheadBuffer,
    IN  UINT                    LookaheadBufferSize,
    IN  UINT                    PacketSize,
	IN	void *					plan
    );

#define NdisMEthIndicateReceiveComplete(MiniportAdapterHandle)

VOID NdisMIndicateStatus(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  NDIS_STATUS             GeneralStatus,
    IN  PVOID                   StatusBuffer,
    IN  UINT                    StatusBufferSize
    );

#define NdisMIndicateStatusComplete(MiniportAdapterHandle)

#define NdisMWanSendComplete    NdisMSendComplete

VOID NdisMWanIndicateReceive(
    OUT PNDIS_STATUS            Status,
    IN NDIS_HANDLE              MiniportAdapterHandle,
    IN NDIS_HANDLE              NdisLinkContext,
    IN PUCHAR                   Packet,
    IN ULONG                    PacketSize
    );               

VOID NdisStallExecution(IN ULONG Microsecs);

#define NdisMWanIndicateReceiveComplete(MiniportAdapterHandle, NdisLinkContext)

//
// Misc.
//
#define InterlockedIncrement    atomic_inc

#define InterlockedDecrement    atomic_dec

#endif // _GPSNULL_H_

