/*+I*************************************************************************
 * TriplePoint C Library Definitions
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *
 * This source code may be used by anyone provided the TriplePoint copyright
 * is maintained in any derivitive works.
 *-I*************************************************************************/
/****************************************************************************
@doc INTERNAL GpUtil GpUtil_h

@module GpDebug.c |

    This module implements some of the basic C library functions.  This makes
    porting to some platforms easier, where the C run-time libraries are not
    available.

@head3 Contents |
@index class,mfunc,func,msg,mdata,struct,enum | GpUtil_h

@end
 ****************************************************************************/
#ifndef _GpUTIL_H
#define _GpUTIL_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined(macintosh)
# include <size_t.h>
#elif defined(linux)
# if !defined(NO_Gp_OVERRIDES)
#  define NO_Gp_OVERRIDES
# endif /* NO_Gp_OVERRIDES */
# if !defined(_SIZE_T)
    typedef unsigned int size_t;
#   define _SIZE_T
# endif /* _SIZE_T */
#else
#  define NO_Gp_OVERRIDES
# if !defined(_SIZE_T_DEFINED)
    typedef unsigned int size_t;
#   define _SIZE_T_DEFINED
# endif  /* _SIZE_T_DEFINED */
#endif

/*
// Include Gp routines in all debug builds, but not the release build
// if they are explicitly excluded by defining NO_Gp_UTILS
*/
#if (DBG != 0) || defined(DEBUG) || defined(_DEBUG) || !defined(NO_Gp_UTILS)

#if !defined(NO_Gp_OVERRIDES)
# if !defined(isalnum)
#  define isalnum        Gp_isalnum
#  define isalpha        Gp_isalpha
#  define iscntrl        Gp_iscntrl
#  define isdigit        Gp_isdigit
#  define isgraph        Gp_isgraph
#  define islower        Gp_islower
#  define isprint        Gp_isprint
#  define ispunct        Gp_ispunct
#  define isspace        Gp_isspace
#  define isupper        Gp_isupper
#  define isxdigit       Gp_isxdigit
#  define isascii        Gp_isascii
#  define toascii        Gp_toascii
# endif

# define Gp_memcpy      memcpy
# define Gp_memset      memset
# define Gp_memcmp      memcmp
# define Gp_memchr      memchr
# define Gp_strlen      strlen
# define Gp_strcpy      strcpy
# define Gp_strncpy     strncpy
# define Gp_strcat      strcat
# define Gp_strncat     strncat
# define Gp_strcmp      strcmp
# define Gp_strncmp     strncmp
# define Gp_stricmp     stricmp
# define Gp_strnicmp    strnicmp
# define Gp_strchr      strchr
# define Gp_strrchr     strrchr
# define Gp_strstr      strstr
# ifdef sprintf
#  undef sprintf
# endif
# define Gp_sprintf     sprintf
# define Gp_vsprintf    vsprintf
#endif /* NO_Gp_OVERRIDES */

#ifndef  SIZE_OF_ARRAY
# define SIZE_OF_ARRAY(arr) (sizeof(arr) / sizeof(arr[0]))
#endif

/* Variable argument list definitions */
#ifndef va_start
#ifndef  _INTSIZEOF
# define _INTSIZEOF(n)   ((sizeof(n) + sizeof(int) - 1) & ~(sizeof(int) - 1))
#endif

typedef char * va_list;
# define va_start(ap,v)  (ap = (va_list)&v + _INTSIZEOF(v))
# define va_arg(ap,t)    (*(t *)((ap += _INTSIZEOF(t)) - _INTSIZEOF(t)))
# define va_end(ap)      (ap = (va_list)0)
#endif

/****************************************************************************
 * Memory handling functions
 ****************************************************************************/
void *Gp_memcpy(void *dst, const void *src, size_t n);
void *Gp_memmove(void *dst, const void *src, size_t n);
void *Gp_memchr(const void *ptr, int val, size_t n);
int Gp_memcmp(const void *ptr1, const void *ptr2, size_t n);
void *Gp_memset(void *dst, int val, size_t n);

/****************************************************************************
 * Misc functions
 ****************************************************************************/
unsigned long Gp_min(unsigned long x, unsigned long y);
unsigned long Gp_max(unsigned long x, unsigned long y);
unsigned short Gp_swaps(unsigned short x);
unsigned long Gp_swapl(unsigned long x);

/****************************************************************************
 * Character handling functions
 ****************************************************************************/
#define _U   0x01  /* upper */
#define _L   0x02  /* lower */
#define _D   0x04  /* digit */
#define _C   0x08  /* cntrl */
#define _P   0x10  /* punct */
#define _S   0x20  /* white space (space/lf/tab) */
#define _X   0x40  /* hex digit */
#define _SP  0x80  /* hard space (0x20) */

extern unsigned char const _Gp_ctype[];

#define __ismask(x)     (_Gp_ctype[(int)(unsigned char)(x)])
#define Gp_isalnum(c)  ((__ismask(c)&(_U|_L|_D)) != 0)
#define Gp_isalpha(c)  ((__ismask(c)&(_U|_L)) != 0)
#define Gp_iscntrl(c)  ((__ismask(c)&(_C)) != 0)
#define Gp_isdigit(c)  ((__ismask(c)&(_D)) != 0)
#define Gp_isgraph(c)  ((__ismask(c)&(_P|_U|_L|_D)) != 0)
#define Gp_islower(c)  ((__ismask(c)&(_L)) != 0)
#define Gp_isprint(c)  ((__ismask(c)&(_P|_U|_L|_D|_SP)) != 0)
#define Gp_ispunct(c)  ((__ismask(c)&(_P)) != 0)
#define Gp_isspace(c)  ((__ismask(c)&(_S)) != 0)
#define Gp_isupper(c)  ((__ismask(c)&(_U)) != 0)
#define Gp_isxdigit(c) ((__ismask(c)&(_D|_X)) != 0)
#define Gp_isascii(c)  (((unsigned char)(c))<=0x7f)
#define Gp_toascii(c)  (((unsigned char)(c))&0x7f)

char Gp_toupper(char cp);
char Gp_tolower(char cp);

/****************************************************************************
 * String handling functions
 ****************************************************************************/
unsigned char Gp_basedigit(char digit, unsigned char base, unsigned char *pValue);
unsigned long Gp_strtoul(const char *cp, char **endp, unsigned char base);
char *Gp_strcat(char *dst, const char *src);
char *Gp_strncat(char *dst, const char *src, size_t n);
char *Gp_strcpy(char *dst, const char *src);
char *Gp_strncpy(char *dst, const char *src, size_t n);
char *Gp_strlwr(char *s);
char *Gp_strupr(char *s);
size_t Gp_strlen(const char *s);
size_t Gp_strnlen(const char *s, size_t n);
int Gp_strcmp(const char *s1, const char *s2);
int Gp_strncmp(const char *s1, const char *s2, size_t n);
int Gp_strcasecmp(const char *s1, const char *s2);
int Gp_strncasecmp(const char *s1, const char *s2, size_t n);
size_t Gp_strspn(const char *s, const char *set);
char *Gp_strpbrk(const char *s, const char *set);
char *Gp_strchr(const char *s, int c);
char *Gp_strrchr(const char *s, int c);
char *Gp_strstr(const char *s1, const char *s2);
int Gp_vsprintf(char *buf, const char *fmt, va_list args);
int Gp_sprintf(char *buffer, const char *fmtString, ...);
int Gp_sprintf255(unsigned char *buf255, /* 255 bytes max! */const char *fmt, ...);

#endif /* (DBG != 0) || defined(DEBUG) || defined(_DEBUG) || !defined(NO_Gp_UTILS) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _GpUTIL_H */

