#ifdef MESSAGE_HANDLING
#ifndef MESSAGE_HANDLING_H									// only define these values once
#define MESSAGE_HANDLING_H

#include "ndis.h"
#include "debug.h"
#include "atmusb.h"
#include "atmuni.h"
#include "crc.h"
#include "gtiextn.h"
#include "gtidsp.h"
#include "gti_eoca.h"
#include "dsl.h"
#include "atmusb.h"

extern BOOLEAN IsTxThreadRunning ( PVOID UsbPhy);
extern void SetDspState ( PVOID UsbPhy, ULONG NewDspState, DSP_CONN_ACTION Action);
extern void SetDspAccess(GTI_XCVR_INFO *pSysPtr , GTI_BYTE bType);
extern GTI_BYTE GetDspAccess(GTI_XCVR_INFO *pSysPtr );

void MsgHandling_Init( PATM_UNI AtmUni);
void MsgHandling_AocSend( PATM_UNI AtmUni, PDSL_EX pDslEx);
void MsgHandling_AocPolling( PATM_UNI AtmUni);
void MsgHandling_Process(
	PVOID				SystemSpecific1,
	PATM_UNI			AtmUni,	   
	PVOID				SystemSpecific2,
	PVOID				SystemSpecific3
);
void MsgHandling_ProcessSiemens1_3( PATM_UNI AtmUni);
void MsgHandling_ProcessSiemens2_0( PATM_UNI AtmUni);
void MsgHandling_ProcessEci( PATM_UNI AtmUni);

USHORT MsgHandling_Eci_GetPayload( PUCHAR Buffer, BOOLEAN AddHeader);
void MsgHandling_Eci_CPmsg( 
	PATM_UNI_VC AtmUniVc, 
	PUCHAR NewBuffer, 
	ULONG NewState,
	PUCHAR OldBuffer,
	ULONG OldState
);
void MsgHandling_SetState( PATM_UNI	AtmUni, ULONG NewState, UINT TimerValue);
void MsgHandling_SetTimer( PATM_UNI AtmUni, UINT TimerValue);
void MsgHandling_Start( PATM_UNI AtmUni);
void MsgHandling_Stop( PATM_UNI AtmUni);
void MsgHandling_Send( PATM_UNI_VC AtmUniVc, PUCHAR Buffer, USHORT BufferSize, USHORT PduLength);
void MsgHandling_Receive( PATM_UNI_VC AtmUniVc, PUCHAR Buffer, USHORT BufferSize, USHORT PduLength);

#define DT_LEGACY_MODE              0x0001                  // DT Legacy Mode

#define DSLAM_TYPE_IS_ECI           0x12
#define DSLAM_TYPE_IS_S1_3          0x14
#define DSLAM_TYPE_IS_S2_0          0x13

#define MSG_HANDLING_SDU_SIZE       1528

/*********************************> Siemens (shared) <************************************/

#define S_MESSAGING_VPI				191
#define S_MESSAGING_VCI				32
#define S_MIN_CELL_LENGTH			40
#define S_MESSAGING_NEED_START		0


#define S_FW_VER1_MSG8F_IDX	        12                      // FW Major Version
#define S_FW_VER2_MSG8F_IDX	        13                      // FW Minor Version
#define S_FW_VER3_MSG8F_IDX	        14                      // FW Build
#define S_FW_ISSUE_MSG8F_IDX	    17                      // FW Issue

#define S_AOC_MSG_LENGTH			13						// max length of AOC message
#define S_AOC_MSG_COUNT				11						// number of AOC messages
#define S_AOC_MSG_SENDS				4                       // ensure 1 empty superframe
#define S_MESSAGING_AOC_WAIT        500						// 500 ms (first AOC wait)
#define S_MESSAGING_AOC_POLL  		750						// 750 microseconds (AOC polling)

typedef struct _COMGMT_AOC {
  UCHAR     DataLen;    
  UCHAR		AocData[S_AOC_MSG_LENGTH];
} COMGMT_AOC;


/************************************> Siemens 1.3 <**************************************/

#define S13_SEQ_OFFSET              6
#define S13_OPCODE_OFFSET			7
#define S13_MIN_PDU_LENGTH          10

#define S13_MESSAGING_DEF_TIMEOUT	25						// 25 ms (default timeout)
#define S13_MESSAGING_MSG_TIMEOUT	(1000 * 60)				// 60 secs (message timeout)
#define S13_MESSAGING_HB_TIMEOUT	(1000 * 56)				// 56 secs (heartbeat timeout)
#define S13_MESSAGING_8F_TIMEOUT	(100 * 14)				// 1.4 secs (before sending 8F)
#define S13_MESSAGING_8D_TIMEOUT	500     				// 500 ms (before sending 8D)

typedef enum {
	S13_MESSAGING_NEED_START	=	S_MESSAGING_NEED_START,
    S13_MESSAGING_NEED_SENDAOC,
    S13_MESSAGING_NEED_SEND8F,
    S13_MESSAGING_NEED_SEND8D,
    S13_MESSAGING_NEED_SEND93,
	S13_MESSAGING_NEED_SEND89_1,
    S13_MESSAGING_NEED_SEND82_1,
    S13_MESSAGING_NEED_ACK59,
    S13_MESSAGING_NEED_ACK8C,
    S13_MESSAGING_NEED_SEND89_2,
    S13_MESSAGING_NEED_SEND82_2,
    S13_MESSAGING_COMPLETE
} S13_MESSAGING_STATES;

typedef enum {
	S13_MESSAGING_NOACKS		=	0x00000000,
    S13_MESSAGING_ACK59         =	0x00000001,
	S13_MESSAGING_ACK91_1		=	0x00000002,
	S13_MESSAGING_ACK91_2		=	0x00000004,
	S13_MESSAGING_ACK8C_1		=	0x00000008,
	S13_MESSAGING_ACK8C_2		=	0x00000010
} S13_MESSAGING_ACKS;


// CP defined AOC messages (10 messages of max 13 bytes)
static COMGMT_AOC S13_Aoc[S_AOC_MSG_COUNT]=
{
	//{ {DataLen}, {AocData} }
	{{1}, 
	{0x00}},	// empty message

  	{{13}, 
	{0xC3, 0x00, 0x43, 0x4E, 0x2B, 0x4D, 0x3A, 0x4E, 0x54, 0x41, 0x44, 0x3A, 0x31}},

    {{13}, 
//	{0xC3, 0x01, 0x49, 0x3A, 0x52, 0x34, 0x35, 0x3A, 0x41, 0x45, 0x20, 0x20, 0x53}},
    {0xC3, 0x01, 0x49, 0x3A, 0x52, 0x34, 0x35, 0x3A, 0x41, 0x45, 0x20, 0x20, '*'}},

    {{13}, 
//	{0xC3, 0x02, 0x35, 0x30, 0x30, 0x31, 0x30, 0x2D, 0x4D, 0x31, 0x30, 0x38, 0x34}},
	{0xC3, 0x02, ' ', 'G', 'L', 'O', 'B', 'E', 'S', 'P', 'A', 'N', ' '}},

    {{13},
//	{0xC3, 0x03, 0x2D, 0x41, 0x35, 0x30, 0x31, 0x2D, 0x31, 0x31, 0x53, 0x35, 0x30}},
	{0xC3, 0x03, 'A', 'D', 'S', 'L', ' ', '*', ' ', ' ', '*', ' ', 'U'}},

    {{13},
//	{0xC3, 0x04, 0x30, 0x31, 0x30, 0x2D, 0x4D, 0x31, 0x30, 0x38, 0x34, 0x2D, 0x41}},
	{0xC3, 0x04, 'S', 'B', ' ', 'M', 'O', 'D', 'E', 'M', ' ', 'Q', '.'}},

    {{13},
//	{0xC3, 0x05, 0x35, 0x2D, 0x20, 0x20, 0x31, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00}},
	{0xC3, 0x05, '1', '1', ' ', '*', ' ', ' ', 0x00, 0x00, 0x00, 0x00, 0x00}},

    {{11},
	{0xCC, 0x00, 0x20, 0x31, 0x33, 0x36, 0x32, 0x32, 0x38, 0x02, 0x32}},
	
	{{11},
	{0xC9, 0x01, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}},
	
    {{11},
	{0xCC, 0x01, 0x00, 0x01, 0x02, 0xFF, 0x10, 0x11, 0x12, 0xFF, 0x00}},
	
	{{3},
	{0xC1, 0x00, 0x80}},
};

// CP defined transmit cells (2 cell PDUs and defined length)
static UCHAR S13_Pdu0E[] = 
{
    0xAA, 0xAA, 0x03, 0x00, 0x80, 0xC2, 0x00, 0x0E, 0x42, 0x42, 0x03, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x08, 0x00, 0x06, 0x36, 0x55, 0x0C, 
    0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x08, 0x00, 0x06, 0x36, 0x55, 0x0C, 
    0x80, 0x02, 0x00, 0x00, 0x14, 0x00, 0x02, 0x00, 0x0F, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};  // CRC: F7 01 8E 85

#define S13_Pdu0E_Size	68

static UCHAR S13_Pdu8F[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x8F, 0x03, 0x00, 0x03, 0x03, 
    0x00, 0x03, 0x03, 0x00,	0x00, 0x39, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x01, 0x00, 0x00, 0x1D, 0x01, 0x20, 0x01, 0x01,	0x00, 0x09, 0x0E, 0x0E, 
    0x02, 0x02, 0x03, 0x01, 0x02, 0x00, 0x09, 0x0E, 0x00, 0x00, 0x00, 0x03,
	0x01, 0x03, 0x00, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00
};  // CRC: 27 73 79 52

#define S13_Pdu8F_Size	64

// CP defined transmit cells (40 bytes + PDU length)
static UCHAR S13_Pdu8D[] = 
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0A, 0x8D, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x0B
};  // CRC: 06 D9 95 51

static UCHAR S13_Pdu93[] = 
{
    0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x93, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x08
};  // CRC: 9D 08 DC 8B

static UCHAR S13_Pdu89_1[] = 
{
    0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x89, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x08
};  // CRC:  F3 A9 84 3C

static UCHAR S13_Pdu82_1[] = 
{
    0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x82, 0x0F, 0xF0, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x08
};  // CRC: 05 6D D0 F3

static UCHAR S13_Pdu50[] = 
{	
    0x01, 0xE5, 0x02, 0x75, 0x02, 0x01, 0x0A, 0x50, 0xAA, 0x00, 0x03, 0x00,
	0x03, 0x03, 0x00, 0x03, 0x03, 0x00, 0x00, 0x39, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x14 
};  // CRC: variable (template)

static UCHAR S13_Pdu89_2[] = 
{
    0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x89, 0xAA, 0x00, 0x00, 0x00,
    0x03, 0x03, 0x00, 0x03, 0x03, 0x00, 0x00, 0x39, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x08
};  // CRC:  87 7C 1F C3

static UCHAR S13_Pdu82_2[] = 
{
    0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x82, 0xAA, 0x00, 0x00, 0x00,
    0x03, 0x03, 0x00, 0x03, 0x03, 0x00, 0x00, 0x39, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x08
};  // CRC: C1 FF 22 AA


/************************************> Siemens 2.0 <**************************************/

#define S20_OPCODE_OFFSET			7
#define S20_MIN_PDU_LENGTH          10
#define S20_MAX_PDU_LENGTH			2000

#define S20_MESSAGING_DEF_TIMEOUT	25						// 25 ms (default timeout)
#define S20_MESSAGING_MSG_TIMEOUT	(1000 * 60)				// 60 secs (message timeout)

typedef enum {
	S20_MESSAGING_NEED_START	=	S_MESSAGING_NEED_START,
    S20_MESSAGING_NEED_SENDAOC,
	S20_MESSAGING_NEED_SEND61_1,
	S20_MESSAGING_NEED_SEND8F,
	S20_MESSAGING_NEED_SEND98,
	S20_MESSAGING_NEED_SEND82,
	S20_MESSAGING_NEED_SEND89,
	S20_MESSAGING_NEED_ACK88,
	S20_MESSAGING_NEED_SEND61_2,
	S20_MESSAGING_NEED_ACK60,
	S20_MESSAGING_NEED_SEND98_2,
	S20_MESSAGING_NEED_ACK97,
	S20_MESSAGING_COMPLETE
} S20_MESSAGING_STATES;

typedef enum {
	S20_MESSAGING_NOACKS		=	0x00000000,
	S20_MESSAGING_ACK41			=	0x00000001,
	S20_MESSAGING_ACK81			=	0x00000002,
	S20_MESSAGING_ACK88			=	0x00000004,
	S20_MESSAGING_ACK45_1		=	0x00000008,
	S20_MESSAGING_ACK45_2		=	0x00000010,
	S20_MESSAGING_ACK60_1		=	0x00000020,
	S20_MESSAGING_ACK60_2		=	0x00000040,
	S20_MESSAGING_ACK97			=	0x00000080
} S20_MESSAGING_ACKS;

// CP defined AOC messages (10 messages of max 13 bytes)
static COMGMT_AOC S20_Aoc[S_AOC_MSG_COUNT]=
{
	//{ {DataLen}, {AocData} }
	{{1}, 
	{0x00}},	// empty message

	{{13}, 
	{0xc3, 0x00, 0x43, 0x4e, 0x2b, 0x4d, 0x3a, 0x4e, 0x54, 0x41, 0x44, 0x3a, 0x31}},

	{{13}, 
//	{0xc3, 0x01, 0x49, 0x3a, 0x52, 0x34, 0x3a, 0x41, 0x45, 0x20, 0x20, 0x20, 0x53}},
	{0xc3, 0x01, 0x49, 0x3a, 0x52, 0x34, 0x3a, 0x41, 0x45, 0x20, 0x20, 0x20, '*'}},
	
	{{13}, 
//	{0xc3, 0x02, 0x35, 0x30, 0x30, 0x31, 0x30, 0x2d, 0x4d, 0x31, 0x30, 0x36, 0x33}},
	{0xc3, 0x02, ' ', 'G', 'L', 'O', 'B', 'E', 'S', 'P', 'A', 'N', ' '}},

	{{13}, 
//	{0xc3, 0x03, 0x2d, 0x41, 0x35, 0x30, 0x31, 0x2d, 0x30, 0x36, 0x53, 0x35, 0x30}},
	{0xc3, 0x03, 'A', 'D', 'S', 'L', ' ', '*', ' ', ' ', '*', ' ', 'U'}},

	{{13}, 
//	{0xc3, 0x04, 0x30, 0x31, 0x30, 0x2d, 0x4d, 0x31, 0x30, 0x36, 0x33, 0x2d, 0x41}},
	{0xc3, 0x04, 'S', 'B', ' ', 'M', 'O', 'D', 'E', 'M', ' ', 'Q', '.'}},

	{{13}, 
//	{0xc3, 0x05, 0x20, 0x20, 0x35, 0x2d, 0x48, 0x32, 0x00, 0x00, 0x00, 0x00, 0x00}},
	{0xc3, 0x05, '1', '1', ' ', '*', ' ', ' ', 0x00, 0x00, 0x00, 0x00, 0x00}},

	{{11}, 
	{0xcc, 0x00, 0x20, 0x39, 0x30, 0x32, 0x37, 0x33, 0x38, 0x02, 0x32}},

	{{11}, 
	{0xc9, 0x01, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}},

	{{11}, 
	{0xcc, 0x01, 0x00, 0x01, 0x02, 0xff, 0x10, 0x11, 0x12, 0xff, 0x00}},

	{{3}, 
	{0xc1, 0x00, 0x80}}

};

// CP defined transmit cells (40 bytes + PDU length)
static UCHAR Cell_8F[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x8F, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x01, 0x07, 0x00, 0x00, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
	0x20, 0x21, 0x22, 0x23, 0x18
};

static UCHAR Cell_98[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x98, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x01, 0x07, 0x00, 0x00, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
	0x20, 0x21, 0x22, 0x23, 0x08
};

static UCHAR Cell_82[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x82, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x08
};

static UCHAR Cell_89[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x89, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x08
};

static UCHAR Cell_61[] = 
{
	0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x0A, 0x61, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x81, 0x00, 0x00, 0x0C, 0x10, 0x16, 0xCE, 0xDF, 0x41, 0x00, 0x00, 0x0A,
	0x69, 0x9E, 0x85, 0xFD, 0x0C
};


/************************************> ECI <**************************************/

#define E_MESSAGING_VPI			15
#define E_MESSAGING_VCI			32

#define E_SEQUENCE_OFFSET		8
#define E_OPCODE_OFFSET			9
#define E_MSG_TYPE_OFFSET		12
#define E_MSG_LENGTH_OFFSET		16

#define E_MESSAGING_DEF_TIMEOUT	25						// 25 ms (default timeout)
#define E_MESSAGING_MSG_TIMEOUT	(1000 * 12)				// 12 secs (message timeout)

#define E_CO_HANDSHAKE_INIT		0x00000000

typedef enum {
	E_MESSAGING_NEED_START	=	0,
	E_MESSAGING_NEED_ACK1,
	E_MESSAGING_NEED_ACK2,
	E_MESSAGING_NEED_ACK3,
	E_MESSAGING_NEED_ACK4,
	E_MESSAGING_NEED_ACK5,
	E_MESSAGING_NEED_ACK6,
	E_MESSAGING_NEED_ACK7,
	E_MESSAGING_COMPLETE
} E_MESSAGING_STATES;


static UCHAR cp_msg0[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x18, 0xEE, 0xEE, 0x00, 0x00, 0x00, 0x01, 0xEE, 0xEE, 0xEE, 0xEE, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

static UCHAR cp_msg1[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x0B, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x01 ,0x08, 0x00, 0x00, 0x00, 0x00, 0x04, 0x03, 0x00, 0x00, 0x00, 0x00, 0x03, 0x01, 0x00, 0x00,
	0x30, 0x32, 0x2E, 0x32, 0x34, 0x69, 0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0x2E, 0x32, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x31, 0x2E, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x54, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66,
	0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x54, 0x66, 0x66, 0x66, 0x66, 0x66,
	0x66, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x45, 0x43, 0x49, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x48,
	0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x0C, 0x5E, 0xEC, 0x00, 0x00, 0x00, 0x02,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x3C, 0x96, 0xA0, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x06,
	0x00, 0x3F, 0xBE, 0x4C, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x30, 0x2E, 0x30, 0x00, 0x00, 0x32,
	0x00, 0x0D, 0xD2, 0x08, 0x00, 0x53, 0x55, 0x20, 0x49, 0x53, 0x44, 0x4E, 0x20, 0x30, 0x34, 0x30,
	0x20, 0x44, 0x33, 0x00, 0x00, 0x3C, 0x81, 0x78, 0x00, 0x00, 0x90, 0x02, 0x00, 0x0C, 0x5E, 0xEC,
	0x00, 0x3C, 0x81, 0x50, 0x00, 0x3F, 0xBF, 0x78
};

static UCHAR cp_msg2[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x14, 0x81, 0x50, 0x00, 0x00, 0x04, 0x06, 0x00, 0x00, 0x00, 0x00, 0x03, 0x04, 0x00, 0x00,
	0x02, 0x03, 0x81, 0x88 
};

static UCHAR cp_msg3[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x0D, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x94, 0xD2, 0x08, 0x00, 0x00, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00, 0x04, 0x01, 0x00, 0x00,
	0x31, 0x39, 0x32, 0x2E, 0x31, 0x36, 0x38, 0x2E, 0x32, 0x30, 0x2E, 0x38, 0x30, 0x00, 0x5E, 0xEC,
	0x00, 0x32, 0x35, 0x35, 0x2E, 0x32, 0x35, 0x35, 0x2E, 0x32, 0x35, 0x35, 0x2E, 0x30, 0x00, 0x20,
	0x00, 0x00, 0x31, 0x39, 0x32, 0x2E, 0x31, 0x36, 0x38, 0x2E, 0x32, 0x30, 0x2E, 0x31, 0x30, 0x00,
	0x00, 0x3C, 0x81, 0x31, 0x30, 0x2E, 0x30, 0x2E, 0x30, 0x2E, 0x31, 0x35, 0x00, 0x04, 0x66, 0x28,
	0x03, 0xFF, 0xFF, 0xFF, 0x32, 0x35, 0x35, 0x2E, 0x32, 0x35, 0x35, 0x2E, 0x30, 0x2E, 0x30, 0x00,
	0x00, 0x00, 0x00, 0x2E, 0x30, 0x31, 0x30, 0x2E, 0x30, 0x2E, 0x30, 0x2E, 0x31, 0x00, 0x00, 0x00,
	0x00, 0x15, 0x8E, 0x20, 0x00, 0x3F, 0x30, 0x3A, 0x32, 0x30, 0x3A, 0x38, 0x66, 0x3A, 0x33, 0x3A,
	0x30, 0x3A, 0x35, 0x30, 0x00, 0x01, 0x7C, 0x40, 0x00, 0x3C, 0x81, 0x50, 0x00, 0x3F, 0xBF, 0x78,
	0x00, 0x00, 0x00, 0x02
};

static UCHAR cp_msg4[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x3C, 0x81, 0x50, 0x01, 0x0A, 0xBF, 0x03,
	0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x01
};

static UCHAR cp_msg5[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x0F, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x18, 0x81, 0x50, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x03, 0x02, 0x00, 0x03,
	0x00, 0x3C, 0x81, 0x88, 0x00, 0x00, 0x00, 0x01
};

static UCHAR cp_msg6[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x10, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x04, 0x07, 0x00, 0x3C, 0x81, 0x88, 0x33, 0x30, 0x34, 0x64,
	0x62, 0x65, 0x30, 0x33, 0x00, 0x00, 0x00, 0xDC
};

static UCHAR cp_msg7[] = 
{
	0xEC, 0xEC, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x11, 0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x0F,
	0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x04, 0x08, 0x62, 0x65, 0x30, 0x33, 0x00, 0x00, 0x00, 0xDC
};


#endif // MESSAGE_HANDLING_H
#endif // MESSAGE_HANDLING
