#ifndef _API1577_H_
#define _API1577_H_

#include "Gp.h"


#define LENGTH_OF_ATM_NSAP_ADDRESS     20
#define LENGTH_OF_ATM_E164_ADDRESS     20
#define IP_ADDRESS_SIZE                4
#define SUB_IP_ADDRESS_SIZE            4
#define CALLBACK_FUNC                  void * 
#define LLC_SNAP_ENCAPSULATION          0  
#define RFC1577_ERROR                   ((ULONG)-1)
#define RFC1577_ERROR_IP                ((UINT)-1)   


#ifndef ETHERTYPE_IP
    #ifdef GTI_LITTLE_ENDIAN
    #define ETHERTYPE_IP                0x0008
    #elif (defined(GTI_BIG_ENDIAN))
    #define ETHERTYPE_IP                0x0800
    #elif
    NO_ENDIAN_MODE_DEFINED
    #endif
#endif
            
#define    RFC1577_MAX_PACKET_POOL              100
#define PROTOCOL_RESERVED_LEN               64 
#define IP_HEADER_SEQ                       2


typedef struct _MAC_HEADER
 {
    UCHAR       DstAddress[ETH_LENGTH_OF_ADDRESS];
    UCHAR       SrcAddress[ETH_LENGTH_OF_ADDRESS];
    USHORT      EtherType;

} MAC_HEADER, *PMAC_HEADER;

typedef struct tETH_ARP_PACKET {
    USHORT	ArpHrd;
    USHORT	ArpPro;
    UCHAR 	ArpHwLen;
    UCHAR 	ArpProtLen;
    USHORT	ArpOpCode;
    UCHAR 	SendHwAddr[ ETH_LENGTH_OF_ADDRESS];
    UCHAR 	SendIpAddr[ IP_ADDRESS_SIZE];
    UCHAR 	DestHwAddr[ ETH_LENGTH_OF_ADDRESS];
    UCHAR 	DestIpAddr[ IP_ADDRESS_SIZE];
} ETH_ARP_PACKET, *PETH_ARP_PACKET;



typedef struct _IP_HEADER
{
    UCHAR   Version :4; //4 bits
    UCHAR   HeaderLength:4; //4 bits
    UCHAR   Tos;           //8 bits Type of Service
    USHORT  TotalLength;   //16 bits
    USHORT  Identification;
    USHORT  FragmentOffset;
    UCHAR   TTL;
    UCHAR   Protocol;
    USHORT   HeaderChecksum;
    UINT    SourceIP;
    UINT    DestinationIP;

}IP_HEADER , *PIP_HEADER;

typedef struct _PACKET_HEADER
{
    UCHAR       DstAddress[ETH_LENGTH_OF_ADDRESS];
    UCHAR       SrcAddress[ETH_LENGTH_OF_ADDRESS];
    USHORT      EtherType;

    UCHAR   Version :4; //4 bits
    UCHAR   HeaderLength:4; //4 bits
    UCHAR   Tos;           //8 bits Type of Service
    USHORT  TotalLength;   //16 bits
    USHORT  Identification;
    USHORT  FragmentOffset;
    UCHAR   TTL;
    UCHAR   Protocol;
    USHORT   HeaderChecksum;
    UINT    SourceIP;
    UINT    DestinationIP;

}PACKET_HEADER, *PPACKET_HEADER;

RFC1577_HANDLE Api1577AllocateSpinLock( PATM_USB_INSTANCE Instance );
BOOLEAN Api1577FreeSpinLock(PATM_USB_INSTANCE Instance , RFC1577_HANDLE handle);//shut down

BOOLEAN Api1577AcquireSpinLock(PATM_USB_INSTANCE Instance , RFC1577_HANDLE handle);
BOOLEAN Api1577ReleaseSpinLock(PATM_USB_INSTANCE Instance , RFC1577_HANDLE handle);


BOOLEAN Api1577Initialize(PATM_USB_INSTANCE Instance);

RFC1577_HANDLE GetFreeSpinLockHandle(PRFC1577_SPIN_LOCK pRfc1577SpinLock);
RFC1577_HANDLE GetFreeTimerHandle( PRFC1577_TIMER pRfc1577Timer );

RFC1577_HANDLE Api1577InitializeTimer(PATM_USB_INSTANCE Instance, CALLBACK_FUNC callbackfunc );
BOOLEAN Api1577SetTimer(PATM_USB_INSTANCE Instance, RFC1577_HANDLE handle, UINT MillisecondsToDelay );
BOOLEAN Api1577CancelTimer(PATM_USB_INSTANCE Instance, RFC1577_HANDLE handle, UINT MillisecondsToDelay );
UINT    Api1577GetDestinationIP(PNDIS_PACKET  		Packet);
UINT    Api1577GetDestIPFromEther (PNDIS_PACKET Packet);
USHORT  Api1577GetType(PNDIS_PACKET  		Packet);
PETH_ARP_PACKET Api1577GetEtherHeader(PNDIS_PACKET Packet);

PNDIS_PACKET Api1577MakeNdisPacket(PATM_USB_INSTANCE Instance , PXFER_DATA_BUFFER txBuffer);
BOOLEAN      Api1577FreeNdisPacket(PATM_USB_INSTANCE Instance , PNDIS_PACKET NdisPkt );

PXFER_DATA_BUFFER Api1577GetBuffer(PATM_USB_INSTANCE Instance);
BOOLEAN Api1577FreeBuffer(PATM_USB_INSTANCE Instance , PXFER_DATA_BUFFER buffer);

BOOLEAN Api1577GetXferBuffer(PATM_USB_INSTANCE Instance , PNDIS_PACKET NdisPkt , PXFER_DATA_BUFFER txBuffer);

ATM_UNI_HANDLE Api1577OpenVC(PATM_USB_INSTANCE Instance , UCHAR VPI, USHORT VCI , RFC1483_MODE Encap);
BOOLEAN Api1577CloseVC(PATM_USB_INSTANCE Instance , ATM_UNI_HANDLE AtmUniHandle );
void * Api1577GetContext(PATM_USB_INSTANCE Instance);

BOOLEAN Api1577GetDestMacAddress(PNDIS_PACKET  Packet , PUCHAR Buffer);


#endif

