////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.3 $
//     $Modtime: 8/15/02 1:20p $
//     $Author: kattre $
//
// ABSTRACT:
//	   Contains data structures, function declarations, and macros for the
//     ATM UNI (SoftSAR) module
//
//
////////////////////////////////////////////////////////////////////////////////
#ifndef ATMUNI_H_
#define ATMUNI_H_

#include "atmphy.h"
#include "dsl.h"

typedef void * ATM_UNI_HANDLE;
typedef void * ATM_UNI_USER_HANDLE;

#define ATM_UNI_VC_HASH_TABLE_SIZE   		101
#define ATM_AAL5_TRAILER_SIZE            	8
#define ATM_AAL5_PT_END_OF_PDU_MASK         0x2
#define ATM_CELL_SET_CLP_MASK				0x1
#define ATM_CELL_CLEAR_CLP_MASK				0xFE
#define ATM_CELL_HEADER_CLP_OFFSET			3
#define ATM_AAL5_TRAILER_CPCS_UU_OFFSET		0
#define ATM_AAL5_TRAILER_CPI_OFFSET			1
#define ATM_AAL5_TRAILER_PDU_LENGTH_OFFSET  2
#define ATM_AAL5_TRAILER_CRC_OFFSET			4

#define ATM_CELL_PAYLOAD_SIZE  				48
#define	ATM_CELL_FULL_HEADER_SIZE			5
#define ATM_CELL_RCV_HEADER_SIZE   			3	// 3 bytes to enable header byte-stripping, 5 for full header
#define ATM_CELL_TXFR_SIZE         			(ATM_CELL_PAYLOAD_SIZE + ATM_CELL_FULL_HEADER_SIZE)
#define ATM_CELL_RCV_SIZE          			(ATM_CELL_PAYLOAD_SIZE + ATM_CELL_RCV_HEADER_SIZE)

#if (ATM_CELL_RCV_HEADER_SIZE == ATM_CELL_FULL_HEADER_SIZE)
#define ATM_CELL_FULL_HEADER				TRUE
#else
#define ATM_CELL_FULL_HEADER				FALSE
#endif	// full header (with HEC)

#define ATM_UNI_PHY_RX_RESOURCE_COUNT		MAX_RX_RESOURCE_COUNT
#define ATM_UNI_PHY_RX_BUFFER_SIZE			64

#define ATM_UNI_SHORT_PACKETS_POOL_SIZE     1

#define LOOPBACK_MAX_BUFFERS				20

/*
**	This is the count for the MAX number of PVC we are supporting right now.
**    This can be scaled later as to the hardware limitations.
*/
#define 		MAX_PVC_SUPPORTED		4

//------------------------------------------------------------------------------
// ATM_UNI_STATUS
//     Possible states for the atm uni layer
//------------------------------------------------------------------------------
typedef enum {
    ATM_UNI_STATE_NONE            = 0x00000000,
    ATM_UNI_STATE_INITIALIZED     = 0x00000001,
    ATM_UNI_STATE_HALTING         = 0x00000002,
    ATM_UNI_STATE_SUSPENDING      = 0x00000004,
    ATM_UNI_STATE_DISCONNECTED    = 0x00000008,
    ATM_UNI_STATE_RESETTING       = 0x00000010,
    ATM_UNI_STATE_DONT_ACCEPT     = 0x0000000E,    // (halting, suspending, or line down : dont accept data)
    ATM_UNI_STATE_LAYER_DOWN      = 0x0000001E     // (halting, suspending, resetting, or line down : dont pass down data)
} ATM_UNI_STATE;


// ATM cell for receive
typedef struct tATM_CELL_RCV
{
  UCHAR hdr[ATM_CELL_RCV_HEADER_SIZE];	// ATM cell header
  UCHAR info[ATM_CELL_PAYLOAD_SIZE];	// ATM cell payload
} ATM_CELL_RCV, *PATM_CELL_RCV;


// ATM cell for transfer
typedef struct tATM_CELL_TXFR
{
  UCHAR hdr[ATM_CELL_FULL_HEADER_SIZE];	// ATM cell header
  UCHAR info[ATM_CELL_PAYLOAD_SIZE];	// ATM cell payload
} ATM_CELL_TXFR, *PATM_CELL_TXFR;


//------------------------------------------------------------------------------
//
// GET_VC_HASH_KEY
//     Given the VPI/VCI (packed into the first 24 bits of a 32 bit word with
//     the VCI in the least significant 16 bits [0-15] and the VPI in bits
//     16-23), returns the hash key to be used when looking the VC up in the
//     VC hash table.
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define GET_VC_HASH_KEY( vpiVci) ((vpiVci & 0x0FFFFFF0) % ATM_UNI_VC_HASH_TABLE_SIZE)
#else
#define GET_VC_HASH_KEY( vpiVci) ((vpiVci & 0x00F0FFF0) % ATM_UNI_VC_HASH_TABLE_SIZE)
#endif // ATM_CELL_FULL_HEADER


//------------------------------------------------------------------------------
//
// PVC_COMPARE
//		Compares PVCs to determine if they match
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define PVC_COMPARE( pvc1, pvc2)	( (pvc1 & 0x0FFFFFF0) == (pvc2 & 0x0FFFFFF0) )
#else
#define PVC_COMPARE( pvc1, pvc2)	( (pvc1 & 0x00F0FFF0) == (pvc2 & 0x00F0FFF0) )
#endif // ATM_CELL_FULL_HEADER


//------------------------------------------------------------------------------
//
// SET_ATM_VPIVCI
//		Given cell vpi and vci, set the VPIVCI
//
//------------------------------------------------------------------------------
#define SET_ATM_VPIVCI( Vpi, Vci) (((ULONG)Vpi << 20) | ((ULONG)Vci << 4))


//------------------------------------------------------------------------------
//
// GET_ATM_VPIVCI
//		Given receive cell header, returns the VPIVCI
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define GET_ATM_VPIVCI( CellHeader)	(((ULONG)CellHeader) & 0x0FFFFFF0)
#else
#define GET_ATM_VPIVCI( CellHeader)	( (((ULONG)CellHeader) & 0xF0FFF000) >> 8)
#endif // ATM_CELL_FULL_HEADER


//------------------------------------------------------------------------------
//
// GET_ATM_VPI
//		Given receive cell header, returns the VPI
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define GET_ATM_VPI( CellHeader) ( (((ULONG)CellHeader) & 0x0FF00000) >> 20)
#else
#define GET_ATM_VPI( CellHeader) ( (((ULONG)CellHeader) & 0xF0000000) >> 28)
#endif // ATM_CELL_FULL_HEADER


//------------------------------------------------------------------------------
//
// GET_ATM_VCI
//		Given receive cell header, returns the VCI
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define GET_ATM_VCI( CellHeader) ( (((ULONG)CellHeader) & 0x000FFFF0) >> 4)
#else
#define GET_ATM_VCI( CellHeader) ( (((ULONG)CellHeader) & 0x00FFF000) >> 12)
#endif // ATM_CELL_FULL_HEADER



//------------------------------------------------------------------------------
//
// GET_ATM_PTI
//		Given receive cell header, returns the PTI
//
//------------------------------------------------------------------------------
#if ATM_CELL_FULL_HEADER
#define GET_ATM_PTI( CellHeader) (((ULONG)CellHeader) & 0x0000000E)
#else
#define GET_ATM_PTI( CellHeader) ( (((ULONG)CellHeader) & 0x00000E00) >> 8)
#endif // ATM_CELL_FULL_HEADER


//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
typedef struct
{
/* DO NOT CHANGE THE SIZE OF THE STRUCTURE OR THE LOCATION OF THE FIELDS */
/* This structure can not be bigger than (3*PVOID). These information will
** be stored in the NDIS_PACKET and Microsoft does not provide more space.
*/
   PNDIS_PACKET    NextPkt;            /* Pointer to next NDIS_PACKET */

   PVOID           RfcReservedPtr;     /* Field used by the RFC layer to save pointer */
   
   UCHAR           Flags;              /* (ArpFlag << 7) | SkipFirstBuffer */
   UCHAR           CpcsUu;             /* Information about next NDIS_PACKET */
   UCHAR           Cpi;
   UCHAR           Clp;                /* (Index << 7) | Clp */
} NDIS_PACKET_DESCRIPTOR, * PNDIS_PACKET_DESCRIPTOR;

typedef struct
{
   PNDIS_PACKET    HeadPkt;
   PNDIS_PACKET    TailPkt;
   ULONG           Counter;
} NDIS_PACKET_QUEUE, * PNDIS_PACKET_QUEUE;

#define NDIS_PACKET_SET_CLP(clp, reservedClp) (reservedClp | (clp & 0x1))
#define NDIS_PACKET_GET_CLP(reservedClp) (reservedClp & 0x1)

#define NDIS_PACKET_SET_INDEX(index, reservedIndex) (reservedIndex | (index << 1))
#define NDIS_PACKET_GET_INDEX(reservedIndex) (reservedIndex >> 1)

#define NDIS_PACKET_RESERVED_FIELD(_pkt_)  ((PNDIS_PACKET_DESCRIPTOR)((_pkt_)->MiniportReservedEx))

#define INSERT_NDIS_PACKET(PktQ, Packet)                                   \
       if ( (((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt) == NULL )                \
           ((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt = Packet;                   \
       else                                                                \
           ((NDIS_PACKET_RESERVED_FIELD(((PNDIS_PACKET_QUEUE)PktQ)->TailPkt))->NextPkt) = Packet;  \
       (((PNDIS_PACKET_QUEUE)PktQ)->TailPkt) = Packet;                     \
       ((PNDIS_PACKET_QUEUE)(PktQ)->Counter)++;                            \
       ((NDIS_PACKET_RESERVED_FIELD(Packet))->NextPkt) = NULL;

#define REMOVE_NDIS_PACKET(PktQ)                                           \
       ((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt;                                                    \
       {                                                                   \
           if ((((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt) != NULL)                                  \
           {                                                                                   \
               ((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt = NDIS_PACKET_RESERVED_FIELD(((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt)->NextPkt;  \
               ((PNDIS_PACKET_QUEUE)(PktQ)->Counter)--;                                        \
           }                                                                                   \
       }
       
#define EMPTY_NDIS_PACKET(PktQ)    (((PNDIS_PACKET_QUEUE)PktQ)->HeadPkt)
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////

//------------------------------------------------------------------------------
//
// SET_PACKET_MINIPORT_RESERVED
//		Puts the given pointer into the NDIS packet's miniport reserved area
//
//------------------------------------------------------------------------------
#define SET_PACKET_MINIPORT_RESERVED( _Packet, _Data)		\
{															\
	PULONG	_Temp;											\
	_Temp = ((PULONG)(_Packet)->MiniportReserved);			\
	*(_Temp) = (ULONG)(_Data);								\
}

//------------------------------------------------------------------------------
//
// GET_PACKET_MINIPORT_RESERVED
//		Gets a void pointer from the NDIS packet's miniport reserved area
//
//------------------------------------------------------------------------------
#define GET_PACKET_MINIPORT_RESERVED( _Packet)				\
	(PVOID)(*((PULONG)(_Packet)->MiniportReserved))

//------------------------------------------------------------------------------
//
// SET_PACKET_VC_MINIPORT_RESERVED
//		Puts the given pointer into the NDIS packet's miniport reserved area
//		in the space reserved for user VC context
//
//------------------------------------------------------------------------------
#define SET_PACKET_VC_MINIPORT_RESERVED( _Packet, _Data)		   			   \
{																			   \
	PULONG	_Temp;															   \
	_Temp = ((PULONG)((PUCHAR)(_Packet)->MiniportReserved + sizeof(PVOID)));   \
	*(_Temp) = (ULONG)(_Data);								                   \
}

//------------------------------------------------------------------------------
//
// GET_PACKET_VC_MINIPORT_RESERVED
//		Gets a void pointer from the NDIS packet's miniport reserved area from
//		the space reserved for user VC context
//
//------------------------------------------------------------------------------
#define GET_PACKET_VC_MINIPORT_RESERVED( _Packet)				\
	(PVOID)(*(PULONG)((_Packet)->MiniportReserved+sizeof(PVOID)))


//------------------------------------------------------------------------------
//
// AAL5_TRAILER
//		Defines the ATM AAL5 trailer structure
//
//------------------------------------------------------------------------------
#include "pshpack1.h"
typedef struct tAAL5_TRAILER {
	UCHAR	CpcsUu;
	UCHAR	Cpi;
	USHORT	PduLength;
	ULONG	Crc;
} AAL5_TRAILER, *PAAL5_TRAILER;
#include "poppack.h"

//------------------------------------------------------------------------------
//
// ATM_UNI_STATS
//		Defines the statistics kept by the ATM UNI (kept both globally and
//      per VC)
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI_STATS {
	ULONG	TxPdusOk;
	ULONG	TxPdusError;
	ULONG	RxPdusOk;
	ULONG	RxPdusError;
	ULONG	RxPdusNoBuffer;
	ULONG	RxCrcError;
	ULONG	RxCellsOk;
	ULONG	TxCellsOk;
	ULONG	RxCellsDropped;
	ULONG	RxInvalidVpiVci;
	ULONG	OAMLoopBkSuccess;	// # of loopback cells received successfully
	ULONG	OAMLoopBkFailure;	// # of loopback cells that were not formatted correctly
} ATM_UNI_STATS, *PATM_UNI_STATS;

//------------------------------------------------------------------------------
//
// ATM_UNI_COUNTERS
//     Defines the statistics kept by the ATM UNI (kept both globally and
//      per VC)
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI_COUNTERS
{
   ULONG   TxCellsOk;          // Number of cells transmitted correctly or OK
   ULONG   TxCellsDrop;        // Number of cells dropped in transmit direction
   ULONG   TxOAMCellsOk;       // Number of F4/F5/OAM cells transmitted correctly
   ULONG   TxOAMCellsDrop;     // Number of F4/F5/OAM cells dropped in transmit direction
   ULONG   TxPacketOk;         // Number of PDUs/Packets/Frame transmitted correctly or OK
   ULONG   TxNoBuffer;         // Number of PDUs/Packets/Frame dropped because no buffer
   ULONG   TxPacketError;      // Number of PDUs/Packets/Frame dropped
   ULONG   TxTotalCells;       // Total Number of Cells transmitted (Data cells, OAM cells, etc.)

   ULONG   RxCellsOk;          // Number of Received Cells are correct or OK
   ULONG   RxCellsDrop;        // Number of Received Cells are dropped 
   ULONG   RxOAMCellsOk;       // Number of F4/F5/OAM Cells received are OK
   ULONG   RxOAMCellsDrop;     // Number of F4/F5/OAM Cells received are dropped
   ULONG   RxPacketOk;         // Number of PDUs/packets/Frames received correctly or OK
   ULONG   RxPacketError;      // Number of PDUs/Packets/Frames received errorly (Bad length, etc.)
   ULONG   RxNoBuffer;         // Number of times the PDU/Packet/Frame was dropped because no buffer or overflow in the buffer,
   ULONG   RxTotalCells;       // Total Number of Cells received (Data cells, OAM cells, dropped cells, etc.)
   ULONG   RxCellsOutOfSynch;  // Number of times the data path out of synch.
   ULONG   RxAAL5Error;        // Number of PDUs/Packets/Frames received with AAL5 Errors
} ATM_UNI_COUNTERS, *PATM_UNI_COUNTERS;

typedef struct tATM_STATISTICS
{
    ULONG	txCellsOk;
    ULONG	rxCellsOk;
    ULONG	rxCellsDropped;
	ULONG	rxInvalidVpiVci;
    ULONG   rxCrc32;
	ULONG	OAMLoopBkSuccess;	// # of loopback cells received successfully
	//ULONG	OAMLoopBkSuccessTx;	// # of loopback cells transmit successfully
} ATM_STATISTICS, *PATM_STATISTICS;


typedef struct tLOOPBACK_VPIVCI
{
   UCHAR   Vpi;
   USHORT  Vci;
} LOOPBACK_VPIVCI, *PLOOPBACK_VPIVCI;

typedef struct tOAMLOOPBACK_STRUCT
{
   UCHAR   type;
   UCHAR   vpi;
   USHORT  vci;
   UCHAR   pti;
   UCHAR   reserved[52];
} OAMLOOPBACK_STRUCT, *POAMLOOPBACK_STRUCT;


//------------------------------------------------------------------------------
//
// ATM_UNI_AAL
//		Defines the possible values for the ATM UNI AAL (ATM Adaptation Layer)
//		The AAL value is specified on a per-VC basis
//
//------------------------------------------------------------------------------
typedef enum
 {
   AAL1_,
   AAL2_,
   AAL34_,
   AAL5_
} ATM_UNI_AAL;

//------------------------------------------------------------------------------
//
// ATM_UNI_STATUS
//		Defines the status values returned by ATM UNI functions
//
//------------------------------------------------------------------------------
typedef enum {
   ATM_UNI_STATUS_SUCCESS,
   ATM_UNI_STATUS_FAILURE,
   ATM_UNI_STATUS_PENDING,
   ATM_UNI_STATUS_RESOURCES,
   ATM_UNI_STATUS_DUPLICATE_VPI_VCI,
   ATM_UNI_STATUS_INVALID_DATA
} ATM_UNI_STATUS;

//------------------------------------------------------------------------------
//
// ATM_UNI_USER_RECEIVE
//		Defines the prototype for the user receive function.  The user receive
//      function is called to indicate reassembled PDUs the owner of a VC.
//
//------------------------------------------------------------------------------
typedef void (*ATM_UNI_USER_RECEIVE) 
                 ( ATM_UNI_USER_HANDLE, ATM_UNI_USER_HANDLE, 
                   PXFER_DATA_BUFFER, UCHAR, UCHAR); 

//------------------------------------------------------------------------------
//
// ATM_UNI_USER_LINK_STATUS_CHANGE
//		Defines the prototype for the user link status change function.  The
//		user link status change function is called whenever the status of
//		the physical link has changed.
//
//		Note:  This routine is optional and specified when the ATM UNI is 
//			   created.
//
//------------------------------------------------------------------------------
typedef void (*ATM_UNI_USER_LINK_STATUS_CHANGE) 
                 ( ATM_UNI_USER_HANDLE,ATM_PHY_LINK_STATE,ULONG,ULONG,USHORT);

//------------------------------------------------------------------------------
//
// ATM_UNI_USER_RECEIVE_PACKET
//		Defines the prototype for the user receive packet function.  The user 
//		receive packet function is called to indicate reassembled PDUs 
//		the owner of a VC in the form of NDIS packets.
//
//------------------------------------------------------------------------------
typedef void (*ATM_UNI_USER_RECEIVE_PACKET) 
                 ( ATM_UNI_USER_HANDLE, ATM_UNI_USER_HANDLE, 
                   PNDIS_PACKET,UCHAR,UCHAR);
                  
//------------------------------------------------------------------------------
//
// ATM_UNI_USER_SEND_COMPLETE
//		Defines the prototype for the user send complete function.  The user
//		send complete function is called to indicate to the owner of a VC that
//		a previously submitted send request has completed.  The user send 
//		complete function is NOT called if the send request completes
//		synchronously (i.e. with status ATM_UNI_STATUS_SUCCESS)
//
//------------------------------------------------------------------------------
typedef void (*ATM_UNI_USER_SEND_COMPLETE)
                 ( ATM_UNI_USER_HANDLE, ATM_UNI_USER_HANDLE, PNDIS_PACKET);


//------------------------------------------------------------------------------
//
// ATM_UNI_VC_STATE
//		Defines the possible states that an ATM UNI VC can take
//
//------------------------------------------------------------------------------
typedef enum {
	ATM_UNI_VC_STATE_INACTIVE,
	ATM_UNI_VC_STATE_RESOURCES_RESERVED,
	ATM_UNI_VC_STATE_ACTIVE

} ATM_UNI_VC_STATE;


//
// Forward declaration for cross linked structures
//
typedef struct tATM_UNI;

//------------------------------------------------------------------------------
//
// ATM_UNI_VC
//		Describes an ATM UNI VC (virtual circuit)
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI_VC {
	LIST_NODE       			Link;
	ATM_UNI_VC_STATE			State;
	NDIS_SPIN_LOCK				Lock;
	struct tATM_UNI *			AtmUni;

	ATM_UNI_USER_HANDLE     	UserVcHandle;
	ATM_UNI_USER_HANDLE			UserPacketContext;
	ATM_UNI_USER_RECEIVE      	UserReceive;
	ATM_UNI_USER_RECEIVE_PACKET	UserReceivePacket;
	ATM_UNI_USER_SEND_COMPLETE	UserSendComplete;

	ULONG           			VpiVci;
	ATM_UNI_AAL     			Aal;
	UCHAR						CpcsUu;
	UCHAR						Cpi;
	BOOLEAN						Clp;
	UCHAR           			CellHeader[ ATM_CELL_FULL_HEADER_SIZE];

	USHORT          			MaxSduSize;
	PUCHAR          			VcEncapsulation;
	USHORT          			VcEncapsulationLength;
    int                         Rfc1483Mode;
    PUCHAR                      ArpEncapsulation;
	USHORT          			ArpEncapsulationLength;


	ULONG           			TxPartialCrc;
	UCHAR           			TxCellRemainder;

	PXFER_DATA_BUFFER			CurrentRxBuffer;
	PUCHAR	         			ReassemblyPointer;
	ULONG   	     			ReassemblySpace;
	ULONG        	 			RxPartialCrc;
    NDIS_HANDLE                 ShortPacketsBufferPool;

	ATM_UNI_STATS				Stats;
   ATM_UNI_COUNTERS                AtmCounters;

} ATM_UNI_VC, *PATM_UNI_VC;

//------------------------------------------------------------------------------
//
// ATM_UNI
// 		Describes an ATM UNI (User-Network Interface)
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI {
	ATM_UNI_USER_HANDLE				UserHandle;
	ATM_UNI_USER_LINK_STATUS_CHANGE	UserLinkStatusChange;
	ATM_UNI_USER_SEND_COMPLETE	    UserSendComplete;
	BOOLEAN							IndicatePackets;
	ULONG							MediaSpecificSize;
	NDIS_HANDLE	      				MiniportAdapterHandle;
	ATM_PHY_HANDLE	    			PhyHandle;
	ATM_PHY_SEND					PhySend;
	ATM_PHY_UNBIND					PhyUnbind;
    ATM_UNI_STATE                   AtmUniState;

	UCHAR							AtmPhyRxPadding;

    BOOLEAN							isOutOfSync;
   UCHAR                           InvalidHECLimitCounter;
    ULONG							validHeaderCount;
	UCHAR							PhyUniqueId[ UNIQUE_ID_SIZE];
	USHORT             				MaxSduSize;
	ATM_PHY_LINK_STATE				LinkState;
	ULONG							UpstreamRate;
	ULONG							DownstreamRate;

	NDIS_SPIN_LOCK     				VcListLock;
	LIST_NODE          				VcHashTable[ ATM_UNI_VC_HASH_TABLE_SIZE];
	LIST_NODE						InactiveVcList;

	NDIS_SPIN_LOCK					RxBufferFreeListLock;
	SINGLE_LIST_NODE    			RxBufferFreeList;
	ULONG             				RxBufferSize;
	USHORT							RxBufferCount;
	NDIS_HANDLE						RxPacketPoolHandle;
	NDIS_HANDLE						RxBufferPoolHandle;

	NDIS_SPIN_LOCK					TxBufferFreeListLock;
	SINGLE_LIST_NODE    			TxBufferFreeList;
	ULONG             				TxBufferSize;
	USHORT							TxBufferCount;
	USHORT							AtmTxCount;

	BOOLEAN							NeedQueueAtmProcess;
	LIST_NODE						ProcessBufferFreeList;
	LIST_NODE    					ProcessBufferTxList;

#ifdef MESSAGE_HANDLING
	PATM_UNI_VC						MsgHandling_SiemensHandle;
	PATM_UNI_VC						MsgHandling_EciHandle;
   	NDIS_SPIN_LOCK     				MsgHandlingLock;
	NDIS_EVENT						MsgHandling_Event;
	NDIS_MINIPORT_TIMER				MsgHandling_Timer;

	ULONG							MsgHandling_StateMachine;
	ULONG							MsgHandling_Received;
	BOOLEAN							MsgHandling_Connected;
	UCHAR							MsgHandling_Pdu[2000];
	USHORT							MsgHandling_BufferSize;
	USHORT							MsgHandling_PduSize;
#endif // MESSAGE_HANDLING

#ifdef USB_WIN_AUTO_CONFIG
    PVOID                           pAutoConfig;
#endif // USB_WIN_AUTO_CONFIG

	USHORT							COType;

	ATM_UNI_STATS					Stats;
    DSL_EX                          DslEx;
	NDIS_MINIPORT_TIMER				DspStatusTimer;
	NDIS_EVENT						DspStatusTimerEvent;
    NDIS_SPIN_LOCK                  DspSpinLock;
    NDIS_PACKET_QUEUE               NdisPacketQueue;

#if 0 //KJP FIXME
    PATM_UNI_VC                     AtmUniVc[128];
#endif //KJP FIXME
//KJP FIXME
    /* FIXME: temp fix to work around getting buffers with loopback / auto-config
    /  PVCs which would then skip dequeueing NDIS PACKETs.  We always want to try
    /  and grab the next NDIS PACKET.  We should really save the PVC pointer for
    /  every NDIS PACKET */
    PATM_UNI_VC                     AtmUniVc;
//KJP FIXME

    PATM_UNI_VC                     LbAtmUniVc;
    ULONG                           LbVpiVci;
   
   ATM_UNI_COUNTERS                AtmCounters;
  
} ATM_UNI, *PATM_UNI;


//------------------------------------------------------------------------------
//
// Function Declarations
//
//------------------------------------------------------------------------------
void
AtmUniLinkStatusChange(
	PATM_UNI			AtmUni,
	ATM_PHY_LINK_STATE	LinkState,
	ULONG				UpstreamRate,
	ULONG				DownstreamRate,
	USHORT              COType,
	UCHAR				RxCellPadding
);

#ifdef ATMUNI_C
static void
AtmUniSendComplete(
	PATM_UNI			AtmUni,
	PXFER_DATA_BUFFER 		TxBuffer,
	ATM_PHY_USER_HANDLE	NotUsed,
	ATM_PHY_STATUS		Status
);
#endif // ATMUNI_C

USHORT
AtmUniAal5Segment(
	PATM_UNI_VC AtmUniVc,
	PPUCHAR	   	OutputBuffer,
	PUCHAR		InputBuffer,
	USHORT      InputLength,
	BOOLEAN     EndOfPdu,
	USHORT      PduLength
);

#ifdef  ATMUNI_C
static USHORT
AtmUniAal5WritePadAndTrailer(
	PATM_UNI_VC	 AtmUniVc,
	PPUCHAR    	 OutputBuffer,
	UCHAR        BytesLeftInCurrentCell,
	USHORT       PduLength
);

static BOOLEAN 
AtmUniHECValid(
	PATM_UNI AtmUni, 
	PUCHAR Buffer
);

static UCHAR
AtmUniAal5Reassemble(
   PATM_UNI			AtmUni,
   PXFER_DATA_BUFFER 	InputBuffer
);

static BOOLEAN 
UpdateSyncCount( 
	PATM_UNI	AtmUni 
);

static BOOLEAN ProcessInboundLoopback( 
    PATM_UNI AtmUni, 
    PATM_UNI_VC AtmUniVc, 
    PUCHAR Buffer
);
#endif //ATMUNI_C

#ifdef _NTDDK_
void
AtmUniGetAtmStats(
   PATM_UNI			AtmUni,
   PATM_STATISTICS  atmStats
);
#elif defined(linux)
void AtmUniGetAtmStats( PATM_UNI AtmUni, PATM_UNI_STATS atmStats);
#endif // linux

void AtmUniClearAtmStats(PATM_UNI AtmUni);

#ifdef ATMUNI_C
static void
AtmUniAal5ProcessInboundCell(
	PATM_UNI_VC		AtmUniVc,
	PUCHAR			Buffer,
	BOOLEAN			EndOfPdu,
	PULONG			ReassembledPduLength,
	PAAL5_TRAILER *	Aal5Trailer
);
#endif // ATMUNI_C

BOOLEAN
AtmUniAllocateBuffers(
   PATM_UNI	AtmUni,
   USHORT   RxBufferCount,
   USHORT   TxBufferCount
);

BOOLEAN
AtmUniBindWithPhy(
	PATM_UNI 				AtmUni,
	ATM_PHY_BIND			PhyBind,
	ATM_PHY_DEVICE_HANDLE	PhyDeviceHandle
);

void AtmUniClosePhyBinding( PATM_UNI AtmUni);

void AtmUniFreeBuffers( PATM_UNI AtmUni);

#ifdef ATMUNI_C
static PATM_UNI_VC
AtmUniLookupVc(
   PATM_UNI  AtmUni,
   ULONG     VpiVci
);
#endif // ATMUNI_C

void
AtmUniAddVcToTable(
	PATM_UNI	AtmUni,
	PATM_UNI_VC	AtmUniVc
);

void
AtmUniRemoveVcFromTable(
	PATM_UNI	AtmUni,
	PATM_UNI_VC	AtmUniVc
);

void AtmProcessFunction( IN PATM_UNI AtmUni);

void AtmUniDeleteAllVcs( PATM_UNI AtmUni);

BOOLEAN
AtmUniInitialize( void);

ATM_UNI_HANDLE
AtmUniCreate( 
	ATM_UNI_USER_HANDLE				UserHandle,
	ATM_UNI_USER_LINK_STATUS_CHANGE	UserLinkStatusChange,
	BOOLEAN							IndicatePackets,
	ULONG							MediaSpecificInformationSize,
	USHORT							MaxSduSize, 
	USHORT							RxBufferCount, 
	USHORT							TxBufferCount, 
	NDIS_HANDLE						MiniportAdapterHandle,
	ATM_PHY_BIND 					PhyBind,
	ATM_PHY_DEVICE_HANDLE			PhyDeviceHandle,
	PUCHAR							PhyUniqueId,
	PVOID							UsbPhyHandle
);

void AtmCheckDspStatus(
	PVOID				SystemSpecific1,
	PATM_UNI			AtmUni,	   
	PVOID				SystemSpecific2,
	PVOID				SystemSpecific3
);

void AtmUniTxTimer( 
	PVOID				SystemSpecific1,
	PATM_UNI         AtmUni,
	PVOID				SystemSpecific2,
	PVOID				SystemSpecific3);
   
ATM_UNI_STATUS AtmUniTxProcess(
                   PATM_UNI_VC         AtmUniVc,
                   PNDIS_PACKET        Packet,
                   PXFER_DATA_BUFFER   TxBuffer);

void AtmUniShutdownDspStatusEx( PATM_UNI AtmUni);
void AtmUniShutdownDspStatus( PATM_UNI AtmUni);
void AtmUniShutdownTransmit( PATM_UNI AtmUni);
void AtmUniFreeProcessBuffer( PATM_UNI AtmUni, PPROCESS_DATA_BUFFER	ProcessBuffer);
void AtmUniFreeTransmitBuffer( PATM_UNI AtmUni, PXFER_DATA_BUFFER Buffer);
PXFER_DATA_BUFFER AtmUniGetTransmitBuffer( PATM_UNI AtmUni);
BOOLEAN AtmUniQueueTransmitBuffer( PATM_UNI AtmUni, PXFER_DATA_BUFFER Buffer);
void AtmUniProcessBuffers( PATM_UNI AtmUni);

void
AtmUniDestroy( ATM_UNI_HANDLE AtmUniHandle);

ATM_UNI_HANDLE
AtmUniCreateVc(
	ATM_UNI_HANDLE				AtmUniHandle,
   	ATM_UNI_USER_HANDLE     	UserVcHandle,
  	ATM_UNI_USER_RECEIVE      	UserReceive,
	ATM_UNI_USER_RECEIVE_PACKET	UserReceivePacket,
   	ATM_UNI_USER_SEND_COMPLETE	UserSendComplete
);

void
AtmUniDeleteVc(	ATM_UNI_HANDLE	AtmUniVcHandle);

ATM_UNI_STATUS
AtmUniActivateVc( 
   ATM_UNI_HANDLE	AtmUniVcHandle,
   UCHAR       		Vpi, 
   USHORT      		Vci, 
   ATM_UNI_AAL 		Aal, 
   USHORT      		MaxSduSize, 
   PUCHAR     		VcEncapsulation, 
   USHORT      		VcEncapsulationLength,
   BOOLEAN			ReserveResources,
   UCHAR            Gfc
);

void
AtmUniDeactivateVc( ATM_UNI_HANDLE AtmUniVcHandle);
 
void NdisPacketAddVcToTable( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc);
void NdisPacketRemoveVcFromTable( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc);
BOOLEAN NdisPacketLookupVc( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc, PUCHAR Index);
 
ATM_UNI_STATUS
AtmUniSend( 
	ATM_UNI_HANDLE	AtmUniVcHandle, 
	PNDIS_PACKET 	Packet,
	BOOLEAN			SkipFirstBuffer,
	UCHAR			CpcsUu,
	UCHAR			Cpi,
	BOOLEAN			Clp,
    BOOLEAN          ArpFlag
);


#ifdef USB_WIN_AUTO_CONFIG
void
AcAtmUniSend(
	PATM_UNI_VC         AtmUniVc,
	PVOID        		PduData,
    USHORT              PduLength,
	UCHAR				CpcsUu,
	UCHAR				Cpi
);
#endif // USB_WIN_AUTO_CONFIG


void
AtmUniSetVcPacketContext(
	ATM_UNI_HANDLE	AtmUniVcHandle,
	PVOID			Context
);

PVOID
AtmUniGetVcPacketContext( PNDIS_PACKET	Packet);

ATM_UNI_USER_HANDLE
AtmUniGetUserVcHandle( ATM_UNI_HANDLE	AtmUniVcHandle);

void
AtmUniGetStats(
	ATM_UNI_HANDLE	AtmUniHandle,
	PATM_UNI_STATS	AtmUniStats
);

void
AtmUniGetVcStats(
	ATM_UNI_HANDLE	AtmUniVcHandle,
	PATM_UNI_STATS	AtmUniStats
);

void
AtmUniReceiveComplete(
   ATM_UNI_HANDLE  	AtmUniVcHandle, 
   PXFER_DATA_BUFFER		RxBuffer
);

void
AtmUniReceivePacketComplete(
   ATM_UNI_HANDLE  	AtmUniVcHandle, 
   PNDIS_PACKET		Packet
);

void AtmUniEmptySendPacketQueue(PATM_UNI AtmUni);
int AtmUniGetRfc1483Mode(ATM_UNI_HANDLE AtmUniVcHandle);
BOOLEAN AtmUniSetRfc1483Mode(ATM_UNI_HANDLE AtmUniVcHandle , int Rfc1483Mode );
BOOLEAN AtmUniSetRfc1577Encap(ATM_UNI_HANDLE AtmUniVcHandle , PUCHAR Encap , USHORT EncapLen );
ATM_UNI_HANDLE AtmUniIsVCExist(ATM_UNI_HANDLE AtmHandle , unsigned int VPI , unsigned int VCI );
PXFER_DATA_BUFFER AtmUniGetTxBuffer( ATM_UNI_HANDLE AtmHandle );
BOOLEAN AtmUniFreeTxBuffer(ATM_UNI_HANDLE AtmHandle, PXFER_DATA_BUFFER TxBuffer);
ATM_UNI_USER_HANDLE AtmUniGetUserHandle(ATM_UNI_HANDLE AtmHandle);
BOOLEAN AtmUniShortPacketPaddingAdd( ATM_UNI_HANDLE AtmHandle, PNDIS_PACKET pNdisPacket,ULONG ulSize);
BOOLEAN AtmUniShortPacketPaddingRemove( ATM_UNI_HANDLE AtmHandle, PNDIS_PACKET pNdisPacket);

void AtmUniSendF4F5_Loopback	(ATM_UNI_HANDLE	AtmHandle, PLOOPBACK_VPIVCI VpiVci);
void AtmUni_OAM_IF(ATM_UNI_HANDLE AtmHandle, POAMLOOPBACK_STRUCT oamPtr);

#endif // ATMUNI_H_
