////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.2 $
//     $Modtime: 5/15/02 12:18p $
//     $Author: kattre $
//
// ABSTRACT:
//       Contains data structures and macros for the ATM USB driver
//
////////////////////////////////////////////////////////////////////////////////
#ifndef ATMUSB_H
#define ATMUSB_H

#include "debug.h"
#if defined(linux)
#include "GpSnull.h"
#endif // linux
#include "atmuni.h"
#include "gti.h"
#include "gsi_ioctl.h"
#ifdef _NTDDK_
#include "version.h"
#ifdef WAN_MINIPORT
#include <ndiswan.h>
#include <ndistapi.h>
#endif /* WAN_MINIPORT */
#endif // _NTDDK_



#define PLINEPTR_NUM_LINKS             1   // Defines logical links to be 1:1 with physical links
#define PLINEPTR_TAPI_NUM_ADDRESSES    1   // Only one TAPI address ID per line device(zero based)
#define PLINEPTR_TAPI_NUM_CALLS        1   // There is only one TAPI call per line instance.
#define PLINEPTR_MAX_DIALED_VC         9   // includes room for comma (e.g. "1023,1023")

#define PLINEPTR_NUM_TRAIN_RETRIES     60
#define VALUE_IN_SECOND                1000
#define MAX_DROP_COUNTER               1
#define PLINEPTR_NO_ANSWER_TIMEOUT     40000   // 40 seconds
#define PLINEPTR_NO_TRAIN_TIMEOUT      3000

#define _7MBPS                         70000   // 7 meg (max in 100bps)
#define PLINEPTR_LINK_SPEED            _7MBPS  // Media link speed in bits per second.

#define PLINEPTR_MAX_TRANSMITS         ATM_USB_DEFAULT_TX_BUFFER_COUNT
#define PLINEPTR_SEND_WINDOW           PLINEPTR_MAX_TRANSMITS
#define MAX_PDU_SIZE                   1492

#define IS_VALID_LINK(Instance, Link)  (Link && (Link->Instance == Instance))

typedef struct _PLINEPTR_CALL          PLINEPTR_CALL, *PPLINEPTR_CALL , *PATM_USB_INSTANCE_CALL;
#ifdef WAN_MINIPORT
typedef struct _PLINEPTR_LINK          PLINEPTR_LINK, *PPLINEPTR_LINK , *PATM_USB_INSTANCE_LINK;
#endif // WAN_MINIPORT

#if defined(linux)
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,43))
#define net_device device
#endif /* LINUX_VERSION_CODE */

#define _NELEM(arr)     (sizeof(arr) / sizeof(arr[0]))
#define _OFFSETOF(t,m)  ((u32) &((t *)0)->m)
#define _MALIGN(l)      (((l) + 4 - 1) & ~(4 - 1))

#ifdef GSI_LAN
#define GSI_LAN_BUFFER_TAG          0xBABE0001
#define GSI_LAN_NUM_NDIS_PACKETS    8
#define GSI_LAN_NDIS_BUFFER_SIZE    1536

#define _TX_FULL        (0x01 << 0)
#define _NO_CARRIER     (0x01 << 1)

typedef struct {
    struct net_device          *dev;
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,43))
    char                        devName[IFNAMSIZ];
 #endif /* LINUX_VERSION_CODE */
    u8                         *bufferPool[GSI_LAN_NUM_NDIS_PACKETS];
    NDIS_HANDLE                 NdisBufferPoolHandle;
    NDIS_HANDLE                 NdisPacketPoolHandle;
    struct net_device_stats     stats;
    spinlock_t                  slock;
    u8                          flags;
    int                         txAvailable;
    u8                          txHead;
    PNDIS_PACKET                txPacket;
	ATM_UNI_HANDLE				Rfc1483VcHandle; /* This is the PVC handle which we will be using later */
	struct _adapter_t			*pAdapter ; /* Lets keep a pointer to our adapter */
} gsi_lan_t;
#endif // GSI_LAN

#ifdef GSI_WAN
#define GSI_WAN_BUFFER_TAG          0xBABE0002
#define GSI_WAN_NUM_NDIS_PACKETS    8
#define GSI_WAN_NDIS_BUFFER_SIZE    1536
#define GSI_WAN_MAX_CHANNELS        1
#define GSI_WAN_DEF_CHANNEL         0

typedef struct {
    struct _adapter_t  *pAdapter;
    int                 count;
    u32                 flags;
    spinlock_t          slock;
    struct tq_struct    tqueue;
    struct termios      normal_termios;
    struct termios      callout_termios;
    struct tty_struct  *tty;
    unsigned long       event;
    u8                  msr;
    u8                  mcr;
    u8                  lsr;
    u8                 *bufferPool[GSI_WAN_NUM_NDIS_PACKETS];
    NDIS_HANDLE         NdisBufferPoolHandle;
    NDIS_HANDLE         NdisPacketPoolHandle;
    int                 txAvailable;
    u8                  txHead;
    u32                 txBytes;
    PNDIS_PACKET        txPacket;
    u32                 txPacketLen;

    u8                  rxData[GSI_WAN_NDIS_BUFFER_SIZE];
    u8                  rxFlags[GSI_WAN_NDIS_BUFFER_SIZE];
    wait_queue_head_t   HangupCheckQ;
} channel_t;

#define _TTY_ALLOC      (0x01 << 0)
#define _CU_ALLOC       (0x01 << 1)
#define _TX_HALTED      (0x01 << 2)
#define _RX_HALTED      (0x01 << 3)

typedef struct {
    u8                  flags;
    struct tty_driver   tty;
    struct tty_driver   cu;
    int                 refcount;
    struct tty_struct  *table[GSI_WAN_MAX_CHANNELS];
    struct termios     *termios[GSI_WAN_MAX_CHANNELS];
    struct termios     *termios_locked[GSI_WAN_MAX_CHANNELS];
    channel_t           channel[GSI_WAN_MAX_CHANNELS];
} gsi_wan_t;
#endif // GSI_WAN

#define _LAN_OPEN        (0x01 << 0)
#define _WAN_OPEN          (0x01 << 1)
#define _IRQ_ALLOC          (0x01 << 2)
#define _LAN_ALLOC          (0x01 << 3)
#define _WAN_ALLOC          (0x01 << 4)
#define _ATMUNI_ALLOC       (0x01 << 5)
#define _RFC1483_ALLOC      (0x01 << 6)
#define _RFC1577_ALLOC      (0x01 << 7)
#define _RFC2364_ALLOC      (0x01 << 8)
#define _RFC2364_VC_ALLOC   (0x01 << 9)
#define _USB_BUS_CONNECTED  (0x01 << 10)

#endif // linux

#ifndef RFC_HANDLE_DEFINED
/* Used to pass pointer between two layers */
#define RFC_HANDLE_DEFINED
typedef void *     RFC2516_HANDLE;
typedef void *     RFC2516_VC_HANDLE;
typedef void *     RFC1483_HANDLE;
typedef void *     RFC1483_VC_HANDLE;
typedef void *     SESSION_HANDLE;
#endif // RFC_HANDLE_DEFINED

#define REG_PARAM_LENGTH               40
#define GSI_NUM_OF_XCVR                1

#ifdef _NTDDK_
//------------------------------------------------------------------------------
//
// Global Variable Declarations
//
//------------------------------------------------------------------------------
extern ANSI_STRING Global_AtmUsbVendorDescriptor;
#endif // _NTDDK_


//------------------------------------------------------------------------------
//
// Macro Definitions
//
//------------------------------------------------------------------------------
#define ATM_USB_VENDOR_DESCRIPTOR        GUSB_DESCRIPTION

#ifdef WAN_MINIPORT
#define ATM_USB_NDIS_MAJOR_VERSION     3
#else
#define ATM_USB_NDIS_MAJOR_VERSION     5
#endif

#define ATM_USB_NDIS_MINOR_VERSION     0
#define ATM_USB_DRIVER_VERSION         ((ATM_USB_NDIS_MAJOR_VERSION << 8) | \
                                          ATM_USB_NDIS_MINOR_VERSION)
#define ATM_USB_DEFAULT_RX_BUFFER_COUNT    50
#define ATM_USB_DEFAULT_TX_BUFFER_COUNT    70

#ifdef _NTDDK_
/* Default parameter values for PPPoE(Rfc2516) layer:
** Use this value, if can not find in the registry
*/
#define RFC2516_DEFAULT_DISABLE        FALSE
#define RFC2516_DEFAULT_TIMER_VALUE    3    /* 3 * 1000 Sec. */
#define RFC2516_NUM_OF_RETIRES         20

#if defined (ATM_MINIPORT)

#define    ATMMP_DEFAULT_MAX_SDU_SIZE  18190    // minimum for Classic IP and LANE
#define    ATMMP_DEFAULT_MAX_VCS       16
#endif // ATM_MINIPORT
#endif // _NTDDK_

#if defined (LAN_MINIPORT) || defined (PPPOE_MINIPORT) || defined(GSI_LAN)

#define RFC1483_DEFAULT_VPI1               0
#define RFC1483_DEFAULT_VPI2               1
#define RFC1483_DEFAULT_VPI3               2
#define RFC1483_DEFAULT_VPI4               3

#define RFC1483_DEFAULT_VCI1               85 //50
#define RFC1483_DEFAULT_VCI2               86 //50
#define RFC1483_DEFAULT_VCI3               87 //50
#define RFC1483_DEFAULT_VCI4               88 //50
#define RFC1483_MAX_MULTICAST_ADDRESSES    32

//------------------------------------------------------------------------------
//
// RFC1483_MODE
//    This enumerated type defines the possible values for the RFC1483 mode
//
//------------------------------------------------------------------------------

typedef enum {
    RFC1483_MODE_BRIDGED_ETH_LLC   = 0,
    RFC1483_MODE_BRIDGED_ETH_VC    = 1,
    RFC1483_MODE_ROUTED_IP_LLC     = 2,
    RFC1483_MODE_ROUTED_IP_VC      = 3, 
    RFC1483_MODE_RFC1577_ENCAP     = 4
} RFC1483_MODE;

#define RFC1483_DEFAULT_MODE           RFC1483_MODE_BRIDGED_ETH_LLC
#define RFC1577_DEFAULT_DISABLE        FALSE
#define RFC1577_DEFAULT_ENABLE         TRUE
//---------------------------------------------------------------------------------------
// RFC1577
//-----------------------------------------------------------------------------------------

#define RFC1577_SPIN_LOCK_COUNT           8
#define RFC1577_TIMER_COUNT               16
#define RFC1577_HANDLE                  ULONG 



#define E164_ADDR          0x8          
#define NSAP_ADDR          0x3          
#define MAX_ATMADDR_LEN    20           /* Maximum ATM address len */

#define ATM_ADDRESS_13_BYTES_STRING                 "1234567890123"
#define ATM_ADDRESS_13_BYTES_STRING_SIZE            13     
#define MAX_ARP_ENTRIES           16 

//////////////////////////////////////////////////////////////////////////////
// Used for saving Buffer pointer in the NDIS_PACKET
//////////////////////////////////////////////////////////////////////////////
#define SET_PACKET_RFC_MINIPORT_RESERVED( _Pkt_, _BuffPtr_)                    \
{                                                                              \
    PULONG  _Temp;                                                              \
    _Temp = ((PULONG)((PUCHAR)(_Pkt_)->MiniportReservedEx + sizeof(PVOID)));    \
    *(_Temp) = (ULONG)(_BuffPtr_);                                              \
}

#define GET_PACKET_RFC_MINIPORT_RESERVED( _Pkt_ )                              \
    (PVOID)(*(PULONG)((_Pkt_)->MiniportReservedEx + sizeof(PVOID)))
    
#if defined(linux)
typedef struct tATM_USB_INSTANCE *PATM_USB_INSTANCE;
#endif // linux

typedef NDIS_STATUS(*RFC1577RFC1483SEND)(
                            PATM_USB_INSTANCE, 
                            PNDIS_PACKET,  
                            ATM_UNI_HANDLE, 
                            BOOLEAN,
							void *plan
                            );

typedef void (*RFC1577RFC1483RECEIVECOMPLETE)(PATM_USB_INSTANCE , 
                                    PXFER_DATA_BUFFER );

typedef void (*RFC1483RFC1577SENDCOMPLETE)(
                PATM_USB_INSTANCE    ,
                ATM_UNI_USER_HANDLE    ,
                PNDIS_PACKET           ,
                BOOLEAN );

typedef void (*RFC1483RFC1577RECEIVE)(
            void *,
            ATM_UNI_USER_HANDLE        ,
            PXFER_DATA_BUFFER        ,
            PUCHAR                  ,
            ULONG  ,
            BOOLEAN );

typedef int (* RFC1577CLOSE)(void *);

typedef struct CONFIG {
    char ATMAddress[MAX_ATMADDR_LEN];
    char ATMAddrType;
    char ATMSubAddress[MAX_ATMADDR_LEN];
    char ATMSubAddrType;

    
    /*
     * Currently Not used, used only in the SVC  based environment
     */
    char ServATMAddress[MAX_ATMADDR_LEN];
    char ServAddrType;
    char ServSubATMAddress[MAX_ATMADDR_LEN];
    char ServATMAddrType;
    unsigned char  DefaultVPI;
    unsigned short  DefaultVCI;
    int  DefaultEncap;
    unsigned int DefaultIPAddr;
    ATM_UNI_HANDLE DefaultHandle;

    RFC1577RFC1483SEND            Rfc1577Rfc1483Send; //gs
    RFC1577RFC1483RECEIVECOMPLETE Rfc1577Rfc1483ReceiveComplete; //gs
    RFC1483RFC1577SENDCOMPLETE    Rfc1483Rfc1577SendComplete;    //fa
    RFC1483RFC1577RECEIVE         Rfc1483Rfc1577Receive;         //fa
    RFC1577CLOSE                  Rfc1577Close;                   //fa
                                                                 
     /*
     * Following will be filled by Rfc1577 and returned
     */
    void *Rfc1577Context;    //FA
    BOOLEAN Rfc1577Enable;   //gs
    ULONG  Rfc1577MTUSize;                 

}Rfc1577Config_t;
    
typedef struct  Rfc1577Status 
{
    unsigned int IntfAddr;
    unsigned int TgtIPAddr;
    UCHAR VPI;
    USHORT VCI;
    int  State;
    int NoOfTries;
    int  TTL; 
} Rfc1577Status_t;


typedef struct _QUERY_ARP_BUFFER
{
int Count;
Rfc1577Status_t ArpEntry[MAX_ARP_ENTRIES];
}QUERY_ARP_BUFFER , *PQUERY_ARP_BUFFER;

typedef struct _RFC1577_SPIN_LOCK
{
    NDIS_SPIN_LOCK NdisSpinLock;
    BOOLEAN busy;
}RFC1577_SPIN_LOCK , *PRFC1577_SPIN_LOCK;

typedef struct _RFC1577_TIMER
{
    BOOLEAN busy;
    NDIS_MINIPORT_TIMER NdisTimer;
    NDIS_EVENT NdisTimerEvent;
    PNDIS_TIMER_FUNCTION   TimerFunction;
}RFC1577_TIMER , *PRFC1577_TIMER;


typedef struct _RFC1577
{

    NDIS_HANDLE                Rfc1577NdisPktPoolHandle;
    NDIS_HANDLE                Rfc1577NdisBuffPoolHandle;


    RFC1577_SPIN_LOCK Rfc1577SpinLock[RFC1577_SPIN_LOCK_COUNT];
    RFC1577_TIMER     Rfc1577Timer[RFC1577_TIMER_COUNT]; 

}RFC1577 , *PRFC1577;

#endif // defined (LAN_MINIPORT) || defined (PPPOE_MINIPORT)

#if defined (WAN_MINIPORT) || defined(GSI_WAN)

#define RFC2364_DEFAULT_VCI         116
#define RFC2364_DEFAULT_VPI         0
#define RFC2364_LLC_ENCAP_SIZE    4 
#define RFC2364_VC_ENCAP_SIZE     0    
#define GSI_WAN_TTY_NAME            "ttyG"
#define GSI_WAN_CU_NAME             "cug"

typedef enum 
{
    RFC2364_MODE_VC                 = 0,
    RFC2364_MODE_LLC                = 1

}RFC2364_MODE;

#define RFC2364_DEFAULT_MODE    RFC2364_MODE_VC

#ifdef _NTDDK_
/* Default VPI/VCI will be zero to indicate 
** there was no entry in the registry, and
** the values being used is from phone number.
*/
#define DEFAULT_VPI            0
#define DEFAULT_VCI            0
#endif // _NTDDK_
#endif //WAN_MINIPORT

#if defined(WAN_MINIPORT) || defined(GSI_WAN)
typedef struct _PLINEPTR_CALL
{
   // The WAN wrapper supplied handle to be used on Miniport calls
   // such as NdisMWanIndicateReceive().
   NDIS_HANDLE    NdisLinkContext;

#ifdef _NTDDK_
   // The Connection wrapper supplied handle for use when indicating
   // TAPI LINECALLSTATE events.
   HTAPI_CALL     htCall;
#endif // _NTDDK_

   UINT           VPI;
   UINT           VCI;

#ifdef _NTDDK_
   ULONG          CallState;  // Current state
   // TAPI line addresses are assigned during installation and saved in the
   // registry.  It is read from the registry and saved here at init time.
   CHAR           LineAddress[0x14];

   // This timer is used to keep track of the call state when a call is
   // coming in or going out.
   NDIS_MINIPORT_TIMER     CallTimer;
   NDIS_EVENT            CallTimerEvent;

   struct _PLINEPTR_LINK *Link;   // Back ptr.

   //Number of times to check for training
   USHORT         trainCounter;
#endif //_NTDDK
}ATM_USB_INSTANCE_CALL;


#ifdef _NTDDK_   
// This structure contains all the link information maintained by the
// Miniport driver.  It encompasses both the TAPI and WAN connections.
typedef struct _PLINEPTR_LINK
{
   // The Connection wrapper supplied handle for use when indicating
   // TAPI LINE_EVENT's.
   HTAPI_LINE htLine;

   // The number of active calls on this link
   ULONG NumActiveCalls;

   //PATM_USB_INSTANCE Instance;
    PVOID    Instance;
   // Array of calls on the link (one call per address)
   PLINEPTR_CALL CallsOnLink[PLINEPTR_TAPI_NUM_ADDRESSES];
   ULONG CallStatesCaps;   // Events currently supported by all calls

   // Pointer back to the Adapter data structure so we can get at it
   // when passed a pointer to this structure in NdisMWanSend().
   //struct _PLINEPTR_ADAPTER *Adapter;

   // The current TAPI LINEDEVSTATE of the associated with the link.
   ULONG DevState;                 // Current state
   ULONG DevStatesCaps;            // Events currently supported
   ULONG DevStatesMask;            // Events currently enabled

   // The current TAPI LINEADDRESSSTATE of the associated with the link.
   ULONG AddressState;             // Current state
   ULONG AddressStatesCaps;        // Events currently supported
   ULONG AddressStatesMask;        // Events currently enabled

   // The current TAPI media mode(s) supported by the card.
   ULONG MediaMode;                // Current state
   ULONG MediaModesCaps;           // Events currently supported
   ULONG MediaModesMask;           // Events currently enabled

   // The speed provided by the link in bits per second.  This value is
   // passed up by the Miniport during the LINE_UP indication.
   ULONG LinkSpeed;

   // The quality of service provided by the link.  This value is passed
   // up by the Miniport during the LINE_UP indication.
   NDIS_WAN_QUALITY Quality;

   // The number of packets that should be sent before pausing to wait
   // for an acknowledgement.  This value is passed up by the Miniport
   // during the LINE_UP indication.
   USHORT SendWindow;

   // The current settings associated with this link as passed in via
   // the OID_WAN_SET_LINK_INFO request.
   NDIS_WAN_SET_LINK_INFO WanLinkInfo;
};
#endif // _NTDDK_
#endif // (WAN_MINIPORT)

//------------------------------------------------------------------------------
//
// ATM_USB_STATUS
//    This enumerated type defines the possible values for the ATM USB interface
//    status.  
//
//------------------------------------------------------------------------------
typedef enum {
    ATM_USB_STATUS_NONE            = 0x00000000,
    ATM_USB_STATUS_INITIALIZED     = 0x00000001,
    ATM_USB_SLINK_INITIALIZED      = 0x00000002,
    ATM_USB_STATUS_HALTING         = 0x00000004,
    ATM_USB_STATUS_SUSPENDING      = 0x00000008,
    ATM_USB_STATUS_HUNG            = 0x80000000
} ATM_USB_STATUS;

//------------------------------------------------------------------------------
//
// RFC_INSTANCE
//    Return values being defines for RFC layers
//
//------------------------------------------------------------------------------
typedef enum
{
   RFC_STATUS_SUCCESS,
   RFC_STATUS_FAILURE,
   RFC_STATUS_PENDING,
   RFC_STATUS_RESOURCES,
   RFC_STATUS_INVALID_DATA
} RFC_STATUS;

//------------------------------------------------------------------------------
//
// ATM_USB_INSTANCE
//    Defines the data kept for each adapter instance
//
//------------------------------------------------------------------------------
typedef struct tATM_USB_INSTANCE
{
    ATM_USB_STATUS  Status;
    NDIS_HANDLE     MiniportAdapterHandle;
    ATM_UNI_HANDLE  AtmUniHandle;
    UCHAR           UniqueId[UNIQUE_ID_SIZE];
    UCHAR           StationAddress[ UNIQUE_ID_SIZE];
    ATM_PHY_LINK_STATE  LinkState;
#ifdef _NTDDK_
    NDIS_EVENT              IrpEvent;
#endif // _NTDDK_

   LIST_NODE               PendingReceiveList;
   NDIS_SPIN_LOCK          PendingReceiveListLock;
   NDIS_MINIPORT_TIMER     PendingReceiveTimer;
   NDIS_EVENT              PendingReceiveTimerEvent;
   BOOLEAN                 NeedQueueRcvProcess;

#ifdef _NTDDK_
   UINT                    RxPcktsWithNdisCount;
   NDIS_EVENT              RxPcktsWithNdisZeroEvent;
#endif // _NTDDK_

    ULONG           UpstreamRate;
    ULONG           DownstreamRate;
    USHORT          TxBufferCount;
    USHORT          RxBufferCount;

#ifdef _NTDDK_
   PDEVICE_OBJECT          DeviceObject;
   NDIS_HANDLE             WrapperHandle;
   NDIS_HANDLE             NdisDeviceHandle;
   NDIS_STRING             deviceNameUniString;
   NDIS_STRING             symbolicNameUniString;    
   int                     DeviceIndex;
#endif // _NTDDK_
    PVOID           UsbPhyHandle;

#ifdef _NTDDK_
   DBG_PRINT_LEVEL         DbgPrintLevel;
   DBG_PRINT_MODULE        DbgPrintModule;
   UCHAR                   Rfc2516Enable; // Flag for the PPPoE stack (PPPOE_MINIPORT compile switch)
#endif // _NTDDK_
#if defined (ATM_MINIPORT)
   USHORT                  AtmMpMaxSduSize;
   ULONG                   AtmMpMaxVcs;
   NDIS_HANDLE             AtmMpPacketPoolHandle;
   NDIS_HANDLE             AtmMpBufferPoolHandle;
   PNDIS_REQUEST           NdisRequest;
#endif // ATM_MINIPORT
#if defined (LAN_MINIPORT) || defined (PPPOE_MINIPORT) || defined(GSI_LAN)
    RFC1483_MODE    Rfc1483Mode;
    RFC1483_HANDLE  Rfc1483Handle;
    PUCHAR          Rfc1483Encapsulation;
    UCHAR           Rfc1483EncapsulationSize;
    UCHAR           Rfc1483Vpi[MAX_PVC_SUPPORTED];      /* these are the different VCIs as configured by the user */
    USHORT          Rfc1483Vci[MAX_PVC_SUPPORTED];      /* these are the different VCIs as configured by the user */
    ATM_UNI_HANDLE  Rfc1483VcHandle[MAX_PVC_SUPPORTED]; /* These are the number of PVC handles */
    ULONG           Rfc1483PacketFilter;
    UCHAR           Rfc1483MulticastFilter[RFC1483_MAX_MULTICAST_ADDRESSES * ETH_LENGTH_OF_ADDRESS];
    USHORT          Rfc1483MulticastFilterSize;
    RFC1577         Rfc1577;
    BOOLEAN         Rfc1577Enable;
    Rfc1577Config_t Rfc1577Config;
#if defined(linux)    
    gsi_lan_t       lan[MAX_PVC_SUPPORTED]; /* Lets scale it to support PVCs */
	USHORT			LinkUpCount;
#endif // linux
#endif // GSI_LAN

#if defined (WAN_MINIPORT) || defined(GSI_WAN)
#ifdef _NTDDK_
   NDIS_WAN_INFO           WanInfo;
   // A unique instance number to be used to generate a unique
   // WAN permanent address (not really permanent, but it's unique).
   CHAR                    InstanceNumber;
   BOOLEAN                 NeedReset;
   CHAR                    ProviderInfo[48]; // This size is the max allowed by RAS
   USHORT                  ProviderInfoSize; // Size in bytes of both strings.
   PLINEPTR_LINK           WanLink;

   //     Spinlock to protect fields in this structure..
   NDIS_SPIN_LOCK          Lock;
   ULONG                   DeviceIdBase;
   CHAR DeviceName         [REG_PARAM_LENGTH];

   RFC2516_HANDLE          Rfc2516Handle;
   ATM_UNI_HANDLE          Rfc2516VcHandle;
#endif // _NTDDK_

    PUCHAR          Encapsulation;
    USHORT          EncapsulationSize;
    RFC2364_MODE    Rfc2364Mode;
    UCHAR           Rfc2364Vpi;
    USHORT          Rfc2364Vci;
    ATM_UNI_HANDLE  Rfc2364VcHandle;
    PATM_USB_INSTANCE_CALL  Call;
    UINT                    Rfc2364Connections;
    NDIS_HANDLE             NdisPktPoolHandle;
    NDIS_HANDLE             NdisBuffPoolHandle;
   PNDIS_PACKET            NdisPkt;
   PVOID            NdisWanPkt;
   ULONG                   transmitFrameCounter;
#ifdef _NTDDK_   
   ULONG                   PPPDisconnectCounter;
   ULONG                   PPPDisconnectTimeout;
#endif // _NTDDK_
   UCHAR                   Vpi;
   USHORT                  Vci;
   USHORT                  MaxFrameSize;

#ifdef MESSAGE_HANDLING
    USHORT              MsgMode;
#endif // MESSAGE_HANDLING

#ifdef ANNEXB
    UCHAR               TIVpi;
	USHORT              TIVci;

	UCHAR               ADIVpi;
	USHORT              ADIVci;

	UCHAR               DefaultVpi;
	USHORT              DefaultVci;

    UCHAR               Ti13FwVer1;
    UCHAR               Ti13FwVer2;
    UCHAR               Ti13FwVer3;
    UCHAR               Ti13FwIssue;

    UCHAR               Ti20FwVer1;
    UCHAR               Ti20FwVer2;
    UCHAR               Ti20FwVer3;
    UCHAR               Ti20FwIssue;
#endif // ANNEXB

#ifdef _NTDDK_
   USHORT               Win2kSurpriseRemoval;
#endif // _NTDKK_

#if defined(linux)    
    gsi_wan_t       wan;
    ATM_USB_INSTANCE_CALL  LinkCall;
#endif // linux
#endif // GSI_WAN

#if defined(linux)
    gsi_version_t   version;
    gsi_config_t    config; 
    NDIS_MINIPORT_TIMER IntUrbTimer;
    struct _adapter_t   *next;
    wait_queue_head_t   StatusCheckQ;
    USHORT             flags;
    UCHAR              macAddr[ETH_LENGTH_OF_ADDRESS];
    struct usb_device   *usbDev;
#endif // linux
} ATM_USB_INSTANCE;

#if defined(linux)
typedef  ATM_USB_INSTANCE adapter_t, _adapter_t;
#if (defined(GSI_LAN) && defined(GSI_WAN))
void GpLinkStatusChange(PATM_USB_INSTANCE instance,
    ATM_PHY_LINK_STATE linkState, ULONG upstreamRate, ULONG downstreamRate);
#endif // (GSI_LAN && GSI_WAN)
#ifdef GSI_LAN
int GpLanAlloc(adapter_t *pAdapter,int ipvcindex);
int GpLanRegister(adapter_t *pAdapter, int ilanindex);
void GpLanUnregister(adapter_t *pAdapter, int ilanindex);
void GpLanFree(adapter_t *pAdapter, int ipvcindex);
void GpLanRx(adapter_t *pAdapter, void *pHeader, u32 headerSize,
    void *pData, u32 dataSize, gsi_lan_t *plan);
void GpLanTxComplete(adapter_t *pAdapter, int status, gsi_lan_t *pLan);
void GpLanMediaChange(adapter_t *pAdapter);
#endif // GSI_LAN

#ifdef GSI_WAN
int GpWanAlloc(adapter_t *pAdapter);
int GpWanRegister(adapter_t *pAdapter);
void GpWanUnregister(adapter_t *pAdapter);
void GpWanFree(adapter_t *pAdapter);
void GpWanRx(adapter_t *pAdapter, channel_t *pChan,
    void *pPacket, u32 packetSize);
void GpWanTxComplete(adapter_t *pAdapter, channel_t *pChan, int status);
void GpWanMediaChange(adapter_t *pAdapter);
#endif // GSI_WAN

#endif // linux

//------------------------------------------------------------------------------
//
// ATM_USB_MINIPORT_CALLBACK
//        Defines the prototype for the Miniport context callback function.
//        The miniport context callback function is used in conjunction with
//        AtmUsbSwitchToMiniport
//
//------------------------------------------------------------------------------
typedef void (*ATM_USB_MINIPORT_CALLBACK)
                 (PATM_USB_INSTANCE,PVOID,ATM_UNI_HANDLE);

//------------------------------------------------------------------------------
//
// ATM_USB_CALLBACK_CONTEXT
//        Context passed to NdisIMQueueMiniportCallback
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI_CALLBACK_CONTEXT {
    LIST_NODE                    Link;
    ATM_USB_MINIPORT_CALLBACK    MiniportCallback;
    PVOID                        MiniportContext;
    ATM_UNI_HANDLE              AtmUniVc;

} ATM_USB_CALLBACK_CONTEXT, *PATM_USB_CALLBACK_CONTEXT;

#ifdef _NTDDK_
//------------------------------------------------------------------------------
//
// Function prototypes
//
//------------------------------------------------------------------------------
NDIS_STATUS 
AtmUsbInitialize(
    OUT PNDIS_STATUS     OpenErrorStatus,
    OUT PUINT             SelectedMediumIndex,
    IN PNDIS_MEDIUM     MediumArray,
    IN UINT             MediumArraySize,
    IN NDIS_HANDLE         MiniportAdapterHandle,
    IN NDIS_HANDLE         WrapperConfigurationContext
);

void
AtmUsbShutdown( PATM_USB_INSTANCE Instance);

void
AtmUsbHalt( PATM_USB_INSTANCE Instance);

BOOLEAN
AtmUsbCheckForHang( PATM_USB_INSTANCE    Instance );

NDIS_STATUS
AtmUsbReset (
    PBOOLEAN         AddressingReset,
    PATM_USB_INSTANCE Instance
);

void AtmUsbShutdownReceive ( PATM_USB_INSTANCE Instance);

#endif // _NTDDK_

BOOLEAN
AtmUsbScheduleReceiveCallback(
    PATM_USB_INSTANCE            Instance,
    ATM_USB_MINIPORT_CALLBACK    MiniportCallback,
    PVOID                        MiniportContext,
    ATM_UNI_HANDLE              AtmUniVcHandle
    );

void
AtmUsbPendingReceiveTimerCallback(
    PVOID                SystemSpecific1,
    PATM_USB_INSTANCE    Instance,       
    PVOID                SystemSpecific2,
    PVOID                SystemSpecific3
);

#ifdef _NTDDK_
void AtmUsbCreateDevice( PATM_USB_INSTANCE Instance, PDRIVER_OBJECT DriverObject, NDIS_HANDLE WrapperHandle);
void AtmUsbDeleteDevice( PATM_USB_INSTANCE Instance);
void AtmUsbUnHookDevice( PATM_USB_INSTANCE Instance);
void AtmUsbHookDevice( PATM_USB_INSTANCE Instance);

#endif  // _NTDDK_

BOOLEAN 
SafeCancelTimer(
    PNDIS_MINIPORT_TIMER Timer,
    PNDIS_EVENT Event,
    UINT Timeout
);

BOOLEAN 
SafeSetTimer(
    PATM_USB_INSTANCE Instance,
    PNDIS_EVENT Event,
    PNDIS_MINIPORT_TIMER Timer,
    UINT Delay
);

#endif // ATMUSB_H



