////////////////////////////////////////////////////////////////////////////////
//
// INTEL CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with Intel Corporation and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998 Intel Corporation. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.3 $
//     $Modtime: 8/06/01 4:02p $
//     $Author: kattre $
//
// ABSTRACT:
//     Contains debug related macros and definitions
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _DEBUG_H
#define _DEBUG_H

//------------------------------------------------------------------------------
//
// DBG_PRINT_LEVEL
//  This enumerated type defines the possible values for the debug print
//  level.
//
//------------------------------------------------------------------------------
typedef enum {
    DBG_PRINT_LEVEL_NONE  = 0x00000000,
    DBG_PRINT_LEVEL_ERR   = 0x00000001,
    DBG_PRINT_LEVEL_TRACE  = 0x00000002,
    DBG_PRINT_LEVEL_WARN = 0x00000004,
    DBG_PRINT_LEVEL_ALL   = 0xFFFFFFFF
} DBG_PRINT_LEVEL;

//------------------------------------------------------------------------------
//
// DBG_PRINT_MODULE
//  This enumerated type defines the possible values for the debug print
//  module.
//
//------------------------------------------------------------------------------
typedef enum {
    DBG_PRINT_MODULE_NONE         = 0x00000000,
    DBG_PRINT_MODULE_NDIS_ALL     = 0x00000001,
    DBG_PRINT_MODULE_ATMUNI_RX    = 0x00000010,
    DBG_PRINT_MODULE_ATMUNI_TX    = 0x00000020,
    DBG_PRINT_MODULE_ATMUNI_ALL   = 0x00000030,
	DBG_PRINT_MODULE_MSG_HANDLING = 0x00000040,
    DBG_PRINT_MODULE_AUTO_CONFIG  = 0x00000080,
    DBG_PRINT_MODULE_USB_RX       = 0x00000100,
    DBG_PRINT_MODULE_USB_TX       = 0x00000200,
    DBG_PRINT_MODULE_USB_ALL      = 0x00000300,
    DBG_PRINT_MODULE_RFC1483_TX   = 0x00001000,
    DBG_PRINT_MODULE_RFC1483_RX   = 0x00002000,
    DBG_PRINT_MODULE_RFC1483_ALL  = 0x00003000,
    DBG_PRINT_MODULE_ATMMP_TX     = 0x00010000,
    DBG_PRINT_MODULE_ATMMP_RX     = 0x00020000,
    DBG_PRINT_MODULE_ATMMP_ALL    = 0x00030000,
    DBG_PRINT_MODULE_GSICPL       = 0x00100000,
    DBG_PRINT_MODULE_RFC2364_TX   = 0x01000000,
    DBG_PRINT_MODULE_RFC2364_RX   = 0x02000000,
    DBG_PRINT_MODULE_RFC2364_ALL  = 0x03000000,
    DBG_PRINT_MODULE_RFC1577_ALL  = 0x30000000,
#ifdef PPPOE_MINIPORT
    DBG_PRINT_MODULE_RFC2516_ALL  = 0x10000000,
#endif /* PPPOE_MINIPORT */
    DBG_PRINT_MODULE_ALL          = 0xFFFFFFFF
} DBG_PRINT_MODULE;

//
// Debug function prototypes
//
DBG_PRINT_LEVEL DbgGetDbgPrintLevel( void);
void DbgSetDbgPrintLevel( DBG_PRINT_LEVEL Level);
DBG_PRINT_MODULE DbgGetDbgPrintModule( void);
void DbgSetDbgPrintModule( DBG_PRINT_MODULE Module);

#ifdef GTI_TRACE
void PrintTrace(void);
#endif


//
// Debug related macros
//
#if DBG

#define DBGPRINT(_Module, _Level, _Msg)                         \
{                                                               \
    if ( (DbgGetDbgPrintLevel() & (_Level)) &&                  \
         (DbgGetDbgPrintModule() & (_Module)) )                 \
    {                                                           \
        if ((_Level) & DBG_PRINT_LEVEL_ERR) \
            {printk("ERR (%6.6s:%4.4d) ",__FILE__,__LINE__); printk _Msg; DBG_BREAKPOINT;}  \
        else if ((_Level) & DBG_PRINT_LEVEL_WARN) \
            {printk("WRN (%6.6s:%4.4d) ",__FILE__,__LINE__); printk _Msg; }  \
        else if ((_Level) & DBG_PRINT_LEVEL_TRACE) \
            {printk("(%6.6s:%4.4d) ",__FILE__,__LINE__); printk _Msg;}  \
    }                                                           \
}

#define DBGBREAK()  DbgBreakPoint()

#else

#define DBGPRINT( _Module, _Level, _Msg)
#define DBGBREAK()

#endif // DBG

#include "GpDebug.h"

#ifndef ASSERT
# define ASSERT(C)  if (!(C)) { \
                        DBG_PRINT(gDbgInfo,("ASSERT(%s) -- %s#%d\n", \
                                  #C, __FILE__, __LINE__)); \
                        DBG_TRAP;}
#endif

#endif // _DEBUG_H
