////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  2001-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.1 $
//     $Modtime: 5/15/02 12:18p $
//     $Author: kattre $
//
// ABSTRACT:
//     This file provides the interface to the GlobespanVirata's EOC module.
//
////////////////////////////////////////////////////////////////////////////////
#ifndef DSL_H
#define DSL_H

#include "gti.h"
#include "gti_eoca.h"
  
typedef void * USB_PHY_HANDLE;
typedef void * DSP_HANDLE;

#define EOC_MESSAGE_SIZE                   2
#define EOC_CELL_MSGS                                               (32/EOC_MESSAGE_SIZE)

#define EMPTY_PATTERN                      0x0C0C

#define DYING_GASP_NUMBER                  6 
#define EOC_DYING_GASP_DELAY               7       // 7 ms
#ifdef _NTDDK_
#define DSP_STATUS_TIMER                                            1500    // 1500 ms
#elif defined(linux)
#define DSP_STATUS_TIMER                   ((1500 * HZ)/1000)    // 1500 ms
#endif

// Defines used by retrain due to excessive CRC errors
#define DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD                    40
#define DEFAULT_DOWNSTREAM_RATE_TO_MAX_CRC_ERRORS_PER_MIN_DIVIDER   32
#define SECS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_SEC              11
#define MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN              5        

#ifdef _NTDDK_
#define SECS_PER_DSP_STATUS_TICK                                    (DSP_STATUS_TIMER / 1000)
#define DSP_STATUS_TICKS_PER_SEC                                    (1000 / DSP_STATUS_TIMER) 
#define DSP_STATUS_TICKS_PER_MIN                                    ((60 * 1000) / DSP_STATUS_TIMER)
#define CRC_ERRORS_PER_DSP_STATUS_TICK_THRESHOLD                    ((USHORT) ((DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD * DSP_STATUS_TIMER) / 1000))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_TICK            ((USHORT) ((SECS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_SEC * 1000) / DSP_STATUS_TIMER))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN             ((USHORT) ((MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN * 60 * 1000) / DSP_STATUS_TIMER))

#define ConvertPerSecondCountToPerTickCount(Count)                  (USHORT) ((Count * DSP_STATUS_TIMER) / 1000)
#elif defined(linux)
#define DSP_STATUS_TIMER_MILLISECS									(DSP_STATUS_TIMER * 1000 / HZ)
#define SECS_PER_DSP_STATUS_TICK                                    (DSP_STATUS_TIMER_MILLISECS / 1000)
#define DSP_STATUS_TICKS_PER_SEC                                    (1000 / DSP_STATUS_TIMER_MILLISECS) 
#define DSP_STATUS_TICKS_PER_MIN                                    ((60 * 1000) / DSP_STATUS_TIMER_MILLISECS)
#define CRC_ERRORS_PER_DSP_STATUS_TICK_THRESHOLD                    ((USHORT) ((DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD * DSP_STATUS_TIMER_MILLISECS) / 1000))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_TICK            ((USHORT) ((SECS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_SEC * 1000) / DSP_STATUS_TIMER_MILLISECS))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN             ((USHORT) ((MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN * 60 * 1000) / DSP_STATUS_TIMER_MILLISECS))

#define ConvertPerSecondCountToPerTickCount(Count)                  (USHORT) ((Count * DSP_STATUS_TIMER_MILLISECS) / 1000)
#endif //linux

#define QWORK_ITEM_LEN                     20
#define EOC_BUFFER_SIZE                    QWORK_ITEM_LEN

#define MAX_CONTROL_PIPE_SIZE              64
#ifdef _NTDDK_
#define MAX_INTERRUPT_PIPE_SIZE            64
#elif defined(linux)
#define MAX_INTERRUPT_PIPE_SIZE			   32
#endif // linux
//
// Defines for EOC ATU-R Configuration Register
//
#define ATU_R_CFG_BF_AS0                                            0
#define ATU_R_CFG_BI_AS0                                            1
#define ATU_R_CFG_BF_AS1                                            2
#define ATU_R_CFG_BI_AS1                                            3
#define ATU_R_CFG_BF_AS2                                            4
#define ATU_R_CFG_BI_AS2                                            5
#define ATU_R_CFG_BF_AS3                                            6
#define ATU_R_CFG_BI_AS3                                            7

#define ATU_R_CFG_BF_LS0_DWN                                        8
#define ATU_R_CFG_BI_LS0_DWN                                        9
#define ATU_R_CFG_BF_LS1_DWN                                        10
#define ATU_R_CFG_BI_LS1_DWN                                        11
#define ATU_R_CFG_BF_LS2_DWN                                        12
#define ATU_R_CFG_BI_LS2_DWN                                        13

#define ATU_R_CFG_BF_LS0_UP                                         15
#define ATU_R_CFG_BI_LS0_UP                                         16
#define ATU_R_CFG_BF_LS1_UP                                         17
#define ATU_R_CFG_BI_LS1_UP                                         18
#define ATU_R_CFG_BF_LS2_UP                                         19
#define ATU_R_CFG_BI_LS2_UP                                         20

#define ATU_R_CFG_RSF_DWN                                           22
#define ATU_R_CFG_RSI_DWN                                           23
#define ATU_R_CFG_S_DWN                                             24
#define ATU_R_CFG_D_DWN                                             25

#define ATU_R_CFG_RSF_UP                                            26
#define ATU_R_CFG_RSI_UP                                            27
#define ATU_R_CFG_S_UP                                              28
#define ATU_R_CFG_D_UP                                              29

#define DEFAULT_EOC_VENDOR_ID_HIGH                                  0x00B54753
#define DEFAULT_EOC_VENDOR_ID_LOW                                   0x504e0039

/* structures for EOC messages  */ 

typedef enum {
    DSL_PM_STATE_IDLE,
    DSL_PM_STATE_PENDING,
    DSL_PM_STATE_GRANTED,
    DSL_PM_STATE_REJECTED,
    DSL_PM_STATE_NOTCONNECTED,
    DSL_PM_STATE_TIMEOUT,
    DSL_PM_STATE_ERROR
} DSL_PM_STATE;

typedef enum
{
   DSP_NO_ACTION,
   DSP_FAST_RETRAIN,
   DSP_PROFILE_MNGT,
   DSP_EOC_MSG,
   DSP_SNR_MARGIN_PER_BIN,
   DSP_LCL_SNR_MARGIN,
   DSP_DOWN_BIT_LOADING_PER_BIN,
   DSP_UP_BIT_LOADING_PER_BIN,
   DSP_INTERRUPT,
   DSP_BIT_SWAPPING,
   DSP_WRITE_EEPROM_ACTION,
   DSP_RESET,
#ifdef WINHIP
   DSP_PROCESS_WHIP_MSG,
#endif
   DSP_POKE_EEPROM_ACTION
}DSP_CONN_ACTION;

typedef struct
{
   LIST_NODE        listEntry;
   DSP_CONN_ACTION    action;        
} DSP_PHY_QWORK_ITEM, *PDSP_PHY_QWORK_ITEM;

typedef struct _EOC_STATUS
{
   GTI_DWORD    RxEocOk;
   GTI_DWORD    RxEocError;
} EOC_STATUS , *PEOC_STATUS;

typedef struct _EOC_DSP_BUFFER
{
   GTI_WORD  rxBuff[GTI_EOC_MAX_NUMOF_MSG];
   GTI_WORD  rxFlag; 
   GTI_WORD  txFlag;
}EOC_DSP_BUFFER ,  *PEOC_DSP_BUFFER;

typedef struct _DSL_EX
{  
   EOC_DSP_BUFFER  EocDspBuffer[EOC_BUFFER_SIZE]; 
   GTI_BYTE        CellWriteInd;
   GTI_BYTE        CellReadInd;
   GTI_BYTE        CellPendingCnt;
   
   GTIEOC_CMD      EocCmd;
   EOC_STATUS      status;                    
   USB_PHY_HANDLE  UsbPhyHandle;
   DSP_HANDLE      DspHandle;
} DSL_EX , *PDSL_EX;

/* This Type is used communicate with Eoc Debbuger */
typedef struct _EOC_MESSAGES_COMMON
{
   GTI_WORD    rxBuff[EOC_BUFFER_SIZE][GTI_EOC_MAX_NUMOF_MSG];
   GTI_WORD    txBuff[EOC_BUFFER_SIZE][GTI_EOC_MAX_NUMOF_MSG];
   
   GTI_BYTE    CellWriteInd;
   GTI_BYTE    CellReadInd;
   GTI_BYTE    CellPendingCnt;
} EOC_MESSAGES_COMMON , *PEOC_MESSAGES_COMMON;

#ifdef MESSAGE_HANDLING
void TriggerEocSend( USB_PHY_HANDLE UsbPhy);
#endif // MESSAGE_HANDLING

/* Routins for EOC messages  */
void DslEocParse( PDSL_EX pDslEx , GTI_BYTE *dataPtr);

void DslEocBind(PDSL_EX pDslEx, USB_PHY_HANDLE UsbPhy);

void DslEocInitialize( PDSL_EX pDslEx , USB_PHY_HANDLE UsbPhy, USHORT *actualCfg);
void DslSetEocAtuRCfg( USHORT *actualCfg);

BOOLEAN DslEocSendDyingGasp(PDSL_EX pDslEx);

void DslShutdown( USB_PHY_HANDLE UsbPhy);

void DslEocProc(PDSL_EX pDslEx);

void DslStatusParse(PDSL_EX pDslEx , GTI_BYTE *dataPtr);

GTI_WORD isEocValid(GTI_WORD *pBuffer, int nSize);

#endif /* DSL_H */
