/*
	@(#)gti_eoc.h	1.33 
 */

/************************************************************************
 * FILE NAME   : gti_eoc.h                                              *
 * DESCRIPTION : *
 *                                                                      *
 * COMMENT     : *
 *                                                                      *
 * (C) Copyright 1999 by GlobeSpan Semiconductor Inc                    *
 *                                                                      *
 ************************************************************************/

#ifndef  GTI_EOC_H
#define  GTI_EOC_H

/************************************************************************
*                         EXTERNAL REFERENCES
************************************************************************/

/************************************************************************
*                             CONSTANTS
************************************************************************/

/* ************ EOC ERROR CODE ************ */
#define  GTI_EOC_COMPLETE            0    /* Command is Completed */
#define  GTI_EOC_ERROR               1    /* Error on the Host Command */
#define  GTI_EOC_BUSY                2    /* Command is Busy (in Progress) */
#define  GTI_EOC_INVALID_CMD         3    /* Invalid Command from Host */
#define  GTI_EOC_INVALID_LEN         4    /* Invalid Length from Host */
#define  GTI_EOC_UTC                 5    /* "unable to Comply" */
#define  GTI_EOC_LPR                 6    /* Dying gasp, LPR detected in ATU-R */

#define GTI_EOC_STATUS_REQPDN        7    /* Autonomous message for Power Mannagement is sent*/
#define GTI_EOC_STATUS_GNTPDN        8    /* Grant Power management from ATU_C for power management*/
#define GTI_EOC_STATUS_REJPDN        9    /* Reject Power management from ATU_C for power management*/
#define GTI_EOC_STATUS_WAIT_PDN      10   /* Waiting for Power management Grant or reject messages from ATU-R*/

/* ************ EOC COMMANDS/OPCODES ************ */
#define  GTI_EOC_CMD_HOLD            0x01  /* d<->u Hold State */
#define  GTI_EOC_CMD_RTN             0xF0  /* d<->u Return To Normal */
#define  GTI_EOC_CMD_SLFTEST         0x02  /* d<->u Self-Test */
#define  GTI_EOC_CMD_UTC             0x04  /* d<->u Unable To Comply */
#define  GTI_EOC_CMD_REQCOR          0x07  /* d<->u Request Corrupt CRC */
#define  GTI_EOC_CMD_REQEND          0x08  /* d<->u Req End of Corrupt CRC */
#define  GTI_EOC_CMD_NOTCOR          0x0B  /* d<->u Notify Corrupt CRC */
#define  GTI_EOC_CMD_NOTEND          0x0D  /* d<->u Notify End of Corrupt CRC */
#define  GTI_EOC_CMD_EOD             0x0E  /* d<->u End of Data */
#define  GTI_EOC_CMD_NEXT            0x10  /* d<->u Next Byte */
#define  GTI_EOC_CMD_REQTPU          0x13  /* d<->u Req Test Parameter Update */
/* Write Data Register 0-F */
#define  GTI_EOC_CMD_WRITEREG0       0x20  /* d<->u Write VendorID */
#define  GTI_EOC_CMD_WRITEREG1       0x23  /* d<->u Write Revision Number */
#define  GTI_EOC_CMD_WRITEREG2       0x25  /* d<->u Write Serial Number */
#define  GTI_EOC_CMD_WRITEREG3       0x26  /* d<->u Write Self-Test Result */
#define  GTI_EOC_CMD_WRITEREG4       0x29  /* d<->u Write Vendor Discretion */
#define  GTI_EOC_CMD_WRITEREG5       0x2A  /* d<->u Write Vendor Discretion */
#define  GTI_EOC_CMD_WRITEREG6       0x2C  /* d<->u Write Line Attenuation */
#define  GTI_EOC_CMD_WRITEREG7       0x2F  /* d<->u Write SNR Margin */
#define  GTI_EOC_CMD_WRITEREG8       0x31  /* d<->u Write ATU-R Configuration */
#define  GTI_EOC_CMD_WRITEREG9       0x32  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGA       0x34  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGB       0x37  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGC       0x38  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGD       0x3B  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGE       0x3D  /* d<->u Reserved */
#define  GTI_EOC_CMD_WRITEREGF       0x3E  /* d<->u Reserved */
/* Read Data Register 0-F */
#define  GTI_EOC_CMD_READREG0        0x40  /* d<->u Read VendorID */
#define  GTI_EOC_CMD_READREG1        0x43  /* d<->u Read Revision Number */
#define  GTI_EOC_CMD_READREG2        0x45  /* d<->u Read Serial Number */
#define  GTI_EOC_CMD_READREG3        0x46  /* d<->u Read Self-Test Result */
#define  GTI_EOC_CMD_READREG4        0x49  /* d<->u Read Vendor Discretionary */
#define  GTI_EOC_CMD_READREG5        0x4A  /* d<->u Read Vendor Discretionary */
#define  GTI_EOC_CMD_READREG6        0x4C  /* d<->u Read Line Attenuation */
#define  GTI_EOC_CMD_READREG7        0x4F  /* d<->u Read SNR Margin */
#define  GTI_EOC_CMD_READREG8        0x51  /* d<->u Read ATU-R Configuration */
#define  GTI_EOC_CMD_READREG9        0x52  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGA        0x54  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGB        0x57  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGC        0x58  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGD        0x5B  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGE        0x5D  /* d<->u Reserved */
#define  GTI_EOC_CMD_READREGF        0x5E  /* d<->u Reserved */
/* Vendor Proprietary Protocols */
#define  GTI_EOC_CMD_VPP0            0x19  /* d<->u Vendor Protrietary Prtcl */
#define  GTI_EOC_CMD_VPP1            0x1A  /* d<->u Vendor Protrietary Prtcl */
#define  GTI_EOC_CMD_VPP2            0x1C  /* d<->u Vendor Protrietary Prtcl */
#define  GTI_EOC_CMD_VPP3            0x1F  /* d<->u Vendor Protrietary Prtcl */
#define  GTI_EOC_CMD_DGASP           0xE7  /* Dying Gasp */

#define  GTI_EOC_CMD_REQPDN          0x15  /* Request Power management*/
#define  GTI_EOC_CMD_GNTPDN          0x16  /*grant power management*/
#define  GTI_EOC_CMD_REJPDN          0x83  /*reject power management*/

#define  GTI_EOC_CMD_RESET           0xFF  /* GlobeSpan Reset EOC Module */

/* ************ VALID EOC LENGTHs ************ */
#define  GTI_EOC_VENDOR_LEN          8
#ifdef GTI_DSPVIKING
#define  GTI_EOC_REV_NUM_LEN         8
#else
#define  GTI_EOC_REV_NUM_LEN         32
#endif
#define  GTI_EOC_SERIAL_LEN          32
#define  GTI_EOC_SELFTEST_LEN        32
#define  GTI_EOC_VEN_DISC1_LEN       32
#define  GTI_EOC_VEN_DISC2_LEN       32
#define  GTI_EOC_ATTENU_LEN          1
#define  GTI_EOC_SNRMARGN_LEN        1
#define  GTI_EOC_CONFIG_LEN          30

#define  GTI_EOC_MAX_DATA_LEN        32
#ifdef GTI_EOC_PCU
    #ifndef GTI_DSPOCTANE
        #define  GTI_EOC_MAX_AUTONOMOUS_LEN_TX  4
        #define  GTI_EOC_MAX_AUTONOMOUS_LEN_RX  6
    #else
        #define  GTI_EOC_MAX_AUTONOMOUS_LEN_TX  10
        #define  GTI_EOC_MAX_AUTONOMOUS_LEN_RX  10
    #endif
#else
    #define  GTI_EOC_MAX_AUTONOMOUS_LEN  6
#endif
#define  GTI_EOC_MAX_AUTO_LEN_X      26
#define  GTI_EOC_REGA_LEN            1
#define  GTI_EOC_VEN_PROP_LEN        8
#define  GTI_EOC_VEN_PROP3_LEN       2 
#define  GTI_MAX_AOC_LEN             32
#define  GTI_MAX_AOC_TIMES           4

/*************** Extended EOC Constants *****************/
#define GTI_SIZE_OF_LINE_MONITORING_RESP    768
#define GTI_EXT_EOC_PROTOCOL_VERSION        1
#define GTI_NUMBER_OF_EXT_EOC_PAGES         24
#define GTI_LINE_ATTEN_PER_BIN_OFFSET       0
#define GTI_LINE_ATTEN_PER_BIN_SIZE         256
#define GTI_SNR_MARGIN_PER_BIN_OFFSET       256
#define GTI_SNR_MARGIN_PER_BIN_SIZE         256
#define GTI_NOISE_PER_BIN_OFFSET            512
#define GTI_NOISE_PER_BIN_SIZE              256

#ifdef GTI_PAGE_I_BUILT

/************************************************************************
			  OVERHEAD COMMANDS
************************************************************************/
#define GTI_OH_CMD_OLR			       0x01
#define GTI_OH_CMD_EOC                 0x41
#define GTI_OH_CMD_TIME                 0x42
#define GTI_OH_CMD_INVENTORY            0x43
#define GTI_OH_CMD_CPR                  0x04
#define GTI_OH_CMD_MCR                  0x05
#define GTI_OH_CMD_PM                   0x07
#define GTI_OH_CMD_CEOC                 0x08
#define GTI_OH_CMD_NSF                  0x3F
#define GTI_OH_CMD_TPR              0x81
#define GTI_OH_CMD_NSF_LP               0xBF
#define GTI_OH_CMD_GSPN                 0xFF
#define GTI_OH_CMD_OLR_TYPE1            0x01
#define GTI_OH_CMD_OLR_TYPE2            0x02
#define GTI_OH_CMD_OLR_TYPE3            0x03
#define GTI_OH_CMD_EOC_PST              0x01
#define GTI_OH_CMD_EOC_UTP              0x02
#define GTI_OH_CMD_EOC_STX_CCRC         0x03
#define GTI_OH_CMD_EOC_ETX_CCRC         0x04
#define GTI_OH_CMD_EOC_SRX_CCRC         0x05
#define GTI_OH_CMD_EOC_ERX_CCRC         0x06
#define GTI_OH_CMD_TIME_SET         	0x01
#define GTI_OH_CMD_TIME_READ            0x02
#define GTI_OH_CMD_INV_ID             	0x01
#define GTI_OH_CMD_INV_AUX_ID           0x02
#define GTI_OH_CMD_INV_STR              0x03
#define GTI_OH_CMD_INV_PMD              0x04
#define GTI_OH_CMD_INV_PMS_TC           0x05
#define GTI_OH_CMD_INV_TPS_TC           0x06
#define GTI_OH_CMD_CPR_PMD              0x01
#define GTI_OH_CMD_CPR_PMS_TC           0x02
#define GTI_OH_CMD_CPR_TPS_TC           0x03
#define GTI_OH_CMD_MCR_READ             0x01
#define GTI_OH_CMD_PM_TYPE1             0x01
#define GTI_OH_CMD_PM_TYPE2             0x02
#define GTI_OH_CMD_PM_TYPE3             0x03
#define GTI_OH_CMD_CEOC_SEND            0x01
#define GTI_OH_CMD_NSF_SEND             0x01
#define GTI_OH_CMD_TPR_SINGLE           0x01
#define GTI_OH_CMD_TPR_MULTIPLE         0x02
#define GTI_OH_CMD_TPR_NEXT             0x03
#define GTI_OH_CMD_TPR_MULTIPLE_PLUS    0x04
#define GTI_OH_CMD_TPR_SGL_CTF          0x01
#define GTI_OH_CMD_TPR_SGL_QLN          0x03
#define GTI_OH_CMD_TPR_SGL_SNR          0x04
#define GTI_OH_CMD_TPR_SGL_MP           0x05
#define GTI_OH_CMD_TPR_SGL_LATN         0x21
#define GTI_OH_CMD_TPR_SGL_SATN         0x22
#define GTI_OH_CMD_TPR_SGL_SNRM         0x23
#define GTI_OH_CMD_TPR_SGL_ATTNDR       0x24
#define GTI_OH_CMD_TPR_SGL_NACTATP      0x25
#define GTI_OH_CMD_TPR_SGL_FACTATP      0x26
#define GTI_OH_CMD_GSPN_RESET      	    0xFF
#define GTI_OH_CMD_GSPN_LPR           	0xFE
#define GTI_OH_CMD_WRITE_ID            0x00
#define GTI_OH_CMD_WRITE_AUX_ID        0x01
#define GTI_OH_CMD_WRITE_STR                0x02
#define GTI_OH_CMD_WRITE_LATN               0x03
#define GTI_OH_CMD_WRITE_SNRM               0x04
#define GTI_OH_CMD_WRITE_MULTI_TP           0x05
#define GTI_OH_CMD_OLR_DEFER1               0x81
#define GTI_OH_CMD_OLR_DEFER2               0x82
#define GTI_OH_CMD_OLR_DEFER3               0x83
#define GTI_OH_CMD_PM_REJECT                0x81
#define GTI_OH_CMD_PM_REJECT_L2             0x83
#define GTI_OH_CMD_PM_REJECT_TRIM           0x85
#define GTI_OH_CMD_CEOC_NACK                0x81
#define GTI_OH_CMD_NSF_NACK                 0x81
#define GTI_OH_CMD_TPR_NACK                 0x80
#define GTI_OH_CMD_NSF_LP_NACK              0x81
#endif

/************************************************************************
*                          DATA DECLARATIONS
************************************************************************/

/************************************************************************
*                             DATA TYPES
************************************************************************/
typedef struct
{
   GTI_BYTE    vendorId[ GTI_EOC_VENDOR_LEN ];
   GTI_BYTE    revisionNum[ GTI_EOC_REV_NUM_LEN ];
   GTI_BYTE    serialNum[ GTI_EOC_SERIAL_LEN ];
   GTI_BYTE    selfTest[ GTI_EOC_SELFTEST_LEN ];
   GTI_BYTE    vendorDisc1[ GTI_EOC_VEN_DISC1_LEN ];
   GTI_BYTE    vendorDisc2[ GTI_EOC_VEN_DISC2_LEN ];
   GTI_BYTE    lineAtten[ GTI_EOC_ATTENU_LEN ];
   GTI_BYTE    snrMargin[ GTI_EOC_SNRMARGN_LEN ];
   GTI_BYTE    aturConfig[ GTI_EOC_CONFIG_LEN ];
   GTI_BYTE    registryA[GTI_EOC_REGA_LEN];
   GTI_BYTE    vendorProp[ GTI_EOC_VEN_PROP_LEN ];
} GTIEOC_DATA_REG;
#define GTI_OH_MAX_CMD_LEN 3
#define GTI_OH_MAX_NUM_REG 20
typedef struct
{
   GTI_BYTE ohCmd[GTI_OH_MAX_CMD_LEN];
   GTI_BYTE ohLen;
   GTI_WORD ohLoc;
} GTIOH_REG_INFO;

#ifdef GTI_PAGE_I_BUILT

#define GTI_OH_TOTAL_WRITE_REG 6
#define GTI_OH_TOTAL_NACK_RESP 3
#define GTI_OH_TOTAL_NACK_REG 7
#define GTI_OH_MAX_DATA_LEN 64
#define GTI_OH_TOTAL_NUM_REG 11
#define GTI_OH_TIME_DATA_LEN 8
#define GTI_OH_MULTIPLE_MAX_SC 255
#define GTI_OH_MULTIPLE_PLUS_MAX_SC 511    

typedef struct
{
    GTI_BYTE ohWriteCmd;
    GTI_BYTE    ohCmd[GTI_OH_MAX_CMD_LEN];
} GTIOH_WRITE_REG;

typedef struct
{
   GTI_BYTE ohCmd;
   GTI_BYTE totalNack;
   GTI_BYTE respNack[GTI_OH_TOTAL_NACK_RESP];
} GTIOH_RESP_NACK;

#endif

typedef struct
{
   GTI_WORD Status;

   GTI_BYTE AutoLen; 
   GTI_BYTE AutoData[GTI_EOC_MAX_AUTO_LEN_X];

   GTI_BYTE EocCmd;
   GTI_BYTE EocLen;

   GTI_BYTE EocData[64];
   GTI_BYTE EocOHCmd[3]; /* placed here to make it compitable with earlier dmt release */ 
} TXEOC, RXEOC;

/* This structure is the interface between the host and EOC module */
typedef struct
{
   TXEOC tx;   /* TX - Input from the host */
   RXEOC rx;   /* RX - Output to the host */
} GTIEOC_CMD;

#ifdef GTI_PAGE_I_BUILT
typedef struct _GTI_OH_STATUS
{
    GTI_BYTE lprFlag;
    GTI_BYTE txLock;
    GTI_BYTE busyFlag;
    GTI_BYTE rcvLen;
    GTI_BYTE rspFlag;
}   GTI_OH_STATUS;
#endif

/************************************************************************
*                        FUNCTION DECLARATIONS
************************************************************************/
extern void GTIEocProc(GTI_XCVR_INFO *GtiSysPtr, GTIEOC_CMD *eocPtr);

/*
* Macro
*/
#define DIVBYTWO(inVal) ((GTI_BYTE)(((inVal) % 2) + ((inVal) / 2)))

#ifdef GTI_PAGE_I_BUILT
/*
* Function Declaration
*/

void Gti_EocProcBis(GTI_XCVR_INFO *, GTIEOC_CMD *);
GTI_BYTE Gti_OhValidStatCmd(GTIEOC_CMD *, GTIEOC_INFO *, GTI_OH_STATUS *);
void Gti_OhReadStatus(GTI_XCVR_INFO *, GTIEOC_CMD *, GTI_OH_STATUS *);
void Gti_OhSendCommand(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *, GTI_OH_STATUS *);
void Gti_OhReadResp(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *, GTI_OH_STATUS *);
void Gti_OhReadRegIndex(GTI_XCVR_INFO *);
GTI_BYTE Gti_OhWriteReg(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *);
GTI_BYTE Gti_OhReset(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *);
GTI_BYTE Gti_OhRespNack(GTIEOC_INFO *,GTI_BYTE );
#endif

#ifdef GTI_EOC_PCU
/*
* These structures, constants, functions are only
* used in EOC processing in PCU
*/

typedef struct _GTI_EOC_STATUS
{
    GTI_BYTE adataFlag;
    GTI_BYTE lprFlag;
    GTI_BYTE utcFlag;
    GTI_BYTE commDone;
    GTI_BYTE curState;
    GTI_BYTE dataLen;
}   GTI_EOC_STATUS;

#define GTI_MAX_AUTO_WAIT   10

/*
* Function Declaration
*/

void Gti_EocProcPcu(GTI_XCVR_INFO *, GTIEOC_CMD *);
void Gti_EocPcuReadStatus(GTI_XCVR_INFO *, GTI_EOC_STATUS *);
void Gti_EocPcuAutoData(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *, GTI_EOC_STATUS *);
void Gti_EocPcuSendOpcode(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *);
void Gti_EocPcuSendAuto(GTI_XCVR_INFO *, GTIEOC_CMD *);
void Gti_EocPcuReadReg(GTI_XCVR_INFO *, GTIEOC_CMD *, GTI_EOC_STATUS *);
void Gti_EocPcuResetEoc(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *);

#ifndef GTI_DSPOCTANE
    GTI_BYTE Gti_ValidEocStatCmdLen(GTI_XCVR_INFO *, GTIEOC_CMD *, GTIEOC_INFO *);
#else
    GTI_BYTE Gti_ValidEocStatCmdLen(GTIEOC_CMD *, GTIEOC_INFO *);
#endif
/*
* Macro
*/
#else
    void Gti_EocProcHost(GTI_XCVR_INFO *, GTIEOC_CMD *);
#endif   /* GTI_EOC_PCU */
#ifdef GTI_XDP_EOC
    void Gti_EocProcXdp(GTI_XCVR_INFO *, GTIEOC_CMD *);
#endif   /* GTI_EOC_PCU */

#endif   /* GTI_EOC_H */








