/* 
  @(#)gti_eoca.h	1.9 
 */

/************************************************************************
 * FILE NAME   : gti_eoca.h                                              *
 * DESCRIPTION : *
 *                                                                      *
 * COMMENT     : *
 *                                                                      *
 * (C) Copyright 1999 by GlobeSpan Semiconductor Inc                    *
 *                                                                      *
 ************************************************************************/

#ifndef  GTI_EOCA_H
#define  GTI_EOCA_H

/************************************************************************
*                         EXTERNAL REFERENCES
************************************************************************/
#include "gti_eoc.h"

/************************************************************************
*                             CONSTANTS
************************************************************************/

#define GTI_EOC_NUMOF_SUPERFRAME     1  /* Number of superframe */
#define GTI_EOC_NUMOF_FRAMES         68 /* Number of frames per superframe */
#define GTI_EOC_MAX_NUMOF_FRAMES     (GTI_EOC_NUMOF_FRAMES-4)
#define GTI_EOC_MAX_NUMOF_MSG        (GTI_EOC_NUMOF_FRAMES/2)
#define GTI_EOC_MAX_DGASP_MSG        9  /* At least 4 messages to be sent */
#define GTI_EOC_EOC_CNT              3  /* Limit on EOC Counter */
#define GTI_EOC_ECHO_CNT             3  /* Limit on ECHO Counter */
#define GTI_EOC_MAGIC_NUM            3

/* ********************* PCU MEMORY MAP ********************* */
/* Following are ISRAM offsets */
#define  GTI_EOC_TXBUFF_OFFSET       0x80
#define  GTI_EOC_RXBUFF_OFFSET       0xA2
#define  GTI_EOC_TXBUFF_FLAG_OFFSET  0xC4
#define  GTI_EOC_RXBUFF_FLAG_OFFSET  0xC5

#ifdef GTI_PAGE_D_BUILT
	#define  GTI_AOC_TXBUFF_FLAG_OFFSET  0xB4
#endif

/* Following are ISRAM values written to the offsets */
#define  GTI_EOC_RXBUFF_EMPTY        0x00
#define  GTI_EOC_RXBUFF_FULL         0x8000

#define  GTI_EOC_TXBUFF_EMPTY        0x8000
#define  GTI_EOC_TXBUFF_FULL         0x00

#ifdef GTI_PAGE_D_BUILT
	#define  GTI_AOC_TXBUFF_EMPTY        0x0000
	#define  GTI_AOC_TXBUFF_FULL         0x8000
#endif

/* Following are PSRAM0W offsets */
#define  GTI_EOC_REQCOR_OFFSET       0x13
#define  GTI_EOC_NOTCOR_OFFSET       0x13
#define  GTI_EOC_RETX_LIMIT_OFFSET   0x55
#define  GTI_EOC_RETX_COUNT_OFFSET   0x66

/* Following are PSRAM0W values written to the offsets */
#define  GTI_EOC_REQCOR_ENABLE       0x1
#define  GTI_EOC_REQCOR_DISABLE      0x0

#define  GTI_EOC_NOTCOR_ENABLE       0x1
#define  GTI_EOC_NOTCOR_DISABLE      0x0

#define   GTI_EOC_RETX_SET_COUNT     0xFFFF

/* ********************* EOC MESSAGE FIELD ********************* */
#define  GTI_EOC_OVERHEAD_B0         0x03
#define  GTI_EOC_OVERHEAD_B1         0x01
#define  GTI_EOC_ATUC_ADDRESS        0x0C
#define  GTI_EOC_ATUR_ADDRESS        0x00
#define  GTI_EOC_OPCODE_FIELD        0x10
#define  GTI_EOC_DATA_FIELD          0x00
#define  GTI_EOC_EVEN_PARITY         0x00
#define  GTI_EOC_ODD_PARITY          0x20
#define  GTI_EOC_MSG_FIELD           0x40
#define  GTI_EOC_AUTONOMOUS_FIELD    0x00
#define  GTI_EOC_AUTONOMOUS_MSG_CO   (GTI_EOC_OVERHEAD_B0 | GTI_EOC_ATUR_ADDRESS)
#define  GTI_EOC_AUTONOMOUS_MSG_CP   (GTI_EOC_OVERHEAD_B0 | GTI_EOC_ATUC_ADDRESS)
#define  GTI_EOC_EOC_MSG             ( GTI_EOC_OVERHEAD_B0 | \
                                    GTI_EOC_ATUC_ADDRESS  | \
                                    GTI_EOC_OPCODE_FIELD  | \
                                    GTI_EOC_ODD_PARITY    | \
                                    GTI_EOC_MSG_FIELD )
#define  GTI_EOC_DGASP_MSG_B0       0xBF                                    
#define  GTI_EOC_DGASP_MSG_B1       0xE7

/* ********************* EOC MISC ********************* */
#define  GTI_EOC_DEFAULT_CRC         0xFF
#define  GTI_EOC_DEFAULT_IB          0xFF
#define  GTI_EOC_FRAME_34            17
#define  GTI_EOC_SYNCH_CONTROL       0x0C0C
#define  GTI_EOC_AUTONOMOUS_MASK     0x0040
#define  GTI_EOC_ECHO_MASK           0x0850
#define  GTI_EOC_ADDRESS_MASK        0x000C
#define  GTI_EOC_OVERHEAD_MASK       0x0101
#define  MAX_RMT_VENDOR_ID_STATUS_LEN GTI_RMT_VENDOR_ID_G_DMT_STATUS_LEN

/************************************************************************
*                             DATA TYPES
************************************************************************/
typedef struct
{
   GTI_BYTE    addr;       /* 3: ATU-C 0: ATU-R */
   GTI_BYTE    isOpcode;   /* 1: Opcode 0: Data */
   GTI_BYTE    parity;     /* 1: Odd 0: Even */
   GTI_BYTE    isMsg;      /* 1: Message 0: Autonomous */
   GTI_BYTE    info;       /* Information Field */
   GTI_WORD    msg;        /* 16-bit EOC Message */
} GTIEOC_FIELDS;

typedef struct
{
   GTI_BYTE regID;
   GTI_BYTE *regData;
   GTI_BYTE regLen;
} GTIEOC_REG_TABLE;

/************************************************************************
*                          DATA DECLARATIONS
************************************************************************/

/************************************************************************
*                              MACROS
************************************************************************/
#define  GTI_EOC_M_MAKE_MSG(a, o, p, i)    (GTI_WORD)  \
            ((GTI_EOC_OVERHEAD_B0|GTI_EOC_MSG_FIELD|a|o|p|((i&0x1) << 7)) | \
                        ((GTI_EOC_OVERHEAD_B1 | i)<<8))

#define  GTI_EOC_M_ADDR_FIELD(msg)     (GTI_BYTE)( (msg >> 2) & 0x3 )
#define  GTI_EOC_M_OPCODE_FIELD(msg)   (GTI_BYTE)( (msg >> 4) & 0x1 )
#define  GTI_EOC_M_PARITY_FIELD(msg)   (GTI_BYTE)( (msg >> 5) & 0x1 )
#define  GTI_EOC_M_MSG_FIELD(msg)      (GTI_BYTE)( (msg >> 6) & 0x1 )
#define  GTI_EOC_M_INFO_FIELD(msg)     (GTI_BYTE)( ((msg >> 7) & 0x1) | \
                                                  ((msg >> 8) & 0xFE) )
#define  NOT_USED(v)                   ((void) v)
/************************************************************************
*                        FUNCTION DECLARATIONS
************************************************************************/
GTI_WORD GTIEocExecCmd(
                  GTI_XCVR_INFO *GTISysPtr,
                  GTIEOC_CMD *hostPtr,
                  GTI_WORD eocMsg);

void GTIEocInit(GTIEOC_INFO *eInfoPtr);

GTI_WORD GTIEocHostCmd(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr);
void GTIDgaspProc(GTI_XCVR_INFO *GTISysPtr);
#ifdef GTI_PAGE_D_BUILT
	void GTIAocProc(GTI_XCVR_INFO *GTISysPtr, GTIEOC_INFO *eocInfoPtr,
        GTIEOC_CMD *hostPtr);
#endif
#endif   /* GTI_EOCA_H */
