/*
    @(#)gti_whip.h	1.18
*/

#ifndef  INCLUDE_FILE_GTIWHIP_H
#define  INCLUDE_FILE_GTIWHIP_H

#ifdef GTI_ISOS
#define GTI_LITTLE_ENDIAN
#endif

/****************************************************************************/
/* (C) Copyright 1998 by GlobeSpan Semiconductor Inc                        */
/*                                                                          */
/* File name: GTI_WHIP.H                         Created: April 06, 1998    */
/* Author: Dmitry Katz                                                      */
/*                                                                          */
/* Description: This file defines Serial Communication Protocol data.       */
/****************************************************************************/

#define L2_MAX_MSG_LENGTH 255
#define L2_RCV_BUF_MAX    256
#define L2_XMT_BUF_MAX    524

#define L2_MAX_RETRIES      3

/*-------------------------------------------------------------------------- */
/* Message delimiters                                                        */
/*-------------------------------------------------------------------------- */
#define L2_DLE  0x95
#define L2_STX  0x87
#define L2_ETX  0x88

#define L2_STATE_ACTIVE     0
#define L2_STATE_DLE_START  1
#define L2_STATE_IDLE       2
#define L2_STATE_DLE_STOP   3

/*-------------------------------------------------------------------------- */
/* Endian Swap macros. Note: One of the Endians mode should be defined.      */
/*-------------------------------------------------------------------------- */

#if defined(GTI_BIG_ENDIAN)
    #define SWAP_DWORD(pDst, pSrc){ \
    GTI_BYTE _b0, _b1, _b2, _b3;    \
    _b0 = ((GTI_BYTE *)pSrc)[0];    \
    _b1 = ((GTI_BYTE *)pSrc)[1];    \
    _b2 = ((GTI_BYTE *)pSrc)[2];    \
    _b3 = ((GTI_BYTE *)pSrc)[3];    \
    ((GTI_BYTE *)pDst)[0] = _b3;    \
    ((GTI_BYTE *)pDst)[1] = _b2;    \
    ((GTI_BYTE *)pDst)[2] = _b1;    \
    ((GTI_BYTE *)pDst)[3] = _b0;}
#elif defined(GTI_LITTLE_ENDIAN)
    #define SWAP_DWORD(pDst, pSrc){ \
    GTI_BYTE _b0, _b1, _b2, _b3;    \
    _b0 = ((GTI_BYTE *)pSrc)[0];    \
    _b1 = ((GTI_BYTE *)pSrc)[1];    \
    _b2 = ((GTI_BYTE *)pSrc)[2];    \
    _b3 = ((GTI_BYTE *)pSrc)[3];    \
    ((GTI_BYTE *)pDst)[0] = _b0;    \
    ((GTI_BYTE *)pDst)[1] = _b1;    \
    ((GTI_BYTE *)pDst)[2] = _b2;    \
    ((GTI_BYTE *)pDst)[3] = _b3;}
#else
    NO_ENDIAN_MODE_DEFINED
#endif

#if defined(GTI_BIG_ENDIAN)
    #define SWAP_WORD(pDst, pSrc){  \
    GTI_BYTE _bHi, _bLow;           \
    _bHi  = ((GTI_BYTE *)pSrc)[0];  \
    _bLow = ((GTI_BYTE *)pSrc)[1];  \
    ((GTI_BYTE *)pDst)[0] = _bLow;  \
    ((GTI_BYTE *)pDst)[1] = _bHi;}
#elif defined(GTI_LITTLE_ENDIAN)
    #define SWAP_WORD(pDst, pSrc){  \
    GTI_BYTE _bHi, _bLow;           \
    _bLow  = ((GTI_BYTE *)pSrc)[0]; \
    _bHi = ((GTI_BYTE *)pSrc)[1];   \
    ((GTI_BYTE *)pDst)[0] = _bLow;  \
    ((GTI_BYTE *)pDst)[1] = _bHi;}
#else
    NO_ENDIAN_MODE_DEFINED
#endif

/*-------------------------------------------------------------------------- */
/* Message Header                                                            */
/*-------------------------------------------------------------------------- */
#define L2_MAX_MSGBODY_LENGTH   240

#define L2_HEADER_LEN_OFS 0
#define L2_HEADER_MID_OFS 1
#define L2_HEADER_SID_OFS 2
#define L2_HEADER_TYP_OFS 3

#define L2_MSG_OPCODE_OFS 0
#define L2_MSG_BODY_OFS   5

#define XCVR_HOST_DUMP_TYPE         0x701 /* special translation            */

#define GLOBAL_DUMP_TYPE            0xf01 /* special translation            */

#define IPU_CODE_DUMP_TYPE          0x801 /* special translation            */
#define FDP_CODE_DUMP_TYPE          0x901 /* special translation            */
#define PCU_CODE_DUMP_TYPE          0xA01 /* special translation            */
#define BE_CODE_DUMP_TYPE           0xB01 /* special translation            */
#define TR_CODE_DUMP_TYPE           0xC01 /* special translation            */
#define AVM_CODE_DUMP_TYPE          0xD01 /* special translation            */
#define XDP_CODE_DUMP_TYPE          0xE01 /* special translation            */

#define SRAM1_DUMP_TYPE             0x101 /* special translation            */
#define SRAM2_DUMP_TYPE             0x201 /* special translation            */
#define SHADOW_IREG_DUMP_TYPE       0x301 /* special translation            */
#define HREG_DUMP_TYPE              0x401 /* special translation            */
#define EXTERNAL_LBRAM              0x402 /* special translation            */
#define EXTERNAL_RSRAM              0x502 /* special translation            */
#define E1T1RAM_TYPE                0x602 /* special translation            */

#define VREG0_DUMP_TYPE             0x103 /* special translation            */
#define VREG1_DUMP_TYPE             0x203 /* special translation            */

#define XCVR_HOST_DUMP_WORD_LENGTH  16    /* default one word               */
#define IPU_CODE_DUMP_WORD_LENGTH   16    /* default one word               */
#define FDP_CODE_DUMP_WORD_LENGTH   32    /* default two words              */
#define PCU_CODE_DUMP_WORD_LENGTH   16    /* default one word               */
#define BE_CODE_DUMP_WORD_LENGTH    48    /* default one word               */
#define TR_CODE_DUMP_WORD_LENGTH    16    /* default one word               */
#define AVM_CODE_DUMP_WORD_LENGTH   48    /* default one word               */
#define XDP_CODE_DUMP_WORD_LENGTH   16    /* default one word               */

#define SRAM1_DUMP_WORD_LENGTH      8     /* default one byte               */
#define SRAM2_DUMP_WORD_LENGTH      8     /* default one byte               */
#define HREG_DUMP_WORD_LENGTH       8     /* default one byte               */
#define EXT_RAM_DUMP_WORD_LENGTH    8     /* default one byte               */
#define EXT_ROM_DUMP_WORD_LENGTH    8     /* default one byte               */
#define E1T1RAM_WORD_LENGTH         8     /* default one byte               */

/* -------------------------Read - Write memory ---------------------------- */

#define L2_MEMORY_RW_MID_OFS    1
#define L2_MEMORY_RW_SEG_OFS    3
#define L2_MEMORY_RW_OFF_OFS    5
#define L2_MEMORY_RW_LEN_OFS    7
#define L2_MEMORY_RW_DAT_OFS    8

#define WHIP_GET_MEM_TYPE(mem_id)       ((GTI_WORD)mem_id&0x0fff)
#define WHIP_GET_CHIP_ID(mem_id)        (((GTI_WORD)mem_id>>12)&0x07)
#define WHIP_GET_NEEDACK(mem_id)        ((GTI_WORD)mem_id>>15)
#define WHIP_GET_MEM_ID(mem,chip,need)  ((mem&0x0fff)|(chip<<12)|(need<<15))

/* --------------------------- System Status ------------------------------- */

#define L2_SSTATUS_CHNL_OFS     1
#define L2_SSTATUS_NUMB_OFS     2
#define L2_SSTATUS_LIST_OFS     3

/* ----------------------------   Any Status ------------------------------- */

#define L2_WT_ANYSTAT_CHNL_OFS  1
#define L2_WT_ANYSTAT_STID_OFS  2
#define L2_WT_ANYSTAT_FST_OFS   3
#define L2_WT_ANYSTAT_LEN_OFS   5

#define L2_TW_ANYSTAT_CHNL_OFS  1
#define L2_TW_ANYSTAT_STID_OFS  2
#define L2_TW_ANYSTAT_TOTL_OFS  3
#define L2_TW_ANYSTAT_FST_OFS   5
#define L2_TW_ANYSTAT_LEN_OFS   7
#define L2_TW_ANYSTAT_BUF_OFS   8

/* ------------------------- Execute Command message ---------------------- */

#define L2_MAX_EXEC_OPTIONS_NUMBER 80
#define L2_EXECUTE_COMMAND_REPLY    2

#define L2_WT_EXEC_PAR_MAX_OFS  1
#define L2_WT_EXEC_PAR_LST_OFS  2

#define L2_TW_EXEC_ACTION_OFS   1
#define L2_TW_EXEC_RESULT_OFS   2

/* ---------------------------- Read Trace message ------------------------ */

#define L2_TRACE_START_OFS      1
#define L2_TRACE_TYPE_OFS       3
#define L2_TRACE_NUM_OFS        3
#define L2_TRACE_BUF_OFS        4

#define L2_MAX_TRACE_LINES      30
#define L2_MAX_TIMESTAMP_LINES  120

/* ---------------------------- Read LBUS trace  message -------------------*/

#define L2_MAX_LBUS_TRACE_LINES 60

/* --------------------- Set Generic Parameter message -------------------- */

#define L2_MAX_GENERIC_PARAM_NUMBER 70

#define L2_FRAMER_PARAM_TYPE             1

#define L2_PARAM_INTERFACE               2
#define L2_MASTER_LP_TMG_SEL             3
#define L2_PARAM_ZBITS                   4
#define L2_PARAM_SEND_TYPE               4  /* Optis Only */

#define L2_EOC_PARAM_TYPE                5

#define L2_PIN_PARAM_TYPE                6
#define L2_SEND_TOM_DATA_TYPE            7
#define L2_OH_PARAM_TYPE                 8
#define L2_XCVR_LATENCY_TYPE             9
#define L2_SELECT_NTR_SOURCE_CLOCK_TYPE  10
#define L2_OH_RECEIVE_TYPE               11
#define L2_TXCLAV_MODE_TYPE              12
#define L2_RESET_HISTORY_TYPE            13
#define L2_ENABLE_TRACE_TYPE             14
#define L2_BGA_CLKMODE_TYPE              15
#define L2_PSMODE_TYPE                   17
#define L2_INTERFACE_TEST                18
#define L2_RELOAD_DEBUG_POINTERS_TYPE    19
#define L2_EOC_TEST_TYPE                 20  /* for eoc test only */
#define L2_RESET_RXPTR_TYPE              21  /* for eoc test only */
#define L2_EXTEOC_FLAG_ON_TYPE           22  /* for eoc test only */
#define L2_EXTEOC_FLAG_OFF_TYPE          23  /* for eoc test only */
#define L2_ISW_MASK_TYPE                 24
#define L2_PARAMETRIC_TEST_TYPE          25
#define L2_SELECT_BIN_USAGE              28
#define L2_SHUT_OFF_XCVR                 29
#define L2_DUMP_INSTALL_RESULTS          30
#define L2_SET_PERFERED_MODE             31
#define L2_SET_INSTALL_SELECTION         32
#define L2_START_PARAMETRIC_TEST         33
#define L2_CLEAR_DYING_GASP              34
#define L2_SWITCH_CHANNEL_MODE_TYPE      35
#define L2_AUTO_RETRAIN                  36
#define L2_SET_RINGING                   37


#define L2_SET_GEN_PARAMETER_FAILED 0
#define L2_GEN_PARAMETER_NOT_EXISTS 1
#define L2_GEN_INCOMPATIBLE_NUMBER  2
#define L2_SET_GEN_PARAMETER_OK     3

#define L2_WT_SET_GEN_TYP_OFS   1
#define L2_WT_SET_GEN_MAX_OFS   2
#define L2_WT_SET_GEN_BUF_OFS   3

#define L2_TW_SET_GEN_TYP_OFS   1
#define L2_TW_SET_GEN_RES_OFS   2

/* ----------------------- Breakpoint info message ------------------------ */

#define L2_MAX_BREAKPOINTS 5

#define L2_IPU_BREAKPOINTS 0
#define L2_FDP_BREAKPOINTS 1
#define L2_XDP_BREAKPOINTS 2
#define L2_PCU_BREAKPOINTS 3

#define L2_MAX_DSP_BP_TYPE 4

#define L2_MAX_BP_DUMP_LENGTH 75

#define L2_SAS_BREAKPOINTS 8

#define L2_BRKPNT_STATE_DISABLED   0
#define L2_BRKPNT_STATE_ENABLED    1
#define L2_BRKPNT_STATE_ILLEGAL    2

#define L2_BRKPNTI_TYP_OFS 1
#define L2_BRKPNTI_STA_OFS 2
#define L2_BRKPNTI_INF_OFS (L2_MAX_BREAKPOINTS + 2)

#define L2_BRKPNTI_AUX_OFS (L2_BRKPNTI_INF_OFS+(L2_MAX_BREAKPOINTS *4))

#define L2_BRKPHIT_NUM_OFS 1
#define L2_BRKPHIT_PC_OFS  2
#define L2_BRKPHIT_DAT_OFS 4

/* ------------------ Random Memory Read Reply message  ------------------- */

#define L2_MAX_RANDOM_MEMORY_REQUEST   50

#define L2_RNDREAD_CHNL_OFS 1
#define L2_RNDREAD_MEMT_OFS 2
#define L2_RNDREAD_NUMB_OFS 3
#define L2_RNDREAD_DATA_OFS 4

/* -------------------------- Init XCVR  message -------------------------- */

#define INIT_XCVR_RESET_DSP         0
#define INIT_XCVR_INT_DSP           1
#define INIT_XCVR_INT_FRMR          2
#define INIT_XCVR_INT_HOST          3
#define INIT_XCVR_RESET_CORE_DSP    4
#define INIT_XCVR_RESET_CHIP_DSP    5

#define INIT_XCVR_INT_OFF   0
#define INIT_XCVR_INT_ON    1
#define INIT_XCVR_INT_CLR   2

#define L2_INITXCVR_TYPE_OFS 1
#define L2_INITXCVR_ACTN_OFS 2

/* ------------------- Set Active Configuration Info--- -------------------- */

#define L2_WT_ACTCONF_XCVR_OFS  1
#define L2_WT_ACTCONF_TARG_OFS  2

#define L2_TW_ACTCONF_XCVR_OFS  1
#define L2_WT_ACTCONF_MAX_OFS   2
#define L2_WT_ACTCONF_FUT1_OFS  3
#define L2_WT_ACTCONF_FUT2_OFS  4

/* ----------------------- Repport Coeff ----------------------------------- */

#define MAX_COEFF_DATA_IN_ONE_MSG 48

#define L2_TW_RC_LOOP_NUMBER       1

#define L2_TW_RC_PLOT_CHIP         1
#define L2_TW_RC_BEG_IDX           2
#define L2_TW_RC_END_IDX           4
#define L2_TW_RC_LOOP_IDX          6
#define L2_TW_RC_DATA_IDX          8

/* -------------------------- BERT Test message ----------------------------- */

#define L2_BERTEST_CHNL_OFS 1
#define L2_BERTEST_BITR_OFS 2
#define L2_BERTEST_SYNC_OFS 4
#define L2_BERTEST_ERR_OFS  5

/* ---------------------------- Contellation  ------------------------------- */

#define L2_CONSTELL_BIN_OFS 1
#define L2_CONSTELL_X_OFS   2
#define L2_CONSTELL_Y_OFS   6

/* --------------------- Local Bus Test message ------------------------ */

#define L2_LBUSTEST_BEG_OFS 1
#define L2_LBUSTEST_END_OFS 5
#define L2_LBUSTEST_RES_OFS 9

#define L2_WT_LBUSTEST_TYP_OFS 9
#define L2_WT_LBUSTEST_COD_OFS 10
#define L2_WT_LBUSTEST_DAT_OFS 11

#define L2_TW_LBUSTEST_TYP_OFS 10

#define LB_TEST_TYPE    0
#define LB_CHKSUM_TYPE  1
#define LB_LOAD_BUS     2
#define LB_LOAD_BUS_ADV 3

#define WHIP_MAX_LBUS_DATA_LEN  230
/* ------------------------ EOC command message --------------------------- */

#define WHIP_DISABLE_EOC        0
#define WHIP_XMTDATA_EOC        1
#define WHIP_POLLDATA_EOC       2
#define WHIP_BUFFER_VALID_EOC   3
#define WHIP_BUFFER_EMPTY_EOC   4
#define WHIP_XMTDATAMODE_EOC    5
#define WHIP_POLLDATAMODE_EOC   6

#ifdef GTI_PROD_DMT
#define WHIP_MAX_EOC_DATA_LEN   32
#else
#define WHIP_MAX_EOC_DATA_LEN   200
#endif

#define L2_WT_EOCCMD_COD   1
#define L2_WT_EOCCMD_BEG   2
#define L2_WT_EOCCMD_CMD   3
#define L2_WT_EOCCMD_LEN   4
#define L2_WT_EOCCMD_DAT   5

#define L2_TW_EOCCMD_COD   1
#define L2_TW_EOCCMD_DAT   2

#define L2_TW_EOCDAT_LEN   2
#define L2_TW_EOCDAT_NXT   3
#define L2_TW_EOCDAT_DAT   5

/* ----------------------------  Message Opcodes -------------------------- */

#define L2_REGISTRATION_MSG_OPC     1
#define L2_HOT_LINE_MSG_OPC         2
#define L2_ACK_OPC                  5
#define L2_NACK_OPC                 6

#define L2_SET_CONFIGURATION_OPC    19
#define L2_READMEMORY_OPC           20
#define L2_WRITEMEMORY_OPC          21
#define L2_WRITEMEMORY_BDCST_OPC    22
#define L2_EXEC_COMMAND_OPC         23
#define L2_JUMP_TO_BOOT_COMMAND_OPC 24
#define L2_INIT_DEBUG_TABLES_OPC    25
#define L2_READ_TRACE_BUFFER_OPC    26
#define L2_BREAKPOINT_INFO_OPC      27
#define L2_BREAKPOINT_HIT_OPC       28
#define L2_SET_REP_COEF_DATA_OPC    29
#define L2_GET_REP_COEF_DATA_OPC    30
#define L2_SET_REP_COEF_TRIG_OPC    31
#define L2_SYSTEM_STATUS_OPC        32
#define L2_INIT_XCVR_OPC            33
#define L2_RANDOM_MEMREAD_OPC       34
#define L2_READ_CONSTELLATION_OPC   35
#define L2_EOC_COMMAND_OPC          36
#define L2_ANY_STATUS_OPC           37
#define L2_ACTIVE_CONFIG_OPC        38
#define L2_BERT_TEST_OPC            39
#define L2_SET_GENERIC_PARAM_OPC    40
#define L2_TEST_LOCAL_BUS_OPC       41
#define L2_READ_TIMESTAMP_OPC       42
#define L2_LBUS_TRACE_OPC           43
#define L2_TRAFFIC_DEBUG_OPC        45
#define L2_CUSTOM_MSG_OPC           46

#endif /* INCLUDE_FILE_GTIWHIP_H */
