////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.1 $
//     $Modtime: 5/15/02 12:18p $
//     $Author: kattre $
//
// ABSTRACT:
//     Contains data structures, function declarations, and macros for the
//     RFC1483 module.
//
////////////////////////////////////////////////////////////////////////////////
#ifndef RFC1483_H
#define RFC1483_H

#include "atmusb.h"

typedef void * RFC_USER_HANDLE;

#define ETHERNET_HEADER_SIZE	        14
#define ETHERNET_MAX_PACKET_SIZE		1500
#define ETHERNET_LAN_FCS_SIZE			4

#ifndef ETHERTYPE_IP    /* IP protocol */
    #ifdef GTI_LITTLE_ENDIAN
    #define ETHERTYPE_IP        0x0008  
    #elif (defined (GTI_BIG_ENDIAN))
    #define ETHERTYPE_IP        0x0800  
    #elif
    NO_ENDIAN_MODE_DEFINED
    #endif
#endif

#ifndef ETHERTYPE_ARP   /* Addr. resolution protocol */
    #ifdef GTI_LITTLE_ENDIAN
    #define ETHERTYPE_ARP       0x0608  
    #elif (defined (GTI_BIG_ENDIAN))
    #define ETHERTYPE_ARP       0x0806  
    #elif
    NO_ENDIAN_MODE_DEFINED
    #endif
#endif


#ifndef ETHERNET_ARP_OPCODE_REPLY
    #ifdef GTI_LITTLE_ENDIAN
    #define ETHERNET_ARP_OPCODE_REPLY       0x0200
    #elif (defined (GTI_BIG_ENDIAN))
    #define ETHERNET_ARP_OPCODE_REPLY       0x0002
    #elif
    NO_ENDIAN_MODE_DEFINED
    #endif
#endif





// #define ETHERTYPE_IP					0x0008	// Little Endian Order /* NK */
// #define ETHERTYPE_ARP					0x0608	

// #define ETHERNET_ARP_OPCODE_REPLY		0x0200

#define IP_ADDRESS_SIZE					4

#define RFC1483_LLC_ENCAP_BRIDGED_ETH_SIZE	10
#define RFC1483_LLC_ENCAP_ROUTED_IP_SIZE	8
#define RFC1483_VC_ENCAP_BRIDGED_ETH_SIZE	2
#define RFC1483_MAX_ENCAPSULATION_SIZE		10
#define RFC1483_RFC1577_ENCAP_SIZE          8


#define RFC1483_MAX_SDU_SIZE			ETHERNET_MAX_PACKET_SIZE       + \
										ETHERNET_HEADER_SIZE           + \
										RFC1483_MAX_ENCAPSULATION_SIZE + \
										ETHERNET_LAN_FCS_SIZE


typedef void (* RFC1483_USER_LINK_STATUS_CHANGE)
                       (PATM_USB_INSTANCE      Instance,
                       ATM_PHY_LINK_STATE      LinkState,
                       ULONG                   UpstreamRate,
                       ULONG                   DownstreamRate,
                       USHORT                  COType);

typedef void (* RFC1483_USER_RECEIVE)
                       (PATM_USB_INSTANCE      Instance,
                       RFC1483_VC_HANDLE       Rfc2516VcHandle,
                       PXFER_DATA_BUFFER       Buffer, //KAMAL
                       UCHAR                   CpcsUu,
                       UCHAR                   Cpi);

typedef void (* RFC1483_USER_SEND_COMPLETE)
                       (PATM_USB_INSTANCE      Instance,
                       ATM_UNI_USER_HANDLE     NotUsed,
                       PNDIS_PACKET            Packet);

typedef struct
{
   RFC1483_VC_HANDLE                   Rfc1483VcHandle;
   RFC1483_USER_RECEIVE                UserReceive;
   RFC1483_USER_LINK_STATUS_CHANGE     UserLinkStatusChange;
   RFC1483_USER_SEND_COMPLETE          UserSendComplete;
   RFC_USER_HANDLE                     UserHandle;
   NDIS_HANDLE                         MiniportAdapterHandle;
   ATM_UNI_HANDLE                      AtmUniHandle;
} RFC1483_INSTANCE, * PRFC1483_INSTANCE;

//------------------------------------------------------------------------------
//
// Function Declarations
//
//------------------------------------------------------------------------------
BOOLEAN
Rfc1483Initialize( 
	PATM_USB_INSTANCE 		Instance,
	ATM_PHY_BIND 			PhyBind,
	ATM_PHY_DEVICE_HANDLE	PhyDeviceHandle

);

void
Rfc1483Shutdown( PATM_USB_INSTANCE Instance);


void
Rfc1483SendComplete(
    PATM_USB_INSTANCE	Instance,
	ATM_UNI_USER_HANDLE	NotUsed,
    PNDIS_PACKET   		Packet);

void
Rfc1483EthernetArpReply(
	PATM_USB_INSTANCE	Instance, 
	PNDIS_PACKET		Packet,
	gsi_lan_t			*plan);

void
Rfc1483ReceiveData(
    PATM_USB_INSTANCE	Instance,
	ATM_UNI_USER_HANDLE	 AtmUniVcHandle,
	PXFER_DATA_BUFFER		Buffer,
	UCHAR				CpcsUu,
	UCHAR				Cpi
);



void
Rfc1483IndicateReceivedData(
	PATM_USB_INSTANCE	Instance,
	PXFER_DATA_BUFFER		Buffer,
    ATM_UNI_HANDLE          AtmUniVc
    );



NDIS_STATUS 
Rfc1483TransferData(
	PNDIS_PACKET 		Packet,
	PUINT 				BytesTransferred,
	PATM_USB_INSTANCE	Instance,
	PUCHAR 				Buffer,
	UINT 				ByteOffset,
	UINT 				BytesToTransfer
);

NDIS_STATUS
Rfc1483QueryInformation(
	PATM_USB_INSTANCE	Instance,
	NDIS_OID       		Oid,
	PVOID          		InformationBuffer,
	ULONG          		InformationBufferLength,
	PULONG        		BytesWritten,
	PULONG        		BytesNeeded
);

NDIS_STATUS
Rfc1483SetInformation (
	PATM_USB_INSTANCE 	Instance,
	NDIS_OID 			Oid,
	PVOID 				InformationBuffer,
	ULONG 				InformationBufferLength,
	PULONG 				BytesRead,
	PULONG 				BytesNeeded
);

void
Rfc1483LinkStatusChange(
	PATM_USB_INSTANCE	Instance,
	ATM_PHY_LINK_STATE	LinkState,
	ULONG				UpstreamRate,
	ULONG				DownstreamRate,
	USHORT              COType
);


BOOLEAN Rfc1483GetEncapsulation(
                            PATM_USB_INSTANCE	Instance,
                            RFC1483_MODE Rfc1483Mode,
                            PUCHAR *Encapsulation,
                            PUCHAR EncapsulationSize
                            );
NDIS_STATUS 
Rfc1483Send(
	PATM_USB_INSTANCE	Instance,
	PNDIS_PACKET  		Packet,
    ATM_UNI_HANDLE      VCHandle,
    BOOLEAN              Arp,
	gsi_lan_t           *plan);



void Rfc1483ReceiveComplete(
       	gsi_lan_t			*plan,
    	PXFER_DATA_BUFFER	Buffer);

void Rfc1483ReadNetworkAddress(
                   PATM_USB_INSTANCE   Instance,
                   PUCHAR              PermanentAddress);

RFC1483_HANDLE Rfc1483Create(
       PATM_USB_INSTANCE               Instance,
       RFC1483_USER_LINK_STATUS_CHANGE UserLinkStatusChange,
       BOOLEAN                         IndicatePackets,
       ULONG                           MediaSpecificInformationSize,
       USHORT                          MaxSduSize,
       USHORT                          RxBufferCount,
       USHORT                          TxBufferCount,
       NDIS_HANDLE                     MiniportAdapterHandle,
       ATM_PHY_BIND                    PhyBind,
       ATM_PHY_DEVICE_HANDLE           PhyDeviceHandle,
       PUCHAR                          PhyUniqueId,
       PVOID                           UsbPhyHandle);

RFC1483_VC_HANDLE Rfc1483CreateVc(
       RFC1483_HANDLE                  Rfc1483Handle,
  	    RFC1483_USER_RECEIVE            UserReceive,
       RFC1483_USER_SEND_COMPLETE      UserSendComplete,
	int									ilanIndex);

RFC_STATUS Rfc1483ActivateVc(
       PATM_USB_INSTANCE               Instance,
       RFC1483_VC_HANDLE               Rfc1483VcHandle,
       UCHAR                           Vpi,
       USHORT                          Vci,
       ATM_UNI_AAL                     Aal,
       USHORT                          MaxSduSize,
       PUCHAR                          VcEncapsulation,
       USHORT                          VcEncapsulationLength,
       BOOLEAN                         ReserveResources);

void Rfc1483DeactivateVc(RFC1483_VC_HANDLE Rfc1483VcHandle);

void Rfc1483DeleteVc(RFC1483_VC_HANDLE Rfc1483VcHandle);

#ifdef ANNEXB
void Rfc1483OpenVc(
	PATM_USB_INSTANCE	Instance,
	ATM_PHY_LINK_STATE	LinkState,
	USHORT              COType);
#endif // ANNEXB

#endif // RFC1483_H

