/*+I*************************************************************************
 * TriplePoint C Library Definitions
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *
 * This source code may be used by anyone provided the TriplePoint copyright
 * is maintained in any derivitive works.
 *-I*************************************************************************/
/****************************************************************************
@doc INTERNAL TpiUtil TpiUtil_h

@module TpiDebug.c |

    This module implements some of the basic C library functions.  This makes
    porting to some platforms easier, where the C run-time libraries are not
    available.

@head3 Contents |
@index class,mfunc,func,msg,mdata,struct,enum | TpiUtil_h

@end
 ****************************************************************************/
#ifndef _TPIUTIL_H
#define _TPIUTIL_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined(macintosh)
# include <size_t.h>
#elif defined(linux)
# if !defined(NO_TPI_OVERRIDES)
#  define NO_TPI_OVERRIDES
# endif /* NO_TPI_OVERRIDES */
# if !defined(_SIZE_T)
    typedef unsigned int size_t;
#   define _SIZE_T
# endif /* _SIZE_T */
#else
#  define NO_TPI_OVERRIDES
# if !defined(_SIZE_T_DEFINED)
    typedef unsigned int size_t;
#   define _SIZE_T_DEFINED
# endif  /* _SIZE_T_DEFINED */
#endif

/*
// Include TPI routines in all debug builds, but not the release build
// if they are explicitly excluded by defining NO_TPI_UTILS
*/
#if (DBG != 0) || defined(DEBUG) || defined(_DEBUG) || !defined(NO_TPI_UTILS)

#if !defined(NO_TPI_OVERRIDES)
# if !defined(isalnum)
#  define isalnum        tpi_isalnum
#  define isalpha        tpi_isalpha
#  define iscntrl        tpi_iscntrl
#  define isdigit        tpi_isdigit
#  define isgraph        tpi_isgraph
#  define islower        tpi_islower
#  define isprint        tpi_isprint
#  define ispunct        tpi_ispunct
#  define isspace        tpi_isspace
#  define isupper        tpi_isupper
#  define isxdigit       tpi_isxdigit
#  define isascii        tpi_isascii
#  define toascii        tpi_toascii
# endif

# define tpi_memcpy      memcpy
# define tpi_memset      memset
# define tpi_memcmp      memcmp
# define tpi_memchr      memchr
# define tpi_strlen      strlen
# define tpi_strcpy      strcpy
# define tpi_strncpy     strncpy
# define tpi_strcat      strcat
# define tpi_strncat     strncat
# define tpi_strcmp      strcmp
# define tpi_strncmp     strncmp
# define tpi_stricmp     stricmp
# define tpi_strnicmp    strnicmp
# define tpi_strchr      strchr
# define tpi_strrchr     strrchr
# define tpi_strstr      strstr
# ifdef sprintf
#  undef sprintf
# endif
# define tpi_sprintf     sprintf
# define tpi_vsprintf    vsprintf
#endif /* NO_TPI_OVERRIDES */

#ifndef  SIZE_OF_ARRAY
# define SIZE_OF_ARRAY(arr) (sizeof(arr) / sizeof(arr[0]))
#endif

/* Variable argument list definitions */
#ifndef va_start
#ifndef  _INTSIZEOF
# define _INTSIZEOF(n)   ((sizeof(n) + sizeof(int) - 1) & ~(sizeof(int) - 1))
#endif

#ifndef	va_list
typedef char * va_list;
#endif
# define va_start(ap,v)  (ap = (va_list)&v + _INTSIZEOF(v))
# define va_arg(ap,t)    (*(t *)((ap += _INTSIZEOF(t)) - _INTSIZEOF(t)))
# define va_end(ap)      (ap = (va_list)0)
#endif

/****************************************************************************
 * Memory handling functions
 ****************************************************************************/
void *tpi_memcpy(void *dst, const void *src, size_t n);
void *tpi_memmove(void *dst, const void *src, size_t n);
void *tpi_memchr(const void *ptr, int val, size_t n);
int tpi_memcmp(const void *ptr1, const void *ptr2, size_t n);
void *tpi_memset(void *dst, int val, size_t n);

/****************************************************************************
 * Misc functions
 ****************************************************************************/
unsigned long tpi_min(unsigned long x, unsigned long y);
unsigned long tpi_max(unsigned long x, unsigned long y);
unsigned short tpi_swaps(unsigned short x);
unsigned long tpi_swapl(unsigned long x);

/****************************************************************************
 * Character handling functions
 ****************************************************************************/
#define _U   0x01  /* upper */
#define _L   0x02  /* lower */
#define _D   0x04  /* digit */
#define _C   0x08  /* cntrl */
#define _P   0x10  /* punct */
#define _S   0x20  /* white space (space/lf/tab) */
#define _X   0x40  /* hex digit */
#define _SP  0x80  /* hard space (0x20) */

extern unsigned char const _tpi_ctype[];

#define __ismask(x)     (_tpi_ctype[(int)(unsigned char)(x)])
#define tpi_isalnum(c)  ((__ismask(c)&(_U|_L|_D)) != 0)
#define tpi_isalpha(c)  ((__ismask(c)&(_U|_L)) != 0)
#define tpi_iscntrl(c)  ((__ismask(c)&(_C)) != 0)
#define tpi_isdigit(c)  ((__ismask(c)&(_D)) != 0)
#define tpi_isgraph(c)  ((__ismask(c)&(_P|_U|_L|_D)) != 0)
#define tpi_islower(c)  ((__ismask(c)&(_L)) != 0)
#define tpi_isprint(c)  ((__ismask(c)&(_P|_U|_L|_D|_SP)) != 0)
#define tpi_ispunct(c)  ((__ismask(c)&(_P)) != 0)
#define tpi_isspace(c)  ((__ismask(c)&(_S)) != 0)
#define tpi_isupper(c)  ((__ismask(c)&(_U)) != 0)
#define tpi_isxdigit(c) ((__ismask(c)&(_D|_X)) != 0)
#define tpi_isascii(c)  (((unsigned char)(c))<=0x7f)
#define tpi_toascii(c)  (((unsigned char)(c))&0x7f)

char tpi_toupper(char cp);
char tpi_tolower(char cp);

/****************************************************************************
 * String handling functions
 ****************************************************************************/
unsigned char tpi_basedigit(char digit, unsigned char base, unsigned char *pValue);
unsigned long tpi_strtoul(const char *cp, char **endp, unsigned char base);
char *tpi_strcat(char *dst, const char *src);
char *tpi_strncat(char *dst, const char *src, size_t n);
char *tpi_strcpy(char *dst, const char *src);
char *tpi_strncpy(char *dst, const char *src, size_t n);
char *tpi_strlwr(char *s);
char *tpi_strupr(char *s);
size_t tpi_strlen(const char *s);
size_t tpi_strnlen(const char *s, size_t n);
int tpi_strcmp(const char *s1, const char *s2);
int tpi_strncmp(const char *s1, const char *s2, size_t n);
int tpi_strcasecmp(const char *s1, const char *s2);
int tpi_strncasecmp(const char *s1, const char *s2, size_t n);
size_t tpi_strspn(const char *s, const char *set);
char *tpi_strpbrk(const char *s, const char *set);
char *tpi_strchr(const char *s, int c);
char *tpi_strrchr(const char *s, int c);
char *tpi_strstr(const char *s1, const char *s2);
int tpi_vsprintf(char *buf, const char *fmt, va_list args);
int tpi_sprintf(char *buffer, const char *fmtString, ...);
int tpi_sprintf255(unsigned char *buf255, /* 255 bytes max! */const char *fmt, ...);

#endif /* (DBG != 0) || defined(DEBUG) || defined(_DEBUG) || !defined(NO_TPI_UTILS) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _TPIUTIL_H */

