////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.2 $
//     $Modtime: 5/15/02 12:18p $
//     $Author: kattre $
//
// ABSTRACT:
//	   Contains data structures and macros for the USB CDC interface
//
//	   >>> Corresponds to Rev 1.0.9a of the CDC Specification <<<
//
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _USBCDC_H_
#define _USBCDC_H_

// 
// The structures in this file need to packed on a 1-byte boundary
// 
#include <pshpack1.h>

//------------------------------------------------------------------------------
//	
// CDC Related constant definitions
//
//------------------------------------------------------------------------------
#define CDC_COMM_DEVICE_CLASS_CODE				0x02
#define CDC_COMM_INTERFACE_CLASS_CODE			0x02
#define CDC_COMM_ATM_INTERFACE_SUBCLASS_CODE	0x07 // TBD in 1.0.9a
#define CDC_DATA_INTERFACE_CLASS_CODE			0x0A
#define CDC_DATA_ATM_INTERFACE_SUBCLASS_CODE	0x00
#define CDC_RELEASE_NUMBER						0x0110 
#define CDC_ESI_SIZE							7
#define CDC_MAX_NOTIFICATION_SIZE				16

//------------------------------------------------------------------------------
//	
// CDC Functional Descriptor related constants
//
//------------------------------------------------------------------------------
#define CDC_DESCRIPTOR_TYPE_CS_INTERFACE	0x24
#define CDC_DESCRIPTOR_TYPE_CS_ENDPOINT		0x25
#define CDC_DESCRIPTOR_SUBTYPE_HEADER		0x00
#define CDC_DESCRIPTOR_SUBTYPE_ATM			0x0F

//------------------------------------------------------------------------------
//	
// CDC_COMMON_FUNCTIONAL_DESCRIPTOR
//		The common part of each functional descriptor (always at the beginning)
//
//------------------------------------------------------------------------------
typedef struct tCDC_COMMON_FUNCTIONAL_DESCRIPTOR
{
	UCHAR bFunctionLength;
	UCHAR bDescriptorType;
	UCHAR bDescriptorSubType;

} CDC_COMMON_FUNCTIONAL_DESCRIPTOR, *PCDC_COMMON_FUNCTIONAL_DESCRIPTOR;

//------------------------------------------------------------------------------
//	
// CDC_HEADER_FUNCTIONAL_DESCRIPTOR
//		Descriptor that starts the class-specific descriptor
//
//------------------------------------------------------------------------------
typedef struct tCDC_HEADER_FUNCTIONAL_DESCRIPTOR
{
    UCHAR 	bFunctionLength;
    UCHAR 	bDescriptorType;
    UCHAR 	bDescriptorSubType;
    USHORT	bcdCDC;

} CDC_HEADER_FUNCTIONAL_DESCRIPTOR, *PCDC_HEADER_FUNCTIONAL_DESCRIPTOR;

//------------------------------------------------------------------------------
//	
// CDC_ATM_FUNCTIONAL_DESCRIPTOR
//		Descriptor that describes the operational modes supported by
//		the communication class interface (when operating under the ATM
//		Networking control model)
//
//------------------------------------------------------------------------------
typedef struct tCDC_ATM_FUNCTIONAL_DESCRIPTOR
{
	UCHAR	bFunctionLength;
	UCHAR	bDescriptorType;
	UCHAR	bDescriptorSubtype;
	UCHAR	iEndSystemIdentifier;
	UCHAR	bmDataCapabilities;
	UCHAR	bmATMDeviceStatistics;
	USHORT	wType2MaxSegmentSize;
	USHORT	wType3MaxSegmentSize;
	USHORT	wMaxVc;

} CDC_ATM_FUNCTIONAL_DESCRIPTOR,
  *PCDC_ATM_FUNCTIONAL_DESCRIPTOR;

//------------------------------------------------------------------------------
//	
// ATM Networking functional descriptor related constants
//
//------------------------------------------------------------------------------
#define CDC_ATM_DATA_TYPE_0	0x01
#define CDC_ATM_DATA_TYPE_1	0x02
#define CDC_ATM_DATA_TYPE_2	0x04
#define CDC_ATM_DATA_TYPE_3	0x08

#define CDC_ATM_VC_US_CELLS_SENT   				0x10
#define CDC_ATM_VC_DS_CELLS_RECEIVED   			0x08
#define CDC_ATM_DS_CELLS_HEC_ERROR_CORRECTED	0x04
#define CDC_ATM_DEV_US_CELLS_SENT				0x02
#define CDC_ATM_DEV_DS_CELLS_RECEIVED			0x01

//------------------------------------------------------------------------------
//	
// ATM Networking class specific request codes
//
//------------------------------------------------------------------------------
#define CDC_REQ_SEND_ENCAPSULATED_COMMAND	0x00
#define CDC_REQ_GET_ENCAPSULATED_RESPONSE	0x01
#define CDC_REQ_SET_ATM_DATA_FORMAT			0x50
#define CDC_REQ_GET_ATM_DEVICE_STATISTICS	0x51
#define CDC_REQ_SET_ATM_DEFAULT_VC			0x52
#define CDC_REQ_GET_ATM_VC_STATISTICS		0x53

//------------------------------------------------------------------------------
//	
// ATM device statistics feature selector codes
//
//------------------------------------------------------------------------------
#define CDC_ATM_DEV_STAT_SEL_US_CELLS_SENT				0x01
#define CDC_ATM_DEV_STAT_SEL_DS_CELLS_RECEIVED			0x02
#define CDC_ATM_DEV_STAT_SEL_DS_CELLS_DISCARDED			0x03
#define CDC_ATM_DEV_STAT_SEL_CELLS_HEC_ERROR_CORRECTED	0x04
#define CDC_ATM_DEV_STAT_SEL_CELLS_USB_CONGESTION		0x05

//------------------------------------------------------------------------------
//	
// ATM VC statistics feature selector codes
//
//------------------------------------------------------------------------------
#define CDC_ATM_VC_STAT_SEL_US_CELLS_SENT		0x01
#define CDC_ATM_VC_STAT_SEL_DS_CELLS_RECEIVED	0x02

//------------------------------------------------------------------------------
//	
// Class specific notification codes
//
//------------------------------------------------------------------------------
#define CDC_NOTIFICATION_NETWORK_CONNECTION			0x00
#define CDC_NOTIFICATION_RESPONSE_AVAILABLE			0x01
#define CDC_NOTIFICATION_CONNECTION_SPEED_CHANGE	0x2A
#define CDC_NOTIFICATION_DSP_INT                   0xF0

//------------------------------------------------------------------------------
//	
// Network Connection notification related constants
//
//------------------------------------------------------------------------------
#define CDC_NETWORK_CONNECTION_DISCONNECT	0x0000
#define CDC_NETWORK_CONNECTION_CONNECT		0x0001

#define CDC_MINIMUM_NOTIFICATION_MSG_SIZE 		8
#define CDC_NETWORK_CONNECTION_MSG_SIZE			8
#define CDC_CONNECTION_SPEED_CHANGE_MSG_SIZE	16

//------------------------------------------------------------------------------
//	
// CDC_NOTIFICATION
//		Data structure representing the common notification message.  Each
//		CDC begins with this structure, followed by zero or more bytes of
//		notification specific data
//
//------------------------------------------------------------------------------
typedef struct tCDC_NOTIFICATION
{
	UCHAR	bmRequestType;
	UCHAR	bNotification;
	USHORT	wValue;
	USHORT	wIndex;
	USHORT	wLength;
	UCHAR	Data[1];

} CDC_NOTIFICATION, *PCDC_NOTIFICATION;

//------------------------------------------------------------------------------
//	
// CDC_CONNECTION_SPEED_CHANGE
//		Data structure used to communicate the connection speed change 
//		information
//
//------------------------------------------------------------------------------
typedef struct tCDC_CONNECTION_SPEED_CHANGE
{
	ULONG	UpStreamBitRate;
	ULONG	DownStreamBitRate;

} CDC_CONNECTION_SPEED_CHANGE, *PCDC_CONNECTION_SPEED_CHANGE;

//------------------------------------------------------------------------------
//	
// CDC_DSP_DOWNLOAD_INFO
//		Data structure used to communicate when more DSP code needs
//    to be downloaded. 
//
//------------------------------------------------------------------------------
typedef struct tCDC_DSP_DOWNLOAD_INFO
{
	USHORT   isIpuReady;
} CDC_DSP_DOWNLOAD_INFO, *PCDC_DSP_DOWNLOAD_INFO;

//
// Restore the previous structure packing
//
#include <poppack.h>


#endif // _USBCDC_H_
