////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.4 $
//     $Modtime: 10/29/02 4:50p $
//     $Author: kattre $
//
// ABSTRACT:
//       Contains data structures, function declarations, and macros for the
//     USB Phy module
//
//
////////////////////////////////////////////////////////////////////////////////
#ifndef USBPHY_H_
#define USBPHY_H_

#ifdef _NTDDK_
#include "usbcdc.h"
#elif defined(linux)
#include "linux/spinlock.h"
#include "GpSnull.h"
#endif // linux
#include "atmphy.h"
#include "atmusb.h"
#include "dsl.h"

#ifdef GSI_USB
#include "gti.h"
#include "gti_mem.h"
#include "gtiextn.h"
#endif /* GSI_USB */

#define AUTOMODE_DISABLE                       0
#define AUTOMODE_ENABLE                        1

#define RETRAIN_DISABLE                        0
#define RETRAIN_ENABLE                         1

#define SELECT_STANDARD_DISABLE                0
#define SELECT_STANDARD_ENABLE                 1

#define MAX_SELECT_TIME                        80 /* (1500ms * 80) = 120 Secs. */

#define DSP_RESTART_REQ                        0xFF

#define BINS_DOWN_COUNT                        256
#define BINS_UP_COUNT                          32

#if EEPROMv2
#define MAX_EEPROM_SIZE                        1024
#else
#define MAX_EEPROM_SIZE                        64
#endif//EEPROMv2

#define GSPN_MAX_DATA_SIZE              4088 /* data = 4096 - 8 header size */
#define GV_MAX_DATA_SIZE                GSPN_MAX_DATA_SIZE

#define DSP_MAX_STATUS_LEN                     0xFF

#define MAC_EEPROM_END_OFFSET                  13 //EEPROMv2 change: changed name from orig one:EEPROM_SIZE
#define EEPROM_SIZE_13                         13
#define ETH_LENGTH_OF_ADDRESS                  6

#ifdef QUASAR
#define PHY_RX_MAX_DATA_SIZE                   51
#else
#define PHY_RX_MAX_DATA_SIZE                   64
#endif//QUASAR

#define MAX_DOWN_RATE_TABLE_SIZE               6
#define USB_ISO_RATE_ALT0                      0x17
#define USB_ISO_RATE_ALT1                      0x1FFF
#define USB_ISO_RATE_ALT2                      0xC0
#define USB_ISO_RATE_ALT3                      0xA0
#define USB_ISO_RATE_ALT4                      0x60
#define USB_ISO_RATE_ALT5                      0x30
#define USB_ISO_RATE_DEFAULT                   USB_ISO_RATE_ALT1
#define USB_ALT_SETTINGS_INDEX_DEFAULT         0

#define USB_ISO_MODE                           0
#define USB_BULK_MODE                          1
#define MAX_ISO_PIPE_PACKET_SIZE               1023

#ifdef _NTDDK_
#define WIN2K_SURPRISE_REMOVAL_OK               1
#define WIN2K_SURPRISE_REMOVAL_NOT_OK           0
#endif // _NTDDK_

#define USBP_PORT_ENABLED	                    0x01
#define USBP_PORT_CONNECTED                     0x02

#define GTI_VERSION_SIZE            0x07
#define CHIP_ANCHOR					0
#define CHIP_QUASAR					1
#define CHIP_PANDA					2
#define CHIP_MONACO					3

#if	defined(linux)
// Defines for Type of Pipe aborting
#define	CONTROL_PIPE_ABORT_PROGRESS		0x01
#define	BULKTX_PIPE_ABORT_PROGRESS		0x02
#define	BULKRX_PIPE_ABORT_PROGRESS		0x04
#define	INT_PIPE_ABORT_PROGRESS			0x08
#define	ISOTX_PIPE_ABORT_PROGRESS		0x10
#define	ISORX_PIPE_ABORT_PROGRESS		0x20
#ifndef	SPINLOCK_DEFINED
#define	SPINLOCK_DEFINED	1
typedef	spinlock_t		KSPIN_LOCK;
typedef	urb_t	URB;
typedef	urb_t	*PURB;
typedef	unsigned long	KIRQL;
#endif	/* SPINLOCK_DEFINED */
//------------------------------------------------------------------------------
// DSP_THREAD_STATE
//        Defines the possible states that DSP Thread can take
//------------------------------------------------------------------------------
typedef enum 
{
    DSP_START,
    DSP_RUNNING,
	DSP_WAITING,
    DSP_STOP,
	DSP_STOPPED
} DSP_THREAD_STATE;

#endif	/* linux */

#ifdef _NTDDK_
//------------------------------------------------------------------------------
// OS_TYPE
//        Defines the possible OS versions
//------------------------------------------------------------------------------
typedef enum {
    OS_WIN_XP,    // Windows XP
    OS_WIN_98,    // Windows 98, First or Second Edition
    OS_WIN_2K,    // Windows 2000
    OS_WIN_ME,    // Windows Millenium
    OS_WIN_95,    // Windows 95
    OS_WIN_OTHER    
} OS_TYPE;


typedef enum
{
   ATM_TYPE,
   WAN_TYPE,
   LAN_TYPE,
} DRIVER_TYPE;

#endif //_NTDDK_
//------------------------------------------------------------------------------
//
// USB_PHY_STATE
//        Defines the possible states that a USB Phy can take on
//
//------------------------------------------------------------------------------
typedef enum {
    USB_PHY_STATE_OPERATIONAL,
    USB_PHY_STATE_SHUTDOWN,
    USB_PHY_STATE_RESETTING_PORT,
    USB_PHY_STATE_CONFIG,
    USB_PHY_STATE_RECONFIG,
    USB_PHY_STATE_SUSPEND,
    USB_PHY_STATE_RESUME,
    USB_PHY_STATE_HALTED
} USB_PHY_STATE;

#ifdef GSI_USB
//------------------------------------------------------------------------------
// DSP_STATE
//        Defines the possible states that a DSP can take on
//------------------------------------------------------------------------------
typedef enum {
   DSP_PHY_IDLE,
   DSP_PHY_TRAINING,
   DSP_PHY_CONNECTED,
   DSP_PHY_SPECTRUM_MODE,
   DSP_PHY_EMPTY,
   DSP_PHY_WAIT_CELLDEL,
   DSP_PHY_LOOPBACK_MODE,
   DSP_PHY_RESTART,
   DSP_PHY_NO_SIGNAL,
   DSP_PHY_DISCONNECT
} DSP_PHY_STATE;

typedef struct
{
   PURB     urbPtr;
#if defined(linux)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20)
   struct usb_ctrlrequest dr;
#else
   devrequest dr;
#endif
#endif // linux
} DSP_FW;
#endif /* GSI_USB */


/* This structure was moved for the ISO buffer scheme */
/* Need the reduce the field and need to slim down the ISO scheme */
//------------------------------------------------------------------------------
//
// USB_PHY_TRANSFER_RESOURCE
//        Contains all of the information and resources required to issue a
//        send or receive request to USBD
//
//------------------------------------------------------------------------------
typedef struct tUSB_PHY_TRANSFER_RESOURCE
{
    SINGLE_LIST_NODE    Link;
    PURB                Urb;
#ifdef _NTDDK_    
    PIRP                Irp;
   KEVENT         Done;
#endif //_NTDDK_
    PVOID               UsbPhy;
   ULONG          Frame;
#if defined(linux)   
    BOOLEAN		InProgress;
#endif //linux
   ULONG          tag;
    ATM_PHY_USER_HANDLE    UserContext;
    PXFER_DATA_BUFFER        Buffer;
} USB_PHY_TRANSFER_RESOURCE, *PUSB_PHY_TRANSFER_RESOURCE;


typedef struct tUSB_ISO_RECEIVE_INFO
{
#ifdef _NTDDK_
   LIST_ENTRY      listEntry;
   PDEVICE_OBJECT  DeviceObject;
   PIRP            Irp;
#endif //_NTDDK_
   PVOID           Context;
} USB_ISO_RECEIVE_INFO;

#if defined(linux)
typedef	int	USBD_PIPE_HANDLE;
typedef	struct	usb_interface_descriptor	USBD_INTERFACE_INFORMATION;
typedef	struct	usb_interface_descriptor	*PUSBD_INTERFACE_INFORMATION;
typedef	int64_t	LONGLONG;
#define	CDC_ESI_SIZE	7
#endif //linux

typedef  struct _CONTROLLER_PARAM
{
    UCHAR       chipType;    
    UCHAR       version[ GTI_VERSION_SIZE + 1 ];

} CONTROLLER_PARAM, *PCONTROLLER_PARAM;


//------------------------------------------------------------------------------
//
// USB_PHY
//        Describes an instance of the USB Phy
//
//------------------------------------------------------------------------------
typedef struct tUSB_PHY
{
    USB_PHY_STATE                    State;
#ifdef GSI_USB
    KSPIN_LOCK                        UsbStateSpinLock;        // protects USB states
    KSPIN_LOCK                        DspStateSpinLock;        // protects DSP states
    DSP_PHY_STATE                    dspState;                // DSP state
    LONGLONG                        LQAutoModeCount;        // forces restart after MAX_SELECT_TIME seconds
    UCHAR                            LQFailureCount;            // Triggers a DSL RESTART after LQFailureCount failure(s)
#endif /* GSI_USB */
    ATM_PHY_USER_HANDLE                UserHandle;
    ATM_PHY_USER_RECEIVE            UserReceive;
    ATM_PHY_USER_HANDLE             DslHandle;
    GTI_XCVR_INFO                   GTISysBlock[1];
    GTI_XCVR_INFO                   GTISysBlockCpl;

#ifdef GSI_USB
    ATM_PHY_USER_CPL                UserCPLGetATMStats;
    ATM_PHY_USER_CPL2               UserCPLClearATMStats;
#endif/* GSI_USB */
    ATM_PHY_USER_SEND_COMPLETE        UserSendComplete;
    ATM_PHY_USER_LINK_STATUS_CHANGE    UserLinkStatusChange;
#ifdef _NTDDK_
    PDEVICE_OBJECT                    TopOfStackDeviceObject;

    USBD_CONFIGURATION_HANDLE        ConfigurationHandle;
#elif defined(linux)
    adapter_t				*AdapterHandle;
#endif //_NTDDK
    UCHAR                            DataInterfaceNumber;
    UCHAR                            DataInterfaceOnAlternateSetting;
    UCHAR                            DataInterfaceOffAlternateSetting;

    USHORT                            LanguageId;

    USBD_PIPE_HANDLE                InterruptPipeHandle;
    USHORT                            InterruptPipePacketSize;

    USBD_PIPE_HANDLE                BulkReceivePipeHandle;
    USHORT                            BulkReceivePipePacketSize;

    USBD_PIPE_HANDLE                BulkTransmitPipeHandle;
    USHORT                            BulkTransmitPipePacketSize;

#ifdef _NTDDK_
    PUSB_PHY_TRANSFER_RESOURCE        BulkTransmitResource;
#endif //_NTDDK_
#if defined(linux)    
    SINGLE_LIST_NODE                BulkTransmitResourceList; // The Free Res List
    SINGLE_LIST_NODE                BulkTransmitUsedResourceList; // The Free Res List
    ULONG                            BulkTransmitResourceCount;
    KSPIN_LOCK                        BulkTransmitResourceListLock;
    SINGLE_LIST_NODE                BulkReceiveUsedResourceList;
	UCHAR							eocBuffer[MAX_INTERRUPT_PIPE_SIZE * 2];
	USHORT							eocDatalen;
#endif //linux
    atomic_t	                    BulkTransmitPending;

    SINGLE_LIST_NODE                BulkReceiveResourceList;
    ULONG                            BulkReceiveResourceCount;
    KSPIN_LOCK                        BulkReceiveResourceListLock;
    USHORT                            BulkReceiveBufferSize;

    atomic_t				IsoPendingReceive;
    USBD_PIPE_HANDLE                IsoReceivePipeHandle;
    UCHAR                            IsoReceivePipeEndpointAddress;
    USHORT                          IsoReceivePipePacketSize;
#ifdef _NTDDK_
    SINGLE_LIST_NODE                IsoReceiveResourceList;
#endif //_NTDDK_
    ULONG                            IsoReceiveResourceCount;
    USHORT                            IsoReceiveBufferSize;
    ULONG                            IsoReceivePipeTransferSize;
#ifdef _NTDDK_
    SINGLE_LIST_NODE                InterruptResourceList;
#elif defined(linux)
    PUSB_PHY_TRANSFER_RESOURCE        InterruptResource;
    USHORT                            InterruptInterval;
#endif //linux
    USHORT                            InterruptResourceCount;
    KSPIN_LOCK                        InterruptResourceListLock;
    USHORT                            InterruptBufferSize;
    atomic_t                            InterruptPendingCount;

#ifdef GSI_USB
   KSPIN_LOCK                        DspQueueFuncLock;
   KSPIN_LOCK                        DspLock;
#endif /* GSI_USB */

    UCHAR                            AtmEndSystemIdentifier[ CDC_ESI_SIZE];
    UCHAR                            AtmDeviceStatistics;

    ATM_PHY_LINK_STATE                LinkState;
    ULONG                            UpstreamRate;
    ULONG                            DownstreamRate;
    UCHAR                            RxCellPadding;

    atomic_t				PendingIoCount;

    UCHAR                            RetrainEnable;
    USHORT                          MaxCrcErrorsPerSecThreshold;
    USHORT                          DownstreamRateToMaxCrcErrorsPerMinDivider;

#ifdef GSI_USB
    struct PARAM_XCVR_ARRAY            xcvr[1];

    GTI_CHIP_INFO                     ChipInfo;

#ifdef GTI_CAP
    struct PARAM_XCVR_ARRAY            xcvrCap[1];
#endif /* GTI_CAP */
    DSP_FW                            dspFW;
    USHORT                            isIpuReady;
    USHORT                            getNextState;
    ULONG                            validCounter;
    UCHAR                            dspAction;
#endif /* GSI_USB */

   ULONG                          DslFailures[GTI_FAILURES_LEN];
   ULONG                          DslCounters[GTI_COUNTERS_LEN];

    /* Currently using all the fields below in the USBPHY structure
    ** but later use the proper name as before with bulk
    */
    PVOID                            TransferBuffer;
    ULONG                            TransferBufferLength;
    ULONG                            FramesPerBuffer;
    ULONG                            BufferCount;
    ULONG                            PacketSize;
    PUSB_PHY_TRANSFER_RESOURCE        TransferResource;
#ifdef _NTDDK_
    BOOLEAN                         isoPipeAborting;
#elif defined(linux)
    BOOLEAN                         PipeAborting;
#endif // linux
    UCHAR                            partialCellLen;
    UCHAR                            ReceiveDataSize;
    PUCHAR                            cellPtr;
    char                            lclSnrMargin;
    USHORT                            snrMarginPerBin[BINS_DOWN_COUNT];
    UCHAR                            bitLoadCoDwnCpUp[BINS_DOWN_COUNT];
    UCHAR                            bitLoadCoUpCpDwn[BINS_UP_COUNT];
    UCHAR                            StatusEx;
    UCHAR                            autoMode;
	CONTROLLER_PARAM                 ControllerParam;

#ifdef WINHIP
    LIST_ENTRY whipMessageList;
    KSPIN_LOCK whipLock;
#endif
    UCHAR                            ExpectedIsoPktTag;                  /* Next sequence number to be expected */
    PVOID                            IsoSeqPacket[MAX_RX_RESOURCE_COUNT];/* Pointer to an array which points to isoQbuff */
    USB_ISO_RECEIVE_INFO            isoQBuff[MAX_RX_RESOURCE_COUNT];    /* Descriptor keeps track of the URB packets received */

    UCHAR                            eeprom[MAX_EEPROM_SIZE];
    USHORT                            maxDownRate;
    UCHAR                            maxDownRateEnable;

#ifdef _NTDDK_
    OS_TYPE                            OsType;
#endif //_NTDDK_
    USHORT                            IsoMaxBWLimit;
    UCHAR                            ActualDsDataRate;
    UCHAR                            ActualIsoBwUsed; 
    UCHAR                            VendorId;
#ifdef _NTDDK_
    UCHAR                            Win2kSurpriseRemoval;
#endif //_NTDDK_

    UCHAR                            InPipeMode;
    BOOLEAN                            ShutdownAck;
    BOOLEAN                            CheckDslStatus;

    USHORT                            DspCounters[GTI_COUNTERS_LEN/sizeof(USHORT)];
    USHORT                            CrcErrorsPerMinThreshold;
    USHORT                            CrcErrorsPerMin;
    USHORT                            ExcessiveConsecutiveCrcErrorsPerTickCount;
    USHORT                            ExcessiveConsecutiveCrcErrorsPerMinCount;
    USHORT                            TickCounter;

    // Most significant four bytes of the EOC vendor ID read in from registry.
    ULONG                               EocVendorIdHigh;

    // Least significant four bytes of the EOC vendor ID read in from registry.
    ULONG                               EocVendorIdLow;
    
    USHORT                             SelectStandard;
    ULONG                              SelectStandardCounter;

#ifdef _NTDDK_
    BOOLEAN                            NeedQueueDspProcess;
    WORK_QUEUE_ITEM                    UsbProcessWorkItem;
#elif defined(linux)
DSP_THREAD_STATE                  NeedQueueDspProcess;
#endif //linux
    LIST_NODE                        DspProcessCommandList;
    LIST_NODE                        DspProcessFreeList;
    KSPIN_LOCK                        DspBufferListLock;

    USHORT                              DataDelay;
    UCHAR                               LowSuspendCurrent;
    
   UCHAR                               Cont8051Version[GTI_VERSION_SIZE];
   UCHAR                               BootcodeVersion[GTI_VERSION_SIZE];
   ULONG                               UsbVendorID;
   ULONG                               UsbProductID;
   ULONG                               UsbDeviceID;
#ifdef _NTDDK_
   DRIVER_TYPE                         drvType;
#endif //_NTDDK_
   UCHAR                               data[GSPN_MAX_DATA_SIZE];
   UCHAR                               IoCtlStatusBusy;
   UCHAR                               BitSwapCounterFlag;
   USHORT                            DspActualTrainingMode;
} USB_PHY, *PUSB_PHY;


//------------------------------------------------------------------------------
//
// Function Declarations
//
//------------------------------------------------------------------------------
ATM_PHY_STATUS
UsbPhyBind
(
    PATM_PHY_BIND_INPUT        BindInput,
    PATM_PHY_BIND_OUTPUT    BindOutput
);

void
UsbPhyUnbind
(
    PUSB_PHY UsbPhy
);

NTSTATUS
UsbPhyInitialize
(
    PUSB_PHY UsbPhy,
    BOOLEAN InitStartup
);

void UsbPhyShutdown( PUSB_PHY UsbPhy);
void UsbPhyCheckDspStatus( PUSB_PHY    UsbPhy);

#ifdef _NTDDK_
PUSB_DEVICE_DESCRIPTOR
UsbPhyGetDeviceDescriptor(
    PUSB_PHY    UsbPhy
);

void
UsbPhyFreeDeviceDescriptor( PUSB_DEVICE_DESCRIPTOR    DeviceDescriptor);

PUSB_CONFIGURATION_DESCRIPTOR
UsbPhyGetConfigurationDescriptor
(
    PUSB_PHY    UsbPhy
);

void
UsbPhyFreeConfigurationDescriptor(
    PUSB_CONFIGURATION_DESCRIPTOR    ConfigurationDescriptor
);

PUSB_STRING_DESCRIPTOR
UsbPhyGetStringDescriptor(
    PUSB_PHY    UsbPhy,
    UCHAR        Index,
    USHORT        LanguageId
);

void
UsbPhyFreeStringDescriptor( PUSB_STRING_DESCRIPTOR    StringDescriptor);

NTSTATUS
UsbPhyGetLanguageId(
    PUSB_PHY    UsbPhy,
    PUSHORT        LanguageId
);

PUSBD_INTERFACE_LIST_ENTRY
UsbPhyGetInterfaceList
(
    PUSB_PHY                 UsbPhy,
    PUSB_CONFIGURATION_DESCRIPTOR     ConfigurationDescriptor
);

void
UsbPhyFreeInterfaceList(
    PUSBD_INTERFACE_LIST_ENTRY    InterfaceList
);

PCDC_COMMON_FUNCTIONAL_DESCRIPTOR
UsbPhyParseConfigDescriptor(
    PUSB_CONFIGURATION_DESCRIPTOR    ConfigurationDescriptor,
    UCHAR                            DescriptorType,
    UCHAR                            DescriptorSubType,
    USHORT                            DescriptorSize
);

NTSTATUS
UsbPhyConfigureDevice(
    PUSB_PHY                UsbPhy,
    PUSB_CONFIGURATION_DESCRIPTOR    ConfigurationDescriptor,
    PUSBD_INTERFACE_LIST_ENTRY        InterfaceList
);

#elif defined(linux)
int
UsbPhyGetStringDescriptor(
    PUSB_PHY    UsbPhy,
    UCHAR        Index,
    USHORT        LanguageId,
	PWCHAR		pBuff,
	int			nBuffLen
);
#endif // linux

NTSTATUS
UsbPhyUnconfigureDevice( PUSB_PHY    UsbPhy);

NTSTATUS
UsbPhyConfigurePipes
(
    PUSB_PHY             UsbPhy,
    PUSBD_INTERFACE_INFORMATION    CommInterface,
    PUSBD_INTERFACE_INFORMATION    DataInterface
);

#ifdef _NTDDK_
NTSTATUS
UsbPhyCallUSBD
(
    PUSB_PHY    UsbPhy,
    PURB        Urb
);
#endif // _NTDDK_

NTSTATUS
UsbPhyResetPipe
(
    PUSB_PHY     UsbPhy,
    USBD_PIPE_HANDLE    PipeHandle

);

NTSTATUS
UsbPhyAbortPipe
(
    PUSB_PHY             UsbPhy,
    USBD_PIPE_HANDLE    PipeHandle

);

#ifdef _NTDDK_
NTSTATUS
UsbPhyGetAlternateSetting
(
    PUSB_PHY UsbPhy,
    UCHAR    InterfaceNumber,
    PUCHAR   AlternateSetting
);

NTSTATUS
UsbPhySetAlternateSetting(
    IN PUSB_PHY UsbPhy,
    IN UCHAR    InterfaceNumber,
    IN UCHAR    AlternateSetting
    );
#endif // _NTDDK_

ULONG
UsbPhyDecrementIoCount
(
    IN PUSB_PHY UsbPhy
);

ULONG
UsbPhyIncrementIoCount
(
    IN PUSB_PHY UsbPhy
);

ATM_PHY_STATUS
UsbPhySend
(
    PUSB_PHY            UsbPhy,
    PXFER_DATA_BUFFER    Buffer,
    ATM_PHY_USER_HANDLE    UserContext
);

#ifdef GSI_USB
NTSTATUS
UsbPhyStartBulk
(
   PUSB_PHY UsbPhy
);

void 
QueueUsbThread
( 
    PUSB_PHY UsbPhy
);

int 
UsbProcessFunction
( 
    PUSB_PHY UsbPhy
);

void 
AddDspCommand 
( 
    PUSB_PHY UsbPhy, 
    DSP_CONN_ACTION Action
);

void 
ProcessDspCommands
(
    PUSB_PHY UsbPhy, 
    DSP_CONN_ACTION dspAction
);

BOOLEAN 
ProcessUsbCommands 
(
    PUSB_PHY UsbPhy, 
    DSP_CONN_ACTION dspAction
);

NTSTATUS
UsbPhyStartXDSLDsp
(
   PUSB_PHY UsbPhy
);

BOOLEAN 
IsDspIdle 
(
    PUSB_PHY UsbPhy
);

BOOLEAN
DspNotConnected
(
    PUSB_PHY UsbPhy
);

DSP_PHY_STATE
GetDspState
(
    PUSB_PHY UsbPhy
);

void
SetDspState
(
    PUSB_PHY UsbPhy, 
    DSP_PHY_STATE NewDspState,
    DSP_CONN_ACTION Action
);

USB_PHY_STATE  
GetUsbState 
(
    PUSB_PHY UsbPhy
);

void 
SetUsbState 
(
    PUSB_PHY UsbPhy, 
    USB_PHY_STATE NewDspState
);


#ifdef EEPROM
void 
readMAC_Address
(
    PUSB_PHY                    UsbPhy,
    GTI_XCVR_INFO            *pXcvrInfo,
    PUCHAR                      PhyUniqueId
);

void
UsbPhyEEPROMAccess(
    PUSB_PHY                    UsbPhy,
    GTI_XCVR_INFO            *pXcvrInfo
   );
#endif // EEPROM

#endif /* GSI_USB */

#ifdef _NTDDK_
static NTSTATUS
UsbPhySendComplete
(
    IN PDEVICE_OBJECT                 DeviceObject,
    IN PIRP                         Irp,
    IN PUSB_PHY_TRANSFER_RESOURCE    SendResource
);
#elif defined(linux)
void
UsbPhySendComplete(PURB	Urb);
#endif // linux

void
UsbPhyInitializeBulkReceiveTransferResource
(
    PUSB_PHY                    UsbPhy,
    PUSB_PHY_TRANSFER_RESOURCE    TransferResource
);

#ifdef _NTDDK_
static NTSTATUS
UsbPhyBulkReceiveComplete
(
    PDEVICE_OBJECT     DeviceObject,
    PIRP             Irp,
    PVOID             Context
);
#elif defined(linux)
void
UsbPhyBulkReceiveComplete(PURB	Urb);
#endif // linux


void
FreeIsoResources
( 
    PUSB_PHY UsbPhy,
    PUSB_PHY_TRANSFER_RESOURCE TransferResource
);

#ifdef _NTDDK_
static NTSTATUS
UsbPhyIsoReceiveComplete
(
    PDEVICE_OBJECT     DeviceObject,
    PIRP             Irp,
    PVOID             Context
);
#elif defined(linux)
void
UsbPhyIsoReceiveComplete(PURB	Urb);
#endif // linux

void
UsbPhyUnicodeStringHexToBytes(
    PWCHAR    UnicodeString,
    PUCHAR    Bytes,
    ULONG    BytesToConvert
);

void
UsbPhyInitializeInterruptTransferResource
(
    PUSB_PHY                    UsbPhy,
    PUSB_PHY_TRANSFER_RESOURCE    TransferResource
);

#ifdef _NTDDK_
NTSTATUS
UsbPhyInterruptComplete
(
    PDEVICE_OBJECT     DeviceObject,
    PIRP             Irp,
    PVOID             Context
);
#elif defined(linux)
void
UsbPhyInterruptComplete(PURB	Urb);
#endif // linux

void
UsbPhyProcessNotification(
    PUSB_PHY        UsbPhy,
    PXFER_DATA_BUFFER    Buffer
);

void UsbPhyInitXcvr(PUSB_PHY UsbPhy);
void UsbPhyCpl2DisconDsp(PUSB_PHY UsbPhy);
void UsbPhyDslConnected(PUSB_PHY UsbPhy);
void UsbPhyDslDisconnected(PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyStartBulkTransmit( PUSB_PHY UsbPhy);

void
UsbPhyStopBulkTransmit( PUSB_PHY UsbPhy);


NTSTATUS
UsbPhyStartBulkReceive( PUSB_PHY UsbPhy);

void
UsbPhyStopBulkReceive( PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyBulkReceiveSubmitUrbs(PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyStartIsoReceive( PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyIsoSubmitUrbs(PUSB_PHY UsbPhy);

void
UsbPhyStopIsoReceive( PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyStartInterruptReceive( PUSB_PHY UsbPhy);

void
UsbPhyStopInterruptReceive( PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyIsoSubmitUrbs(PUSB_PHY UsbPhy);

NTSTATUS 
UsbPhyResetIsoPipe(PUSB_PHY UsbPhy);

NTSTATUS
UsbPhyInterruptSubmitUrbs(PUSB_PHY UsbPhy);

void 
UsbPhyBindDsl(PUSB_PHY UsbPhy, ATM_PHY_USER_HANDLE DslHandle);

#ifdef POWERMGMT
void SetSuspend (PUSB_PHY UsbPhy);
#endif

BOOLEAN IsDspThreadRunning (PUSB_PHY UsbPhy);
BOOLEAN IsTxThreadRunning (PUSB_PHY UsbPhy);
BOOLEAN IsNotOperational (PUSB_PHY UsbPhy);
void UsbPhyGetMacAddress (PUSB_PHY UsbPhy , PUCHAR Buffer );
void UsbPhyConfigureIsoBW (PUSB_PHY UsbPhy);
ULONG  UsbPhyGetStatusOfParentPort( PUSB_PHY UsbPhy, ULONG *status );

#ifdef QUASAR
NTSTATUS
UsbPhyDownLoadCode(PUSB_PHY UsbPhy , unsigned char *pFirmware, unsigned long ulSize);
#endif //QUASAR
void FreeIsoUrbFromQedPkts(PUSB_PHY UsbPhy);

#if defined(linux)
PUSB_PHY_TRANSFER_RESOURCE 
RemoveResourceFromList 
(
	SINGLE_LIST_NODE *ListHead, 
	PUSB_PHY_TRANSFER_RESOURCE SendResource,
	KSPIN_LOCK	*pListLock 
);
#endif // linux

#endif // USBPHY_H_
