/*+M*************************************************************************
 * GlobeSpan USB ADSL device driver for Linux.
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 *
 * History:
 *
 *---------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * ALTERNATIVELY, this driver may be distributed under the terms of
 * the following license, in which case the provisions of this license
 * are required INSTEAD OF the GNU General Public License. (This clause
 * is necessary due to a potential bad interaction between the GPL and
 * the restrictions contained in a BSD-style copyright.)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, and the entire permission notice in its entirety,
 *    including the disclaimer of warranties.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *-M*************************************************************************/

#ifdef MODULE
#include <linux/module.h>
#endif /* MODULE */

#include "Gp.h"
#include "rfc1483.h"
#include "rfc2364.h"
#include "GULinux.h"

/* Global Declarations */
adapter_t		*g_psAdapterList = NULL;

#ifdef	USB_STEP
int	g_break = 0;
#endif


/* Extern declerations */

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18))
# ifdef MODULE
#  define module_init(x)    int init_module(void) { return x(); }
#  define module_exit(x)    void cleanup_module(void) { x(); }
# else /* MODULE */
#  define module_init(x)    int x##_module(void) { return x(); }
#  define module_exit(x)    void x##_module(void) { x(); }
# endif /* MODULE */
#endif /* LINUX_VERSION_CODE */

/* Static function declerations */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0))
static void *GSModem_Probe (struct usb_device *usbdev, unsigned int ifnum,
                           const struct usb_device_id *id);
#else
static void *GSModem_Probe (struct usb_device *usbdev, unsigned int ifnum);
#endif /* LINUX_VERSION_COMPARE */
static void GSModem_Disconnect (struct usb_device *usbdev, void *ptr);



/*--------------------------------------------------------------------------*/
/* Parameters that can be set with 'insmod' */
#ifdef GSI_LAN

u8	  		  PVCCount 				= 1;
static u8     Rfc1483Mode           = RFC1483_DEFAULT_MODE;

/*
**	Lets set the default values for all PVCs that we are supporting.
*/
static u16    Rfc1483Vci1            = RFC1483_DEFAULT_VCI1;
static u16    Rfc1483Vci2            = RFC1483_DEFAULT_VCI2;
static u16    Rfc1483Vci3            = RFC1483_DEFAULT_VCI3;
static u16    Rfc1483Vci4            = RFC1483_DEFAULT_VCI4;

static u16    Rfc1483Vpi1            = RFC1483_DEFAULT_VPI1;
static u16    Rfc1483Vpi2            = RFC1483_DEFAULT_VPI2;
static u16    Rfc1483Vpi3            = RFC1483_DEFAULT_VPI3;
static u16    Rfc1483Vpi4            = RFC1483_DEFAULT_VPI4;

#endif // GSI_LAN
#ifdef GSI_WAN
static u8     Rfc2364Mode           = RFC2364_DEFAULT_MODE;

static u16    Rfc2364Vci            = RFC2364_DEFAULT_VCI;
static u16    Rfc2364Vpi            = RFC2364_DEFAULT_VPI;
#endif // GSI_WAN
static u8     Verbose               = FALSE;
static u16    RxBufferCount         = GSI_RX_BUFFER_COUNT;
static u16    TxBufferCount         = GSI_TX_BUFFER_COUNT;

static u16    GtiStandard           = GTI_MULTIMODE_STANDARD;
static u16    GtiTrellis            = GTI_TRELLIS_ENABLE;
static u16    GtiTxPowerAtten       = GTI_0DB_TX_PWR;
static u16    GtiCodingGain         = GTI_AUTO_CODING_GAIN;
static u16    GtiMaxBitsPerBin      = 0x0E;
static u16    GtiTxStartBin         = 0x06;
static u16    GtiTxEndBin           = 0x1F;
static u16    GtiRxStartBin         = 0x20;
static u16    GtiRxEndBin           = 0xFF;
static u16    GtiRxBinAdjust        = GTI_RX_BIN_DISABLE;
static u16    GtiFramingStruct      = GTI_FRAMING_TYPE_3;
static u16    GtiExpandedExchange   = GTI_EXPANDED_EXCHANGE;
#ifdef GSI_PULSAR 
static u16    GtiClockType          = GTI_CLOCK_CRYSTAL;
#else 
static u16    GtiClockType          = GTI_CLOCK_OSCILLATOR;
#endif
static u16    GtiMaxDownRate        = 8192;

#ifdef MODULE
MODULE_AUTHOR("GlobeSpan");
MODULE_DESCRIPTION("GlobeSpan USB ADSL LAN/WAN Device Driver");
#ifdef GSI_LAN
MODULE_PARM_DESC(Rfc1483Mode, "RFC1483 Mode");
MODULE_PARM(Rfc1483Mode, "b");
MODULE_PARM_DESC(Rfc1483Vci1, "RFC1483 VCI");
MODULE_PARM(Rfc1483Vci1, "h");
MODULE_PARM_DESC(Rfc1483Vci2, "RFC1483 VCI");
MODULE_PARM(Rfc1483Vci2, "h");
MODULE_PARM_DESC(Rfc1483Vci3, "RFC1483 VCI");
MODULE_PARM(Rfc1483Vci3, "h");
MODULE_PARM_DESC(Rfc1483Vci4, "RFC1483 VCI");
MODULE_PARM(Rfc1483Vci4, "h");

MODULE_PARM_DESC(Rfc1483Vpi1, "RFC1483 VPI");
MODULE_PARM(Rfc1483Vpi1, "h");
MODULE_PARM_DESC(Rfc1483Vpi2, "RFC1483 VPI");
MODULE_PARM(Rfc1483Vpi2, "h");
MODULE_PARM_DESC(Rfc1483Vpi3, "RFC1483 VPI");
MODULE_PARM(Rfc1483Vpi3, "h");
MODULE_PARM_DESC(Rfc1483Vpi4, "RFC1483 VPI");
MODULE_PARM(Rfc1483Vpi4, "h");

MODULE_PARM_DESC(PVCCOUNT, "Count for the number of PVCs");
MODULE_PARM(PVCCount, "h");

#endif // GSI_LAN
#ifdef GSI_WAN
MODULE_PARM_DESC(Rfc2364Mode, "RFC2364 Mode");
MODULE_PARM(Rfc2364Mode, "b");
MODULE_PARM_DESC(Rfc2364Vci, "RFC2364 VCI");
MODULE_PARM(Rfc2364Vci, "h");
MODULE_PARM_DESC(Rfc2364Vpi, "RFC2364 VPI");
MODULE_PARM(Rfc2364Vpi, "h");
#endif // GSI_WAN
MODULE_PARM_DESC(Verbose, "Verbose parameter reporting");
MODULE_PARM(Verbose, "b");
MODULE_PARM_DESC(TxBufferCount, "Number of ATM transmit cells");
MODULE_PARM(TxBufferCount, "h");
MODULE_PARM_DESC(RxBufferCount, "Number of ATM receive cells");
MODULE_PARM(RxBufferCount, "h");
MODULE_PARM_DESC(GtiStandard, "DSL standard");
MODULE_PARM(GtiStandard, "h");
MODULE_PARM_DESC(GtiTrellis, "Trellis mode");
MODULE_PARM(GtiTrellis, "h");
MODULE_PARM_DESC(GtiTxPowerAtten, "Transmit power attenuation");
MODULE_PARM(GtiTxPowerAtten, "h");
MODULE_PARM_DESC(GtiCodingGain, "Coding gain");
MODULE_PARM(GtiCodingGain, "h");
MODULE_PARM_DESC(GtiMaxBitsPerBin, "Maximum bits per bin");
MODULE_PARM(GtiMaxBitsPerBin, "h");
MODULE_PARM_DESC(GtiTxStartBin, "Transmit start bin");
MODULE_PARM(GtiTxStartBin, "h");
MODULE_PARM_DESC(GtiTxEndBin, "Transmit end bin");
MODULE_PARM(GtiTxEndBin, "h");
MODULE_PARM_DESC(GtiRxStartBin, "Receive start bin");
MODULE_PARM(GtiRxStartBin, "h");
MODULE_PARM_DESC(GtiRxEndBin, "Receive end bin");
MODULE_PARM(GtiRxEndBin, "h");
MODULE_PARM_DESC(GtiRxBinAdjust, "Receive bin adjust");
MODULE_PARM(GtiRxBinAdjust, "h");
MODULE_PARM_DESC(GtiFramingStruct, "Framing structure");
MODULE_PARM(GtiFramingStruct, "h");
MODULE_PARM_DESC(GtiExpandedExchange, "Expanded exchange");
MODULE_PARM(GtiExpandedExchange, "h");
MODULE_PARM_DESC(GtiClockType, "Clock type");
MODULE_PARM(GtiClockType, "h");
MODULE_PARM_DESC(GtiMaxDownRate, "Maximum downstream rate (kbps)");
MODULE_PARM(GtiMaxDownRate, "h");

#endif /* MODULE */

#if DBG
static u32    DebugFlags  = DBG_DEFAULTS;
# ifdef MODULE
MODULE_PARM_DESC(DebugFlags, "Debug Flags [<DBG_DEFAULTS>]");
MODULE_PARM(DebugFlags, "l");
# endif /* MODULE */
#endif /* DBG */

static const char version[] =
"GSPNModem " GSI_DRIVER_VERSION " " GSI_DRIVER_DATE;
static int enumerated = 0;

#define MAX(a,b) (((a)>(b))?(a):(b))

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0))
static void *GSModem_Probe (struct usb_device *usbdev, unsigned int ifnum,
                           const struct usb_device_id *id)
#else
static void *GSModem_Probe (struct usb_device *usbdev, unsigned int ifnum)
#endif /* LINUX_VERSION_COMPARE */
{
	int			ilanindex = 0;
	u8			u8NumConf;
	adapter_t	*pAdapter = NULL;
	
	DBG_FUNC("GSModem_Probe");

	if((PVCCount > MAX_PVC_SUPPORTED) || (PVCCount == 0))	
	{
		printk("Invalid PVC count configured by user, initialization failed \n");
		goto _EXIT;
	}
	
    DBG_ENTER(DbgInfo);
	
	// Proceed only if the driver is being probed for first time
	if( enumerated == 1)
		goto _EXIT;
	
	// Set enumerated to 1 to indicate that driver has been probed once
	enumerated = 1;
	
	u8NumConf = usbdev->descriptor.bNumConfigurations;

	if( u8NumConf < GSFW_NUM_CONFIG_SUPPORTED )
	{
		DBG_PRINT(DbgInfo,("GSModem_Probe Called with invalid Config\n"))
		usbdev->descriptor.bNumConfigurations = GSFW_NUM_CONFIG_SUPPORTED;
		if( usb_get_configuration(usbdev) < 0 )
		{
			DBG_WARNING( DbgInfo, ("Error getting device configurations"));
			goto _EXIT;;
		}
	}
#ifdef	CONFIG_USB_DEBUG
	usb_show_device(usbdev);
#endif	/* CONFIG_USB_DEBUG */

	/* Allocate memory for the Adapter list */
	pAdapter = (adapter_t *) kmalloc(sizeof(adapter_t), GFP_KERNEL);
	if( pAdapter == NULL )
	{
		DBG_WARNING( DbgInfo, ("Error in memory allocation"));
		goto _EXIT;
	}

	memset(pAdapter, 0, sizeof(*pAdapter));
	pAdapter->usbDev = usbdev;
	pAdapter->LinkState = ATM_PHY_LINK_STATE_DISCONNECTED;
	init_waitqueue_head( &(pAdapter->StatusCheckQ) );
	if( GSModemInitProtocols( pAdapter ) < 0 )
	{
		DBG_PRINT(DbgInfo,("GSModemInitProtocols Failed\n"))
		GSModem_Disconnect( usbdev, pAdapter );
		pAdapter = NULL;
		goto _EXIT;
	}
#ifdef GSI_LAN
	/*
	**	PVCCount is the number of PVCs that user wants to configure
	*/
	for(ilanindex = 0; ilanindex < PVCCount ; ilanindex++ )
	{
		if (GpLanAlloc(pAdapter, ilanindex))
		{
			DBG_WARNING(DbgInfo, ("Unable to allocate network device"));
			GSModem_Disconnect( usbdev, pAdapter );
			pAdapter = NULL;
			goto _EXIT;
		}
		pAdapter->lan[ilanindex].Rfc1483VcHandle = pAdapter->Rfc1483VcHandle[ilanindex];
		pAdapter->flags |= _LAN_ALLOC;
	} /* End of GpLanAlloc */

	/*
	**	PVCCount is the number of PVCs that user wants to configure
	*/
	for(ilanindex = 0; ilanindex < PVCCount ; ilanindex++ )
	{
		if (GpLanRegister(pAdapter, ilanindex))
		{
			DBG_WARNING(DbgInfo,("Unable to register adapter network device\n"));
			GSModem_Disconnect( usbdev, pAdapter );
			pAdapter = NULL;
			goto _EXIT;
		}
	} /* End of GpLanAlloc */
#endif // GSI_LAN

#ifdef GSI_WAN
	if (GpWanAlloc(pAdapter))
	{
		DBG_WARNING(DbgInfo,("Unable to allocate adapter network device\n"));
		GSModem_Disconnect( usbdev, pAdapter );
		pAdapter = NULL;
		goto _EXIT;
	}
	pAdapter->flags |= _WAN_ALLOC;

	if (GpWanRegister(pAdapter))
	{
		DBG_WARNING(DbgInfo, ("Unable to register adapter tty device\n"));
		GSModem_Disconnect( usbdev, pAdapter );
		pAdapter = NULL;
		goto _EXIT;
	}
#endif // GSI_WAN

	/* if this is First Adapter set the current adapter as head of global adapter list */
	pAdapter->flags |= _USB_BUS_CONNECTED;
	if( g_psAdapterList == NULL )
	{
		g_psAdapterList = pAdapter;
	}
	else
	{
		/* Add the Adapter to end of already existing Adapter List */
		adapter_t *pTmpAdapter = g_psAdapterList;
		while( pTmpAdapter->next )
			pTmpAdapter = pTmpAdapter->next;

		pTmpAdapter->next = pAdapter;
	}

_EXIT:
    DBG_LEAVE(DbgInfo);
	return pAdapter;
}


/*+F*************************************************************************
 * Function:
 *   GpQueryConfig
 *
 * Description:
 *   Query the current configuration for reporting through the IOCTL
 *   inteface.
 *-F*************************************************************************/
void
GpQueryConfig( adapter_t  *pAdapter )
{
    DBG_FUNC("GpQueryConfig")
    gsi_config_t   *pCfg;

#ifdef		PCI_ADAPTER
    pDslModem = pAdapter->custstruct;
#endif	/* PCI_ADAPTER */

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("(0x%p)", pAdapter));

    memset(&(pAdapter->version), 0, sizeof(pAdapter->version));

    pAdapter->version.protocol.major = GSI_PROTOCOL_VERSION_MAJOR;
    pAdapter->version.protocol.minor = GSI_PROTOCOL_VERSION_MINOR;

    DBG_ASSERT(sizeof(pAdapter->version.driver) >= sizeof(GSI_DRIVER_VERSION));
    memcpy(pAdapter->version.driver, GSI_DRIVER_VERSION,
        sizeof(GSI_DRIVER_VERSION));

	memcpy(pAdapter->version.firmware , RELEASE, GTI_FW_REL_LEN);

    pCfg = &(pAdapter->config);
    memset(pCfg, 0, sizeof(*pCfg));

#ifdef GSI_LAN
    pCfg->interface            |= GSI_INTERFACE_LAN;
    pCfg->rfc1483Mode           = Rfc1483Mode;

	/*
	**	Lets set the VCIs values.
	*/
	pCfg->rfc1483Vci[0]            = Rfc1483Vci1;
	pCfg->rfc1483Vci[1]            = Rfc1483Vci2;
	pCfg->rfc1483Vci[2]            = Rfc1483Vci3;
	pCfg->rfc1483Vci[3]            = Rfc1483Vci4;

	pCfg->rfc1483Vpi[0]            = Rfc1483Vpi1;
	pCfg->rfc1483Vpi[1]            = Rfc1483Vpi2;
	pCfg->rfc1483Vpi[2]            = Rfc1483Vpi3;
	pCfg->rfc1483Vpi[3]            = Rfc1483Vpi4;
    
#endif // GSI_LAN
#ifdef GSI_WAN
    pCfg->interface            |= GSI_INTERFACE_WAN;
    pCfg->rfc2364Mode           = Rfc2364Mode;

	/*
	**	Lets save the values.
	*/
	pCfg->rfc2364Vci            = Rfc2364Vci;
    pCfg->rfc2364Vpi            = Rfc2364Vpi;
#endif // GSI_WAN
    pCfg->rxBufferCount         = RxBufferCount;
    pCfg->txBufferCount         = TxBufferCount;

    pCfg->gtiStandard           = GtiStandard;
    pCfg->gtiTrellis            = GtiTrellis;
    pCfg->gtiTxPowerAtten       = GtiTxPowerAtten;
    pCfg->gtiCodingGain         = GtiCodingGain;
    pCfg->gtiMaxBitsPerBin      = GtiMaxBitsPerBin;
    pCfg->gtiTxStartBin         = GtiTxStartBin;
    pCfg->gtiTxEndBin           = GtiTxEndBin;
    pCfg->gtiRxStartBin         = GtiRxStartBin;
    pCfg->gtiRxEndBin           = GtiRxEndBin;
    pCfg->gtiRxBinAdjust        = GtiRxBinAdjust;
    pCfg->gtiFramingStruct      = GtiFramingStruct;
    pCfg->gtiExpandedExchange   = GtiExpandedExchange;
    pCfg->gtiClockType          = GtiClockType;
    pCfg->gtiMaxDownRate        = GtiMaxDownRate;

    DBG_LEAVE(DbgInfo);
}


#if (defined(GSI_LAN) && defined(GSI_WAN))
/*+F*************************************************************************
 * Function:
 *   GpLinkStatusChange
 *
 * Description:
 *   Adapter link status change routine. This routine is called when the
 *   low level DSP detects that the link status has changed.
 *-F*************************************************************************/
void
GpLinkStatusChange(
    adapter_t *   pAdapter,
    ATM_PHY_LINK_STATE  linkState,
    ULONG               upstreamRate,
    ULONG               downstreamRate)
{
    DBG_FUNC("gsi_link_status_change")
    NDIS_STATUS     ndisStatus;

    DBG_ENTER(DbgInfo);

	DBG_PRINT(DbgInfo,("Link Status Changing from %d to %d\n",pAdapter->LinkState,linkState));

    pAdapter->UpstreamRate   = upstreamRate;
    pAdapter->DownstreamRate = downstreamRate;

    // If the link state has changed, indicate the new link state to NDIS
    if (linkState != pAdapter->LinkState)
    {
        pAdapter->LinkState = linkState;

        if (ATM_PHY_LINK_STATE_CONNECTED == linkState)
        {
            ndisStatus = NDIS_STATUS_MEDIA_CONNECT;
        }
        else
        {
            ndisStatus = NDIS_STATUS_MEDIA_DISCONNECT;
        }

        NdisMIndicateStatus(
                           pAdapter->MiniportAdapterHandle,
                           ndisStatus,
                           NULL,
                           0);
        NdisMIndicateStatusComplete(pAdapter->MiniportAdapterHandle);
    }

    DBG_LEAVE(DbgInfo);
}
#endif // (GSI_LAN && GSI_WAN)

/*+F*************************************************************************
 * Function:
 *   GSModemInitProtocols
 *
 * Description:
 *   Initialize the adapter software interfaces.
 *-F*************************************************************************/
int
GSModemInitProtocols( adapter_t  *pAdapter )
{
	DBG_FUNC("GSModemInitProtocols")
	gsi_config_t   *pCfg;
	int     status = 0;
	int     ilanindex = 0;

	DBG_ENTER(DbgInfo);
	DBG_PARAMF(DbgInfo, "pAdapter", ("(0x%p)", pAdapter));
	if (!AtmUniInitialize())
	{
		DBG_WARNING(DbgInfo, ("AtmUniInitialize failed!\n"));
		status = -ENODEV;
		goto gsi_adapter_init_exit;
	}

	/* Query the driver configuration */
	GpQueryConfig(pAdapter);
	pCfg = &(pAdapter->config);

	pAdapter->MiniportAdapterHandle = pAdapter;
    pAdapter->NeedQueueRcvProcess = TRUE;
	INITIALIZE_LIST_HEAD( &(pAdapter->PendingReceiveList));
	NdisAllocateSpinLock( &(pAdapter->PendingReceiveListLock));
	NdisMInitializeTimer(
		&(pAdapter->PendingReceiveTimer),
		pAdapter,
		AtmUsbPendingReceiveTimerCallback,
		pAdapter);

	NdisInitializeEvent( &pAdapter->PendingReceiveTimerEvent);
    NdisSetEvent( &pAdapter->PendingReceiveTimerEvent);


#ifdef GSI_LAN
	pAdapter->Rfc1483Mode     = pCfg->rfc1483Mode;

	for(ilanindex=0; ilanindex< MAX_PVC_SUPPORTED; ilanindex++)
	{
		pAdapter->Rfc1483Vci[ilanindex] = pCfg->rfc1483Vci[ilanindex];
		pAdapter->Rfc1483Vpi[ilanindex] = pCfg->rfc1483Vpi[ilanindex];
	}

#endif // GSI_LAN
#ifdef GSI_WAN
	pAdapter->Rfc2364Mode     = pCfg->rfc2364Mode;

	pAdapter->Rfc2364Vci      = pCfg->rfc2364Vci;
	pAdapter->Rfc2364Vpi      = pCfg->rfc2364Vpi;
#endif // GSI_WAN
	pAdapter->RxBufferCount   = pCfg->rxBufferCount;
	pAdapter->TxBufferCount   = pCfg->txBufferCount;

	if (Verbose)
	{
		printk(KERN_INFO "GSI Parameters:\n");
#ifdef GSI_LAN
		printk(KERN_INFO "  RFC1483 Mode:        %d\n", pCfg->rfc1483Mode);
		printk(KERN_INFO "  RFC1483 VCI:         %d\n", pCfg->rfc1483Vci);
		printk(KERN_INFO "  RFC1483 VPI:         %d\n", pCfg->rfc1483Vpi);
#endif // GSI_LAN                                                            
#ifdef GSI_WAN                                                               
		printk(KERN_INFO "  RFC2364 Mode:        %d\n", pCfg->rfc2364Mode);
		printk(KERN_INFO "  RFC2364 VCI:         %d\n", pCfg->rfc2364Vci);
		printk(KERN_INFO "  RFC2364 VPI:         %d\n", pCfg->rfc2364Vpi);
#endif // GSI_WAN                                                            
		printk(KERN_INFO "  Rx Buffer Count:     %d\n", pCfg->rxBufferCount);
		printk(KERN_INFO "  Tx Buffer Count:     %d\n", pCfg->txBufferCount);
		printk(KERN_INFO "  GtiStandard:         0x%04x\n", pCfg->gtiStandard);
		printk(KERN_INFO "  GtiTrellis:          0x%04x\n", pCfg->gtiTrellis);
		printk(KERN_INFO "  GtiTxPowerAtten:     0x%04x\n", pCfg->gtiTxPowerAtten);
		printk(KERN_INFO "  GtiCodingGain:       0x%04x\n", pCfg->gtiCodingGain);
		printk(KERN_INFO "  GtiMaxBitsPerBin:    0x%04x\n", pCfg->gtiMaxBitsPerBin);
		printk(KERN_INFO "  GtiTxStartBin:       0x%04x\n", pCfg->gtiTxStartBin);
		printk(KERN_INFO "  GtiTxEndBin:         0x%04x\n", pCfg->gtiTxEndBin);
		printk(KERN_INFO "  GtiRxStartBin:       0x%04x\n", pCfg->gtiRxStartBin);
		printk(KERN_INFO "  GtiRxEndBin:         0x%04x\n", pCfg->gtiRxEndBin);
		printk(KERN_INFO "  GtiRxBinAdjust:      0x%04x\n", pCfg->gtiRxBinAdjust);
		printk(KERN_INFO "  GtiFramingStruct:    0x%04x\n", pCfg->gtiFramingStruct);
		printk(KERN_INFO "  GtiExpandedExchange: 0x%04x\n", pCfg->gtiExpandedExchange);
		printk(KERN_INFO "  GtiClockType:        0x%04x\n", pCfg->gtiClockType);
		printk(KERN_INFO "  GtiMaxDownRate:      %d kbps\n", pCfg->gtiMaxDownRate);
	}

#if (defined(GSI_LAN) && defined(GSI_WAN))
	pAdapter->AtmUniHandle = AtmUniCreate(
					pAdapter,
					(ATM_UNI_USER_LINK_STATUS_CHANGE) GpLinkStatusChange,
					FALSE,
					0,
					MAX(RFC1483_MAX_SDU_SIZE, RFC2364_MAX_SDU_SIZE),
					pAdapter->RxBufferCount,
					pAdapter->TxBufferCount,
					pAdapter->MiniportAdapterHandle,
					UsbPhyBind,
					pAdapter,
					pAdapter->UniqueId,
					&(pAdapter->UsbPhyHandle));
	if (pAdapter->AtmUniHandle == NULL)
	{
		DBG_WARNING(DbgInfo, ("AtmUniCreate failed!\n"));
		status = -ENODEV;
		goto gsi_adapter_init_exit;
	}

	pAdapter->flags |= _ATMUNI_ALLOC;

#endif // (GSI_LAN && GSI_WAN)

	DBG_PRINT(DbgInfo,("pAdapter->AtmUniHandle %p\n",pAdapter->AtmUniHandle));

#ifdef GSI_LAN
	if (!Rfc1483Initialize(pAdapter, UsbPhyBind, pAdapter))
	{
		DBG_WARNING(DbgInfo, ("Rfc1483Initialize failed!\n"));
		status = -ENODEV;
		goto gsi_adapter_init_exit;
	}
	pAdapter->flags |= _RFC1483_ALLOC;

	if (!Rfc1577Initialize(pAdapter))
	{
		DBG_WARNING(DbgInfo, ("Rfc1577Init failed!\n"));
		status = -ENODEV;
		goto gsi_adapter_init_exit;
	}
	pAdapter->flags |= _RFC1577_ALLOC;

#endif // GSI_LAN

#ifdef GSI_WAN
	if (!Rfc2364Initialize(pAdapter, UsbPhyBind, pAdapter))
	{
		DBG_WARNING(DbgInfo, ("Rfc2364Initialize failed!\n"));
		status = -ENODEV;
		goto gsi_adapter_init_exit;
	}
	pAdapter->flags |= _RFC2364_ALLOC;

	DBG_PRINT(DbgInfo, ("pAdapter->wan.channel[GSI_WAN_DEF_CHANNEL] in GpAdapterInit %4p\n", &(pAdapter->wan.channel[GSI_WAN_DEF_CHANNEL])));
#endif // GSI_WAN

gsi_adapter_init_exit:
	DBG_RETURNF(DbgInfo, ("status %d", status));
	return status;
}


static void GSModem_Disconnect (struct usb_device *usbdev, void *pDrvPtr)
{
	adapter_t	*pAdapter = (adapter_t *)pDrvPtr;
	int			ilanindex = 0;

	DBG_FUNC("GSModem_Disconnect ");
	DBG_ENTER( DbgInfo );

	pAdapter->flags &= ~(_USB_BUS_CONNECTED);

#ifdef	GSI_WAN

	if (pAdapter->flags & _WAN_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** WAN Shutdown \n"));
		GpWanUnregister(pAdapter);
		GpWanFree(pAdapter);
		pAdapter->flags &= ~_WAN_ALLOC;
	}

	if (pAdapter->flags & _RFC2364_VC_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** RFC2364 VC Delete\n"));
	    Rfc2364DeleteVc(pAdapter, 
				pAdapter->Rfc2364Vpi, pAdapter->Rfc2364Vci);
	    pAdapter->flags &= ~_RFC2364_VC_ALLOC;
	}

	if (pAdapter->flags & _RFC2364_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** RFC2364 Shutdown\n"));
		Rfc2364Shutdown(pAdapter);
		pAdapter->flags &= ~_RFC2364_ALLOC;
	}

#endif // GSI_WAN

#ifdef GSI_LAN
	if (pAdapter->flags & _LAN_ALLOC)
	{
		for(ilanindex = 0; ilanindex < PVCCount ; ilanindex++ )
		{
			GpLanUnregister(pAdapter, ilanindex);
			GpLanFree(pAdapter, ilanindex );
		}/* End of GpLanFree loop */
		pAdapter->flags &= ~_LAN_ALLOC;
	}
	if (pAdapter->flags & _RFC1577_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** RFC1577 Shutdown\n"));
		
		if( pAdapter->Rfc1577Config.Rfc1577Close )
		{
			pAdapter->Rfc1577Config.Rfc1577Close(
					pAdapter->Rfc1577Config.Rfc1577Context);
		}
		else
		{
			DBG_PRINT(DbgInfo, ("**** RFC1577 Close is NULL\n"));
		}

		pAdapter->Rfc1577Config.Rfc1577Context = NULL;
		pAdapter->flags &= ~_RFC1577_ALLOC;
	}

	if (pAdapter->flags & _RFC1483_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** RFC1483 Shutdown\n"));
		Rfc1483Shutdown(pAdapter);
		pAdapter->flags &= ~_RFC1483_ALLOC;
	}
#if (defined(GSI_LAN) && defined(GSI_WAN))
	if (pAdapter->flags & _ATMUNI_ALLOC)
	{
		DBG_PRINT(DbgInfo, ("**** AtmUni Shutdown\n"));
		AtmUniDestroy(pAdapter->AtmUniHandle);
		pAdapter->flags &= ~_ATMUNI_ALLOC;
	}
#endif // (GSI_LAN && GSI_WAN)

#endif // GSI_LAN

	DBG_PRINT(DbgInfo, ("**** Memory Release for pAdapter structure %p\n",pDrvPtr));    
    printk(KERN_INFO MTAG": Globespan USB Modem Driver  Disconnected\n"); 
	kfree(pDrvPtr);

	// Set enumerated back to zero to indicate that driver can reload
	enumerated = 0;
	DBG_RETURNF( DbgInfo, ("Disconnected"));
}


struct usb_device_id GSModem_ids[] =  
{
	{USB_DEVICE(GS_VENDID_MODEM, GS_PRODID_MODEM) },
	{}	/* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, GSModem_ids);

static struct usb_driver GSModemDriver =
{
	name:		"GSPN_Modem",
	probe:		GSModem_Probe,
	disconnect:	GSModem_Disconnect,
	id_table:		GSModem_ids,
};


static int __init GSModem_init(void)
{
	DBG_FUNC("GSModem_init");
	/* First Activity should be to initialize the Debugging subsystem */
	DBG_DEF_SYSTEM("USB:", DBG_USB_SYS);
	DBG_DEF_SYSTEM("DSP:", DBG_DSP_SYS);
	DBG_DEF_SYSTEM("EEPROM:", DBG_EEPROM_SYS);
	DBG_DEF_SYSTEM("NDIS:", DBG_NDIS_SYS);
	DBG_DEF_SYSTEM("PHY:", DBG_PHY_SYS);
	DBG_DEF_SYSTEM("LAN:", DBG_LAN_SYS);
	DBG_DEF_SYSTEM("WAN:", DBG_WAN_SYS);

	DBG_INIT(DbgInfo, MTAG, 0, DebugFlags, DBG_SYSTEMS);

#if	DBG
	DBG_SYSTEM(DbgInfo) &= ~( DBG_IOCTL_SYS |DBG_ISR_SYS );
	DBG_FLAGS(DbgInfo) |= ( DBG_TRACE_ON |DBG_PARAM_ON );
#endif

	DbgSetDbgPrintLevel(DBG_PRINT_LEVEL_ALL | DBG_PRINT_LEVEL_TRACE);
	DbgSetDbgPrintModule(DBG_PRINT_MODULE_ALL | DBG_PRINT_MODULE_RFC1577_ALL);		
	//DbgSetDbgPrintModule(
	//	DbgGetDbgPrintLevel() & ~(DBG_PRINT_MODULE_USB_ALL));		

	DBG_ENTER(DbgInfo);

	if (usb_register(&GSModemDriver))
	{
		DBG_WARNING( DbgInfo, ("GSPN Modem Driver registration Failed"));
		return -1;
	}

    printk(KERN_INFO MTAG": USB Modem Driver Build (%s-%s) is Registered\n",
        __DATE__, __TIME__); 

	DBG_RETURNF(DbgInfo, ("Modem Driver Registered\n"));
	return 0;
}

static void __exit GSModem_cleanup (void)
{
	DBG_FUNC("GSModem_cleanup")
	DBG_ENTER(DbgInfo);

	/* Deregister From the USB Subsystem */
	usb_deregister (&GSModemDriver);
	
    printk(KERN_INFO MTAG": Globespan USB Modem Driver  DeRegistered\n"); 

	DBG_LEAVE(DbgInfo);

	DBG_HALT(DbgInfo);
}


EXPORT_NO_SYMBOLS;

module_init (GSModem_init);
module_exit (GSModem_cleanup);
