/*+M*************************************************************************
 * TriplePoint Debug Interface Definition
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *
 * This source code may be used by anyone provided the TriplePoint copyright
 * is maintained in any derivitive works.
 *-M*************************************************************************/
/****************************************************************************
@doc INTERNAL TpiDebug TpiDebug_c

@module TpiDebug.c |

    This module implements a set of routines to support the debugging
    features provided by using the macros defined in <f TpiDebug\.h>.

@head3 Contents |
@index class,mfunc,func,msg,mdata,struct,enum | TpiDebug_c

@end
 ****************************************************************************/

#include "Gp.h"
#include "GpDebug.h"

#if DBG

#if (DBG_ENABLE_FILE || DBG_ENABLE_MESSAGE)
# if defined(WIN32)
#  include <windows.h>
#  include <winuser.h>
# elif defined(macintosh)
#  pragma error("WARNING: You need to define DBG_FILE or DBG_MESSAGE for MacOS!")
# elif !defined(_SYS_)
#  include <stdio.h>
# endif
#endif

#if defined(_VXD_)
# include <basedef.h>
# include <vmm.h>
#endif

#if defined(macintosh)
# include <stdarg.h>
# include <Memory.h>
# include <Traps.h>
#endif

#if defined(linux)
# if defined(__KERNEL__)
#  include <linux/kernel.h>
# else /* __KERNEL__ */
#  include <stdio.h>
# endif /* __KERNEL__ */
# include <stdarg.h>
#endif
#if (!(defined(linux)) && (!(defined(__KERNEL__))))
# include "drvh/GpUtil.h"
#endif


#define	tpi_vsprintf	vsprintf
#define	tpi_sprintf	sprintf

#if defined(UNDER_CE)
# if UNDER_CE < 300
    extern void __cdecl NKDbgPrintfW(LPWSTR, ...);
# else
    extern void __cdecl NKDbgPrintfW(LPCWSTR, ...);
# endif
#endif

int DbgSystemIndex(
    IN unsigned long    mask
    );

#define DBG_GLOBAL_DEFAULTS(S) {\
    DBG_DEFAULTS,\
    0,\
    S,\
    DBG_ALL_SYS,\
    0,\
    0,\
    DbgPrintOut,\
    DbgMessageOut,\
    DBG_MESSAGES\
}

#if defined(_VXD_)
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("VXD");
#elif defined(_EXE_)
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("EXE");
#elif defined(_DLL_)
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("DLL");
#elif defined(_SYS_)
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("SYS");
#elif defined(macintosh)
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("MAC");
#else
DBG_STRUCT    gDbgSettings = DBG_GLOBAL_DEFAULTS("TPI");
#endif

/* Always a global debug context */
PDBG_STRUCT   gDbgInfo = &gDbgSettings;

#if (DBG_ENABLE_FILE)

typedef struct DBG_OPEN_FILES
{
    void *       Handle;
    char *       Name;
    int          RefCount;
}   DBG_OPEN_FILES, * PDBG_OPEN_FILES;

#define MAX_OPEN_FILES  8
static  DBG_OPEN_FILES  ExistingFiles[MAX_OPEN_FILES] = {{0}};

#endif /* DBG_ENABLE_FILE */

void DbgOpenFile(
    IN PDBG_STRUCT      pDbgInfo
    )
{
#if (DBG_ENABLE_FILE)

    unsigned long       n;
    static char         FileName[128] = {0};
#if defined(UNICODE)
    static short        FileNameW[128] = {0};
#endif

    if ((DBG_FLAGS(pDbgInfo) & DBG_FILE_ON) && DBG_LPARAM(pDbgInfo) == 0)
    {
        unsigned long   avail = MAX_OPEN_FILES;

        /* Look for an available entry or find an existing file */
        for (n = 0; n < MAX_OPEN_FILES; n++)
        {
            if (ExistingFiles[n].RefCount &&
                Gp_strcmp(ExistingFiles[n].Name, DBG_NAME(pDbgInfo)) == 0)
            {
                DBG_LPARAM(pDbgInfo) = ExistingFiles[n].Handle;
                ExistingFiles[n].RefCount++;
                return;
            }
            else if (ExistingFiles[n].RefCount == 0)
            {
                avail = n;
            }
        }
        if (avail < MAX_OPEN_FILES)
        {
            n = tpi_sprintf(FileName, DBG_FILE_NAME, DBG_NAME(pDbgInfo));
#if defined(UNICODE)
            /* Convert the ASCII string to wide string. */
            ++n;
            while (n)
            {
                FileNameW[n] = FileName[n];
                --n;
            }
            FileNameW[n] = FileName[n];

            /* If file doesn't already exist create it. */
            DBG_LPARAM(pDbgInfo) = CreateFile(FileNameW,
                                              GENERIC_WRITE,
                                              0,
                                              NULL,
                                              CREATE_NEW,
                                              FILE_ATTRIBUTE_NORMAL,
                                              NULL);
            if (DBG_LPARAM(pDbgInfo) != INVALID_HANDLE_VALUE)
            {
                CloseHandle(DBG_LPARAM(pDbgInfo));
            }

            /* Now try to delete the file so we know if it's already open. */
            if (DeleteFile(FileNameW))
            {
                /* First to open creates it as shared. */
                DBG_LPARAM(pDbgInfo) = CreateFile(FileNameW,
                                                  GENERIC_WRITE,
                                                  FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                  NULL,
                                                  CREATE_ALWAYS,
                                                  FILE_ATTRIBUTE_NORMAL,
                                                  NULL);
            }
            else
            {
                /* Subsequent opens just open existing. */
                DBG_LPARAM(pDbgInfo) = CreateFile(FileNameW,
                                                  GENERIC_WRITE,
                                                  FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                  NULL,
                                                  OPEN_EXISTING,
                                                  FILE_ATTRIBUTE_NORMAL,
                                                  NULL);
            }
#elif defined(WIN32)
            /* If file doesn't already exist create it. */
            DBG_LPARAM(pDbgInfo) = CreateFile(FileName,
                                              GENERIC_WRITE,
                                              0,
                                              NULL,
                                              CREATE_NEW,
                                              FILE_ATTRIBUTE_NORMAL,
                                              NULL);
            if (DBG_LPARAM(pDbgInfo) != INVALID_HANDLE_VALUE)
            {
                CloseHandle(DBG_LPARAM(pDbgInfo));
            }

            /* Now try to delete the file so we know if it's already open. */
            if (DeleteFile(FileName))
            {
                /* First to open creates it as shared. */
                DBG_LPARAM(pDbgInfo) = CreateFile(FileName,
                                                  GENERIC_WRITE,
                                                  FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                  NULL,
                                                  CREATE_ALWAYS,
                                                  FILE_ATTRIBUTE_NORMAL,
                                                  NULL);
            }
            else
            {
                /* Subsequent opens just open existing. */
                DBG_LPARAM(pDbgInfo) = CreateFile(FileName,
                                                  GENERIC_WRITE,
                                                  FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                  NULL,
                                                  OPEN_EXISTING,
                                                  FILE_ATTRIBUTE_NORMAL,
                                                  NULL);
            }
#elif defined(_STDIO_DEFINED)
            DBG_LPARAM(pDbgInfo) = fopen(FileName,"w");
#endif
            if (DBG_LPARAM(pDbgInfo))
            {
                ExistingFiles[avail].Handle = DBG_LPARAM(pDbgInfo);
                ExistingFiles[avail].Name = DBG_NAME(pDbgInfo);
                ExistingFiles[avail].RefCount++;
            }
        }
        else
        {
            DBG_LPARAM(pDbgInfo) = 0;
        }
    }
#endif /* DBG_ENABLE_FILE */
}

void DbgCloseFile(
    IN PDBG_STRUCT      pDbgInfo
    )
{
#if (DBG_ENABLE_FILE)
    unsigned long       n;

    if ((DBG_FLAGS(pDbgInfo) & DBG_FILE_ON) && DBG_LPARAM(pDbgInfo))
    {
        for (n = 0; n < MAX_OPEN_FILES; n++)
        {
            if (ExistingFiles[n].Handle == DBG_LPARAM(pDbgInfo))
            {
                if (ExistingFiles[n].RefCount)
                {
                    if (--ExistingFiles[n].RefCount == 0)
                    {
                        ExistingFiles[n].Handle = 0;
                        ExistingFiles[n].Name = 0;
#if   defined(WIN32)
                        CloseHandle(DBG_LPARAM(pDbgInfo));
#elif defined(_STDIO_DEFINED)
                        fclose(DBG_LPARAM(pDbgInfo));
#endif
                    }
                }
                DBG_LPARAM(pDbgInfo) = 0;
                break;
            }
        }
    }
#endif /* DBG_ENABLE_FILE */
}

void DbgInit(
    IN PDBG_STRUCT      pDbgInfo
    )
{
    DbgOpenFile(pDbgInfo);
}

void DbgHalt(
    IN PDBG_STRUCT      pDbgInfo
    )
{
    DbgCloseFile(pDbgInfo);
}

int DbgSystemIndex(
    IN unsigned long    mask
    )
{
    int                 n;

    for (n = 0; mask; n++)
    {
        mask >>= 1;
    }
    return (n);
}

void DbgSystemDefine(
    IN const char *     name,
    IN unsigned long    mask
    )
{
    gDbgInfo->Systems[DbgSystemIndex(mask)] = name;
}

const char * DbgSystemName(
    IN unsigned long    mask
    )
{
    const char *        name;
    name = gDbgInfo->Systems[DbgSystemIndex(mask)];
    if (!name)
    {
        name = "";
    }
    return (name);
}

/* Declare DbgPrint for those platforms it's not available with. */
#if defined(_VXD_)

#if !defined(NDIS_DOS)

unsigned long __cdecl DbgPrint(
    char *              format,
    ...
    )
{
    unsigned long       result;

    __asm lea  eax, (format + 4)
    __asm push eax
    __asm push format
    VMMCall(_Debug_Printf_Service)
    __asm add esp, 4*2
    __asm mov result, eax

    return (result);
}
#endif

#elif (defined(_EXE_) || defined(_DLL_)) && !defined(UNDER_CE)

unsigned long __cdecl DbgPrint(
    char *              format,
    ...
    )
{
    va_list             params;
    unsigned long       result;
    static char         buffer[DBG_BUFFER_LEN];

#if defined(WIN16)
    extern void _far _pascal OutputDebugString(char _far *);

    va_start(params, format);
    result = tpi_vsprintf(buffer, format, params);
    va_end(params);

    OutputDebugString(buffer);
#else
    va_start(params, format);
    result = tpi_vsprintf(buffer, format, params);
    va_end(params);

    OutputDebugString(buffer);
#endif

    if (result >= sizeof(buffer))
    {
        /* We just overwrote memory */
        OutputDebugString("1BUG BUG BUG - DbgPrint message too big - BUG BUG BUG\n");
        PROCESSOR_BREAKPOINT;
    }
    return (result);
}

#elif defined(macintosh)

char IsDbgPresent(void); /* Prototype required */
DBG_STATIC char DbgPresent = -1;    /* Initial value must < 0 */

/****************************************************************************
@doc INTERNAL Debug Debug_c IsDbgPresent

@func

    <f IsDbgPresent> checks to see whether a debugger is active.

@rdesc IsDbgPresent returns one of the following values:
    @flag 0 |
        If no debugger is installed.
    @flag 1 |
        If the debugger is hooked into the system.
 ****************************************************************************/
char IsDbgPresent(void)
{
    char        *pDebuggerFlag = (char *) 0x0BFF;   /* System global flag */

    if ((*pDebuggerFlag & 0x20) == 0)
    {
        return (0);
    }
    return (1);
}

void DbgBreakPoint(void)
{
    if (DbgPresent < 0)
    {
        DbgPresent = IsDbgPresent();
    }
    if (DbgPresent)
    {
        Debugger();
    }
}

unsigned long __cdecl DbgPrint(
    char *              format,
    ...
    )
{
    va_list             params;
    unsigned long       result;
    unsigned long       index;
    static char         buffers[4][DBG_BUFFER_LEN];
    static char         inUse = 0;
    char *              buffer;

    if (++inUse > 4)
    {
        DbgPrintMac("\pBUG BUG BUG - DbgPrint Nested - BUG BUG BUG");
        --inUse;
        return 0;
    }
    buffer = &buffers[inUse-1][0];

    if (DbgPresent < 0)
    {
        DbgPresent = IsDbgPresent();
    }

    va_start(params, format);
    result = tpi_vsprintf(&buffer[1], format, params);
    va_end(params);

    if (result >= (DBG_BUFFER_LEN-4))
    {
        /* We just overwrote memory */
        DbgPrintMac("\p2BUG BUG BUG - DbgPrint message too big - BUG BUG BUG");
    }

    /*
    // Don't allow any semi-colons in the output or MacBug will intepret it
    // as the start of a command.
    */
    for (index = 1; index < result; ++index)
    {
        if (buffer[index] == ';')
        {
            buffer[index] = ':';    /* Close enough... */
        }
    }
    /*
    // Remove the newline and tell MacsBug just to print and go.
    */
    if (buffer[result] == '\n') --result;
    buffer[1+result++] = ';';
    buffer[1+result++] = 'G';
    buffer[1+result] = 0;
    buffer[0] = result;

    if (DbgPresent)
    {
        DbgPrintMac((StringPtr) buffer);
    }

    --inUse;
    return (result);
}

#elif defined(linux)

unsigned long __cdecl DbgPrint(
    const char *        format,
    ...
    )
{
    va_list             params;
    unsigned long       length;
    static char         dbgHeader = 1;
    static char         buffer[DBG_BUFFER_LEN];

#if defined(__KERNEL__)
    va_start(params, format);
    length = tpi_vsprintf(buffer, format, params);
    va_end(params);

    if (length >= (DBG_BUFFER_LEN-4))
    {
        /* We just overwrote memory */
        printk(KERN_DEBUG "3BUG BUG BUG - DbgPrint message too big - BUG BUG BUG\n");
    }

    if (dbgHeader)
    {
        printk(KERN_DEBUG "%s", buffer);
    }
    else
    {
        printk("%s", buffer);
    }

    /* Only put on debug header after we have seen a newline */
    if (length > 0)
    {
        dbgHeader = (buffer[length - 1] == '\n') ? 1 : 0;
    }
#else /* __KERNEL__ */
    va_start(params, format);
    length = vsprintf(buffer, format, params);
    va_end(params);

    if (length >= (DBG_BUFFER_LEN-4))
    {
        /* We just overwrote memory */
        fprintf(stderr, "4BUG BUG BUG - DbgPrint message too big - BUG BUG BUG\n");
    }

    fprintf(stderr, "%s", buffer);
#endif /* __KERNEL__ */

    return (length);
}

#endif /* DbgPrint */

void DbgMessageOut(
    IN PDBG_STRUCT      pDbgInfo
    )
{
#if (DBG_ENABLE_MESSAGE)
    /* See if we should display a message box. */
    if ((DBG_FLAGS(pDbgInfo) & DBG_MESSAGE_ON) &&
        (DBG_ALT_FLAGS(pDbgInfo) & DBG_GET_MASK()))
    {
#if defined(UNICODE)
        unsigned long   n;

        /* Convert the ASCII string to wide string. */
        for (n = 0; n < gDbgInfo->BufferIndex; ++n)
        {
            gDbgInfo->BufferW[n] = gDbgInfo->Buffer[n];
        }
        gDbgInfo->BufferW[n] = 0;
        MessageBox(NULL, gDbgInfo->BufferW, TEXT("TpiDebug"), MB_OK);
#else
        MessageBox(NULL, gDbgInfo->Buffer, "TpiDebug", MB_OK);
#endif
    }
#endif /* DBG_ENABLE_MESSAGE */
}

void DbgPrintOut(
    IN PDBG_STRUCT      pDbgInfo
    )
{
#if (DBG_ENABLE_DEBUGGER)
#   if defined(UNDER_CE)
        NKDbgPrintfW(L"%s", gDbgInfo->Buffer);
#   else
        DbgPrint("%s", gDbgInfo->Buffer);
#   endif
#endif

#if (DBG_ENABLE_FILE)
    /* Write the output string to the debug file. */
    if ((DBG_FLAGS(pDbgInfo) & DBG_FILE_ON) && DBG_LPARAM(pDbgInfo))
    {
#if defined(WIN32)
        unsigned long   n = gDbgInfo->BufferIndex-1;
        unsigned long   ulPos;

        /* CR/NL at EOL when using binary WriteFile */
        if (gDbgInfo->Buffer[n] == '\n')
        {
            gDbgInfo->Buffer[n++] = '\r';
            gDbgInfo->Buffer[n]   = '\n';
            gDbgInfo->BufferIndex++;
        }
        /*
        // Make sure we're positioned at the end of the file.
        // This is necessary because the debug file may be opened by
        // multiple instances of the same application.
        */
        ulPos = SetFilePointer(DBG_LPARAM(pDbgInfo), 0, NULL, FILE_END);
        /* LockFile(DBG_LPARAM(pDbgInfo), ulPos, 0, ulPos + gDbgInfo->BufferIndex, 0); */
        WriteFile(DBG_LPARAM(pDbgInfo), gDbgInfo->Buffer, gDbgInfo->BufferIndex,
                  &n, NULL);
        /* UnlockFile(DBG_LPARAM(pDbgInfo), ulPos, 0, ulPos + n, 0); */
        FlushFileBuffers(DBG_LPARAM(pDbgInfo));
#elif defined(_STDIO_DEFINED)
        fprintf(DBG_LPARAM(pDbgInfo), "%s", gDbgInfo->Buffer);
#endif
    }
#endif /* DBG_ENABLE_FILE */
}

void __cdecl DbgPrintf(
    IN const char *     format,
    ...
    )
{
    va_list             params;
    unsigned long       result;

    /* Add this new output to end of the current buffer. */
    va_start(params, format);
#if defined(linux) && !defined(__KERNEL__)
    result = vsprintf((char *)&gDbgInfo->Buffer[gDbgInfo->BufferIndex],
                          format, params);
#else
    result = tpi_vsprintf((char *)&gDbgInfo->Buffer[gDbgInfo->BufferIndex],
                          format, params);
#endif
    va_end(params);
    gDbgInfo->BufferIndex += result;

    if (gDbgInfo->BufferIndex >= DBG_BUFFER_LEN-1)
    {
        /* We just overwrote memory */
#   if defined(UNDER_CE)
        NKDbgPrintfW(L"5BUG BUG BUG - DbgPrintf message too big - BUG BUG BUG\n");
#   else
        DbgPrint("6BUG BUG BUG - DbgPrintf message too big - BUG BUG BUG\n");
#   endif
        PROCESSOR_BREAKPOINT;
    }

    /*
    // Display the output buffer only when it ends with a newline or formfeed.
    // Otherwise we keep accumulating output.
    */
    if (gDbgInfo->Buffer[gDbgInfo->BufferIndex-1] == '\n' ||
        gDbgInfo->Buffer[gDbgInfo->BufferIndex-1] == '\f')
    {
        if (gDbgInfo->Buffer[gDbgInfo->BufferIndex-1] == '\f')
        {
            /* Formfeed character is used to force a flush of the buffer. */
            gDbgInfo->Buffer[--gDbgInfo->BufferIndex] = 0;
        }
        if (DBG_PRINT_FUNC(DBG_GET_INFO()))
            DBG_PRINT_FUNC(DBG_GET_INFO())(DBG_GET_INFO());

        if (DBG_ALT_FUNC(DBG_GET_INFO()))
            DBG_ALT_FUNC(DBG_GET_INFO())(DBG_GET_INFO());

        /* Start next line at beginning of buffer. */
        gDbgInfo->BufferIndex = 0;
    }
}


/* Print the given buffer in a debug format. */
void DbgPrintData(
    IN PDBG_STRUCT      pDbgInfo,
    IN unsigned char *  data,
    IN unsigned long    numBytes,
    IN unsigned long    offset
    )
{
    unsigned long       lineStart;
    unsigned long       lineIndex;

    /* Display the caller's buffer with up to 16 bytes per line. */
    for (lineStart = 0; lineStart < numBytes; lineStart += 16)
    {
        /* Display the starting offset of the line. */
        DbgPrintf("%04lX: ", lineStart + offset);

        /* Display a line of HEX byte values. */
        for (lineIndex = lineStart; lineIndex < (lineStart+16); lineIndex++)
        {
            if (lineIndex < numBytes)
            {
                DbgPrintf("%02X ",
                          (unsigned long)((unsigned char)*(data+lineIndex)));
            }
            else
            {
                DbgPrintf("   ");
            }
        }
        /* A little white space between HEX and ASCII. */
        DbgPrintf("  ");

        /* Display the corresponding ASCII byte values if they are printable. */
        // (i.e. 0x20 <= N <= 0x7F).
        for (lineIndex = lineStart; lineIndex < (lineStart+16); lineIndex++)
        {
            if (lineIndex < numBytes)
            {
                unsigned char c = *(data+lineIndex);

                if ((c < ' ') || (c > 'z'))
                {
                    c = '.';
                }
                DbgPrintf("%c", c);
            }
            else
            {
                DbgPrintf(" ");
            }
        }
        DBG_PRINT(pDbgInfo, ("\n"));     /* End of line. */
    }
}

/* Queue the debug data to the internal memory debug queue. */
void DbgQueueData(
    IN unsigned char *  data,
    IN unsigned long    numBytes,
    IN unsigned short   flags
    )
{
    DBG_LOG *           pLogEntry = &(gDbgInfo->LogBuffer[gDbgInfo->LogIndex++]);
    char *              bp;

    /* Wrap around on the next entry if needed. */
    if (gDbgInfo->LogIndex >= DBG_LOG_ENTRIES)
    {
        gDbgInfo->LogIndex = 0;
    }

    /* Don't try to save more than we have room for. */
    if (numBytes > DBG_LOG_SIZE)
    {
        numBytes = DBG_LOG_SIZE;
    }

    pLogEntry->Flags = flags;
    pLogEntry->NumBytes = (unsigned short) numBytes;
    bp = (char *)pLogEntry->Buffer;
    while (numBytes--)
    {
        *(bp++) = *(data++);
    }

    numBytes = DBG_LOG_SIZE - pLogEntry->NumBytes;
    while (numBytes--)
    {
        *(bp++) = '\0';
    }
}

/* Convert a block of data to a string */
char *
DbgDataToString(
    unsigned char  *pData,
    unsigned long   offset,
    unsigned long   length,
    char           *pBuffer
    )
{
    static char     Buffer[DBG_BUFFER_LEN]; /* Debug only! */
    unsigned char  *pD;
    char           *pB;
    unsigned long   x;

    /* If the user buffer was not supplied, use the internal buffer. */
    if (pBuffer == 0x0)
    {
        pBuffer = Buffer;
    }

    pB = pBuffer;

    if (length < 128)
    {
        pD = (pData + offset);
        for (x = 0; x < length; x++)
        {
            if ((pD[x] >= ' ') && (pD[x] <= 'z'))
            {
                *(pB++) = pD[x];
            }
            else
            {
                const char      digits[] = "0123456789ABCDEF";
                unsigned char   val = pD[x];

                *(pB++) = '[';
                *(pB++) = digits[(val >> 4) & 0x0F];
                *(pB++) = digits[val & 0x0F];
                *(pB++) = ']';
            }

            if (x >= (DBG_BUFFER_LEN - 12))
            {
                *(pB++) = '.';
                *(pB++) = '.';
                *(pB++) = '.';
                break;
            }
        }
    }
    else
    {
        *(pB++) = '.';
        *(pB++) = '.';
        *(pB++) = '.';
    }
    *(pB++) = '\0';

    /* Truncate lines that are too long. */
    if ((pB - pBuffer) > 512)
    {
        pBuffer[0] = '.';
        pBuffer[1] = '.';
        pBuffer[2] = '.';
        pBuffer[3] = '\0';
    }

    return pBuffer;
}

#endif /* DBG */

