/*+M*************************************************************************
 * TriplePoint NDIS Abstraction Layer
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *-M*************************************************************************/
#include "Gp.h"

#define RUN_AT(x) (jiffies + (x))

VOID GpDelay(ULONG delay )
{
	NdisStallExecution(delay);
	return;
}	       


VOID NdisStallExecution(IN ULONG Microsecs)
{
	udelay(Microsecs);
}

ULONG GpAllocateSpinLock(VOID **pXcvrLock) 
{ 
	PNDIS_SPIN_LOCK         SpinLock;
	
	SpinLock = (PNDIS_SPIN_LOCK)kmalloc(sizeof(NDIS_SPIN_LOCK), GFP_KERNEL);
	NdisAllocateSpinLock(SpinLock);
	 *pXcvrLock = (VOID *)SpinLock;
	return 0;
}


ULONG GpAcquireSpinLock(VOID *pXcvrLock) 
{ 
	NdisAcquireSpinLock((PNDIS_SPIN_LOCK)pXcvrLock);
	return 0;
}


ULONG GpReleaseSpinLock(VOID *pXcvrLock) 
{ 
	NdisReleaseSpinLock((PNDIS_SPIN_LOCK)pXcvrLock);
	return 0;
}

void  KeInitializeSpinLock( KSPIN_LOCK  *pLock  )
{
	spin_lock_init(pLock );
}

void	KeAcquireSpinLock( KSPIN_LOCK *pLock, KIRQL *pIrql)
{
	unsigned long	flags;
	spin_lock_irqsave(pLock, flags);
	*pIrql = flags;
}

void  KeReleaseSpinLock( KSPIN_LOCK  *pLock, KIRQL  Irql )
{
	spin_unlock_irqrestore( pLock, Irql );
}

ULONG GpFreeSpinLock(VOID *pXcvrLock) 
{ 
	NdisFreeSpinLock((PNDIS_SPIN_LOCK)pXcvrLock);
	kfree(pXcvrLock);
	return 0;
}




//
// NDIS Spin Locks
//
VOID NdisAllocateSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    )
{
    DBG_FUNCX("NdisAllocateSpinLock", DBG_NDIS_SYS)

#if defined(linux)
    DBG_UNUSED;
    spin_lock_init(&(SpinLock->slock));
#elif defined(MAC_OS)
    DBG_ASSERT(sizeof(MPS_INTR_STATE) <= sizeof(USHORT));
    SpinLock->slock = 0;
#endif /* OS */
}

VOID NdisFreeSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    )
{
    DBG_FUNCX("NdisFreeSpinLock", DBG_NDIS_SYS)

#if defined(linux)
    DBG_UNUSED;
#elif defined(MAC_OS)
    DBG_ASSERT(SpinLock->slock == 0);
#endif /* OS */
}

VOID NdisAcquireSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    )
{
    DBG_FUNCX("NdisAcquireSpinLock", DBG_NDIS_SYS)
    //DBG_ENTER(gDbgInfo);

#if defined(linux)
    DBG_UNUSED;
    spin_lock_irqsave(&(SpinLock->slock), SpinLock->flags);
#elif defined(MAC_OS)
    MPS_INTR_STATE              intrstate;
    mps_intr_disable(&intrstate);
    while (SpinLock->slock)
    {
        mps_intr_enable(&intrstate);
        SystemTask();
        mps_intr_disable(&intrstate);
    }
    SpinLock->slock += 1;
    DBG_ASSERT(SpinLock->slock == 1);
    mps_intr_enable(&intrstate);
#endif /* OS */
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisReleaseSpinLock(
    IN  PNDIS_SPIN_LOCK         SpinLock
    )
{
    DBG_FUNCX("NdisReleaseSpinLock", DBG_NDIS_SYS)
    //DBG_ENTER(gDbgInfo);

#if defined(linux)
    DBG_UNUSED;
    spin_unlock_irqrestore(&(SpinLock->slock), SpinLock->flags);
#elif defined(MAC_OS)
    MPS_INTR_STATE              intrstate;
    mps_intr_disable(&intrstate);
    SpinLock->slock -= 1;
    DBG_ASSERT(SpinLock->slock == 0);
    mps_intr_enable(&intrstate);
#endif /* OS */
    //DBG_LEAVE(gDbgInfo);
}

//
// NDIS Buffers
//
VOID NdisQueryBuffer(
    IN  PNDIS_BUFFER            Buffer,
    OUT PVOID *                 VirtualAddress OPTIONAL,
    OUT PUINT                   Length
    )
{
    DBG_FUNCX("NdisQueryBuffer", DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(Buffer && Buffer->Tag == NDIS_BUFFER_TAG);
    DBG_ASSERT(Length);

    if (ARGUMENT_PRESENT(VirtualAddress))
    {
        *((PVOID *)VirtualAddress) = Buffer->Address+Buffer->ByteOffset;
    }
    *Length = Buffer->ByteCount;
}

VOID NdisGetNextBuffer(
    IN  PNDIS_BUFFER            CurrentBuffer,
    OUT PNDIS_BUFFER *          NextBuffer
    )
{
    DBG_FUNCX("NdisGetNextBuffer", DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(CurrentBuffer && CurrentBuffer->Tag == NDIS_BUFFER_TAG);
    DBG_ASSERT(NextBuffer);

    if (NextBuffer && CurrentBuffer)
    {
        *NextBuffer = CurrentBuffer->Next;
    }
}

VOID NdisQueryBufferOffset(
    IN  PNDIS_BUFFER            Buffer,
    OUT PUINT                   Offset,
    OUT PUINT                   Length
    )
{
    DBG_FUNCX("NdisQueryBufferOffset", DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(Buffer);

    *Offset = Buffer->ByteOffset;
    *Length = Buffer->ByteCount;
}

VOID NdisAdjustBufferOffset(
    IN  PNDIS_BUFFER            Buffer,
    IN  UINT                    Offset
    )
{
    DBG_FUNCX("NdisAdjustBufferOffset",DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(Buffer);

    Buffer->ByteOffset = Offset;

    DBG_ASSERT(Buffer->ByteOffset <= Buffer->Size);
}

VOID NdisQueryBufferSize(
    IN  PNDIS_BUFFER            Buffer,
    OUT PUINT                   Size
    )
{
    DBG_FUNCX("NdisQueryBufferSize",DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(Buffer);

    *Size = Buffer->Size;
}

VOID NdisAdjustBufferLength(
    IN  PNDIS_BUFFER            Buffer,
    IN  UINT                    Length
    )
{
    DBG_FUNCX("NdisAdjustBufferLength", DBG_NDIS_SYS)

    DBG_UNUSED;
    DBG_ASSERT(Buffer);

    Buffer->ByteCount = Length;
}

VOID NdisAllocateBufferPool(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            PoolHandle,
    IN  UINT                    NumberOfDescriptors
    )
{
    DBG_FUNCX("NdisAllocateBufferPool", DBG_NDIS_SYS)

    PNDIS_BUFFER_POOL           Pool;
    ULONG                       PoolSize;
    ULONG                       DescriptorSize;
    PNDIS_BUFFER                CurrBuffer;
    PNDIS_BUFFER                NextBuffer;

    //DBG_ENTER(gDbgInfo);
    //DBG_ASSERT(Status);
    //DBG_ASSERT(PoolHandle);
    //DBG_ASSERT(NumberOfDescriptors);

    DescriptorSize = sizeof(NDIS_BUFFER);
    PoolSize = sizeof(NDIS_BUFFER_POOL) + NumberOfDescriptors * DescriptorSize;
    Pool = kmalloc(PoolSize, GFP_KERNEL);
    *PoolHandle = Pool;
    if (Pool == NULL)
    {
        *Status = NDIS_STATUS_FAILURE;
        DBG_ERROR(gDbgInfo,("NdisAllocateBufferPool FAILED\n"));
    }
    else
    {
        *Status = NDIS_STATUS_SUCCESS;
        memset(Pool, 0, PoolSize);
#if DBG
        Pool->Tag = NDIS_BUFFER_POOL_TAG;
#endif
        Pool->PoolSize = PoolSize;
        Pool->NumberOfDescriptors = NumberOfDescriptors;
        Pool->DescriptorSize = DescriptorSize;
        NdisAllocateSpinLock(&Pool->SpinLock);
        CurrBuffer = (PNDIS_BUFFER) &Pool[1];
        Pool->BufferList = CurrBuffer;
        while (NumberOfDescriptors)
        {
            if (--NumberOfDescriptors == 0)
            {
                NextBuffer = NULL;
            }
            else
            {
                NextBuffer = (PNDIS_BUFFER) ((PUCHAR)CurrBuffer+DescriptorSize);
            }
#if DBG
            CurrBuffer->Tag = NDIS_BUFFER_TAG;
#endif
            CurrBuffer->Next = NextBuffer;
            CurrBuffer->Pool = Pool;
            CurrBuffer = NextBuffer;
        }
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisFreeBufferPool(
    IN  NDIS_HANDLE             PoolHandle
    )
{
    DBG_FUNCX("NdisFreeBufferPool", DBG_NDIS_SYS)

    PNDIS_BUFFER_POOL           Pool = (PNDIS_BUFFER_POOL)PoolHandle;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Pool && Pool->Tag == NDIS_BUFFER_POOL_TAG);

    if (Pool && Pool->NumberOfDescriptors)
    {
        Pool->NumberOfDescriptors = 0;
        kfree(PoolHandle);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisFreeBufferPool Invalid PoolHandle\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisAllocateBuffer(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_BUFFER *          Buffer,
    IN  NDIS_HANDLE             PoolHandle,
    IN  PVOID                   VirtualAddress,
    IN  UINT                    Length
    )
{
    DBG_FUNCX("NdisAllocateBuffer", DBG_NDIS_SYS)

    PNDIS_BUFFER_POOL           Pool = (PNDIS_BUFFER_POOL)PoolHandle;
    PNDIS_BUFFER                pBuffer = NULL;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Pool && Pool->Tag == NDIS_BUFFER_POOL_TAG);
    DBG_ASSERT(Status);
    DBG_ASSERT(Buffer);

    *Status = NDIS_STATUS_FAILURE;
    if (Pool && Pool->NumberOfDescriptors)
    {
        NdisAcquireSpinLock(&Pool->SpinLock);
        pBuffer = Pool->BufferList;
        if (pBuffer)
        {
            DBG_ASSERT(pBuffer && pBuffer->Tag == NDIS_BUFFER_TAG);
            Pool->BufferList = pBuffer->Next;
            pBuffer->Next = NULL;
            pBuffer->Size = Length;
            pBuffer->Address = VirtualAddress;
            pBuffer->ByteOffset = 0;
            pBuffer->ByteCount = Length;
            *Status = NDIS_STATUS_SUCCESS;

            DBG_ASSERT(pBuffer && pBuffer->Tag == NDIS_BUFFER_TAG);
        }
        NdisReleaseSpinLock(&Pool->SpinLock);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisAllocateBuffer Invalid PoolHandle\n"));
    }
    *Buffer = pBuffer;
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisFreeBuffer(
    IN PNDIS_BUFFER             Buffer
    )
{
    DBG_FUNCX("NdisFreeBuffer", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Buffer && Buffer->Tag == NDIS_BUFFER_TAG);

    if (Buffer && Buffer->Pool)
    {
        NdisAcquireSpinLock(&Buffer->Pool->SpinLock);
        Buffer->Next = Buffer->Pool->BufferList;
        Buffer->Pool->BufferList = Buffer;
        NdisReleaseSpinLock(&Buffer->Pool->SpinLock);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisFreeBuffer Invalid Buffer\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}

//
// NDIS Packets
//
VOID NdisAllocatePacketPool(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            PoolHandle,
    IN  UINT                    NumberOfDescriptors,
    IN  UINT                    ProtocolReservedLength
    )
{
    DBG_FUNCX("NdisAllocatePacketPool", DBG_NDIS_SYS)

    PNDIS_PACKET_POOL           Pool;
    ULONG                       PoolSize;
    ULONG                       DescriptorSize;
    PNDIS_PACKET                CurrPacket;
    PNDIS_PACKET                NextPacket;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Status);
    DBG_ASSERT(PoolHandle);
    DBG_ASSERT(NumberOfDescriptors);
    DBG_ASSERT(ProtocolReservedLength >= sizeof(CurrPacket->ProtocolReserved));
    DBG_ASSERT(!(ProtocolReservedLength & 0x03));

    DescriptorSize = sizeof(NDIS_PACKET) + ProtocolReservedLength -
                        sizeof(CurrPacket->ProtocolReserved);
    PoolSize = sizeof(NDIS_PACKET_POOL) + NumberOfDescriptors * DescriptorSize;
    Pool = kmalloc(PoolSize, GFP_KERNEL);
    *Status = NDIS_STATUS_FAILURE;
    *PoolHandle = Pool;
    if (Pool == NULL)
    {
        *Status = NDIS_STATUS_FAILURE;
        DBG_ERROR(gDbgInfo,("NdisAllocatePacketPool FAILED\n"));
    }
    else
    {
        *Status = NDIS_STATUS_SUCCESS;
        memset(Pool, 0, PoolSize);
#if DBG
        Pool->Tag = NDIS_PACKET_POOL_TAG;
#endif
        Pool->PoolSize = PoolSize;
        Pool->NumberOfDescriptors = NumberOfDescriptors;
        Pool->DescriptorSize = DescriptorSize;
        NdisAllocateSpinLock(&Pool->SpinLock);
        CurrPacket = (PNDIS_PACKET) &Pool[1];
        Pool->PacketList = CurrPacket;
        while (NumberOfDescriptors)
        {
            if (--NumberOfDescriptors == 0)
            {
                NextPacket = NULL;
            }
            else
            {
                NextPacket = (PNDIS_PACKET) ((PUCHAR)CurrPacket+DescriptorSize);
            }
#if DBG
            CurrPacket->Private.Tag = NDIS_PACKET_TAG;
#endif
            CurrPacket->Private.Head = (PNDIS_BUFFER)NextPacket;
            CurrPacket->Private.Pool = Pool;
            CurrPacket = NextPacket;
        }
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisFreePacketPool(
    IN  NDIS_HANDLE             PoolHandle
    )
{
    DBG_FUNCX("NdisFreePacketPool", DBG_NDIS_SYS)

    PNDIS_PACKET_POOL           Pool = (PNDIS_PACKET_POOL)PoolHandle;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Pool && Pool->Tag == NDIS_PACKET_POOL_TAG);

    if (Pool && Pool->NumberOfDescriptors)
    {
        Pool->NumberOfDescriptors = 0;
        kfree(PoolHandle);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisFreePacketPool Invalid PoolHandle\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisFreePacket(
    IN  PNDIS_PACKET            Packet
    )
{
    DBG_FUNCX("NdisFreePacket", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);

    if (Packet && Packet->Private.Pool)
    {
        NdisAcquireSpinLock(&Packet->Private.Pool->SpinLock);
        Packet->Private.Head = (PNDIS_BUFFER)Packet->Private.Pool->PacketList;
        Packet->Private.Pool->PacketList = Packet;
        NdisReleaseSpinLock(&Packet->Private.Pool->SpinLock);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisFreePacket Invalid Packet\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisAllocatePacket(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_PACKET *          Packet,
    IN  NDIS_HANDLE             PoolHandle
    )
{
    DBG_FUNCX("NdisAllocatePacket", DBG_NDIS_SYS)

    PNDIS_PACKET_POOL           Pool = (PNDIS_PACKET_POOL)PoolHandle;
    PNDIS_PACKET                pPacket = NULL;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Status);
    DBG_ASSERT(Packet);

    *Status = NDIS_STATUS_FAILURE;
    if (Pool && Pool->NumberOfDescriptors)
    {
        NdisAcquireSpinLock(&Pool->SpinLock);
        pPacket = Pool->PacketList;
        if (pPacket)
        {
            *Status = NDIS_STATUS_SUCCESS;
            Pool->PacketList = (PNDIS_PACKET)(pPacket->Private.Head);
            NdisZeroMemory(pPacket->ProtocolReserved,
                           Pool->DescriptorSize
                                - sizeof(NDIS_PACKET)
                                + sizeof(pPacket->ProtocolReserved));
            NdisZeroMemory(pPacket->MiniportReserved,
                           sizeof(pPacket->MiniportReserved));
            NdisReinitializePacket(pPacket);
            pPacket->Private.Tail = NULL;
            pPacket->Private.Flags = 0;
            pPacket->Private.Status = 0;
        }
        NdisReleaseSpinLock(&Pool->SpinLock);
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisAllocatePacket Invalid PoolHandle\n"));
    }
    *Packet = pPacket;
    DBG_ASSERT(pPacket && pPacket->Private.Tag == NDIS_PACKET_TAG);
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisRecalculatePacketCounts(
    IN OUT PNDIS_PACKET         Packet
    )
{
    DBG_FUNCX("NdisRecalculatePacketCounts", DBG_NDIS_SYS)

    PNDIS_BUFFER                TmpBuffer = Packet->Private.Head;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);

    if (TmpBuffer)
    {
        while (TmpBuffer->Next)
        {
            TmpBuffer = TmpBuffer->Next;
        }
        Packet->Private.Tail = TmpBuffer;
    }
    Packet->Private.ValidCounts = FALSE;

    //DBG_LEAVE(gDbgInfo);
}

VOID NdisReinitializePacket(
    IN OUT PNDIS_PACKET         Packet
    )
{
    DBG_FUNCX("NdisReinitializePacket", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);

    Packet->Private.Head = (PNDIS_BUFFER)NULL;
    Packet->Private.ValidCounts = FALSE;

    //DBG_LEAVE(gDbgInfo);
}

VOID NdisQueryPacket(
    IN  PNDIS_PACKET            Packet,
    OUT PUINT                   PhysicalBufferCount OPTIONAL,
    OUT PUINT                   BufferCount OPTIONAL,
    OUT PNDIS_BUFFER *          FirstBuffer OPTIONAL,
    OUT PUINT                   TotalPacketLength OPTIONAL
    )
{
    DBG_FUNCX("NdisQueryPacket", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);

    if (FirstBuffer != NULL)
    {
        *FirstBuffer = Packet->Private.Head;
    }
    if (TotalPacketLength || BufferCount || PhysicalBufferCount)
    {
        if (!Packet->Private.ValidCounts)
        {
            PNDIS_BUFFER TmpBuffer = Packet->Private.Head;
            UINT PTotalLength = 0, PPhysicalCount = 0, PAddedCount = 0;
            UINT PacketLength, Offset;

            while (TmpBuffer != (PNDIS_BUFFER)NULL)
            {
                NdisQueryBufferOffset(TmpBuffer, &Offset, &PacketLength);
                PTotalLength += PacketLength;
                PPhysicalCount += (UINT)NDIS_BUFFER_TO_SPAN_PAGES(TmpBuffer);
                ++PAddedCount;
                TmpBuffer = TmpBuffer->Next;
            }
            Packet->Private.Count = PAddedCount;
            Packet->Private.TotalLength = PTotalLength;
            Packet->Private.PhysicalCount = PPhysicalCount;
            Packet->Private.ValidCounts = TRUE;
        }

        if (PhysicalBufferCount)
        {
            *PhysicalBufferCount = Packet->Private.PhysicalCount;
        }
        if (BufferCount)
        {
            *BufferCount = Packet->Private.Count;
        }
        if (TotalPacketLength)
        {
            *TotalPacketLength = Packet->Private.TotalLength;
        }
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisChainBufferAtBack(
    IN OUT PNDIS_PACKET         Packet,
    IN OUT PNDIS_BUFFER         Buffer
    )
{
    DBG_FUNCX("NdisChainBufferAtBack", DBG_NDIS_SYS)

    PNDIS_BUFFER                TmpBuffer = Buffer;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);
    DBG_ASSERT(Buffer && Buffer->Tag == NDIS_BUFFER_TAG);

    Packet->Private.Tail = Buffer;

    if (Packet->Private.Head == NULL)
    {
    	 Packet->Private.Head = Buffer;
    }
    else
    {
	    TmpBuffer = Packet->Private.Head;
	    for (;;)
	    {
	        if (TmpBuffer->Next == (PNDIS_BUFFER)NULL)
	            break;
	        TmpBuffer = TmpBuffer->Next;
	    }
	    TmpBuffer->Next = Buffer;
	    Packet->Private.ValidCounts = FALSE;
    }
    //DBG_LEAVE(gDbgInfo);
}

VOID NdisUnchainBufferAtBack(
    IN OUT PNDIS_PACKET         Packet,
    OUT PNDIS_BUFFER *          Buffer
    )
{
    DBG_FUNCX("NdisUnchainBufferAtBack", DBG_NDIS_SYS)

    PNDIS_BUFFER                TmpBuffer;
    PNDIS_BUFFER                TmpBufferBack;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);
    DBG_ASSERT(Buffer);

    TmpBuffer = Packet->Private.Head;
    TmpBufferBack = TmpBuffer;
    if (TmpBuffer)
    {
	    for (;;)
	    {
    		if (TmpBuffer->Next == (PNDIS_BUFFER)NULL)
	            break;
	        TmpBufferBack = TmpBuffer;
	        TmpBuffer = TmpBuffer->Next;
	    }
	    if( TmpBuffer == TmpBufferBack )
	    {
	    		Packet->Private.Head = Packet->Private.Tail = NULL;
	    }
	    else
	    {
	    		TmpBufferBack->Next = NULL;
	    		Packet->Private.Tail = TmpBufferBack;
	    }
	    *Buffer = TmpBuffer;
           Packet->Private.ValidCounts = FALSE;
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisUnchainBufferAtFront Invalid Packet\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}


VOID NdisChainBufferAtFront(
    IN OUT PNDIS_PACKET         Packet,
    IN OUT PNDIS_BUFFER         Buffer
    )
{
    DBG_FUNCX("NdisChainBufferAtFront", DBG_NDIS_SYS)

    PNDIS_BUFFER                TmpBuffer = Buffer;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);
    DBG_ASSERT(Buffer && Buffer->Tag == NDIS_BUFFER_TAG);

    for (;;)
    {
        if (TmpBuffer->Next == (PNDIS_BUFFER)NULL)
            break;
        TmpBuffer = TmpBuffer->Next;
    }
    if (Packet->Private.Head == NULL)
    {
        Packet->Private.Tail = TmpBuffer;
    }
    TmpBuffer->Next = Packet->Private.Head;
    Packet->Private.Head = Buffer;
    Packet->Private.ValidCounts = FALSE;

    //DBG_LEAVE(gDbgInfo);
}

VOID NdisUnchainBufferAtFront(
    IN OUT PNDIS_PACKET         Packet,
    OUT PNDIS_BUFFER *          Buffer
    )
{
    DBG_FUNCX("NdisUnchainBufferAtFront", DBG_NDIS_SYS)

    PNDIS_BUFFER                TmpBuffer;

    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Packet && Packet->Private.Tag == NDIS_PACKET_TAG);
    DBG_ASSERT(Buffer);

    *Buffer = TmpBuffer = Packet->Private.Head;
    if (TmpBuffer)
    {
        Packet->Private.Head = TmpBuffer->Next;
        TmpBuffer->Next = NULL;
        Packet->Private.ValidCounts = FALSE;
    }
    else
    {
        DBG_ERROR(gDbgInfo,("NdisUnchainBufferAtFront Invalid Packet\n"));
    }
    //DBG_LEAVE(gDbgInfo);
}

#if defined(linux)
//
// NDIS Timers
//
void LinuxTimerFunc(
    unsigned long   arg
    )
{
    PNDIS_MINIPORT_TIMER    pNdisTimer = (PNDIS_MINIPORT_TIMER) arg;

    if (pNdisTimer->MiniportTimerFunction != NULL)
    {
        (*(pNdisTimer->MiniportTimerFunction))(NULL,
            pNdisTimer->MiniportTimerContext, NULL, NULL);
    }
}
#endif /* linux */

VOID NdisMSetTimer(
    IN  PNDIS_MINIPORT_TIMER    Timer,
    IN  UINT                    MillisecondsToDelay
    )
{
    DBG_FUNCX("NdisMSetTimer", DBG_NDIS_SYS)
    //DBG_PRINT(DbgInfo, ("@@@@@@@@ SET TIMER %p @@@@@@@\n",Timer));
    //DBG_ENTER(gDbgInfo);
    DBG_ASSERT(Timer && Timer->Tag == NDIS_TIMER_TAG);

#if defined(linux)
    {
    struct timer_list *pLTimer = &(Timer->TimerInfo);

    del_timer(pLTimer);
    pLTimer->expires = RUN_AT(MillisecondsToDelay);
    add_timer(pLTimer);
    }
#endif /* linux */

    //DBG_LEAVE(gDbgInfo);
}

VOID NdisMInitializeTimer(
    IN  OUT PNDIS_MINIPORT_TIMER Timer,
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PNDIS_TIMER_FUNCTION    TimerFunction,
    IN  PVOID                   FunctionContext
    )
{
    DBG_FUNCX("NdisMInitializeTimer", DBG_NDIS_SYS)

    DBG_ENTER(gDbgInfo);

    DBG_PRINT(DbgInfo, ("@@@@@@@@ INIT TIMER %p @@@@@@@\n",Timer));
    NdisZeroMemory(Timer, sizeof(Timer));
#if DBG
    Timer->Tag = NDIS_TIMER_TAG;
#endif
    Timer->MiniportTimerFunction = TimerFunction;
    Timer->MiniportTimerContext  = FunctionContext;
    Timer->MiniportAdapterHandle = MiniportAdapterHandle;

#if defined(linux)
    {
    struct timer_list *pLTimer = &(Timer->TimerInfo);

    init_timer(pLTimer);
    pLTimer->function = LinuxTimerFunc;
    pLTimer->data = (unsigned long) Timer;
    }
#endif /* linux */

    DBG_LEAVE(gDbgInfo);
}

VOID NdisMCancelTimer(
    IN  PNDIS_MINIPORT_TIMER    Timer,
    OUT PBOOLEAN                TimerCancelled
    )
{
    DBG_FUNCX("NdisMCancelTimer", DBG_NDIS_SYS)

    DBG_ENTER(gDbgInfo);
    DBG_PRINT(DbgInfo, ("@@@@@@@@ CANCEL TIMER %p @@@@@@@\n",Timer));
    DBG_ASSERT(Timer && Timer->Tag == NDIS_TIMER_TAG);

#if defined(linux)
    {
    struct timer_list *pLTimer = &(Timer->TimerInfo);
    int     result;

    result = del_timer(pLTimer);

    if (TimerCancelled)
    {
        *TimerCancelled = result ? TRUE : FALSE;
    }
    }
#endif /* linux */

    DBG_LEAVE(gDbgInfo);
}

//
// NDIS Work Items
//
VOID NdisInitializeWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem,
    IN  NDIS_PROC               Routine,
    IN  PVOID                   Context
    )
{
    DBG_FUNCX("NdisInitializeWorkItem", DBG_NDIS_SYS)

    DBG_ENTER(gDbgInfo);

    // Only used by AtmLoopbackTimer to schedule a callback.
    WorkItem->Routine = Routine;
    WorkItem->Context = Context;

    DBG_LEAVE(gDbgInfo);
}

NDIS_STATUS NdisScheduleWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem
    )
{
    DBG_FUNCX("NdisScheduleWorkItem", DBG_NDIS_SYS)

    NDIS_STATUS                 Status = NDIS_STATUS_SUCCESS;

    DBG_ENTER(gDbgInfo);

    DBG_ASSERT(WorkItem);
    DBG_ASSERT(WorkItem->Routine);

    // Only used by AtmLoopbackTimer to schedule a callback.
    // Just call the function now since we're already on a timer thread.
    (*WorkItem->Routine)(WorkItem, WorkItem->Context);

    DBG_RETURN(gDbgInfo,(long)(Status));
    return (Status);
}

//
// NDIS Memory
//
NDIS_STATUS NdisAllocateMemoryWithTag(
    OUT PVOID *                 VirtualAddress,
    IN  UINT                    Length,
    IN  ULONG                   Tag
    )
{
//    DBG_FUNCX("NdisAllocateMemoryWithTag", DBG_NDIS_SYS)

    NDIS_STATUS                 Status = NDIS_STATUS_SUCCESS;
    PVOID                       pMemory;

    //DBG_ENTER(gDbgInfo);
    //DBG_ASSERT(VirtualAddress);
    //DBG_ASSERT(Length);
    //DBG_ASSERT(Tag);

    // Pad to multiple of 4 bytes and add 4 longs for our work area
    pMemory = kmalloc(((Length+3)&~3)+(4*sizeof(ULONG)), GFP_KERNEL);

    //DBG_ASSERT(((ULONG)pMemory & 3) == 0); // 4 byte alignment

    if (pMemory == NULL)
    {
        //DBG_ERROR(gDbgInfo,("NdisAllocateMemoryWithTag failed\n"));
        Status = NDIS_STATUS_FAILURE;
    }
    else
    {
        PULONG pLongMem = (PULONG)pMemory;
        pLongMem[1+(Length-1)/sizeof(ULONG)] = Length;
        pLongMem[2+(Length-1)/sizeof(ULONG)] = Tag;
        pLongMem[3+(Length-1)/sizeof(ULONG)] = Tag ^ Length;
        pLongMem[4+(Length-1)/sizeof(ULONG)] = NDIS_MEMORY_TAG;
    }
    *VirtualAddress = pMemory;
    //DBG_RETURN(gDbgInfo,(long)(Status));
    return (Status);
}

VOID NdisFreeMemory(
    IN  PVOID                   VirtualAddress,
    IN  UINT                    Length,
    IN  UINT                    MemoryFlags
    )
{
    //DBG_FUNCX("NdisFreeMemory", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);

    if (VirtualAddress == NULL)
    {
        //DBG_ERROR(gDbgInfo,("NdisFreeMemory called with NULL pointer\n"));
    }
    else
    {
#if DBG
        PULONG pLongMem = (PULONG)VirtualAddress;
        //DBG_ASSERT(pLongMem[1+(Length-1)/sizeof(ULONG)] == Length);
        //DBG_ASSERT(pLongMem[2+(Length-1)/sizeof(ULONG)] ==
                  //(pLongMem[3+(Length-1)/sizeof(ULONG)] ^ Length));
        //DBG_ASSERT(pLongMem[4+(Length-1)/sizeof(ULONG)] == NDIS_MEMORY_TAG);
#endif
        kfree(VirtualAddress);
    }
    //DBG_LEAVE(gDbgInfo);
}

//
// NDIS Events
//
VOID NdisInitializeEvent(
    IN  PNDIS_EVENT             Event
    )
{
    DBG_FUNCX("NdisInitializeEvent", DBG_NDIS_SYS)

    DBG_ENTER(gDbgInfo);

    init_waitqueue_head(Event);

    DBG_LEAVE(gDbgInfo);
}

VOID NdisSetEvent(
    IN PNDIS_EVENT  Event
    )
{
    DBG_FUNCX("NdisSetEvent", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);

    wake_up_interruptible(Event);

    //DBG_LEAVE(gDbgInfo);
}

VOID NdisResetEvent(
    IN  PNDIS_EVENT             Event
    )
{
    DBG_FUNCX("NdisResetEvent", DBG_NDIS_SYS)

    //DBG_ENTER(gDbgInfo);

    // Do nothing here

    //DBG_LEAVE(gDbgInfo);
}

BOOLEAN NdisWaitEvent(
    IN  PNDIS_EVENT             Event,
    IN  UINT                    msToWait
    )
{
    DBG_FUNCX("NdisWaitEvent", DBG_NDIS_SYS)
    int event_jiffies = 0;
	long Result;
    
    DBG_ENTER(gDbgInfo);

    event_jiffies = 1 + (msToWait * HZ)/1000;
    
    interruptible_sleep_on_timeout(Event, event_jiffies);
    
    DBG_RETURN(gDbgInfo,(long)(Result));
    return (TRUE);
}

/*+F*************************************************************************
 * Function:
 *   NdisMIndicateStatus
 *
 * Description:
 *   Notify the LAN/WAN subsystems that the link state has changed.
 *-F*************************************************************************/
void
NdisMIndicateStatus(
    NDIS_HANDLE     ndisHandle,
    NDIS_STATUS     linkStatus,
    PVOID           statusBuffer,
    UINT            statusBufferSize
    )
{
    DBG_FUNCX("NdisMIndicateStatus", DBG_NDIS_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;

    DBG_ENTER(DbgInfo);

    switch (linkStatus)
    {
    case NDIS_STATUS_MEDIA_DISCONNECT:
        DBG_PRINT(DbgInfo, ("**************************\n"));
        printk(KERN_INFO MTAG"**** Media Disconnect ****\n");
        DBG_PRINT(DbgInfo, ("**************************\n"));
        printk(KERN_INFO MTAG"  Link down\n");
        break;

    case NDIS_STATUS_MEDIA_CONNECT:
        DBG_PRINT(DbgInfo, ("*************************************\n"));
        printk(KERN_INFO MTAG"**** Media Connect ****\n");
        DBG_PRINT(DbgInfo, ("****   UpRate:%ld  DownRate:%ld\n",
            pAdapter->UpstreamRate, pAdapter->DownstreamRate));
        DBG_PRINT(DbgInfo, ("*************************************\n"));
        printk(KERN_INFO MTAG"  Link established (%ld/%ld)\n",
            pAdapter->DownstreamRate, pAdapter->UpstreamRate);
        break;

    default:
        DBG_ERROR(DbgInfo, ("Unhandled indicate status 0x%lx\n", linkStatus));
        break;
    }

#ifdef GSI_LAN
    if (pAdapter != NULL)
    {
        GpLanMediaChange(pAdapter);
    }
#endif // GSI_LAN

#ifdef GSI_WAN
    if (pAdapter != NULL)
    {
        GpWanMediaChange(pAdapter);
    }
#endif // GSI_WAN

    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   NdisMSendComplete
 *
 * Description:
 *   The transmit is complete. Notify the LAN/WAN subsystems so the buffers
 *   can be freed.
 *-F*************************************************************************/
void
NdisMSendComplete(
    NDIS_HANDLE     ndisHandle,
    PNDIS_PACKET    pPacket,
    NDIS_STATUS     status,
	void		    *plan	/* This is gsi_lan_t * We are getting this from the GpLanTx */
    )
{
    DBG_FUNCX("NdisMSendComplete", DBG_TX_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;
    PVOID      *pProtoReserved = (PVOID *) pPacket->ProtocolReserved;

    //DBG_ENTER(DbgInfo);

#ifdef GSI_LAN
    if (*pProtoReserved == NULL)
    {
        GpLanTxComplete(pAdapter, (status == NDIS_STATUS_SUCCESS) ? 0 : -1, plan);
    }
#endif // GSI_LAN

#ifdef GSI_WAN
    if (*pProtoReserved != NULL)
    {
        channel_t  *pChan = *((channel_t **) pProtoReserved);

        GpWanTxComplete(pAdapter, pChan, (status == NDIS_STATUS_SUCCESS) ? 0 : -1);
    }
#endif // GSI_WAN

    //DBG_LEAVE(DbgInfo);
}

#ifdef GSI_LAN
/*+F*************************************************************************
 * Function:
 *   NdisMEthIndicateReceive
 *
 * Description:
 *   Notify that LAN subsystem that a new receive buffer is available.
 *-F*************************************************************************/
void
NdisMEthIndicateReceive(
    NDIS_HANDLE     ndisHandle,
    NDIS_HANDLE     ndisReceiveContext,
    PVOID           pHeader,
    UINT            headerSize,
    PVOID           pLookAhead,
    UINT            lookAheadSize,
    UINT            packetSize,
	void			*plan /* This is pointer to lan interface */
    )
{
    DBG_FUNCX("NdisMEthIndicateReceive", DBG_RX_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;

    DBG_ENTER(DbgInfo);
    DBG_ASSERT(packetSize == lookAheadSize);

    GpLanRx(pAdapter, pHeader, headerSize, pLookAhead, lookAheadSize,plan);

    DBG_LEAVE(DbgInfo);
}
#endif // GSI_LAN

#ifdef GSI_WAN
/*+F*************************************************************************
 * Function:
 *   NdisMWanIndicateReceive
 *
 * Description:
 *   Notify that WAN subsystem that a new receive buffer is available.
 *-F*************************************************************************/
void
NdisMWanIndicateReceive(
    PNDIS_STATUS    pStatus,
    NDIS_HANDLE     ndisHandle,
    NDIS_HANDLE     ndisLinkContext,
    PUCHAR          pPacket,
    ULONG           packetSize
    )
{
    DBG_FUNCX("NdisMWanIndicateReceive", DBG_RX_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;
    channel_t  *pChan = (channel_t *) ndisLinkContext;

    //DBG_ENTER(DbgInfo);

    //DBG_PRINT(DbgInfo, ("Apapter Handle %p\n", pAdapter));
    //DBG_PRINT(DbgInfo, ("NdisLink Context %p\n", pChan));


    GpWanRx(pAdapter, pChan, pPacket, packetSize);
	*pStatus = NDIS_STATUS_SUCCESS;

    //DBG_LEAVE(DbgInfo);
}
#endif // GSI_WAN

