/*
  %W%
*/

/* **************************************************************************
** FILE NAME:      usb_msg.c
** DESCRIPTION:    This layer collects all of the low level calls from the TSM
**                 and packs them into ONE single function.
**                 This can actually make the download faster in some cases.
** FILE CREATOR:   KPatel
** ************************************************************************** */

/* **************************************************************************
**                             HEADER_FILES
** ************************************************************************** */

#include "Gp.h"
#include "gtidsp.h"
#include "gti_sym.h"
#include "GULinux.h"
#include "common.h"
#include "debug.h"
#include "atmphy.h"
#include "usbphy.h"
#include "usbcdc.h"

GTI_BYTE lastRcvdStatusBlockId = 0xff;
GTI_CHIP_INFO chipInfo;

//Set Function
void Gti_SetRegDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg, GTI_BYTE bData);
void Gti_SetByteDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg,  GTI_BYTE bData);
void Gti_SetWordDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bDataL, GTI_BYTE bDataH);
void Gti_SetSpecialDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bAddrL, GTI_BYTE bAddrH, GTI_BYTE bData);
void Gti_SetAddrDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bAddrL, GTI_BYTE bAddrH);
void gsiSetMsg(GTI_MSG_LAYER_INFO *pMsgInfoPtr);
void Gti_SetBlockDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg, GTI_WORD wLen,
                       GTI_BYTE GTI_CONST_QUAL GTI_ADDRESS_QUAL *pStrt);
void Gti_SetSpecialDspIO(GTI_XCVR_INFO *, GTI_BYTE, GTI_BYTE, GTI_BYTE, GTI_BYTE);
void SetDspAccess(GTI_XCVR_INFO *pSysPtr , GTI_BYTE bType);
GTI_BYTE GetDspAccess(GTI_XCVR_INFO *pSysPtr );

//Get Function
GTI_BYTE Gti_GetByteDspIO(GTI_XCVR_INFO *, GTI_BYTE, GTI_BYTE);
GTI_WORD Gti_GetWordDspIO(GTI_XCVR_INFO *, GTI_BYTE);
void gsiGetDspBlock(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE *pStrt, GTI_WORD wLen);
void Gti_GetBlockDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE wordSize, GTI_WORD wLen, GTI_BYTE *pBuffer);
void Gti_GetExtBlockDspIO(GTI_XCVR_INFO *pSysInfo, GTI_BYTE wordSize, GTI_WORD wLen, GTI_BYTE *pBuffer);

GTI_BYTE Gti_GetRegDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bReg);
void gtiGetAccessType(GTI_XCVR_INFO *pSysPtr, GTI_BYTE cfg, GTI_BYTE isItRead, GTI_BYTE size);

//Miscellaneous 
void Gti_UpdateDspIO(GTI_XCVR_INFO *pSysPtr);
void Gti_ConfigureDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bData);
void ResetDSP(GTI_XCVR_INFO *pXcvrInfo ,GTI_BYTE bValue);
void Gti_InitUsbMsgIO(GTI_XCVR_INFO *pSysPtr);

GTI_WORD Gti_LocalBusTest(GTI_XCVR_INFO* pXcvrInfo, GTI_DWORD dwStart, GTI_DWORD dwEnd, GTI_DWORD *dwCurAddr);

GTI_WORD ExecuteXCVR_USB (GTI_XCVR_INFO *pXcvrInfo,
                           GTI_WORD ipuReady,
                           GTI_WORD nextState); /*We need to change this name to  Gti_ExecuteXCVR_USB */

void SetDataMode_USB (GTI_XCVR_INFO *pXcvrInfo);

GTI_WORD Gti_ExecuteXCVR(GTI_XCVR_INFO *pXcvrInfo);

GTI_WORD Gti_SStatusXCVR_USB(GTI_XCVR_INFO *pSys, 
            GTI_WORD nStatusType,        
            void *pStatus,               
            GTI_WORD nLength,            
            GTI_BYTE nChanID );

void InitStatus_USB(GTI_XCVR_INFO *GTISysPtr);

void gtiSendMsg(GTI_XCVR_INFO *pSysPtr);

void gsiSendMsg(GTI_XCVR_INFO *pSysPtr, GTI_BYTE type, GTI_WORD value, GTI_WORD index,
                GTI_BYTE *bDataPtr, int wDataLen);

void GTI_ParseDspStatus(GTI_XCVR_INFO *pXcvrInfo,
                           GTI_BYTE *dataPtr);

BOOLEAN UsbMsgLayerInit(GTI_XCVR_INFO *pXcvrInfo , PUSB_PHY );

//extern func
extern void GTISendUSBCmdEx(void *usbphy, unsigned char type,
                     unsigned short value, unsigned short index,
                     GTI_BYTE *pData , unsigned long length, unsigned char getData);

extern GTI_WORD Gti_ExecuteXCVR(GTI_XCVR_INFO *pXcvrInfo);

extern GTI_WORD Gti_SStatusXCVR(GTI_XCVR_INFO *pSys, 
            GTI_WORD nStatusType,        
            void *pStatus,               
            GTI_WORD nLength,            
            GTI_BYTE nChanID );

// internal use of this block
void  DoAutoIncrement( GTI_MSG_LAYER_INFO *pMsgInfoPtr);

#ifdef	USB_HEXDUMP
void HexDump(unsigned char *pBuffer, unsigned long nSize);
#endif	//USB_HEXDUMP

/* **************************************************************************
**                               FUNCTIONS
** ************************************************************************** */

/* ************************************************
** gsiSetMsg
**
** ************************************************ */

////////////////////////////////////////////////////////////////////////////////
//
// NAME: SetDspAccess
//
// ABSTRACT:
//         Sets direct or not direct DSP memory access.        
//         We need direct memory access just during code downloading 
//         
// RETURNS:
//        void
//
// ASSUMPTIONS:
//        none
//
////////////////////////////////////////////////////////////////////////////////
void SetDspAccess(GTI_XCVR_INFO *pSysPtr , GTI_BYTE bType)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("SetDspAccess\n"));
    
   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   pMsgInfoPtr->DspAccess = bType;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: GetDspAccess
//
// ABSTRACT:
//         Gets direct or not direct DSP memory access.        
//         We need direct memory access just during code downloading 
//         
// RETURNS:
//        Current Access type
//
// ASSUMPTIONS:
//        none
//
////////////////////////////////////////////////////////////////////////////////
GTI_BYTE GetDspAccess(GTI_XCVR_INFO *pSysPtr )
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("GetDspAccess\n"));

   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   return pMsgInfoPtr->DspAccess;    
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbMsgLayerInit
//
// ABSTRACT:
//              Initialize translation layer   
//              It Allocates memory.
//              It will be using one status block and two  GTI_MSG_LAYER_INFO
//              One for the CPL one for Xcvr.
//                  
//              Translation Layer is not reentrant.         
//
// RETURNS:
//        BOLEAN
//
// ASSUMPTIONS:
//            This function has to be called before using translation layer
//
////////////////////////////////////////////////////////////////////////////////
BOOLEAN UsbMsgLayerInit(GTI_XCVR_INFO *pXcvrInfo , PUSB_PHY pUsbPhy)
{
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("UsbMsgLayerInit\n"));

   pXcvrInfo->CustXcvrInfo.msgInfo = NULL;
   ALLOCATE_MEMORY(&pXcvrInfo->CustXcvrInfo.msgInfo, sizeof(GTI_MSG_LAYER_INFO) );
   if( !(pXcvrInfo->CustXcvrInfo.msgInfo) )
   {
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
              ("UsbMsgLayerInit ERROR Allocate memory\n "));
       return FALSE;
   }
    
   pXcvrInfo->CustXcvrInfo.status = NULL;
   ALLOCATE_MEMORY(&pXcvrInfo->CustXcvrInfo.status , sizeof(GTI_STATUS_BLOCK) );
   if( !(pXcvrInfo->CustXcvrInfo.status) )
   {
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
              ("UsbMsgLayerInit ERROR Allocate memory\n "));
       return FALSE;
   }

   pUsbPhy->GTISysBlockCpl.CustXcvrInfo.status = pXcvrInfo->CustXcvrInfo.status;

   ALLOCATE_MEMORY(&pUsbPhy->GTISysBlockCpl.CustXcvrInfo.msgInfo, sizeof(GTI_MSG_LAYER_INFO) );
   if(!pUsbPhy->GTISysBlockCpl.CustXcvrInfo.msgInfo )
   {
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
              ("UsbMsgLayerInit ERROR Allocate memory\n "));
       return FALSE;
   }

   Gti_InitUsbMsgIO(pXcvrInfo);
   SetDspAccess(pXcvrInfo , FALSE);

   Gti_InitUsbMsgIO(&pUsbPhy->GTISysBlockCpl);
   SetDspAccess(&pUsbPhy->GTISysBlockCpl, FALSE);

   pUsbPhy->GTISysBlockCpl.CustXcvrInfo.msgInfo->pUsbPhy = pUsbPhy;
   pXcvrInfo->CustXcvrInfo.msgInfo->pUsbPhy = pUsbPhy;
   InitStatus_USB(pXcvrInfo);

	pXcvrInfo->pChipInfo = &chipInfo;
	pUsbPhy->GTISysBlockCpl.pChipInfo = &chipInfo;


   return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbMsgLayerClose
//
// ABSTRACT:
//       Closes translation layer. It Deallocates memory.
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       You can not use Translation Layer after calling this function
//       It will be called during shutdown the driver
////////////////////////////////////////////////////////////////////////////////
void UsbMsgLayerClose(GTI_XCVR_INFO *pXcvrInfo)
{
    PUSB_PHY pUsbPhy = pXcvrInfo->CustXcvrInfo.msgInfo->pUsbPhy;

    if (pXcvrInfo->CustXcvrInfo.msgInfo)
    {
       FREE_MEMORY (pXcvrInfo->CustXcvrInfo.msgInfo, sizeof(GTI_MSG_LAYER_INFO ));
       pXcvrInfo->CustXcvrInfo.msgInfo = NULL;
    }

    if (pUsbPhy->GTISysBlockCpl.CustXcvrInfo.msgInfo)
    {
       FREE_MEMORY (pUsbPhy->GTISysBlockCpl.CustXcvrInfo.msgInfo, sizeof(GTI_MSG_LAYER_INFO ));
       pXcvrInfo->CustXcvrInfo.msgInfo = NULL;
    }

    if (pXcvrInfo->CustXcvrInfo.status)
    {
       FREE_MEMORY (pXcvrInfo->CustXcvrInfo.status, sizeof(GTI_STATUS_BLOCK ));
       pXcvrInfo->CustXcvrInfo.status = NULL;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_GetByteDspIO
//
// ABSTRACT:
//       It will be called by using macro READ_DSP_BYTE
//
// RETURNS:
//       GTI_BYTE
//
// ASSUMPTIONS:
//       Local copy of DSP RAM is valid, USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
GTI_BYTE Gti_GetByteDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE chanel, GTI_BYTE action)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;
GTI_BYTE bValue = (GTI_BYTE)(-1);
GTI_BYTE bMemType;  
GTI_BYTE bAddr;
GTI_BYTE bLocalCopy = FALSE;
PUSB_PHY pUsbPhy = pSysPtr->CustXcvrInfo.msgInfo->pUsbPhy;

   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   bMemType = (pMsgInfoPtr->index >> 8) & 0x7F;
   bAddr    = (GTI_BYTE)(pMsgInfoPtr->index & 0x00FF);
  
   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
   //           ("Gti_GetByteDspIO MemType = %x, Addr = %x\n ", bMemType, bAddr));

   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_READ_BLOCK);                           

   /* Is it correct state? */
   if ((pMsgInfoPtr->State != GTI_ML_STATE_ADDR))
       return bValue;
   
   switch(bMemType)
   {
       case GTI_FDPSRAM6:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           bValue = pSysPtr->CustXcvrInfo.status->fdpram6[bAddr][0];
           bLocalCopy  = TRUE;
           break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

       case GTI_FDPSRAM7:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           bValue = pSysPtr->CustXcvrInfo.status->fdpram7[bAddr][0];
           bLocalCopy  = TRUE;
           break;
    
               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;
    
       case GTI_FDPSRAMC:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if(bAddr >= (GTI_cpTxToneOrdTbl_LOC & 0xFF) )
           {
               bValue = pSysPtr->CustXcvrInfo.status->fdpramc[bAddr][0];
               bLocalCopy  = TRUE;
           }
           break;
        
               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;
        
       case GTI_FDPSRAM0:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if(bAddr == GTI_cpnoTxBins_LOC)
           {
               bValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsEx[0];
               bLocalCopy  = TRUE;
           }    
           else if(bAddr == GTI_cpnoRxBins_LOC)
           {
               bValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsEx[0];
               bLocalCopy  = TRUE;
           }
           break;                

            case GTI_TYPE_ANNEX_C:
                if(bAddr == GTI_cpnoTxBinsFextHost_LOC)
                {
                    bValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsFext[0];
                    bLocalCopy  = TRUE;

                }    
                else if(bAddr == GTI_cpnoRxBinsFextHost_LOC)
                {
                    bValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsFext[0];
                    bLocalCopy  = TRUE;

                }
                else if(bAddr == GTI_cpnoTxBinsNextHost_LOC)
                {
                    bValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsNext[0];
                    bLocalCopy  = TRUE;
                }
                else if(bAddr == GTI_cpnoRxBinsNextHost_LOC)
                {
                    bValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsNext[0];
                    bLocalCopy  = TRUE;
                
                }
            break;                
           }
           break;

       case GTI_FDPSRAMA:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
                   bValue = pSysPtr->CustXcvrInfo.status->fdprama[bAddr][0];
                   bLocalCopy  = TRUE;
                   break;
           }
           break;

       case GTI_FDPSRAME:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
                   bValue = pSysPtr->CustXcvrInfo.status->fdprame[bAddr][0];
                   bLocalCopy  = TRUE;
                   break;
           }
           break;

       case GTI_FDPSRAMF:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
                   bValue = pSysPtr->CustXcvrInfo.status->fdpramf[bAddr][0];
                   bLocalCopy  = TRUE;
                   break;
           }
            break;                

       case GTI_PSRAM0W:
           if(bAddr == 0x03)
           {/* where is the #define of this value? Sasha*/
               bValue = pSysPtr->CustXcvrInfo.status->rxByteStatus;
               bLocalCopy  = TRUE;
           }
           else if(bAddr == 0x02)
           {/* where is the #define of this value? Sasha*/
               bValue = pSysPtr->CustXcvrInfo.status->txByteStatus;
               bLocalCopy  = TRUE;
           }
           break;
        
       default:
           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("default\n"));
           bLocalCopy = FALSE;
           break;
   }
   
   if( !(bLocalCopy) )
   {
       pMsgInfoPtr->currIdx = GET_DSP_WORD_BYTES(bMemType);
       SEND_DSP_MSG(pSysPtr);
       bValue = pMsgInfoPtr->dataBuff[0];
   }
   
   return bValue;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_GetWordDspIO
//
// ABSTRACT:
//       It will be called by using macro READ_DSP_WORD
//
// RETURNS:
//       GTI_WORD
//
// ASSUMPTIONS:
//       Local copy of DSP RAM is valid, USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
GTI_WORD Gti_GetWordDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE chanel)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;
GTI_WORD wValue = (GTI_WORD)(-1);
GTI_BYTE bMemType;  
GTI_BYTE bAddr;
GTI_BYTE bMaxAddrCounter;
GTI_BYTE bMaxAddrFailure;
GTI_BYTE bMaxAddrRxEoc;
GTI_BYTE bIndex;
DSL_EX *pDslEx;
GTI_BYTE bLocalCopy = FALSE;
GTI_BYTE size;
GTI_BYTE bDspAccess = GetDspAccess( pSysPtr );
PUSB_PHY pUsbPhy = pSysPtr->CustXcvrInfo.msgInfo->pUsbPhy;

   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   bMemType = (pMsgInfoPtr->index >> 8) & 0x7F;
   bAddr    = (GTI_BYTE)(pMsgInfoPtr->index & 0x00FF);
    
   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
   //           ("Gti_GetWordDspIO MemType = %x, Addr = %x\n ", bMemType, bAddr));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_READ_BLOCK);                           

   /* Is it correct state? */
   if ((pMsgInfoPtr->State != GTI_ML_STATE_ADDR))
       return wValue;
       
   switch( bMemType)
   {
       case GTI_FDPSRAM6:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if( !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->fdpram6[bAddr][1];
               wValue|= ((pSysPtr->CustXcvrInfo.status->fdpram6[bAddr][2]) << 8);
               bLocalCopy = TRUE;
           }
           break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

       case GTI_FDPSRAM7:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if( !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->fdpram7[bAddr][1];
               wValue|= ((pSysPtr->CustXcvrInfo.status->fdpram7[bAddr][2]) << 8);
               bLocalCopy = TRUE;
           }
           break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

       case GTI_FDPSRAMC: /* 0x74*/
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if(bAddr >= (GTI_cpTxToneOrdTbl_LOC & 0xFF) && !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->fdpramc[bAddr][1];
               wValue|= ((pSysPtr->CustXcvrInfo.status->fdpramc[bAddr][2]) << 8);
               bLocalCopy = TRUE;
           }
           break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

       case GTI_FDPSRAM0:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if(bAddr == GTI_cpnoTxBins_LOC && !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsEx[1];
               wValue|= ((pSysPtr->CustXcvrInfo.status->TxbTotalBinsEx[2]) << 8);
               bLocalCopy = TRUE;
           }
           else if(bAddr == GTI_cpnoRxBins_LOC && !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsEx[1];
               wValue|= ((pSysPtr->CustXcvrInfo.status->RxbTotalBinsEx[2]) << 8);
               bLocalCopy = TRUE;
           }
           break;

               case GTI_TYPE_ANNEX_C:
                   if(bAddr == GTI_cpnoTxBinsFextHost_LOC && !bDspAccess)
                   {

                        wValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsFext[1];
                        wValue|= ((pSysPtr->CustXcvrInfo.status->TxbTotalBinsFext[2]) << 8);
                        bLocalCopy = TRUE;
           
                   }         
                   else if(bAddr == GTI_cpnoRxBinsFextHost_LOC && !bDspAccess)
                   {
                        wValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsFext[1];
                        wValue|= ((pSysPtr->CustXcvrInfo.status->RxbTotalBinsFext[2]) << 8);
                        bLocalCopy = TRUE;
                   }     
           
                   if(bAddr == GTI_cpnoTxBinsNextHost_LOC && !bDspAccess)
                   {

                        wValue = pSysPtr->CustXcvrInfo.status->TxbTotalBinsNext[1];
                        wValue|= ((pSysPtr->CustXcvrInfo.status->TxbTotalBinsNext[2]) << 8);
                        bLocalCopy = TRUE;
           
                   }         
                   else if(bAddr == GTI_cpnoRxBinsNextHost_LOC && !bDspAccess)
                   {
                        wValue = pSysPtr->CustXcvrInfo.status->RxbTotalBinsNext[1];
                        wValue|= ((pSysPtr->CustXcvrInfo.status->RxbTotalBinsNext[2]) << 8);
                        bLocalCopy = TRUE;
                   }
                   break;
           }
           break;

        case GTI_FDPSRAMA:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
			{
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;
				
               case GTI_TYPE_ANNEX_C:
			        if( !bDspAccess)
			        {
				wValue = pSysPtr->CustXcvrInfo.status->fdprama[bAddr][1];
				wValue|= ((pSysPtr->CustXcvrInfo.status->fdprama[bAddr][2]) << 8);
				bLocalCopy = TRUE;
            }
			break;
           }
			break;

        case GTI_FDPSRAME:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			if( !bDspAccess)
			{
				wValue = pSysPtr->CustXcvrInfo.status->fdprame[bAddr][1];
				wValue|= ((pSysPtr->CustXcvrInfo.status->fdprame[bAddr][2]) << 8);
				bLocalCopy = TRUE;
			}
            break;
           }
           break;

		case GTI_FDPSRAMF: 
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
            if(bAddr >= (GTI_cpTxToneOrdTblFextHost_LOC & 0xFF) && !bDspAccess)
            {
                wValue = pSysPtr->CustXcvrInfo.status->fdpramf[bAddr][1];
                wValue|= ((pSysPtr->CustXcvrInfo.status->fdpramf[bAddr][2]) << 8);
                bLocalCopy = TRUE;
           }
           break;
           }
           break;

		case GTI_IRAMD:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
			{
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			    wValue = pSysPtr->CustXcvrInfo.status->iramd[bAddr][0];
                wValue|= ((pSysPtr->CustXcvrInfo.status->iramd[bAddr][1]) << 8);
                bLocalCopy = TRUE;
                   break;
			}
			break;

		case GTI_IRAME:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
			{
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			    wValue = pSysPtr->CustXcvrInfo.status->irame[bAddr][0];
                wValue|= ((pSysPtr->CustXcvrInfo.status->irame[bAddr][1]) << 8);
                bLocalCopy = TRUE;
			break;
           }         
           break;

       case GTI_ISRAM:
           bMaxAddrRxEoc   = GTI_EOC_RXBUFF_OFFSET + GTI_EOC_MAX_NUMOF_MSG;
           pDslEx = (DSL_EX *)pSysPtr->CustXcvrInfo.msgInfo->pDslEx;

           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           bMaxAddrCounter = GTI_local_RSI_counter_LOC + GTI_COUNTERS_LEN / sizeof(GTI_WORD);
           bMaxAddrFailure = GTI_failure_overall_LOC   + GTI_FAILURES_LEN / sizeof(GTI_WORD);
                   break;

               case GTI_TYPE_ANNEX_C:
           bMaxAddrCounter = GTI_local_RSI_counter_C_LOC + GTI_COUNTERS_LEN / sizeof(GTI_WORD);
           bMaxAddrFailure = GTI_failure_overall_C_LOC + GTI_FAILURES_LEN / sizeof(GTI_WORD);
                   break;
           }

           if( bAddr == GTI_OP_CH_LOCATION && !bDspAccess )
           {
               wValue = pSysPtr->CustXcvrInfo.status->opState;
               bLocalCopy = TRUE;
           }
           else if(bAddr == GTI_XMIT_LOCATION)
           {
               wValue = pSysPtr->CustXcvrInfo.status->xmitPwr;
               bLocalCopy = TRUE;
           }
           else if(bAddr == GTI_PROGRESS_LOCATION && !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->startProgress;
               bLocalCopy = TRUE;
           }
           else if ( (bAddr >= 0xf4) && (bAddr <= 0xff) && !bDspAccess )
		   {
			    wValue = pSysPtr->CustXcvrInfo.status->isram[bAddr][0];
                wValue|= ((pSysPtr->CustXcvrInfo.status->isram[bAddr][1]) << 8);

           		bLocalCopy = TRUE;
		   
		   }
          else if(bAddr == GTI_host_BSCounter_LOC && !bDspAccess)
          {
			    wValue = pSysPtr->CustXcvrInfo.status->BitSwapCounter;
           	bLocalCopy = TRUE;
          }

           else if(bAddr == GTI_EOC_RXBUFF_FLAG_OFFSET && !bDspAccess)
           {
               wValue = pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxFlag;
               bLocalCopy = TRUE;     
           }
           else if( bAddr == GTI_EOC_TXBUFF_FLAG_OFFSET && !bDspAccess)
           {
               wValue = pDslEx->EocDspBuffer[pDslEx->CellReadInd].txFlag;
               bLocalCopy = TRUE;
           }
           else if(bAddr >= GTI_EOC_RXBUFF_OFFSET && bAddr < bMaxAddrRxEoc && !bDspAccess)
           {
               bIndex = bAddr - GTI_EOC_RXBUFF_OFFSET;
               wValue = pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxBuff[bIndex];
               pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxBuff[bIndex] = EMPTY_PATTERN;
               bLocalCopy = TRUE;
           }
           else if (bAddr == GTI_I2H_ready )
           {
               wValue = pSysPtr->CustXcvrInfo.status->isIpuReady;
               bLocalCopy = TRUE;
           }
           else if( bAddr == GTI_I2H_next )
           {
               wValue = pSysPtr->CustXcvrInfo.status->getNextState;
               bLocalCopy = TRUE;
           }
           else
           {
               switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
               {
                   case GTI_TYPE_ANNEX_A:
                   case GTI_TYPE_ANNEX_B:
                       if( (bAddr >= GTI_local_RSI_counter_LOC) && (bAddr < bMaxAddrCounter) )
           {
               bIndex = bAddr - GTI_local_RSI_counter_LOC;
               wValue = pSysPtr->CustXcvrInfo.status->counters[bIndex];
               pSysPtr->CustXcvrInfo.status->counters[bIndex] = 0;
               bLocalCopy = TRUE;
           }
           else if( (bAddr >= GTI_failure_overall_LOC) && (bAddr < bMaxAddrFailure) && (!bDspAccess) )
           {
               bIndex = bAddr - GTI_failure_overall_LOC;
               wValue = pSysPtr->CustXcvrInfo.status->failures[bIndex];
               pSysPtr->CustXcvrInfo.status->failures[bIndex] = 0;
               bLocalCopy = TRUE;
           }
                       break;

                   case GTI_TYPE_ANNEX_C:
                       if( (bAddr >= GTI_local_RSI_counter_C_LOC) && (bAddr < bMaxAddrCounter) )
           {
               bIndex = bAddr - GTI_local_RSI_counter_C_LOC;
               wValue = pSysPtr->CustXcvrInfo.status->counters[bIndex];
               pSysPtr->CustXcvrInfo.status->counters[bIndex] = 0;
               bLocalCopy = TRUE;
           }
           else if( (bAddr >= GTI_failure_overall_C_LOC) && (bAddr < bMaxAddrFailure) && (!bDspAccess) )
           {
               bIndex = bAddr - GTI_failure_overall_C_LOC;
               wValue = pSysPtr->CustXcvrInfo.status->failures[bIndex];
               pSysPtr->CustXcvrInfo.status->failures[bIndex] = 0;
               bLocalCopy = TRUE;
           }
                       break;
           }
           }
           break;
           
       case GTI_PSRAM0W:
           if(bAddr == 0x6 && !bDspAccess)
           {
               wValue = pSysPtr->CustXcvrInfo.status->FramingStructure;
               bLocalCopy = TRUE;
           }

           else if( (bAddr >= 0x6F) && (bAddr <= 0x7A ) )
		   {
			    wValue = pSysPtr->CustXcvrInfo.status->psram0w[bAddr][0];
                wValue|= ((pSysPtr->CustXcvrInfo.status->psram0w[bAddr][1]) << 8);

         		bLocalCopy = TRUE;
		   
		   }

           break;
           
       default:
           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("default\n"));
           bLocalCopy = FALSE;
   }
   
   if( !bLocalCopy)
   {
       size = GET_DSP_WORD_BYTES(bMemType); 
       pMsgInfoPtr->currIdx = size;
       SEND_DSP_MSG(pSysPtr);
       
       ASSERT(size > 1);
       wValue  = pMsgInfoPtr->dataBuff[size - 2];  
       wValue |= (pMsgInfoPtr->dataBuff[size - 1]) << 8; 
   }
   
   /* Simulate Auto Increment!*/
   DoAutoIncrement(pMsgInfoPtr);
   
   return wValue;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_SetWordDspIO
//
// ABSTRACT:
//       It will be called by using macro WRITE_DSP_WORD
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void Gti_SetWordDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bDataL, GTI_BYTE bDataH)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
GTI_BYTE bMemType,bAddr;
DSL_EX *pDslEx;
GTI_BYTE bDspAccess = GetDspAccess( pSysPtr );
GTI_BYTE bLocalCopy = FALSE;
PUSB_PHY pUsbPhy = pSysPtr->CustXcvrInfo.msgInfo->pUsbPhy;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("Gti_SetWordDspIO\n"));

   bMemType = (pMsgInfoPtr->index >> 8) & 0x7F;
   bAddr    = (GTI_BYTE)(pMsgInfoPtr->index & 0x00FF);

   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_WRITE_BLOCK );                           

 
   if( pMsgInfoPtr->State == GTI_ML_STATE_ADDR || pMsgInfoPtr->State == GTI_ML_STATE_WRITE_BLOCK )
   {
       switch(bMemType)
       {
           case GTI_ISRAM:
               pDslEx = (DSL_EX *)pSysPtr->CustXcvrInfo.msgInfo->pDslEx;
               if(bAddr == GTI_EOC_RXBUFF_FLAG_OFFSET && !bDspAccess)
               {
                   pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxFlag = (bDataH<<8) | bDataL;
                   bLocalCopy = TRUE;
               }
               else if(bAddr == GTI_EOC_TXBUFF_FLAG_OFFSET &&  !bDspAccess)
               {
                   pDslEx->EocDspBuffer[pDslEx->CellReadInd].txFlag = (bDataH<<8) | bDataL;
                   bLocalCopy = TRUE;
               }
               else if(bAddr == GTI_EOC_TXBUFF_OFFSET && !bDspAccess)
               {
                   pMsgInfoPtr->type = GTI_WRITE_SUPER_FRAME;
                   pMsgInfoPtr->State = GTI_ML_STATE_WRITE_BLOCK;
                   bLocalCopy = (pMsgInfoPtr->currIdx < 0x22) ? FALSE : TRUE;
               }
               else if ( bAddr == GTI_I2H_ready && !bDspAccess)
               {
                   pSysPtr->CustXcvrInfo.status->isIpuReady = (bDataH<<8) | bDataL;
                   bLocalCopy = TRUE;
               }
               else if(bAddr == GTI_I2H_ready)
               {
                   bLocalCopy = TRUE;
               }
               else
               {
                   switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
                   {
                       case GTI_TYPE_ANNEX_A:
                       case GTI_TYPE_ANNEX_B:
                           if(bAddr == GTI_clrCounters_LOC)
                   bLocalCopy = TRUE;
               else if( (bAddr == GTI_failure_overall_LOC) && (!bDspAccess) )
                   bLocalCopy = TRUE;
                           break;

                       case GTI_TYPE_ANNEX_C:
                           if(bAddr == GTI_clrCounters_C_LOC)
                   bLocalCopy = TRUE;
               else if( (bAddr == GTI_failure_overall_C_LOC) && (!bDspAccess) )
                   bLocalCopy = TRUE;
                           break;
                   }
               }
               break;

           default:
               //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("default\n"));
               break;
       }
       
       if( !(bLocalCopy) )
       {
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bDataL;
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bDataH;
           pMsgInfoPtr->State = GTI_ML_STATE_WRITE_BLOCK;
       }
   }
   else if ( (pMsgInfoPtr->State == GTI_ML_STATE_WRITE_REG) && GetDspAccess(pSysPtr) )
   {
       WRITE_DSP_REG(pSysPtr, GTI_DATAL_DSP, bDataL);
       WRITE_DSP_REG(pSysPtr, GTI_DATAH_DSP, bDataH);
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: gsiSendMsg
//
// ABSTRACT:
//       Sends current Message to the DSP.
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void gsiSendMsg(GTI_XCVR_INFO *pSysPtr, GTI_BYTE type, GTI_WORD value, GTI_WORD index,
                GTI_BYTE *bDataPtr, int wDataLen)
{
GTI_BYTE in;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
   //         ("gsiSendMsg type = %x, value = %x, wDataLen = %x, index = %x\n ",
   //           type,value,wDataLen,index));

   switch (type)
   {
       case GTI_FASTREAD_MEM_REQ:
       case GTI_FASTREAD_REG_REQ:
       case GTI_READ_EEPROM:
#ifdef QUASAR
        case GTI_GET_RT_VERSION:
        case GTI_GET_BC_VERSION:
#endif //QUASAR

           in = GTI_TRUE;
           break;
        
       case GTI_WRITE_AI_BLOCK_REQ:
       case GTI_WRITE_DMA_BLOCK_REQ:
       case GTI_WRITE_REG_REQ:
       case GTI_WRITE_SUPER_FRAME:
       case GTI_START_INTERRUPT_SCAN:
       case GTI_INDICATE_DATAMODE:
       case GTI_INDICATE_RESTART:
       case GTI_INDICATE_RESET:
       case GTI_LOOPBACK_MODE:
       case GTI_USB_LOOPBACK_MODE:
       case GTI_WRITE_EEPROM:  
       case GTI_ISO_BANDWIDTH:   
       case GTI_SET_LOW_SUSPEND_CURRENT:
           in = GTI_FALSE;
           break;
        
       default:
           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
               ("Wrong type specified type = %x\n ", type));
           return;
   }
    
   GTISendUSBCmdEx(pSysPtr->CustXcvrInfo.msgInfo->pUsbPhy,
                       type,
                       value,
                       index,
                       bDataPtr,
                       wDataLen,
                       in);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: gsiSetMsg
//
// ABSTRACT:
//       Initilaize Message object
//       It is called , when macro ML_SET_MSG is called
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void gsiSetMsg(GTI_MSG_LAYER_INFO *pMsgInfoPtr)
{
   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("gsiSetMsg \n "));
   pMsgInfoPtr->type = 0;
   pMsgInfoPtr->value = 0;
   pMsgInfoPtr->index = 0;
   pMsgInfoPtr->currIdx = 0;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_InitUsbMsgIO
//
// ABSTRACT:
//       Initilaize Message object and updates state
//       It is called , when macro ML_SET_MSG is called
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void Gti_InitUsbMsgIO(GTI_XCVR_INFO *pSysPtr)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("Gti_InitUsbMsgIO\n"));

   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   pMsgInfoPtr->State = GTI_ML_STATE_NULL;
   
   /* one-time initialization of message buffers */
   ML_SET_MSG(pSysPtr->CustXcvrInfo.msgInfo);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: gtiGetAccessType
//
// ABSTRACT:
//       iT Sets access type for usb Message.
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void gtiGetAccessType(GTI_XCVR_INFO *pSysPtr, GTI_BYTE cfg, GTI_BYTE isItRead, GTI_BYTE size)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("gtiGetAccessType \n "));

   /* Look at the cfg and set the msg type */
   switch (cfg & GTI_ACCESS_TYPE_FIELD)
   {
       case GTI_NO_AI:     /* 0x00 Normal Access */
       case GTI_AI:        /* 0x08 Auto Increment*/
       case GTI_AP:        /* 0x0c Auto page     */
           pMsgInfoPtr->type = (isItRead) ? GTI_FASTREAD_MEM_REQ : GTI_WRITE_AI_BLOCK_REQ;
           pMsgInfoPtr->value |= size;
           break;

       case GTI_DMA:       /* 0x04 DMA mode      */
           pMsgInfoPtr->type = (isItRead) ? GTI_FASTREAD_MEM_REQ : GTI_WRITE_DMA_BLOCK_REQ;
           pMsgInfoPtr->value |= size;
           break;

       default:
           /* This should never occur */
           break;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: gtiSendMsg
//
// ABSTRACT:
//       Sends message to usb
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void gtiSendMsg(GTI_XCVR_INFO *pSysPtr)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("gtiSendMsg\n"));

   switch(pMsgInfoPtr->type)
   {
       case GTI_FASTREAD_MEM_REQ:
       case GTI_FASTREAD_REG_REQ:
       case GTI_WRITE_AI_BLOCK_REQ:
       case GTI_WRITE_DMA_BLOCK_REQ:
       case GTI_WRITE_REG_REQ:
       case GTI_WRITE_SUPER_FRAME:
       case  GTI_READ_EEPROM:
       case GTI_WRITE_EEPROM:
#ifdef QUASAR	    
       case GTI_GET_RT_VERSION:
       case GTI_GET_BC_VERSION:
#endif

           if ( !(pMsgInfoPtr->currIdx) )
               return;

       case GTI_START_INTERRUPT_SCAN:
       case GTI_INDICATE_DATAMODE:
       case GTI_INDICATE_RESTART:
       case GTI_INDICATE_RESET:
       case GTI_LOOPBACK_MODE:
       case GTI_USB_LOOPBACK_MODE:
       case GTI_ISO_BANDWIDTH:
       case GTI_SET_LOW_SUSPEND_CURRENT:
           gsiSendMsg(pSysPtr ,
                      pMsgInfoPtr->type,
                      pMsgInfoPtr->value,
                      pMsgInfoPtr->index,
                      &pMsgInfoPtr->dataBuff[0],
                      pMsgInfoPtr->currIdx );
           break;
   
       default:
           break;
   }
   pMsgInfoPtr->currIdx=0;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DoAutoIncrement
//
// ABSTRACT:
//       It simulates Auto Increment of DSP Address
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void  DoAutoIncrement( GTI_MSG_LAYER_INFO *pMsgInfoPtr)
{
   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("DoAutoIncrement\n"));
   
   if( (pMsgInfoPtr->cfgData & GTI_ML_AI_MASK) != GTI_NO_AI )
   {
        pMsgInfoPtr->index = (pMsgInfoPtr->index & 0xFF00) | ((pMsgInfoPtr->index  + 1) & 0x00FF);
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_ConfigureDspIO
//
// ABSTRACT:
//       CONFIGURE_DSP macro is calling Gti_ConfigureDspIO function
//       It sends previous message,Init new one and configure it.
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
////////////////////////////////////////////////////////////////////////////////
void Gti_ConfigureDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bData)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("Gti_ConfigureDspIO\n"));

   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_CONFIG );                           

   switch (pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:          
       case GTI_ML_STATE_WRITE_BLOCK:     
       case GTI_ML_STATE_READ_BLOCK:      
           UPDATE_DSP(pSysPtr);
           /* Initialize everything back to NULL and than write the configuration */
           ML_SET_MSG(pMsgInfoPtr);

       case GTI_ML_STATE_WRITE_REG:       
           /* Save the configuration value, bData */
           bData |= GTI_BUS_MODE|GTI_INT_MODE;
           pMsgInfoPtr->cfgData = bData;
           pMsgInfoPtr->cfgId = bId;
           pMsgInfoPtr->value = (bData << 8);
           pMsgInfoPtr->State = GTI_ML_STATE_CONFIG;
           break;

       default:
           break;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Gti_SetAddrDspIO
//
// ABSTRACT:
//       WRITE_DSP_ADDR macro is calling Gti_SetAddrDspIO function
//
// RETURNS:
//       void
//
// ASSUMPTIONS:
//       USB layer is alive.            
//
////////////////////////////////////////////////////////////////////////////////
void Gti_SetAddrDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bAddrL, GTI_BYTE bAddrH)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr    = (pSysPtr->CustXcvrInfo.msgInfo);
GTI_BYTE           isItRead        = (bAddrH & GTI_READ_DSP) ? GTI_TRUE : GTI_FALSE;
GTI_BYTE           mem_type        = bAddrH & 0x7F;
GTI_BYTE           IsRead          = bAddrH & 0x80;   
GTI_BYTE           size            = GET_DSP_WORD_BYTES(mem_type);
GTI_BYTE           cfg             = pMsgInfoPtr->cfgData & GTI_ACCESS_TYPE_FIELD;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_SetAddrDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_ADDR );
   
   switch (pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_ADDR:
           if((mem_type == GTI_IREGA) && (!IsRead) && GetDspAccess(pSysPtr) )
           {
               WRITE_DSP_REG(pSysPtr, GTI_ADDL_DSP, bAddrL);
               WRITE_DSP_REG(pSysPtr, GTI_ADDH_DSP, bAddrH);
           }
           else
           {
               size = GET_DSP_WORD_BYTES(mem_type);
               gtiGetAccessType(pSysPtr, pMsgInfoPtr->cfgData, isItRead, size );
               pMsgInfoPtr->index = (bAddrH << 8) | bAddrL;
               pMsgInfoPtr->State = GTI_ML_STATE_ADDR;
           }
           break;
           
       case GTI_ML_STATE_WRITE_REG:
           CONFIGURE_DSP(pSysPtr, GTI_DSP_A, GTI_NO_AI); 
           WRITE_DSP_REG(pSysPtr, GTI_ADDL_DSP, bAddrL);
           WRITE_DSP_REG(pSysPtr, GTI_ADDH_DSP, bAddrH);
           break;

       case GTI_ML_STATE_WRITE_BLOCK:
           /* Since, we have multiple write addresses, we need to split the msg */
           /* Send a msg and start another msg. */
           SEND_DSP_MSG(pSysPtr);
           pMsgInfoPtr->State = GTI_ML_STATE_NULL;
           CONFIGURE_DSP(pSysPtr, pMsgInfoPtr->cfgId, pMsgInfoPtr->cfgData);

           /* Get All the information-Set the msgType */
           gtiGetAccessType(pSysPtr, pMsgInfoPtr->cfgData, isItRead,
                   (GET_DSP_WORD_BYTES(mem_type)));

           /* Setting the address */
           pMsgInfoPtr->index = (bAddrH << 8) | bAddrL;
           pMsgInfoPtr->State = GTI_ML_STATE_ADDR;
           break;
   }
}

/* ************************************************
** Gti_UpdateDspIO
**
** ************************************************ */
void Gti_UpdateDspIO(GTI_XCVR_INFO *pSysPtr)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,("Gti_UpdateDspIO\n"));
   
   switch (pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           /* Send a msg */
           SEND_DSP_MSG(pSysPtr);

           pMsgInfoPtr->State = GTI_ML_STATE_NULL;
           break;

       case GTI_ML_STATE_ADDR:
           /* We do not have anything to send, just Initialize the structure */
           ML_SET_MSG(pMsgInfoPtr);

           pMsgInfoPtr->State = GTI_ML_STATE_NULL;
           break;

       default:
           break;
   }
}

/* ************************************************
** Gti_SetRegDspIO
**
** ************************************************ */
void Gti_SetRegDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg, GTI_BYTE bData)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
GTI_BYTE ID = pMsgInfoPtr->cfgId;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_SetRegDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_WRITE_REG );                           

   switch(pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
           pMsgInfoPtr->type = GTI_WRITE_REG_REQ;
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = GTI_CONFIG_DSP | GTI_OFFSET_REG;
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = GTI_NO_AI|GTI_BUS_MODE|GTI_INT_MODE;

           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bReg | M_REG_OFFSET(ID);
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bData;

           pMsgInfoPtr->State = GTI_ML_STATE_WRITE_REG;
           break;

       default:
           pMsgInfoPtr->type = GTI_WRITE_REG_REQ;
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bReg | M_REG_OFFSET(ID);
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bData;
           break;
   }
}

/* ************************************************
** Gti_GetRegDspIO
**
** ************************************************ */
GTI_BYTE Gti_GetRegDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bReg)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,("Gti_GetRegDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_READ_REG );                           
   
   switch(pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_NULL:
           /* Intialize the structure */
           ML_SET_MSG(pMsgInfoPtr);

           pMsgInfoPtr->type = GTI_FASTREAD_REG_REQ;
           pMsgInfoPtr->index = bReg;
    
           pMsgInfoPtr->currIdx = 1;   /* Read only one register at a time */

           SEND_DSP_MSG(pSysPtr);
           break;
   }

   return (pMsgInfoPtr->dataBuff[0]);
}

/* ************************************************
** Gti_SetByteDspIO
**
** ************************************************ */
void Gti_SetByteDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg, GTI_BYTE bData)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_SetByteDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_WRITE_BLOCK );                           

   switch(pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
       default:       
           if(bReg ==GTI_MASKTEST_DSP)
           {
               pMsgInfoPtr->type = GTI_WRITE_REG_REQ;
               pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bReg | M_REG_OFFSET(bId);
           }
            
           pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bData;
           pMsgInfoPtr->State = GTI_ML_STATE_WRITE_BLOCK;
           break;
   }
}

/* ************************************************
** Gti_SetBlockDspIO
**
** ************************************************ */
void Gti_SetBlockDspIO(GTI_XCVR_INFO *pSysPtr, GTI_BYTE bId, GTI_BYTE bReg, 
                       GTI_WORD wLen,
                       GTI_BYTE GTI_CONST_QUAL GTI_ADDRESS_QUAL *pStrt)
{
GTI_WORD i;
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,("Gti_SetBlockDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr, GTI_ML_STATE_WRITE_BLOCK );                           

   switch(pMsgInfoPtr->State)
   {
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
           /* Byte to write */
           for(i=0; i < wLen; i++)
           {
               pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = *pStrt++;
           }
           pMsgInfoPtr->State = GTI_ML_STATE_WRITE_BLOCK;
           UPDATE_DSP(pSysPtr);
           break;
   }
}

/* ************************************************
** Gti_GetBlockDspIO
**
** ************************************************ */
void Gti_GetBlockDspIO(GTI_XCVR_INFO *pSysPtr,
                       GTI_BYTE chan, 
                       GTI_WORD wLen, 
                       GTI_BYTE *pStrt)
{
GTI_WORD i;
GTI_MSG_LAYER_INFO *pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
GTI_BYTE bMemType;  
GTI_BYTE bAddr;
GTI_BYTE bMaxAddrCounter;
GTI_BYTE bMaxAddrFailure;
GTI_BYTE bMaxAddrRxEoc;
GTI_BYTE bIndex;
DSL_EX *pDslEx;
GTI_BYTE bLocalCopy = FALSE;
GTI_BYTE size = 1;
GTI_BYTE bDspAccess = GetDspAccess( pSysPtr );
PUSB_PHY pUsbPhy = pSysPtr->CustXcvrInfo.msgInfo->pUsbPhy;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_GetBlockDspIO\n"));
   TRACE_ERRORS(pMsgInfoPtr , GTI_ML_STATE_READ_BLOCK);
   
   pMsgInfoPtr = (pSysPtr->CustXcvrInfo.msgInfo);
   bMemType = (pMsgInfoPtr->index >> 8) & 0x7F;
   bAddr    = (GTI_BYTE)(pMsgInfoPtr->index & 0x00FF);

   if(pMsgInfoPtr->type != GTI_READ_EEPROM)
       size = GET_DSP_WORD_BYTES(bMemType); 

#ifdef QUASAR
   if((pMsgInfoPtr->State != GTI_ML_STATE_ADDR) &&  (pMsgInfoPtr->type != GTI_GET_RT_VERSION) && (pMsgInfoPtr->type != GTI_GET_BC_VERSION)  )
#else
   if(pMsgInfoPtr->State != GTI_ML_STATE_ADDR)
#endif //QUASAR       
       return;
   switch(bMemType)
   {  
        case GTI_ISRAM:
            bMaxAddrRxEoc   = GTI_EOC_RXBUFF_OFFSET + GTI_EOC_MAX_NUMOF_MSG;
            pDslEx = (DSL_EX *)pSysPtr->CustXcvrInfo.msgInfo->pDslEx;

if( (bAddr >= GTI_EOC_RXBUFF_OFFSET) && (bAddr < bMaxAddrRxEoc) && (wLen <= GTI_EOC_MAX_NUMOF_MSG) && !bDspAccess )
            {
                COPY_MEMORY(pStrt,pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxBuff, wLen /* size */ );

                for(i = 0; i < wLen / size; i++ )   
                    pDslEx->EocDspBuffer[pDslEx->CellReadInd].rxBuff[i] = EMPTY_PATTERN;
                bLocalCopy = TRUE;
            }
            else if (bAddr == GTI_OP_CH_LOCATION && !bDspAccess )
            {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->opState;
               bLocalCopy = TRUE;
            }
           else if(bAddr == GTI_XMIT_LOCATION)
           {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->xmitPwr;
               bLocalCopy = TRUE;
           }
           else if(bAddr == GTI_PROGRESS_LOCATION && !bDspAccess)
           {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->startProgress;
               bLocalCopy = TRUE;
           }
           else if ( (bAddr >= 0xf4) && (bAddr <= 0xff) && !bDspAccess )
		    {
               COPY_MEMORY(pStrt, &(pSysPtr->CustXcvrInfo.status->isram[bAddr][0]), wLen /* size*/ );
           	bLocalCopy = TRUE;
		    }
          else if(bAddr == GTI_host_BSCounter_LOC && !bDspAccess)
          {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->BitSwapCounter;
           	bLocalCopy = TRUE;
          }
          else
          {
               switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
               {
                   case GTI_TYPE_ANNEX_A:
                   case GTI_TYPE_ANNEX_B:
                       bMaxAddrCounter = GTI_local_RSI_counter_LOC + GTI_COUNTERS_LEN / sizeof(GTI_WORD);
                       bMaxAddrFailure = GTI_failure_overall_LOC   + GTI_FAILURES_LEN / sizeof(GTI_WORD);

                       if( (bAddr >= GTI_local_RSI_counter_LOC) && (bAddr < bMaxAddrCounter) && (wLen <= GTI_COUNTERS_LEN) && (!bDspAccess) )
                       {
                            COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->counters, wLen /* size*/ );
                            ZERO_MEMORY( pSysPtr->CustXcvrInfo.status->counters, wLen /* size */ );
                            bLocalCopy = TRUE;
                       }
                       else if( (bAddr >= GTI_failure_overall_LOC) && (bAddr < bMaxAddrFailure) && (!bDspAccess) && (wLen <= GTI_FAILURES_LEN) )
                       {
                            COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->failures, wLen /* size*/ );
                            ZERO_MEMORY( pSysPtr->CustXcvrInfo.status->failures, wLen /* size*/ );
                            bLocalCopy = TRUE;
                       }
                       break;

                   case GTI_TYPE_ANNEX_C:
                       bMaxAddrCounter = GTI_local_RSI_counter_C_LOC + GTI_COUNTERS_LEN / sizeof(GTI_WORD);
                       bMaxAddrFailure = GTI_failure_overall_C_LOC   + GTI_FAILURES_LEN / sizeof(GTI_WORD);

                       if( !bDspAccess && (bAddr == GTI_local_RSI_counter_C_LOC) && ( wLen <= GTI_COUNTERS_LEN ) )
                       {
                            COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->counters, wLen /* size*/ );
                            ZERO_MEMORY( pSysPtr->CustXcvrInfo.status->counters, wLen /* size */ );
                            bLocalCopy = TRUE;
                       }
                       else if( !bDspAccess && (bAddr >= GTI_failure_overall_C_LOC) && ( wLen <= GTI_FAILURES_LEN  )  )
                       {
                            COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->failures, wLen /* size*/ );
                            ZERO_MEMORY( pSysPtr->CustXcvrInfo.status->failures, wLen /* size*/ );
                            bLocalCopy = TRUE;
                       }
                       break;
               }
          }
           break;
         
       case GTI_PSRAM0W:
           if(bAddr == 0x6 && !bDspAccess)
           {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->FramingStructure;
               bLocalCopy = TRUE;
           }
           else if( (bAddr >= 0x6F) && (bAddr <= 0x7A ) )
		    {
			    *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->psram0w[bAddr][0];
               *((USHORT*)pStrt) |= ((pSysPtr->CustXcvrInfo.status->psram0w[bAddr][1]) << 8);

         		bLocalCopy = TRUE;
		    }
            break;
         
        case GTI_FDPSRAM6:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
            if( !bDspAccess)
            {
                COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdpram6[bAddr], wLen /* size */ );
                bLocalCopy = TRUE;
            }
            break;
            
               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;
            
        case GTI_FDPSRAM7:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
            if( !bDspAccess)
            {
                COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdpram7[bAddr], wLen /* size */ );
                bLocalCopy = TRUE;
            }
            break;
            
               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;
            
        case GTI_FDPSRAMC: /* 0x74*/
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
            if(bAddr >= (GTI_cpTxToneOrdTbl_LOC & 0xFF) && !bDspAccess)
            {
                COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdpramc[bAddr], wLen /* size */ );
                bLocalCopy = TRUE;
            }
            break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

       case GTI_FDPSRAM0:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           if(bAddr == GTI_cpnoTxBins_LOC && !bDspAccess)
           {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->TxbTotalBinsEx[1];
               *((USHORT*)pStrt) |= ((pSysPtr->CustXcvrInfo.status->TxbTotalBinsEx[2]) << 8);
               bLocalCopy = TRUE;
           }
           else if(bAddr == GTI_cpnoRxBins_LOC && !bDspAccess)
           {
               *((USHORT*)pStrt) = pSysPtr->CustXcvrInfo.status->RxbTotalBinsEx[1];
               *((USHORT*)pStrt) |= ((pSysPtr->CustXcvrInfo.status->RxbTotalBinsEx[2]) << 8);
               bLocalCopy = TRUE;
           }
           break;

               case GTI_TYPE_ANNEX_C:
                   break;
           }
           break;

        case GTI_FDPSRAMA:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
            if( !bDspAccess)
            {
                COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdprama[bAddr], wLen  );
                bLocalCopy = TRUE;
            }
            break;
           }
           break;

        case GTI_FDPSRAME:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			if( !bDspAccess)
			{
            COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdprame[bAddr], wLen  );
				bLocalCopy = TRUE;
			}
            break;
           }
           break;

		case GTI_FDPSRAMF: 
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
            if(bAddr >= (GTI_cpTxToneOrdTblFextHost_LOC & 0xFF) && !bDspAccess)
            {
		        COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->fdpramf[bAddr], wLen  );
				bLocalCopy = TRUE;
           }
			break;
           }
			break;

		case GTI_IRAME:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			if(!bDspAccess)
			{
				COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->irame[bAddr], wLen  );
  		
				bLocalCopy = TRUE;
			}
			break;
           }
			break;

		case GTI_IRAMD:
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
                   break;

               case GTI_TYPE_ANNEX_C:
			if(!bDspAccess)
			{
				COPY_MEMORY(pStrt,pSysPtr->CustXcvrInfo.status->iramd[bAddr], wLen  );
  		
				bLocalCopy = TRUE;
			}
			break;
           }
			break;
         
         default:
            bLocalCopy = FALSE;
            break;
   }
        
   if( !(bLocalCopy) )
   {
        pMsgInfoPtr->currIdx = wLen /* size */;
        SEND_DSP_MSG(pSysPtr);

        /* Copy the data into the host buffer */
        COPY_MEMORY(pStrt,pMsgInfoPtr->dataBuff, wLen /* size*/ );
   }
}


/* ************************************************
** Gti_GetExtBlockDspIO
**
** ************************************************ */
void Gti_GetExtBlockDspIO(GTI_XCVR_INFO *pSysInfo, GTI_BYTE wordSize, GTI_WORD wLen, GTI_BYTE *pBuffer)
{
   Gti_GetBlockDspIO( pSysInfo,wordSize, wLen, pBuffer);
}

/* ***********************************************************************
   *********************************************************************** */
#define FDPRAM_SIZE 3

/* ************************************************
** GTIExecuteXCVR_USB
** (HOST->TSM)
** ************************************************ */
GTI_WORD ExecuteXCVR_USB (GTI_XCVR_INFO *pXcvrInfo,
                           GTI_WORD ipuReady,
                           GTI_WORD nextState) 
{
GTI_WORD result;
GTI_WORD wFrame = 0;
GTI_WORD value, i;
GTIEOC_INFO *eocInfoPtr;
GTI_WORD bTotalBins;
GTI_BYTE Data[FDPRAM_SIZE];
PUSB_PHY pUsbPhy = pXcvrInfo->CustXcvrInfo.msgInfo->pUsbPhy;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("Gti_ExecuteXCVR_USB \n "));

   /* Get the saved structure for EOC */
   eocInfoPtr = (GTIEOC_INFO *)&pXcvrInfo->eocInfo;

   pXcvrInfo->CustXcvrInfo.status->isIpuReady = ipuReady;
   pXcvrInfo->CustXcvrInfo.status->getNextState = nextState;
    
   SetDspAccess(pXcvrInfo, TRUE);
   result = Gti_ExecuteXCVR(pXcvrInfo);
   SetDspAccess(pXcvrInfo, FALSE);

   switch(result)
   {
       case GTI_ACTIVE:
           /* Read the startup progress for correct information on Handshake */
           SetDspAccess(pXcvrInfo, TRUE);

           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_PROGRESS_LOCATION, GTI_ISRAM|GTI_READ_DSP);
           pXcvrInfo->CustXcvrInfo.status->startProgress = READ_DSP_WORD(pXcvrInfo, GTI_DSP_A);

            CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_NO_AI);
            WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_OP_CH_LOCATION, GTI_ISRAM|GTI_READ_DSP);
			pXcvrInfo->CustXcvrInfo.status->opState =  READ_DSP_WORD(pXcvrInfo, GTI_DSP_A);

           SetDspAccess(pXcvrInfo, FALSE);

           /* Indicate the USB controller to poll the DSP */
           SET_DSP_MSG_TYPE(pXcvrInfo,  GTI_START_INTERRUPT_SCAN); 
           UPDATE_DSP(pXcvrInfo);
           break;

       case GTI_COMPLETE_PASS:
           SetDspAccess(pXcvrInfo, TRUE);
           /* Intialize the EOC buffer, because AOC has start and we may have problem
           ** with some of the DSLAM. Also problem with some of the EOC messages as well.
           */
           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x80, GTI_ISRAM);
           for(i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
               WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x0C, 0x0C);
           UPDATE_DSP(pXcvrInfo);
           
		   
           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
 		   WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET,  GTI_ISRAM );
           WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, GTI_EOC_TXBUFF_EMPTY & 0xFF, (GTI_EOC_TXBUFF_EMPTY >> 8) & 0xFF );

           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x06, GTI_PSRAM0W|GTI_READ_DSP);
           pXcvrInfo->CustXcvrInfo.status->FramingStructure = READ_DSP_WORD(pXcvrInfo,  GTI_DSP_A);
           
           switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
           {
               case GTI_TYPE_ANNEX_A:
               case GTI_TYPE_ANNEX_B:
           /* Read the Total Tx Bins for SNR margin */
           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoTxBins_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
           READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A, GTI_DATAL_DSP,3 ,Data);

           pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsEx[0] = Data[0];
           pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsEx[1] = Data[1];
           pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsEx[2] = Data[2];

           /* Read the Total Rx Bins for SNR margin */
           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoRxBins_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
           READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A,GTI_DATAL_DSP,3 ,Data);

           pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsEx[0] = Data[0];
           pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsEx[1] = Data[1];
           pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsEx[2] = Data[2];
                   break;

               case GTI_TYPE_ANNEX_C:
        	CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
            WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoTxBinsFextHost_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
			READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A, GTI_DATAL_DSP,3 ,Data);

			pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsFext[0] = Data[0];
            pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsFext[1] = Data[1];
            pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsFext[2] = Data[2];

        	CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
            WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoRxBinsFextHost_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
			READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A, GTI_DATAL_DSP,3 ,Data);

			pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsFext[0] = Data[0];
            pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsFext[1] = Data[1];
            pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsFext[2] = Data[2];

        	CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
            WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoTxBinsNextHost_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
			READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A, GTI_DATAL_DSP, 3 ,Data);

			pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsNext[0] = Data[0];
            pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsNext[1] = Data[1];
            pXcvrInfo->CustXcvrInfo.status->TxbTotalBinsNext[2] = Data[2];

         	CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
            WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_cpnoRxBinsNextHost_LOC, GTI_FDPSRAM0|GTI_READ_DSP);
			READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A,GTI_DATAL_DSP, 3,Data);

			pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsNext[0] = Data[0];
            pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsNext[1] = Data[1];
            pXcvrInfo->CustXcvrInfo.status->RxbTotalBinsNext[2] = Data[2];

//init rx buffer
            CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_AI);
		    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x87, GTI_PSRAM0W );
		    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x00, 0x00);
		    UPDATE_DSP(pXcvrInfo);
                   break;
           }

           COPY_MEMORY(pUsbPhy->GTISysBlockCpl.paramsDB ,pXcvrInfo->paramsDB,  GTI_RAMSize);
           pUsbPhy->GTISysBlockCpl.CurrentMode      = pXcvrInfo->CurrentMode;

           CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_NO_AI);
           WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_OP_CH_LOCATION, GTI_ISRAM|GTI_READ_DSP);
		   pXcvrInfo->CustXcvrInfo.status->opState =  READ_DSP_WORD(pXcvrInfo, GTI_DSP_A);

           SetDspAccess(pXcvrInfo, FALSE);
           break;

       default:
           break;
   }
   
   return result;
}

/* ************************************************
** SetDataMode_USB
** (HOST->TSM)
** ************************************************ */
void SetDataMode_USB (GTI_XCVR_INFO *pXcvrInfo) 
{
   /* For best performance with the Anchor USB Modem at low bit-rates, 
   ** you must give an indication to the controller as to how much
   ** Iso bandwidth is available
   */
   SET_DSP_MSG_TYPE(pXcvrInfo,  GTI_ISO_BANDWIDTH); 
   UPDATE_DSP(pXcvrInfo);

   /* Indicate dataMode to USB Controller */
   if( (pXcvrInfo->CustXcvrInfo.actionItem != GTI_ABORT_REQ) &&
       (pXcvrInfo->CustXcvrInfo.actionItem != GTI_DISCONNECT_REQ) )
   {
       SET_DSP_MSG_TYPE(pXcvrInfo,  GTI_INDICATE_DATAMODE); 
       UPDATE_DSP(pXcvrInfo);
   }
}

/* ************************************************
** GTISetParamXCVR_USB
** (HOST->TSM)
** ************************************************ */
GTI_WORD GTISetParamXCVR_USB(GTI_XCVR_INFO *pXcvrInfo, struct PARAM_XCVR_ARRAY *ParamTbl)
{
GTI_WORD result;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("GTISetParamXCVR_USB\n"));

   result = Gti_SetParamXCVR(pXcvrInfo, ParamTbl);
   if (result != GTI_OK)
       return result;

   /* Intialize the actionItem to some dummy value */
   pXcvrInfo->CustXcvrInfo.actionItem = 0;
   
   switch(ParamTbl->item[GTI_ACTION_ITEM])
   {
       case GTI_STARTUP_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);

           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);

           break;
           
       case GTI_FAST_RETRAIN_REQ:
           /*to idicate DATA Mode in the end of FR Message 0xD8*/
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);

           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);
           break;
           
       case GTI_PROF_MNGT_REQ:
           InitStatus_USB(pXcvrInfo);
           break;
 
       case GTI_SPECTRUM_REVERB_REQ:
       case GTI_SPECTRUM_MEDLEY_REQ:
       case GTI_SPECTRUM_PILOT_REQ:
       case GTI_SPECTRUM_C_PBIN_REQ:
       case GTI_SPECTRUM_R_PBIN_REQ:
       case GTI_SPECTRUM_REVERB_RX_REQ:
       case GTI_SPECTRUM_C_MTPR_RX_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);
           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);
           break;
 
       case GTI_BIT_SWAPPING_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */

           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);
           break;
        
       case GTI_ANALOG_LB_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);

           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);

       case GTI_DIGITAL_LB_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_TRUE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);

           /* Initialize the USB status parameter */
           InitStatus_USB(pXcvrInfo);
           break;
           
       case GTI_ABORT_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_FALSE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);
           break;
           
       case GTI_DISCONNECT_REQ:
           pXcvrInfo->CustXcvrInfo.status->startupFlag = GTI_FALSE;
           /* Indicate reset to USB Controller */
           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESET);
           UPDATE_DSP(pXcvrInfo);

           SET_DSP_MSG_TYPE(pXcvrInfo, GTI_INDICATE_RESTART);
           UPDATE_DSP(pXcvrInfo);
           break;
           
       default:
           break;
   } //switch(ParamTbl->item[GTI_ACTION_ITEM])

   /* Save the action Item for Abort (which indicates the LED to keep training) */   
   pXcvrInfo->CustXcvrInfo.actionItem = ParamTbl->item[GTI_ACTION_ITEM];
   
   return result;
}

/* ************************************************
** GTISetLoopback_USB
** (HOST->TSM)
** ************************************************ */
void GTISetLoopback(GTI_XCVR_INFO *pSysPtr)
{
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("GTISetLoopback \n "));
/* NOTE: We will not support this function from the driver because
**     this should be part of manufacturing test and driver.
*/
}


/* ************************************************
** GTIEocProc_USB
** (HOST->TSM)
** ************************************************ */



/* Supporting Functions */
void InitStatus_USB(GTI_XCVR_INFO *pXcvrInfo)
{
GTI_BYTE i;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("InitStatus_USB\n"));

   pXcvrInfo->CustXcvrInfo.status->opState = 0;
   pXcvrInfo->CustXcvrInfo.status->startProgress = 0;
   pXcvrInfo->CustXcvrInfo.status->xmitPwr = 0;
   pXcvrInfo->CustXcvrInfo.status->txByteStatus = 0;
   pXcvrInfo->CustXcvrInfo.status->rxByteStatus = 0;
   for(i=0; i< (GTI_COUNTERS_LEN / sizeof(GTI_WORD)); i++)
       pXcvrInfo->CustXcvrInfo.status->counters[i] = 0;
   for(i=0; i < (GTI_FAILURES_LEN / sizeof(GTI_WORD)); i++)
       pXcvrInfo->CustXcvrInfo.status->failures[i] = 0;

   pXcvrInfo->CustXcvrInfo.status->flagCounter = 0;
   pXcvrInfo->CustXcvrInfo.status->flag        = GTI_TRUE;
}

void GTI_ParseDspStatus(GTI_XCVR_INFO *pXcvrInfo,
                           GTI_BYTE *dataPtr)
{
GTI_BYTE i;
GTI_WORD tmpValue;
GTI_STATUS_BLOCK *usbBlockPtr ; 
GTI_BYTE j;

   if(!pXcvrInfo->CustXcvrInfo.msgInfo)
       return;
       
   usbBlockPtr = (GTI_STATUS_BLOCK *)(pXcvrInfo->CustXcvrInfo.status);
   /*
   ** Do NOT collect the status information,
   ** if the state is NOT connected
   */

   /* Currently just place them in the allocated space.
   ** This will use up more memory in the host, but we have
   ** lots of it
   */
   /* Check the group */
#ifdef GTI_PROD_DMT
   switch(*dataPtr)
   {
       case 1:
           
		   pXcvrInfo->CustXcvrInfo.status->opState = ((dataPtr[2] << 8) | dataPtr[1]);
           pXcvrInfo->CustXcvrInfo.status->startProgress = ((dataPtr[8] << 8) | dataPtr[7]);
           pXcvrInfo->CustXcvrInfo.status->xmitPwr = ((dataPtr[10] << 8) | dataPtr[9]);
           break;

       case 3:
           if (lastRcvdStatusBlockId==2)
           {
               tmpValue = ((dataPtr[2] << 8) | dataPtr[1]);
               pXcvrInfo->CustXcvrInfo.status->counters[12] += tmpValue;

               tmpValue = ((dataPtr[4] << 8) | dataPtr[3]);
               pXcvrInfo->CustXcvrInfo.status->counters[13] += tmpValue;

               tmpValue = ((dataPtr[6] << 8) | dataPtr[5]);
               pXcvrInfo->CustXcvrInfo.status->counters[14] += tmpValue;

               tmpValue = ((dataPtr[8] << 8) | dataPtr[7]);
               pXcvrInfo->CustXcvrInfo.status->counters[15] += tmpValue;

               tmpValue = ((dataPtr[10] << 8) | dataPtr[9]);
               pXcvrInfo->CustXcvrInfo.status->counters[16] += tmpValue;

               tmpValue = ((dataPtr[12] << 8) | dataPtr[11]);
               pXcvrInfo->CustXcvrInfo.status->counters[17] += tmpValue;

               tmpValue = ((dataPtr[14] << 8) | dataPtr[13]);
               pXcvrInfo->CustXcvrInfo.status->counters[18] += tmpValue;

               tmpValue = ((dataPtr[16] << 8) | dataPtr[15]);
               pXcvrInfo->CustXcvrInfo.status->counters[19] += tmpValue;

               tmpValue = ((dataPtr[18] << 8) | dataPtr[17]);
               pXcvrInfo->CustXcvrInfo.status->counters[20] += tmpValue;

               tmpValue = ((dataPtr[20] << 8) | dataPtr[19]);
               pXcvrInfo->CustXcvrInfo.status->counters[21] += tmpValue;
           }
           break;

       case 2:
           if (lastRcvdStatusBlockId==1)
           {
               tmpValue = ((dataPtr[2] << 8) | dataPtr[1]);
               pXcvrInfo->CustXcvrInfo.status->counters[0] += tmpValue;

               tmpValue = ((dataPtr[4] << 8) | dataPtr[3]);
               pXcvrInfo->CustXcvrInfo.status->counters[1] += tmpValue;

               tmpValue = ((dataPtr[6] << 8) | dataPtr[5]);
               pXcvrInfo->CustXcvrInfo.status->counters[2] += tmpValue;

               tmpValue = ((dataPtr[8] << 8) | dataPtr[7]);
               pXcvrInfo->CustXcvrInfo.status->counters[3] += tmpValue;

               tmpValue = ((dataPtr[10] << 8) | dataPtr[9]);
               pXcvrInfo->CustXcvrInfo.status->counters[4] += tmpValue;

               tmpValue = ((dataPtr[12] << 8) | dataPtr[11]);
               pXcvrInfo->CustXcvrInfo.status->counters[5] += tmpValue;

               tmpValue = ((dataPtr[14] << 8) | dataPtr[13]);
               pXcvrInfo->CustXcvrInfo.status->counters[6] += tmpValue;

               tmpValue = ((dataPtr[16] << 8) | dataPtr[15]);
               pXcvrInfo->CustXcvrInfo.status->counters[7] += tmpValue;

               tmpValue = ((dataPtr[18] << 8) | dataPtr[17]);
               pXcvrInfo->CustXcvrInfo.status->counters[8] += tmpValue;

               tmpValue = ((dataPtr[20] << 8) | dataPtr[19]);
               pXcvrInfo->CustXcvrInfo.status->counters[9] += tmpValue;

               tmpValue = ((dataPtr[22] << 8) | dataPtr[21]);
               pXcvrInfo->CustXcvrInfo.status->counters[10] += tmpValue;

               tmpValue = ((dataPtr[24] << 8) | dataPtr[23]);
               pXcvrInfo->CustXcvrInfo.status->counters[11] += tmpValue;
           }
           break;

       case 4:
           if (lastRcvdStatusBlockId==3)
           {
               tmpValue = ((dataPtr[2] << 8) | dataPtr[1]);
               pXcvrInfo->CustXcvrInfo.status->failures[0] += tmpValue;

               tmpValue = ((dataPtr[4] << 8) | dataPtr[3]);
               pXcvrInfo->CustXcvrInfo.status->failures[1] += tmpValue;

               tmpValue = ((dataPtr[6] << 8) | dataPtr[5]);
               pXcvrInfo->CustXcvrInfo.status->failures[2] += tmpValue;

               tmpValue = ((dataPtr[8] << 8) | dataPtr[7]);
               pXcvrInfo->CustXcvrInfo.status->failures[3] += tmpValue;

               tmpValue = ((dataPtr[10] << 8) | dataPtr[9]);
               pXcvrInfo->CustXcvrInfo.status->failures[4] += tmpValue;

               tmpValue = ((dataPtr[12] << 8) | dataPtr[11]);
               pXcvrInfo->CustXcvrInfo.status->failures[5] += tmpValue;

               tmpValue = ((dataPtr[14] << 8) | dataPtr[13]);
               pXcvrInfo->CustXcvrInfo.status->failures[6] += tmpValue;

               tmpValue = ((dataPtr[16] << 8) | dataPtr[15]);
               pXcvrInfo->CustXcvrInfo.status->failures[7] += tmpValue;

               tmpValue = ((dataPtr[18] << 8) | dataPtr[17]);
               pXcvrInfo->CustXcvrInfo.status->failures[8] += tmpValue;

               tmpValue = ((dataPtr[20] << 8) | dataPtr[19]);
               pXcvrInfo->CustXcvrInfo.status->failures[9] += tmpValue;

               tmpValue = ((dataPtr[22] << 8) | dataPtr[21]);
               pXcvrInfo->CustXcvrInfo.status->failures[10] += tmpValue;

               tmpValue = ((dataPtr[24] << 8) | dataPtr[23]);
               pXcvrInfo->CustXcvrInfo.status->failures[11] += tmpValue;
           }
           break;
           
       case 5:
           if (lastRcvdStatusBlockId==4)
           {
               tmpValue = ((dataPtr[2] << 8) | dataPtr[1]);
               pXcvrInfo->CustXcvrInfo.status->failures[12] += tmpValue;

               tmpValue = ((dataPtr[4] << 8) | dataPtr[3]);
               pXcvrInfo->CustXcvrInfo.status->failures[13] += tmpValue;
           }
           break;

       case 6:

           
		  j = 0xf4;

		  for (i = 1; i < 24;)
           {
               pXcvrInfo->CustXcvrInfo.status->isram[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->isram[j][1] = dataPtr[i];
               i++;
               
               j++;
           }
		   
		   break;

	   case 7:

		  j = 0x6f;

		  for (i = 1; i < 24;)
           {
               pXcvrInfo->CustXcvrInfo.status->psram0w[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->psram0w[j][1] = dataPtr[i];
               i++;
               
               j++;
           }
		   
		   break;

           case 0x8:
               tmpValue = ((dataPtr[2] << 8) | dataPtr[1]);
               pXcvrInfo->CustXcvrInfo.status->BitSwapCounter = tmpValue;
               break;

       case 0x10: /* 1 */
       case 0x11: /* 2 */
       case 0x12: /* 3 */
       case 0x13: /* 4 */
       case 0x14: /* 5 */
       case 0x15: /* 6 */
       case 0x16: /* 7 */
       case 0x17: /* 8 */
       case 0x18: /* 9 */
       case 0x19: /* 10 */
       case 0x1A: /* 11 */
       case 0x1B: /* 12 */
       case 0x1C: /* 13 */
       case 0x1D: /* 14 */
       case 0x1E: /* 15 */
       case 0x1F: /* 16 */
       case 0x20: /* 17 */
       case 0x21: /* 18 */
       case 0x22: /* 19 */
       case 0x23: /* 20 */
       case 0x24: /* 21 */
       case 0x25: /* 22 */
       case 0x26: /* 23 */
       case 0x27: /* 24 */
       case 0x28: /* 25 */
       case 0x29: /* 26 */
       case 0x2A: /* 27 */
       case 0x2B: /* 28 */
       case 0x2C: /* 29 */
       case 0x2D: /* 30 */
       case 0x2E: /* 31 */
       case 0x2F: /* 32 */
           j = (8 * (dataPtr[0] - 0x10));
           for (i=1; i<24; )
           {
               pXcvrInfo->CustXcvrInfo.status->fdpram6[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpram6[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpram6[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
           break;
         
       case 0x30: /* 1 */
       case 0x31: /* 2 */
       case 0x32: /* 3 */
       case 0x33: /* 4 */
       case 0x34: /* 5 */
       case 0x35: /* 6 */
       case 0x36: /* 7 */
       case 0x37: /* 8 */
       case 0x38: /* 9 */
       case 0x39: /* 10 */
       case 0x3A: /* 11 */
       case 0x3B: /* 12 */
       case 0x3C: /* 13 */
       case 0x3D: /* 14 */
       case 0x3E: /* 15 */
       case 0x3F: /* 16 */
       case 0x40: /* 17 */
       case 0x41: /* 18 */
       case 0x42: /* 19 */
       case 0x43: /* 20 */
       case 0x44: /* 21 */
       case 0x45: /* 22 */
       case 0x46: /* 23 */
       case 0x47: /* 24 */
       case 0x48: /* 25 */
       case 0x49: /* 26 */
       case 0x4A: /* 27 */
       case 0x4B: /* 28 */
       case 0x4C: /* 29 */
       case 0x4D: /* 30 */
       case 0x4E: /* 31 */
       case 0x4F: /* 32 */
           j = (8 * (dataPtr[0] - 0x30));
           for (i=1; i<24;)
           {
               pXcvrInfo->CustXcvrInfo.status->fdpram7[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpram7[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpram7[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
           break;
         
       case 0x50: /* 1 */
       case 0x51: /* 2 */
       case 0x52: /* 3 */
       case 0x53: /* 4 */
       case 0x54: /* 5 */
       case 0x55: /* 6 */
       case 0x56: /* 7 */
       case 0x57: /* 8 */

       case 0x58: /* 1 */
       case 0x59: /* 2 */
       case 0x5A: /* 3 */
       case 0x5B: /* 4 */
       case 0x5C: /* 5 */
       case 0x5D: /* 6 */
       case 0x5E: /* 7 */
       case 0x5F: /* 8 */

           j = 0x80 + (8 * (dataPtr[0] - 0x50));
           for (i=1; i < 24;)
           {
               pXcvrInfo->CustXcvrInfo.status->fdpramc[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpramc[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpramc[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
           break;
      
//ANNEX_C
      case 0x60: //1
      case 0x61: //2
      case 0x62: //3
      case 0x63: //4
      case 0x64: //5
      case 0x65: //6
      case 0x66: //7
      case 0x67: //8
      case 0x68: //9
      case 0x69: //10
      case 0x6A: //11
      case 0x6B: //12
      case 0x6C: //13
      case 0x6D: //14
      case 0x6E: //15
      case 0x6F: //16
      case 0x70: //17
      case 0x71: //18
      case 0x72: //19
      case 0x73: //20
      case 0x74: //21 

		  j = (12 * (dataPtr[0] - 0x60));

		  for (i=1; i<24;)
           {
               pXcvrInfo->CustXcvrInfo.status->iramd[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->iramd[j][1] = dataPtr[i];
               i++;
               
               j++;
           }
         break;
      case 0x75: 
          j = (12 * (dataPtr[0] - 0x60));


          for (i=1; i < 8;)
           {
               pXcvrInfo->CustXcvrInfo.status->iramd[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->iramd[j][1] = dataPtr[i];
               i++;
               
               j++;
           }
         break;
      case 0x76: //1
      case 0x77: //2
      case 0x78: //3
      case 0x79: //4
      case 0x7A: //5
      case 0x7B: //6
      case 0x7C: //7
      case 0x7D: //8
      case 0x7E: //9
      case 0x7F: //10
      case 0x80: //11
      case 0x81: //12
      case 0x82: //13
      case 0x83: //14
      case 0x84: //15
      case 0x85: //16
      case 0x86: //17
      case 0x87: //18
      case 0x88: //19
      case 0x89: //20
      case 0x8A: //21 

          j = (12 * (dataPtr[0] - 0x76));
 
		  for (i=1; i < 24;)
           {
               pXcvrInfo->CustXcvrInfo.status->irame[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->irame[j][1] = dataPtr[i];
               i++;
              
               j++;
           }
         break;
      case 0x8B: 
          j = (12 * (dataPtr[0] - 0x76));


          for (i=1; i < 8;)
           {
               pXcvrInfo->CustXcvrInfo.status->irame[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->irame[j][1] = dataPtr[i];
               i++;
               
               j++;
           }
         break;

	  case 0x8C: // 1 
	  case 0x8D: // 2 
	  case 0x8E: // 3 
	  case 0x8F: // 4 
	  case 0x90: // 5 
	  case 0x91: // 6 
	  case 0x92: // 7 
	  case 0x93: // 8 
	  case 0x94: // 9 
	  case 0x95: // 10
	  case 0x96: // 11
	  case 0x97: // 12
	  case 0x98: // 13
	  case 0x99: // 14
	  case 0x9A: // 15
	  case 0x9B: // 16
	  case 0x9C: // 17
	  case 0x9D: // 18
	  case 0x9E: // 19
	  case 0x9F: // 20
	  case 0xA0: // 21
	  case 0xA1: // 22
	  case 0xA2: // 23
	  case 0xA3: // 24
	  case 0xA4: // 25
	  case 0xA5: // 26
	  case 0xA6: // 27
	  case 0xA7: // 28
	  case 0xA8: // 29
	  case 0xA9: // 30
	  case 0xAA: // 31
	  case 0xAB: // 32
           j = (8 * (dataPtr[0] - 0x8C));
           for (i=1; i < 24; )
           {
               pXcvrInfo->CustXcvrInfo.status->fdprama[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdprama[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdprama[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
         break;

	  case 0xAC: // 1 
	  case 0xAD: // 2 
	  case 0xAE: // 3 
	  case 0xAF: // 4 
	  case 0xB0: // 5 
	  case 0xB1: // 6 
	  case 0xB2: // 7 
	  case 0xB3: // 8 
	  case 0xB4: // 9 
	  case 0xB5: // 10
	  case 0xB6: // 11
	  case 0xB7: // 12
	  case 0xB8: // 13
	  case 0xB9: // 14
	  case 0xBA: // 15
	  case 0xBB: // 16
	  case 0xBC: // 17
	  case 0xBD: // 18
	  case 0xBE: // 19
	  case 0xBF: // 20
	  case 0xC0: // 21
	  case 0xC1: // 22
	  case 0xC2: // 23
	  case 0xC3: // 24
	  case 0xC4: // 25
	  case 0xC5: // 26
	  case 0xC6: // 27
	  case 0xC7: // 28
	  case 0xC8: // 29
	  case 0xC9: // 30
	  case 0xCA: // 31
	  case 0xCB: // 32
           j = (8 * (dataPtr[0] - 0xAC));
           for (i=1; i < 24; )
           {
               pXcvrInfo->CustXcvrInfo.status->fdprame[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdprame[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdprame[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
         break;

	  case 0xCC: // 1 
	  case 0xCD: // 2 
	  case 0xCE: // 3 
	  case 0xCF: // 4 
	  case 0xD0: // 5 
	  case 0xD1: // 6 
	  case 0xD2: // 7 
	  case 0xD3: // 8 
	  case 0xD4: // 9 
	  case 0xD5: // 10
	  case 0xD6: // 11
	  case 0xD7: // 12
	  case 0xD8: // 13
	  case 0xD9: // 14
	  case 0xDA: // 15
	  case 0xDB: // 16
	  case 0xDC: // 17
	  case 0xDD: // 18
	  case 0xDE: // 19
	  case 0xDF: // 20
	  case 0xE0: // 21
	  case 0xE1: // 22
	  case 0xE2: // 23
	  case 0xE3: // 24
	  case 0xE4: // 25
	  case 0xE5: // 26
	  case 0xE6: // 27
	  case 0xE7: // 28
	  case 0xE8: // 29
	  case 0xE9: // 30
	  case 0xEA: // 31
	  case 0xEB: // 32
           j = (8 * (dataPtr[0] - 0xCC));
           for (i=1; i < 24; )
           {
               pXcvrInfo->CustXcvrInfo.status->fdpramf[j][0] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpramf[j][1] = dataPtr[i];
               i++;
               pXcvrInfo->CustXcvrInfo.status->fdpramf[j][2] = dataPtr[i];
               i++;
               
               j++;
           }
         break;
 //ANNEX_C

       default:
           break;
   }
   pXcvrInfo->CustXcvrInfo.status->flag = GTI_TRUE;
   pXcvrInfo->CustXcvrInfo.status->lockTimerFR = GTI_FALSE; //unlock timer during Fast retrain
   lastRcvdStatusBlockId = *dataPtr;
#endif /* GTI_PROD_DMT */
}


void Gti_SetSpecialDspIO(GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE bId, GTI_BYTE bAddrL, GTI_BYTE bAddrH, GTI_BYTE bData)
{
GTI_MSG_LAYER_INFO *pMsgInfoPtr;
GTI_BYTE           isItRead = (bAddrH & GTI_READ_DSP) ? GTI_TRUE : GTI_FALSE;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_SetSpecialDspIO\n"));

   pMsgInfoPtr = (pXcvrInfo->CustXcvrInfo.msgInfo);
   pMsgInfoPtr->dataBuff[pMsgInfoPtr->currIdx++] = bData;
   pMsgInfoPtr->value |= GTI_SPECIAL_CRAM;
           
   gtiGetAccessType(pXcvrInfo, pMsgInfoPtr->cfgData, isItRead, GTI_SPECIAL_CRAM );

   /* Setting the address */
   pMsgInfoPtr->index = (bAddrH << 8) | bAddrL;
   pMsgInfoPtr->State = GTI_ML_STATE_ADDR;
}

void ResetDSP(GTI_XCVR_INFO *pXcvrInfo ,GTI_BYTE bValue)
{

}

GTI_WORD Gti_LocalBusTest(GTI_XCVR_INFO* pXcvrInfo, GTI_DWORD dwStart, GTI_DWORD dwEnd, GTI_DWORD *dwCurAddr)
{
   return TRUE;
}

void Gti_SetTypeDspIO(void *pXcvrParam, GTI_BYTE bType)
{
GTI_XCVR_INFO *pXcvrInfo = (GTI_XCVR_INFO *) pXcvrParam;
GTI_MSG_LAYER_INFO *pMsgInfo = pXcvrInfo->CustXcvrInfo.msgInfo;
PUSB_PHY  UsbPhy = pMsgInfo->pUsbPhy;

   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("Gti_SetTypeDspIO\n"));
   
   ASSERT(pMsgInfo);

   ML_SET_MSG(pMsgInfo);
   pMsgInfo->type = bType;
    
   switch(bType)
   {
       case GTI_READ_EEPROM:
   	    case GTI_WRITE_EEPROM:
#ifdef QUASAR      
            pMsgInfo->index = (EEP_ADDR_HIGH << 8) | EEP_ADDR_LOW;
#endif //QUASAR		    
            pMsgInfo->State = GTI_ML_STATE_ADDR;
           break;
#ifdef QUASAR
        case GTI_GET_RT_VERSION:
        case GTI_GET_BC_VERSION:
			pMsgInfo->State = GTI_ML_STATE_ADDR;
		    break;
#endif   
           
#ifdef QUASAR
        case GTI_SET_LOW_SUSPEND_CURRENT:
           pMsgInfo->value = (UsbPhy->LowSuspendCurrent == 1 ) ? 0x0100 : 0; 
            break;
#endif
            
            
       case GTI_ISO_BANDWIDTH:
           pMsgInfo->State = GTI_ML_STATE_WRITE_BLOCK;
           pMsgInfo->value = 256 -(UsbPhy->IsoReceivePipePacketSize/17);
           break;

       case GTI_INDICATE_DATAMODE:
           pMsgInfo->State = GTI_ML_STATE_WRITE_BLOCK;
           pMsgInfo->value = (UsbPhy->InPipeMode == USB_BULK_MODE) ? 0x100 : 0;
           break;

       default:
           pMsgInfo->State = GTI_ML_STATE_WRITE_BLOCK;
           break;
   }
}

GTI_WORD Gti_SStatusXCVR_USB(GTI_XCVR_INFO *pSys, 
            GTI_WORD nStatusType,        
            void *pStatus,               
            GTI_WORD nLength,            
            GTI_BYTE nChanID )
{
    GTI_WORD result;
    PUSB_PHY pUsbPhy = pSys->CustXcvrInfo.msgInfo->pUsbPhy;

    switch(pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM])
    {
        case GTI_TYPE_ANNEX_C:
   pUsbPhy->GTISysBlockCpl.bActiveProjectId = (UCHAR)pUsbPhy->xcvr[0].item[GTI_ANNEX_TYPE_ITEM]; //KAMAL
           break;
    }
    result = Gti_SStatusXCVR(&pUsbPhy->GTISysBlockCpl, nStatusType, pStatus , nLength, nChanID);
    return result;
}

void Gti_SetProccessToSleep(unsigned int msSleepTime)
{
#ifdef  _NTDDK_
LONG           longTimeout  = (-1) * (msSleepTime * 10000);    // msSleepTime ms
KEVENT         waitEvent;
LARGE_INTEGER  longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);

   KeInitializeEvent( &waitEvent , NotificationEvent, FALSE);
   KeWaitForSingleObject(
        &waitEvent,
        Executive,   
        KernelMode,
        FALSE ,
        &longIntTimeout);
#elif defined(linux)
    /* Convert to millisecs and add one so that it sleeps for 
     * 10 ms even if request is to sleep for less than 10 ms
     */
    LONG    longTimeout  = 1 + (msSleepTime * HZ)/1000; 
	if(!in_interrupt())
	{
    	set_current_state( TASK_INTERRUPTIBLE );
    	schedule_timeout( 1 + longTimeout ); 
	}
	else
	{
		int k;
		for(k=0;k<19990;k++)
			;
	}
#endif //linux
}

void Gti_TraceErrors(GTI_MSG_LAYER_INFO *pMsgInfo ,unsigned int nNewState )
{
   switch(pMsgInfo->State)
   {
       case  GTI_ML_STATE_NULL:  
           Gti_TraceErrorsStateNull(pMsgInfo,nNewState);
           break;
           
       case  GTI_ML_STATE_CONFIG:
           Gti_TraceErrorsStateConfig(pMsgInfo ,nNewState );
           break;
       case  GTI_ML_STATE_ADDR:  
           Gti_TraceErrorsStateAddr(pMsgInfo,nNewState );
           break;
       case  GTI_ML_STATE_WRITE_BLOCK: 
           Gti_TraceErrorsStateWriteBlock(pMsgInfo,nNewState );
           break;
       case  GTI_ML_STATE_READ_BLOCK: 
           Gti_TraceErrorsStateReadBlock(pMsgInfo,nNewState );
           break;
       case  GTI_ML_STATE_WRITE_REG:   
           Gti_TraceErrorsStateWriteReg(pMsgInfo,nNewState );
           break;
       default:
           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("ERROR State %d\n", pMsgInfo->State));
           break;
    }
}

void Gti_TraceErrorsStateNull(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
           break;
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_READ_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           DbgPrint("ERROR from State %d to State %d\n", pMsgInfo->State, nNewState );
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
   }
#endif //WORKING_IN_PROGRESS      
}

void Gti_TraceErrorsStateConfig(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_ADDR:
           break;
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_READ_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           DbgPrint("ERROR from State %d to State %d\n", pMsgInfo->State, nNewState );
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
   }
#endif //WORKING_IN_PROGRESS
}

void Gti_TraceErrorsStateAddr(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_READ_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
    }
#endif //WORKING_IN_PROGRESS
}
void Gti_TraceErrorsStateWriteBlock(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_WRITE_BLOCK:
           break;
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_READ_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           DbgPrint("ERROR from State %d to State %d\n", pMsgInfo->State, nNewState );
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
   }
#endif  //WORKING_IN_PROGRESS
}
void Gti_TraceErrorsStateReadBlock(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_READ_BLOCK:
           break;
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_WRITE_REG:
           DbgPrint("ERROR from State %d to State %d\n", pMsgInfo->State, nNewState );
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
   }
#endif  //WORKING_IN_PROGRESS
}

void Gti_TraceErrorsStateWriteReg(GTI_MSG_LAYER_INFO *pMsgInfo , unsigned int nNewState )
{
#ifdef WORKING_IN_PROGRESS
   switch(nNewState)
   {
       case GTI_ML_STATE_NULL:
       case GTI_ML_STATE_CONFIG:
       case GTI_ML_STATE_WRITE_REG:
           break;
       case GTI_ML_STATE_ADDR:
       case GTI_ML_STATE_WRITE_BLOCK:
       case GTI_ML_STATE_READ_BLOCK:
           DbgPrint("ERROR from State %d to State %d\n", pMsgInfo->State, nNewState );
           break;
       default:
           DbgPrint("ERROR New State %d\n", nNewState);
           break;
   }
#endif //WORKING_IN_PROGRESS
}
