/************************************************************************
 * FILE NAME   : gsiusb.c replaced gsihic.c                             *
 * DESCRIPTION : This file contains all the host interface control      *
 *               rotuines for the GSI NOVA XCVR chip                    *
 *                                                                      *
 * COMMENT     :                                                        *
 *                                                                      *
 * (C) Copyright 1998-2002 by GlobespanVirata Inc                       *
 ************************************************************************/
#ifdef _NTDDK_
#ifdef GSI_USB
#include "wdm.h"
#include "usb100.h"
#include "usbdi.h"
#include "usbdlib.h"
#include "common.h"
#include "debug.h"
#include "atmphy.h"
#include "usbphy.h"
#endif // GSI_USB
#elif defined(linux)
#include "Gp.h"
#include "gti.h"
#include "gtidsp.h"
#include "dsl.h"
#endif // linux

void GSISetXcvrParameters(PUSB_PHY Adapter, UCHAR totalXcvr);
USHORT GSICallXcvr(PUSB_PHY Adapter, UCHAR action, UCHAR xcvrNum);
ULONG GSIGetStatusXcvr(PUSB_PHY Adapter, USHORT *statusField, GTI_WORD size,
                     UCHAR opCode, UCHAR xcvrNum);
extern GTI_WORD GTISetParamXCVR_USB(GTI_XCVR_INFO *pXcvrInfo, struct PARAM_XCVR_ARRAY *ParamTbl);


/************************************************************************
* Function     : GSISetXcvrParameters
* Description  : Get called during initailization time and will be used
*                 when SetParamXCVR gets called. This routine gets called once
*
* Input        : Adapter: A pointer ot our adapter information structure.
*                totalXcvr: Total number of XCVR being used.
* Output       : None
* Return       : None
************************************************************************/
void GSISetXcvrParameters(
                          PUSB_PHY Adapter,
                          UCHAR totalXcvr)
{
   UCHAR i;

   for (i=0; i<totalXcvr; i++)
   {
#ifdef GTI_CAP
/* CAP Options ONLY */
      Adapter->xcvr[i].length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;

      Adapter->xcvr[i].item[GTI_ACTION_ITEM]=0;   /* action to be replaced */
      Adapter->xcvr[i].item[GTI_MODE_ITEM] = GTI_ATU_R;
      Adapter->xcvr[i].item[GTI_DWN_CONST_ITEM] = GTI_8_CONST;
      Adapter->xcvr[i].item[GTI_UP_CONST_ITEM] = GTI_8_CONST;
      Adapter->xcvr[i].item[GTI_MARGIN_ITEM] = GTI_NO_CHANGE_MARGIN;
      Adapter->xcvr[i].item[GTI_REMOTE_MARGIN_ITEM] = GTI_NO_CHANGE_MARGIN;
      Adapter->xcvr[i].item[GTI_BAUD_ITEM] = GTI_340_BAUD;
      Adapter->xcvr[i].item[GTI_INITIATE_ITEM] = GTI_DEFAULT_INITIATE;
      Adapter->xcvr[i].item[GTI_TX_POWER_ITEM] = GTI_FULL_TX;
      Adapter->xcvr[i].item[GTI_REMOTE_TX_POWER_ITEM] = GTI_NO_CHANGE_RMT_TX;
      Adapter->xcvr[i].item[GTI_INTERLEAVE_DEPTH_ITEM] = GTI_NO_CHANGE_INTERLEAVE;
      Adapter->xcvr[i].item[GTI_UPSTREAM_BAUD_ITEM] = GTI_136_UP_BAUD;
      Adapter->xcvr[i].item[GTI_FRAMING_MODES_ITEM] = GTI_UTOPIA_L1_FRAMING;
      Adapter->xcvr[i].item[GTI_A2D_D2A_ITEM] = GTI_SLADE_A2D_TYPE;
      Adapter->xcvr[i].item[GTI_LINE_INTERFACE_ITEM] = GTI_ELANTEC_INTERFACE;
      Adapter->xcvr[i].item[GTI_POWER_DOWN_ITEM] = GTI_PWR_DN_DISABLE;
      Adapter->xcvr[i].item[GTI_CLOCK_TYPE_ITEM] = GTI_CLOCK_CRYSTAL;
      Adapter->xcvr[i].item[GTI_DOWN_TX_FILTER_ITEM] = GTI_NO_CHANGE_TX_FILTER;
      Adapter->xcvr[i].item[GTI_UP_TX_FILTER_ITEM] = GTI_NO_CHANGE_TX_FILTER;
      Adapter->xcvr[i].item[GTI_STANDBY_DETECTION_ITEM] = GTI_NO_CHANGE_STANDBY;
      Adapter->xcvr[i].item[GTI_FIX_BIT_ITEM] = GTI_NO_CHANGE_FIX_BIT;
      Adapter->xcvr[i].item[GTI_UtopiaL2TxAddr_ITEM] = 0x0;
      Adapter->xcvr[i].item[GTI_UtopiaL2RxAddr_ITEM] = 0x0;
#endif /* GTI_CAP */

#ifdef GTI_DMT
/* DMT OPtions ONLY */
/* ATU-R/CPE  */
       Adapter->xcvr[i].length                     = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;
       Adapter->xcvr[i].item[GTI_ACTION_ITEM]      = 0;
       Adapter->xcvr[i].item[GTI_INITIATE_ITEM]    = GTI_DEFAULT_INITIATE;
#endif /* GTI_DMT */
   }
}

/************************************************************************
* Function     : GSIInitXcvrParameters
* Description  : Get called during initailization time and will be used
*                 when SetParamXCVR gets called. This routine gets called once
*
* Input        : Adapter: A pointer ot our adapter information structure.
*                totalXcvr: Total number of XCVR being used.
* Output       : None
* Return       : None
************************************************************************/
void GSIInitXcvrParameters(
                          PUSB_PHY Adapter,
                          UCHAR totalXcvr)
{
	UCHAR i;
	gsi_config_t   *pCfg;

	pCfg = &(Adapter->AdapterHandle->config);
    Adapter->IsoMaxBWLimit = USB_ISO_RATE_DEFAULT;
    Adapter->maxDownRateEnable = 0;
    Adapter->maxDownRate = 0xfff;
    Adapter->autoMode = AUTOMODE_DISABLE;
    //Adapter->autoMode = AUTOMODE_ENABLE;
    Adapter->SelectStandard = SELECT_STANDARD_DISABLE;
    Adapter->RetrainEnable = RETRAIN_ENABLE;
    Adapter->MaxCrcErrorsPerSecThreshold = 
        DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD;
    Adapter->DownstreamRateToMaxCrcErrorsPerMinDivider = 
        DEFAULT_DOWNSTREAM_RATE_TO_MAX_CRC_ERRORS_PER_MIN_DIVIDER;
    Adapter->EocVendorIdHigh = DEFAULT_EOC_VENDOR_ID_HIGH;
    Adapter->EocVendorIdLow = DEFAULT_EOC_VENDOR_ID_LOW;
    Adapter->DataDelay = 0;
    Adapter->LowSuspendCurrent = 0;
    Adapter->VendorId = GTI_ID_GLOBESPAN;

	for (i=0; i<totalXcvr; i++)
	{
#ifdef GTI_CAP
		/* CAP Options ONLY */
		Adapter->xcvr[i].length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;
		/* action to be replaced */
		Adapter->xcvr[i].item[GTI_ACTION_ITEM]=GTI_STARTUP_REQ; 
		Adapter->xcvr[i].item[GTI_MODE_ITEM] = GTI_ATU_R;
		Adapter->xcvr[i].item[GTI_DWN_CONST_ITEM] = GTI_8_CONST;
		Adapter->xcvr[i].item[GTI_UP_CONST_ITEM] = GTI_8_CONST;
		Adapter->xcvr[i].item[GTI_MARGIN_ITEM] = GTI_NO_CHANGE_MARGIN;
		Adapter->xcvr[i].item[GTI_REMOTE_MARGIN_ITEM] = GTI_NO_CHANGE_MARGIN;
		Adapter->xcvr[i].item[GTI_BAUD_ITEM] = GTI_340_BAUD;
		Adapter->xcvr[i].item[GTI_INITIATE_ITEM] = GTI_DEFAULT_INITIATE;
		Adapter->xcvr[i].item[GTI_TX_POWER_ITEM] = GTI_FULL_TX;
		Adapter->xcvr[i].item[GTI_REMOTE_TX_POWER_ITEM] = GTI_NO_CHANGE_RMT_TX;
		Adapter->xcvr[i].item[GTI_INTERLEAVE_DEPTH_ITEM] = GTI_NO_CHANGE_INTERLEAVE;
		Adapter->xcvr[i].item[GTI_UPSTREAM_BAUD_ITEM] = GTI_136_UP_BAUD;
		Adapter->xcvr[i].item[GTI_FRAMING_MODES_ITEM] = GTI_UTOPIA_L1_FRAMING;
		Adapter->xcvr[i].item[GTI_A2D_D2A_ITEM] = GTI_SLADE_A2D_TYPE;
		Adapter->xcvr[i].item[GTI_LINE_INTERFACE_ITEM] = GTI_ELANTEC_INTERFACE;
		Adapter->xcvr[i].item[GTI_POWER_DOWN_ITEM] = GTI_PWR_DN_DISABLE;
		Adapter->xcvr[i].item[GTI_CLOCK_TYPE_ITEM] = GTI_CLOCK_CRYSTAL;
		Adapter->xcvr[i].item[GTI_DOWN_TX_FILTER_ITEM] = GTI_NO_CHANGE_TX_FILTER;
		Adapter->xcvr[i].item[GTI_UP_TX_FILTER_ITEM] = GTI_NO_CHANGE_TX_FILTER;
		Adapter->xcvr[i].item[GTI_STANDBY_DETECTION_ITEM] = GTI_NO_CHANGE_STANDBY;
		Adapter->xcvr[i].item[GTI_FIX_BIT_ITEM] = GTI_NO_CHANGE_FIX_BIT;
		Adapter->xcvr[i].item[GTI_UtopiaL2TxAddr_ITEM] = 0x0;
		Adapter->xcvr[i].item[GTI_UtopiaL2RxAddr_ITEM] = 0x0;
#endif /* GTI_CAP */

#ifdef GTI_DMT
#ifdef ANNEXA
		Adapter->xcvr[i].length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;
        Adapter->xcvr[i].item[GTI_ACTION_ITEM] = GTI_STARTUP_REQ;
        Adapter->xcvr[i].item[GTI_MODE_ITEM] = GTI_ATU_R;
        Adapter->xcvr[i].item[GTI_INITIATE_ITEM] = GTI_DEFAULT_INITIATE;
        Adapter->xcvr[i].item[GTI_DATA_INTERFACE_TYPE_ITEM] = GTI_UTOPIA_L1_FRAMING;
        Adapter->xcvr[i].item[GTI_TRELLIS_ITEM] = GTI_TRELLIS_ENABLE;
        Adapter->xcvr[i].item[GTI_EC_FDM_ITEM] = GTI_FDM_MODE;
        Adapter->xcvr[i].item[GTI_STANDARD_ITEM] = GTI_MULTIMODE_STANDARD;
        Adapter->xcvr[i].item[GTI_MAX_BITS_PER_BIN_ITEM] = 0xE; /* 14 */
        Adapter->xcvr[i].item[GTI_TX_START_BIN_ITEM] = 0x06;
        Adapter->xcvr[i].item[GTI_TX_END_BIN_ITEM] = 0x1F;
        Adapter->xcvr[i].item[GTI_LINE_DR_STBY_ITEM] = GTI_LINE_DR_STDBY_DISABLE;
        Adapter->xcvr[i].item[GTI_RX_START_BIN_ITEM] = 0x20;
        Adapter->xcvr[i].item[GTI_RX_END_BIN_ITEM] = 0xFF;
        Adapter->xcvr[i].item[GTI_RX_BIN_ADJUST_ITEM] = GTI_RX_BIN_DISABLE;
        Adapter->xcvr[i].item[GTI_TX_POWER_ATTENUATION_ITEM] = GTI_0DB_TX_PWR;
        Adapter->xcvr[i].item[GTI_NTR_ITEM] = GTI_NTR_DISABLE;
        Adapter->xcvr[i].item[GTI_PMODE_ITEM] = GTI_DISABLE_PMODE;
        Adapter->xcvr[i].item[GTI_CODING_GAIN_ITEM] = GTI_AUTO_CODING_GAIN;
        Adapter->xcvr[i].item[GTI_TONE_ORDERING_ITEM] = GTI_ENABLE_TONE_ORDERING;
        Adapter->xcvr[i].item[GTI_FRAMING_STRUCTURE_ITEM] = GTI_FRAMING_TYPE_3;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_TX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_RX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_CLOCK_TYPE_ITEM] = GTI_CLOCK_CRYSTAL;
        Adapter->xcvr[i].item[GTI_VENDOR_REV_SHORT_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_EXPANDED_EXCHANGE_ITEM] = GTI_EXPANDED_EXCHANGE;
        Adapter->xcvr[i].item[GTI_ESCAPE_FAST_RETRAIN_ITEM] = GTI_ESCAPE_DISABLE;
        Adapter->xcvr[i].item[GTI_FAST_RETRAIN_ENABLE_ITEM] = GTI_DISABLE_FAST_RETRAIN;
        Adapter->xcvr[i].item[GTI_BIT_SWAP_ITEM] = GTI_BIT_SWAP_ENABLE;
        Adapter->xcvr[i].item[GTI_US_BIT_SWAP_ITEM] = GTI_US_BIT_SWAP_DISABLE;
        Adapter->xcvr[i].item[GTI_ERASE_PROFILES_ITEM] = GTI_NO_ERASE_PROFILES;
        Adapter->xcvr[i].item[GTI_TRIGGER_MODE_ITEM] = GTI_DISABLE_TGR;
        Adapter->xcvr[i].item[GTI_ANNEX_TYPE_ITEM] = GTI_ANNEX_A_ENABLE;
        Adapter->xcvr[i].item[GTI_ANNEX_C_MODE_ITEM] = GTI_ANNEX_C_FBM;
        Adapter->xcvr[i].item[GTI_ANNEX_C_PILOT_TONE_ITEM] = GTI_PILOT_NEXT_ENABLE;
        Adapter->xcvr[i].item[GTI_TX_BIN_ADJUST_ITEM] = GTI_TX_BIN_ENABLE;
        Adapter->xcvr[i].item[GTI_PILOT_TONE_REQUEST_ITEM] = GTI_ENABLE_PILOT_TONE;
        Adapter->xcvr[i].item[GTI_DUAL_LATENCY_ITEM] = GTI_DISABLE_DUAL_LATENCY;
#ifndef	MONACO		
        Adapter->xcvr[i].item[GTI_HW_REV_ITEM] = GTI_HW_REV_ZERO;
        Adapter->xcvr[i].item[GTI_CUTBACK_ITEM] = GTI_CUTBACK_ENABLE;
#else
        Adapter->xcvr[i].item[GTI_HW_REV_ITEM] = GTI_HW_REV_ONE;
        Adapter->xcvr[i].item[GTI_CUTBACK_ITEM] = GTI_CUTBACK_DISABLE;
#endif		
        Adapter->xcvr[i].item[GTI_ANNEX_CH_OFFSET_TTR_R_ITEM] = GTI_TTR_R_OFFSET_0;

#ifdef QUASAR
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_64;
#else
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_256;
#endif // QUASAR

#endif // ANNEXA
#ifdef ANNEXB
		Adapter->xcvr[i].length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;
        Adapter->xcvr[i].item[GTI_ACTION_ITEM] = GTI_STARTUP_REQ;
        Adapter->xcvr[i].item[GTI_MODE_ITEM] = GTI_ATU_R;
        Adapter->xcvr[i].item[GTI_INITIATE_ITEM] = GTI_DEFAULT_INITIATE;
        Adapter->xcvr[i].item[GTI_DATA_INTERFACE_TYPE_ITEM] = GTI_UTOPIA_L1_FRAMING;
        Adapter->xcvr[i].item[GTI_TRELLIS_ITEM] = GTI_TRELLIS_ENABLE;
        Adapter->xcvr[i].item[GTI_EC_FDM_ITEM] = GTI_FDM_MODE;
        Adapter->xcvr[i].item[GTI_STANDARD_ITEM] = GTI_G_DMT_STANDARD;
        Adapter->xcvr[i].item[GTI_MAX_BITS_PER_BIN_ITEM] = 0xE; /* 14 */
        Adapter->xcvr[i].item[GTI_TX_START_BIN_ITEM] = 0x20;
        Adapter->xcvr[i].item[GTI_TX_END_BIN_ITEM] = 0x3f;
        Adapter->xcvr[i].item[GTI_LINE_DR_STBY_ITEM] = GTI_LINE_DR_STDBY_DISABLE;
        Adapter->xcvr[i].item[GTI_RX_START_BIN_ITEM] = 0x40;
        Adapter->xcvr[i].item[GTI_RX_END_BIN_ITEM] = 0xFF;
        Adapter->xcvr[i].item[GTI_RX_BIN_ADJUST_ITEM] = GTI_RX_BIN_DISABLE;
        Adapter->xcvr[i].item[GTI_TX_POWER_ATTENUATION_ITEM] = GTI_0DB_TX_PWR;
        Adapter->xcvr[i].item[GTI_NTR_ITEM] = GTI_NTR_DISABLE;
        Adapter->xcvr[i].item[GTI_PMODE_ITEM] = GTI_ENABLE_PMODE;
        Adapter->xcvr[i].item[GTI_CODING_GAIN_ITEM] = GTI_AUTO_CODING_GAIN;
        Adapter->xcvr[i].item[GTI_TONE_ORDERING_ITEM] = GTI_ENABLE_TONE_ORDERING;
        Adapter->xcvr[i].item[GTI_FRAMING_STRUCTURE_ITEM] = GTI_FRAMING_TYPE_3;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_TX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_RX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_CLOCK_TYPE_ITEM] = GTI_CLOCK_CRYSTAL;
        Adapter->xcvr[i].item[GTI_VENDOR_REV_SHORT_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_EXPANDED_EXCHANGE_ITEM] = GTI_EXPANDED_EXCHANGE;
        Adapter->xcvr[i].item[GTI_ESCAPE_FAST_RETRAIN_ITEM] = GTI_ESCAPE_DISABLE;
        Adapter->xcvr[i].item[GTI_FAST_RETRAIN_ENABLE_ITEM] = GTI_DISABLE_FAST_RETRAIN;
        Adapter->xcvr[i].item[GTI_BIT_SWAP_ITEM] = GTI_BIT_SWAP_ENABLE;
        Adapter->xcvr[i].item[GTI_US_BIT_SWAP_ITEM] = GTI_US_BIT_SWAP_DISABLE;
        Adapter->xcvr[i].item[GTI_ERASE_PROFILES_ITEM] = GTI_NO_ERASE_PROFILES;
        Adapter->xcvr[i].item[GTI_TRIGGER_MODE_ITEM] = GTI_DISABLE_TGR;
        Adapter->xcvr[i].item[GTI_ANNEX_TYPE_ITEM] = GTI_TYPE_ANNEX_B;
        Adapter->xcvr[i].item[GTI_ANNEX_C_MODE_ITEM] = GTI_ANNEX_C_FBM;
        Adapter->xcvr[i].item[GTI_ANNEX_C_PILOT_TONE_ITEM] = GTI_PILOT_NEXT_ENABLE;
        Adapter->xcvr[i].item[GTI_TX_BIN_ADJUST_ITEM] = GTI_TX_BIN_ENABLE;
        Adapter->xcvr[i].item[GTI_PILOT_TONE_REQUEST_ITEM] = GTI_ENABLE_PILOT_TONE;
        Adapter->xcvr[i].item[GTI_DUAL_LATENCY_ITEM] = GTI_DISABLE_DUAL_LATENCY;
        Adapter->xcvr[i].item[GTI_HW_REV_ITEM] = GTI_HW_REV_ZERO;
        Adapter->xcvr[i].item[GTI_CUTBACK_ITEM] = GTI_CUTBACK_DISABLE;
        Adapter->xcvr[i].item[GTI_ANNEX_CH_OFFSET_TTR_R_ITEM] = GTI_TTR_R_OFFSET_0;
#ifdef QUASAR
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_64;
#else
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_256;
#endif // QUASAR
#endif // ANNEXB
#ifdef ANNEXC
		Adapter->xcvr[i].length = GTI_NUM_OF_ATU_R_CMD_PARAMETERS;
        Adapter->xcvr[i].item[GTI_ACTION_ITEM] = GTI_STARTUP_REQ;
        Adapter->xcvr[i].item[GTI_MODE_ITEM] = GTI_ATU_R;
        Adapter->xcvr[i].item[GTI_INITIATE_ITEM] = GTI_DEFAULT_INITIATE;
        Adapter->xcvr[i].item[GTI_DATA_INTERFACE_TYPE_ITEM] = GTI_UTOPIA_L1_FRAMING;
        //Adapter->xcvr[i].item[GTI_TRELLIS_ITEM] = GTI_TRELLIS_ENABLE;
        Adapter->xcvr[i].item[GTI_EC_FDM_ITEM] = GTI_EC_MODE;
        Adapter->xcvr[i].item[GTI_STANDARD_ITEM] = GTI_G_DMT_STANDARD;
        Adapter->xcvr[i].item[GTI_MAX_BITS_PER_BIN_ITEM] = 0xE; /* 14 */
        Adapter->xcvr[i].item[GTI_TX_START_BIN_ITEM] = 0x06;
        Adapter->xcvr[i].item[GTI_TX_END_BIN_ITEM] = 0x1F;
        Adapter->xcvr[i].item[GTI_LINE_DR_STBY_ITEM] = GTI_LINE_DR_STDBY_DISABLE;
        Adapter->xcvr[i].item[GTI_RX_START_BIN_ITEM] = 0x20;
        Adapter->xcvr[i].item[GTI_RX_END_BIN_ITEM] = 0xFF;
        Adapter->xcvr[i].item[GTI_RX_BIN_ADJUST_ITEM] = GTI_RX_BIN_DISABLE;
        Adapter->xcvr[i].item[GTI_TX_POWER_ATTENUATION_ITEM] = GTI_0DB_TX_PWR;
        Adapter->xcvr[i].item[GTI_NTR_ITEM] = GTI_NTR_DISABLE;
        Adapter->xcvr[i].item[GTI_PMODE_ITEM] = GTI_ENABLE_PMODE;
        Adapter->xcvr[i].item[GTI_CODING_GAIN_ITEM] = GTI_AUTO_CODING_GAIN;
        Adapter->xcvr[i].item[GTI_TONE_ORDERING_ITEM] = GTI_ENABLE_TONE_ORDERING;
        Adapter->xcvr[i].item[GTI_FRAMING_STRUCTURE_ITEM] = GTI_FRAMING_TYPE_3;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_TX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_UTOPIA_L2_RX_ADDR_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_CLOCK_TYPE_ITEM] = GTI_CLOCK_CRYSTAL;
        //Adapter->xcvr[i].item[GTI_VENDOR_REV_SHORT_ITEM] = 0x0;
        Adapter->xcvr[i].item[GTI_EXPANDED_EXCHANGE_ITEM] = GTI_EXPANDED_EXCHANGE;
        Adapter->xcvr[i].item[GTI_ESCAPE_FAST_RETRAIN_ITEM] = GTI_ESCAPE_DISABLE;
        Adapter->xcvr[i].item[GTI_FAST_RETRAIN_ENABLE_ITEM] = GTI_DISABLE_FAST_RETRAIN;
        Adapter->xcvr[i].item[GTI_BIT_SWAP_ITEM] = GTI_BIT_SWAP_ENABLE;
        Adapter->xcvr[i].item[GTI_US_BIT_SWAP_ITEM] = GTI_US_BIT_SWAP_DISABLE;
        Adapter->xcvr[i].item[GTI_ERASE_PROFILES_ITEM] = GTI_NO_ERASE_PROFILES;
        Adapter->xcvr[i].item[GTI_TRIGGER_MODE_ITEM] = GTI_DISABLE_TGR;
        Adapter->xcvr[i].item[GTI_ANNEX_TYPE_ITEM] = GTI_ANNEX_C_ENABLE;
        Adapter->xcvr[i].item[GTI_ANNEX_C_MODE_ITEM] = GTI_ANNEX_C_DBM;
        Adapter->xcvr[i].item[GTI_ANNEX_C_PILOT_TONE_ITEM] = GTI_PILOT_NEXT_DISABLE;
        Adapter->xcvr[i].item[GTI_TX_BIN_ADJUST_ITEM] = GTI_TX_BIN_ENABLE;
        Adapter->xcvr[i].item[GTI_PILOT_TONE_REQUEST_ITEM] = GTI_ENABLE_PILOT_TONE;
        Adapter->xcvr[i].item[GTI_DUAL_LATENCY_ITEM] = GTI_DISABLE_DUAL_LATENCY;
        Adapter->xcvr[i].item[GTI_HW_REV_ITEM] = GTI_HW_REV_ZERO;
        Adapter->xcvr[i].item[GTI_CUTBACK_ITEM] = GTI_CUTBACK_ENABLE;
        Adapter->xcvr[i].item[GTI_ANNEX_CH_OFFSET_TTR_R_ITEM] = GTI_TTR_R_OFFSET_42;
#ifdef QUASAR
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_64;
#else
        Adapter->xcvr[i].item[GTI_MAX_D_CP_ITEM] = GTI_MAX_D_CP_256;
#endif // QUASAR

#endif // ANNEXC
#endif /* GTI_DMT */
	}
}

/************************************************************************
* Function     : GSICallXcvr
* Description  : Calls the ParamXCVR and ExcuteXCVR with appropriate action
*
* Input        : Adapter: A pointer ot our adapter information structure.
*              : action: Type of Parameter to set
*              : xcvrNum: Which XCVR to take this action on.
* Output       : None
* Return       : return the result from the ExecuteXCVR()
************************************************************************/
USHORT GSICallXcvr(
                   PUSB_PHY Adapter,
                   UCHAR action,
                   UCHAR xcvrNum)
{
   USHORT result;

   Adapter->xcvr[xcvrNum].item[GTI_ACTION_ITEM] = action;
#ifdef GTI_DMT
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("Standard=%x\n",Adapter->xcvr[0].item[GTI_STANDARD_ITEM]));
#ifndef ANNEXC
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("Trellis=%x\n",Adapter->xcvr[0].item[GTI_TRELLIS_ITEM]));
#endif /* ANNEXC */
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("TxPowerAtten=%x\n",Adapter->xcvr[0].item[GTI_TX_POWER_ATTENUATION_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("CodingGain=%x\n",Adapter->xcvr[0].item[GTI_CODING_GAIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("MaxBitsPerBin=%x\n",Adapter->xcvr[0].item[GTI_MAX_BITS_PER_BIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("TxStartBin=%x\n",Adapter->xcvr[0].item[GTI_TX_START_BIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("TxEndBin=%x\n",Adapter->xcvr[0].item[GTI_TX_END_BIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("RxStartBin=%x\n",Adapter->xcvr[0].item[GTI_RX_START_BIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("RxEndBin=%x\n",Adapter->xcvr[0].item[GTI_RX_END_BIN_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("RxAdjustBin=%x\n",Adapter->xcvr[0].item[GTI_RX_BIN_ADJUST_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("FramingStruct=%x\n",Adapter->xcvr[0].item[GTI_FRAMING_STRUCTURE_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("ExpandXchange=%x\n",Adapter->xcvr[0].item[GTI_EXPANDED_EXCHANGE_ITEM]));
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("ClockType=%x\n",Adapter->xcvr[0].item[GTI_CLOCK_TYPE_ITEM]));
#endif /* GTI_DMT */
   result = GTISetParamXCVR_USB(&Adapter->GTISysBlock[xcvrNum], &Adapter->xcvr[xcvrNum]);
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      ("Set Parameter XCVR is finished with result value = %x\n",result));
   return result;
}
