////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.4 $
//     $Modtime: 8/30/02 4:57p $
//     $Author: kattre $
//
// ABSTRACT:
//		Contains the implementation of the USB ATM Phy module
//
////////////////////////////////////////////////////////////////////////////////
#ifdef _NTDDK_
#include "wdm.h"
#include "usb100.h"
#include "usbdi.h"
#include "usbdlib.h"
#ifdef QUASAR
#include "ndis.h"
#include "firmware.h"
#endif //QUASAR
#elif defined(linux)
#include "Gp.h"
#include "GULinux.h"
#ifdef QUASAR
#include "firmware.h"
#endif //QUASAR
#endif // linux
#include "common.h"
#include "debug.h"
#include "atmphy.h"
#include "usbphy.h"
#include "usbcdc.h"

#ifdef GSI_USB
#include "gtidsp.h"
#include "dsl.h"
#include "gti_eoc.h"

#ifdef _NTDDK_
extern void WriteConfigReg(PUSB_PHY UsbPhy);
#endif // _NTDDK_
static NTSTATUS UsbPhyResetParentPort(void *UsbPhyParam);
extern BOOLEAN UsbMsgLayerInit(GTI_XCVR_INFO *pGTISysBlock, PUSB_PHY pUsbPhy);
extern void UsbMsgLayerClose(GTI_XCVR_INFO *pGTISysBlock);
extern GTI_WORD ExecuteXCVR_USB(GTI_XCVR_INFO *,
                        GTI_WORD isIpuReady,
                        GTI_WORD getNextState);
extern void SetDataMode_USB (GTI_XCVR_INFO *pXcvrInfo);
void UsbPhyPokeEEPROM( PUSB_PHY UsbPhy);

extern GTI_WORD Gti_SStatusXCVR_USB(GTI_XCVR_INFO *pSys, 
            GTI_WORD nStatusType,        
            void *pStatus,               
            GTI_WORD nLength,            
            GTI_BYTE nChanID );

#endif

//------------------------------------------------------------------------------
//
// Local Constant Definitions
//
//------------------------------------------------------------------------------
#define USB_PHY_MAX_CONFIGURATION_DESCRIPTOR_SIZE	1024
#define USB_PHY_MAX_STRING_DESCRIPTOR_SIZE			256
#define USB_PHY_DATA_INTERFACE_NUMBER_OF_PIPES		2
#ifdef _NTDDK_
#define USB_PHY_INTERRUPT_PIPE_RESOURCE_COUNT       2
#elif defined(linux)
#define USB_PHY_INTERRUPT_PIPE_RESOURCE_COUNT		1
#endif // linux
#define NUM_OF_CELLS_TX              7


#define USB_PHY_NO_DEVICE_CLASS      0xFF
#define USB_PHY_NO_DEVICE_SUBCLASS   0xFF
#define USB_PHY_NO_DEVICE_PROTOCOL   0xFF 
#define USB_PHY_INTERFACE_COUNT      1
#define USB_PHY_SERIAL_NUMBER_INDEX  4
#define USB_PHY_DATA_INTERFACE_INDEX 0
#define USB_PHY_PIPES_COUNT          3

#define MAX_TIME_2_SELECT_STANDARD                 3 /* 3 Seconds */
#define MAX_NCDX_ERRORS_PER_SEC                    32 // 32 errors

#ifdef GTI_CAP
#define  GSI_MAX_UNCORRECTED_RS     24
#define  GSI_MAX_LINE_LIMIT         14
#define  GSI_MAX_FAILURES           8

USHORT MEAN_SQ_ERROR_TABLE[] =
{
   43651,   34673,   27542,   21877,   17378,   13803,   10964,
   8709,    6918,    5495,    4365,    3467,    2754,    2187,
   1737,    1380,    1096,    870,     691,     549,     436,
   346,     275,     218,     174,     138,     109,     87,
   69,      55,      44,      35,      27,      22
};
#endif /* GTI_CAP */

#ifdef GSI_USB
#define  GTI_MAX_STATUS_LEN         0xFF
extern void GSISetXcvrParameters(PUSB_PHY Adapter, UCHAR totalXcvr);
extern USHORT GSICallXcvr(PUSB_PHY Adapter, UCHAR action, UCHAR xcvrNum);

#ifdef GSIREG
NTSTATUS
ReadConfigurationFromReg( PUSB_PHY Adapter);
#endif // GSIREG
#endif /* GSI_USB */


//------------------------------------------------------------------------------
//
// Extermal Function Declarations
//
//------------------------------------------------------------------------------
extern void GSISetXcvrParameters(PUSB_PHY Adapter, UCHAR totalXcvr);
extern USHORT GSICallXcvr(PUSB_PHY Adapter, UCHAR action, UCHAR xcvrNum);
#ifdef WINHIP
void WhipProtocolInit(void);
#endif

void UsbPhyCplReq(PUSB_PHY UsbPhy, UCHAR request);

#ifdef POWERMGMT
extern void CompletePwrSuspend( PVOID pAtmuni);
extern void StopIrpAndTimersForPwrSuspend( PVOID pAtmuni);
extern void CompletePwrResume( PVOID pAtmuni, BOOLEAN Success);
#endif


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyBind
//
// ABSTRACT:
//		Called by the Phy user to establish a binding with the USB phy
//		layer
//
// RETURNS:
//	   ATM_PHY_STATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
ATM_PHY_STATUS
UsbPhyBind
(
	PATM_PHY_BIND_INPUT		BindInput,
	PATM_PHY_BIND_OUTPUT	BindOutput
)
{
	PUSB_PHY    		UsbPhy;
	NTSTATUS    		NtStatus;
	ATM_PHY_STATUS		PhyStatus;
	PURB				urbPtr;
	PDSP_PHY_QWORK_ITEM qworkItemPtr;
	UCHAR				i;


	ASSERT( BindInput);
	ASSERT( BindOutput);

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyBind\n"));


	//
	// Allocate our instance data structure
	//
	ALLOCATE_MEMORY( &UsbPhy, sizeof( USB_PHY));

	if ( NULL == UsbPhy) 
	{
		return( ATM_PHY_STATUS_RESOURCES);
	}

    //
	// Initialize spin locks
	//
	KeInitializeSpinLock( &(UsbPhy->BulkReceiveResourceListLock));
	KeInitializeSpinLock( &(UsbPhy->InterruptResourceListLock));
#if defined(linux)
	KeInitializeSpinLock( &(UsbPhy->BulkTransmitResourceListLock));
#endif // linux
	KeInitializeSpinLock( &(UsbPhy->UsbStateSpinLock));
	KeInitializeSpinLock( &(UsbPhy->DspStateSpinLock));
	KeInitializeSpinLock( &(UsbPhy->DspLock));
	KeInitializeSpinLock( &(UsbPhy->DspBufferListLock));

	//
	// Save the bind input parameters
	//
	UsbPhy->UserHandle 				= BindInput->UserHandle;
	UsbPhy->UserReceive				= BindInput->UserReceive;
	UsbPhy->UserSendComplete		= BindInput->UserSendComplete;
	UsbPhy->UserLinkStatusChange		= BindInput->UserLinkStatusChange;
    	UsbPhy->UserCPLGetATMStats          	= BindInput->UserCPLGetATMStats;
    	UsbPhy->UserCPLClearATMStats        	= BindInput->UserCPLClearATMStats;

	UsbPhy->ReceiveDataSize			= BindInput->ReceiveDataSize;

	UsbPhy->IsoReceiveBufferSize		= BindInput->PhyRxBufferSize * UsbPhy->ReceiveDataSize;
	UsbPhy->IsoReceiveResourceCount	= BindInput->PhyRxResourceCount;
#if (defined(linux))
	UsbPhy->AdapterHandle 			= (adapter_t *)(BindInput->PhyDeviceHandle);
	UsbPhy->AdapterHandle->UsbPhyHandle = UsbPhy;
	UsbPhy->BulkTransmitResourceCount	= BindInput->PhyTxResourceCount;
#elif (defined(_NTDDK_))
    	UsbPhy->TopOfStackDeviceObject      = (PDEVICE_OBJECT)(BindInput->PhyDeviceHandle);
 #endif	/* OS Type */
 	UsbPhy->BulkReceiveBufferSize		= BindInput->PhyRxBufferSize * PHY_RX_MAX_DATA_SIZE;
	UsbPhy->BulkReceiveResourceCount	= BindInput->PhyRxResourceCount;

	UsbPhy->InterruptResourceCount	= USB_PHY_INTERRUPT_PIPE_RESOURCE_COUNT;
	UsbPhy->InterruptBufferSize	  	= MAX_INTERRUPT_PIPE_SIZE;
#ifdef _NTDDK_
	UsbPhy->NeedQueueDspProcess			= TRUE;
#elif defined(linux)
	UsbPhy->NeedQueueDspProcess		= DSP_START;
#endif // linux
	UsbPhy->ShutdownAck				= FALSE;
	UsbPhy->dspState					= DSP_PHY_IDLE;    // DSP state
	UsbPhy->LQAutoModeCount			= 0;    // forces restart after MAX_SELECT_TIME seconds
	UsbPhy->LQFailureCount			= 0;	// Triggers a DSL RESTART after 1 failure (1500ms)
	UsbPhy->CheckDslStatus			= FALSE;
#if defined(linux)
	UsbPhy->PipeAborting				= 0;
#endif // linux
	UsbPhy->ActualDsDataRate			= USB_ALT_SETTINGS_INDEX_DEFAULT;


	UsbMsgLayerInit(UsbPhy->GTISysBlock , UsbPhy);


	// Allocate DSP buffers
	INITIALIZE_LIST_HEAD( &UsbPhy->DspProcessFreeList);
	INITIALIZE_LIST_HEAD( &UsbPhy->DspProcessCommandList);
#ifdef WINHIP
    NdisInitializeEvent( &UsbPhy->WhipEvent);
    NdisSetEvent( &UsbPhy->WhipEvent);
#endif // WINHIP
	for (i=0; i < QWORK_ITEM_LEN; i++)
	{
		ALLOCATE_MEMORY( &qworkItemPtr, sizeof(DSP_PHY_QWORK_ITEM));

		if (qworkItemPtr)
		{
			INSERT_HEAD_LIST( &UsbPhy->DspProcessFreeList, (PLIST_NODE)qworkItemPtr);
		}
		else
		{
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("Could not allocate buffers for DSP\n"));

			SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
			UsbPhyUnbind( UsbPhy);
			return( ATM_PHY_STATUS_FAILURE);
		}
	}

	// Allocate UrbPtr
	ALLOCATE_URB(&urbPtr, sizeof(struct _URB_CONTROL_VENDOR_OR_CLASS_REQUEST));
	if (urbPtr == NULL)
	{
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, 
			("Could not Allocate URB buffer \n"));
		SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
		UsbPhyUnbind( UsbPhy);
		return( ATM_PHY_STATUS_FAILURE);
	}

	// save the information
	UsbPhy->dspFW.urbPtr  = urbPtr;

#ifdef _NTDDK_
	// Read registry settings
	if (!NT_SUCCESS( ReadConfigurationFromReg( UsbPhy) ))
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			 (" Read Configuration from Registry failed\n"));

		SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
		UsbPhyUnbind( UsbPhy);
		return( ATM_PHY_STATUS_FAILURE);
	}
#elif defined(linux)
	GSIInitXcvrParameters( UsbPhy, 1);
#endif // _NTDDK_


	// Initialize the USB Phy device
	NtStatus = UsbPhyInitialize( UsbPhy, TRUE);
	if ( !NT_SUCCESS( NtStatus)) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("UsbPhyInitialize failed\n"));

		SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
		UsbPhyUnbind( UsbPhy);
		return( ATM_PHY_STATUS_FAILURE);
	}

	//
	// Fill in the bind output parameters
	//
	BindOutput->PhyHandle			= UsbPhy;
	BindOutput->PhySend				= UsbPhySend;
	BindOutput->PhyUnbind			= UsbPhyUnbind;

#ifdef WINHIP
	WhipProtocolInit();
#endif

	COPY_MEMORY(
		BindOutput->PhyUniqueId,
		UsbPhy->AtmEndSystemIdentifier,
		UNIQUE_ID_SIZE);

	// start the DSP processing function with startup
	SetDspState(UsbPhy, DSP_PHY_IDLE, DSP_INTERRUPT);

	return( ATM_PHY_STATUS_SUCCESS);

} // UsbPhyBind

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyUnbind
//
// ABSTRACT:
//		Called by the Phy user to remove a binding previously established with
//		UsbPhyBind
//
// RETURNS:
//	   	void
//
// ASSUMPTIONS:
//		The USB Phy has been successfully bound using UsbPhyBind
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyUnbind
(
    PUSB_PHY UsbPhy
)
{
#ifdef _NTDDK_
    LONG				longTimeout = (-1) * (50 * 10000);
    LARGE_INTEGER		longIntTimeout;
    KEVENT				resetEvent;
#elif defined(linux)
    LONG				longTimeout = (50 * HZ) / 1000; // 50 ms timer
#endif // linux
	PDSP_PHY_QWORK_ITEM	qworkItemPtr;


	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("> UsbPhyUnbind\n"));

	// send dying gasp and cleanup resources
	SetUsbState (UsbPhy, USB_PHY_STATE_SHUTDOWN);
	if (!UsbPhy->ShutdownAck)	// dont wait on event if already shutdown
	{
#ifdef _NTDDK_
		KeInitializeEvent(&resetEvent , NotificationEvent, FALSE);
		longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);

#endif // _NTDDK_
		// wait for completion of the shut down process 
		while (!UsbPhy->ShutdownAck)
		{
			// wait 50 ms
#ifdef _NTDDK_
			KeWaitForSingleObject(
				&resetEvent,
				Executive,   
				KernelMode,
				FALSE ,
				&longIntTimeout );
#elif defined(linux)
			set_current_state( TASK_INTERRUPTIBLE );
			schedule_timeout( longTimeout );
#endif // linux
		}
	}

#if defined(linux)
	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("Wait for Shutdown Ended Ack = %d\n",UsbPhy->ShutdownAck));

	//kill the dsp thread
	UsbPhy->NeedQueueDspProcess = DSP_STOP;
	wake_up_interruptible(&(UsbPhy->AdapterHandle->StatusCheckQ));
	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("Entering Wait for Finishing of Thread\n"));
	while (UsbPhy->NeedQueueDspProcess != DSP_STOPPED)
	{
		// wait 50 ms
		set_current_state( TASK_INTERRUPTIBLE );
		schedule_timeout( longTimeout );
	}
	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("Thread Finished\n"));
#endif // linux
	//
	// Free DSP buffers
	//
	while ( !IS_LIST_EMPTY( &UsbPhy->DspProcessFreeList) )
	{
		qworkItemPtr = (PDSP_PHY_QWORK_ITEM) REMOVE_TAIL_LIST( &UsbPhy->DspProcessFreeList);
		FREE_MEMORY( qworkItemPtr, sizeof( DSP_PHY_QWORK_ITEM));
	}

	while ( !IS_LIST_EMPTY( &UsbPhy->DspProcessCommandList) )
	{
		qworkItemPtr = (PDSP_PHY_QWORK_ITEM) REMOVE_TAIL_LIST( &UsbPhy->DspProcessCommandList);
		FREE_MEMORY( qworkItemPtr, sizeof( DSP_PHY_QWORK_ITEM));
	}

    UsbMsgLayerClose(UsbPhy->GTISysBlock);

	// Free the allcated memory
	if ( UsbPhy->dspFW.urbPtr )
		FREE_URB( UsbPhy->dspFW.urbPtr, sizeof(struct _URB_CONTROL_VENDOR_OR_CLASS_REQUEST) );

	FREE_MEMORY( UsbPhy, sizeof( USB_PHY));

	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("< UsbPhyUnbind\n"));

} // UsbPhyUnbind


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyInitialize
//
// ABSTRACT:
//    	Carries out the steps required to bring the USB Phy to a working state,
//		including verifying all of the various descriptors provided by the
//		device.
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyInitialize
(
	PUSB_PHY UsbPhy,
	BOOLEAN InitStartup
)
{
	NTSTATUS							NtStatus = STATUS_SUCCESS;
#ifdef _NTDDK_
	PUSB_DEVICE_DESCRIPTOR				DeviceDescriptor;
	PUSB_STRING_DESCRIPTOR				StringDescriptor;
	PUSB_CONFIGURATION_DESCRIPTOR		ConfigurationDescriptor;
	PUSBD_INTERFACE_LIST_ENTRY			InterfaceList;
	PUSBD_INTERFACE_INFORMATION			DataInterface;
	PUSB_INTERFACE_DESCRIPTOR			DataInterfaceDescriptor;
#elif defined(linux)
	struct usb_device_descriptor			*DeviceDescriptor = NULL;
	struct usb_config_descriptor				*ConfigurationDescriptor = NULL;
	struct usb_interface					*InterfaceList = NULL;
	struct usb_interface_descriptor			*DataInterfaceDescriptor = NULL;
	struct usb_device						*usbdev = NULL;
	int									nLength = 0;
	WCHAR								szSerialNumber[32];
#endif // linux
	UCHAR								AltSetting;
	BOOLEAN								SetDateRate = FALSE;
	ULONG                               ulStatus;
	USHORT								MaxDownRateTable[] = {	USB_ISO_RATE_ALT0,
																USB_ISO_RATE_ALT1,
																USB_ISO_RATE_ALT2,
																USB_ISO_RATE_ALT3,
																USB_ISO_RATE_ALT4,
																USB_ISO_RATE_ALT5	};

#ifdef QUASAR
    UCHAR                               Data[GTI_VERSION_SIZE];
#endif //QUASAR

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyInitialize\n"));


	// Assume device will become operational
	SetUsbState (UsbPhy, USB_PHY_STATE_OPERATIONAL);
#ifdef _NTDDK_
	NtStatus = UsbPhyGetStatusOfParentPort(UsbPhy, &ulStatus);

	if ( !NT_SUCCESS( NtStatus)) 
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unable to reset USB port\n"));
		return( STATUS_UNSUCCESSFUL);
		}



	if(  !((ulStatus & USBP_PORT_ENABLED) &&  (ulStatus & USBP_PORT_CONNECTED)) )
	{
		NtStatus = UsbPhyResetParentPort(UsbPhy);
	
		if ( !NT_SUCCESS( NtStatus)) 
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unable to reset USB port\n"));
		return( STATUS_UNSUCCESSFUL);
		}

	}

    //
	// Get the device descriptor
	//
	DeviceDescriptor = UsbPhyGetDeviceDescriptor( UsbPhy);

	if ( NULL == DeviceDescriptor) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read device descriptor\n"));
		return( STATUS_UNSUCCESSFUL);
	}

   UsbPhy->UsbVendorID = (USHORT)DeviceDescriptor->idVendor;
   UsbPhy->UsbProductID = (USHORT)DeviceDescriptor->idProduct;
   UsbPhy->UsbDeviceID = (USHORT)DeviceDescriptor->bcdDevice;
   
	//
	// Check the device descriptor

#elif defined(linux)
	usbdev = UsbPhy->AdapterHandle->usbDev;
	DeviceDescriptor = &(usbdev->descriptor);
#endif // linux
	//
	// Check the device class, device subclass, and device protocol fields
	//
	if ( (DeviceDescriptor->bDeviceClass != USB_PHY_NO_DEVICE_CLASS      ) ||
	     (DeviceDescriptor->bDeviceSubClass != USB_PHY_NO_DEVICE_SUBCLASS) ||
		 (DeviceDescriptor->bDeviceProtocol != USB_PHY_NO_DEVICE_PROTOCOL) )
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Device descriptor isn't compatible\n"));
#ifdef _NTDDK_
		UsbPhyFreeDeviceDescriptor( DeviceDescriptor);
#endif // _NTDDK_
		return( STATUS_UNSUCCESSFUL);
	}
#ifdef _NTDDK_
	UsbPhyFreeDeviceDescriptor( DeviceDescriptor);
#endif // _NTDDK_


	//
	// Get the configuration descriptor
	//
#ifdef _NTDDK_
	// A call to read the configuraton descriptor returns not only the
	// configuration descriptor, but also all of the interface descriptors
	// and endpoint descriptors (including class and vendor specific descriptors
	// related to the standard descriptors).
	//
	ConfigurationDescriptor = UsbPhyGetConfigurationDescriptor( UsbPhy);
#elif defined(linux)
	ConfigurationDescriptor = usbdev->actconfig;
#endif // linux
	if ( NULL == ConfigurationDescriptor) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_TRACE,
			("Unable to read configuration descriptor\n"));
#ifdef _NTDDK_
		    return( STATUS_UNSUCCESSFUL);
#elif defined(linux)
		usbdev->descriptor.bNumConfigurations = GSFW_NUM_CONFIG_SUPPORTED;
		if( usb_get_configuration(usbdev) < 0 )
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Error reading configuration descriptor\n"));
			return( STATUS_UNSUCCESSFUL);
		}
#endif // linux
	}
#if defined(linux)
	if( usb_set_configuration( usbdev,  usbdev->config[0].bConfigurationValue ) < 0 )
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to set configuration \n"));
		return( STATUS_UNSUCCESSFUL);
	}
	
	ConfigurationDescriptor = usbdev->actconfig;
#endif // linux
	
	//
	// Check the configuration descriptor
	//
	// If the number of interfaces provided by the device doesn't match the
	// number of interfaces we are expecting, return failure
	//
	// The device should have two interfaces: a communications class interface
	// and a data class interface.
	//
	if ( ConfigurationDescriptor->bNumInterfaces != USB_PHY_INTERFACE_COUNT ) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Device has wrong # of interfaces\n"));
#ifdef _NTDDK_
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
#endif // _NTDDK_
		return( STATUS_UNSUCCESSFUL);
	}
#ifdef _NTDDK_
	//
	// Get the (first) language ID supported by the device.
	// The language ID is required when retrieving subsequent string
	// descriptors.
	//
	NtStatus = UsbPhyGetLanguageId( UsbPhy,&(UsbPhy->LanguageId));

	if ( !NT_SUCCESS( NtStatus)) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read the Language ID\n"));
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
		return( STATUS_UNSUCCESSFUL);
	}

	StringDescriptor =
		UsbPhyGetStringDescriptor(
			UsbPhy,
			USB_PHY_SERIAL_NUMBER_INDEX ,
			UsbPhy->LanguageId);

	if ( NULL == StringDescriptor) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read End System Identifier String Descriptor\n"));
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
		return( STATUS_UNSUCCESSFUL);
	}

	//
	// The ESI string must be exactly 14 Unicode characters long
	// ( or 28 bytes).  The string length is obtained by subtracting two
	// from the descriptor length.
	//
	if ( (StringDescriptor->bLength-2) != 28 ) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("ESI String Descriptor has invalid length (%d bytes)\n",
			(StringDescriptor->bLength-2)));
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
		return( STATUS_UNSUCCESSFUL);
	}

	UsbPhyFreeStringDescriptor( StringDescriptor);

#elif defined(linux)
	/* No need to explicitely get the string ID in linux as it is retrieved automaticaly the first time */
	nLength = UsbPhyGetStringDescriptor(
			UsbPhy,
			USB_PHY_SERIAL_NUMBER_INDEX ,
			UsbPhy->LanguageId,
			szSerialNumber,
			sizeof(szSerialNumber));

	if ( nLength != CDC_ESI_SIZE * 2 ) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read End System Identifier String Descriptor %d\n",
			nLength));
		return( STATUS_UNSUCCESSFUL);
	}

	//
	// Convert the Unicode string of hex digits to bytes
	//
	UsbPhyUnicodeStringHexToBytes(
		szSerialNumber,
		UsbPhy->AtmEndSystemIdentifier,
		CDC_ESI_SIZE);
#endif // linux
	// Get the interface list
	//
	// The interface list is a data structure used when configuring the device
	// through USBD.  It contains an entry for each device interface, with
	// each entry containing both standard USB interface information as well
	// as USBD specific interface information.
	//
	//
#ifdef _NTDDK_
	InterfaceList = UsbPhyGetInterfaceList( UsbPhy,ConfigurationDescriptor);
#elif defined(linux)
	InterfaceList = UsbPhy->AdapterHandle->usbDev->actconfig->interface;
#endif // linux
	if ( NULL == InterfaceList) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to get the interface list\n"));
#ifdef _NTDDK_
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
#endif // _NTDDK_
		return( STATUS_UNSUCCESSFUL);
	}

// ****************** BEGIN:  Iso Bandwidth negotiation ******************* //

	// Search for starting AltSetting according to registry
	for (AltSetting = 0; AltSetting < MAX_DOWN_RATE_TABLE_SIZE; AltSetting++)
	{
		if (MaxDownRateTable[AltSetting] == UsbPhy->IsoMaxBWLimit)
			break;
	}

	// If bad registry value default to max setting
	if (AltSetting == MAX_DOWN_RATE_TABLE_SIZE)
	{
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("Max setting selected\n"));
		AltSetting = 1;
	}

	// Start from the AltSetting, which was determined
	for (; AltSetting <= MAX_DOWN_RATE_TABLE_SIZE; AltSetting++)
	{
		if (AltSetting == MAX_DOWN_RATE_TABLE_SIZE)
			AltSetting = 0;

        DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Alt Setting Iteration %d %d \n", AltSetting, UsbPhy->ActualDsDataRate));
		// Save the actual amount of Iso bandwidth that will be used by the device
		UsbPhy->ActualIsoBwUsed = AltSetting;
#ifdef _NTDDK_
		DataInterfaceDescriptor =
			USBD_ParseConfigurationDescriptorEx(
		        ConfigurationDescriptor,
		        ConfigurationDescriptor,
		        USB_PHY_DATA_INTERFACE_INDEX,
		        AltSetting,
		        -1,
		        -1,
		        -1);

		if ( NULL == DataInterfaceDescriptor)
        {
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Device has no data class interfaces alternate setting %u\n",AltSetting));

			UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
			UsbPhyFreeInterfaceList( InterfaceList);
			return( STATUS_UNSUCCESSFUL);
		}

		InterfaceList[USB_PHY_DATA_INTERFACE_INDEX].InterfaceDescriptor = DataInterfaceDescriptor;
		UsbPhy->DataInterfaceOnAlternateSetting = AltSetting;

		// Configure the device
		NtStatus = UsbPhyConfigureDevice(
						UsbPhy,
						ConfigurationDescriptor,
						InterfaceList);
#elif defined(linux)
		DataInterfaceDescriptor = &(InterfaceList[ USB_PHY_DATA_INTERFACE_INDEX].altsetting[AltSetting]);

		UsbPhy->DataInterfaceOnAlternateSetting = AltSetting;

		// Configure the device
		if( usb_set_interface( usbdev, 
			InterfaceList[ USB_PHY_DATA_INTERFACE_INDEX].altsetting[0].bInterfaceNumber, 
			AltSetting) < 0 )
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Couldn't Select required interface\n"));

			return( STATUS_UNSUCCESSFUL);
		}
                NtStatus = STATUS_SUCCESS;
#endif // linux
		if ( (!SetDateRate) && (NT_SUCCESS( NtStatus)) )
		{
			DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            	("Setting Data Rate\n"));
			SetDateRate = TRUE;
			UsbPhy->xcvr[0].item[GTI_MAX_DOWN_RATE_ITEM] = MaxDownRateTable[AltSetting];
           
			if ( GTI_TYPE_ANNEX_C == UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] )
           {
               /* FIXME: This change is temporarly...
           ** For AnnexC release, the MaxDownRate is 8 times as much
           ** Calculation done in bits instead of Byte.
           ** This will be changed in the firmware next time.
           */
           if (MaxDownRateTable[AltSetting] != USB_ISO_RATE_ALT1)
			    UsbPhy->xcvr[0].item[GTI_MAX_DOWN_RATE_ITEM] = (MaxDownRateTable[AltSetting] * 8);
           }
		}

		if ( (NT_SUCCESS( NtStatus)) && (!AltSetting) )
		{
            DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
                ("Breaking due to BULK \n"));
			break;	// Bulk mode success
		}
		else if ( (NT_SUCCESS( NtStatus)) && (UsbPhy->ActualDsDataRate > 0) && (AltSetting >= UsbPhy->ActualDsDataRate) )
		{
            DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
                ("Breaking due to ISO \n"));
			break;	// found a suitable ISO setting
		}
		else if (!AltSetting)
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unable to configure the device\n"));
#ifdef _NTDDK_
			UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
			UsbPhyFreeInterfaceList( InterfaceList);
#endif // _NTDDK_
			return( NtStatus);
		}
	}
	
	if (AltSetting != 0)	// ISO mode
	{
		UsbPhy->InPipeMode = USB_ISO_MODE;
		UsbPhy->RxCellPadding = 0;
	}
	else					// Bulk mode (Pad bulk receive cells:  64 - cell size)
	{
		UsbPhy->InPipeMode = USB_BULK_MODE;
		UsbPhy->RxCellPadding = PHY_RX_MAX_DATA_SIZE - UsbPhy->ReceiveDataSize;
	}

	// **************************** END:  Iso Bandwidth negotiation *************************** //

	//
	// Configure the pipes
	//
#ifdef _NTDDK_
	DataInterface = InterfaceList[USB_PHY_DATA_INTERFACE_INDEX].Interface;
#elif defined(linux)
	DataInterfaceDescriptor = 
		&(InterfaceList[ USB_PHY_DATA_INTERFACE_INDEX].altsetting[AltSetting]);
#endif // linux
	NtStatus = UsbPhyConfigurePipes( UsbPhy, NULL, DataInterfaceDescriptor);

	if (!NT_SUCCESS( NtStatus)) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Pipe Configuration failed\n"));
#ifdef _NTDDK_
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
		UsbPhyFreeInterfaceList( InterfaceList);
#endif // _NTDDK_
		return( NtStatus);
	}
#ifdef _NTDDK_
   /* We need to release the buffer */
   UsbPhyFreeInterfaceList( InterfaceList);
   UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
#endif // _NTDDK_


   // If InitStartup set receive pipe handles to NULL 
   // to prevent ALL functions including deallocation
   if (InitStartup)
   {
        //UsbPhy->BulkReceivePipeHandle = NULL;
        //UsbPhy->IsoReceivePipeHandle = NULL;
   
#ifdef QUASAR
		UsbPhy->ControllerParam.chipType = CHIP_QUASAR;
#else
		UsbPhy->ControllerParam.chipType = CHIP_ANCHOR;
#endif   

   }

#ifdef QUASAR
	if (InitStartup)
	{

		//get Boot Code Version	Temp solution
    //dowload code		
		NtStatus =UsbPhyDownLoadCode(UsbPhy , pFirmware, PFIRMWARE_LEN + 1 );
		SLEEP_PROCCESS(1000);

		SET_DSP_MSG_TYPE( UsbPhy->GTISysBlock, GTI_GET_BC_VERSION);
		READ_DSP_BLOCK( UsbPhy->GTISysBlock,GTI_DSP_A,GTI_DATAL_DSP,GTI_VERSION_SIZE ,UsbPhy->ControllerParam.version );
       COPY_MEMORY(UsbPhy->BootcodeVersion, UsbPhy->ControllerParam.version, GTI_VERSION_SIZE);

		
		//get Run Time Version	Temp solution
		SET_DSP_MSG_TYPE( UsbPhy->GTISysBlock, GTI_GET_RT_VERSION);
		READ_DSP_BLOCK( UsbPhy->GTISysBlock,GTI_DSP_A,GTI_DATAL_DSP,GTI_VERSION_SIZE ,Data);
       COPY_MEMORY(UsbPhy->Cont8051Version, Data, GTI_VERSION_SIZE);

        SET_DSP_MSG_TYPE( UsbPhy->GTISysBlock , GTI_SET_LOW_SUSPEND_CURRENT );
        UPDATE_DSP( UsbPhy->GTISysBlock );
	}
#endif

#if defined(linux)
	/* submit Receive URBs only if already done LanOpen or Wanopen */
	if( UsbPhy->AdapterHandle->flags & (_WAN_OPEN|_LAN_OPEN) )
	{
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("Starting Bulk Receive Flags %d\n",UsbPhy->AdapterHandle->flags));

#endif // linux
		if (UsbPhy->BulkReceivePipeHandle)
		{
			//
			// Start the Bulk Receive pipe
			//
			NtStatus = UsbPhyStartBulkReceive( UsbPhy);

			if (!NT_SUCCESS(NtStatus))
			{
				DBGPRINT(
					DBG_PRINT_MODULE_USB_ALL,
					DBG_PRINT_LEVEL_ERR,
					("Start Bulk Receive failed\n"));
				return( NtStatus);
			} // if
		}

		if (UsbPhy->IsoReceivePipeHandle)
		{
			//
			// Start the Iso Receive pipe
			//
			NtStatus = UsbPhyStartIsoReceive(UsbPhy);
			if (!NT_SUCCESS(NtStatus))
			{
				DBGPRINT(
					DBG_PRINT_MODULE_USB_ALL,
					DBG_PRINT_LEVEL_ERR,
					("Start Iso Receive failed\n"));
				return( NtStatus);
			} // if
		}
#if defined(linux)
	}
#endif // linux
	//
	// Start the Bulk Transmit pipe
	//
	NtStatus = UsbPhyStartBulkTransmit( UsbPhy);

	if (!NT_SUCCESS(NtStatus))
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Start Bulk Transmit failed\n"));
		return( NtStatus);
	} // if

   UsbPhy->BitSwapCounterFlag = TRUE;
#ifdef GSI_USB
	// Reselect the operational data interface to make sure that the
	// NETWORK_CONNECTION and CONNECTION_SPEED_CHANGE notifications
	// are sent by the device

	// Start bringing the DSP into DataMode
	if (InitStartup)
	{
		NtStatus = UsbPhyStartXDSLDsp( UsbPhy);
		if (!NT_SUCCESS(NtStatus))
		{
		  DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Start XDSL DSP failed\n"));
			return( NtStatus);
		}
	}
#endif // GSI_USB

//#if defined( _NTDDK_ )
	// Start the Interrupt Receive pipe
	// Note:  must start after DSP interrupts are turned off
	//        to prevent crashing when restarting computer
	//        and self-powered since modem is in data mode.
	//
	NtStatus = UsbPhyStartInterruptReceive( UsbPhy);
	if (!NT_SUCCESS(NtStatus))
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Start Interrupt Receive failed\n"));
		return( NtStatus);
	} // if
//#endif	/* defined(_NTDDK_) */

	// Allow customer to force DSL rate to registry
	if (UsbPhy->maxDownRateEnable)
		UsbPhy->xcvr[0].item[GTI_MAX_DOWN_RATE_ITEM] = UsbPhy->maxDownRate;

    return( STATUS_SUCCESS);

} // UsbPhyInitialize


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyShutdown
//
// ABSTRACT:
//    	Carries out the steps required to bring the USB Phy device to a
//		shutdown state (all pipes closed and the device unconfigured).
//
// RETURNS:
//	   void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyShutdown( PUSB_PHY UsbPhy)
{
	int		  lanindex = 0;
	adapter_t *pAdapter = (adapter_t *)UsbPhy->AdapterHandle;

	ASSERT( UsbPhy);
	ASSERT(pAdapter);

	//
	// Stop each of the pipes that has a valid handle
	//
	if( UsbPhy->InterruptPipeHandle )
		UsbPhyStopInterruptReceive(UsbPhy);

	if (UsbPhy->BulkTransmitPipeHandle)
		UsbPhyStopBulkTransmit( UsbPhy);

#ifdef GSI_LAN
	/*
	**	Run this loop for the count of MAX_PVC_SUPPORTED and call dev_close
	*/
	for(lanindex = 0; lanindex < MAX_PVC_SUPPORTED; lanindex++)
	{
		if( pAdapter->lan[lanindex].dev && (pAdapter->lan[lanindex].dev->flags & IFF_UP) )
		{
			/* 
			** Close the Network device before shutting down the USB Channel. 
			** We are not deregistering the device from N/W subsystem
			*/
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
				("Closing LAN Device\n"));
			dev_close(pAdapter->lan[lanindex].dev);
	}	/* End of for loop */
}
#endif // GSI_LAN

#if defined(linux)
	if( pAdapter->flags & (_WAN_OPEN|_LAN_OPEN) )
	{
#endif // linux
		if (UsbPhy->IsoReceivePipeHandle)
			UsbPhyStopIsoReceive( UsbPhy);
		
		if (UsbPhy->BulkReceivePipeHandle) 
			UsbPhyStopBulkReceive( UsbPhy);
#if defined(linux)
	}
#endif // linux
    // Mark all pipes as halted
    UsbPhy->BulkReceivePipeHandle 	= NULL;
	UsbPhy->IsoReceivePipeHandle 	= NULL;
	UsbPhy->BulkTransmitPipeHandle 	= NULL;
    UsbPhy->InterruptPipeHandle     = NULL;

	// Put the device in its unconfigured state
	UsbPhyUnconfigureDevice( UsbPhy);

} // UsbPhyShutdown


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyCheckDspStatus
//
// ABSTRACT:
//    	Trigger status read every 1500 ms
//
// RETURNS:
//	   void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void UsbPhyCheckDspStatus( PUSB_PHY	UsbPhy)
{
	UsbPhy->CheckDslStatus = TRUE;
	AddDspCommand ( UsbPhy, DSP_NO_ACTION );	// this will check status
}

#ifdef _NTDDK_
////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetDeviceDescriptor
//
// ABSTRACT:
//		Reads the USB device descriptor from the device.
//
// RETURNS:
//	   A pointer to the device descriptor structure or NULL if the device
//	   descriptor could not be read.  When the caller is done with the
//     device descriptor, it must be freed with a call to
//	   UsbPhyFreeDeviceDescriptor.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
PUSB_DEVICE_DESCRIPTOR
UsbPhyGetDeviceDescriptor(
	PUSB_PHY	UsbPhy
)
{
    PUSB_DEVICE_DESCRIPTOR	DeviceDescriptor;
    PURB    				Urb;
	NTSTATUS				NtStatus;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetDeviceDescriptor\n"));

	ALLOCATE_MEMORY( &Urb, sizeof(URB));

	if ( NULL == Urb) {
		return( NULL);
	}

	ALLOCATE_MEMORY( &DeviceDescriptor, sizeof( USB_DEVICE_DESCRIPTOR));

	if ( NULL == DeviceDescriptor) {
		FREE_MEMORY( Urb, sizeof(URB));
		return( NULL);
	}

    UsbBuildGetDescriptorRequest(
		Urb,
		sizeof( struct _URB_CONTROL_DESCRIPTOR_REQUEST),
		USB_DEVICE_DESCRIPTOR_TYPE,
		0,
		0,
		DeviceDescriptor,
		NULL,
		sizeof( USB_DEVICE_DESCRIPTOR),
		NULL);

    NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

	FREE_MEMORY( Urb, sizeof( URB));

    if ( !NT_SUCCESS( NtStatus)) {
		FREE_MEMORY( DeviceDescriptor,sizeof( USB_DEVICE_DESCRIPTOR));
		return( NULL);
    }

	return( DeviceDescriptor);

} // UsbPhyGetDeviceDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyFreeDeviceDescriptor
//
// ABSTRACT:
//		Releases the storage for a previously retrieved device descriptor
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyFreeDeviceDescriptor( PUSB_DEVICE_DESCRIPTOR	DeviceDescriptor)
{
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyFreeDeviceDescriptor\n"));

	ASSERT( DeviceDescriptor);

	FREE_MEMORY( DeviceDescriptor, sizeof( USB_DEVICE_DESCRIPTOR));

} // UsbPhyFreeDeviceDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetConfigurationDescriptor
//
// ABSTRACT:
//		Reads the configuration descriptor from the device.
//
// RETURNS:
//	   	A pointer to the configuration descriptor or NULL if the configuration
//	  	descriptor could not be read.  When the caller is done with the
//      configuration descriptor, it must be freed with a call to
//		UsbPhyFreeConfigurationDescriptor.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
PUSB_CONFIGURATION_DESCRIPTOR
UsbPhyGetConfigurationDescriptor
(
	PUSB_PHY    UsbPhy
)
{
	NTSTATUS    					NtStatus;
	PURB    						Urb;
	PUSB_CONFIGURATION_DESCRIPTOR   ConfigurationDescriptor;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetConfigurationDescriptor\n"));

	ALLOCATE_MEMORY( &Urb, sizeof( URB));

	if ( NULL == Urb ) {
		return( NULL);
	}

	//
    // USB_PHY_MAX_CONFIGURATION_DESCRIPTOR_SIZE defines the maximum amount
	// of data we will accept for the configuration descriptor as described
	// above.  This constant must be changed in order to support larger
	// amounts of configuration data.
    //
	ALLOCATE_MEMORY(
		&ConfigurationDescriptor,
		USB_PHY_MAX_CONFIGURATION_DESCRIPTOR_SIZE);

	if ( NULL == ConfigurationDescriptor) {
		FREE_MEMORY( Urb, sizeof( URB));
		return( NULL);
	}

	//
	// Build the Get Configuration Descriptor request and submit to USBD
	//
    UsbBuildGetDescriptorRequest(
    	Urb,
		sizeof( struct _URB_CONTROL_DESCRIPTOR_REQUEST),
		USB_CONFIGURATION_DESCRIPTOR_TYPE,
		0,
		0,
		ConfigurationDescriptor,
		NULL,
		USB_PHY_MAX_CONFIGURATION_DESCRIPTOR_SIZE,
		NULL);

    NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

	FREE_MEMORY( Urb, sizeof( URB));

	if ( !NT_SUCCESS(NtStatus)) {
		UsbPhyFreeConfigurationDescriptor( ConfigurationDescriptor);
		return( NULL);
	}

	return( ConfigurationDescriptor);

} // UsbPhyGetConfigurationDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyFreeConfigurationDescriptor
//
// ABSTRACT:
//		Releases the storage for a previously retrieved configuration descriptor
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyFreeConfigurationDescriptor(
	PUSB_CONFIGURATION_DESCRIPTOR	ConfigurationDescriptor
)
{
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyFreeConfigurationDescriptor\n"));

	ASSERT( ConfigurationDescriptor);

	FREE_MEMORY(
		ConfigurationDescriptor,
		USB_PHY_MAX_CONFIGURATION_DESCRIPTOR_SIZE);

} // UsbPhyFreeConfigurationDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetStringDescriptor
//
// ABSTRACT:
//		Reads a string descriptor from the USB device according to the
//		provided index
//
// RETURNS:
//	   A pointer to the string descriptor structure or NULL if the string
//	   descriptor could not be read.  When the caller is done with the string
//	   descriptor, it must be freed using UsbPhyFreeStringDescriptor.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
PUSB_STRING_DESCRIPTOR
UsbPhyGetStringDescriptor(
	PUSB_PHY	UsbPhy,
	UCHAR		Index,
	USHORT		LanguageId
)
{
    PUSB_STRING_DESCRIPTOR	StringDescriptor;
    PURB    				Urb;
	NTSTATUS				NtStatus;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetStringDescriptor\n"));

	ALLOCATE_MEMORY( &Urb, sizeof(URB));

	if ( NULL == Urb) {
		return( NULL);
	}

	ALLOCATE_MEMORY( &StringDescriptor,USB_PHY_MAX_STRING_DESCRIPTOR_SIZE);

	if ( NULL == StringDescriptor) {
		FREE_MEMORY( Urb, sizeof(URB));
		return( NULL);
	}

    UsbBuildGetDescriptorRequest(
		Urb,
		sizeof( struct _URB_CONTROL_DESCRIPTOR_REQUEST),
		USB_STRING_DESCRIPTOR_TYPE,
		Index,
		LanguageId,
		StringDescriptor,
		NULL,
		USB_PHY_MAX_STRING_DESCRIPTOR_SIZE,
		NULL);

    NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

	FREE_MEMORY( Urb, sizeof( URB));

    if ( !NT_SUCCESS( NtStatus)) {
		FREE_MEMORY( StringDescriptor,USB_PHY_MAX_STRING_DESCRIPTOR_SIZE);
		return( NULL);
    }

	return( StringDescriptor);

} // UsbPhyGetStringDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyFreeStringDescriptor
//
// ABSTRACT:
//		Releases the storage for a previously retrieved string descriptor
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyFreeStringDescriptor( PUSB_STRING_DESCRIPTOR	StringDescriptor)
{
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyFreeStringDescriptor\n"));

	ASSERT( StringDescriptor);

	FREE_MEMORY( StringDescriptor,USB_PHY_MAX_STRING_DESCRIPTOR_SIZE);

} // UsbPhyFreeStringDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetLanguageId
//
// ABSTRACT:
//		Retrieves the first language ID from the device. Note that the device
//		returns its supported language IDs in string descriptor zero.
//
// RETURNS:
//		NT_STATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyGetLanguageId(
	PUSB_PHY	UsbPhy,
	PUSHORT		LanguageId
)
{
	PUSB_STRING_DESCRIPTOR	StringDescriptor;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetLanguageId\n"));

	ASSERT( UsbPhy);
	ASSERT( LanguageId);

	//
	// Retrieve string descriptor zero, which contains the supported language
	// IDs
	//
	StringDescriptor = UsbPhyGetStringDescriptor( UsbPhy,0,0);

	if ( NULL == StringDescriptor) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read Language ID String Descriptor\n"));
		return( STATUS_UNSUCCESSFUL);
	}

	//
	// We expect at least one language ID
	//
	if ( (StringDescriptor->bLength-2) < sizeof(USHORT)) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Lang. ID string descriptor has invalid length (%d bytes)\n",
			(StringDescriptor->bLength-2)));
		UsbPhyFreeStringDescriptor( StringDescriptor);
		return( STATUS_UNSUCCESSFUL);
	}

	//
	// We're interested in the first language ID
	//
	*LanguageId = (USHORT)(StringDescriptor->bString[0]);

	UsbPhyFreeStringDescriptor( StringDescriptor);

	return( STATUS_SUCCESS);

} // UsbPhyGetLanguageId

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetInterfaceList
//
// ABSTRACT:
//		Parses the given configuration descriptor for the communications
//		class interface descriptor and the data class interface descriptor
//		and places them in an interface list.
//
// 		The interface list will have two entries ordered as follows:
//
// 				InterfaceList[0] ==> Communications class interface
//				InterfaceList[1] ==> Data class interface ON (With Endpoints)
//
// RETURNS:
//	   	A pointer to the interface list or NULL if the interface list couldn't
//	   	be created.  When the caller is done with the interface list, it must
//		be freed with a call to UsbPhyFreeInterfaceList.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
PUSBD_INTERFACE_LIST_ENTRY
UsbPhyGetInterfaceList
(
	PUSB_PHY 						UsbPhy,
	PUSB_CONFIGURATION_DESCRIPTOR 	ConfigurationDescriptor
)
{
    NTSTATUS 					NtStatus;
    PURB    					Urb;
    PUSB_INTERFACE_DESCRIPTOR	DataInterfaceDescriptor;
    PUSBD_INTERFACE_LIST_ENTRY  InterfaceList;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetIntefaceList\n"));

	//
	// Allocate memory for the interface list.  Note that the interface list is
	// terminated with an entry whose members are set to NULL.
	//
	ALLOCATE_MEMORY(
		&(InterfaceList),
		sizeof( USBD_INTERFACE_LIST_ENTRY) *
		(ConfigurationDescriptor->bNumInterfaces + 1));

	if ( NULL == InterfaceList) {
		return( NULL);
	}

	//
	// Parse the configuration descriptor to find the data class interface
	// descriptor.  If the data class descriptor is not present, the interface
	// list can't be created.
	//
	DataInterfaceDescriptor =
		USBD_ParseConfigurationDescriptorEx(
	        ConfigurationDescriptor,
	        ConfigurationDescriptor,
	        USB_PHY_DATA_INTERFACE_INDEX,
	        -1,
	        -1,
	        -1,
	        -1);

	if ( NULL == DataInterfaceDescriptor) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Device has no data class interfaces\n"));
		UsbPhyFreeInterfaceList( InterfaceList);
		return( NULL);
	}

	InterfaceList[USB_PHY_DATA_INTERFACE_INDEX].InterfaceDescriptor = DataInterfaceDescriptor;
	UsbPhy->DataInterfaceOnAlternateSetting = 0;
	UsbPhy->DataInterfaceOffAlternateSetting = 5;

	return( InterfaceList);

} // UsbPhyGetInterfaceList

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbFreeInterfaceList
//
// ABSTRACT:
//		Releases the storage for a previously retrieved interface list
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyFreeInterfaceList(
	PUSBD_INTERFACE_LIST_ENTRY	InterfaceList
)
{
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyFreeInterfaceList\n"));

	ASSERT( InterfaceList);

	FREE_MEMORY( InterfaceList, sizeof( USBD_INTERFACE_LIST_ENTRY) * (USB_PHY_INTERFACE_COUNT + 1) );

} // UsbPhyFreeInterfaceList

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyParseConfigDescriptor
//
// ABSTRACT:
//		Parses the configuration descriptor (as returned by
//		UsbPhyGetConfigurationDescriptor) looking for a functional descriptor
//		with a matching type and subtype
//
// RETURNS:
//		A pointer to the requested functional descriptor  (within the given
//		configuration descriptor) or NULL if the functional descriptor could
//		not be found.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
PCDC_COMMON_FUNCTIONAL_DESCRIPTOR
UsbPhyParseConfigDescriptor(
	PUSB_CONFIGURATION_DESCRIPTOR	ConfigurationDescriptor,
	UCHAR							DescriptorType,
	UCHAR							DescriptorSubType,
	USHORT							DescriptorSize
)
{
	PUSB_COMMON_DESCRIPTOR				Descriptor;
	PCDC_COMMON_FUNCTIONAL_DESCRIPTOR	FunctionalDescriptor;
	PVOID								StartPosition;


	ASSERT( ConfigurationDescriptor);

	//
	// Size must at least as large as the common functional descriptor
	//
	if ( DescriptorSize < sizeof( CDC_COMMON_FUNCTIONAL_DESCRIPTOR) ) {
		return( NULL);
	}

	StartPosition 	= ConfigurationDescriptor;

	while ( TRUE ) {
		//
		// Parse the config descriptor for a descriptor with a
		// matching descriptor type
		//
		Descriptor =
			USBD_ParseDescriptors(
				ConfigurationDescriptor,
				ConfigurationDescriptor->wTotalLength,
				StartPosition,
				DescriptorType);

        if (Descriptor == NULL) {
	        return( NULL);
		}

		//
		// If the descriptor we found is not the right length, continue
		// searching
		//
		if ( Descriptor->bLength != DescriptorSize ) {
			StartPosition = ((PUCHAR)Descriptor + Descriptor->bLength);
		} else {
			//
			// Check the descriptor subtype
			//
			FunctionalDescriptor =
				(PCDC_COMMON_FUNCTIONAL_DESCRIPTOR)Descriptor;

			if ( FunctionalDescriptor->bDescriptorSubType ==
				 DescriptorSubType)
			{
				return( FunctionalDescriptor);
			}
			StartPosition = ((PUCHAR)Descriptor + Descriptor->bLength);
		}
	}
	return( NULL);

} // UsbPhyParseConfigDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyConfigureDevice
//
// ABSTRACT:
//		Given a configuration descriptor and an interface list, builds and
//		issues a request to USBD to configure the device
//
// RETURNS:
//		NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyConfigureDevice(
	PUSB_PHY				UsbPhy,
	PUSB_CONFIGURATION_DESCRIPTOR	ConfigurationDescriptor,
	PUSBD_INTERFACE_LIST_ENTRY		InterfaceList
)
{
	NTSTATUS					NtStatus;
	PURB						Urb;
	PUSBD_INTERFACE_INFORMATION	DataInterface;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyConfigureDevice\n"));

	ASSERT( UsbPhy);
	ASSERT( ConfigurationDescriptor);
	ASSERT( InterfaceList);

	//
	// Create the URB for the configuration request.  Prior to submitting the
	// URB to USBD for processing, we can alter the default settings for each
	// interface as desired.
	//
	Urb = USBD_CreateConfigurationRequestEx(
			ConfigurationDescriptor,
			InterfaceList);

	if ( NULL == Urb ) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

   /* ISO pipe requires user to indicate the maximum Transfer size.
   ** We are allocating 64k (0xFFFF) transfer size.
   ** This variable probably allocates memory in the host controller
   */
   DataInterface = InterfaceList[USB_PHY_DATA_INTERFACE_INDEX].Interface;
   DataInterface->Pipes[0].MaximumTransferSize = 0xFFFF;

	//
	// Pass the configuration request to USBD with the default settings
	//
	NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

	if ( !NT_SUCCESS( NtStatus)) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Configure attempt failed\n"));
		FREE_MEMORY( Urb, sizeof(URB));
		return( NtStatus);
	}
   DataInterface = InterfaceList[USB_PHY_DATA_INTERFACE_INDEX].Interface;

    //
	// Save the handle for this configuration as well as the interface
	// information needed when changing the device settings (i.e changing
	// the alternate setting for an interface)
	//
	UsbPhy->ConfigurationHandle =
		Urb->UrbSelectConfiguration.ConfigurationHandle;

    UsbPhy->DataInterfaceNumber =
		DataInterface->InterfaceNumber;

	return( STATUS_SUCCESS);

} // UsbPhyConfigureDevice

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyUnconfigureDevice
//
// ABSTRACT:
//		Returns the device to it's unconfigured state
//
// RETURNS:
//		NTSTATUS
//
// ASSUMPTIONS:
//		The device has been successfully configured
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyUnconfigureDevice( PUSB_PHY	UsbPhy)
{
	PURB		Urb;
	NTSTATUS	NtStatus;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyUnconfigureDevice\n"));

	ASSERT( UsbPhy);

	ALLOCATE_MEMORY( &Urb, sizeof(URB));
	if ( NULL == Urb) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

	UsbBuildSelectConfigurationRequest(
		Urb,
		sizeof(struct _URB_SELECT_CONFIGURATION),
		NULL);

	NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

	FREE_MEMORY( Urb, sizeof(URB));

	return( NtStatus);

} // UsbPhyUnconfigureDevice

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyConfigurePipes
//
// ABSTRACT:
//		Finds and initializes the pipes required for operation ( Bulk In,
//		Bulk Out, and Interrupt)
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyConfigurePipes
(
	PUSB_PHY 			UsbPhy,
    PUSBD_INTERFACE_INFORMATION	CommInterface,
    PUSBD_INTERFACE_INFORMATION	DataInterface
)
{
	ULONG	i;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyConfigurePipes\n"));
    ASSERT(DataInterface);

	if ( DataInterface->NumberOfPipes != USB_PHY_PIPES_COUNT) {
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Data Intf wrong # pipes\n"));
		return( STATUS_UNSUCCESSFUL);
	}


	UsbPhy->BulkReceivePipeHandle 	= NULL;
	UsbPhy->IsoReceivePipeHandle 	= NULL;
	UsbPhy->BulkTransmitPipeHandle 	= NULL;
    UsbPhy->InterruptPipeHandle = NULL;

   /* We should not get more than 2 pipes for the data interface */
   for(i=0; i < DataInterface->NumberOfPipes; i++)
   {
		switch (DataInterface->Pipes[i].PipeType)
		{
			case UsbdPipeTypeIsochronous:
			case UsbdPipeTypeBulk:
            		//
		// Is the pipe an IN pipe and have we found our IN pipe yet?
		//
		if ( USB_ENDPOINT_DIRECTION_IN(
				DataInterface->Pipes[i].EndpointAddress) &&
			 ( UsbPhy->InPipeMode == USB_ISO_MODE ))
		{
			UsbPhy->IsoReceivePipeHandle =
				DataInterface->Pipes[i].PipeHandle;
			UsbPhy->IsoReceivePipeEndpointAddress =
				DataInterface->Pipes[i].EndpointAddress;
			UsbPhy->IsoReceivePipePacketSize =
				DataInterface->Pipes[i].MaximumPacketSize;
			UsbPhy->IsoReceivePipeTransferSize =
            DataInterface->Pipes[i].MaximumTransferSize;
		}
        else if ( USB_ENDPOINT_DIRECTION_IN(
				DataInterface->Pipes[i].EndpointAddress) &&
			    ( UsbPhy->InPipeMode == USB_BULK_MODE ))
		{
			UsbPhy->BulkReceivePipeHandle =
				DataInterface->Pipes[i].PipeHandle;
			UsbPhy->BulkReceivePipePacketSize =
				DataInterface->Pipes[i].MaximumPacketSize;
		}
		//
		// Is the pipe an OUT pipe and have we found our OUT pipe yet?
		//
		else if ( USB_ENDPOINT_DIRECTION_OUT(
					DataInterface->Pipes[i].EndpointAddress) &&
				  ( UsbPhy->BulkTransmitPipeHandle == NULL ))
		{
			UsbPhy->BulkTransmitPipeHandle =
				DataInterface->Pipes[i].PipeHandle;
			UsbPhy->BulkTransmitPipePacketSize =
				DataInterface->Pipes[i].MaximumPacketSize;
		}
		//
		// The pipe must be invalid
		//
		else
		{
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Data Intf Invalid Pipe\n"));
			return( STATUS_UNSUCCESSFUL);
		}

    
        break;
       case UsbdPipeTypeInterrupt:
    	
            UsbPhy->InterruptPipeHandle 	=
		    DataInterface->Pipes[i].PipeHandle;
	        UsbPhy->InterruptPipePacketSize =
		    DataInterface->Pipes[i].MaximumPacketSize;


				break;
			default:
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
					  ("Data Interface pipe[%d] and pipeType=%d is not in the list\n"
													,i,DataInterface->Pipes[i].PipeType));
				return( STATUS_UNSUCCESSFUL);
		}//end switch

	}//end for

	return( STATUS_SUCCESS);

} // UsbPhyConfigurePipes

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyCallUSBD
//
// ABSTRACT:
//		Passes the given URB to USBD and waits for the URB to be completed.
//		Intended for use with control commands (not data transfers).
//    This routine uses the URB passed in by the caller as the request
//    block for the USB software stack. This routine issues a synchronous
//    request to the USB stack, we could implement this as an asynchronous
//    request in which you set an IoCompletionRoutine to be called when
//    the request is complete.
//
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyCallUSBD
(
	PUSB_PHY	UsbPhy,
	PURB    	Urb
)
{
    NTSTATUS    		NtStatus;
    PIRP    			Irp;
    KEVENT    			Event;
    IO_STATUS_BLOCK		IoStatus;
    PIO_STACK_LOCATION  NextStack;

	LONG            longTimeout = (-1) * (3000 * 10000);		// 3 Sec. timer
	LARGE_INTEGER   longIntTimeout = RtlConvertLongToLargeInteger(longTimeout);


    KeInitializeEvent(&Event, NotificationEvent, FALSE);

	//
	// Build an IRP in which to pass the URB to USBD
	//
    Irp = IoBuildDeviceIoControlRequest(
				IOCTL_INTERNAL_USB_SUBMIT_URB,
				UsbPhy->TopOfStackDeviceObject,  // Points to the next-lower driver's device object
				NULL,       // Optional input buffer
				0,          // input buffer length
				NULL,       // Optional output buffer
				0,          // Output buffer length
				TRUE,       // if TRUE, the target driver's Dispatch routine for
                        // IRP_MJ_INTERNAL_DEVICE_CONTROL is called. Otherwise,
                        // the Dispatch routine for IRP_MJ_DEVICE_CONTROL is called
				&Event,     // Event to be signalled on completion
				&IoStatus); // Specifies an I/O status block to be set when the request is
                        // completed by lower drivers

   if (Irp == NULL)
   {
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL,
			    DBG_PRINT_LEVEL_ERR,
               ("Could not build IRP with IoBuildDeviceIoControlRequest\n"));
      return STATUS_UNSUCCESSFUL;
   }

   // This routine gives a higher level driver access to the nest-lower driver's
   // I/O stack location in an IRP so the caller can set it up for the lower driver
   NextStack = IoGetNextIrpStackLocation( Irp);
   ASSERT( NextStack);

    //
    // Pass the URB to the USB driver stack
    //
    NextStack->Parameters.Others.Argument1 = Urb;

   // IoCallDriver sends an IRP to the next-lower-level driver after the caller has
   // set up the I/O stack location in the IRP for that driver.
   NtStatus = IoCallDriver(UsbPhy->TopOfStackDeviceObject, Irp);

	//
	// If our request is pending, wait until it completes
	//
    if (NtStatus == STATUS_PENDING)
    {
        NtStatus = KeWaitForSingleObject(
                       &Event,
                       Executive,
                       KernelMode,
                       FALSE,
                       &longIntTimeout);
	}
    else
    {
        IoStatus.Status = NtStatus;
    }

    //
    // USBD maps the error code for us
    //
    NtStatus = IoStatus.Status;


	//
	// If the NT status is successful, check the URB status
	//
	if ( NT_SUCCESS( NtStatus)) {
		if (!USBD_SUCCESS( URB_STATUS( Urb))) {
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("URB Status Failed 0x%X\n",
				URB_STATUS( Urb)));
			NtStatus = STATUS_UNSUCCESSFUL;
		}
	}

    return( NtStatus);

} // UsbPhyCallUSBD

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyResetPipe
//
// ABSTRACT:
//		Issues a reset for the pipe identified by the pipe handle
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyResetPipe
(
	PUSB_PHY 			UsbPhy,
	USBD_PIPE_HANDLE	PipeHandle

)
{
    NTSTATUS 	NtStatus;
    PURB 		Urb;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyResetPipe\n"));

    ALLOCATE_MEMORY( &Urb, sizeof(URB));

	if ( NULL == Urb) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

    Urb->UrbHeader.Length 			= sizeof(struct _URB_PIPE_REQUEST);
    Urb->UrbHeader.Function 		= URB_FUNCTION_RESET_PIPE;
    Urb->UrbPipeRequest.PipeHandle 	= PipeHandle;

    NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

    FREE_MEMORY( Urb, sizeof(URB));

	return( NtStatus);

} // UsbPhyResetPipe

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyAbortPipe
//
// ABSTRACT:
//		Issues an abort request for the pipe identified by the pipe handle.
//		The abort request causes all pending requests for the pipe to be
//		cancelled.
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyAbortPipe
(
	PUSB_PHY 			UsbPhy,
	USBD_PIPE_HANDLE	PipeHandle

)
{
    NTSTATUS 	NtStatus;
    PURB 		Urb;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyAbortPipe\n"));

    ALLOCATE_MEMORY( &Urb, sizeof(URB));

	if ( NULL == Urb) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

    Urb->UrbHeader.Length 			= sizeof(struct _URB_PIPE_REQUEST);
    Urb->UrbHeader.Function 		= URB_FUNCTION_ABORT_PIPE;
    Urb->UrbPipeRequest.PipeHandle 	= PipeHandle;

    NtStatus = UsbPhyCallUSBD( UsbPhy, Urb);

    FREE_MEMORY( Urb, sizeof(URB));

	return( NtStatus);

} // UsbPhyAbortPipe

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetAlternateSetting
//
// ABSTRACT:
//		Gets the current alternate setting for the specified interface of
//		the device
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyGetAlternateSetting
(
	PUSB_PHY UsbPhy,
	UCHAR    InterfaceNumber,
	PUCHAR   AlternateSetting
)
{
    NTSTATUS     NtStatus;
    PURB         Urb;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyGetAlternateSetting\n"));

	ASSERT( UsbPhy);
	ASSERT( AlternateSetting);

	ALLOCATE_MEMORY( &Urb, sizeof( URB));
	if ( NULL == Urb) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

	//
	// Build the URB
	//
    Urb->UrbHeader.Length 	= sizeof(struct _URB_CONTROL_GET_INTERFACE_REQUEST);
    Urb->UrbHeader.Function = URB_FUNCTION_GET_INTERFACE;
    Urb->UrbControlGetInterfaceRequest.TransferBufferLength = sizeof( UCHAR);
    Urb->UrbControlGetInterfaceRequest.TransferBuffer = AlternateSetting;
    Urb->UrbControlGetInterfaceRequest.Interface = InterfaceNumber;

    NtStatus = UsbPhyCallUSBD( UsbPhy,Urb);

	FREE_MEMORY( Urb, sizeof( URB));

    return( NtStatus);

} // UsbPhyGetAlternateSetting

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhySetAlternateSetting
//
// ABSTRACT:
//		Sets the current alternate setting for the specified interface of
//		the device
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		Should be used only with the data interface of the device
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhySetAlternateSetting(
    IN PUSB_PHY UsbPhy,
    IN UCHAR    InterfaceNumber,
    IN UCHAR    AlternateSetting
    )
{
    NTSTATUS						NtStatus;
    PURB    						Urb;
    PUSBD_INTERFACE_INFORMATION		Interface;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhySetAlternateSetting\n"));

	ASSERT( UsbPhy);
	ASSERT( AlternateSetting);

	ALLOCATE_MEMORY(
		&Urb,
		GET_USBD_INTERFACE_SIZE( USB_PHY_DATA_INTERFACE_NUMBER_OF_PIPES));
	if ( NULL == Urb) {
		return( STATUS_INSUFFICIENT_RESOURCES);
	}

    Interface = &Urb->UrbSelectInterface.Interface;

    //
    // set up the input parameters in our Interface request
    // structure.
    //
    Interface->Length =
        GET_USBD_INTERFACE_SIZE( USB_PHY_DATA_INTERFACE_NUMBER_OF_PIPES);
    Interface->InterfaceNumber = InterfaceNumber;
    Interface->AlternateSetting = AlternateSetting;

    Urb->UrbHeader.Function = URB_FUNCTION_SELECT_INTERFACE;
    Urb->UrbSelectInterface.ConfigurationHandle =
        UsbPhy->ConfigurationHandle;

    NtStatus = UsbPhyCallUSBD( UsbPhy,Urb);

	FREE_MEMORY(
		Urb,
		GET_USBD_INTERFACE_SIZE( USB_PHY_DATA_INTERFACE_NUMBER_OF_PIPES));

    return NtStatus;

} // UsbPhySetAlternateSetting

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyDecrementIoCount
//
// ABSTRACT:
//		Decrements the pending IO count for the USB Phy.  If the new pending
//		IO count is zero, the remove event is signaled.
//
// RETURNS:
//		The new pending IO count
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
ULONG
UsbPhyDecrementIoCount
(
    IN PUSB_PHY UsbPhy
)
{
    ULONG PendingIoCount;

    PendingIoCount = InterlockedDecrement(&UsbPhy->PendingIoCount);

	return PendingIoCount;

} // UsbPhyDecrementIoCount

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyIncrementIoCount
//
// ABSTRACT:
//		Increments the pending IO count for the USB Phy.
//
// RETURNS:
//		The new pending IO count
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
ULONG
UsbPhyIncrementIoCount
(
    IN PUSB_PHY UsbPhy
)
{
    InterlockedIncrement(&UsbPhy->PendingIoCount);

	return( UsbPhy->PendingIoCount);

} // UsbPhyIncrementIoCount

#elif defined(linux)
////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyGetStringDescriptor
//
// ABSTRACT:
//		Reads a string descriptor from the USB device according to the
//		provided index
//
// RETURNS:
//	   A pointer to the string descriptor structure or NULL if the string
//	   descriptor could not be read.  When the caller is done with the string
//	   descriptor, it must be freed using UsbPhyFreeStringDescriptor.
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
int
UsbPhyGetStringDescriptor(
	PUSB_PHY	UsbPhy,
	UCHAR		Index,
	USHORT		LanguageId,
	PWCHAR		pBuff,
	int			nBuffLen
)
{
	int	nRetVal = 0;

	nRetVal = usb_string( UsbPhy->AdapterHandle->usbDev, Index, 
					(char *)pBuff, nBuffLen);
	if( nRetVal <= 0 )
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to read String Descriptor for Index %d\n",Index));
	}

	return( nRetVal);
} // UsbPhyGetStringDescriptor

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyUnconfigureDevice
//
// ABSTRACT:
//		Returns the device to it's unconfigured state
//
// RETURNS:
//		NTSTATUS
//
// ASSUMPTIONS:
//		The device has been successfully configured
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyUnconfigureDevice( PUSB_PHY	UsbPhy)
{
	struct		usb_device *dev;
	int 			nRet = 0;
	
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyUnconfigureDevice\n"));

	ASSERT( UsbPhy);

	dev = UsbPhy->AdapterHandle->usbDev;
	nRet = usb_control_msg(dev , usb_sndctrlpipe(dev, 0), 
                        	USB_REQ_SET_CONFIGURATION, 0, 
                        	0, 0, NULL, 0 , HZ * SET_TIMEOUT);
	if ( nRet < 0 )
   	{
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("UsbPhyUnconfigureDevicei FAILED\n"));
   		return(STATUS_UNSUCCESSFUL);
	}

	//dev->actconfig = NULL;
	//dev->toggle[0] = 0;
	//dev->toggle[1] = 0;

	return( STATUS_SUCCESS);

} // UsbPhyUnconfigureDevice

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyConfigurePipes
//
// ABSTRACT:
//		Finds and initializes the pipes required for operation ( Bulk In,
//		Bulk Out, and Interrupt)
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyConfigurePipes
(
	PUSB_PHY 			UsbPhy,
	struct usb_interface_descriptor *CommInterface,
	struct usb_interface_descriptor *DataInterface
)
{
	struct usb_device *dev;
	ULONG	i;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyConfigurePipes\n"));

	ASSERT(DataInterface);
	dev = UsbPhy->AdapterHandle->usbDev;
	if ( DataInterface->bNumEndpoints  != USB_PHY_PIPES_COUNT) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_USB_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Data Intf wrong # pipes\n"));
		return( STATUS_UNSUCCESSFUL);
	}

	UsbPhy->BulkReceivePipeHandle  = 0;
	UsbPhy->IsoReceivePipeHandle 	= 0;
	UsbPhy->BulkTransmitPipeHandle = 0;
    UsbPhy->InterruptPipeHandle = 0;

   /* We should not get more than 2 pipes for the data interface */
	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
        ("No of EP = %d\n",DataInterface->bNumEndpoints));
   for(i=0; i < DataInterface->bNumEndpoints; i++)
   {
		switch ( DataInterface->endpoint[i].bmAttributes & USB_ENDPOINT_XFERTYPE_MASK )
		{
			case USB_ENDPOINT_XFER_ISOC:
			case USB_ENDPOINT_XFER_BULK:
            		//
			// Is the pipe an IN pipe and have we found our IN pipe yet?
			//
			if ((( DataInterface->endpoint[i].bEndpointAddress & USB_ENDPOINT_DIR_MASK) == 
					USB_DIR_IN ) &&
				 ( UsbPhy->InPipeMode == USB_ISO_MODE ))
			{
				UsbPhy->IsoReceivePipeHandle =
					usb_rcvisocpipe( dev, DataInterface->endpoint[i].bEndpointAddress & 0xf);
			 	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            		("ISO RX PIPE ATTR = %d HANDLE = %x PktSize %d\n",
					DataInterface->endpoint[i].bmAttributes,
					UsbPhy->IsoReceivePipeHandle,
                    DataInterface->endpoint[i].wMaxPacketSize));

				UsbPhy->IsoReceivePipeEndpointAddress =
					DataInterface->endpoint[i].bEndpointAddress;
				UsbPhy->IsoReceivePipePacketSize =
					DataInterface->endpoint[i].wMaxPacketSize;
				UsbPhy->IsoReceivePipeTransferSize =
			            DataInterface->endpoint[i].wMaxPacketSize;
			}
        		else if (((DataInterface->endpoint[i].bEndpointAddress & USB_ENDPOINT_DIR_MASK) == 
					USB_DIR_IN ) &&
			    		( UsbPhy->InPipeMode == USB_BULK_MODE ))
			{
				UsbPhy->BulkReceivePipeHandle =
					usb_rcvbulkpipe( dev, DataInterface->endpoint[i].bEndpointAddress & 0xf);
			 	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            		("BULK RX PIPE ATTR = %d HANDLE = %x PktSize %d\n",
					DataInterface->endpoint[i].bmAttributes,
					UsbPhy->BulkReceivePipeHandle,
                    DataInterface->endpoint[i].wMaxPacketSize));

				UsbPhy->BulkReceivePipePacketSize =
					DataInterface->endpoint[i].wMaxPacketSize;
			}
			//
			// Is the pipe an OUT pipe and have we found our OUT pipe yet?
			//
			else if (((DataInterface->endpoint[i].bEndpointAddress & USB_ENDPOINT_DIR_MASK)== 
					USB_DIR_OUT ) &&
					( UsbPhy->BulkTransmitPipeHandle == 0 ))
			{
				UsbPhy->BulkTransmitPipeHandle =
					usb_sndbulkpipe( dev, DataInterface->endpoint[i].bEndpointAddress );
			 	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            		("BULK TX PIPE ATTR = %d HANDLE = %x\n",
					DataInterface->endpoint[i].bmAttributes,
					UsbPhy->BulkTransmitPipeHandle ));

				UsbPhy->BulkTransmitPipePacketSize =
					DataInterface->endpoint[i].wMaxPacketSize;
			}
			//
			// The pipe must be invalid
			//
			else
			{
				DBGPRINT(
					DBG_PRINT_MODULE_USB_ALL,
					DBG_PRINT_LEVEL_ERR,
					("Data Intf Invalid Pipe\n"));
				return( STATUS_UNSUCCESSFUL);
			}

    
        		break;
		       case USB_ENDPOINT_XFER_INT:
		    		/* Init receive endpoints */
		            UsbPhy->InterruptPipeHandle = usb_rcvintpipe( dev, 
							DataInterface->endpoint[i].bEndpointAddress );
					UsbPhy->InterruptInterval = 
						(DataInterface->endpoint[i].bInterval > 0 ? 
							DataInterface->endpoint[i].bInterval : HZ / 2);

			 	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
            		("INT PIPE ATTR = %d HANDLE = %x IVAL = %d Size=%d\n",
					DataInterface->endpoint[i].bmAttributes,
					UsbPhy->InterruptPipeHandle, UsbPhy->InterruptInterval,
				    DataInterface->endpoint[i].wMaxPacketSize));

				    UsbPhy->InterruptPipePacketSize =
				    DataInterface->endpoint[i].wMaxPacketSize;
#if defined(linux)					
					UsbPhy->InterruptBufferSize	 = 
						UsbPhy->InterruptPipePacketSize; 
#endif //linux
			break;

			default:
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
					  ("Data Interface pipe[%ld] and pipeType=%u is not in the list\n",
						i,DataInterface->endpoint[i].bmAttributes));
				return( STATUS_UNSUCCESSFUL);
		}//end switch

	}//end for

	return( STATUS_SUCCESS);

} // UsbPhyConfigurePipes

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyResetPipe
//
// ABSTRACT:
//		Issues a reset for the pipe identified by the pipe handle
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyResetPipe
(
	PUSB_PHY 			UsbPhy,
	USBD_PIPE_HANDLE	PipeHandle

)
{
	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyResetPipe\n"));
	
	if( usb_clear_halt( UsbPhy->AdapterHandle->usbDev, PipeHandle ) < 0 )
	{
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("UsbPhyResetPipe FAILED\n"));
		
		return( STATUS_UNSUCCESSFUL );
	}
	return( STATUS_SUCCESS );
} // UsbPhyResetPipe

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyAbortPipe
//
// ABSTRACT:
//		Issues an abort request for the pipe identified by the pipe handle.
//		The abort request causes all pending requests for the pipe to be
//		cancelled.
//
// RETURNS:
//	   NTSTATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NTSTATUS
UsbPhyAbortPipe
(
	PUSB_PHY 			UsbPhy,
	USBD_PIPE_HANDLE	PipeHandle

)
{
	if (usb_pipebulk(PipeHandle))
	{
		if( usb_pipeout( PipeHandle ))
		{
			if (UsbPhy->BulkTransmitPipeHandle)
				UsbPhyStopBulkTransmit(UsbPhy);
		}
		else
		{
			if (UsbPhy->BulkReceivePipeHandle) 
				UsbPhyStopBulkReceive(UsbPhy);
		}
	}
	else if( usb_pipeisoc(PipeHandle))
	{
		if (UsbPhy->IsoReceivePipeHandle)
			UsbPhyStopIsoReceive(UsbPhy);
	}
	else if( usb_pipeint(PipeHandle))
	{
		if( UsbPhy->InterruptPipeHandle )
			UsbPhyStopInterruptReceive(UsbPhy);
	}
	else
	{
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("UsbPhyAbortPipe Wrong Pipe handle\n"));
		return(STATUS_UNSUCCESSFUL);
	}
	return( STATUS_SUCCESS );
} // UsbPhyAbortPipe

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyDecrementIoCount
//
// ABSTRACT:
//		Decrements the pending IO count for the USB Phy.  If the new pending
//		IO count is zero, the remove event is signaled.
//
// RETURNS:
//		The new pending IO count
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
ULONG
UsbPhyDecrementIoCount
(
    IN PUSB_PHY UsbPhy
)
{
	if(atomic_read(&(UsbPhy->PendingIoCount)))
		atomic_dec(&(UsbPhy->PendingIoCount));
	return( atomic_read(&(UsbPhy->PendingIoCount)));
} // UsbPhyDecrementIoCount

////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyIncrementIoCount
//
// ABSTRACT:
//		Increments the pending IO count for the USB Phy.
//
// RETURNS:
//		The new pending IO count
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
ULONG
UsbPhyIncrementIoCount
(
    IN PUSB_PHY UsbPhy
)
{
	atomic_inc(&(UsbPhy->PendingIoCount));
	return(atomic_read(&(UsbPhy->PendingIoCount)));
} // UsbPhyIncrementIoCount
#endif // linux


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyUnicodeStringHexToBytes
//
// ABSTRACT:
//		Converts the given Unicode string of hex digits to bytes
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
UsbPhyUnicodeStringHexToBytes(
	PWCHAR	UnicodeString,
	PUCHAR	Bytes,
	ULONG	BytesToConvert
)
{
	ULONG	i;
	UCHAR	AsciiChar;
	UCHAR	Byte;

	DBGPRINT(
		DBG_PRINT_MODULE_USB_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("UsbPhyUnicodeStringHexToBytes\n"));

	ASSERT( UnicodeString);
	ASSERT( Bytes);

	for ( i = 0; i < BytesToConvert; i++) {
		Byte = 0;
	  	AsciiChar = (UCHAR)(UnicodeString[i*2]);
		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("Unicode Char (0x%X)\n", AsciiChar));
		if ( (AsciiChar >= '0') && (AsciiChar <= '9')) {
			Byte |= AsciiChar - '0';
		} else if ( (AsciiChar >= 'A') && (AsciiChar <= 'F')) {
			Byte |= (AsciiChar - 'A') + 10;
		} else {
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unicode String has invalid character (0x%X)\n",
				AsciiChar));
			Byte = 0;
		}
		Byte <<= 4;

		AsciiChar = (UCHAR)(UnicodeString[i*2 + 1]);

		DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("Unicode Char (0x%X)\n", AsciiChar));

		if ( (AsciiChar >= '0') && (AsciiChar <= '9')) {
			Byte |= AsciiChar - '0';
		} else if ( (AsciiChar >= 'A') && (AsciiChar <= 'F')) {
			Byte |= (AsciiChar - 'A') + 10;
		} else {
			DBGPRINT(
				DBG_PRINT_MODULE_USB_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unicode String has invalid character (0x%X)\n",
				AsciiChar));
			Byte = 0;
		}

		Bytes[i] = Byte;
	}

} // UsbPhyUnicodeStringHexToBytes

#ifdef GSI_USB
////////////////////////////////////////////////////////////////////////////////
// NAME: UsbPhyStartXDSLDsp
//
// ABSTRACT: Initialize the DSP and start the STARTUP_REQ process
//
// RETURNS:
//	   STATUS_UNSUCCESSFUL
//    STATUS_SUCCESS
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
NTSTATUS UsbPhyStartXDSLDsp( PUSB_PHY UsbPhy)
{
	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("UsbPhyStartXDSLDsp\n"));

	// Init before calling abort
        InitXCVR(UsbPhy->GTISysBlock);

	// ABORT_REQ to clear interrupts
	GSICallXcvr(UsbPhy, GTI_ABORT_REQ, 0);
	ExecuteXCVR_USB(UsbPhy->GTISysBlock, 0, 0);
#if defined(linux)
	set_current_state( TASK_INTERRUPTIBLE );
	schedule_timeout( (HZ * 5)/1000 );
#endif // linux
	UsbPhy->dspAction = GTI_STARTUP_REQ;

#ifdef EEPROM
	readMAC_Address(UsbPhy, UsbPhy->GTISysBlock , UsbPhy->AtmEndSystemIdentifier);
#endif // EEPROM

	return(STATUS_SUCCESS);
}

static void ctrl_callback( struct urb *urb )
{
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("Control Urb Status is %d\n",urb->status));
}

void GTISendUSBCmdEx(PVOID usbphy, UCHAR type, 
                     USHORT value, USHORT index,
                     GTI_BYTE *pData,  ULONG length, UCHAR getData)
{
#ifdef _NTDDK_
        NTSTATUS NtStatus;
#elif defined(linux)
	int	pipe = 0;
	int	ven = 0;
	int	ret = 0;
#endif // linux
	PUSB_PHY UsbPhy = (PUSB_PHY)usbphy;

	/* If the USB interface is in the HALTED state */
	if ( IsNotOperational (UsbPhy) )
	{
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("USB IS HALTED and CANNOT DOWNLOAD\n"));
		return;
	}
#ifdef _NTDDK_
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("Type=0x%x, Value=0x%x, Index=0x%x, Length=0x%x\n",type,value,index,length));

   UsbBuildVendorRequest(UsbPhy->dspFW.urbPtr,
                     URB_FUNCTION_VENDOR_DEVICE,
                     sizeof( struct _URB_CONTROL_VENDOR_OR_CLASS_REQUEST),
                     ((getData)?USBD_TRANSFER_DIRECTION_IN:USBD_TRANSFER_DIRECTION_OUT),
                     0,
                     type,
                     value,
                     index,
                     pData,
                     NULL,
                     length, /* When getData is set, length is MAXIMUM */
                     NULL);

   NtStatus = UsbPhyCallUSBD( UsbPhy, UsbPhy->dspFW.urbPtr);

	DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
      ("NTSTATUS for the FW_DOWNLOAD=%x\n",NtStatus));
#elif defined(linux)
	if( getData )
	{
		pipe = usb_rcvctrlpipe( UsbPhy->AdapterHandle->usbDev, 0 );
		ven = VEN_REQUEST_IN;
	}
	else
	{
		pipe = usb_sndctrlpipe( UsbPhy->AdapterHandle->usbDev, 0 );
		ven = VEN_REQUEST_OUT;
	}
	if( in_interrupt())
	{
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
			("Sending control in interrupt\n"));
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20)
        UsbPhy->dspFW.dr.bRequestType = ven;
        UsbPhy->dspFW.dr.bRequest = type;
        UsbPhy->dspFW.dr.wValue = cpu_to_le16(value);
        UsbPhy->dspFW.dr.wIndex = cpu_to_le16(index);
        UsbPhy->dspFW.dr.wLength = cpu_to_le16(length);
#else
		UsbPhy->dspFW.dr.requesttype = ven;
		UsbPhy->dspFW.dr.request = type;
		UsbPhy->dspFW.dr.value = cpu_to_le16(value);
		UsbPhy->dspFW.dr.index = cpu_to_le16(index);
		UsbPhy->dspFW.dr.length = cpu_to_le16(length);
#endif
		FILL_CONTROL_URB(UsbPhy->dspFW.urbPtr, UsbPhy->AdapterHandle->usbDev,
				pipe, (char *)&(UsbPhy->dspFW.dr), pData, length, ctrl_callback, 
				HZ * SET_TIMEOUT * 200);
		ret = usb_submit_urb( UsbPhy->dspFW.urbPtr );
		if( ret != USB_ST_NOERROR )
		{
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
				("Control Message failed in INTERRUPT %d\n", ret));
		}
	}
	else
	{
		ret = usb_control_msg( UsbPhy->AdapterHandle->usbDev, pipe, type, 
				ven, value, index, pData, length, HZ * SET_TIMEOUT * 100);
		if ( ret < 0 )
		{
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
				("Control Message failed in Non interrupt %d\n", ret));
		}
	}
#endif // linux		
	
}


///////////////////////////////////////////////////////////////////////////////
//  NAME: QueueUsbThread
//
//  ABSTRACT:  Spawns a thread to call USB layer functions.
///////////////////////////////////////////////////////////////////////////////
void QueueUsbThread( PUSB_PHY UsbPhy)
{
#ifdef _NTDDK_
	ExInitializeWorkItem( &UsbPhy->UsbProcessWorkItem, UsbProcessFunction, UsbPhy);
    ExQueueWorkItem( &UsbPhy->UsbProcessWorkItem, CriticalWorkQueue);
#elif defined(linux)
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
        ("> QueueUsbThread\n"));
	kernel_thread( UsbProcessFunction, UsbPhy,  (CLONE_FS | CLONE_FILES
                                         | CLONE_SIGHAND | SIGCHLD));
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
        ("< QueueUsbThread\n"));
#endif // linux
}
	

///////////////////////////////////////////////////////////////////////////////
//  NAME: UsbProcessFunction
//
//  ABSTRACT:  Call USB layer processing functions.
///////////////////////////////////////////////////////////////////////////////
#ifdef _NTDDK_
void UsbProcessFunction( PUSB_PHY UsbPhy)
#elif defined(linux)
int UsbProcessFunction( PUSB_PHY UsbPhy )
#endif // linux
{
	PDSP_PHY_QWORK_ITEM qworkItemPtr;
	KIRQL				Irql;

#if defined(linux)
# if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10) )	
	/* use the kernel's daemonize and reparent_to_init so that no 
	 * defunct thread is left once this thread finishes
	 */
	daemonize();
	reparent_to_init();
# elif ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )
	/* use the kernel's daemonize */
    daemonize();

	/* We'd like to reparent to init, but don't have a function to do it, 
	 * and symbols like child_reaper aren't exported to modules
	 */
#endif	
#endif // linux	

	//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
	//	("> UsbProcessFunction\n"));
	while ( TRUE )
	{
		KeAcquireSpinLock( &UsbPhy->DspBufferListLock, &Irql);

		if ( !IS_LIST_EMPTY( &UsbPhy->DspProcessCommandList) )
		{
			//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
			//	("UsbProcessFunction List not empty. Flag = %x\n", 
			//	UsbPhy->NeedQueueDspProcess ));

			qworkItemPtr = (PDSP_PHY_QWORK_ITEM)REMOVE_TAIL_LIST( &UsbPhy->DspProcessCommandList);
			INSERT_HEAD_LIST( &UsbPhy->DspProcessFreeList,(PLIST_NODE)qworkItemPtr);
			KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);

			if ( ProcessUsbCommands( UsbPhy, qworkItemPtr->action) )
			{
#ifdef _NTDDK_
                               return;
#elif defined(linux)
				if( UsbPhy->NeedQueueDspProcess == DSP_STOP )
				{
					DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
						("Stopping thread UsbProcessFunction \n"));
					UsbPhy->NeedQueueDspProcess = DSP_STOPPED;
					return(STATUS_PENDING);
				}
				continue;
#endif // linux
			}
		}
		else
		{
#ifdef _NTDDK_
			UsbPhy->NeedQueueDspProcess = TRUE;
			KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);
			return;
#elif defined(linux)
			//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
			//	("UsbProcessFunction List Empty. Flag = %x\n", 
			//	UsbPhy->NeedQueueDspProcess ));
			if( UsbPhy->NeedQueueDspProcess == DSP_STOP )
			{
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
					("Stopping thread UsbProcessFunction List Empty\n"));
				UsbPhy->NeedQueueDspProcess = DSP_STOPPED;
				return(STATUS_PENDING);
			}
			UsbPhy->NeedQueueDspProcess = DSP_WAITING;
			KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);

			interruptible_sleep_on(&(UsbPhy->AdapterHandle->StatusCheckQ));
			//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
			//	("Waking up from sleep %x\n", UsbPhy->NeedQueueDspProcess));
#endif // linux
		}
	}	// loop WHILE list not empty
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
		("Exiting from thread UsbProcessFunction \n"));
}


///////////////////////////////////////////////////////////////////////////////
//  NAME: AddDspCommand
//
//  ABSTRACT:  Add a command to process in USB layer functions.
///////////////////////////////////////////////////////////////////////////////
void AddDspCommand ( PUSB_PHY UsbPhy, DSP_CONN_ACTION Action)
{
	PDSP_PHY_QWORK_ITEM qworkItemPtr;
	KIRQL				Irql;

	KeAcquireSpinLock( &UsbPhy->DspBufferListLock, &Irql);

	// get element off the free work item list
	if ( !IS_LIST_EMPTY( &UsbPhy->DspProcessFreeList) )
	{
		//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
		//	("DSP buffer is not empty\n"));
		qworkItemPtr = (PDSP_PHY_QWORK_ITEM) REMOVE_TAIL_LIST( &UsbPhy->DspProcessFreeList);
		qworkItemPtr->action = Action;
		INSERT_HEAD_LIST( &UsbPhy->DspProcessCommandList, (PLIST_NODE)qworkItemPtr);
	}
	else
	{
		//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
		//	("DSP buffer is empty!\n"));
	}

	//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
	//	("Thread state is %x\n", UsbPhy->NeedQueueDspProcess ));
	// run queue processing routine if not currently running
	if ( UsbPhy->NeedQueueDspProcess == DSP_START )
	{
#ifdef _NTDDK_
		UsbPhy->NeedQueueDspProcess = FALSE;
#elif defined(linux)
		UsbPhy->NeedQueueDspProcess = DSP_RUNNING;
#endif // linux
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
			("Starting Thread first time!\n"));
		KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);
		QueueUsbThread( UsbPhy);
	}
#ifdef _NTDDK_
	else
		KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);
#elif defined(linux)
	else if( UsbPhy->NeedQueueDspProcess == DSP_WAITING )
	{

		KeReleaseSpinLock( &UsbPhy->DspBufferListLock, Irql);
		UsbPhy->NeedQueueDspProcess = DSP_RUNNING;
		{
			//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
			//	("Waking Dsp Thread\n"));
			wake_up_interruptible(&(UsbPhy->AdapterHandle->StatusCheckQ));
			//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
			//	("Dsp Thread Awakened\n"));
		}
	}
#endif // linux
}


////////////////////////////////////////////////////////////////////////////////
// NAME: ProcessDspCommands
//
// ABSTRACT: This routine is used to process DSP commands.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
void ProcessDspCommands( PUSB_PHY UsbPhy, DSP_CONN_ACTION dspAction)
{
	ULONG				result;
	USHORT              progress[GTI_START_PROGRESS_LEN];

		//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
		//	("> ProcessDspCommands Dsp State = %d, DspAction = %d\n",
		//	GetDspState(UsbPhy), dspAction));

	if (dspAction == DSP_WRITE_EEPROM_ACTION)
	{
		UsbPhyEEPROMAccess( UsbPhy, UsbPhy->GTISysBlock);
	}
#ifdef GTI_DMT
	else if (dspAction == DSP_EOC_MSG)	// always try to process the EOC buffer to keep in sync
	{
		DslEocProc(UsbPhy->DslHandle);
	}
#endif // GTI_DMT
#ifdef WINHIP 
    else if( dspAction == DSP_PROCESS_WHIP_MSG )
    {
        WhipDispatchMsg_USB(UsbPhy);
    }
#endif

	else
	{
		switch ( GetDspState (UsbPhy) )
		{
			case DSP_PHY_WAIT_CELLDEL:
			   break;

			case DSP_PHY_DISCONNECT:
			case DSP_PHY_RESTART:

			   // ABORT_REQ
			   GSICallXcvr(UsbPhy, GTI_ABORT_REQ, 0);
			   ExecuteXCVR_USB( UsbPhy->GTISysBlock, 0, 0);

			  	UsbPhy->LQAutoModeCount = 0;
			  	UsbPhy->CrcErrorsPerMin = 0;
			  	UsbPhy->ExcessiveConsecutiveCrcErrorsPerTickCount = 0;
				UsbPhy->ExcessiveConsecutiveCrcErrorsPerMinCount = 0;
				UsbPhy->TickCounter = 0;

			   // Do we currently want to start up again?
			   if (GetDspState (UsbPhy) == DSP_PHY_RESTART)
			   {
					// Determine ISO resources and start training
					UsbPhy->dspAction = GTI_STARTUP_REQ;
					SetDspState(UsbPhy, DSP_PHY_IDLE, DSP_INTERRUPT);
			   }
			   else
			   {
					SetDspState(UsbPhy, DSP_PHY_IDLE, DSP_NO_ACTION);
			   }

			   SetUsbState(UsbPhy, USB_PHY_STATE_CONFIG);

			   break;

			case  DSP_PHY_IDLE:
			   if (dspAction == DSP_INTERRUPT)
			   {
#ifdef GTI_DMT
//					InitXCVR_CS( UsbPhy->GTISysBlock, 0, GTI_DSP_A_MSK, 0, 0, 0, 0);
#endif // GTI_DMT
					InitXCVR( UsbPhy->GTISysBlock);

					// Turn OFF the interrupt
					InitSTARInterrupt( UsbPhy->GTISysBlock, GTI_FALSE);

					// Initialize the ParamArray
					GSISetXcvrParameters( UsbPhy, 1);
					DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
						("XCVR Called with Act=%d\n",UsbPhy->dspAction));

					GSICallXcvr(UsbPhy, UsbPhy->dspAction, 0);

					SetDspState(UsbPhy, DSP_PHY_NO_SIGNAL, DSP_NO_ACTION);
					// Start to download the code
					result = ExecuteXCVR_USB( UsbPhy->GTISysBlock, 0, 0);

					DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
						   ("Start STARTUP_REQ result = %lx\n", result));

					if (result == 0)
					{
					   UsbPhyConfigureIsoBW(UsbPhy);
					}
					else
					{
					   // Check, if there is no signal for DSL
					   Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_START_PROGRESS,
							 &(progress[0]), GTI_START_PROGRESS_LEN, GTI_DSP_A);
                      /* Progress, will indicate if there is some type of signal */
					   if ((progress[0] == 0xA0) || (progress[0] == 0x1) || (progress[0] == 0x2))
						  SetDspState(UsbPhy, DSP_PHY_NO_SIGNAL, DSP_NO_ACTION);
					   else
						  SetDspState(UsbPhy, DSP_PHY_TRAINING, DSP_NO_ACTION);
					}
			   }
			   break;

			case DSP_PHY_NO_SIGNAL:
			case  DSP_PHY_TRAINING:
			   if (dspAction == DSP_INTERRUPT)
			   {
				   // Continue with the download process
				   //DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
					//	("nextState=%x\nDSP DOWNLOAD IN PROGRESS\n",UsbPhy->GTISysBlock[0].CustXcvrInfo.status->getNextState));

				   // Initalilze the outstanding cells
				   UsbPhy->partialCellLen = 0;
				   if ( !(ExecuteXCVR_USB( UsbPhy->GTISysBlock, UsbPhy->isIpuReady, UsbPhy->getNextState)) )
				   {
					   UsbPhyConfigureIsoBW(UsbPhy);
				   }
				  else
				  {
					   // Check, if there is no signal for DSL
					   Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_START_PROGRESS,
								 &(progress[0]), GTI_START_PROGRESS_LEN, GTI_DSP_A);
                      /* Progress, will indicate if there is some type of signal */
					   if ((progress[0] == 0xA0) || (progress[0] == 0x1) || (progress[0] == 0x2))
						  SetDspState(UsbPhy, DSP_PHY_NO_SIGNAL, DSP_NO_ACTION);
					   else
						  SetDspState(UsbPhy, DSP_PHY_TRAINING, DSP_NO_ACTION);
				  }
			   }
			   break;

			case  DSP_PHY_CONNECTED:

#ifdef GTI_DMT
			   switch(dspAction)
			   {
				case DSP_PROFILE_MNGT:

					GSICallXcvr(UsbPhy, GTI_PROF_MNGT_REQ, 0);

					if ( !(ExecuteXCVR_USB( UsbPhy->GTISysBlock, 0, 0)) )
					{
						UsbPhy->GTISysBlock[0].CustXcvrInfo.status->opState = GTI_DATA_OP;

						SetDspState(UsbPhy, DSP_PHY_CONNECTED, DSP_NO_ACTION);
						UsbPhy->LinkState = ATM_PHY_LINK_STATE_CONNECTED;
						// Indicate the new link state
						(UsbPhy->UserLinkStatusChange)(
							UsbPhy->UserHandle,
							UsbPhy->LinkState,
							UsbPhy->UpstreamRate,
							UsbPhy->DownstreamRate,
							0,
							UsbPhy->RxCellPadding);
					}

					break;

				case DSP_BIT_SWAPPING:
					GSICallXcvr(UsbPhy, GTI_BIT_SWAPPING_REQ, 0);
					if ( !(ExecuteXCVR_USB( UsbPhy->GTISysBlock, 0, 0)) )
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
					   ("ExecuteXCVR_USB return Value DSP_BIT_SWAPPING= 0"));

					   UsbPhy->GTISysBlock[0].CustXcvrInfo.status->opState = GTI_DATA_OP;
                                           UsbPhy->BitSwapCounterFlag = TRUE;
					}
				   break;

				case DSP_FAST_RETRAIN:
					GSICallXcvr(UsbPhy, GTI_FAST_RETRAIN_REQ, 0);
					SetDspState(UsbPhy, DSP_PHY_TRAINING,DSP_NO_ACTION);
					if ( !(ExecuteXCVR_USB( UsbPhy->GTISysBlock, 0, 0)) )
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
							("ExecuteXCVR_USB return Value DSP_FAST_RETRAIN = 0"));
					}
					break;           

				case DSP_EOC_MSG:	// this case should be handled above
				   DslEocProc(UsbPhy->DslHandle);
				   break;

				default:
				   break;
			   } //end switch
#endif // GTI_DMT
			   break;

			default:
			   break;

		}  // oldState SWITCH
	}	// if not EEPROM or EOC_MSG
	//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
	//	("< ProcessDspCommands\n"));
}

////////////////////////////////////////////////////////////////////////////////
// NAME: UsbPhyInitXcvr
//
// ABSTRACT: Initialize transceiver settings.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
void UsbPhyInitXcvr(PUSB_PHY UsbPhy)
{
    // FIXME : old function call that incorrectly allocates memory locally and
    // sets our pointer to this memory which immediately leaves scope.
    // Temp fix resets the pointer to our own memory to prevent page fault.
    // We need to use the newer APIs for a permanent solution.
  //  InitXCVR_CS( UsbPhy->GTISysBlock, 0, GTI_DSP_A_MSK, 0, 0, 0, 0);
    UsbPhy->GTISysBlock->pChipInfo = &UsbPhy->ChipInfo;

    InitXCVR( UsbPhy->GTISysBlock);
    
    // Turn OFF the interrupt
    InitSTARInterrupt( UsbPhy->GTISysBlock, GTI_FALSE);
}

////////////////////////////////////////////////////////////////////////////////
// NAME: UsbPhyDslConnected
//
// ABSTRACT: Physical layer(DSL) is established and connected. Let other layers
//             know, so they can start transferring data.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
void UsbPhyDslConnected(PUSB_PHY UsbPhy)
{
	KIRQL		Irql;
	USHORT		ret;
	USHORT		actualConfig[GTI_ACTUAL_CONFIGURATION_LEN];
	USHORT      COType;
   USHORT      i;
#ifdef _NTDDK_
    KEVENT          Event;
#endif // _NTDDK_
	LONG            longTimeout;
	LARGE_INTEGER   longIntTimeout; 
#ifdef ANNEXB
	USHORT      rmtVendorIdStatus;
	USHORT      rmtVendorRevStatus;
	USHORT      statusLen=0;
#endif // ANNEXB

	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		   ("PHY LINK IS CONNECTED\n"));

	/* Intialize the speedRate */
	UsbPhy->UpstreamRate	= 0;
	UsbPhy->DownstreamRate	= 0;
   UsbPhy->SelectStandardCounter = 0;

	for(i=0;i<(GTI_COUNTERS_LEN/sizeof(USHORT));i++)
   {
		UsbPhy->DspCounters[i] = 0;
       UsbPhy->DslCounters[i] = 0;
   }
   for(i=0; i < (GTI_FAILURES_LEN/sizeof(USHORT)); i++)
       UsbPhy->DslFailures[i] = 0;

	/* Change the Physical state to connected */
	if (UsbPhy->dspAction == GTI_DIGITAL_LB_REQ)
	{
		SetDspState(UsbPhy, DSP_PHY_LOOPBACK_MODE, DSP_NO_ACTION);
	}
	else if((UsbPhy->dspAction == GTI_SPECTRUM_MEDLEY_REQ)  | 
		   (UsbPhy->dspAction == GTI_SPECTRUM_REVERB_REQ)  |
		   (UsbPhy->dspAction == GTI_SPECTRUM_PILOT_REQ)   |
		   (UsbPhy->dspAction == GTI_SPECTRUM_R_PBIN_REQ))
	{
		SetDspState(UsbPhy, DSP_PHY_SPECTRUM_MODE, DSP_NO_ACTION);
	}
	else
	{
#ifdef    _NTDDK_
		/* Update the standard value in the registry */
		if (UsbPhy->autoMode)
			WriteConfigReg (UsbPhy);
#endif    //_NTDDK_

		/* Get the Link Speed by calling Actual Configuration */
		KeAcquireSpinLock( &(UsbPhy->DspLock), &Irql);
		ret = Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_ACTUAL_CONFIGURATION,
			   &(actualConfig[0]), GTI_ACTUAL_CONFIGURATION_LEN, GTI_DSP_A);
		KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);

		UsbPhy->DownstreamRate= (USHORT)(actualConfig[GTI_AS0_BYTES_ACTUAL] * 32);
		UsbPhy->UpstreamRate = (USHORT)(actualConfig[GTI_LS0_BYTES_ACTUAL] * 32);
                COType = actualConfig[GTI_SELECTED_STANDARD_ACTUAL];

#ifdef ANNEXB
       /* Check for ADI */
       if (COType == GTI_T1_413_STANDARD)
       {
           /* Find remote information - VendorID and VendorRev */
           statusLen = Gti_SStatusXCVR( UsbPhy->GTISysBlock, GTI_RMT_VENDOR_REV_STATUS, &(rmtVendorRevStatus),
				  	          GTI_RMT_VENDOR_REV_STATUS_LEN, GTI_DSP_A);

           statusLen = Gti_SStatusXCVR( UsbPhy->GTISysBlock, GTI_RMT_VENDOR_ID_STATUS, &(rmtVendorIdStatus),
				  	          GTI_RMT_VENDOR_ID_STATUS_LEN, GTI_DSP_A);
                             
           /* is it ADI930 ? */
           if ((rmtVendorIdStatus == 0x1C) && (rmtVendorRevStatus == 0x0D))
               COType = 0x12;
       }
#endif // ANNEXB

	    UsbPhy->DspActualTrainingMode = COType;
		SetDspState(UsbPhy, DSP_PHY_CONNECTED, DSP_NO_ACTION);
	}

	DslEocInitialize( UsbPhy->DslHandle, UsbPhy, actualConfig);	// init EOC
	SetDataMode_USB( UsbPhy->GTISysBlock);	// set ISO bandwidth and notify controller of data mode

#ifdef _NTDDK_
   if (UsbPhy->DataDelay)
   {
       // mike : Add a delay for WHQL
       KeInitializeEvent( &Event, NotificationEvent, FALSE);
       longTimeout = (-1) * (UsbPhy->DataDelay * 10000000);		// delay in seconds
       longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);
       KeWaitForSingleObject(
               &Event,
               Executive,   
               KernelMode,
               FALSE ,
               &longIntTimeout);
   }
#endif // _NTDDK_

	UsbPhy->LinkState = ATM_PHY_LINK_STATE_CONNECTED;

	/* Indicate the new link state */
	(UsbPhy->UserLinkStatusChange)(
			   UsbPhy->UserHandle,
			   UsbPhy->LinkState,
			   UsbPhy->UpstreamRate,
			   UsbPhy->DownstreamRate,
				COType,
			   UsbPhy->RxCellPadding);
}


////////////////////////////////////////////////////////////////////////////////
// NAME: UsbPhyDslDisconnected
//
// ABSTRACT: Physical layer(DSL) is disconnected. Let other layers
//             know, so they can stop transferring data.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
void UsbPhyDslDisconnected(PUSB_PHY UsbPhy)
{
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
           ("PHY LINK IS DISCONNECTED\n"));
   
	UsbPhy->LinkState =   ATM_PHY_LINK_STATE_DISCONNECTED;
        UsbPhy->SelectStandardCounter = 0;

	UsbPhy->UpstreamRate	= 0;
	UsbPhy->DownstreamRate	= 0;
	/* Indicate the new link state to upper layers */
	(UsbPhy->UserLinkStatusChange)(
			UsbPhy->UserHandle,
			UsbPhy->LinkState,
			UsbPhy->UpstreamRate,
			UsbPhy->DownstreamRate,
			0,
			UsbPhy->RxCellPadding);
}

////////////////////////////////////////////////////////////////////////////////
// NAME: UsbPhyDslCheckStatus
//
// ABSTRACT: Periodically check the Physical layer(DSL) status to perform correct
//             action based on the status event. This function gets called when
//             in data mode.
// 
//	        Check for Excessive CRC Errors to see if we need to retrain.
//
//	        There are currently three criteria defined for starting a retrain 
//	        because of excessive CRC errors (AN-029; Rev 1.0; Sept 12, 2000):
//	
//         1. OVERALL_FAILURE :
//
//	        2. "Short term" : 25 or more crc errors per second for 3 consecutive
//					          seconds.
//	        3. "Long term": error rate per minute exceeeds the number of bytes per 
//					        DMT symbol for 5 consecutive minutes.
//
//	        All calculations and thresholds in the code below are derived from 
//	        defines in DSP.H and registry keys that can override them. They are 
//         then adjusted to use the period of the DSP_STATUS_CHECK (a.k.a "tick" 
//	        or "dsp_status_tick") as the time base.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
void UsbPhyDslCheckStatus(PUSB_PHY UsbPhy)
{
USHORT     ret;
USHORT		retFailure  [(GTI_FAILURES_LEN)/sizeof(USHORT)];
USHORT		Counters    [(GTI_COUNTERS_LEN)/sizeof(USHORT)];
UCHAR		retOpState  [GTI_OP_STATE_LEN];
USHORT     crcErrCount;
USHORT	    ncdxErrCount;
BOOLEAN    retrain = FALSE;
KIRQL		Irql;
USHORT		i;

   UsbPhy->LQAutoModeCount = 0;

   KeAcquireSpinLock( &(UsbPhy->DspLock), &Irql);
   /* 
   **  Read ERROR COUNTERS from DSP. 
   */
   ret = Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock,
	      GTI_COUNTERS, (USHORT *) Counters, sizeof(Counters), GTI_CH_A);

   if ( (ret == 0) || (ret != sizeof(Counters)) )
   {
       /* 
       **  This should not have happened, something is wrong
       **  so do not update the UsbPhy copy of the counters.
       **  However, we could still have valid readings for 
       **  GTI_FAILURES, so we don't want to bail out here.
       **  Just zero out the local copy of the counters before 
       **  we continue so that further code that may use it is 
       **  not fooled with incorrect count info.
       */
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
	         ("Error in reading Counters:Requested= %d, Actually= %d\n",
	         sizeof(Counters), ret));
       NdisZeroMemory(Counters, sizeof(Counters));
       KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);
       return;
   }

   /*
   **  Read the GTI_OP_STATE. 
   */
   ret = Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_OP_STATE,
	                &(retOpState[0]), GTI_OP_STATE_LEN, GTI_DSP_A);

   if ( (ret == 0) || (ret != GTI_OP_STATE_LEN) )
   {
       /*
       **  If the GTI_OP_STATE cannot be read there is no point
       **  for the retrain check, so just break out.
       */
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
	    ("Error Reading Dsp's GTI_OP_STATE ! Check for Retrain need skipped !\n"));
       KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);
       return;
   }

   /*
   **  Read the GTI_FAILURES. 
   */
	ret = Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_FAILURES,
			    &(retFailure[0]), GTI_FAILURES_LEN, GTI_DSP_A);

   if ( (ret == 0) || (ret != GTI_FAILURES_LEN) )
   {
	    DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
   		 ("Error Reading Dsp for GTI_FAILURES ! Skipping OVERALL_FAILURES check.\n"));
       KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);
       return;
   }
   KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);

   switch(retOpState[0])
   {
       case GTI_BIT_SWAPPING_OP:
           if (UsbPhy->BitSwapCounterFlag)
           {
               UsbPhy->BitSwapCounterFlag = FALSE;
               SetDspState(UsbPhy, DSP_PHY_CONNECTED, DSP_BIT_SWAPPING);
               return;
           }
           break;
   }
   /*
   **  Counters[] is a local array to this procedure in which the DSP
   **  hardware counters are read each tick. Since the hardware counters  
   **  get cleared after read, this array will hold the count during the
   **  last tick only.
   **
   **  If reading was successful the contents of Counters[] are accumulated 
   **  into the DspCounters[] array in the instance structure, from where
   **  any entity that needs a count that spans more than one tick time 
   **  can read them.
   **
   **  NOTE: Roll over protection of the instance copy is not provided on
   **  purpose at this time. Being that it is a 16 bit counter we don't 
   **  expect this to happen without trigering OVERALL_FAILURES which will
   **  cause the retrain anyway. On top of that the Counters of the DSP hw/fw
   **  itself are 16 bit with no rollover protection. 
   **
   ** The information provided above for Counters[] is true for failures[].
   */
   for(i=0; i < (GTI_COUNTERS_LEN/sizeof(USHORT)); i++)
   {
	   UsbPhy->DspCounters[i] += Counters[i];
	   UsbPhy->DslCounters[i] += Counters[i];
   }

   for(i=0; i < (GTI_FAILURES_LEN/sizeof(USHORT)); i++)
   {
	   UsbPhy->DslFailures[i] += retFailure[i];
   }
   /*
	**  Assume only one of Fast or Interleaved CRC errors can be nonzero at 
   **  one time.
   **
   **  NOTE: Roll over protection of the instance per minute crc error 
   **  count is not provided on purpose at this time. Being that it is 
   **  a 16 bit counter we don't expect this to happen without triggering
   **  OVERALL_FAILURES or at least having excess per tick CRC errors. 
   **  Any of the latter two will cause the retrain anyway. Also, the 
   **  Counters of the DSP hw/fw itself are 16 bit with no rollover 
   **  protection. 
   */
   ncdxErrCount = Counters[GTI_LOCAL_NCDI_COUNTER] + Counters[GTI_LOCAL_NCDF_COUNTER];
	crcErrCount = Counters[GTI_LOCAL_CRSI_COUNTER] + Counters[GTI_LOCAL_CRSF_COUNTER];
	UsbPhy->CrcErrorsPerMin += crcErrCount;

   /* Check if the connected standard is correct and does not require
   ** special action. The AutoStandardFlag parameter allows user to
   ** enable/disable the feature.
   */
   if (UsbPhy->SelectStandard)
   {
#ifdef ANNEXB
       /* Check, if the errors occur only for the NCDx (No cell delineation) */
		if ((ncdxErrCount >= MAX_NCDX_ERRORS_PER_SEC) && (crcErrCount == 0))
       {
           /* Select the standard within MAX_TIME_2_SELECT_STANDARD time after going to data mode */
           if (UsbPhy->SelectStandardCounter >= MAX_TIME_2_SELECT_STANDARD)
           {
               UsbPhy->SelectStandardCounter = 0;
		        retrain = TRUE;

		        switch (UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM])
		        {
		            case GTI_ALCTAnnB_STANDARD:
		                //UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = GTI_ALCTAnnB4_1_STANDARD;
		                UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = 0x15;
		                break;
       
		            //case GTI_ALCTAnnB4_1_STANDARD:
		            case 0x15:
		                UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = GTI_ALCTAnnB_STANDARD;
		                break;
       
		            default:
                       /* Do not want to select the default value */
		                break;
		        }
		        DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
				         ("%d consecutive Excessive CRC errors per tick detected! Retrain will be requested\n", 
				         UsbPhy->ncdxErrorsPerTickCounter));
		    }
           else
           {
               /* Number of times the error has been encountered, wait for the threshold */
               UsbPhy->SelectStandardCounter++;
               return; // We can return because nothing to do except to wait.
           }
       }
#endif //ANNEXB
   }

	/*
	**	Check to see if we need to retrain because of overall 
	**	failures or excessive CRC errors. 
	*/
   if (UsbPhy->RetrainEnable) 
	{
		//DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		//		 ("LQFailureCounter= %d TickCounter= %d, CrcErrorsThisTick= %d, CrcErrorsPerMin= %d\n",
		//		 UsbPhy->LQFailureCount, UsbPhy->TickCounter, crcErrCount, UsbPhy->CrcErrorsPerMin));

       /*
       **	Criteria 1) OVERALL_FAILURE :
       */ 
       if (retFailure[GTI_OVERALL_FAILURE] != 0)
       {
           UsbPhy->LQFailureCount++;
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
						("LQ failure count incremented to %d\n", UsbPhy->LQFailureCount));

           // RESTART if 1 consecutive failure(s)
           if (UsbPhy->LQFailureCount >= 1) 
           {
               UsbPhy->LQFailureCount = 0;
               retrain = TRUE;
           }
       }
		else
       {
           UsbPhy->LQFailureCount = 0;
       }

		UsbPhy->TickCounter++;

		/*
       **
       ** Criteria 2) "Short term" : Least 25 crc/second for 3 consecutive.
       ** Check for excessive CRC errors per tick (closest to per sec)
       **
       */
		if (crcErrCount >= ConvertPerSecondCountToPerTickCount(UsbPhy->MaxCrcErrorsPerSecThreshold))
		{
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
					 ("Excessive CRC Errors per DSP Status Tick (%d) detected! Consecutive Times = %d \n", \
					 crcErrCount, UsbPhy->ExcessiveConsecutiveCrcErrorsPerTickCount));

			UsbPhy->ExcessiveConsecutiveCrcErrorsPerTickCount++;

           // Have we reached the consecutive times needed for a retrain already ?
			if(UsbPhy->ExcessiveConsecutiveCrcErrorsPerTickCount >= TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_TICK)
			{
				retrain = TRUE;
			}
		}
		else
		{
           // Since this tick there weren't excessive errors, clear the consecutive count.
			UsbPhy->ExcessiveConsecutiveCrcErrorsPerTickCount = 0;
		}

       /*
       ** Criteria 3) "Long term": CRC rate/minute exceeeds the # of bytes/symbol 
       **              for 5 consecutive minutes.
       ** Has 1 minute elapsed ? 
       */
		if (UsbPhy->TickCounter >= DSP_STATUS_TICKS_PER_MIN)
		{
		    // Calculate threshold for error rate per minute which is just equal to the number 
		    // of bytes per DMT symbol (Default for DownstreamRateToMaxCrcErrorsPerMinDivider
		    // is 32 and provides this computation, but can be overwritten from the registry).
			UsbPhy->CrcErrorsPerMinThreshold = (USHORT) (UsbPhy->DownstreamRate / UsbPhy->DownstreamRateToMaxCrcErrorsPerMinDivider);
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
					("DownstreamRate = %d, CrcErrorsPerMinThreshold = %d\n",
					UsbPhy->DownstreamRate, UsbPhy->CrcErrorsPerMinThreshold));

			// Check for excessive CRC errors per min 
			if(UsbPhy->CrcErrorsPerMin >= UsbPhy->CrcErrorsPerMinThreshold)
			{
				UsbPhy->ExcessiveConsecutiveCrcErrorsPerMinCount++;
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
						("Excessive CRC Errors per Min (%d) detected! Consecutive Times = %d \n", \
					  	UsbPhy->CrcErrorsPerMin, UsbPhy->ExcessiveConsecutiveCrcErrorsPerMinCount));

               // Have we reached the consecutive times needed for a retrain already ?
				if(UsbPhy->ExcessiveConsecutiveCrcErrorsPerMinCount >= MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN)
				{
					retrain = TRUE;
				}
			}
			else
			{
               // Since this minute there weren't excessive errors, clear the consecutive count.
				UsbPhy->ExcessiveConsecutiveCrcErrorsPerMinCount = 0;
			} // else (UsbPhy->CrcErrorsPerMin >= UsbPhy->CrcErrorsPerMinThreshold)

			// Reset Counters so that we start keeping track of the next minute.
			UsbPhy->CrcErrorsPerMin = UsbPhy->TickCounter = 0;
		}
	}

	/*
	**	Set the retrain Dsp Status (DSP_PHY_RESTART) if needed.
	*/
	if (retrain)
	{
	 	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
	 		("** DSP state set to restart **\n"));

	 	// The dspState is changed to disconnect, but actual
	 	// disconnect will not occur till later.
		SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
	}
}

////////////////////////////////////////////////////////////////////////////////
// NAME: ProcessUsbCommands
//
// ABSTRACT: This routine will do status, suspend, resume, reset, or halt.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
////////////////////////////////////////////////////////////////////////////////
BOOLEAN ProcessUsbCommands (PUSB_PHY UsbPhy, DSP_CONN_ACTION dspAction)
{
#ifdef _NTDDK_
	LONG				longTimeout  = (-1) * (50 * 10000);
	LARGE_INTEGER		longIntTimeout;
	KEVENT				resetEvent;
#elif defined(linux)
	LONG				longTimeout  = (-1) * (50 * 10000);
	LARGE_INTEGER		longIntTimeout;
#endif // linux
	BOOLEAN				ResetError;
	ULONG				ResetCount;



	//DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_TRACE,
	//	("> ProcessUsbCommands UsbState = %d, DspState = %d, Act = %d\n", 
	//	GetUsbState(UsbPhy), GetDspState(UsbPhy), dspAction));

	if (dspAction != DSP_NO_ACTION)	// always try to process
		ProcessDspCommands( UsbPhy, dspAction);

	// State will indicate what we need to do (status, suspend, resume, reset, halt)
	switch ( GetUsbState (UsbPhy) )
	{
		case USB_PHY_STATE_OPERATIONAL:
#ifdef GTI_DMT
           /* Has the counter been reached to the timer expiry? */
	   if ( UsbPhy->CheckDslStatus )		// 1500 ms has passed, check Line Quality
	   {
		   UsbPhy->CheckDslStatus = FALSE;

               /* Check, if PHY connected? */
		   switch ( GetDspState(UsbPhy) )
		   {
			   case DSP_PHY_CONNECTED:
                       /* Check up on the DSP status */
                       UsbPhyDslCheckStatus(UsbPhy);
				   break;
       
			   case DSP_PHY_IDLE:
			   case DSP_PHY_LOOPBACK_MODE:
			   case DSP_PHY_SPECTRUM_MODE:
					UsbPhy->LQAutoModeCount = 0;
				   break;
    
			   default:
				   /* Automatically select the standard to be connected */
					if ((UsbPhy->LQAutoModeCount++) >= MAX_SELECT_TIME)
					{
						if (UsbPhy->autoMode)
						{
						   /* Change the standard and try again */
						   switch (UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM])
						   {
							        case GTI_T1_413_STANDARD:
								   UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = GTI_G_DMT_STANDARD;
								   break;
                       
							   case GTI_G_LITE_STANDARD:
							   case GTI_G_DMT_STANDARD:
								        UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = GTI_T1_413_STANDARD;
								   break;
                       
							   default:
								        UsbPhy->xcvr[0].item[GTI_STANDARD_ITEM] = GTI_G_DMT_STANDARD;
								   break;
						   }
							    SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
						}
					   else if (UsbPhy->RetrainEnable)
							    SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
						UsbPhy->LQAutoModeCount = 0;
					}
				   break;
		   } // switch ( GetDspState(UsbPhy) )
	   } // if ( UsbPhy->CheckDslStatus )
	#endif /* GTI_DMT */
		break;

		case USB_PHY_STATE_CONFIG:

            UsbPhyDslDisconnected(UsbPhy);
			UsbPhyShutdown(UsbPhy);

			// Start off using minimal bandwidth resources
			UsbPhy->ActualDsDataRate = USB_ALT_SETTINGS_INDEX_DEFAULT;
			if ( !NT_SUCCESS(UsbPhyInitialize(UsbPhy, FALSE)) )
			{
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
					("UsbPhyInitialize failed during CONFIG... will shutdown\n"));

				SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
			}

		break;

		case USB_PHY_STATE_RECONFIG:

			DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
				("Reconfiguration taking place\n"));
			UsbPhyShutdown(UsbPhy);

			if ( !NT_SUCCESS(UsbPhyInitialize(UsbPhy, FALSE)) )
			{
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
					("UsbPhyInitialize failed during RECONFIG... will shutdown\n"));

				SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
			}
			else if (UsbPhy->maxDownRateEnable)		// dont retrain
			{
				UsbPhyDslConnected(UsbPhy);
			}
			else if ( (!UsbPhy->ActualIsoBwUsed) && (UsbPhy->ActualDsDataRate) )	// wanted ISO but got bulk
			{
				SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
			}
			else if ( UsbPhy->ActualDsDataRate < UsbPhy->ActualIsoBwUsed )	// data rate > ISO bandwidth allocated
			{
				SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
			}
			else	// data rate <= bandwidth allocated
			{	
				UsbPhyDslConnected(UsbPhy);
			}
		break;

		case USB_PHY_STATE_HALTED:
			UsbPhy->ShutdownAck = TRUE;		// signal shut down completion
			return TRUE;		// exit this routine since the driver has been shut down

		case USB_PHY_STATE_SHUTDOWN:
			DslEocSendDyingGasp( UsbPhy->DslHandle);	// send dying gasp
			SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
			UsbPhyShutdown( UsbPhy);
		break;

	#ifdef POWERMGMT
		case USB_PHY_STATE_SUSPEND:
			DslEocSendDyingGasp( UsbPhy->DslHandle);	// send dying gasp

			// update connection status
			if ( DSP_PHY_CONNECTED == UsbPhy->dspState )
			{
				SetDspState(UsbPhy, DSP_PHY_IDLE, DSP_NO_ACTION);
				UsbPhyDslDisconnected(UsbPhy);
			}

			StopIrpAndTimersForPwrSuspend( UsbPhy->UserHandle);	// stop IRP handling and timers

			SetUsbState (UsbPhy, USB_PHY_STATE_RESUME);
			UsbPhyShutdown(UsbPhy);

			UsbPhy->ShutdownAck = TRUE;		// signal thread completion (in case suspend failed)

			CompletePwrSuspend( UsbPhy->UserHandle);	// complete PM status

			return TRUE;		// exit this routine since the driver has been suspended
		break;

		case USB_PHY_STATE_RESUME:

			UsbPhy->ShutdownAck	= FALSE;

            // Report failure of USBD if initialize returns status of failure
			if ( !NT_SUCCESS(UsbPhyInitialize(UsbPhy, TRUE)) )
			{
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
					("UsbPhyInitialize failed ... will re-enumerate\n"));

				SetUsbState (UsbPhy, USB_PHY_STATE_HALTED);
				UsbPhy->ShutdownAck = TRUE;		// signal thread completion
				CompletePwrResume( UsbPhy->UserHandle, FALSE);
				return TRUE;	// exit this routine since the driver has failed resume
			}

			CompletePwrResume( UsbPhy->UserHandle, TRUE);
			SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
		break;
	#endif	// POWERMGMT

		case USB_PHY_STATE_RESETTING_PORT:
		//	  A case that resets the iso pipe or resets the port, and then reissues URBs to the USBD.  
		//    We need to reset the iso pipe when to solve a problem where a USB problem
		//    shuts down the pipe and returns a URB with a Bad Frame error.
		//    We need to reset the pipe after the USB port is disabled because of a babble violation.

            // stop transmitting data
            (UsbPhy->UserLinkStatusChange)( UsbPhy->UserHandle, ATM_PHY_LINK_STATE_RESETTING, 0, 0, 0, 0);

#ifdef _NTDDK_
			KeInitializeEvent(&resetEvent , NotificationEvent, FALSE);
#endif // _NTDDK_
			DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, 
				("Resetting Port\n"));
			ResetError = TRUE;
			while (ResetError)
			{
				ResetError = FALSE;

				// abort all the pipes to clear all URBs out of the queue.
				DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("Aborting all pipes\n"));
#ifdef _NTDDK_
				if (UsbPhy->InterruptPendingCount!=0) 
#elif defined(linux)
				if (atomic_read(&(UsbPhy->InterruptPendingCount))!=0) 
#endif // linux
				{
					UsbPhyAbortPipe(UsbPhy, UsbPhy->InterruptPipeHandle);
					ResetError = TRUE;
				}

#ifdef _NTDDK_
				if (UsbPhy->BulkTransmitPending!=0)
#elif defined(linux)
				if (atomic_read(&(UsbPhy->BulkTransmitPending)) != 0)
#endif // linux
				{
					UsbPhyAbortPipe(UsbPhy, UsbPhy->BulkTransmitPipeHandle);
					ResetError = TRUE;
				}

#ifdef _NTDDK_
				if (UsbPhy->PendingIoCount!=0)
#elif defined(linux)
				if (atomic_read(&(UsbPhy->PendingIoCount)) != 0)
#endif // linux
				{
					UsbPhyAbortPipe(UsbPhy, UsbPhy->BulkReceivePipeHandle);
					ResetError = TRUE;
				}

#ifdef _NTDDK_
				if (UsbPhy->IsoPendingReceive!=0)
#elif defined(linux)
				if (atomic_read(&(UsbPhy->IsoPendingReceive)) != 0)
#endif // linux
				{
					UsbPhyAbortPipe(UsbPhy,UsbPhy->IsoReceivePipeHandle);
#ifdef _NTDDK_
                   /*
                   ** When testing suspend/hibernate under WHQL, problem is caused
                   ** by our driver in performing and preventing these tests to run.
                   ** Following is the reason for the problem:
                   ** Receving suspend command in the middle of resetting port
                   ** causes machine not to suspend/hibernate. The stats change
                   ** from RESETTING_PORT to SUSPEND causes some of the URBs to
                   ** be queued for processing. We cannot queue any packet when
                   ** resetting port.
                   */
                   /* Always check, if any of the Iso packets are stored in the Queue.
                   */
                   FreeIsoUrbFromQedPkts(UsbPhy);
#endif // _NTDDK_
					ResetError = TRUE;
				}

				if (ResetError)
				{
					// Wait 50 ms
#ifdef _NTDDK_
					longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);
					KeWaitForSingleObject(
						&resetEvent,
						Executive,   
						KernelMode,
						FALSE ,
						&longIntTimeout );
#elif defined(linux)
					set_current_state( TASK_INTERRUPTIBLE );
					if(!in_interrupt())
						schedule_timeout( (HZ * 5)/1000 );
#endif // linux
				}
			}

			ResetCount = 0;
			ResetError = TRUE;
			while (ResetError)
			{
				ResetCount++;
				ResetError = FALSE;
#ifdef _NTDDK_
				if (USB_PHY_STATE_SHUTDOWN != GetUsbState (UsbPhy))
#elif defined(linux)
                                // Nobody resets USB state in linux to shutdown so we check for Resetting_port status
				if (USB_PHY_STATE_RESETTING_PORT == GetUsbState (UsbPhy))
#endif // linux
				{
					// Don't know if we really need to reset the pipes, but it won't hurt.
					DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("Resetting all pipes\n"));

					if ( !NT_SUCCESS( UsbPhyResetPipe(UsbPhy, UsbPhy->InterruptPipeHandle) )) 
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
							("UsbPhyResetPipe:  Unable to submit URB for InterruptPipeHandle\n"));
						ResetError = TRUE;
					}

					if ( !NT_SUCCESS( UsbPhyResetPipe(UsbPhy, UsbPhy->BulkTransmitPipeHandle) )) 
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
							("UsbPhyResetPipe:  Unable to submit URB for BulkTransmitPipeHandle\n"));
						ResetError = TRUE;
					}

					if (UsbPhy->BulkReceivePipeHandle)
					{
						if ( !NT_SUCCESS( UsbPhyResetPipe(UsbPhy, UsbPhy->BulkReceivePipeHandle)) ) 
						{
							DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
								("UsbPhyResetPipe:  Unable to submit URB for BulkReceivePipeHandle\n"));
							ResetError = TRUE;
						}
					}

					if (UsbPhy->IsoReceivePipeHandle) 
					{
						 if (!NT_SUCCESS( UsbPhyResetIsoPipe(UsbPhy) ))
						 {
							DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
								("UsbPhyResetPipe:  Unable to submit URB for IsoReceivePipeHandle\n"));
							ResetError = TRUE;
						 }
					}

					// Now actually ask USBD to activate the port again
					DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("Resetting parent port\n"));

					if ( !NT_SUCCESS( UsbPhyResetParentPort(UsbPhy) )) 
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
							("UsbPhyResetParentPort:  Unable to submit IRP\n"));
						ResetError = TRUE;
					}

					// tear-down WAN connection after exceeding our own-defined max timeout (or driver never unloads)
#ifdef WAN_MINIPORT
					if (ResetCount == 20)	// takes about 3 seconds
						UsbPhyDslDisconnected(UsbPhy);
#endif

					if (ResetError)
					{
						// Wait 50 ms
#ifdef _NTDDK_
					longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);
					KeWaitForSingleObject(
						&resetEvent,
						Executive,   
						KernelMode,
						FALSE ,
						&longIntTimeout );
#elif defined(linux)
					set_current_state( TASK_INTERRUPTIBLE );
					if(!in_interrupt())
						schedule_timeout( (HZ * 5)/1000 );
#endif // linux
					}
				}
			}

#ifdef _NTDDK_
			if (USB_PHY_STATE_SHUTDOWN != GetUsbState (UsbPhy))
#elif defined(linux)
			if (USB_PHY_STATE_RESETTING_PORT == GetUsbState (UsbPhy))
#endif // linux
			{
				// now that the iso pipe should be functional again, we can start processing URBs
				SetUsbState (UsbPhy, USB_PHY_STATE_OPERATIONAL);

				// Start data transfer and interrupt polling again
				if (UsbPhy->IsoReceivePipeHandle)
				{
#ifdef _NTDDK_
					if (!NT_SUCCESS( UsbPhyIsoSubmitUrbs(UsbPhy) ))
#elif defined(linux)
					UsbPhyStopIsoReceive (UsbPhy);
					if (!NT_SUCCESS( UsbPhyStartIsoReceive(UsbPhy) ))
#endif // linux
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
							("UsbPhyIsoSubmitUrbs:  Unable to submit IRP\n"));
					}
				}

				if (UsbPhy->BulkReceivePipeHandle)
				{
#ifdef _NTDDK_
					if ( !NT_SUCCESS( UsbPhyBulkReceiveSubmitUrbs(UsbPhy) )) 
#elif defined(linux)
					UsbPhyStopBulkReceive( UsbPhy);
					if ( !NT_SUCCESS( UsbPhyStartBulkReceive(UsbPhy) )) 
#endif // linux
					{
						DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
							("UsbPhyBulkReceiveSubmitUrbs:  Unable to submit IRP\n"));
					}
				}

#ifdef _NTDDK_
				if ( !NT_SUCCESS( UsbPhyInterruptSubmitUrbs(UsbPhy) )) 
#elif defined(linux)
				if( UsbPhy->InterruptPipeHandle )
					UsbPhyStopInterruptReceive(UsbPhy);
				if ( !NT_SUCCESS( UsbPhyStartInterruptReceive(UsbPhy) )) 
#endif // linux
				{
					DBGPRINT(DBG_PRINT_MODULE_USB_ALL,DBG_PRINT_LEVEL_ERR,
						("UsbPhyInterruptSubmitUrbs:  Unable to submit URB\n"));
				}

                // allow transmission of data
                (UsbPhy->UserLinkStatusChange)( UsbPhy->UserHandle, ATM_PHY_LINK_STATE_RESETDONE, 0, 0, 0, 0);

				DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("Submitted URBs\n"));
			}

		 break;

	  default:
		 DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,("Unhandled state\n"));
		 break;
	} // switch ( GetUsbState (UsbPhy) )

	return FALSE;
}


void UsbPhyCplReq(PUSB_PHY UsbPhy, UCHAR request)
{
KIRQL    Irql;

   switch(request)
   {
       case GTI_DIGITAL_LB_REQ:
       case GTI_ANALOG_LB_REQ:
	   case GTI_SPECTRUM_MEDLEY_REQ:
       case GTI_SPECTRUM_REVERB_REQ:
       case GTI_SPECTRUM_PILOT_REQ:
       case GTI_SPECTRUM_R_PBIN_REQ:

           if (GetDspState(UsbPhy) == DSP_PHY_IDLE)
           {
               UsbPhy->dspAction = request;
  	           SetDspState(UsbPhy, DSP_PHY_EMPTY, DSP_INTERRUPT);
           }
           break;

       case GTI_ABORT_REQ:
           /*
           ** The dspState is changed to disconnect, but actual
           ** disconnect will not occur till later.
           */
           SetDspState(UsbPhy, DSP_PHY_DISCONNECT, DSP_INTERRUPT);
           break;

       case DSP_RESTART_REQ:
           /*
           ** The dspState is changed to disconnect, but actual
           ** disconnect will not occur till later.
           */
           SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
           break;
   }
}


BOOLEAN IsDspIdle (PUSB_PHY UsbPhy)
{
    if (GetDspState(UsbPhy) != DSP_PHY_CONNECTED)
    {
        SetDspState(UsbPhy, DSP_PHY_RESTART, DSP_INTERRUPT);
        return TRUE;
    }
    else
        return FALSE;
}

BOOLEAN DspNotConnected (PUSB_PHY UsbPhy)
{
    return (GetDspState(UsbPhy) != DSP_PHY_CONNECTED);
}


DSP_PHY_STATE  GetDspState (PUSB_PHY UsbPhy)
{
	KIRQL    Irql;
	DSP_PHY_STATE CurrentDspState;

	KeAcquireSpinLock( &(UsbPhy->DspStateSpinLock), &Irql);
	CurrentDspState = UsbPhy->dspState;
	KeReleaseSpinLock( &(UsbPhy->DspStateSpinLock),Irql);
	
	return CurrentDspState;
}

void SetDspState (PUSB_PHY UsbPhy, DSP_PHY_STATE NewDspState, DSP_CONN_ACTION Action)
{
	KIRQL					Irql;

	KeAcquireSpinLock( &(UsbPhy->DspStateSpinLock), &Irql);
	//DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
	//	("SETDSPSTATE Present State : %d, Newstate : %d, Act = %d\n",
	//	UsbPhy->dspState, NewDspState, Action));
	if (NewDspState != DSP_PHY_EMPTY)
	{
			UsbPhy->dspState = NewDspState;
	}
	KeReleaseSpinLock( &(UsbPhy->DspStateSpinLock),Irql);

	if ( (Action != DSP_NO_ACTION) && (IsDspThreadRunning (UsbPhy)) )
	{
		AddDspCommand ( UsbPhy, Action);
	}
}

USB_PHY_STATE  GetUsbState (PUSB_PHY UsbPhy)
{
	KIRQL    Irql;
	USB_PHY_STATE CurrentDspState;

	KeAcquireSpinLock( &(UsbPhy->UsbStateSpinLock), &Irql);
	CurrentDspState = UsbPhy->State;
	KeReleaseSpinLock( &(UsbPhy->UsbStateSpinLock),Irql);
	
	return CurrentDspState;
}

void SetUsbState (PUSB_PHY UsbPhy, USB_PHY_STATE NewUsbState)
{
	KIRQL                  Irql;

	KeAcquireSpinLock( &(UsbPhy->UsbStateSpinLock), &Irql);

	DBGPRINT( DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("SetUsbState Present State : %d, Newstate : %d\n",
		UsbPhy->State, NewUsbState));


	if (USB_PHY_STATE_HALTED == NewUsbState)	// always allow halted stated
	{
		UsbPhy->State = NewUsbState;
	}
	else
	{
		switch ( UsbPhy->State )
		{
			case USB_PHY_STATE_HALTED:			// do not allow these states to be overwritten
			case USB_PHY_STATE_SHUTDOWN:
			break;

			case USB_PHY_STATE_RESUME:			// do not allow a second shutdown on failed resume

				if (USB_PHY_STATE_SHUTDOWN == NewUsbState)
					UsbPhy->State = USB_PHY_STATE_HALTED;
				else if (USB_PHY_STATE_OPERATIONAL == NewUsbState)	// OPERATIONAL follows RESUME
					UsbPhy->State = NewUsbState;
			break;

			case USB_PHY_STATE_SUSPEND:			// do not allow a second shutdown on failed suspend	

				if (USB_PHY_STATE_SHUTDOWN == NewUsbState)
					UsbPhy->State = USB_PHY_STATE_HALTED;
				else if (USB_PHY_STATE_RESUME == NewUsbState)		// RESUME follows SUSPEND
					UsbPhy->State = NewUsbState;
			break;

			default:							// update usb state
				UsbPhy->State = NewUsbState;
			break;
		}
	}

	KeReleaseSpinLock( &(UsbPhy->UsbStateSpinLock),Irql);

	AddDspCommand ( UsbPhy, DSP_NO_ACTION);	// this will spawn thread
}


////////////////////////////////////////////////////////////////////////////////
//
// NAME: UsbPhyResetParentPort
//
// ABSTRACT:
//		Resets the USB port after it has been disabled.  Adapted from the
//    USB CAM code in the DDK.
//
// RETURNS:
//	   none
//
// ASSUMPTIONS:
//		Should only be called after all the URBs have been returned from 
//    the USBD.
//
////////////////////////////////////////////////////////////////////////////////
#ifdef _NTDDK_
NTSTATUS
UsbPhyResetParentPort(void *UsbPhyParam)
{
   NTSTATUS ntStatus, status = STATUS_SUCCESS;
   PIRP irp;
   KEVENT event;
   IO_STATUS_BLOCK ioStatus;
   PIO_STACK_LOCATION nextStack;
   PUSB_PHY UsbPhy = UsbPhyParam;


   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("enter IsoUsb_ResetParentPort\n"));

   // issue a synchronous request
   KeInitializeEvent(&event, NotificationEvent, FALSE);

   irp = IoBuildDeviceIoControlRequest(
               IOCTL_INTERNAL_USB_RESET_PORT,
               UsbPhy->TopOfStackDeviceObject,
               NULL,
               0,
               NULL,
               0,
               TRUE, // internal ( use IRP_MJ_INTERNAL_DEVICE_CONTROL )
               &event,
               &ioStatus);

   // Call the class driver to perform the operation.  If the returned status
   // is PENDING, wait for the request to complete.

   nextStack = IoGetNextIrpStackLocation(irp);
   ASSERT(nextStack != NULL);

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("IsoUsb_ResetParentPort() calling USBD enable port api\n"));

   ntStatus = IoCallDriver(UsbPhy->TopOfStackDeviceObject, irp);

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("IsoUsb_ResetParentPort() return from IoCallDriver USBD %x\n", ntStatus));

   if (ntStatus == STATUS_PENDING) 
   {
      DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
         ("IsoUsb_ResetParentPort() Wait for single object\n"));

      status = KeWaitForSingleObject(
                  &event,
                  Suspended,
                  KernelMode,
                  FALSE,
                  NULL);

      DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
         ("IsoUsb_ResetParentPort() Wait for single object, returned %x\n", status));

   } 
   else 
   {
      ioStatus.Status = ntStatus;
   }

   //
   // USBD maps the error code for us
   //
   ntStatus = ioStatus.Status;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("Exit IsoUsb_ResetPort (%x)\n", ntStatus));

   return (ntStatus);
}


#ifdef POWERMGMT
void SetSuspend (PUSB_PHY UsbPhy)
{
	SetUsbState (UsbPhy, USB_PHY_STATE_SUSPEND);
}
#endif /* POWERMGMT */
#elif defined(linux)
NTSTATUS
UsbPhyResetParentPort(void *UsbPhyParam)
{
	PUSB_PHY UsbPhy = UsbPhyParam;

	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("enter IsoUsb_ResetParentPort\n"));

	if( usb_reset_device( UsbPhy->AdapterHandle->usbDev ) )
		return( STATUS_UNSUCCESSFUL );
	
	return (STATUS_SUCCESS);
}
#endif // linux

#ifdef EEPROM
void readMAC_Address(PUSB_PHY UsbPhy, GTI_XCVR_INFO *pXcvrInfo, PUCHAR PhyUniqueId)
{
ULONG i;
GTI_BYTE Data[EEPROM_SIZE_13];

    DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
      ("Read MAC Address from EEPROM\n"));


	SET_DSP_MSG_TYPE(pXcvrInfo, GTI_READ_EEPROM);
	READ_DSP_BLOCK(pXcvrInfo,GTI_DSP_A,GTI_DATAL_DSP,EEPROM_SIZE_13 ,Data);

    if(Data[0] == 0xFF)
    {
        Data[7] = 0x01;
        Data[8] = 0x00;
        Data[9] = 0x00;
        Data[10] = 0x00;
        Data[11] = 0xFF;
        Data[12] = 0x00;
    }
    
	for (i=0; i<UNIQUE_ID_SIZE; i++)
		*(PhyUniqueId+i) = Data[12-i];

#if defined(linux)
	for (i=0; i<GSI_MAC_ADDRESS_LENGTH; i++)
	{
		UsbPhy->AdapterHandle->macAddr[i] = Data[12 - i];
	} 
#endif // linux
   /* Save the read copy */
   COPY_MEMORY(&(UsbPhy->eeprom[0]), Data, EEPROM_SIZE_13);
}

/* **************************************************************
** Write to EEPROM. Obtain offset/address and length of data to
** set on the EEPROM
*****************************************************************
*/
void UsbPhyPokeEEPROM( PUSB_PHY UsbPhy)
{
GV_MSG_SET_EEPROM *eepromPtr;

   eepromPtr = (GV_MSG_SET_EEPROM *)&(UsbPhy->data[0]);
   
   GTISendUSBCmdEx(UsbPhy,
                   GTI_WRITE_EEPROM,
                   0,
                   (USHORT)eepromPtr->offset,
                   ( ((UCHAR *)eepromPtr) + sizeof(GV_MSG_SET_EEPROM)),
                   eepromPtr->length,
                   GTI_OUT);

   SLEEP_PROCCESS(50);
   UsbPhy->IoCtlStatusBusy = FALSE;
}

/* WRITE and READ the EEPROM */
void UsbPhyEEPROMAccess(PUSB_PHY UsbPhy, GTI_XCVR_INFO *pXcvrInfo)
{
ULONG i;
#if EEPROMv2
i = 0;
#else
i = 1;
#endif//EEPROMv2

   /* Write to the EEPROM */
	SET_DSP_MSG_TYPE(pXcvrInfo, GTI_WRITE_EEPROM);

   for( ; i < EEPROM_SIZE_13; i++) 
                       
   {
	 //   *(txfrblock.currentPtr)++ = UsbPhy->eeprom[i];
	WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_A, GTI_EXT_DATA_DSP, UsbPhy->eeprom[i]);
   }
	UPDATE_DSP(pXcvrInfo);

readMAC_Address(UsbPhy, pXcvrInfo , UsbPhy->AtmEndSystemIdentifier);
   
}
#endif // EEPROM
#endif /* GSI_USB */


BOOLEAN IsDspThreadRunning (PUSB_PHY UsbPhy)
{
	USB_PHY_STATE CurrentUsbState = GetUsbState (UsbPhy);

	return ( (USB_PHY_STATE_OPERATIONAL == CurrentUsbState) || (USB_PHY_STATE_RESETTING_PORT == CurrentUsbState) );
}


BOOLEAN IsTxThreadRunning (PUSB_PHY UsbPhy)
{
	return ( (USB_PHY_STATE_OPERATIONAL == UsbPhy->State) && (DSP_PHY_CONNECTED == UsbPhy->dspState) );
}


BOOLEAN IsNotOperational (PUSB_PHY UsbPhy)
{
	USB_PHY_STATE CurrentUsbState = GetUsbState (UsbPhy);

	return (!( ((USB_PHY_STATE_OPERATIONAL == CurrentUsbState) || (USB_PHY_STATE_SUSPEND == CurrentUsbState))
																|| (USB_PHY_STATE_SHUTDOWN == CurrentUsbState) ));
}


void UsbPhyGetMacAddress( PUSB_PHY UsbPhy , PUCHAR Buffer )
{
   COPY_MEMORY( &Buffer[0] , &UsbPhy->eeprom[MAC_EEPROM_END_OFFSET - ETH_LENGTH_OF_ADDRESS] , ETH_LENGTH_OF_ADDRESS);
}


void UsbPhyBindDsl(PUSB_PHY UsbPhy, ATM_PHY_USER_HANDLE DslHandle)
{
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("UsbPhyBindEoc\n"));

    ASSERT(UsbPhy);
    ASSERT(DslHandle);
    UsbPhy->DslHandle = DslHandle;
}

// RECONFIG bandwidth according to data rate
void UsbPhyConfigureIsoBW (PUSB_PHY UsbPhy)
{
	KIRQL		Irql;
	USHORT		actualConfig[22];
	USHORT		ActualDsRate;
	UCHAR		AltSetting_DataRate;
	USHORT		MaxDownRateTable[] = {	USB_ISO_RATE_ALT0,
										USB_ISO_RATE_ALT1,
										USB_ISO_RATE_ALT2,
										USB_ISO_RATE_ALT3,
										USB_ISO_RATE_ALT4,
										USB_ISO_RATE_ALT5	};


   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("UsbPhyConfigureIsoBW\n"));

	// Get the downstream rate by calling Actual Configuration
	KeAcquireSpinLock( &(UsbPhy->DspLock), &Irql);
	Gti_SStatusXCVR_USB( UsbPhy->GTISysBlock, GTI_ACTUAL_CONFIGURATION,
		&(actualConfig[0]), GTI_ACTUAL_CONFIGURATION_LEN, GTI_DSP_A);
	KeReleaseSpinLock( &(UsbPhy->DspLock),Irql);

	ActualDsRate = (USHORT)(actualConfig[GTI_AS0_BYTES_ACTUAL]);

	if ( ActualDsRate <= USB_ISO_RATE_ALT0 )	// bulk rate
	{
   		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("Reconfig Bulk Rate %d\n", ActualDsRate));
		AltSetting_DataRate = 0;
	}
	else										// iso rate
	{
   		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
			("Reconfig Iso Rate %d\n",ActualDsRate));
		// Limit the ISO bandwidth to the minimum amount required by the data rate
		for (AltSetting_DataRate = (MAX_DOWN_RATE_TABLE_SIZE-1); AltSetting_DataRate > 1; AltSetting_DataRate--)
		{
			if(ActualDsRate <= MaxDownRateTable[AltSetting_DataRate])
				break;
		}
	}
		
	UsbPhy->ActualDsDataRate = AltSetting_DataRate;
	SetUsbState(UsbPhy, USB_PHY_STATE_RECONFIG);
}

#if defined(linux)
PUSB_PHY_TRANSFER_RESOURCE 
RemoveResourceFromList(SINGLE_LIST_NODE *ListHead, 
	PUSB_PHY_TRANSFER_RESOURCE SendResource,
	KSPIN_LOCK	*pListLock )
{
	SINGLE_LIST_NODE	TempList;
	KIRQL				Irql;
	PUSB_PHY_TRANSFER_RESOURCE TempResource = NULL;
	PUSB_PHY_TRANSFER_RESOURCE FreeResource = NULL;

	if( pListLock )
		KeAcquireSpinLock( pListLock, &Irql);
	
	TempList.Next = NULL;
	while(TRUE)
	{
		TempResource = (PUSB_PHY_TRANSFER_RESOURCE)POP_ENTRY_LIST(ListHead);
		if( (!TempResource) ||( TempResource == SendResource ) )
			break;
		PUSH_ENTRY_LIST(&TempList, (PSINGLE_LIST_NODE)TempResource);
	}
	
	FreeResource = TempResource;
	while(TRUE)
	{
		TempResource = (PUSB_PHY_TRANSFER_RESOURCE)POP_ENTRY_LIST(&TempList);
		if( !TempResource )
			break;

		PUSH_ENTRY_LIST(ListHead, (PSINGLE_LIST_NODE)TempResource);
	}

	if( pListLock )
		KeReleaseSpinLock( pListLock, Irql);

	return(FreeResource);
}
#endif // linux

#ifdef _NTDDK_
ULONG  UsbPhyGetStatusOfParentPort( PUSB_PHY UsbPhy , ULONG *ulStatus)
{
   NTSTATUS ntStatus, status = STATUS_SUCCESS;
   PIRP irp;
   KEVENT event;
   IO_STATUS_BLOCK ioStatus;
   PIO_STACK_LOCATION nextStack;
	LONG            longTimeout = (-1) * (50 * 10000);		// 50 ms timer
	LARGE_INTEGER   longIntTimeout;

    longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);


   // issue a synchronous request
   KeInitializeEvent(&event, NotificationEvent, FALSE);

   irp = IoBuildDeviceIoControlRequest(
               IOCTL_INTERNAL_USB_GET_PORT_STATUS,
               UsbPhy->TopOfStackDeviceObject,
               NULL,
               0,
               NULL,
               0,
               TRUE, // internal ( use IRP_MJ_INTERNAL_DEVICE_CONTROL )
               &event,
               &ioStatus);




   // Call the class driver to perform the operation.  If the returned status
   // is PENDING, wait for the request to complete.

   nextStack = IoGetNextIrpStackLocation(irp);
   
   nextStack->Parameters.Others.Argument1 = ulStatus;

   ASSERT(nextStack != NULL);

 
   ntStatus = IoCallDriver(UsbPhy->TopOfStackDeviceObject, irp);


   if (ntStatus == STATUS_PENDING) 
   {

      status = KeWaitForSingleObject(
                  &event,
                  Suspended,
                  KernelMode,
                  FALSE,
                  &longIntTimeout);


   } 
   else 
   {
      ioStatus.Status = ntStatus;
   }

   //
   // USBD maps the error code for us
   //
   ntStatus = ioStatus.Status;


   return ntStatus;

}
#endif // _NTDDK_

#ifdef QUASAR
NTSTATUS
UsbPhyDownLoadCode(PUSB_PHY UsbPhy , unsigned char *pFirmware, unsigned long ulSize)
{
unsigned short ulDone = 0;
unsigned long ulSendNow; 
BOOLEAN bLastPacket = FALSE;
unsigned char localbuffer[GTI_USB_BLOCK_SIZE];
int k;
	
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
		("UsbPhyDownLoadCode ulSize = %d\n", ulSize));

	while(ulSize && !bLastPacket)
	{

		bLastPacket = (ulSize > GTI_USB_BLOCK_SIZE ) ? FALSE:TRUE; 
		
		ulSendNow = ( bLastPacket ) ? ulSize : GTI_USB_BLOCK_SIZE;

		COPY_MEMORY(&localbuffer[0], pFirmware + ulDone, ulSendNow);
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
				("Offset %d , size %d val %d %d\n", ulDone, ulSendNow, 
				 *((ULONG *)(pFirmware + ulDone)),
				 *((ULONG *)(pFirmware + ulDone+4))));
		
		GTISendUSBCmdEx(UsbPhy , GTI_DOWNLOAD_CODE ,
                     ulDone,  (UCHAR)!bLastPacket,
                      &localbuffer[0],//pFirmware + ulDone,  
					  ulSendNow , GTI_OUT);
		
		
		ulDone += GTI_USB_BLOCK_SIZE;
		ulSize        -= GTI_USB_BLOCK_SIZE;
		
		
		SLEEP_PROCCESS(500);
		//for(k=0;k<19990;k++)
		//	;

	}

	for(k=0;k<19990;k++)
		;
	//SLEEP_PROCCESS(500);
return TRUE;
}//UsbPhyDownLoad5081Code
#endif //QUASAR
