/*+M*************************************************************************
 * GlobeSpan ADSL Sample Configuration Application
 *
 * Copyright (c) 2001 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 *-M*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <stdint.h>
#include <net/if.h>
#include <errno.h>
#include <fcntl.h>
#include "ioctl.h"

#define  GTI_STANDARD_ITEM                     0x06     /* CO & CP */
#define  GTI_MAX_DOWN_RATE_ITEM                0x25     /* CP Only */
#define  GTI_TRELLIS_ITEM                      0x04     /* CO & CP */
#define  GTI_CODING_GAIN_ITEM                  0x11     /* CO & CP */
#define  GTI_TX_POWER_ATTENUATION_ITEM         0x0e     /* CO & CP */
#define  GTI_MAX_BITS_PER_BIN_ITEM             0x07     /* CO & CP */
#define  GTI_EXPANDED_EXCHANGE_ITEM            0x19     /* CO & CP */

#define  GTI_TX_START_BIN_ITEM                 0x08     /* CO & CP */
#define  GTI_TX_END_BIN_ITEM                   0x09     /* CO & CP */
#define  GTI_LINE_DR_STBY_ITEM                 0x0a     /* CO & CP */
#define  GTI_RX_START_BIN_ITEM                 0x0b     /* CO & CP */
#define  GTI_RX_END_BIN_ITEM                   0x0c     /* CO & CP */
#define  GTI_RX_BIN_ADJUST_ITEM                0x0d     /* CO & CP */

#define  GTI_FRAMING_STRUCTURE_ITEM            0x13     /* CO & CP */
#define  GTI_UTOPIA_L2_TX_ADDR_ITEM            0x14     /* CO&CP 0-0x1c even number */
#define  GTI_UTOPIA_L2_RX_ADDR_ITEM            0x15     /* CO&CP 0-0x1c even number */
#define  GTI_CLOCK_TYPE_ITEM                   0x16     /* CO & CP */
#define NUM_STANDARD_ACTUAL						11

static const char version[] = "gsi_cfg v1.3 09/17/2001";

extern void usage(const char *progName);

typedef enum {
    FALSE = 0,
    TRUE  = 1
} bool_t;

long    gtiStandard = 4;
long    gtiMaxdownrate = 8192;
long	VendorID	= 0;
long	ProductID 	= 0;
long	zeroByte	= 0;
uint64_t	Macaddress = 0x001112350000;

char *DspStatusStr[] = {"IDLE", 
                        "TRAINING",
                        "CONNECTED", 
                        "SPECTRUM MODE", 
                        "EMPTY", 
                        "WAIT",
						"LOOPBACK",
						"RESTART",
						"NO SIGNAL",
						"DISCONNECT",
						"WHIP MODE"};

char *UsbStatusStr[] = {"OPERATIONAL", 
                        "SHUTDOWN",
                        "RESETTING PORT", 
                        "CONFIG", 
                        "RECONFIG", 
                        "SUSPEND",
						"RESUME",
						"HALTED"};
uint16_t standard[] = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x09,0x80,0xf0};

char *DspModulationStr[] = {"T1.413","G.lite","G.DMT","ALCTL14","Multimode",
                             "ADI","ALCTL","UAWG","T1.413 Auto","Still ALCTL14",
                             "CAP"};


int getID(char *string11)
{
	
  if (!strcmp(string11,"GTI_VENDOR_ID"))
		  return 0x017;
  if (!strcmp(string11,"GTI_CLOCK_TYPE"))
		  return 0x16;
  if (!strcmp(string11,"GTI_VENDOR_REV_SHORT"))
		  return 0x18;
  if (!strcmp(string11,"GTI_EXPANDED_EXCHANGE"))
		  return 0x19;
  if (!strcmp(string11,"GTI_ESCAPE_FAST_RETRAIN"))
		  return 0x1a;
  if (!strcmp(string11,"GTI_FAST_RETRAIN_ENABLE"))
		  return 0x1b;
  if (!strcmp(string11,"GTI_BIT_SWAP"))
		  return 0x1c;
  if (!strcmp(string11,"GTI_US_BIT_SWAP"))
		  return 0x1d;
  if (!strcmp(string11,"GTI_ERASE_PROFILES"))
		  return 0x1e;
  if (!strcmp(string11,"GTI_PILOT_TONE_REQUEST"))
		  return 0x1f;
  if (!strcmp(string11,"GTI_MAX_DOWN_RATE"))
		  return 0x25;
  if (!strcmp(string11,"GTI_DUAL_LATENCY"))
		  return 0x26;
  if (!strcmp(string11,"GTI_MODE"))
		  return 0x01;
  if (!strcmp(string11,"GTI_INITIATE"))
		  return 0x02;
  if (!strcmp(string11,"GTI_DATA_INTERFACE_TYPE"))
		  return 0x03;
  if (!strcmp(string11,"GTI_EC_FDM"))
		  return 0x05;
  if (!strcmp(string11,"GTI_TRELLIS"))
		  return 0x04;
  if (!strcmp(string11,"GTI_STANDARD"))
		  return 0x06;
  if (!strcmp(string11,"GTI_MAX_BITS_PER_BIN"))
		  return 0x07;
  if (!strcmp(string11,"GTI_TX_START_BIN"))
		  return 0x08;
  if (!strcmp(string11,"GTI_TX_END_BIN"))
		  return 0x09;
  if (!strcmp(string11,"GTI_RX_START_BIN"))
		  return 0x0b;
  if (!strcmp(string11,"GTI_RX_END_BIN"))
		  return 0x0c;
  if (!strcmp(string11,"GTI_RX_BIN_ADJUST"))
		  return 0x0d;
  if (!strcmp(string11,"GTI_TX_POWER_ATTENUATION"))
		  return 0x0e;
  if (!strcmp(string11,"GTI_LINE_DR_STBY"))
		  return 0x0a;
  if (!strcmp(string11,"GTI_NTR"))
		  return 0x0f;
  if (!strcmp(string11,"GTI_PMODE"))
		  return 0x10;
  if (!strcmp(string11,"GTI_CODING_GAIN"))
		  return 0x11;
  if (!strcmp(string11,"GTI_TONE_ORDERING"))
		  return 0x12;
  if (!strcmp(string11,"GTI_FRAMING_STRUCTURE"))
		  return 0x13;
  if (!strcmp(string11,"GTI_UTOPIA_L2_TX_ADDR"))
		  return 0x14;
  if (!strcmp(string11,"GTI_UTOPIA_L2_RX_ADDR"))
		  return 0x15;

if (!strcmp(string11,"RFC_1483_MODE"))
		  return 0x30;
if (!strcmp(string11,"RFC_1483_VCI"))
		  return 0x31;
if (!strcmp(string11,"RFC_1483_VPI"))
		  return 0x32;
if (!strcmp(string11,"RFC_2364_MODE"))
		  return 0x33;
if (!strcmp(string11,"RFC_2364_VCI"))
		  return 0x34;
if (!strcmp(string11,"RFC_2364_VPI"))
		  return 0x35;
if (!strcmp(string11,"VERBOSE"))
		  return 0x36;
if (!strcmp(string11,"RX_BUFFER_COUNT"))
		  return 0x37;
if (!strcmp(string11,"TX_BUFFER_COUNT"))
		  return 0x38;
if (!strcmp(string11,"RETRAIN_TIMEOUT"))
		  return 0x39;
if (!strcmp(string11,"MAX_DRIVER_INSTANCE"))
		  return 0x3a;

  return -1;

}

/*+F*************************************************************************
 * Function:
 *   main
 *
 * Description:
 *   Program entry routine.
 *-F*************************************************************************/
int
main(
    int     argc,
    char   *argv[]
    )
{
    char   *ifname;
    int     x;
    int     fd;
    int     status;
	int		RunTestCount = 0;
    bool_t  wanif;
    bool_t  query_version = FALSE;
    bool_t  query_config = FALSE;
    bool_t  query_status = FALSE;
    bool_t  query_debug = FALSE;
    bool_t  set_debug = FALSE;
    bool_t  set_config = FALSE;
    bool_t  query_stats = FALSE;
    bool_t  clear_stats = FALSE;
    bool_t  run_test = FALSE;
    bool_t  start_retrain = FALSE;
    bool_t  start_disconnect = FALSE;
	bool_t  query_IDs = FALSE;
	bool_t  set_IDs = FALSE;
	
	gsi_debug_t	DbgOpts;
	DIAG_STRUCT	OamCell;

	FILE* fp;
	char line[100];
 	char seps[]   = " =";
	char *token, *stop;
	int ChangedValue;
	int ParamType = 0;
	int	TestCase = 0;
	int XcvrParam[200];
	ATM_UNI_STATS	AtmUniStats;


    //PGSIDSL_PANDA_EEPROM pEeprom;

    status = 0;

    ifname = NULL;
    for (x = 1; x < argc; x++)
    {
        switch ((argv[x])[0])
        {
        case '-':
            switch ((argv[x])[1])
            {
            case 'v':
                query_version = TRUE;
                break;

            case 'e':
                query_IDs = TRUE;
                break;
            case 'E':
                set_IDs = TRUE;
                break;

            case 'c':
                query_config = TRUE;
                break;

            case 'C':
                set_config = TRUE;
                break;

            case 's':
                query_status = TRUE;
                break;

            case 'b':
                query_debug = TRUE;
            	memset(&DbgOpts, 0, sizeof(DbgOpts));
                break;

            case 'B':
                set_debug = TRUE;
                break;

            case 'n':
                query_stats = TRUE;
                break;

            case 'N':
                clear_stats = TRUE;
                break;

            case 't':
                run_test = TRUE;
				RunTestCount = 1;
				memset(&OamCell, 0, sizeof(OamCell));
                break;

            case 'r':
                start_retrain = TRUE;
                break;

            case 'd':
                start_disconnect = TRUE;
                break;

            default:
                fprintf(stderr, "ERROR: Unknown option \"%s\"\n", argv[x]);
                usage(argv[0]);
                break;
            }
            break;

        case '+':
            switch ((argv[x])[1])
            {
            case 's':
                if ((argv[x])[2] != '\0')
                {
                    gtiStandard = strtol(&((argv[x])[2]), NULL, 0);
                }
                break;

            case 'd':
                if ((argv[x])[2] != '\0')
                {
                    gtiMaxdownrate = strtol(&((argv[x])[2]), NULL, 0);
                }
                break;
			case 'm':
				{
					int count = 0;
					int	val = 0;
					Macaddress = 0;
                	while((x < argc - 1) && ((argv[++x][0]) != '\0') && (count < 6))
                	{
						val = strtol(((argv[x])), NULL, 16);
                		Macaddress |= val << (count * 8);
						count++;
					}
				}
                break;
				
			case 'v':
                if ((argv[x])[2] != '\0')
                {
                    VendorID = strtol(&((argv[x])[2]), NULL, 0);
                }
                break;
				
			case 'z':
                if ((argv[x])[2] != '\0')
                {
                    zeroByte = strtol(&((argv[x])[2]), NULL, 0);
                }
                break;
				
			case 'p':
                if ((argv[x])[2] != '\0')
                {
                    ProductID = strtol(&((argv[x])[2]), NULL, 0);
                }
                break;

			case 'l':
                if ((argv[x])[2] != '\0')
                {
                    DbgOpts.level = strtol(&((argv[x])[2]), NULL, 0);
					DbgOpts.SetLevel = 1;
                }
				else
					DbgOpts.SetLevel = 0;
					
                break;
				
			case 'o':
                if ((argv[x])[2] != '\0')
                {
                    DbgOpts.module = strtol(&((argv[x])[2]), NULL, 0);
					DbgOpts.SetModule = 1;
                }
				else
					DbgOpts.SetModule = 0;

                break;

			case 'i':
                if ((argv[x])[2] != '\0')
                {
                    TestCase = strtol(&((argv[x])[2]), NULL, 0);
                }
				else
				{
                	fprintf(stderr, "ERROR: Test case not specified\n");
                	usage(argv[0]);
					run_test = FALSE;
				}

                break;

			case 'c':
                if ((argv[x])[2] != '\0')
                {
                	OamCell.vci = strtol(&((argv[x])[2]), NULL, 0);
                }
				else
				{
                	fprintf(stderr, "ERROR: Wrong VCI\n");
                	usage(argv[0]);
					run_test = FALSE;
				}
                break;

			case 'a':
                if ((argv[x])[2] != '\0')
                {
                	OamCell.vpi = strtol(&((argv[x])[2]), NULL, 0);
                }
				else
				{
                	fprintf(stderr, "ERROR: Wrong VPI\n");
                	usage(argv[0]);
					run_test = FALSE;
				}
                break;

			case 't':
                if ((argv[x])[2] != '\0')
                {
                	OamCell.vpi = strtol(&((argv[x])[2]), NULL, 0);
                }
				else
				{
                	fprintf(stderr, "ERROR: Wrong VPI\n");
                	usage(argv[0]);
					run_test = FALSE;
				}
                break;

			case 'n':
                if ((argv[x])[2] != '\0')
                {
                	RunTestCount = strtol(&((argv[x])[2]), NULL, 0);
                }
				else
				{
					RunTestCount = 1;
				}
                break;

            default:
                fprintf(stderr, "ERROR: Unknown option \"%s\"\n", argv[x]);
                usage(argv[0]);
                break;
            }
            break;

        default:
            ifname = argv[x];
            break;
        }
    }

    if (ifname == NULL)
    {
        fprintf(stderr, "Interface name allowed is only eth1\n");
		exit(0);
        ifname = "/dev/ttyG0";
        fprintf(stderr, "Defaulting interface name to: %s\n", ifname);
    }

    // Use the name to determine if we are talking to the WAN interface
    // (/dev/ttyG<n> or /dev/cug<n>) or to the LAN interface (eth<n>).
    if (strstr(ifname, "/dev/") != NULL)
    {
        fprintf(stderr, "Interface name allowed is only eth1\n");
		exit(0);
        wanif = TRUE;
        if ((fd = open(ifname, O_RDWR)) == -1)
        {
            perror("WAN open");
        }
    }
    else
    {
        wanif = FALSE;
        if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
        {
            perror("LAN socket");
        }
    }

    if (fd != -1)
    {
        struct gsi_req     *pReq;
        void               *pArg;
        gsi_version_t       version;
       	//PGSIDSL_PANDA_EEPROM eeprom;
       	GSIDSL_EEPROM		 eeprom;
        gsi_config_t        config;
        gsi_status_t        status;
        char               *rname;


        if (wanif)
        {
            static struct gsi_req       ioctlReq;

            pArg = &(ioctlReq);

            memset(&ioctlReq, 0, sizeof(ioctlReq));
            pReq = &(ioctlReq);
        }
        else
        {
            static struct gsi_lan_req   ioctlReq;

            pArg = &(ioctlReq);

            memset(&ioctlReq, 0, sizeof(ioctlReq));
            strcpy(ioctlReq.ifr_name, ifname);
            pReq = &(ioctlReq.request);
        }

        if (query_version)
        {
            // Query the adapter version information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_VERSION; rname = "GET_VERSION";
            pReq->len = sizeof(version);
            pReq->data = &version;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_VERSION IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "Protocol version: %d.%d\n", 
						version.protocol.major, version.protocol.minor);
                fprintf(stderr, "Driver version:   %s\n", version.driver);
                fprintf(stderr, "Firmware version: %s\n", version.firmware);
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", 
						pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (query_IDs)
        {
            //Query the various IDs like Vendor ID, SubSystem ID and MAC address
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_READ_EEPROM; rname = "READ_EEPROM";
            pReq->len = sizeof(eeprom);
            pReq->data = &eeprom;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan READ_EEPROM IOCTL");
            }

			fprintf(stderr,"coming out of ioctl() call \n");

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
	        	fprintf(stderr, "Zero Byte : %d\n", eeprom.zeroByte);
	        	fprintf(stderr, "Vendor ID: %d %d\n", 
					eeprom.vendorIdHigh, eeprom.vendorIdLow);
	        	fprintf(stderr, "Product ID: %d %d\n", 
					eeprom.productIdHigh, eeprom.productIdLow);
	            fprintf(stderr, 
					"Current MAC Address: %02X-%02X-%02X-%02X-%02X-%02X\n",  
					eeprom.macAddr0,eeprom.macAddr1,eeprom.macAddr2,
					eeprom.macAddr3,eeprom.macAddr4,eeprom.macAddr5);
				exit(0);
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "Result:%d Length:%d Data:%p\n", 
					pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
		}

        
		if (set_IDs)
        {
            // Query the adapter configuration information
            memset(pReq, 0, sizeof(*pReq));
            memset(&eeprom, 0, sizeof(eeprom));
            pReq->command = GSI_IOCTL_READ_EEPROM; rname = "READ_EEPROM";
            pReq->len = sizeof(eeprom);
            pReq->data = &eeprom;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan READ_EEPROM IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
				if( zeroByte )
					eeprom.zeroByte = zeroByte;
				if( VendorID )
				{
					eeprom.vendorIdLow = VendorID & 0xFF;
					eeprom.vendorIdHigh = (VendorID & 0xFF00) >> 8;
					VendorID = 0;
				}

				if( ProductID )
				{
					eeprom.productIdLow = ProductID & 0xFF;
					eeprom.productIdHigh = (ProductID & 0xFF00) >> 8;
					ProductID = 0;
				}
                eeprom.macAddr0 =  (Macaddress & 0xFF);
                eeprom.macAddr0 =  (Macaddress & 0xFF);
                eeprom.macAddr1 =  (Macaddress & 0xFF00)>>8;
                eeprom.macAddr2 =  (Macaddress & 0xFF0000)>>16;
                eeprom.macAddr3 =  (Macaddress & 0xFF000000)>>24;
                eeprom.macAddr4 =  (Macaddress & 0xFF00000000)>>32;
                eeprom.macAddr5 =  (Macaddress & 0xFF0000000000)>>40;

                // Modify the adapter configuration information
                pReq->command = GSI_IOCTL_WRITE_EEPROM; rname = "WRITE_EEPROM";
                pReq->len = sizeof(eeprom);
                pReq->data = &eeprom;
                fprintf(stderr, "================ %s ==================\n", rname);
                if (ioctl(fd, GSI_IOCTL, pArg) < 0)
                {
                    perror("GlobeSpan WRITE_EEPROM IOCTL");
                }
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", 
						pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        
		if (set_config)
        {
            // Query the adapter configuration information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_CONFIG; rname = "GET_CONFIG";
            pReq->len = sizeof(config);
            pReq->data = &config;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_CONFIG IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
				fp = fopen("gp.cfg","r");
			
				while( fgets( line, 60, fp ) != NULL )
				{
					if(strlen(line) > 0) 
					{
						token = strtok( line, seps );
						ParamType  = getID(token);	
						if(ParamType != -1)
						{
							/* Get next token: */
							token = strtok( NULL, seps );
							ChangedValue  = strtol(token,&stop,16);
							XcvrParam[ParamType] = ChangedValue;
						}	
					}
				}

				fclose(fp);

                config.gtiStandard = (uint16_t) XcvrParam[GTI_STANDARD_ITEM];
                config.gtiMaxDownRate = (uint16_t) XcvrParam[GTI_MAX_DOWN_RATE_ITEM];
                config.gtiTrellis = (uint16_t) XcvrParam[GTI_TRELLIS_ITEM];
                config.gtiTxPowerAtten = (uint16_t) XcvrParam[GTI_TX_POWER_ATTENUATION_ITEM];
                config.gtiCodingGain = (uint16_t) XcvrParam[GTI_CODING_GAIN_ITEM];
                config.gtiMaxBitsPerBin = (uint16_t) XcvrParam[GTI_MAX_BITS_PER_BIN_ITEM];
                config.gtiTxStartBin = (uint16_t) XcvrParam[GTI_TX_START_BIN_ITEM];
                config.gtiTxEndBin = (uint16_t) XcvrParam[GTI_TX_END_BIN_ITEM];
                config.gtiRxStartBin = (uint16_t) XcvrParam[GTI_RX_START_BIN_ITEM];
                config.gtiRxEndBin = (uint16_t) XcvrParam[GTI_RX_END_BIN_ITEM];
                config.gtiRxBinAdjust = (uint16_t) XcvrParam[GTI_RX_BIN_ADJUST_ITEM];
                config.gtiFramingStruct = (uint16_t) XcvrParam[GTI_FRAMING_STRUCTURE_ITEM];
                config.gtiExpandedExchange = (uint16_t) XcvrParam[GTI_EXPANDED_EXCHANGE_ITEM];
                config.gtiClockType = (uint16_t) XcvrParam[GTI_CLOCK_TYPE_ITEM];
 
               // Modify the adapter configuration information
                pReq->command = GSI_IOCTL_SET_CONFIG; rname = "SET_CONFIG";
                pReq->len = sizeof(config);
                pReq->data = &config;
                fprintf(stderr, "================ %s ==================\n", rname);
                if (ioctl(fd, GSI_IOCTL, pArg) < 0)
                {
                    perror("GlobeSpan SET_CONFIG IOCTL");
                }
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", 
						pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (query_config)
        {
            // Query the adapter configuration information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_CONFIG; rname = "GET_CONFIG";
            pReq->len = sizeof(config);
            pReq->data = &config;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_CONFIG IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                if (config.interface & GSI_INTERFACE_LAN)
                {
                	fprintf(stderr, "RFC1483 Mode: %d\n", config.rfc1483Mode);
                	fprintf(stderr, "RFC1483 VCI:  %d\n", config.rfc1483Vci);
                	fprintf(stderr, "RFC1483 VPI:  %d\n", config.rfc1483Vpi);
                }

                if (config.interface & GSI_INTERFACE_WAN)
                {
                	fprintf(stderr, "RFC2364 Mode: %d\n", config.rfc2364Mode);
               	 	fprintf(stderr, "RFC2364 VCI:  %d\n", config.rfc2364Vci);
                	fprintf(stderr, "RFC2364 VPI:  %d\n", config.rfc2364Vpi);
                }

                fprintf(stderr, "Rx Buffers:   %d\n", config.rxBufferCount);
                fprintf(stderr, "Tx Buffers:   %d\n", config.txBufferCount);

                fprintf(stderr, "GTI Parameters:\n");
                fprintf(stderr, "  Standard:              0x%04x\n", config.gtiStandard);
                fprintf(stderr, "  Trellis:               0x%04x\n", config.gtiTrellis);
                fprintf(stderr, "  Tx Power Attenuation:  0x%04x\n", config.gtiTxPowerAtten);
                fprintf(stderr, "  Coding Gain:           0x%04x\n", config.gtiCodingGain);
                fprintf(stderr, "  Max Bits Per Bin:      0x%04x\n", config.gtiMaxBitsPerBin);
                fprintf(stderr, "  Tx Start Bin:          0x%04x\n", config.gtiTxStartBin);
                fprintf(stderr, "  Tx End Bin:            0x%04x\n", config.gtiTxEndBin);
                fprintf(stderr, "  Rx Start Bin:          0x%04x\n", config.gtiRxStartBin);
                fprintf(stderr, "  Rx End Bin:            0x%04x\n", config.gtiRxEndBin);
                fprintf(stderr, "  Rx Bin Adjust:         0x%04x\n", config.gtiRxBinAdjust);
                fprintf(stderr, "  Framing Structure:     0x%04x\n", config.gtiFramingStruct);
                fprintf(stderr, "  Expanded Exchange:     0x%04x\n", config.gtiExpandedExchange);
                fprintf(stderr, "  Clock Type:            0x%04x\n", config.gtiClockType);
                fprintf(stderr, "  Max Downstream Rate:   %d kbps\n", config.gtiMaxDownRate);
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (query_status)
        {
            // Query the adapter configuration information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_STATUS; rname = "GET_STATUS";
            pReq->len = sizeof(status);
            pReq->data = &status;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_STATUS IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
				int	i = 0;
                fprintf(stderr, "DSP Status:  %s\n", 
					DspStatusStr[status.u32DspState]);
				for( i=0;i<NUM_STANDARD_ACTUAL;i++)
				{
            		if( status.u32Modulation == standard[i] )
					{
                		fprintf(stderr, "DSP Modulation:  %s\n", 
							DspModulationStr[i]);
						break;
					}
				}
                fprintf(stderr, "USB Status:  %s\n", 
					UsbStatusStr[status.u32UsbState]);
                fprintf(stderr,"UP Stream Rate / Down Stream Rate:  %d / %d\n", 
					status.u32UpRate, status.u32DownRate);

            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (query_debug)
        {
            // Query the Debug options
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_DBG; rname = "GET_DBG";
            pReq->len = sizeof(DbgOpts);
            pReq->data = &DbgOpts;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_DBG IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "Driver Debug Level:   %x\n", DbgOpts.level );
                fprintf(stderr, "Driver Debug Module:   %x\n", DbgOpts.module );
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

		if (set_debug)
        {
			// Modify the adapter configuration information
			pReq->command = GSI_IOCTL_SET_DBG; rname = "SET_DBG";
			pReq->len = sizeof(DbgOpts);
			pReq->data = &DbgOpts;
			fprintf(stderr, "================ %s ==================\n", rname);
			if (ioctl(fd, GSI_IOCTL, pArg) < 0)
			{
				perror("GlobeSpan SET_DBG IOCTL");
			}
            fprintf(stderr, "\n");
        }

        if (query_stats)
        {
            // Query the Atm statistics information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_GET_STATS ; rname = "GET_STATS";
            pReq->len = sizeof(ATM_UNI_STATS);
            pReq->data = &AtmUniStats;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan GET_STATS IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "Number of cells transmitted            %d\n", 
					AtmUniStats.TxCellsOk);
                fprintf(stderr, "Number of cells received               %d\n", 
					AtmUniStats.RxCellsOk);
                fprintf(stderr, "Number of cells dropped                %d\n", 
					AtmUniStats.RxCellsDropped);
                //fprintf(stderr, "Number of cells w/bad header           %d\n", AtmUniStats.RxCellsBadHeader);
                fprintf(stderr, "Number of cells w/invalid Vpi/Vci      %d\n", 
					AtmUniStats.RxInvalidVpiVci);
                fprintf(stderr, "Number of packets sent                 %d\n", 
					AtmUniStats.TxPdusOk);
                fprintf(stderr, "Number of packets received             %d\n", 
					AtmUniStats.RxPdusOk);
                fprintf(stderr, "Number of AAL5 PDU CRC errors          %d\n", 
					AtmUniStats.RxCrcError);
                fprintf(stderr, "Number of OAM Loopback cells received  %d\n", 
					AtmUniStats.OAMLoopBkSuccess);
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (clear_stats)
        {
            // Query the Atm statistics information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_CLEAR_STATS ; rname = "CLEAR_STATS";
            pReq->len = 0;
            pReq->data = NULL;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan CLEAR_STATS IOCTL");
            }

            if (pReq->result != GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (run_test)
        {
            // Query the Atm statistics information
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_RUN_TEST ; rname = "RUN_TEST";
            pReq->len = sizeof(DIAG_STRUCT);
            pReq->data = &OamCell;
            fprintf(stderr, "================ %s ==================\n", rname);
			switch( TestCase )
			{
			case 1:
			case 2:
			case 3:
			case 4:
				break;
			case 5:
				OamCell.vci = OAM_F4_CELL_SEGMENT_VCI;
				break;
			case 6:
				OamCell.vci = OAM_F4_CELL_END2END_VCI;
				break;
			case 7:
				OamCell.pti = ATM_PTI_100;
				break;
			case 8:
				OamCell.pti = ATM_PTI_101;
				break;
			default:
				fprintf(stderr, "ERROR: Invalid Test case number %d\n",TestCase);
				break;
			}
			OamCell.TestCase = TestCase;
			if( RunTestCount == 0 )
				RunTestCount = 1;
			while(RunTestCount)
			{
				if (ioctl(fd, GSI_IOCTL, pArg) < 0)
				{
					perror("GlobeSpan RUN_TEST IOCTL");
					break;
				}

				if (pReq->result != GSI_IOCTL_SUCCESS)
				{
					fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
					fprintf(stderr, "       Result:%d Length:%d Data:%p\n", 
						pReq->result, pReq->len, pReq->data);
					break;
				}
				
				fprintf(stderr, "#");
				RunTestCount--;
			}

            fprintf(stderr, "\n");
        }

        if (start_retrain)
        {
            // Start adapter retrain
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_RETRAIN; rname = "RETRAIN";
            pReq->len = 0;
            pReq->data = NULL;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan RETRAIN IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "    ******** RETRAINING ********\n");
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        if (start_disconnect)
        {
            // Start adapter disconnect
            memset(pReq, 0, sizeof(*pReq));
            pReq->command = GSI_IOCTL_DISCONNECT; rname = "DISCONNECT";
            pReq->len = 0;
            pReq->data = NULL;
            fprintf(stderr, "================ %s ==================\n", rname);
            if (ioctl(fd, GSI_IOCTL, pArg) < 0)
            {
                perror("GlobeSpan DISCONNECT IOCTL");
            }

            if (pReq->result == GSI_IOCTL_SUCCESS)
            {
                fprintf(stderr, "    ******** DISCONNECTING ********\n");
            }
            else
            {
                fprintf(stderr, "ERROR: %s IOCTL request failed!\n", rname);
                fprintf(stderr, "       Result:%d Length:%d Data:%p\n", pReq->result, pReq->len, pReq->data);
            }
            fprintf(stderr, "\n");
        }

        // Close the connection to the driver
        close(fd);
    }

    return status;
}

void
usage(
    const char *progName
    )
{
    fprintf(stderr, "Usage: %s [-h] [-v] [-c] [-C] [-s] [-b] [-B [+l<n>] [+o<n>] ] [-t +i<n> +t<n> +a<> +c<n> [+n<n>]] [-r] [-d] [-e] [-E [+z<n>] [+v<n>] [+p<n>] [+m<n>] ] [interface]\n", progName);
    fprintf(stderr, "	\n");
    fprintf(stderr, "   Control Panel Application Version %s\n",version);
    fprintf(stderr, "	\n");
    fprintf(stderr, "   -h	For Control Panel Application Help \n");
    fprintf(stderr, "   -v	Query version information\n");
    fprintf(stderr, "   -c	Query current configuration parameters\n");
    fprintf(stderr, "   -C	Set configuration parameters\n");
    fprintf(stderr, "   -s	Query current Status and Data Rate if connected\n");
    fprintf(stderr, "   -n	Query Statistics\n");
    fprintf(stderr, "   -N	Clear Statistics\n");
    fprintf(stderr, "   -r	Retrain the DSL connection\n");
    fprintf(stderr, "   -d	Disconnect the DSL connection\n");
    fprintf(stderr, "   -e	Read Zero Byte, Vendor ID, ProductID and MAC address\n");
    fprintf(stderr, "   -E	Change Zero Byte, Vendor ID, ProductID and MAC address\n");
    fprintf(stderr, "   -b	Query current Debug Level\n");
    fprintf(stderr, "   -B 	Set Debug level and Module\n");
    fprintf(stderr, "   -t  Run Diagnostics\n");
    fprintf(stderr, "	\n");
    fprintf(stderr, "	-Configuration Parameters:\n");
    fprintf(stderr, "	Modify gp.cfg file in the driver directory to modify DSP parameters \n" );
    fprintf(stderr, "	Generally Customer doesn't need to modify these parameter\n");
    fprintf(stderr, "	\n");
    fprintf(stderr, "	-EEPROM Parameters:\n");
    fprintf(stderr, "	+z<n>	Set the value for Zero Byte\n");
    fprintf(stderr, "	+v<n>	Set the value for VendorID\n");
    fprintf(stderr, "	+p<n>	Set the value for ProductID\n");
    fprintf(stderr, "	+m<n>	Set the value for MAC Address\n");
    fprintf(stderr, "	-DEBUG Options:\n");
    fprintf(stderr, "	+l<n>	Set the debug level\n");
    fprintf(stderr, "	+o<n>	Set the debug modules\n");
    fprintf(stderr, "	-Diagnostic Test Options:\n");
    fprintf(stderr, "	+i<n>	Run the test specified by number <n>\n");
	fprintf(stderr, "	1	--	Spectrum Reverb Test\n");
    fprintf(stderr, "	2	--	Spectrum Medly Test\n");
    fprintf(stderr, "	3	--	Spectrum Pilot Test\n");
    fprintf(stderr, "	4	--	Spectrum R-MTPR Test\n");
    fprintf(stderr, "	5	--	F4 Loopback Segment Test\n");
    fprintf(stderr, "	6	--	F4 Loopback End To End Test\n");
    fprintf(stderr, "	7	--	F5 Loopback Segment Test\n");
    fprintf(stderr, "	8	--	F5 Loopback End To End Test\n");
    fprintf(stderr, "	+t<n>	PTI to run F4 tests\n");
    fprintf(stderr, "	+a<n>	VPI to run F4 tests\n");
    fprintf(stderr, "	+c<n>	VCI to run F5 tests\n");
    fprintf(stderr, "	+n<n>	Number of time to run test\n");

}

