/*+I*************************************************************************
 * GlobeSpan IOCTL Definitions
 *
 * Copyright (c) 2001 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 * Developed for GlobeSpan Semiconductor Inc. by TriplePoint, Inc.
 *   http://www.triplepoint.com
 * PG added the support for read and write EEPROM 03/05/01
 * 
 *-I*************************************************************************/
#ifndef GSI_IOCTL_H_
#define GSI_IOCTL_H_

#include "debug.h"
//#include "Gp.h"

#define GSI_PROTOCOL_VERSION_MAJOR  1
#define GSI_PROTOCOL_VERSION_MINOR  0

#define GSI_IOCTL               SIOCDEVPRIVATE
#define   GSI_IOCTL_GET_VERSION		0x01
#define   GSI_IOCTL_GET_CONFIG		0x02
#define   GSI_IOCTL_SET_CONFIG		0x03
#define   GSI_IOCTL_DISCONNECT		0x04
#define   GSI_IOCTL_RETRAIN			0x05

// added for reading and writing to EEPROM PG 03/05/01
#define	  GSI_IOCTL_READ_EEPROM		0x06
#define	  GSI_IOCTL_WRITE_EEPROM	0x07	  
#define   GSI_IOCTL_GET_STATUS		0x08
#define   GSI_IOCTL_GET_DBG			0x09
#define   GSI_IOCTL_SET_DBG			0x0a
#define   GSI_IOCTL_GET_STATS		0x0b
#define   GSI_IOCTL_CLEAR_STATS		0x0c
#define   GSI_IOCTL_RUN_TEST		0x0d

#define GSI_IOCTL_SUCCESS   1
#define GSI_IOCTL_ERR_LEN   2
#define GSI_IOCTL_FAILURE   ~0

#if EEPROMv2
#define MAX_EEPROM_SIZE                        1024
#else
#define MAX_EEPROM_SIZE                        64
#endif//EEPROMv2

#define OAM_F4_CELL_SEGMENT_VCI		3
#define OAM_F4_CELL_END2END_VCI		4
#define ATM_PTI_101					5// Payload type for F5 cell (end-to-end)
#define ATM_PTI_100					4// Payload type for F5 cell (segment)


#ifndef PACKED
#  define PACKED __attribute__((packed))
#endif /* PACKED */

/* IOCTL request structure */
struct gsi_req {
    uint8_t     command;    // Command
    uint8_t     result;     // Command status
    uint16_t    len;        // Length in bytes of the buffer pointed to by data
    void       *data;       // Data buffer for additional information
} PACKED;

/* LAN inteface IOCTL request structure */
struct gsi_lan_req {
    union {
        char            ifrn_name[IFNAMSIZ];
    }               ifr_ifrn;

    struct gsi_req  request;
} PACKED;

typedef struct {
    struct {
        uint8_t     major;
        uint8_t     minor;
    }           protocol;
    char        driver[16];
    char        firmware[8];
} PACKED gsi_version_t;


typedef struct {
  //    unsigned char	  multiFunc;
  //    unsigned char	  configNo;
  //   unsigned char	  dslPM;
  //   unsigned char     dslAuxCur;
  //   unsigned char     v90PM;
  //  unsigned char	  v90AuxCur;
  //   unsigned int    dslSubsystemId;
  //  unsigned int    dslVendorId;
  //   unsigned int    v90SubsystemId;
  //   unsigned int    v90VendorId;
  char     macAddr[6];
} PACKED PGSIDSL_PANDA_EEPROM;

typedef struct  _GSIDSL_EEPROM
{
	unsigned char	zeroByte;
	unsigned char	vendorIdLow;
	unsigned char	vendorIdHigh;
	unsigned char	productIdLow;
	unsigned char	productIdHigh;
	unsigned char	releaseLow;
	unsigned char	releaseHigh;
	unsigned char	macAddr0; // MAC address byte 0
	unsigned char	macAddr1;
	unsigned char	macAddr2;
	unsigned char	macAddr3;
	unsigned char	macAddr4;
	unsigned char	macAddr5;
#ifdef EEPROMv2
	unsigned char   usbPower; //bus powered or hub powered.
	unsigned char   busCurrent; //expected current drawn
	unsigned char	eepromSize; //in long words => MAX_EEPROM_SIZE/4 = upto 256.
	unsigned char	proprietaryImage[MAX_EEPROM_SIZE/4 - 16]; //magic# 16: total # of preceding bytes in this structure
#endif//EEPROMv2

} GSIDSL_EEPROM, *PGSIDSL_EEPROM;

#define	EEPROM_SIZE	sizeof(GSIDSL_EEPROM)

typedef struct {
    uint8_t     major;
    uint8_t     minor;
} PACKED gsi_stats_t;

#define GSI_INTERFACE_LAN    (0x01 << 0)
#define GSI_INTERFACE_WAN    (0x01 << 1)

typedef struct {
    uint8_t     interface;

    uint8_t     rfc1483Mode;
    uint16_t    rfc1483Vci;
    uint16_t    rfc1483Vpi;
    uint8_t     rfc2364Mode;
    uint16_t    rfc2364Vci;
    uint16_t    rfc2364Vpi;
    uint16_t    rxBufferCount;
    uint16_t    txBufferCount;

    uint16_t    gtiStandard;
    uint16_t    gtiTrellis;
    uint16_t    gtiTxPowerAtten;
    uint16_t    gtiCodingGain;
    uint16_t    gtiMaxBitsPerBin;
    uint16_t    gtiTxStartBin;
    uint16_t    gtiTxEndBin;
    uint16_t    gtiRxStartBin;
    uint16_t    gtiRxEndBin;
    uint16_t    gtiRxBinAdjust;
    uint16_t    gtiFramingStruct;
    uint16_t    gtiExpandedExchange;
    uint16_t    gtiClockType;
    uint16_t    gtiMaxDownRate;
} PACKED gsi_config_t;

typedef struct {
	uint32_t	u32DspState;
	uint32_t	u32UsbState;
	uint32_t	u32UpRate;
	uint32_t	u32DownRate;
	uint32_t	u32Modulation;
} PACKED gsi_status_t;

typedef	struct	{
	DBG_PRINT_LEVEL		level;
	uint32_t			SetLevel;
	DBG_PRINT_MODULE	module;
	uint32_t			SetModule;
} PACKED gsi_debug_t;

//------------------------------------------------------------------------------
//
// ATM_UNI_STATS
//		Defines the statistics kept by the ATM UNI (kept both globally and
//      per VC)
//
//------------------------------------------------------------------------------
typedef struct tATM_UNI_STATS {
	uint32_t	TxPdusOk;
	uint32_t	TxPdusError;
	uint32_t	RxPdusOk;
	uint32_t	RxPdusError;
	uint32_t	RxPdusNoBuffer;
	uint32_t	RxCrcError;
	uint32_t	RxCellsOk;
	uint32_t	TxCellsOk;
	uint32_t	RxCellsDropped;
	uint32_t	RxInvalidVpiVci;
	uint32_t	OAMLoopBkSuccess;//loopback cells received successfully
	uint32_t	OAMLoopBkFailure;//loopback cells not formatted correctly
	//uint32_t	RxCellsBadHeader;
} ATM_UNI_STATS, *PATM_UNI_STATS;

typedef struct tDIAG_STRUCT
{
	uint32_t	TestCase;
	uint32_t	vpi;
	uint32_t	vci;
	uint32_t	pti;
	uint32_t	type;		// loopback type
} DIAG_STRUCT, *pDIAG_STRUCT;


#ifdef __KERNEL__
#if DBG
extern const char *DbgIoctlCmd(unsigned int cmd);
extern const char *DbgGsiReq(struct gsi_req *pReq);
#endif /* DBG */
#endif /* __KERNEL__ */

#endif /* GSI_IOCTL_H_ */



















