#ifdef GSI_LAN

    #include "GpSnull.h"
    #include "api1577.h"
    #include "atmuni.h"
    #include "rfc1483.h"


UINT  Api1577GetDestinationIP(PNDIS_PACKET  		Packet)
{
	PNDIS_BUFFER  	NdisBuffer,FirstNdisBuffer;
	UINT        	PacketLength;
	UINT        	BufferCount;
	UINT        	PhyBufferCount;
	UINT			BufferLength;
    UINT            DestinationIP;
	PUCHAR	      	InputBuffer;
    int i;
    PIP_HEADER      IpHeader;

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577GetDestinationIP\n"));

    ASSERT( Packet);

    NdisRecalculatePacketCounts( Packet);

    NdisQueryPacket( 
			Packet,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);
	
		ASSERT( BufferCount);
		ASSERT( NdisBuffer);
	FirstNdisBuffer = NdisBuffer;

    for(i = 0; i < IP_HEADER_SEQ; i++)
    {
        NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &BufferLength);
        NdisGetNextBuffer( NdisBuffer, &NdisBuffer);
    }
        
	
	NdisBuffer = FirstNdisBuffer;
    
    /* Now InputBuffer points to IP Header*/
    IpHeader = (PIP_HEADER)InputBuffer;    
    DestinationIP = IpHeader->DestinationIP;
    
    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("SourceIP = %d.%d.%d.%d\n" , 
        (DestinationIP & 0xFF000000) >> 24 , 
        (DestinationIP & 0x00FF0000) >> 16,
        (DestinationIP & 0x0000FF00) >> 8,
        (DestinationIP & 0x000000FF) ));

return DestinationIP;

}
PETH_ARP_PACKET Api1577GetEtherHeader(PNDIS_PACKET Packet)
{
    PNDIS_BUFFER  	NdisBuffer;
	UINT        	PacketLength;
	UINT        	BufferCount;
	UINT        	PhyBufferCount;
	UINT			BufferLength;
	PUCHAR	      	InputBuffer;

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577GetEtherHeader\n"));

    ASSERT( Packet);

    NdisRecalculatePacketCounts( Packet);

    NdisQueryPacket( 
			Packet,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);
	
		ASSERT( BufferCount);
		ASSERT( NdisBuffer);

      
    
    NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &BufferLength);
return (PETH_ARP_PACKET)(InputBuffer + sizeof(MAC_HEADER));    
}


UINT  Api1577GetDestIPFromEther (PNDIS_PACKET Packet)
{
	PNDIS_BUFFER  	NdisBuffer;
	UINT        	PacketLength;
	UINT        	BufferCount;
	UINT        	PhyBufferCount;
	UINT			BufferLength;
    UINT            DestinationIP;
	PUCHAR	      	InputBuffer;
    PETH_ARP_PACKET      IpHeaderArp;

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577GetDestIPFromEther\n"));

    ASSERT( Packet);

    NdisRecalculatePacketCounts( Packet);

    NdisQueryPacket( 
			Packet,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);
	
		ASSERT( BufferCount);
		ASSERT( NdisBuffer);
    
    NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &BufferLength);
        
    IpHeaderArp = (PETH_ARP_PACKET)(InputBuffer + sizeof(MAC_HEADER));    
    

    COPY_MEMORY( (void *)&DestinationIP , IpHeaderArp->DestIpAddr, IP_ADDRESS_SIZE) ;
    
    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("SourceIP = %d.%d.%d.%d\n" , 
        (DestinationIP & 0xFF000000) >> 24 , 
        (DestinationIP & 0x00FF0000) >> 16,
        (DestinationIP & 0x0000FF00) >> 8,
        (DestinationIP & 0x000000FF) ));

return DestinationIP;


}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: Api1577GetDestMacAddress
//
// ABSTRACT:
//		Gets destination Mac Address. This Api's are used by Rfc1577 Layer( ATM ARP)
//		
// RETURNS:
//      TRUE - in case of valid Mac Header		
//      FALSE - in Case of invalid Mac Header
//
// ASSUMPTIONS: PUCHAR Buffer has to be allocated By Caller
//		
//
////////////////////////////////////////////////////////////////////////////////

BOOLEAN Api1577GetDestMacAddress(PNDIS_PACKET  Packet , PUCHAR Buffer)
{
	PNDIS_BUFFER  	NdisBuffer;
	UINT        	PacketLength;
	UINT        	BufferCount;
	UINT        	PhyBufferCount;
	UINT			BufferLength;
	PUCHAR	      	InputBuffer;
    PMAC_HEADER         MacHeader;

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577GetDestMacAddress\n"));

 
    NdisRecalculatePacketCounts( Packet);

    NdisQueryPacket( 
			Packet,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);
	

		ASSERT( BufferCount);
		ASSERT( NdisBuffer);

    NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &BufferLength);

    ASSERT(BufferLength >= sizeof(MAC_HEADER));
    MacHeader   = (PMAC_HEADER)InputBuffer; 
    COPY_MEMORY( Buffer , MacHeader->DstAddress , ETH_LENGTH_OF_ADDRESS);

return TRUE;
}




USHORT  Api1577GetType(PNDIS_PACKET  		Packet)
{
	PNDIS_BUFFER  	NdisBuffer;
	UINT        	PacketLength;
	UINT        	BufferCount;
	UINT        	PhyBufferCount;
	UINT			BufferLength;
    USHORT          Type;
    PMAC_HEADER      MacHeader; 

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577GetType\n"));

    ASSERT( Packet);

	NdisQueryPacket( 
			Packet,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);

		ASSERT( BufferCount);
		ASSERT( NdisBuffer);
		NdisQueryBuffer( NdisBuffer, (PVOID *)&MacHeader, &BufferLength);
		ASSERT( BufferLength >= sizeof( MAC_HEADER) );
        Type =  MacHeader->EtherType;       
 return Type;
}


PNDIS_PACKET Api1577MakeNdisPacket(PATM_USB_INSTANCE Instance , PXFER_DATA_BUFFER txBuffer)
{
    PNDIS_PACKET    NdisPkt;
    PNDIS_BUFFER    NdisBuff;
    NDIS_STATUS     Status;  
    
    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577MakeNdisPacket\n"));

    ASSERT( Instance);
    ASSERT( txBuffer );


    NdisAllocatePacket (&Status, &NdisPkt, Instance->Rfc1577.Rfc1577NdisPktPoolHandle );
    
    if (Status != NDIS_STATUS_SUCCESS)
           return NULL; /* We ran out of buffer */
       else
       {
           //  First, attach the buffer to the packet if available
           NdisAllocateBuffer (
               &Status,
               &NdisBuff,
               Instance->Rfc1577.Rfc1577NdisBuffPoolHandle,
               txBuffer->Data,
               txBuffer->Size);

 

           if (Status != NDIS_STATUS_SUCCESS)
           {
               NdisFreePacket (NdisPkt);
               return NULL;
           }

           NdisChainBufferAtFront (NdisPkt, NdisBuff);
        }

    //Save Pointer to TX Buffer for Future Release this buffer
    NdisPkt->Reserved[0] = (ULONG_PTR) txBuffer;           ;

return NdisPkt;
}


BOOLEAN     Api1577FreeNdisPacket(PATM_USB_INSTANCE Instance , PNDIS_PACKET NdisPkt )
{
     PNDIS_BUFFER   NdisBuff;
     PXFER_DATA_BUFFER txBuffer;
     DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577FreeNdisPacket\n"));

    ASSERT( Instance);
    ASSERT( NdisPkt );
    /* Get Poinetr*/
    txBuffer    = (PXFER_DATA_BUFFER)NdisPkt->Reserved[0];

    
     NdisUnchainBufferAtFront (NdisPkt, &NdisBuff);
        if (NdisBuff)
           NdisFreeBuffer (NdisBuff);
     NdisReinitializePacket (NdisPkt);
     NdisFreePacket (NdisPkt);

    Api1577FreeBuffer(Instance, txBuffer);
return TRUE;

}


/*
PXFER_DATA_BUFFER must to be allocated by the caller

*/
BOOLEAN Api1577GetXferBuffer(PATM_USB_INSTANCE Instance , PNDIS_PACKET NdisPkt, PXFER_DATA_BUFFER txBuffer )
{
	UINT          	PhysicalBufferCount, InputLength;
	UINT          	BufferCount , i;
	PNDIS_BUFFER  	NdisBuffer;
	UINT          	TotalPacketLength;
    PUCHAR          OutputBuffer,InputBuffer;
	
     DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577GetXferBuffer\n"));

    ASSERT( Instance);
    ASSERT( NdisPkt );
    ASSERT( txBuffer );

OutputBuffer = txBuffer->Data;    
txBuffer->Size = 0;

    NdisRecalculatePacketCounts( NdisPkt);
	
    NdisQueryPacket(
	  NdisPkt,
	  &PhysicalBufferCount,
	  &BufferCount,
	  &NdisBuffer,
	  &TotalPacketLength);

	//
	// Verify that the Packet contains data
	//
	if( !NdisBuffer || !TotalPacketLength || !BufferCount)
    {
	  	DBGPRINT(
			DBG_PRINT_MODULE_RFC1577_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Invalid Packet\n"));
		return FALSE;
	}

    if(TotalPacketLength > RFC1483_MAX_SDU_SIZE )
    {
	  	DBGPRINT(
			DBG_PRINT_MODULE_RFC1577_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Invalid Packet Size = %d\n", TotalPacketLength));
		return FALSE;
    
    
    }


	for( i = 0; i < BufferCount; i++ )
    {
		NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &InputLength);

        if( (NULL == InputBuffer) || (0 == InputLength)) 
        {
			DBGPRINT(
				DBG_PRINT_MODULE_RFC1577_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Skipping invalid NDIS buffer\n"));
	  	    NdisGetNextBuffer( NdisBuffer, &NdisBuffer);
		    continue;
		}
		
       COPY_MEMORY(OutputBuffer , InputBuffer, InputLength); 
       OutputBuffer += InputLength;
       txBuffer->Size += InputLength;

       NdisGetNextBuffer( NdisBuffer, &NdisBuffer);
	}//end for
return TRUE;
}


ATM_UNI_HANDLE Api1577OpenVC(PATM_USB_INSTANCE Instance , UCHAR VPI, USHORT VCI , RFC1483_MODE Rfc1483Mode)
{
ATM_UNI_HANDLE Handle;
ATM_UNI_STATUS	AtmUniStatus;
BOOLEAN    EncapsulationStatus;    
UCHAR EncapsulationSize;
PUCHAR Encapsulation;

     DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Api1577OpenVC\n"));

    ASSERT( Instance);


Handle = AtmUniIsVCExist(Instance->AtmUniHandle ,  VPI ,  VCI );

     DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Handle = %x\n", Handle));
    
    if( Instance->Rfc1577Enable && !Handle )
    { //VC doesn't exist

                Handle = AtmUniCreateVc(
                		    Instance->AtmUniHandle,
                		    NULL,
                		    Rfc1483ReceiveData,
                		    NULL,
                		    Rfc1483SendComplete);
                
                    if(!Handle)
                    {
                		DBGPRINT(
                			DBG_PRINT_MODULE_RFC1577_ALL,
                			DBG_PRINT_LEVEL_ERR,
                			("AtmUniCreateVc failed\n"));
                		return NULL;
                    }
                
                   EncapsulationStatus = Rfc1483GetEncapsulation(
                        Instance,
                        Rfc1483Mode,
                        &Encapsulation,
                        &EncapsulationSize
                        );
                
                        if(!EncapsulationStatus )
                        {
                        		DBGPRINT(
                        			DBG_PRINT_MODULE_RFC1577_ALL,
                        			DBG_PRINT_LEVEL_ERR,
                        			("Rfc1483GetEncapsulation failed\n"));
                        		return NULL;
                           
                        }
                        
                        
                        
                        AtmUniStatus = AtmUniActivateVc(
                        		Handle,
                        		VPI,
                        		VCI,
                        		AAL5_,
                        	   	RFC1483_MAX_SDU_SIZE,
                        		Encapsulation,
                        		EncapsulationSize,
                        		FALSE,
                                0);
                        	
                        	if ( ATM_UNI_STATUS_SUCCESS != AtmUniStatus) 
                            {
                        		DBGPRINT(
                        			DBG_PRINT_MODULE_RFC1577_ALL,
                        			DBG_PRINT_LEVEL_ERR,
                        			("AtmUniActivateVc failed\n"));
                        		return( FALSE);
                        	}
                        
         }//end VC doesn't exist
                
    AtmUniSetRfc1483Mode(Handle , Rfc1483Mode );
    
    
    if( Instance->Rfc1577Enable && (Rfc1483Mode == RFC1483_MODE_ROUTED_IP_LLC)  )
    {
    EncapsulationStatus = Rfc1483GetEncapsulation(
                                Instance,
                                RFC1483_MODE_RFC1577_ENCAP,
                                &Encapsulation,
                                &EncapsulationSize
                                );


    }
    else 
    {
        Encapsulation = NULL;
        EncapsulationSize = 0;
    }

    AtmUniSetRfc1577Encap(Handle , Encapsulation , EncapsulationSize );
                        

return Handle;
}

BOOLEAN Api1577CloseVC(PATM_USB_INSTANCE Instance , ATM_UNI_HANDLE AtmUniHandle )
{
     DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577CloseVC\n"));

    ASSERT( Instance);
    ASSERT( AtmUniHandle);


    AtmUniDeactivateVc( AtmUniHandle);
    AtmUniDeleteVc(AtmUniHandle	);

return TRUE;
}


void * Api1577GetContext(PATM_USB_INSTANCE Instance)
{
	DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577GetContext\n"));


ASSERT(Instance);
return Instance->Rfc1577Config.Rfc1577Context;
}


PXFER_DATA_BUFFER Api1577GetBuffer(PATM_USB_INSTANCE Instance)
{

	DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577GetBuffer\n"));

return AtmUniGetTxBuffer( Instance->AtmUniHandle );
}



BOOLEAN Api1577FreeBuffer(PATM_USB_INSTANCE Instance , PXFER_DATA_BUFFER TxBuffer)
{

    DBGPRINT(
		DBG_PRINT_MODULE_RFC1577_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("Rfc1577FreeBuffer\n"));

return AtmUniFreeTxBuffer(Instance->AtmUniHandle, TxBuffer);

}



#endif // GSI_LAN
