////////////////////////////////////////////////////////////////////////////////
//
// GlobeSpan Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobeSpan Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998 GlobeSpan Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.3 $
//     $Modtime: 8/21/01 12:18p $
//     $Author: kattre $
//
// ABSTRACT:
//		Contains the implementation of the ATM UNI (SoftSAR) module
//
////////////////////////////////////////////////////////////////////////////////
#ifndef ATMUNI_C
#define ATMUNI_C
#endif
#include "GpSnull.h"
#include "debug.h"
#include "atmuni.h"
#include "crc.h"

#include "oam.h"	// include cell type and OAM processing for F4 & F5 ATM cells
#ifdef	MESSAGE_HANDLING
#include "Message_Handling.h"
#endif
#ifdef AUTO_CONFIG
#include "AutoConfig.h"
#endif

//extern void UsbPhyCheckDspStatus ( PVOID UsbPhy);

/* FIXME: The defines should be rolled into ONE common.h file */
#define ETHERNET_HEADER_SIZE	        14
#define HEC_LIMIT_COUNTER              3
#ifdef LAN_MINIPORT 
#define PADDING_LEN_MAX      64
#define PADDING              0x00
	UCHAR	EthPadding[PADDING_LEN_MAX] = 
	{
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING, PADDING,
       PADDING, PADDING, PADDING, PADDING
	};
#endif

#define    BIT_MASK_SKIPFIRSTBUFFER    0x01
#define    BIT_MASK_SENDCOMPLETEACK    0x02
#define    BIT_MASK_ARPFLAG            0x80

//------------------------------------------------------------------------------
//
// Local Constant Definitions
//
//------------------------------------------------------------------------------
#define  ATM_PHY_PADDING                     11
UCHAR AtmPhyPaddingPattern[ATM_PHY_PADDING]= {0xFF, 0xFF, 0xFF, 0xFF,
                                             0xFF, 0xFF, 0xFF, 0xFF,
                                             0xFF, 0xFF, 0xFF};


#ifdef QUASAR
#undef ATM_PHY_PADDING
#define ATM_PHY_PADDING      0
#endif //QUASAR


#define    COSET_LEADER                        0x55 /* x^6 + x^4 + x^2 + 1  */
static UCHAR hecTable[256] = {
0x00, 0x07, 0x0e, 0x09, 0x1c, 0x1b, 0x12, 0x15, 0x38, 0x3f,
0x36, 0x31, 0x24, 0x23, 0x2a, 0x2d, 0x70, 0x77, 0x7e, 0x79,
0x6c, 0x6b, 0x62, 0x65, 0x48, 0x4f, 0x46, 0x41, 0x54, 0x53,
0x5a, 0x5d, 0xe0, 0xe7, 0xee, 0xe9, 0xfc, 0xfb, 0xf2, 0xf5,
0xd8, 0xdf, 0xd6, 0xd1, 0xc4, 0xc3, 0xca, 0xcd, 0x90, 0x97,
0x9e, 0x99, 0x8c, 0x8b, 0x82, 0x85, 0xa8, 0xaf, 0xa6, 0xa1,
0xb4, 0xb3, 0xba, 0xbd, 0xc7, 0xc0, 0xc9, 0xce, 0xdb, 0xdc,
0xd5, 0xd2, 0xff, 0xf8, 0xf1, 0xf6, 0xe3, 0xe4, 0xed, 0xea,
0xb7, 0xb0, 0xb9, 0xbe, 0xab, 0xac, 0xa5, 0xa2, 0x8f, 0x88,
0x81, 0x86, 0x93, 0x94, 0x9d, 0x9a, 0x27, 0x20, 0x29, 0x2e,
0x3b, 0x3c, 0x35, 0x32, 0x1f, 0x18, 0x11, 0x16, 0x03, 0x04,
0x0d, 0x0a, 0x57, 0x50, 0x59, 0x5e, 0x4b, 0x4c, 0x45, 0x42,
0x6f, 0x68, 0x61, 0x66, 0x73, 0x74, 0x7d, 0x7a, 0x89, 0x8e,
0x87, 0x80, 0x95, 0x92, 0x9b, 0x9c, 0xb1, 0xb6, 0xbf, 0xb8,
0xad, 0xaa, 0xa3, 0xa4, 0xf9, 0xfe, 0xf7, 0xf0, 0xe5, 0xe2,
0xeb, 0xec, 0xc1, 0xc6, 0xcf, 0xc8, 0xdd, 0xda, 0xd3, 0xd4,
0x69, 0x6e, 0x67, 0x60, 0x75, 0x72, 0x7b, 0x7c, 0x51, 0x56,
0x5f, 0x58, 0x4d, 0x4a, 0x43, 0x44, 0x19, 0x1e, 0x17, 0x10,
0x05, 0x02, 0x0b, 0x0c, 0x21, 0x26, 0x2f, 0x28, 0x3d, 0x3a,
0x33, 0x34, 0x4e, 0x49, 0x40, 0x47, 0x52, 0x55, 0x5c, 0x5b,
0x76, 0x71, 0x78, 0x7f, 0x6a, 0x6d, 0x64, 0x63, 0x3e, 0x39,
0x30, 0x37, 0x22, 0x25, 0x2c, 0x2b, 0x06, 0x01, 0x08, 0x0f,
0x1a, 0x1d, 0x14, 0x13, 0xae, 0xa9, 0xa0, 0xa7, 0xb2, 0xb5,
0xbc, 0xbb, 0x96, 0x91, 0x98, 0x9f, 0x8a, 0x8d, 0x84, 0x83,
0xde, 0xd9, 0xd0, 0xd7, 0xc2, 0xc5, 0xcc, 0xcb, 0xe6, 0xe1,
0xe8, 0xef, 0xfa, 0xfd, 0xf4, 0xf3 };

//------------------------------------------------------------------------------
//
// Local Variable Definitions
//
//------------------------------------------------------------------------------
static UCHAR AtmCellPadding[ ATM_CELL_PAYLOAD_SIZE] = {0};

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniInitialize
//
//  ABSTRACT:
//		Performs one-time intializations for the ATM UNI module
//
//  RETURNS:
//		BOOLEAN
//
//  ASSUMPTIONS:
//		Must be called exactly once and prior to calling any other
//      functions with the ATM UNI module
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN
AtmUniInitialize( void)
{
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniInitialize\n"));

	CrcGenerateTable();

	return( TRUE);

} // AtmUniIntialize

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniCreate
//
//  ABSTRACT:
//		Creates a new ATM UNI interface.  Normally called once during driver
//		initialization.
//
//  RETURNS:
//		A handle to the newly created ATM UNI or NULL if the request failed
//
//  ASSUMPTIONS:
//		AtmUniInitialize must be called prior to calling this function.
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_HANDLE
AtmUniCreate(
	ATM_UNI_USER_HANDLE				UserHandle,
	ATM_UNI_USER_LINK_STATUS_CHANGE	UserLinkStatusChange,
	BOOLEAN							IndicatePackets,
	ULONG							MediaSpecificInformationSize,
	USHORT							MaxSduSize,
	USHORT							RxBufferCount,
	USHORT							TxBufferCount,
	NDIS_HANDLE						MiniportAdapterHandle,
	ATM_PHY_BIND 					PhyBind,
	ATM_PHY_DEVICE_HANDLE			PhyDeviceHandle,
	PUCHAR							PhyUniqueId,
	PVOID							UsbPhyHandle
)
{
	PATM_UNI	AtmUni;
	ULONG    	i;
	BOOLEAN  	Success;


	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniCreate\n"));


	ASSERT( PhyBind);
	ASSERT( PhyUniqueId);

	ALLOCATE_MEMORY( &AtmUni, sizeof( ATM_UNI));
	if( NULL == AtmUni) {
	  return( NULL);
	}

	INITIALIZE_LIST_HEAD( &(AtmUni->InactiveVcList));
	AtmUni->TxBufferCount 			= (USHORT)(TxBufferCount + LOOPBACK_MAX_BUFFERS);
	AtmUni->RxBufferCount 			= RxBufferCount;
	AtmUni->UserHandle	  			= UserHandle;
	AtmUni->UserLinkStatusChange	= UserLinkStatusChange;
	AtmUni->IndicatePackets			= IndicatePackets;
	AtmUni->MediaSpecificSize		= MediaSpecificInformationSize;

	AtmUni->validHeaderCount		= 0;
	AtmUni->isOutOfSync				= FALSE;
   AtmUni->InvalidHECLimitCounter  =0;
	AtmUni->NeedQueueAtmProcess		= TRUE;
	AtmUni->AtmTxCount				= 0;
    AtmUni->LbAtmUniVc              = NULL;
    AtmUni->LbVpiVci                = 0;

	for( i = 0; i < ATM_UNI_VC_HASH_TABLE_SIZE; i++) {
	  INITIALIZE_LIST_HEAD( &(AtmUni->VcHashTable[i]));
	}

	//
	// Allocate locks to protect portions of the ATM UNI structure
	// These locks are used as follows:
	//
	// VcListLock
	//   Must be held prior to adding or removing a VC from either the
	//   inactive VC list or the VC hash table.
	// RxBufferFreeListLock
	//   Must be held prior to adding or removing an RX buffer from the
	//   RX buffer free list.
	// TxBufferFreeListLock
	//   Must be held prior to adding or removing a TX buffer from the
	//   TX buffer free list.
	//
	// Note that ONE AND ONLY ONE LOCK SHOULD BE HELD AT ANY TIME to avoid
	// deadlock.  All locks must be released prior to any calls outside of
	// the ATM UNI module.
	//
	NdisAllocateSpinLock( &AtmUni->VcListLock);
	NdisAllocateSpinLock( &AtmUni->RxBufferFreeListLock);
	NdisAllocateSpinLock( &AtmUni->TxBufferFreeListLock);
	
	NdisMInitializeTimer( &AtmUni->DspStatusTimer,
						MiniportAdapterHandle,
						(PNDIS_TIMER_FUNCTION)AtmCheckDspStatus,
						AtmUni);

	NdisInitializeEvent( &AtmUni->DspStatusTimerEvent);
    NdisAllocateSpinLock( &AtmUni->DspSpinLock);

#ifdef MESSAGE_HANDLING
    NdisAllocateSpinLock( &AtmUni->MsgHandlingLock);
	NdisInitializeEvent( &AtmUni->MsgHandling_Event);
	NdisMInitializeTimer( &AtmUni->MsgHandling_Timer,
						MiniportAdapterHandle,
						MsgHandling_Process,
						AtmUni);
#endif // MESSAGE_HANDLING

	AtmUni->MaxSduSize            = MaxSduSize;
	AtmUni->MiniportAdapterHandle = MiniportAdapterHandle;
    AtmUni->AtmUniState           = ATM_UNI_STATE_INITIALIZED;

	Success = AtmUniAllocateBuffers( AtmUni, AtmUni->RxBufferCount, AtmUni->TxBufferCount);
	if( !Success )
	{
		DBGPRINT(DBG_PRINT_MODULE_ATMUNI_ALL,DBG_PRINT_LEVEL_ERR,
			("AtmUniCreate Failed (unable to allocate buffers)\n"));
		AtmUniDestroy( AtmUni);
		return( NULL);
	}

	//
	// Bind with the PHY layer
	//
	Success = AtmUniBindWithPhy( AtmUni,PhyBind,PhyDeviceHandle);
	if( !Success ) {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("AtmUniCreate Failed (unable to bind with the phy layer)\n"));
		AtmUniDestroy( AtmUni);
		return( NULL);
	}

	// save pointer to UsbPhy for use in power management
	COPY_MEMORY( UsbPhyHandle, &(AtmUni->PhyHandle), sizeof(ULONG));

	COPY_MEMORY( PhyUniqueId, AtmUni->PhyUniqueId, UNIQUE_ID_SIZE);

    DslEocBind(&AtmUni->DslEx, AtmUni->PhyHandle);

#ifdef MESSAGE_HANDLING
	MsgHandling_Init( AtmUni);
#endif // MESSAGE_HANDLING

#ifdef USB_WIN_AUTO_CONFIG
    AtmUni->pAutoConfig = Create_Ac_Layer ( AtmUni, AtmUni->PhyUniqueId);
#endif // USB_WIN_AUTO_CONFIG

    NdisResetEvent( &AtmUni->DspStatusTimerEvent);
	NdisMSetTimer( &AtmUni->DspStatusTimer, DSP_STATUS_TIMER);

	return( AtmUni);

} // AtmUniCreate


////////////////////////////////////////////////////////////////////////////////
// NAME: AtmCheckDspStatus
//
// ABSTRACT:
//		Check DSP status for disconnect.
//
// RETURNS:
//		void
////////////////////////////////////////////////////////////////////////////////
void AtmCheckDspStatus(
	PVOID				SystemSpecific1,
	PATM_UNI			AtmUni,	   
	PVOID				SystemSpecific2,
	PVOID				SystemSpecific3
)
{
    NdisAcquireSpinLock( &AtmUni->DspSpinLock);
    if ( (AtmUni->AtmUniState & ATM_UNI_STATE_HALTING) ||
         (AtmUni->AtmUniState & ATM_UNI_STATE_SUSPENDING) )
    {
        NdisSetEvent( &AtmUni->DspStatusTimerEvent);      
    }
    else
    {
        UsbPhyCheckDspStatus( AtmUni->PhyHandle);
        NdisMSetTimer( &AtmUni->DspStatusTimer, DSP_STATUS_TIMER);
    }
    NdisReleaseSpinLock( &AtmUni->DspSpinLock);
}

////////////////////////////////////////////////////////////////////////////////
// NAME: AtmUniShutdownDspStatus
//
// ABSTRACT:
//		shutdown DSP status checking
//
// RETURNS:
//		void
////////////////////////////////////////////////////////////////////////////////
void AtmUniShutdownDspStatus( PATM_UNI AtmUni)
{
    BOOLEAN		TimerCancelled;

    if ( (AtmUni->AtmUniState & ATM_UNI_STATE_HALTING) ||
         (AtmUni->AtmUniState & ATM_UNI_STATE_SUSPENDING) )
    {
        NdisAcquireSpinLock( &AtmUni->DspSpinLock);
	    NdisMCancelTimer( &AtmUni->DspStatusTimer, &TimerCancelled);    // Cancel the timer
		NdisReleaseSpinLock( &AtmUni->DspSpinLock);

	    if(!TimerCancelled) 
        {
            if ( !NdisWaitEvent( &AtmUni->DspStatusTimerEvent, DSP_STATUS_TIMER * 2) ) 
            {
                DBGPRINT(
	                DBG_PRINT_MODULE_NDIS_ALL,
	                DBG_PRINT_LEVEL_ERR,
	                ("ERROR: Attempt to cancel dsp status timer failed!\n"));
            }
	    }
	    else 
        {
            NdisSetEvent( &AtmUni->DspStatusTimerEvent);
	    }
  	}
}

void AtmUniShutdownDspStatusEx( PATM_UNI AtmUni)
{
    BOOLEAN		TimerCancelled;

    if ( (AtmUni->AtmUniState & ATM_UNI_STATE_HALTING) ||
         (AtmUni->AtmUniState & ATM_UNI_STATE_SUSPENDING) )
    {
	    NdisMCancelTimer( &AtmUni->DspStatusTimer, &TimerCancelled);    // Cancel the timer

	}
}



///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniShutdownTransmit
//
//  ABSTRACT:
//		shutdown transmit thread during halt, suspend, and disconnect
//
//  RETURNS:
//		void
///////////////////////////////////////////////////////////////////////////////
void AtmUniShutdownTransmit( PATM_UNI AtmUni)
{
    PPROCESS_DATA_BUFFER   ProcessBuffer;

    if ( AtmUni->AtmUniState & ATM_UNI_STATE_LAYER_DOWN )
    {
        NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
	    while ( !IS_LIST_EMPTY( &AtmUni->ProcessBufferTxList) )
	    {
		    ProcessBuffer = (PPROCESS_DATA_BUFFER)REMOVE_TAIL_LIST( &AtmUni->ProcessBufferTxList);
            AtmUniFreeTransmitBuffer( AtmUni, ProcessBuffer->Buffer);
            AtmUniFreeProcessBuffer( AtmUni, ProcessBuffer);
        }
        NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        AtmUniEmptySendPacketQueue(AtmUni);
    }
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniFreeProcessBuffer
//
//  ABSTRACT:
//		frees a process buffer
//
//  ASSUMPTIONS:
//      function called within tx spinlock
//
//  RETURNS:
//		void
///////////////////////////////////////////////////////////////////////////////
void AtmUniFreeProcessBuffer( PATM_UNI AtmUni, PPROCESS_DATA_BUFFER	ProcessBuffer)
{
    if ( ProcessBuffer )
    {
        if (ProcessBuffer->memType == MEM_TYPE_STATIC)
        {
            INSERT_HEAD_LIST( &AtmUni->ProcessBufferFreeList, (PLIST_NODE)ProcessBuffer);
        }
        else if (ProcessBuffer->memType == MEM_TYPE_DYNAMIC)
        {
            FREE_MEMORY( ProcessBuffer, sizeof(PROCESS_DATA_BUFFER));
        }
        else
        {
            ASSERT(FALSE);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniFreeTransmitBuffer
//
//  ABSTRACT:
//		free a transmit buffer
//
//  ASSUMPTIONS:
//      function called within tx spinlock
//
//  RETURNS:
//		void
///////////////////////////////////////////////////////////////////////////////
void AtmUniFreeTransmitBuffer( PATM_UNI AtmUni, PXFER_DATA_BUFFER Buffer)
{
    if ( Buffer )
    {
        if (Buffer->memType == MEM_TYPE_STATIC)
        {
            Buffer->AllocatorContext = NULL;
            PUSH_ENTRY_LIST( &(AtmUni->TxBufferFreeList),(PSINGLE_LIST_NODE)Buffer);
        }
        else if (Buffer->memType == MEM_TYPE_DYNAMIC)
        {
            FREE_MEMORY( Buffer->Data,AtmUni->TxBufferSize);
            FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
        }
        else
        {
            ASSERT(FALSE);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniGetTransmitBuffer
//
//  ABSTRACT:
//		get a transmit buffer for processing
//
//  RETURNS:
//		successfully allocated / retrieved buffer or NULL
///////////////////////////////////////////////////////////////////////////////
PXFER_DATA_BUFFER AtmUniGetTransmitBuffer( PATM_UNI AtmUni)
{
    PXFER_DATA_BUFFER Buffer = NULL;

	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
    if ( AtmUni->AtmUniState & ATM_UNI_STATE_DONT_ACCEPT )
    {
        Buffer = NULL;
    }
    else
    {
        Buffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &AtmUni->TxBufferFreeList);

        if ( !Buffer )
        {
		    ALLOCATE_MEMORY( &Buffer, sizeof( XFER_DATA_BUFFER));

		    if( Buffer ) 
            {
                Buffer->AllocatorContext = NULL;
                Buffer->memType = MEM_TYPE_DYNAMIC;

		        ALLOCATE_MEMORY( &Buffer->Data, AtmUni->TxBufferSize);

		        if( NULL == Buffer->Data) 
                {
			        FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
                    Buffer = NULL;
		        }
            }
        }
    }
    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

    return Buffer;
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniQueueTransmitBuffer
//
//  ABSTRACT:
//		queues a transmit buffer for processing
//
//  RETURNS:
//		TRUE if successfully queued, else FALSE
///////////////////////////////////////////////////////////////////////////////
BOOLEAN AtmUniQueueTransmitBuffer( PATM_UNI AtmUni, PXFER_DATA_BUFFER Buffer)
{
    PPROCESS_DATA_BUFFER    ProcessBuffer;


    NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
    if ( AtmUni->AtmUniState & ATM_UNI_STATE_DONT_ACCEPT )
    {
        NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        return FALSE;
    }
    else if ( !IS_LIST_EMPTY( &AtmUni->ProcessBufferFreeList) )
    {
        ProcessBuffer = (PPROCESS_DATA_BUFFER)REMOVE_TAIL_LIST( &AtmUni->ProcessBufferFreeList);
    }
    else
    {
        ALLOCATE_MEMORY( &ProcessBuffer,sizeof( PROCESS_DATA_BUFFER));

        if ( ProcessBuffer )
            ProcessBuffer->memType = MEM_TYPE_DYNAMIC;
    }

    if ( ProcessBuffer )
    {
        ProcessBuffer->Buffer = Buffer;
        INSERT_HEAD_LIST( &(AtmUni->ProcessBufferTxList),(PLIST_NODE)ProcessBuffer);

        // run queue processing routine if not currently running
        if ( AtmUni->NeedQueueAtmProcess )
        {
	        AtmUni->NeedQueueAtmProcess = FALSE;
	        NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
	        AtmProcessFunction( AtmUni);
        }
        else
        {
            NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        }
    }
    else
    {
		NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
		return FALSE;
    }

    return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmUniProcessBuffers
//
//  ABSTRACT:
//		process transmit buffers if needed
//
//  RETURNS:
//		void
///////////////////////////////////////////////////////////////////////////////
void AtmUniProcessBuffers( PATM_UNI AtmUni)
{
    NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);

    // run queue processing routine if not currently running
    if ( AtmUni->NeedQueueAtmProcess )
    {
	    AtmUni->NeedQueueAtmProcess = FALSE;
	    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
	    AtmProcessFunction( AtmUni);
    }
    else
	    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
}


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniDestroy
//
//  ABSTRACT:
//		Destroys a previously created ATM UNI
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniDestroy( ATM_UNI_HANDLE AtmUniHandle)
{
	PATM_UNI	AtmUni;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniDestroy\n"));

	if ( NULL == AtmUniHandle) {
		return;
	}

	AtmUni = AtmUniHandle;

    AtmUni->AtmUniState |= ATM_UNI_STATE_HALTING;

    if ( AtmUni->AtmUniState & ATM_UNI_STATE_INITIALIZED )
    {
	    // Cancel the DSP Timer
        AtmUniShutdownDspStatus( AtmUni);

#ifdef MESSAGE_HANDLING
	    MsgHandling_Stop( AtmUni);
#endif // MESSAGE_HANDLING

#ifdef USB_WIN_AUTO_CONFIG
        Shutdown_Ac_Layer ( AtmUni->pAutoConfig);
#endif // USB_WIN_AUTO_CONFIG

        AtmUniShutdownTransmit( AtmUni);

	    // If we've previously bound with the PHY layer, remove our binding
	    if( AtmUni->PhyHandle)
            AtmUniClosePhyBinding( AtmUni);

	    AtmUniDeleteAllVcs( AtmUni);
	    AtmUniFreeBuffers( AtmUni);

#ifdef MESSAGE_HANDLING
        NdisFreeSpinLock( &AtmUni->MsgHandlingLock);
#endif // MESSAGE_HANDLING

#ifdef USB_WIN_AUTO_CONFIG
        Remove_Ac_Layer ( AtmUni->pAutoConfig);
#endif // USB_WIN_AUTO_CONFIG

        NdisFreeSpinLock( &AtmUni->DspSpinLock);
	    NdisFreeSpinLock( &AtmUni->VcListLock);
	    NdisFreeSpinLock( &AtmUni->RxBufferFreeListLock);
	    NdisFreeSpinLock( &AtmUni->TxBufferFreeListLock);

	    FREE_MEMORY( AtmUni,sizeof( ATM_UNI));
    }
} // AtmUniDestroy


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniCreateVc
//
//  ABSTRACT:
//		Creates a new VC.  Note that the operational parameters for the VC
//		are specified during AtmUniActivateVc.
//
//  RETURNS:
//		ATM_UNI_HANDLE or NULL if the request failed
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_HANDLE
AtmUniCreateVc(
	ATM_UNI_HANDLE					AtmUniHandle,
   	ATM_UNI_USER_HANDLE     		UserVcHandle,
  	ATM_UNI_USER_RECEIVE      		UserReceive,
	ATM_UNI_USER_RECEIVE_PACKET		UserReceivePacket,
   	ATM_UNI_USER_SEND_COMPLETE		UserSendComplete
)
{
	PATM_UNI	AtmUni;
	PATM_UNI_VC	AtmUniVc;
    NDIS_STATUS  Status;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniCreateVc\n"));

	ASSERT( AtmUniHandle);

	AtmUni = AtmUniHandle;

	ALLOCATE_MEMORY( &AtmUniVc, sizeof( ATM_UNI_VC));
	if( NULL == AtmUniVc) {
	  return( NULL);
	}

	//
	// Allocate a spin lock for protecting VC resources.  Note that the VC
	// lock is NOT used to protect the VC 'Link' field; rather, the VcListLock
	// is used for this purpose.
	//
	NdisAllocateSpinLock( &AtmUniVc->Lock);

	AtmUniVc->State				= ATM_UNI_VC_STATE_INACTIVE;
	AtmUniVc->AtmUni 			= AtmUni;
	AtmUniVc->UserVcHandle		= UserVcHandle;
	AtmUniVc->UserSendComplete	= UserSendComplete;
    AtmUni->UserSendComplete    = UserSendComplete;
	//
	// If the ATM UNI was created with the IndicatePackets option set to TRUE,
	// the VC owner must supply the UserReceivePacket routine.  Otherwise, the
	// UserReceive routine must be provided.
	//
	if ( AtmUni->IndicatePackets) {
		ASSERT( UserReceivePacket);
		AtmUniVc->UserReceivePacket	= UserReceivePacket;
	} else {
		AtmUniVc->UserReceive = UserReceive;
	}

	//
	// The VC will be placed on the inactive VC list until it is activated
	//   (when it will move to the VC hash table)
	//
    NdisAllocateBufferPool (
       &Status,
       &AtmUniVc->ShortPacketsBufferPool,
       ATM_UNI_SHORT_PACKETS_POOL_SIZE ); 

    if(Status != NDIS_STATUS_SUCCESS)
    {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("NdisAllocateBufferPool fail\n"));
		return( NULL);
    
    }

    
    NdisAcquireSpinLock( &AtmUni->VcListLock);
	INSERT_HEAD_LIST( &(AtmUni->InactiveVcList),(PLIST_NODE)AtmUniVc);
    NdisPacketAddVcToTable( AtmUni, AtmUniVc);
	NdisReleaseSpinLock( &AtmUni->VcListLock);

	return( AtmUniVc);
} // AtmUniCreateVc

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniDeleteVc
//
//  ABSTRACT:
//		Deletes a previously created VC
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		The VC is currently in the INACTIVE state
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniDeleteVc(	ATM_UNI_HANDLE	AtmUniVcHandle)
{
	PATM_UNI_VC	AtmUniVc;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniDeleteVc\n"));

	ASSERT( AtmUniVcHandle);

	AtmUniVc = AtmUniVcHandle;

	ASSERT( AtmUniVc->State == ATM_UNI_VC_STATE_INACTIVE);

	if ( AtmUniVc->State != ATM_UNI_VC_STATE_INACTIVE) {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Attempt to delete ACTIVE VC\n"));
			//
			// Deactivate the VC so it can be deleted
			//
			AtmUniDeactivateVc( AtmUniVcHandle);
	}

	//
	// Remove the VC from the inactive list
	//
	NdisAcquireSpinLock( &(AtmUniVc->AtmUni->VcListLock));
   	REMOVE_ENTRY_LIST( (PLIST_NODE)AtmUniVc);
    NdisPacketRemoveVcFromTable( AtmUniVc->AtmUni, AtmUniVc);
	NdisReleaseSpinLock( &(AtmUniVc->AtmUni->VcListLock));

	NdisAcquireSpinLock( &(AtmUniVc->Lock));
    NdisFreeBufferPool(AtmUniVc->ShortPacketsBufferPool);
	NdisReleaseSpinLock( &(AtmUniVc->Lock));

	NdisFreeSpinLock( &AtmUniVc->Lock);
    FREE_MEMORY( AtmUniVc, sizeof( ATM_UNI_VC));

} // AtmUniDeleteVc

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniActivateVc
//
//  ABSTRACT:
//		Activates a previously created VC.  May be called multiple times to
//		change the operational parameters of a VC.  If ReserveResources is
//		set to TRUE, the activation request is processed and resources are
//		reserved but the VC is not placed into the active state.
//
//  RETURNS:
//		ATM_UNI_STATUS
//
//  ASSUMPTIONS:
//		The VC was successfully created.
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_STATUS
AtmUniActivateVc(
   ATM_UNI_HANDLE	AtmUniVcHandle,
   UCHAR       		Vpi,
   USHORT      		Vci,
   ATM_UNI_AAL 		Aal,
   USHORT      		MaxSduSize,
   PUCHAR     		VcEncapsulation,
   USHORT      		VcEncapsulationLength,
   BOOLEAN			ReserveResources,
   UCHAR            Gfc
)
{
	PATM_UNI_VC	AtmUniVc;
	PATM_UNI_VC ExistingVc;
	ULONG       VpiVci = SET_ATM_VPIVCI( Vpi, Vci);
	PULONG     	CellHeaderPtr;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniActivateVc\n"));

	ASSERT( AtmUniVcHandle);

	AtmUniVc = AtmUniVcHandle;

	//
	//	If the VC is in the ACTIVE or RESOURCES_RESERVED states, check the
	//	new VC parameters against those provided previously and make any
	//	required adjustments.
	//
	if ( (AtmUniVc->State == ATM_UNI_VC_STATE_ACTIVE) ||
	     (AtmUniVc->State == ATM_UNI_VC_STATE_RESOURCES_RESERVED))
    {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_TRACE,
			("Reactivate VC request\n"));

		//
		// If the VPI/VCI or the AAL have changed, we need to deactivate
		// and reactivate the VC
		//
		if ( ( AtmUniVc->VpiVci != VpiVci) || ( Aal != AtmUniVc->Aal))
		{
			AtmUniDeactivateVc( AtmUniVcHandle);
		} 
		else 
		{
			//
			// If the VC encapsulation has changed, save the new encapsulation
			//
			if ( NULL != VcEncapsulation) {
				NdisAcquireSpinLock( &(AtmUniVc->Lock));
				if ( VcEncapsulationLength != AtmUniVc->VcEncapsulationLength) {
					FREE_MEMORY(
						AtmUniVc->VcEncapsulation,
						VcEncapsulationLength);
					ALLOCATE_MEMORY(
						&(AtmUniVc->VcEncapsulation),
						VcEncapsulationLength);
					if( NULL == AtmUniVc->VcEncapsulation) {
						NdisReleaseSpinLock( &(AtmUniVc->Lock));
						return( ATM_UNI_STATUS_RESOURCES);
					}
				}
				COPY_MEMORY(
					AtmUniVc->VcEncapsulation,
					VcEncapsulation,
					VcEncapsulationLength);
				AtmUniVc->VcEncapsulationLength = VcEncapsulationLength;
				NdisReleaseSpinLock( &(AtmUniVc->Lock));
			}
			//
			// If the max SDU size is acceptable, complete the activation.
			// Otherwise, indicate invalid data.
			//
			if ( MaxSduSize <= AtmUniVc->AtmUni->MaxSduSize) {
				if ( ReserveResources) {
					AtmUniVc->State = ATM_UNI_VC_STATE_RESOURCES_RESERVED;
				} else {
					AtmUniVc->State = ATM_UNI_VC_STATE_ACTIVE;
				}
				return( ATM_UNI_STATUS_SUCCESS);
			} else {
				DBGPRINT(
					DBG_PRINT_MODULE_ATMUNI_ALL,
					DBG_PRINT_LEVEL_ERR,
					("MaxSduSize too large\n"));
				return( ATM_UNI_STATUS_INVALID_DATA);
			}
		}
	}

	//
	// Check for a duplicate VPI/VCI
	//
   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
      (":::::VpiVci := 0x%X\n",VpiVci));

	ExistingVc = AtmUniLookupVc( AtmUniVc->AtmUni, VpiVci);
	if( ExistingVc) {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_TRACE,
			("Duplicate VPI/VCI (VpiVci = 0x%X)\n",VpiVci));
		return( ATM_UNI_STATUS_DUPLICATE_VPI_VCI);
	}

	//
	// Check for a valid Max SDU size
	//
	if( MaxSduSize > AtmUniVc->AtmUni->MaxSduSize ) {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("MaxSduSize too large\n"));
		return( ATM_UNI_STATUS_INVALID_DATA);
	}

	//
	// Check for a supported AAL (currently, only AAL5 is supported)
	//
	if ( Aal != AAL5_ ) {
		return( ATM_UNI_STATUS_INVALID_DATA);
	}

	//
	// Activate the VC
	//
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("VPI:VCI = 0x%X, Max Sdu = %d\n",
		VpiVci,MaxSduSize));

	AtmUniVc->VpiVci       = VpiVci;
	AtmUniVc->Aal          = Aal;
	AtmUniVc->MaxSduSize   = MaxSduSize;

	//
	// Create the cell header template for this VC
	//
	CellHeaderPtr = (ULONG *)AtmUniVc->CellHeader;
	AtmUniVc->CellHeader[0] = (UCHAR)( (Gfc << 4) | ((Vpi & 0xF0) >> 4));
	AtmUniVc->CellHeader[1] = (UCHAR)( (Vpi << 4) | ((Vci & 0xF000) >> 12));
	AtmUniVc->CellHeader[2] = (UCHAR)( (Vci & 0x0FF0) >> 4);
	AtmUniVc->CellHeader[3] = (UCHAR)( (Vci & 0x000F) << 4);
	AtmUniVc->CellHeader[4] = 0;

	if( NULL != VcEncapsulation ) {
		NdisAcquireSpinLock( &AtmUniVc->Lock);
		ALLOCATE_MEMORY( &(AtmUniVc->VcEncapsulation),VcEncapsulationLength);
		if( NULL == AtmUniVc->VcEncapsulation) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("VC encapsulation alloc failed\n"));
			NdisReleaseSpinLock( &AtmUniVc->Lock);
			return( ATM_UNI_STATUS_RESOURCES);
		}
		COPY_MEMORY(
			AtmUniVc->VcEncapsulation,
			VcEncapsulation,
			VcEncapsulationLength);
		AtmUniVc->VcEncapsulationLength = VcEncapsulationLength;
		NdisReleaseSpinLock( &(AtmUniVc->Lock));
	}

	AtmUniVc->State = ATM_UNI_VC_STATE_ACTIVE;

	//
	// Remove the VC from the inactive list and add it in the VC hash
	//   table
	//
	NdisAcquireSpinLock( &(AtmUniVc->AtmUni->VcListLock));
		REMOVE_ENTRY_LIST( (PLIST_NODE)AtmUniVc);
	AtmUniAddVcToTable( AtmUniVc->AtmUni, AtmUniVc);
	NdisReleaseSpinLock( &(AtmUniVc->AtmUni->VcListLock));

	return( ATM_UNI_STATUS_SUCCESS);

} // AtmUniActivateVc

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniDeactivateVc
//
//  ABSTRACT:
//		Deactivates a previously activated VC.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		The VC is assumed to be in the ACTIVE or RESOURCES_RESERVED states
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniDeactivateVc( ATM_UNI_HANDLE AtmUniVcHandle)
{
	PATM_UNI_VC	AtmUniVc;
	PATM_UNI	AtmUni;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniDeactivateVc\n"));

	ASSERT( AtmUniVcHandle);

	AtmUniVc = AtmUniVcHandle;

	ASSERT( AtmUniVc->State != ATM_UNI_VC_STATE_INACTIVE);

	//
	// If the VC is in the INACTIVE state, return immediately
	//
	if ( AtmUniVc->State == ATM_UNI_VC_STATE_INACTIVE) {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Attempt to deactivate INACTIVE VC\n"));
		return;
	}

	AtmUni = AtmUniVc->AtmUni;

    NdisAcquireSpinLock( &AtmUniVc->Lock);

	//
	// If there is a reassembly in progress on the VC, free the Rx buffer
	//
	if ( AtmUniVc->CurrentRxBuffer ) {
		NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
	    PUSH_ENTRY_LIST(
	    	&(AtmUni->RxBufferFreeList),
	    	(PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
		NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
	}

	//
	// Change the VC State to INACTIVE and free the VC Encapsulation
	//
	AtmUniVc->State = ATM_UNI_VC_STATE_INACTIVE;
	if( AtmUniVc->VcEncapsulation) {
		FREE_MEMORY( AtmUniVc->VcEncapsulation,AtmUniVc->VcEncapsulationLength);
	}

	//
	// Remove the VC from the VC hash table and add it to the inactive list
	//
    NdisAcquireSpinLock( &AtmUni->VcListLock);
	AtmUniRemoveVcFromTable( AtmUniVc->AtmUni,AtmUniVc);
	INSERT_HEAD_LIST( &(AtmUniVc->AtmUni->InactiveVcList),(PLIST_NODE)AtmUniVc);
	NdisReleaseSpinLock( &AtmUni->VcListLock);

    NdisReleaseSpinLock( &AtmUniVc->Lock);

} // AtmUniDeactivateVc


///////////////////////////////////////////////////////////////////////////////
//  NAME: NdisPacketAddVcToTable
//
//  ABSTRACT: Add a VC to our NDIS PACKET lookup array
//
//  RETURNS: void
//
//  ASSUMPTIONS: This function should be called from within a spinlock
///////////////////////////////////////////////////////////////////////////////
void NdisPacketAddVcToTable( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc)
{
#if 0 //KJP FIXME
    UCHAR i;

    for (i=0; i<128; i++)
    {
        if (AtmUni->AtmUniVc[i] == NULL)
        {
            AtmUni->AtmUniVc[i] = AtmUniVc;
            return;
        }
    }
#endif //KJP FIXME
}


///////////////////////////////////////////////////////////////////////////////
//  NAME: NdisPacketRemoveVcFromTable
//
//  ABSTRACT: Remove a VC from our NDIS PACKET lookup array
//
//  RETURNS: void
//
//  ASSUMPTIONS: This function should be called from within a spinlock
///////////////////////////////////////////////////////////////////////////////
void NdisPacketRemoveVcFromTable( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc)
{
#if 0 //KJP FIXME
    UCHAR i;

    for (i=0; i<128; i++)
    {
        if (AtmUni->AtmUniVc[i] == AtmUniVc)
        {
            AtmUni->AtmUniVc[i] = NULL;
            return;
        }
    }
#endif //KJP FIXME
}

///////////////////////////////////////////////////////////////////////////////
//  NAME: NdisPacketLookupVc
//
//  ABSTRACT: Lookup a VC in our NDIS PACKET lookup array
//
//  RETURNS: TRUE if VC found else FALSE, index of VC if found
///////////////////////////////////////////////////////////////////////////////
BOOLEAN NdisPacketLookupVc( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc, PUCHAR Index)
{
#if 0 //KJP FIXME
    UCHAR i;

    NdisAcquireSpinLock( &AtmUni->VcListLock);
	
    for (i=0; i<128; i++)
    {
        if (AtmUni->AtmUniVc[i] == AtmUniVc)
        {
            *Index = i;

            NdisReleaseSpinLock( &AtmUni->VcListLock);
            return TRUE;
        }
    }

    NdisReleaseSpinLock( &AtmUni->VcListLock);
#endif //KJP FIXME
    return FALSE;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniShortPacketPaddingAdd
//
//  ABSTRACT:
//		Adds Padding to the packet less then PADDING_LEN_MAX
//
//  RETURNS:
//         TRUE  if padding was added
//         FALSE if zero padding added
//
//  ASSUMPTIONS:
//		The VC must be in the ACTIVE state prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN AtmUniShortPacketPaddingAdd( ATM_UNI_HANDLE AtmHandle, PNDIS_PACKET Packet,ULONG ulSize)
{
#ifdef LAN_MINIPORT
	ULONG ulPadding = 0; 

	PNDIS_BUFFER    NdisBuff;
	NDIS_STATUS     Status;  
	PATM_UNI_VC AtmUniVc = (PATM_UNI_VC)AtmHandle; 

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_TX,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniShortPacketPaddingAdd\n"));

	ASSERT( Packet);
	ASSERT( AtmHandle);

    if((ulSize  < PADDING_LEN_MAX) && ulSize )
    	ulPadding = PADDING_LEN_MAX - ulSize;

   /* Do not increment the bufferCount, if no padding */
   if (ulPadding == 0)
       return FALSE;
       	
	NdisAllocateBuffer(
		&Status,
		&NdisBuff,
		AtmUniVc->ShortPacketsBufferPool,
		EthPadding,
		ulPadding);

	if(NDIS_STATUS_SUCCESS != Status)
	{
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_TX,
			DBG_PRINT_LEVEL_ERR,
			("NdisAllocateBuffer fail \n"));

		return FALSE;
	}

	NdisChainBufferAtBack (Packet, NdisBuff);
    
	return TRUE;
#endif //LAN_MINIPORT 

	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniShortPacketPaddingRemove
//
//  ABSTRACT:
//		Removes Padding from the packet
//
//  RETURNS:
//         TRUE  if padding was added
//
//  ASSUMPTIONS:
//		The VC must be in the ACTIVE state prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN AtmUniShortPacketPaddingRemove( ATM_UNI_HANDLE AtmHandle, PNDIS_PACKET Packet)
{
#ifdef LAN_MINIPORT

    PNDIS_BUFFER    NdisBuff;

	//DBGPRINT( DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_TRACE,
	//	("AtmUniSend\n"));

	ASSERT( Packet);
	ASSERT( AtmHandle);
	
	NdisUnchainBufferAtBack (Packet, &NdisBuff);
	if(NdisBuff)
		NdisFreeBuffer(NdisBuff);
           
#endif// LAN_MINIPORT
return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniEmptySendPacketQueue
//
//  ABSTRACT:
//
//  RETURNS:
//
///////////////////////////////////////////////////////////////////////////////
void AtmUniEmptySendPacketQueue(PATM_UNI AtmUni)
{
    PNDIS_PACKET           Packet;

    NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
    /* Empty the linked list of NDIS_PACKETs */
    Packet = (PNDIS_PACKET)REMOVE_NDIS_PACKET(&(AtmUni->NdisPacketQueue));
    while (Packet)
    {
        /* The layer is down and therefore it makes sense to notify
        ** NDIS on the status of the packet
        */
        (AtmUni->UserSendComplete)(AtmUni->UserHandle, NULL, Packet);

        Packet = (PNDIS_PACKET)REMOVE_NDIS_PACKET(&(AtmUni->NdisPacketQueue));
    }
    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
}

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniSend
//
//  ABSTRACT:
//		Sends an NDIS_PACKET on the given VC
//
//  RETURNS:
//         ATM_UNI_STATUS_SUCCESS
//         ATM_UNI_STATUS_FAILURE
//
//  ASSUMPTIONS:
//		The VC must be in the ACTIVE state prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_STATUS AtmUniSend(
	ATM_UNI_HANDLE		AtmUniVcHandle,
	PNDIS_PACKET 		Packet,
	BOOLEAN				SkipFirstBuffer,
	UCHAR				CpcsUu,
	UCHAR				Cpi,
	BOOLEAN				Clp,
    BOOLEAN				ArpFlag
)
{
	PATM_UNI_VC	  		    AtmUniVc;
    PATM_UNI                AtmUni;
    PNDIS_PACKET_DESCRIPTOR pktDescriptorPtr;
    PXFER_DATA_BUFFER       TxBuffer;
    UCHAR                   Index=0;

    DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_TRACE,	("AtmUniSend\n"));

	ASSERT( Packet);
	ASSERT( AtmUniVcHandle);

	AtmUniVc = AtmUniVcHandle;
	AtmUni = AtmUniVc->AtmUni;
	
	// If the VC is not in the ACTIVE state, return failure
	if( AtmUniVc->State != ATM_UNI_VC_STATE_ACTIVE)
	{
	  	DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
			("Attempt to send on non-active VC\n"));

		return ATM_UNI_STATUS_SUCCESS;
	}
#if 0 //KJP FIXME   
    if ( !NdisPacketLookupVc( AtmUni, AtmUniVc, &Index) )
    {
        DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
			("Error:  could not find VC in NDIS PACKET lookup array\n"));

		return ATM_UNI_STATUS_SUCCESS;
    }
#endif //KJP FIXME   
    /* Queue the buffer for later processing */
    pktDescriptorPtr = NDIS_PACKET_RESERVED_FIELD(Packet);
    pktDescriptorPtr->Flags = 0;

    if (ArpFlag)
       pktDescriptorPtr->Flags |= BIT_MASK_ARPFLAG;

    if (SkipFirstBuffer)
       pktDescriptorPtr->Flags |= BIT_MASK_SKIPFIRSTBUFFER;

    pktDescriptorPtr->CpcsUu = CpcsUu;
    pktDescriptorPtr->Cpi = Cpi;

    NDIS_PACKET_SET_CLP(Clp, pktDescriptorPtr->Clp);
    NDIS_PACKET_SET_INDEX(Index, pktDescriptorPtr->Clp);

    /* Check if there is any buffers avialable */
    NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);

//KJP FIXME
    /* FIXME: temp fix to work around getting buffers with loopback / auto-config
    /  PVCs which would then skip dequeueing NDIS PACKETs.  We always want to try
    /  and grab the next NDIS PACKET.  We should really save the PVC pointer for
    /  every NDIS PACKET */
    AtmUni->AtmUniVc = AtmUniVc;
//KJP FIXME

    if ( AtmUni->AtmUniState & ATM_UNI_STATE_DONT_ACCEPT )
    {
	    DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
		    ("dont send data since driver is shutting down\n"));

        NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        return ATM_UNI_STATUS_SUCCESS;
    }
    else
    {
        TxBuffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &AtmUni->TxBufferFreeList);

        if ( NULL == TxBuffer)
        {
            DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
			    ("No Tx Buffer available for Send\n"));

            INSERT_NDIS_PACKET(&AtmUni->NdisPacketQueue, Packet);
            NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

            AtmUniProcessBuffers( AtmUni);  // Make sure that buffers are being dequeued
            return ATM_UNI_STATUS_PENDING;
        }
    }

    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

    /* Process the NdisPacket */
    if ( ATM_UNI_STATUS_SUCCESS != (AtmUniTxProcess(AtmUniVc, Packet, TxBuffer)) )
    {
        NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
		NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);    
    }

    return ATM_UNI_STATUS_SUCCESS;
}

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniTxProcess
//
//  ABSTRACT:
//		Sends an NDIS_PACKET on the given VC
//
//  RETURNS:
//         ATM_UNI_STATUS_SUCCESS
//         ATM_UNI_STATUS_FAILURE
//
//  ASSUMPTIONS:
//		The VC must be in the ACTIVE state prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_STATUS AtmUniTxProcess(
    PATM_UNI_VC         AtmUniVc,
    PNDIS_PACKET        Packet,
    PXFER_DATA_BUFFER   TxBuffer
)
{
    PATM_UNI	  		AtmUni;
    UCHAR               ArpFlag = 0;
    UCHAR               SkipFirstBuffer = 0;
    UCHAR               CpcsUu;
    UCHAR               Cpi;
    BOOLEAN             Clp;
    PNDIS_PACKET_DESCRIPTOR pktDescriptorPtr;

	UINT          		BufferCount;
	PNDIS_BUFFER  		NdisBuffer;
	PNDIS_BUFFER        FirstNdisBuffer;
	PUCHAR	      		OutputBuffer;
	PUCHAR	      		InputBuffer;
	UINT         		InputLength;
	USHORT        		BytesWritten;
	UINT          		i;
	ULONG         		Aal5PduLength, PduLength;
	BOOLEAN				NeedPadding;
 
	//DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX,DBG_PRINT_LEVEL_TRACE,("AtmUniTxProcess\n"));

    ASSERT(AtmUniVc);

    /* In case we called with AtmUniVc not valid */
    if (AtmUniVc == NULL)
	{
		return ATM_UNI_STATUS_FAILURE;
	}
    AtmUni = (PATM_UNI)AtmUniVc->AtmUni;

    TxBuffer->AllocatorContext = AtmUniVc;
    pktDescriptorPtr = NDIS_PACKET_RESERVED_FIELD(Packet);

    ArpFlag =           (pktDescriptorPtr->Flags & BIT_MASK_ARPFLAG);
    SkipFirstBuffer =   (pktDescriptorPtr->Flags & BIT_MASK_SKIPFIRSTBUFFER);
    CpcsUu =            pktDescriptorPtr->CpcsUu;
    Cpi =               pktDescriptorPtr->Cpi;
    Clp =               NDIS_PACKET_GET_CLP(pktDescriptorPtr->Clp);

	// Retreive Packet information
	NdisRecalculatePacketCounts( Packet);
	NdisQueryPacket(
	  Packet,
	  NULL,
	  &BufferCount,
	  &NdisBuffer,
	  NULL);

	// Verify that the Packet contains data
	if( !NdisBuffer || !BufferCount ) 
	{
	  	DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,("Invalid Packet\n"));

		AtmUni->Stats.TxPdusError++;
		AtmUniVc->Stats.TxPdusError++;
       
       AtmUni->AtmCounters.TxPacketError++;
       AtmUniVc->AtmCounters.TxPacketError++;
        return ATM_UNI_STATUS_FAILURE;
	}

	OutputBuffer = TxBuffer->Data;
	TxBuffer->IsLoopback = FALSE;

	// Initialize per PDU VC parameters
	NdisAcquireSpinLock( &AtmUniVc->Lock);

	// Store the current CPCS UU and CPI in the VC info and update
	// the cell header with the CLP value.
	//
	AtmUniVc->CpcsUu = CpcsUu;
	AtmUniVc->Cpi	 = Cpi;

	if ( Clp )
		AtmUniVc->CellHeader[ATM_CELL_HEADER_CLP_OFFSET] |= ATM_CELL_SET_CLP_MASK;
	else
		AtmUniVc->CellHeader[ATM_CELL_HEADER_CLP_OFFSET] &= ATM_CELL_CLEAR_CLP_MASK;

	AtmUniVc->TxPartialCrc    = CRC_INITIAL_VALUE;
	AtmUniVc->TxCellRemainder = 0;

	Aal5PduLength = 0;
	BytesWritten  = 0;

    /* FIXME: Check to see, if all of the buffers in the NDIS_PACKETS are empty.
    ** if so, do not process the packet and simply return.
    */
	// Before segmenting the packet, segment the VC encapsulation if present
	if(ArpFlag  && AtmUniVc->ArpEncapsulation)
	{	
		BytesWritten += AtmUniAal5Segment(
				AtmUniVc,
				&OutputBuffer,
				AtmUniVc->ArpEncapsulation,
				AtmUniVc->ArpEncapsulationLength,
				FALSE,
				0);
		Aal5PduLength += AtmUniVc->ArpEncapsulationLength;
        TxBuffer->ArpFlag = TRUE;  
	}             
    else if( (!ArpFlag ) && (AtmUniVc->VcEncapsulation) ) 
    {
		BytesWritten +=	AtmUniAal5Segment(
				AtmUniVc,
				&OutputBuffer,
				AtmUniVc->VcEncapsulation,
				AtmUniVc->VcEncapsulationLength,
				FALSE,
				0);
		Aal5PduLength += AtmUniVc->VcEncapsulationLength;
        TxBuffer->ArpFlag = FALSE;
	}
    else
    {
		TxBuffer->ArpFlag = FALSE;
    }

	// We are seeing packets with BufferCount off by one.
	// Verify the buffer count against the Packet.
	//
    PduLength = Aal5PduLength; //Encapsulation Added at this point 

    FirstNdisBuffer = NdisBuffer;
	for( i = 0; i < BufferCount; i++ ) 
    {
		NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &InputLength);
		if( i == (BufferCount-1))
		{
			if((NULL == NdisBuffer) || (NULL == InputBuffer) || (0 == InputLength))
			{
                BufferCount--;
			}
		}
		PduLength += InputLength;
        NdisGetNextBuffer( NdisBuffer, &NdisBuffer);
	}

    if(SkipFirstBuffer )
        PduLength -= ETHERNET_HEADER_SIZE;

    PduLength  -= AtmUniVc->VcEncapsulationLength;
	
    NeedPadding = AtmUniShortPacketPaddingAdd( AtmUniVc, Packet, PduLength);
	if (NeedPadding)
		BufferCount++;

	NdisBuffer = FirstNdisBuffer;
	// Segment each of the buffers in the Packet
	for( i = 0; i < BufferCount; i++ ) 
    {
		NdisQueryBuffer( NdisBuffer, (PVOID *)&InputBuffer, &InputLength);

		if( !InputBuffer || !InputLength )	// Skip invalid buffers 
		{
			DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX,DBG_PRINT_LEVEL_ERR,
				("Skipping invalid NDIS buffer\n"));
           /* FIXME: The last buffer to be NULL or zero length
           ** and if so, we need to indicate EndOfPdu to our current segment
           ** buffer and send it out. If not, it will fail and never send this
           ** buffer out.
           */
#ifdef LAN_MINIPORT
           /* BUG_FIX (FIXME): In Win2000 WHQL (2m_strc) test, placing the padding at the
           ** end of the chain (AtmUniShortPacketPadding routine) cannot be read
           ** correctly when NdisQueryBuffer is called. Due to this, we will end-up
           ** here and buffer fail because of no EndOfPdu.
           ** We need to obtain errata for this from Microsoft because it only happend
           ** when running the WHQL test. Otherwise it works fine.
           ** I tested without the spinlock and recalulating the packet and it did not help.
           */
           if ( (i == (BufferCount-1)) && (NeedPadding) )
           {
				Aal5PduLength += InputLength;
				BytesWritten += AtmUniAal5Segment(
									AtmUniVc,
									&OutputBuffer,
									EthPadding,
									(USHORT)(PADDING_LEN_MAX - PduLength),
									(BOOLEAN)TRUE,
									(USHORT)Aal5PduLength);
           }
#endif /* LAN _MINIPORT */
		}
		else
		{
			if ( SkipFirstBuffer && (i == 0) )	// Skip the first buffer if requested
			{
				// For RFC1483 ROUTED mode, we are required to jump length of EthernetHeader.
				if (InputLength > ETHERNET_HEADER_SIZE)
				{
					Aal5PduLength += InputLength - ETHERNET_HEADER_SIZE;
					BytesWritten += AtmUniAal5Segment(
										AtmUniVc,
										&OutputBuffer,
										(InputBuffer + ETHERNET_HEADER_SIZE),
										(USHORT)(InputLength - ETHERNET_HEADER_SIZE),
										(BOOLEAN)( i == (BufferCount-1) ),
										(USHORT)Aal5PduLength);
				}
			}
			else
			{
				Aal5PduLength += InputLength;
				BytesWritten += AtmUniAal5Segment(
									AtmUniVc,
									&OutputBuffer,
									InputBuffer,
									(USHORT)InputLength,
									(BOOLEAN)( i == (BufferCount-1) ),
									(USHORT)Aal5PduLength);
			}
		}

		NdisGetNextBuffer( NdisBuffer, &NdisBuffer);
	}

	if (NeedPadding)
		AtmUniShortPacketPaddingRemove( AtmUniVc, Packet);

	NdisReleaseSpinLock( &AtmUniVc->Lock);

	// Make sure the packet size and total data size are correct
	if ( (Aal5PduLength > AtmUniVc->MaxSduSize) || (!Aal5PduLength) 
		 || (BytesWritten % (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING)) )
	{
	  	DBGPRINT( DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
			("Invalid packet side (Data = %d) and (Total = %d)\n", Aal5PduLength, BytesWritten));

		AtmUni->Stats.TxPdusError++;
		AtmUniVc->Stats.TxPdusError++;

       AtmUni->AtmCounters.TxPacketError++;
       AtmUniVc->AtmCounters.TxPacketError++;
		return ATM_UNI_STATUS_FAILURE;
	}

  	//DBGPRINT( DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_TRACE,
	//	("Sending %d bytes\n", BytesWritten));
	
	TxBuffer->Size = BytesWritten;

	// Submit the data buffer to transmit list USB for transmission
    if (AtmUniQueueTransmitBuffer( AtmUni, TxBuffer))
	{
        return ATM_UNI_STATUS_SUCCESS;
	}
    else
	{
        return ATM_UNI_STATUS_FAILURE;
	}
}

#ifdef USB_WIN_AUTO_CONFIG
///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AcAtmUniSend
//
//  ABSTRACT:
//		Sends a pdu on the given VC
//
//  RETURNS:
//         ATM_UNI_STATUS_SUCCESS
//         ATM_UNI_STATUS_FAILURE
//
//  ASSUMPTIONS:
//		The VC must be in the ACTIVE state prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
void
AcAtmUniSend(
	PATM_UNI_VC         AtmUniVc,
	PVOID        		PduData,
    USHORT              PduLength,
	UCHAR				CpcsUu,
	UCHAR				Cpi
)
{
	PXFER_DATA_BUFFER  		TxBuffer;
	PUCHAR	      			OutputBuffer;
	PATM_UNI				AtmUni;
 
	DBGPRINT(
		DBG_PRINT_MODULE_AUTO_CONFIG,
		DBG_PRINT_LEVEL_TRACE,
		("AcAtmUniSend\n"));


    if ( AtmUniVc )
	    AtmUni = AtmUniVc->AtmUni;
    else
        return;
	
	if( AtmUniVc->State != ATM_UNI_VC_STATE_ACTIVE) 
    {
	  	DBGPRINT(
			DBG_PRINT_MODULE_AUTO_CONFIG,
			DBG_PRINT_LEVEL_ERR,
			("Attempt to send on non-active VC\n"));

		return;
	}

	if ( (TxBuffer = AtmUniGetTransmitBuffer( AtmUni)) == NULL )
	{
	  	DBGPRINT( DBG_PRINT_MODULE_AUTO_CONFIG, DBG_PRINT_LEVEL_TRACE,
				("No Tx Buffer available for Send\n"));

		AtmUni->Stats.TxPdusError++;
		AtmUniVc->Stats.TxPdusError++;

       AtmUni->AtmCounters.TxPacketError++;
       AtmUniVc->AtmCounters.TxPacketError++;
		AtmUniProcessBuffers( AtmUni);
		return;
	}

	OutputBuffer            = TxBuffer->Data;
	TxBuffer->IsLoopback    = FALSE;
	TxBuffer->ArpFlag       = FALSE;

	// Initialize per PDU VC parameters
	NdisAcquireSpinLock( &AtmUniVc->Lock);

	AtmUniVc->Cpi				= 0x00;
    AtmUniVc->CpcsUu            = CpcsUu;
	AtmUniVc->Cpi	            = Cpi;
	AtmUniVc->TxPartialCrc		= CRC_INITIAL_VALUE;
	AtmUniVc->TxCellRemainder	= 0;

	TxBuffer->Size = AtmUniAal5Segment(
						AtmUniVc,
						&OutputBuffer,
						PduData,
						PduLength,
						TRUE,
						PduLength);

	NdisReleaseSpinLock( &AtmUniVc->Lock);

	DBGPRINT( DBG_PRINT_MODULE_AUTO_CONFIG, DBG_PRINT_LEVEL_TRACE, ("Sending %d bytes\n", TxBuffer->Size));

	// Submit the data buffer to transmit list USB for transmission
    if ( !AtmUniQueueTransmitBuffer( AtmUni, TxBuffer) )
    {
        NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
		NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
    }
}
#endif // USB_WIN_AUTO_CONFIG


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniSetVcPacketContext
//
//  ABSTRACT:
//		Sets the user packet context field for the given VC to the
//		value specified.  The packet context is stored in the upper four
//		bytes of the NDIS packet MiniportReserved field in packets that are
//		indicated to the owner of the VC.
//
//		AtmUniGetVcPacketContext may be used to retrieve the context from
//		a packet.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniSetVcPacketContext(
	ATM_UNI_HANDLE	AtmUniVcHandle,
	PVOID			Context
)
{
	PATM_UNI_VC	AtmUniVc;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniSetVcPacketContext\n"));

	ASSERT( AtmUniVcHandle);
	ASSERT( Context);

	AtmUniVc = AtmUniVcHandle;

	AtmUniVc->UserPacketContext = Context;

} // AtmUniSetVcPacketContext

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniGetVcPacketContext
//
//  ABSTRACT:
//		Gets the VC packet context from the given NDIS packet.  The VC owner
//		must specify the VC packet context with AtmUniSetVcPacketContext when
//		the VC is created before using this function.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
PVOID
AtmUniGetVcPacketContext( PNDIS_PACKET	Packet)
{
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetVcPacketContext\n"));

	ASSERT( Packet);

	return( GET_PACKET_VC_MINIPORT_RESERVED( Packet));

} // AtmUniGetVcPacketContext

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniGetUserVcHandle
//
//  ABSTRACT:
//		Returns the user handle for the given VC.  Provided as a helper function
//		for users that don't wish to allocate their own context to hold the
//		user VC handle.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
ATM_UNI_USER_HANDLE
AtmUniGetUserVcHandle( ATM_UNI_HANDLE	AtmUniVcHandle)
{
	PATM_UNI_VC	AtmUniVc;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetUserVcHandle\n"));

	ASSERT( AtmUniVcHandle);

	AtmUniVc = AtmUniVcHandle;

	return( AtmUniVc->UserVcHandle);

} // AtmUniGetUserVcHandle

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniGetStats
//
//  ABSTRACT:
//		Returns the current global statistics values
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniGetStats(
	ATM_UNI_HANDLE	AtmUniHandle,
	PATM_UNI_STATS	AtmUniStats
)
{
	PATM_UNI	AtmUni;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetStats\n"));

	ASSERT( AtmUniHandle);
	ASSERT( AtmUniStats);

	AtmUni = AtmUniHandle;

	COPY_MEMORY( AtmUniStats,&(AtmUni->Stats),sizeof( ATM_UNI_STATS));

} // AtmUniGetStats

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniGetVcStats
//
//  ABSTRACT:
//		Returns the current statistics values for the given VC
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniGetVcStats(
	ATM_UNI_HANDLE	AtmUniVcHandle,
	PATM_UNI_STATS	AtmUniStats
)
{
	PATM_UNI_VC	AtmUniVc;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetVcStats\n"));

	ASSERT( AtmUniVcHandle);
	ASSERT( AtmUniStats);

	AtmUniVc = AtmUniVcHandle;

	COPY_MEMORY( AtmUniStats,&(AtmUniVc->Stats),sizeof( ATM_UNI_STATS));

} // AtmUniGetVcStats

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniLinkStatusChange
//
//  ABSTRACT:
//		Called by the ATM Phy layer to indicate that the link status has
//		changed.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniLinkStatusChange(
	PATM_UNI			AtmUni,
	ATM_PHY_LINK_STATE	LinkState,
	ULONG				UpstreamRate,
	ULONG				DownstreamRate,
	USHORT              COType,
	UCHAR				RxCellPadding
)
{
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniLinkStatusChange\n"));

	
    switch ( LinkState )
    {
        case ATM_PHY_LINK_STATE_CONNECTED:
        case ATM_PHY_LINK_STATE_DISCONNECTED:

            AtmUni->LinkState 		= LinkState;
	        AtmUni->UpstreamRate	= UpstreamRate;
	        AtmUni->DownstreamRate	= DownstreamRate;
	        AtmUni->AtmPhyRxPadding = RxCellPadding;

            if ( ATM_PHY_LINK_STATE_CONNECTED == LinkState )
                AtmUni->AtmUniState = AtmUni->AtmUniState & (~ATM_UNI_STATE_DISCONNECTED);

	        //
	        // If the user registered a link status change function, call it
	        //
	        if ( AtmUni->UserLinkStatusChange) {
		        (AtmUni->UserLinkStatusChange)(
			        AtmUni->UserHandle,
			        LinkState,
			        UpstreamRate,
			        DownstreamRate,
			        COType);
	        }

            if ( ATM_PHY_LINK_STATE_DISCONNECTED == LinkState )
            {
                AtmUni->AtmUniState |= ATM_UNI_STATE_DISCONNECTED;
                AtmUniShutdownTransmit( AtmUni);
            }

	        AtmUni->COType = COType;

	        // reset sync flags since we know we have cell deliniation
	        AtmUni->isOutOfSync = FALSE;
	        AtmUni->validHeaderCount=0;
           AtmUni->InvalidHECLimitCounter  =0;

#ifdef MESSAGE_HANDLING
	        if (ATM_PHY_LINK_STATE_CONNECTED == LinkState)
		        MsgHandling_Start( AtmUni);
	        else
		        MsgHandling_Stop( AtmUni);
#endif // MESSAGE_HANDLING

#ifdef USB_WIN_AUTO_CONFIG
            Update_Ac_Linkstate ( AtmUni->pAutoConfig, LinkState);
#endif // USB_WIN_AUTO_CONFIG

        break;

        case ATM_PHY_LINK_STATE_RESETTING:
            NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
            AtmUni->AtmUniState |= ATM_UNI_STATE_RESETTING;
            NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        break;

        case ATM_PHY_LINK_STATE_RESETDONE:
            AtmUni->AtmUniState = AtmUni->AtmUniState & (~ATM_UNI_STATE_RESETTING);
            AtmUniProcessBuffers( AtmUni);
        break;

        default:
            DBGPRINT( DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_ERR,
				    ("ERROR: Unhandled link state in AtmUniLinkStatusChange!\n"));
        break;
    }
} // AtmUniLinkStatusChange

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniSendComplete
//
//  ABSTRACT:
//		Handles asynchronous transmit completion with the USB interface
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
static void AtmUniSendComplete(
	PATM_UNI			AtmUni,
	PXFER_DATA_BUFFER 		TxBuffer,
	ATM_PHY_USER_HANDLE	NotUsed,
	ATM_PHY_STATUS		Status
)
{
    PATM_UNI_VC    AtmUniVc;
    PNDIS_PACKET   Packet;
    PNDIS_PACKET_DESCRIPTOR pktDescriptorPtr;


	//DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_TRACE,
	//	("AtmUniSendComplete\n"));
	
	ASSERT( AtmUni);
	ASSERT( TxBuffer);

	PARAM_NOT_USED( NotUsed);

	if ( ATM_PHY_STATUS_SUCCESS == Status )
	{
		AtmUni->Stats.TxPdusOk++;
       
       AtmUni->AtmCounters.TxPacketOk++;
       AtmUni->AtmCounters.TxTotalCells += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
		if (TxBuffer->IsLoopback)
       {
			AtmUni->Stats.TxCellsOk++;
           AtmUni->AtmCounters.TxOAMCellsOk += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
       }
		else
       {
			AtmUni->Stats.TxCellsOk += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
           AtmUni->AtmCounters.TxCellsOk += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
       }
	}
	else
	{
		AtmUni->Stats.TxPdusError++;
       AtmUni->AtmCounters.TxPacketError++;
       AtmUni->AtmCounters.TxTotalCells += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
		if (TxBuffer->IsLoopback)
       {
           AtmUni->AtmCounters.TxOAMCellsDrop += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
       }
       else
       {
           AtmUni->AtmCounters.TxCellsDrop += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
       }
	}

	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);

//KJP FIXME
    /* FIXME: temp fix to work around getting buffers with loopback / auto-config
    /  PVCs which would then skip dequeueing NDIS PACKETs.  We always want to try
    /  and grab the next NDIS PACKET.  We should really save the PVC pointer for
    /  every NDIS PACKET */
    AtmUniVc = AtmUni->AtmUniVc;
//KJP FIXME
    Packet = NULL;

    if (TxBuffer->memType == MEM_TYPE_STATIC)
    {
        /* Try to process a pending NDIS PACKET */
        Packet = (PNDIS_PACKET)REMOVE_NDIS_PACKET(&(AtmUni->NdisPacketQueue));

        if ( NULL == Packet)
            AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
    }
    else
    {
        /* Free this buffer since it was dynamically allocated */
        AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
    }
	NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

    if ( Packet)
    {
#if 0 //KJP FIXME
        pktDescriptorPtr = NDIS_PACKET_RESERVED_FIELD(Packet);
        AtmUniVc = AtmUni->AtmUniVc[NDIS_PACKET_GET_INDEX(pktDescriptorPtr->Clp)];
#endif //KJP FIXME

        if (AtmUniVc)
        {
	        if ( ATM_PHY_STATUS_SUCCESS == Status )
	        {
               AtmUniVc->AtmCounters.TxPacketOk++;
               AtmUniVc->AtmCounters.TxTotalCells += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
		        if (TxBuffer->IsLoopback)
               {
                   AtmUniVc->AtmCounters.TxOAMCellsOk += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
               }
               else
               {
                   AtmUniVc->AtmCounters.TxCellsOk += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
               }
	        }
	        else
	        {
               AtmUniVc->AtmCounters.TxPacketError++;
		        if (TxBuffer->IsLoopback)
               {
                   AtmUniVc->AtmCounters.TxOAMCellsDrop += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
               }
               else
               {
                   AtmUniVc->AtmCounters.TxCellsDrop += TxBuffer->Size / (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);
               }
	        }

        /* Process the NdisPacket */
        if ( ATM_UNI_STATUS_SUCCESS != (AtmUniTxProcess(AtmUniVc, Packet, TxBuffer)) )
        {
            NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		    AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
		    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);    
        }

        (AtmUniVc->UserSendComplete)(AtmUni->UserHandle, NULL, Packet);	
		/* This callback is Rfc1483SendComplete */
    }
        else
        {
            NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
            AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
            NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);    
        }
    }

	if ( ATM_PHY_STATUS_SUCCESS == Status )
		AtmProcessFunction( AtmUni);		// process next buffer in tx list
	else
		AtmUni->NeedQueueAtmProcess = TRUE;	// allow tx in case error can be corrected
}

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAal5Segment
//
//  ABSTRACT:
//		 Peforms AAL5 Segmentation, taking data from the given input
//       buffer and putting cells into the given output buffer. If
//       EndOfPdu is TRUE, AAL5 padding (if necessary) and the AAL5
//       trailer are written to the output buffer.
//
//  RETURNS:
//		The number of bytes written to the output buffer
//
//  ASSUMPTIONS:
//		The output buffer be large enough to hold the entire AAL5 pdu
//      including padding and trailer
//
///////////////////////////////////////////////////////////////////////////////
USHORT
AtmUniAal5Segment(
	PATM_UNI_VC AtmUniVc,
	PPUCHAR	   	OutputBuffer,
	PUCHAR		InputBuffer,
	USHORT      InputLength,
	BOOLEAN     EndOfPdu,
	USHORT      PduLength
)
{
	USHORT   BytesWritten;
	UCHAR    BytesLeftInCurrentCell;
	BOOLEAN  WriteCellHeader;

	//DBGPRINT(
	//	DBG_PRINT_MODULE_ATMUNI_TX,
	//	DBG_PRINT_LEVEL_TRACE,
	//	("AtmUniAal5Segment\n"));

	BytesWritten = 0;
	//
	// Peform segmentation over the input buffer
	//
	if( AtmUniVc->TxCellRemainder ) {
		BytesLeftInCurrentCell = AtmUniVc->TxCellRemainder;
		WriteCellHeader        = FALSE;
	}
	else {
		BytesLeftInCurrentCell = ATM_CELL_PAYLOAD_SIZE;
		WriteCellHeader        = TRUE;
	}

	while ( InputLength ) {
		if( WriteCellHeader) {
			//
			// Copy the cell header. The PT 'end of PDU' bit will be updated
			//   when the AAL5 pad and trailer are written
			//
			COPY_MEMORY(
				*OutputBuffer,
				AtmUniVc->CellHeader,
				ATM_CELL_FULL_HEADER_SIZE);
			*OutputBuffer += ATM_CELL_FULL_HEADER_SIZE;
			BytesWritten  += ATM_CELL_FULL_HEADER_SIZE;
		}
		//
		// If we've reached the end of the input buffer and EndOfPdu is TRUE,
		// write the AAL5 pad and trailer and exit.  Otherwise, if EndOfPdu is
		// false, remember the amount of space left in the current cell in
		// TxCellRemainder
		//
		if( InputLength <= BytesLeftInCurrentCell ) {
			COPY_MEMORY( *OutputBuffer, InputBuffer, InputLength);
			BytesWritten += InputLength;
			AtmUniVc->TxPartialCrc =
				CrcCalculate(
					*OutputBuffer,
					(UCHAR)InputLength,
					AtmUniVc->TxPartialCrc);
			*OutputBuffer += InputLength;
			if( EndOfPdu ) {
	            BytesWritten += AtmUniAal5WritePadAndTrailer(
									AtmUniVc,
									OutputBuffer,
									(UCHAR)(BytesLeftInCurrentCell-InputLength),
									PduLength);
				AtmUniVc->TxCellRemainder = 0;
			}
			else {
				AtmUniVc->TxCellRemainder = BytesLeftInCurrentCell-InputLength;
			}
            /*
            ** Since we do not have any outstanding empty space
            ** in the cell, We can go ahead place the USB padding
            ** needed for this device.
            */
            if ( AtmUniVc->TxCellRemainder == 0)
            {
			   COPY_MEMORY( *OutputBuffer, AtmPhyPaddingPattern, ATM_PHY_PADDING);
			   *OutputBuffer += ATM_PHY_PADDING;
			   BytesWritten  += ATM_PHY_PADDING;
            }
			InputLength = 0;
		}
		//
		// Complete the current cell and continue to the next
		//
		else {
			COPY_MEMORY( *OutputBuffer, InputBuffer, BytesLeftInCurrentCell);
			AtmUniVc->TxPartialCrc = CrcCalculate(
										*OutputBuffer,
										BytesLeftInCurrentCell,
										AtmUniVc->TxPartialCrc);
			BytesWritten  += BytesLeftInCurrentCell;
			*OutputBuffer += BytesLeftInCurrentCell;
			InputBuffer   += BytesLeftInCurrentCell;
			InputLength   -= BytesLeftInCurrentCell;

           /* Add the padding in the Tx packet */
			   COPY_MEMORY( *OutputBuffer, AtmPhyPaddingPattern, ATM_PHY_PADDING);
			   *OutputBuffer += ATM_PHY_PADDING;
			   BytesWritten  += ATM_PHY_PADDING;
			//
			// Next cell will have full payload available and include a header
			//
			BytesLeftInCurrentCell  = ATM_CELL_PAYLOAD_SIZE;
			WriteCellHeader         = TRUE;
		}
	}

	return( BytesWritten);

} // AtmUniAal5Segment

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAal5WritePadAndTrailer
//
//  ABSTRACT:
//		Writes the pad (if any) and AAL5 trailer to the given output buffer
//
//  RETURNS:
//		the number of bytes written to the output buffer
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
static
USHORT
AtmUniAal5WritePadAndTrailer(
	PATM_UNI_VC	AtmUniVc,
	PPUCHAR    	OutputBuffer,
	UCHAR        BytesLeftInCurrentCell,
	USHORT       PduLength
)
{

	PUCHAR CellHeader;
	USHORT BytesWritten;

	//DBGPRINT(
	//	DBG_PRINT_MODULE_ATMUNI_TX,
	//	DBG_PRINT_LEVEL_TRACE,
	//	("AtmUniAal5WritePadAndTrailer\n"));

	ASSERT( AtmUniVc);
	ASSERT( OutputBuffer);

	BytesWritten = 0;
	//
	// Do we have a one or two cell trailer?
	//
	if( BytesLeftInCurrentCell < ATM_AAL5_TRAILER_SIZE) {
		//
		// Two cell padding:
		//    Fill the remainder of the current cell with pad bytes
		//    and generate a second cell into which the AAL5 trailer
		//    will be written
		//
		COPY_MEMORY( *OutputBuffer, &AtmCellPadding, BytesLeftInCurrentCell);
		AtmUniVc->TxPartialCrc =
			CrcCalculate(
				*OutputBuffer,
				BytesLeftInCurrentCell,
				AtmUniVc->TxPartialCrc);
			*OutputBuffer += BytesLeftInCurrentCell;
			BytesWritten  += BytesLeftInCurrentCell;
			COPY_MEMORY( *OutputBuffer, AtmPhyPaddingPattern, ATM_PHY_PADDING);
			*OutputBuffer += ATM_PHY_PADDING;
			BytesWritten  += ATM_PHY_PADDING;
		//
		// Copy the cell header. Since this is the last cell in the PDU, the
		//  'end of PDU' bit in the PT field must be set.
		//
		COPY_MEMORY( *OutputBuffer, AtmUniVc->CellHeader,ATM_CELL_FULL_HEADER_SIZE);
		(*OutputBuffer)[3] |= ATM_AAL5_PT_END_OF_PDU_MASK;
		*OutputBuffer += ATM_CELL_FULL_HEADER_SIZE;
		BytesWritten  += ATM_CELL_FULL_HEADER_SIZE;

		COPY_MEMORY( *OutputBuffer, &AtmCellPadding,
		           ATM_CELL_PAYLOAD_SIZE-ATM_AAL5_TRAILER_SIZE);
		AtmUniVc->TxPartialCrc = CrcCalculate(
		                          *OutputBuffer,
		                          ATM_CELL_PAYLOAD_SIZE-ATM_AAL5_TRAILER_SIZE,
		                          AtmUniVc->TxPartialCrc);
		*OutputBuffer += ATM_CELL_PAYLOAD_SIZE - ATM_AAL5_TRAILER_SIZE;
		BytesWritten  += ATM_CELL_PAYLOAD_SIZE - ATM_AAL5_TRAILER_SIZE;
      /* No Need to add ATM_PHY_PADDING because we do it later */

		goto WriteTrailer;
	}
	else {
		//
		// One cell padding:
		//   Fill the remainder of the current cell (if any) with pad bytes
		//   and write the AAL5 trailer at the end
		//
		COPY_MEMORY(
			*OutputBuffer,
			&AtmCellPadding,
			BytesLeftInCurrentCell-ATM_AAL5_TRAILER_SIZE);
		AtmUniVc->TxPartialCrc =
		CrcCalculate(
			*OutputBuffer,
			(UCHAR)(BytesLeftInCurrentCell - ATM_AAL5_TRAILER_SIZE),
			AtmUniVc->TxPartialCrc);
		*OutputBuffer += BytesLeftInCurrentCell-ATM_AAL5_TRAILER_SIZE;
		BytesWritten  += BytesLeftInCurrentCell-ATM_AAL5_TRAILER_SIZE;
		//
		// Set the PT 'end of PDU' bit in the cell header
		//
		CellHeader    = *OutputBuffer - ATM_CELL_TXFR_SIZE + ATM_AAL5_TRAILER_SIZE;
		CellHeader[3] |= ATM_AAL5_PT_END_OF_PDU_MASK;
	}

	WriteTrailer:
	//
	// Write the trailer
	//
	*(*OutputBuffer)++ = AtmUniVc->CpcsUu;
	*(*OutputBuffer)++ = AtmUniVc->Cpi;
	*((USHORT*)(*OutputBuffer)) = HTONS( PduLength);   // PDU Length
	*OutputBuffer += sizeof(USHORT);
	AtmUniVc->TxPartialCrc = CrcCalculate(
								*OutputBuffer-4,
								ATM_AAL5_TRAILER_SIZE-sizeof(ULONG),
								AtmUniVc->TxPartialCrc);
	*((ULONG*)(*OutputBuffer)) = HTONL( ~AtmUniVc->TxPartialCrc);
	*OutputBuffer += sizeof(ULONG);
	BytesWritten  += ATM_AAL5_TRAILER_SIZE;

	return( BytesWritten);

} // AtmUniAal5WritePadAndTrailer


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniHECValid
//
//  ABSTRACT:
//		Calculate the HEC value based on the ATM Header (4 bytes).
//
//  RETURNS:   TRUE - if calculated bytes match the received byte.
//             FALSE - if incorrect HEC value received.
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
static BOOLEAN AtmUniHECValid(PATM_UNI AtmUni, PUCHAR Buffer)
{
	UCHAR hecAccum=0;
#if ATM_CELL_FULL_HEADER
	int i;

	/* calculate CRC-8 remainder over first four bytes of cell header   */
	/* exclusive-or with coset leader & insert into fifth header byte   */

	for ( i = 0;  i < 4;  i++ )
		hecAccum = hecTable [ hecAccum ^ Buffer[i] ];

	return (Buffer[4] == (hecAccum ^ COSET_LEADER));
#else
        /* Assume GFC=0 and Higher four nibble of VPI=0 */
	hecAccum = hecTable [ 0 ];
	hecAccum = hecTable [ hecAccum ^ (Buffer[0] & 0xF0) ];
	hecAccum = hecTable [ hecAccum ^ Buffer[1] ];
	hecAccum = hecTable [ hecAccum ^ Buffer[2] ];
	if ((Buffer[0] & 0x0F) == ((hecAccum ^ COSET_LEADER) & 0x0F))
           return TRUE;

        /* Assume GFC=0xF and Higher four nibble of VPI=0 */
	hecAccum = hecTable [ 0xF0 ];
	hecAccum = hecTable [ hecAccum ^ (Buffer[0] & 0xF0) ];
	hecAccum = hecTable [ hecAccum ^ Buffer[1] ];
	hecAccum = hecTable [ hecAccum ^ Buffer[2] ];
	if ((Buffer[0] & 0x0F) == ((hecAccum ^ COSET_LEADER) & 0x0F))
           return TRUE;

   /* FIXME: We cannot check for every possible case of VPI, GFC, or VCI
   ** because we only have lower four nibble of the HEC value.
   ** We increase our chance to match the HEC value incorrectly.
   */
   return FALSE;
#endif // ATM_CELL_FULL_HEADER
}


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAal5Reassemble
//
//  ABSTRACT:
//		Peforms AAL5 Reassembly, taking cells from the given input
//      buffer and indicating AAL5 PDUs to the respective VC owners
//      when they have been completely reassembled without error.
//		Note:  When receiving ATM "compressed" header cell, we clear the HEC 
//		Low_byte from the VCI place holder (upper 4-bits).
//
//  RETURNS: void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
static
UCHAR
AtmUniAal5Reassemble(
   PATM_UNI             AtmUni,
   PXFER_DATA_BUFFER    InputBuffer
)
{
	ULONG       	    CellHeader;
	BOOLEAN     	    EndOfPdu;
	ULONG       	    VpiVci;
	PUCHAR      	    Buffer;
	ULONG	    	    Length;
	PATM_UNI_VC 	    AtmUniVc;
	ULONG      		    ReassembledPduLength;
	PAAL5_TRAILER	    Aal5Trailer;
	PNDIS_PACKET	    NdisPacket;
	PNDIS_BUFFER	    NdisBuffer;
	USHORT			    PduLength;
	PXFER_DATA_BUFFER	RxBuffer;


	//DBGPRINT(
	//	DBG_PRINT_MODULE_ATMUNI_RX,
	//	DBG_PRINT_LEVEL_TRACE,
	//	("AtmUniAal5Reassemble\n"));

	ASSERT( AtmUni);
	ASSERT( InputBuffer);

	AtmUniVc = NULL; 

	Buffer = InputBuffer->Data;
	Length = InputBuffer->Size;

	ASSERT( Buffer);
	ASSERT( Length);

	// Add the missing padding (required for our implementation of bulk) to the length
	Length += AtmUni->AtmPhyRxPadding;

	// We're expecting input buffer to contain an integral number of ATM cells
	if( Length % (ATM_CELL_RCV_SIZE+(AtmUni->AtmPhyRxPadding)) ) 
	{
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_RX,
			DBG_PRINT_LEVEL_ERR,
			("Bad input buffer size (%d)\n",
			Length));

        return 0;
	}

	//
	// Process each cell in the input buffer
	//
	while ( Length >= (ATM_CELL_RCV_SIZE + (ULONG)(AtmUni->AtmPhyRxPadding)) ) 
	{
		CellHeader = NTOHL( (*((ULONG *)Buffer)));
		EndOfPdu = (BOOLEAN)(GET_ATM_PTI( CellHeader) & ATM_AAL5_PT_END_OF_PDU_MASK);

        // Lookup the PVC based on the VPI/VCI of the cell header
        VpiVci = GET_ATM_VPIVCI( CellHeader);
        AtmUniVc = AtmUniLookupVc( AtmUni, VpiVci);

		if( NULL == AtmUniVc )                      // cell is not in lookup table
		{
            if ( AtmUniHECValid(AtmUni, Buffer) )   // check to see if HEC is valid
			{
                if ( UpdateSyncCount( AtmUni) )     // valid HEC so update sync counter if not in sync
                {
                    AtmUni->Stats.RxCellsDropped++;
                    
                    AtmUni->AtmCounters.RxCellsDrop++;
                    AtmUni->AtmCounters.RxTotalCells++;
                }
                else if ( !ProcessInboundLoopback( AtmUni, AtmUniVc, Buffer) )  // valid HEC and sync so check if loopback
                {
                    AtmUni->Stats.RxCellsOk++;
                    AtmUni->Stats.RxInvalidVpiVci++;
				    AtmUni->Stats.RxCellsDropped++;
                   
                   AtmUni->AtmCounters.RxCellsDrop++;
                    AtmUni->AtmCounters.RxTotalCells++;
                }
                else
                {
                   AtmUni->AtmCounters.RxTotalCells++;
                   AtmUni->AtmCounters.RxPacketOk++;
                }

				/* HEC Value is correct...let's move on */
               AtmUni->InvalidHECLimitCounter=0;
				Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
				Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
			}
            else if ( PVC_COMPARE( AtmUni->LbVpiVci, VpiVci) )
            {
                if ( !ProcessInboundLoopback( AtmUni, AtmUni->LbAtmUniVc, Buffer) )  // matches extended header loopback check so process
                {
                    AtmUni->Stats.RxCellsOk++;
                    AtmUni->Stats.RxInvalidVpiVci++;
				    AtmUni->Stats.RxCellsDropped++;

                   AtmUni->AtmCounters.RxCellsDrop++;
                    AtmUni->AtmCounters.RxTotalCells++;
                }
                else
                {
                   AtmUni->AtmCounters.RxTotalCells++;
                   AtmUni->AtmCounters.RxPacketOk++;
                }

               AtmUni->InvalidHECLimitCounter=0;
				Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
				Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
            }
            else if ( (AtmUni->InvalidHECLimitCounter < HEC_LIMIT_COUNTER) &&
                       (AtmUni->isOutOfSync == FALSE) )
            {
               AtmUni->InvalidHECLimitCounter++;
				Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
				Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
            }
			else
			{
				if ( !AtmUni->isOutOfSync )         // mark as out-of-sync since cell not in table and HEC is invalid
				{
					AtmUni->isOutOfSync = TRUE;

					DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_TRACE,
						("SAR has detected out-of-synch in the reassembly\n"));
				}
					
				AtmUni->validHeaderCount=0;
				Buffer++;
				Length--;
			}
		}
       /* FIXME: Just because the VPI/VCI Handle was found does not mean this cell
       ** is meant for this connection. We are assuming the upper four nibble of the
       ** VPI field is zero. (For Example, VPI=0x16 and VPI=0x06 means different connection
       ** but the lookupTable does not take this into consideration).
       */
        else if ( UpdateSyncCount( AtmUni) )        // cell found in table so update sync counter if not in sync
        {
            AtmUni->Stats.RxCellsDropped++;
            
           AtmUni->AtmCounters.RxCellsDrop++;
           AtmUni->AtmCounters.RxTotalCells++;
           
            Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
			Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
        }
        else if ( ProcessInboundLoopback( AtmUni, AtmUniVc, Buffer) )   // cell found in table and sync so check if loopback
        {
           AtmUni->InvalidHECLimitCounter=0;
            AtmUniVc->Stats.RxCellsOk++;

           AtmUni->AtmCounters.RxTotalCells++;
           AtmUniVc->AtmCounters.RxTotalCells++;
           AtmUni->AtmCounters.RxPacketOk++;
           AtmUniVc->AtmCounters.RxPacketOk++;
           
			Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
			Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
        }
        else if ( ATM_UNI_VC_STATE_ACTIVE != AtmUniVc->State )  // If the VC is not in the ACTIVE state, drop the cell
        {
			DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR,
				("Dropped Cell - VC not active (VpiVci = 0x%X)\n", VpiVci));

            AtmUni->Stats.RxCellsOk++;
            AtmUniVc->Stats.RxCellsOk++;
			AtmUni->Stats.RxCellsDropped++;
			AtmUniVc->Stats.RxCellsDropped++;

			AtmUni->AtmCounters.RxCellsDrop++;
			AtmUniVc->AtmCounters.RxCellsDrop++;
           AtmUni->AtmCounters.RxTotalCells++;
           AtmUniVc->AtmCounters.RxTotalCells++;

           AtmUni->InvalidHECLimitCounter=0;
			Buffer += (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
			Length -= (ATM_CELL_RCV_SIZE + (AtmUni->AtmPhyRxPadding));
        }
        else    // cell is intended as part of a pdu so increment rcv counters and skip header
        {
           AtmUni->InvalidHECLimitCounter=0;
            AtmUni->Stats.RxCellsOk++;
            AtmUniVc->Stats.RxCellsOk++;
            
			AtmUni->AtmCounters.RxCellsOk++;
			AtmUniVc->AtmCounters.RxCellsOk++;
           AtmUni->AtmCounters.RxTotalCells++;
           AtmUniVc->AtmCounters.RxTotalCells++;
           
		    Buffer += ATM_CELL_RCV_HEADER_SIZE;
		    Length -= ATM_CELL_RCV_HEADER_SIZE;

		    //
		    // Is there a reassembly already in progress on this VC?
		    //
		    if( !AtmUniVc->CurrentRxBuffer ) 
            {
			    //
			    // Get a free RX Buffer. If there are none available, drop the cell
			    //
			    NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
			    RxBuffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &(AtmUni->RxBufferFreeList));
			    NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
			    if ( NULL == RxBuffer) 
                {
				    DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR, ("No Rx Buffer\n"));

				    AtmUni->Stats.RxPdusNoBuffer++;
				    AtmUniVc->Stats.RxPdusNoBuffer++;
				    AtmUni->Stats.RxCellsDropped++;
				    AtmUniVc->Stats.RxCellsDropped++;

			        AtmUni->AtmCounters.RxCellsDrop++;
			        AtmUniVc->AtmCounters.RxCellsDrop++;
			        AtmUni->AtmCounters.RxNoBuffer++;
			        AtmUniVc->AtmCounters.RxNoBuffer++;
                   AtmUni->AtmCounters.RxTotalCells++;
                   AtmUniVc->AtmCounters.RxTotalCells++;
                   
				    Buffer += ATM_CELL_PAYLOAD_SIZE;
				    Length -= ATM_CELL_PAYLOAD_SIZE;
				    Buffer += (AtmUni->AtmPhyRxPadding);
				    Length -= (AtmUni->AtmPhyRxPadding);

				    continue;
			    }
                else
                {
			        //
			        // Initialize the VC parameters for the new PDU reassembly
			        //
			        NdisAcquireSpinLock( &AtmUniVc->Lock);
			        AtmUniVc->CurrentRxBuffer	= RxBuffer;
			        AtmUniVc->ReassemblyPointer = AtmUniVc->CurrentRxBuffer->Data;
			        AtmUniVc->ReassemblySpace   = AtmUniVc->AtmUni->RxBufferSize;
			        AtmUniVc->RxPartialCrc      = CRC_INITIAL_VALUE;
			        NdisReleaseSpinLock( &AtmUniVc->Lock);
                }
		    }

		    ASSERT( AtmUniVc->ReassemblyPointer);

		    //
		    // If there isn't sufficient space in the reassembly buffer to
		    //   hold another cell payload, drop the entire PDU being
		    //   reassembled.  (This can happen if the last cell of a PDU
		    //   is lost, causing two PDUs to get concatenated)
		    //
		    if ( AtmUniVc->ReassemblySpace < ATM_CELL_PAYLOAD_SIZE) 
            {
			    DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR, ("Dropped PDU (no space)\n"));

			    AtmUni->Stats.RxPdusNoBuffer++;
			    AtmUniVc->Stats.RxPdusNoBuffer++;
			    AtmUni->Stats.RxCellsDropped++;
			    AtmUniVc->Stats.RxCellsDropped++;

			    AtmUni->AtmCounters.RxNoBuffer++;
			    AtmUniVc->AtmCounters.RxNoBuffer++;
			    AtmUni->AtmCounters.RxCellsDrop++;
			    AtmUniVc->AtmCounters.RxCellsDrop++;
               AtmUni->AtmCounters.RxTotalCells++;
               AtmUniVc->AtmCounters.RxTotalCells++;
               
			    Buffer += ATM_CELL_PAYLOAD_SIZE;
			    Length -= ATM_CELL_PAYLOAD_SIZE;
                Buffer += (AtmUni->AtmPhyRxPadding);
			    Length -= (AtmUni->AtmPhyRxPadding);

                NdisAcquireSpinLock( &AtmUniVc->Lock);

			    NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
			    PUSH_ENTRY_LIST(
				    &(AtmUni->RxBufferFreeList),
				    (PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
			    NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
			    
			    AtmUniVc->CurrentRxBuffer      = NULL;
			    AtmUniVc->ReassemblyPointer    = NULL;
			    AtmUniVc->ReassemblySpace      = 0;
			    NdisReleaseSpinLock( &AtmUniVc->Lock);

			    continue;
		    }

		    NdisAcquireSpinLock( &AtmUniVc->Lock);
		    AtmUniAal5ProcessInboundCell(
			    AtmUniVc,
			    Buffer,
			    EndOfPdu,
			    &ReassembledPduLength,
			    &Aal5Trailer);
		    NdisReleaseSpinLock( &AtmUniVc->Lock);

		    Buffer += ATM_CELL_PAYLOAD_SIZE;
		    Length -= ATM_CELL_PAYLOAD_SIZE;
		    Buffer += (AtmUni->AtmPhyRxPadding);
		    Length -= (AtmUni->AtmPhyRxPadding);

		    //
		    // If the current cell is the end of a PDU, do the end of PDU processing.
		    //
		    if (EndOfPdu) 
		    {
			    PduLength = NTOHS( Aal5Trailer->PduLength);

			    //
			    // Compare received data Length against AAL5 trailer PDU Length
			    // and computed CRC against AAL5 trailer CRC. If either
			    // mismatch, drop the PDU. Otherwise, indicate the PDU to the user.
			    //
			    if( AtmUniVc->RxPartialCrc != CRC_RESIDUE) 
                {
				    AtmUni->Stats.RxCrcError++;
				    AtmUniVc->Stats.RxCrcError++;

			        AtmUni->AtmCounters.RxAAL5Error++;
			        AtmUniVc->AtmCounters.RxAAL5Error++;
                   
				    DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR,
					    ("CRC ERROR : Expected 0x%X, Actual 0x%X (Total %d)\n", 
                        CRC_RESIDUE, AtmUniVc->RxPartialCrc,
					    AtmUni->Stats.RxCrcError));

			       NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
			       PUSH_ENTRY_LIST(
					    &(AtmUni->RxBufferFreeList),
					    (PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
			       NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
			    }
#ifdef MESSAGE_HANDLING
			    else if ( (AtmUniVc == AtmUni->MsgHandling_SiemensHandle) ||
					      (AtmUniVc == AtmUni->MsgHandling_EciHandle) )
			    {
				    MsgHandling_Receive( AtmUniVc, AtmUniVc->CurrentRxBuffer->Data, 
									    (USHORT)ReassembledPduLength, (USHORT)PduLength);

				    AtmUni->Stats.RxPdusOk++;
				    AtmUniVc->Stats.RxPdusOk++;

				    AtmUni->AtmCounters.RxPacketOk++;
				    AtmUniVc->AtmCounters.RxPacketOk++;
                   
				    NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
				    PUSH_ENTRY_LIST(
						    &(AtmUni->RxBufferFreeList),
						    (PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
				    NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
			    }
#endif // MESSAGE_HANDLING
			    else if( ReassembledPduLength != PduLength) 
                {
				    AtmUni->Stats.RxPdusError++;
				    AtmUniVc->Stats.RxPdusError++;

				    AtmUni->AtmCounters.RxPacketError++;
				    AtmUniVc->AtmCounters.RxPacketError++;

				    DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR,
					    ("LEN ERROR : Expected %d, Actual %d (Total %d)\n",
					    PduLength, ReassembledPduLength,
					    AtmUni->Stats.RxPdusError));

				    NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
				    PUSH_ENTRY_LIST(
						    &(AtmUni->RxBufferFreeList),
						    (PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
				    NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
			    }
#ifdef USB_WIN_AUTO_CONFIG
                else if ( Queue_Ac_Msg ( AtmUni->pAutoConfig, AtmUniVc, AtmUniVc->CurrentRxBuffer->Data, 
                                          (USHORT)ReassembledPduLength, Aal5Trailer->CpcsUu, Aal5Trailer->Cpi) )
                {
                    AtmUni->Stats.RxPdusOk++;
				    AtmUniVc->Stats.RxPdusOk++;

				    AtmUni->AtmCounters.RxPacketOk++;
				    AtmUniVc->AtmCounters.RxPacketOk++;
                   
				    NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
				    PUSH_ENTRY_LIST(
						    &(AtmUni->RxBufferFreeList),
						    (PSINGLE_LIST_NODE)AtmUniVc->CurrentRxBuffer);
				    NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);
                }
#endif // USB_WIN_AUTO_CONFIG
			    else
                {
				    AtmUni->Stats.RxPdusOk++;
				    AtmUniVc->Stats.RxPdusOk++;

				    AtmUni->AtmCounters.RxPacketOk++;
				    AtmUniVc->AtmCounters.RxPacketOk++;
				    DBGPRINT( DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_ERR,
					    ("Received Good AAL5 PDU %d bytes ( Total %d)\n",
					    ReassembledPduLength, AtmUni->Stats.RxPdusOk));

				    //
				    // Indicate the received PDU to the owner of the VC using the
				    // method specified (buffer or NDIS packet) when the ATM UNI
				    // was created.
				    //
				    if ( AtmUni->IndicatePackets ) 
				    {
					    ASSERT( AtmUniVc->UserReceivePacket);
					    NdisPacket = AtmUniVc->CurrentRxBuffer->AllocatorContext;
					    ASSERT( NdisPacket);
					    NdisUnchainBufferAtFront( NdisPacket, &NdisBuffer);
					    ASSERT( NdisBuffer);

					    NdisAdjustBufferLength( NdisBuffer,ReassembledPduLength);
					    NdisChainBufferAtFront( NdisPacket, NdisBuffer);

					    SET_PACKET_VC_MINIPORT_RESERVED(
						    NdisPacket,
						    AtmUniVc->UserPacketContext);
						
					    (AtmUniVc->UserReceivePacket)(
						    AtmUni->UserHandle,
						    AtmUniVc->UserVcHandle,
						    NdisPacket,
						    Aal5Trailer->CpcsUu,
						    Aal5Trailer->Cpi
						    );
				    }
				    else
				    {
					    ASSERT( AtmUniVc->UserReceive);
					    AtmUniVc->CurrentRxBuffer->Size =
						    ReassembledPduLength;
					    (AtmUniVc->UserReceive)(
						    AtmUni->UserHandle,
						    AtmUniVc,
						    AtmUniVc->CurrentRxBuffer,
						    Aal5Trailer->CpcsUu,
						    Aal5Trailer->Cpi);
				    }
			    }

			    NdisAcquireSpinLock( &(AtmUniVc->Lock));
			    AtmUniVc->CurrentRxBuffer   = NULL;
			    AtmUniVc->ReassemblyPointer = NULL;
			    AtmUniVc->ReassemblySpace   = 0;
			    NdisReleaseSpinLock( &(AtmUniVc->Lock));
		    }
        }
	}	// while ( Length >= (ATM_CELL_RCV_SIZE + (ULONG)(AtmUni->AtmPhyRxPadding)) )

	return ((UCHAR)Length - AtmUni->AtmPhyRxPadding);

} // AtmUniAal5Reassemble


/////////////////////////////////////////////////////////////////////////////////////
//
//  NAME: UpdateSyncCount
//
//  ABSTRACT:
//		Increments valid cell counter and notifed resync if 3 consecutive cells found
//
//  RETURNS: TRUE if driver is not in sync
//
//  ASSUMPTIONS:
//		None
//
/////////////////////////////////////////////////////////////////////////////////////
static BOOLEAN UpdateSyncCount ( PATM_UNI	AtmUni )
{
	if (AtmUni->isOutOfSync) /* Check, if the out of synch was detected */
	{
		AtmUni->validHeaderCount++;
		/* More than 3 times is the charm...we have synch again */
		if (AtmUni->validHeaderCount >= 3)
		{
			DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX,	DBG_PRINT_LEVEL_ERR,
				("%d good cells detected - back in sync\n", AtmUni->validHeaderCount));

			AtmUni->isOutOfSync = FALSE;
			AtmUni->validHeaderCount=0;
           AtmUni->AtmCounters.RxCellsOutOfSynch++;
           AtmUni->InvalidHECLimitCounter=0;
		}
		else
		{
			DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX,	DBG_PRINT_LEVEL_ERR,
				("%d good cells detected\n", AtmUni->validHeaderCount));

			return TRUE;
		}
	}

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////////////
//
//  NAME: ProcessInboundLoopback
//
//  ABSTRACT:
//		Perform loopback if proper F4/F5 cell
//
//  RETURNS: TRUE if loopback cell detected and processed
//
/////////////////////////////////////////////////////////////////////////////////////
static BOOLEAN ProcessInboundLoopback( PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc, PUCHAR Buffer)
{
	PXFER_DATA_BUFFER	    LbBuffer;
    UCHAR                   need2Send = TRUE;


	if ( Process_Loopback((PATM_CELL_RCV)Buffer, &AtmUni->Stats, &need2Send, AtmUni, AtmUniVc) )		// Loopback Processing routine
	{
		/* Send the loopback cell out */
		if (need2Send)
		{
            if ( (LbBuffer = AtmUniGetTransmitBuffer( AtmUni)) != NULL )
			{
				LbBuffer->IsLoopback = TRUE;

                /* 54th byte is required to properly interface to the PCU
                ** therefore add 1 to the length
                */
#ifndef QUASAR 				
                LbBuffer->Size = sizeof(ATM_CELL_TXFR) + sizeof(UCHAR);
#else
                LbBuffer->Size = sizeof(ATM_CELL_TXFR);
#endif
				Copy_Loopback (AtmUniVc, (PATM_CELL_TXFR)(LbBuffer->Data),(PATM_CELL_RCV)Buffer);

                if ( !AtmUniQueueTransmitBuffer( AtmUni, LbBuffer) )
                {
                    NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		            AtmUniFreeTransmitBuffer( AtmUni, LbBuffer);
		            NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
                }
			}
			else
			{
	  			DBGPRINT(
					DBG_PRINT_MODULE_ATMUNI_TX,
					DBG_PRINT_LEVEL_ERR,
					("No Tx Buffer available for loopback\n"));

               AtmUni->AtmCounters.TxPacketError++;
               AtmUniVc->AtmCounters.TxPacketError++;
               AtmUni->AtmCounters.TxOAMCellsDrop++;
               AtmUniVc->AtmCounters.TxOAMCellsDrop++;

				AtmUniProcessBuffers( AtmUni);
			}
		}

        AtmUni->Stats.RxPdusOk++;
        AtmUni->Stats.RxCellsOk++;

        return TRUE;
	}

    return FALSE;
}


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAal5ProcessInboundCell
//
//  ABSTRACT:
//		Processes the given cell.  If the cell marks the end of a PDU, end
//		of PDU processing is applied and the PDU information is returned.
//
//  RETURNS: void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
static
void
AtmUniAal5ProcessInboundCell(
	PATM_UNI_VC		AtmUniVc,
	PUCHAR			Buffer,
	BOOLEAN			EndOfPdu,
	PULONG			ReassembledPduLength,
	PAAL5_TRAILER *	Aal5Trailer
)
{
	UCHAR       PadLength;
	UCHAR       DataRemainder;
	USHORT		PduLength;

	AtmUniVc->RxPartialCrc =
		CrcCalculate(
			Buffer,
			ATM_CELL_PAYLOAD_SIZE,
			AtmUniVc->RxPartialCrc);

	//
	// If this cell doesn't mark the end of a PDU, simply copy the payload
	// and compute the new partial CRC.  Otherwise, do end of PDU processing.
	//
	if( !EndOfPdu) {
		COPY_MEMORY(
			AtmUniVc->ReassemblyPointer,
			Buffer,
			ATM_CELL_PAYLOAD_SIZE);

		AtmUniVc->ReassemblyPointer += ATM_CELL_PAYLOAD_SIZE;
		AtmUniVc->ReassemblySpace   -= ATM_CELL_PAYLOAD_SIZE;
	}
	else {
		//
		// The current cell should contain the AAL5 trailer in the
		//  last eight bytes of the payload
		//
		*Aal5Trailer   = (PAAL5_TRAILER)( Buffer +
		                     (ATM_CELL_PAYLOAD_SIZE - ATM_AAL5_TRAILER_SIZE));

#ifdef MESSAGE_HANDLING
		if ( AtmUniVc == AtmUniVc->AtmUni->MsgHandling_SiemensHandle )
			PduLength = (USHORT)(AtmUniVc->AtmUni->RxBufferSize - AtmUniVc->ReassemblySpace) + S_MIN_CELL_LENGTH;
		else
#endif // MESSAGE_HANDLING
		PduLength = NTOHS((*Aal5Trailer)->PduLength);

		//
		// Do either one or two cell depadding
		//
		DataRemainder = (UCHAR)( PduLength % ATM_CELL_PAYLOAD_SIZE);
		if( DataRemainder > (ATM_CELL_PAYLOAD_SIZE-ATM_AAL5_TRAILER_SIZE)) {
		   //
		   // Two Cell Depadding:
		   //   We don't copy any of the data in the current cell
		   //   and we discard any pad data reassembled from the previous
		   //   cell.
		   //
		   PadLength = ATM_CELL_PAYLOAD_SIZE - DataRemainder;
		   AtmUniVc->ReassemblyPointer -= PadLength;
		   AtmUniVc->ReassemblySpace   += PadLength;
		}
		else {
		   //
		   // One Cell Depadding:
		   //  We copy the data portion of the cell and discard the pad
		   //  portion (if any).
		   //
		   COPY_MEMORY( AtmUniVc->ReassemblyPointer,Buffer,DataRemainder);

		   AtmUniVc->ReassemblyPointer += DataRemainder;
		   AtmUniVc->ReassemblySpace   -= DataRemainder;
		}

		//
		// Compute the reassembled PDU length, which represents the size
		// of the PDU as it was received.  This can be compared against
		// the reported PDU length ( in the AAL5 trailer).
		//
		*ReassembledPduLength = AtmUniVc->AtmUni->RxBufferSize -
		                        AtmUniVc->ReassemblySpace;
	}

} // AtmUniAal5ProcessInboundCell

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniReceiveComplete
//
//  ABSTRACT:
//		Returns ownership of a receive Buffer that was previously
//      indicated to the ATM UNI user via 'userReceive'.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniReceiveComplete(
   ATM_UNI_HANDLE  			AtmUniVcHandle,
   PXFER_DATA_BUFFER		RxBuffer
)
{
	PATM_UNI_VC	AtmUniVc;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_RX,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniReceiveComplete\n"));

	ASSERT( AtmUniVcHandle);
	ASSERT( RxBuffer);

	AtmUniVc = AtmUniVcHandle;

	//
	//	Put the Rx buffer back on the free list
	//
	NdisAcquireSpinLock( &(AtmUniVc->AtmUni->RxBufferFreeListLock));
    RxBuffer->Data = (PUCHAR)(RxBuffer->saveDataPtr);
	PUSH_ENTRY_LIST(
		&(AtmUniVc->AtmUni->RxBufferFreeList),
		(PSINGLE_LIST_NODE)RxBuffer);
	NdisReleaseSpinLock( &(AtmUniVc->AtmUni->RxBufferFreeListLock));

} // AtmUniReceiveComplete

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniReceivePacketComplete
//
//  ABSTRACT:
//		Returns ownership of a receive NDIS packet that was previously
//      indicated to the ATM UNI user via 'userReceivePacket'.
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniReceivePacketComplete(
   ATM_UNI_HANDLE  	AtmUniVcHandle,
   PNDIS_PACKET		Packet
)
{
	PATM_UNI_VC		AtmUniVc;
	PXFER_DATA_BUFFER	Buffer;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_RX,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniReceivePacketComplete\n"));

	ASSERT( AtmUniVcHandle);
	ASSERT( Packet);

	AtmUniVc = AtmUniVcHandle;

	//
	// Retrieve the pointer to the Rx buffer from the NDIS packet's
	// miniport reserved area
	//
	Buffer = GET_PACKET_MINIPORT_RESERVED( Packet);
	ASSERT( Buffer);

	//
	//	Put the Rx buffer back on the free list
	//
	NdisAcquireSpinLock( &(AtmUniVc->AtmUni->RxBufferFreeListLock));
	PUSH_ENTRY_LIST(
		&(AtmUniVc->AtmUni->RxBufferFreeList),
		(PSINGLE_LIST_NODE)Buffer);
	NdisReleaseSpinLock( &(AtmUniVc->AtmUni->RxBufferFreeListLock));

} // AtmUniReceivePacketComplete

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniBindWithPhy
//
//  ABSTRACT:
//		Called to bind the ATM UNI with the ATM Phy layer
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN
AtmUniBindWithPhy(
	PATM_UNI 				AtmUni,
	ATM_PHY_BIND			PhyBind,
	ATM_PHY_DEVICE_HANDLE	PhyDeviceHandle
)
{
	ATM_PHY_BIND_INPUT	PhyBindInput;
	ATM_PHY_BIND_OUTPUT	PhyBindOutput;
	ATM_PHY_STATUS		PhyStatus;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniBindWithPhy\n"));

	ASSERT( AtmUni);

	PhyBindInput.UserHandle				= AtmUni;
	PhyBindInput.UserReceive			= AtmUniAal5Reassemble;

    PhyBindInput.UserCPLGetATMStats     = AtmUniGetAtmStats;
	PhyBindInput.UserCPLClearATMStats   = AtmUniClearAtmStats;

	PhyBindInput.UserSendComplete		= AtmUniSendComplete;
	PhyBindInput.UserLinkStatusChange	= AtmUniLinkStatusChange;
	PhyBindInput.PhyDeviceHandle		= PhyDeviceHandle;
	PhyBindInput.PhyRxResourceCount 	= ATM_UNI_PHY_RX_RESOURCE_COUNT;
	PhyBindInput.PhyRxBufferSize		= ATM_UNI_PHY_RX_BUFFER_SIZE;
	PhyBindInput.ReceiveDataSize		= ATM_CELL_RCV_SIZE;

	PhyStatus = (PhyBind)( &PhyBindInput,&PhyBindOutput);

	if ( ATM_PHY_STATUS_SUCCESS == PhyStatus) {
		AtmUni->PhyHandle 	= PhyBindOutput.PhyHandle;
		AtmUni->PhySend		= PhyBindOutput.PhySend;
		AtmUni->PhyUnbind	= PhyBindOutput.PhyUnbind;

		COPY_MEMORY(
			AtmUni->PhyUniqueId,
			PhyBindOutput.PhyUniqueId,
			UNIQUE_ID_SIZE);
		return( TRUE);
	} else {
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_ALL,
			DBG_PRINT_LEVEL_ERR,
			("PhyBind Failed with Status 0x%X\n",
			PhyStatus));
		return( FALSE);
	}
} // AtmUniBindWithPhy

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniClosePhyBinding
//
//  ABSTRACT:
//		Closes a previously established binding with the ATM Phy layer
//
//  RETURNS:
//  	void
//
//  ASSUMPTIONS:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniClosePhyBinding( PATM_UNI AtmUni)
{
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniClosePhyBinding\n"));

	ASSERT( AtmUni);
	ASSERT( AtmUni->PhyUnbind);

	(AtmUni->PhyUnbind)( AtmUni->PhyHandle);
	AtmUni->PhyHandle 	= NULL;
	AtmUni->PhySend		= NULL;
	AtmUni->PhyUnbind	= NULL;

} // AtmUniClosePhyBinding

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAllocateBuffers
//
//  ABSTRACT:
//		Allocates transmit and receive buffers
//
//  RETURNS:
//		BOOLEAN
//
//  ASSUMPTIONS:
//		Called during AtmUniCreate
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN
AtmUniAllocateBuffers(
   PATM_UNI	AtmUni,
   USHORT   RxBufferCount,
   USHORT   TxBufferCount
)
{
	UINT 		 	i;
	PXFER_DATA_BUFFER	Buffer;
	PPROCESS_DATA_BUFFER ProcessBuffer;
	PUCHAR         	Data;
	NDIS_STATUS		NdisStatus;
	NDIS_HANDLE		PacketPoolHandle;
	NDIS_HANDLE		BufferPoolHandle;
	PNDIS_PACKET   	NdisPacket;
	PNDIS_BUFFER   	NdisBuffer;
	PVOID			MediaSpecificData;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniAllocateBuffers\n"));

	ASSERT( AtmUni);

	//
	// Tx buffers must be large enough to accomodate the ATM cells generated
	// when an SDU of size 'MaxSduSize' is segmented.
	// Rx buffer size is simply the max SDU size plus 2 * ATM Cell size (to
	//  hold possible two cell padding during reassembly)
	//
	AtmUni->TxBufferSize = (AtmUni->MaxSduSize/ATM_CELL_PAYLOAD_SIZE +2)
	                       * (ATM_CELL_TXFR_SIZE + ATM_PHY_PADDING);

	// multiply number of cells needed for Max SDU by a full 64-byte transfer and add 2 cell padding
	AtmUni->RxBufferSize = (( ((UINT)(AtmUni->MaxSduSize / ATM_CELL_PAYLOAD_SIZE) ) + 1) 
							* ATM_UNI_PHY_RX_BUFFER_SIZE) + (2 * ATM_UNI_PHY_RX_BUFFER_SIZE);

	INITIALIZE_LIST_HEAD(&(AtmUni->ProcessBufferFreeList));
	INITIALIZE_LIST_HEAD(&(AtmUni->ProcessBufferTxList));

	//
	// Allocate TX buffers
	//
	for( i = 0; i < TxBufferCount; i++ )
	{
		//
		// Allocate buffer descriptor
		//
		ALLOCATE_MEMORY( &Buffer,sizeof( XFER_DATA_BUFFER));
		if( NULL == Buffer) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Failed (Tx Buffer Descriptor)\n"));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		//
		// Allocate buffer
		//
		ALLOCATE_MEMORY( &Data, AtmUni->TxBufferSize);
		if( NULL == Data) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Failed (Tx Buffer)\n"));
			FREE_MEMORY( Buffer,sizeof( XFER_DATA_BUFFER));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		Buffer->Data = Data;
        Buffer->memType = MEM_TYPE_STATIC;
        Buffer->AllocatorContext = NULL;
		//
		// Put the Tx buffer descriptor on the free list
		//
        AtmUniFreeTransmitBuffer( AtmUni, Buffer);

		//
		// Allocate buffer descriptor
		//
		ALLOCATE_MEMORY( &ProcessBuffer,sizeof( PROCESS_DATA_BUFFER));
		if( NULL == ProcessBuffer) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Failed (Process Buffer Descriptor)\n"));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
        ProcessBuffer->memType = MEM_TYPE_STATIC;
       
		//
		// Put the Tx buffer descriptor on the free list
		//
		INSERT_HEAD_LIST( &(AtmUni->ProcessBufferFreeList),(PLIST_NODE)ProcessBuffer);
	}

	//
	// If the IndicatePackets option has been chosen, allocate the NDIS packet
	// and buffer pools with a descriptor for each Rx buffer
	//
	if ( AtmUni->IndicatePackets ) {
		NdisAllocatePacketPool(
			&NdisStatus,
			&PacketPoolHandle,
			RxBufferCount,
			PROTOCOL_RESERVED_SIZE_IN_PACKET);

		if ( NDIS_STATUS_SUCCESS != NdisStatus) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unable to allocate packet pool\n"));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		AtmUni->RxPacketPoolHandle = PacketPoolHandle;

		NdisAllocateBufferPool(
			&NdisStatus,
			&BufferPoolHandle,
			RxBufferCount);

		if ( NDIS_STATUS_SUCCESS != NdisStatus) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Unable to allocate buffer pool\n"));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		AtmUni->RxBufferPoolHandle = BufferPoolHandle;
	}

	//
	// Allocate RX buffers
	//
	for( i = 0; i < RxBufferCount; i++ ) {
		//
		// Allocate Rx buffer descriptor
		//
		ALLOCATE_MEMORY( &Buffer, sizeof( XFER_DATA_BUFFER));
		if( NULL == Buffer) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Failed (Rx Buffer Descriptor)\n"));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		//
		// Allocate Rx buffer
		//
		ALLOCATE_MEMORY( &Data,AtmUni->RxBufferSize);
		if( NULL == Data) {
			DBGPRINT(
				DBG_PRINT_MODULE_ATMUNI_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Failed (Rx Buffer)\n"));
			FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
			AtmUniFreeBuffers( AtmUni);
			return( FALSE);
		}
		Buffer->Data = Data;
        Buffer->saveDataPtr = (PVOID)Data;

		//
		// If the IndicatePackets option has been chosen, allocate an NDIS
		// packet and buffer for the Rx buffer.  Also allocate space for
		// media specific information in the NDIS_PACKET_OOB_DATA structure
		// as specified when the ATM UNI was created.
		//
		if ( AtmUni->IndicatePackets ) {
			//
			// Allocate the packet
			//
			NdisAllocatePacket( &NdisStatus,&NdisPacket,PacketPoolHandle);

			if ( NDIS_STATUS_SUCCESS != NdisStatus) {
				DBGPRINT(
					DBG_PRINT_MODULE_ATMUNI_ALL,
					DBG_PRINT_LEVEL_ERR,
					("Failed (Rx Packet Descriptor)\n"));
				FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
				FREE_MEMORY( Data, AtmUni->RxBufferSize);
				AtmUniFreeBuffers( AtmUni);
				return( FALSE);
			}

			//
			// Allocate the buffer
			//
			NdisAllocateBuffer(
				&NdisStatus,
				&NdisBuffer,
				BufferPoolHandle,
				Data,
				AtmUni->RxBufferSize);

			if ( NDIS_STATUS_SUCCESS != NdisStatus) {
				DBGPRINT(
					DBG_PRINT_MODULE_ATMUNI_ALL,
					DBG_PRINT_LEVEL_ERR,
					("Failed (Rx Buffer Descriptor)\n"));
				FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
				FREE_MEMORY( Data, AtmUni->RxBufferSize);
				NdisFreePacket( NdisPacket);
				AtmUniFreeBuffers( AtmUni);
				return( FALSE);
			}

			//
			// Allocate space for the media specific data
			//
			if ( AtmUni->MediaSpecificSize ) {
				ALLOCATE_MEMORY(
					&MediaSpecificData,
					AtmUni->MediaSpecificSize);
				if ( NULL == MediaSpecificData) {
					DBGPRINT(
						DBG_PRINT_MODULE_ATMUNI_ALL,
						DBG_PRINT_LEVEL_ERR,
						("Failed (Media Specific Information)\n"));
					FREE_MEMORY( Buffer, sizeof( XFER_DATA_BUFFER));
					FREE_MEMORY( Data, AtmUni->RxBufferSize);
					NdisFreePacket( NdisPacket);
					NdisFreeBuffer( NdisBuffer);
					AtmUniFreeBuffers( AtmUni);
					return( FALSE);
				}
				NDIS_SET_PACKET_MEDIA_SPECIFIC_INFO(
					NdisPacket,
					MediaSpecificData,
					AtmUni->MediaSpecificSize);
			}

			NdisChainBufferAtFront( NdisPacket, NdisBuffer);
			Buffer->AllocatorContext = NdisPacket;
			SET_PACKET_MINIPORT_RESERVED( NdisPacket,Buffer);

		}

		//
		// Put the Rx buffer descriptor on the free list
		//
		PUSH_ENTRY_LIST( &(AtmUni->RxBufferFreeList),(PSINGLE_LIST_NODE)Buffer);
	}

	return( TRUE);

} // AtmUniAllocateBuffers

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniFreeBuffers
//
//  ABSTRACT:
//		Frees transmit and receive buffers allocated with AtmUniAllocateBuffers
//
//  RETURNS:
//		void
//
//  ABSTRACT:
//		None
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniFreeBuffers( PATM_UNI AtmUni)
{
	PXFER_DATA_BUFFER	    Buffer;
	PPROCESS_DATA_BUFFER    ProcessBuffer;
	PNDIS_PACKET   	        NdisPacket;
	PNDIS_BUFFER   	        NdisBuffer;
	PVOID			        MediaSpecificData;
	ULONG			        MediaSpecificDataSize;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniFreeBuffers\n"));

	//
	// Free Tx buffers
	//
	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
	while (!IS_LIST_EMPTY(&(AtmUni->ProcessBufferFreeList)))
	{
		ProcessBuffer = (PPROCESS_DATA_BUFFER)REMOVE_TAIL_LIST(&(AtmUni->ProcessBufferFreeList));

		// Free the buffer descriptor
		FREE_MEMORY( ProcessBuffer,sizeof(PROCESS_DATA_BUFFER));
	}
	NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);


	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
	Buffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &(AtmUni->TxBufferFreeList));
	while ( Buffer) {
		//
		// Free the buffer
		//
		FREE_MEMORY( Buffer->Data,AtmUni->TxBufferSize);
		//
		// Free the buffer descriptor
		//
		FREE_MEMORY( Buffer,sizeof(XFER_DATA_BUFFER));

		Buffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &(AtmUni->TxBufferFreeList));
	}
	NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

	//
	// Free Rx buffers
	//
	NdisAcquireSpinLock( &AtmUni->RxBufferFreeListLock);
	Buffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &(AtmUni->RxBufferFreeList));
	while ( Buffer) {
		//
		// Free the buffer
		//
        Buffer->Data = (PUCHAR)(Buffer->saveDataPtr);
		FREE_MEMORY( Buffer->Data,AtmUni->RxBufferSize);

		//
		// Free the NDIS packet and buffer related memory if we were
		// indicating packets
		//
		if ( AtmUni->IndicatePackets) {
			NdisPacket = Buffer->AllocatorContext;
			ASSERT( NdisPacket);
			NdisUnchainBufferAtFront( NdisPacket,&NdisBuffer);
			ASSERT( NdisBuffer);

			//
			// Free the media specific data space if one was allocated
			//
			if ( AtmUni->MediaSpecificSize) {
				NDIS_GET_PACKET_MEDIA_SPECIFIC_INFO(
					NdisPacket,
					&MediaSpecificData,
					&MediaSpecificDataSize);
				ASSERT( MediaSpecificData);
				ASSERT( MediaSpecificDataSize == AtmUni->MediaSpecificSize);
				FREE_MEMORY( MediaSpecificData, MediaSpecificDataSize);
			}
			//
			// Free the NDIS buffer and packet
			//
			NdisFreeBuffer( NdisBuffer);
			NdisFreePacket( NdisPacket);
		}
		//
		// Free the buffer descriptor
		//
		FREE_MEMORY( Buffer,sizeof(XFER_DATA_BUFFER));

		Buffer = (PXFER_DATA_BUFFER)POP_ENTRY_LIST( &(AtmUni->RxBufferFreeList));
	}

	//
	// Free the NDIS packet and buffer pools if they were allocated
	//
	if ( AtmUni->IndicatePackets) {
		if ( AtmUni->RxBufferPoolHandle) {
			NdisFreeBufferPool( AtmUni->RxBufferPoolHandle);
		}
		if ( AtmUni->RxPacketPoolHandle) {
			NdisFreePacketPool( AtmUni->RxPacketPoolHandle);
		}
	}

	NdisReleaseSpinLock( &AtmUni->RxBufferFreeListLock);

} // AtmUniFreeBuffers


///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniLookupVc
//
//  ABSTRACT:
//		Given the VPI and VCI, determines if the VC is in the VC table.
//      Ensures last found PVC is at head of list for quicker searches.
//
//  RETURNS:
//		the VC entry or NULL if the VC wasn't found
//
//  ASSUMPTIONS: None
//
///////////////////////////////////////////////////////////////////////////////
static
PATM_UNI_VC
AtmUniLookupVc(
   PATM_UNI  AtmUni,
   ULONG     VpiVci
)
{
   ULONG         HashKey;
   PATM_UNI_VC	 AtmUniVc, FirstVc;


	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniLookupVc\n"));

	ASSERT( AtmUni);

	HashKey = GET_VC_HASH_KEY( VpiVci);

	FirstVc = AtmUniVc = (PATM_UNI_VC)GET_FIRST_ENTRY_FROM_LIST( &(AtmUni->VcHashTable[ HashKey]));

	while ( AtmUniVc ) 
    {
		if( PVC_COMPARE( AtmUniVc->VpiVci, VpiVci) )                                        // is PVC found in list?
        {
            if ( FirstVc != AtmUniVc)                                                       // is this first entry?
            {
                REMOVE_ENTRY_LIST( (PLIST_NODE)AtmUniVc);                                   // remove entry
                INSERT_HEAD_LIST( &(AtmUni->VcHashTable[ HashKey]), (PLIST_NODE)AtmUniVc);  // insert at head of list
            }

            return( AtmUniVc);
        }

		AtmUniVc = (PATM_UNI_VC)GET_NEXT_ENTRY_FROM_LIST( &(AtmUni->VcHashTable[ HashKey]), (PLIST_NODE)AtmUniVc);
	}
    
    return( NULL);
} // AtmUniLookupVc

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniAddVcToTable
//
//  ABSTRACT:
//		Adds the given VC to the ATM UNI's VC table
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		Caller must hold the VC list lock prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniAddVcToTable(
	PATM_UNI	AtmUni,
	PATM_UNI_VC	AtmUniVc
)
{
	ULONG   HashKey;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniAddVcToTable\n"));

	ASSERT( AtmUni);
	ASSERT( AtmUniVc);

	HashKey = GET_VC_HASH_KEY( AtmUniVc->VpiVci);
	DBGPRINT( DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniAddVcToTable Key = %d\n",HashKey));
	INSERT_HEAD_LIST( &(AtmUni->VcHashTable[ HashKey]),(PLIST_NODE)AtmUniVc);
} // AtmUniAddVcToTable

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniRemoveVcFromTable
//
//  ABSTRACT:
//		Removes the given VC from the ATM UNI's VC table
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		Caller must hold the VC list lock prior to calling this function
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniRemoveVcFromTable(
	PATM_UNI	AtmUni,
	PATM_UNI_VC	AtmUniVc
)
{
	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniRemoveVcFromTable\n"));

	ASSERT( AtmUni);
	ASSERT( AtmUniVc);

	PARAM_NOT_USED( AtmUni);

	REMOVE_ENTRY_LIST( (PLIST_NODE)AtmUniVc);
} // AtmUniRemoveVcFromTable

///////////////////////////////////////////////////////////////////////////////
//
//  NAME: AtmUniDeleteAllVcs
//
//  ABSTRACT:
//		Deletes all VC's for the given ATM UNI
//
//  RETURNS:
//		void
//
//  ASSUMPTIONS:
//		The caller must hold the VC list lock prior to calling this function.
//
///////////////////////////////////////////////////////////////////////////////
void
AtmUniDeleteAllVcs( PATM_UNI AtmUni)
{
	PATM_UNI_VC	AtmUniVc;
	ULONG		i;

	DBGPRINT(
		DBG_PRINT_MODULE_ATMUNI_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUniDeleteAllVcs\n"));

	ASSERT( AtmUni);

    for( i = 0; i < ATM_UNI_VC_HASH_TABLE_SIZE; i++ ) 
    {
	    AtmUniVc = (PATM_UNI_VC)GET_FIRST_ENTRY_FROM_LIST( &(AtmUni->VcHashTable[i]));

	    while ( AtmUniVc ) 
        {
		    if ( AtmUniVc->State == ATM_UNI_VC_STATE_ACTIVE)
                AtmUniDeactivateVc( AtmUniVc);

		    AtmUniDeleteVc( AtmUniVc);

            /* Keep popping the list and delete the Vc */
            AtmUniVc = (PATM_UNI_VC)GET_FIRST_ENTRY_FROM_LIST( &(AtmUni->VcHashTable[i]));
	    }
    }
} // AtmUniDeleteAllVcs
#ifdef _NTDDK_
void AtmUniGetAtmStats(
    PATM_UNI         AtmUni,
    PATM_STATISTICS  atmStats
    )
{
	atmStats->txCellsOk = AtmUni->Stats.TxCellsOk;
    atmStats->rxCellsOk = AtmUni->Stats.RxCellsOk;
    atmStats->rxCellsDropped = AtmUni->Stats.RxCellsDropped;
    atmStats->rxInvalidVpiVci = AtmUni->Stats.RxInvalidVpiVci;
    atmStats->rxCrc32 = AtmUni->Stats.RxCrcError;
	atmStats->OAMLoopBkSuccess = AtmUni->Stats.OAMLoopBkSuccess;
}
#elif defined(linux)
void AtmUniGetAtmStats( PATM_UNI AtmUni, PATM_UNI_STATS atmStats)
{
	if( !AtmUni )
	{	
		DBGPRINT( DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_ERR,
			("Invalid UNI HANDLE While Getting Atm Statistics\n"));
		return;
	}

	*atmStats = AtmUni->Stats;
}
#endif // _NTDDK_

void AtmUniClearAtmStats(PATM_UNI AtmUni)
{
    AtmUni->Stats.TxPdusOk = 0;
    AtmUni->Stats.RxPdusOk = 0;
    AtmUni->Stats.TxCellsOk = 0;
    AtmUni->Stats.RxCellsOk = 0;
    AtmUni->Stats.RxCellsDropped = 0;
    AtmUni->Stats.RxInvalidVpiVci = 0;
    AtmUni->Stats.RxCrcError = 0;
	AtmUni->Stats.OAMLoopBkSuccess = 0;
	AtmUni->Stats.OAMLoopBkFailure = 0;
}  

///////////////////////////////////////////////////////////////////////////////
//  NAME: AtmProcessFunction
//
//  ABSTRACT:   Tx buffer retrieved from process list and PhySend called if
//				device trained and USB layer is OPERATIONAL.
///////////////////////////////////////////////////////////////////////////////
void AtmProcessFunction( IN PATM_UNI AtmUni)
{
	PPROCESS_DATA_BUFFER	ProcessBuffer;


	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);

    if ( (AtmUni->AtmUniState & ATM_UNI_STATE_LAYER_DOWN) ||
         (IS_LIST_EMPTY( &AtmUni->ProcessBufferTxList)) )
    {
		AtmUni->NeedQueueAtmProcess = TRUE; // set flag to run queue routine upon new Tx buffer
    }
	else
	{
		ProcessBuffer = (PPROCESS_DATA_BUFFER)REMOVE_TAIL_LIST(&(AtmUni->ProcessBufferTxList));		
		(AtmUni->PhySend)( AtmUni->PhyHandle, ProcessBuffer->Buffer, NULL);
        AtmUniFreeProcessBuffer( AtmUni, ProcessBuffer);
	}

    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
}

/* FIXME: No reason for this function to be called by other module/layer */
int AtmUniGetRfc1483Mode(ATM_UNI_HANDLE	AtmUniVcHandle)
{
	PATM_UNI_VC	AtmUniVc = (PATM_UNI_VC	)AtmUniVcHandle ;

    DBGPRINT(DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetRfc1483Mode = %d\n " ,AtmUniVc->Rfc1483Mode));

    ASSERT(AtmUniVcHandle);

   return (AtmUniVc->Rfc1483Mode);
}

/* FIXME: No reason for this function to be called by other module/layer */
BOOLEAN AtmUniSetRfc1483Mode(ATM_UNI_HANDLE AtmUniVcHandle, int Rfc1483Mode)
{
	PATM_UNI_VC	AtmUniVc = (PATM_UNI_VC	)AtmUniVcHandle ;

   DBGPRINT(DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniSetRfc1483Mode = %d\n " ,Rfc1483Mode));

    ASSERT(AtmUniVcHandle);

    AtmUniVc->Rfc1483Mode = Rfc1483Mode;
return TRUE;
}

/* FIXME: No reason for the API1577.c file to be calling ATMUNI functions */
ATM_UNI_HANDLE AtmUniIsVCExist(ATM_UNI_HANDLE AtmHandle , unsigned int VPI , unsigned int VCI )
{
	PATM_UNI_VC AtmUniVC;
	ULONG VpiVci = SET_ATM_VPIVCI( VPI, VCI);

   DBGPRINT(DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniIsVCExist\n"));

    AtmUniVC = AtmUniLookupVc(AtmHandle,     VpiVci);
return AtmUniVC;
}

/* FIXME: No reason for the API1577.c file to be calling ATMUNI functions */
BOOLEAN AtmUniSetRfc1577Encap(ATM_UNI_HANDLE AtmUniVcHandle , PUCHAR Encap , USHORT EncapLen )
{
	PATM_UNI_VC	AtmUniVc = (PATM_UNI_VC	)AtmUniVcHandle ;

   DBGPRINT(DBG_PRINT_MODULE_ATMUNI_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniSetRfc1577Encap\n"));

    ASSERT(AtmUniVcHandle);
    
    AtmUniVc->ArpEncapsulation = Encap;
	AtmUniVc->ArpEncapsulationLength = EncapLen;

return TRUE;
}

/* FIXME: No reason for the API1577.c file to be calling ATMUNI functions */
PXFER_DATA_BUFFER AtmUniGetTxBuffer( ATM_UNI_HANDLE AtmHandle )
{
    PATM_UNI AtmUni = AtmHandle;

	DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniGetTxBuffer\n"));

    ASSERT(AtmUni);

	return (AtmUniGetTransmitBuffer( AtmUni));
}

/* FIXME: No reason for the API1577.c file to be calling to free the TxBuffer */
BOOLEAN AtmUniFreeTxBuffer(ATM_UNI_HANDLE AtmHandle, PXFER_DATA_BUFFER TxBuffer)
{
    PATM_UNI AtmUni = AtmHandle;

	DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
		("AtmUniFreeTxBuffer\n"));

	NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
	AtmUniFreeTransmitBuffer( AtmUni, TxBuffer);
	NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);

    return TRUE;
}

/* FIXME: */
ATM_UNI_USER_HANDLE AtmUniGetUserHandle(ATM_UNI_HANDLE AtmHandle)
{
    PATM_UNI AtmUni = AtmHandle;

    ASSERT(AtmHandle);
    return (AtmUni->UserHandle);
}


void AtmUni_OAM_IF(ATM_UNI_HANDLE AtmHandle, POAMLOOPBACK_STRUCT oamPtr)
{
    PXFER_DATA_BUFFER  LbBuffer= NULL;
    PATM_UNI AtmUni = AtmHandle;

    if ( (LbBuffer = AtmUniGetTransmitBuffer( AtmUni)) == NULL )
    {
       DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
		    ("No Tx Buffer available or reached the max limit-loopback\n"));
       
       return;
    }
   
	LbBuffer->IsLoopback = TRUE;
    /* 54th byte is required to properly interface to the PCU
    ** therefore add 1 to the length
    */
#ifndef QUASAR	
    LbBuffer->Size = sizeof(ATM_CELL_TXFR) + sizeof(UCHAR);
#else
    LbBuffer->Size = sizeof(ATM_CELL_TXFR);
#endif

	OAMCreateCell ((PATM_CELL_TXFR)(LbBuffer->Data),
                   oamPtr->type,
                   oamPtr->vpi,
                   oamPtr->vci,
                   oamPtr->pti,
                   &(oamPtr->reserved[0])); // Generate loopback cell

    /* If pvc is in the table just send the data, else add to loopback check */
    if ( AtmUniLookupVc( AtmUni, SET_ATM_VPIVCI( oamPtr->vpi, oamPtr->vci)) == NULL )
    {
        if ( !AtmUni->LbAtmUniVc ) /* If pvc does not exist then create it */
            AtmUni->LbAtmUniVc = AtmUniCreateVc( AtmUni, NULL, NULL, NULL, NULL);

        if ( AtmUni->LbAtmUniVc )
        {
            AtmUni->LbAtmUniVc->CellHeader[0] = (UCHAR)( (0 << 4) | ((oamPtr->vpi & 0xF0) >> 4));
	        AtmUni->LbAtmUniVc->CellHeader[1] = (UCHAR)( (oamPtr->vpi << 4) | ((oamPtr->vci & 0xF000) >> 12));
	        AtmUni->LbAtmUniVc->CellHeader[2] = (UCHAR)( (oamPtr->vci & 0x0FF0) >> 4);
	        AtmUni->LbAtmUniVc->CellHeader[3] = (UCHAR)( (oamPtr->vci & 0x000F) << 4);
	        AtmUni->LbAtmUniVc->CellHeader[4] = 0;

            AtmUni->LbVpiVci = SET_ATM_VPIVCI( oamPtr->vpi, oamPtr->vci);
        }
    }
   
	if ( !AtmUniQueueTransmitBuffer( AtmUni, LbBuffer) )
    {
        NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		AtmUniFreeTransmitBuffer( AtmUni, LbBuffer);
		NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
    }
}

void AtmUniSendF4F5_Loopback (ATM_UNI_HANDLE AtmHandle, PLOOPBACK_VPIVCI VpiVci)
{
    PXFER_DATA_BUFFER	LbBuffer;
    PATM_UNI AtmUni = AtmHandle;

	if ( (LbBuffer = AtmUniGetTransmitBuffer( AtmUni)) != NULL )
	{
		LbBuffer->IsLoopback = TRUE;
        /* 54th byte is required to properly interface to the PCU
        ** therefore add 1 to the length
        */
#ifndef QUASAR	    
        LbBuffer->Size = sizeof(ATM_CELL_TXFR) + sizeof(UCHAR);
#else
        LbBuffer->Size = sizeof(ATM_CELL_TXFR);
#endif
		Generate_Loopback_Cell ((PATM_CELL_TXFR)(LbBuffer->Data), VpiVci->Vpi, VpiVci->Vci);	// Generate loopback cell
		
        if ( !AtmUniQueueTransmitBuffer( AtmUni, LbBuffer) )
        {
            NdisAcquireSpinLock( &AtmUni->TxBufferFreeListLock);
		    AtmUniFreeTransmitBuffer( AtmUni, LbBuffer);
		    NdisReleaseSpinLock( &AtmUni->TxBufferFreeListLock);
        }
	}
	else
	{
	  	DBGPRINT(DBG_PRINT_MODULE_ATMUNI_TX, DBG_PRINT_LEVEL_ERR,
			("No Tx Buffer available for loopback\n"));
	}	
}
		
