////////////////////////////////////////////////////////////////////////////////
//
// GlobespanVirata Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobespanVirata Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998-2002 GlobespanVirata Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.1 $
//     $Modtime: 8/02/02 9:23a $
//     $Author: kattre $
//
// ABSTRACT:
//		Contains initialization and shutdown code for the NDIS interface
//
////////////////////////////////////////////////////////////////////////////////

#ifdef _NTDDK_
#include "ndis.h"
#else
#include "GpSnull.h"
#endif //_NTDDK_

#include "atmusb.h"
#ifdef _NTDDK_
#include "parse.h"
#endif // _NTDDK_

#if defined (ATM_MINIPORT)
	#include "atmmp.h"
#elif defined (LAN_MINIPORT) || defined(GSI_LAN)
	#include "rfc1483.h"
    #include "rfc1577.h"
#elif defined (WAN_MINIPORT) || defined(GSI_WAN)
#include "rfc2364.h"
#endif // ATM_MINIPORT

#ifdef _NTDDK_
#include "usbcpl.h"

NDIS_HANDLE  gNdisWrapperHandle;
UCHAR DeviceBitMap = 0;
PDRIVER_OBJECT pDriverObjectTmp;
PDRIVER_DISPATCH gNdisMajorFunction[IRP_MJ_MAXIMUM_FUNCTION + 1];
PATM_USB_INSTANCE gInstance = NULL;


//------------------------------------------------------------------------------
//
// Global Variable Definitions
//
//------------------------------------------------------------------------------
ANSI_STRING Global_AtmUsbVendorDescriptor = 
				ANSI_STRING_CONST( ATM_USB_VENDOR_DESCRIPTOR);


//------------------------------------------------------------------------------
//
// Extermal Function Declarations
//
//------------------------------------------------------------------------------
extern
ATM_PHY_STATUS
UsbPhyBind
(
	PATM_PHY_BIND_INPUT		BindInput,
	PATM_PHY_BIND_OUTPUT	BindOutput
);

extern NTSTATUS 
GsiDispatch
(
	PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp
); 

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DriverEntry
//
// ABSTRACT:
//		DriverEntry is the first function called in the NDIS driver when the 
//		driver is loaded.  Upon successful completetion, the driver is
//  	completely initialized and ready for operation.
//
// RETURNS:
//		NDIS_STATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NDIS_STATUS
DriverEntry(
	IN	PDRIVER_OBJECT	DriverObject,
	IN	PUNICODE_STRING	RegistryPath
)
{
    NDIS_STATUS                  	NdisStatus = NDIS_STATUS_FAILURE;
    NDIS_MINIPORT_CHARACTERISTICS   Characteristics = {0};
    NDIS_HANDLE                     WrapperHandle;
    BOOLEAN							Success;

	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("DriverEntry\n"));

	//
	// Initialize the NDIS wrapper.
	//
	NdisMInitializeWrapper(
	   &WrapperHandle, 
	   DriverObject, 
	   RegistryPath, 
	   NULL);

	if (NULL == WrapperHandle) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("NdisMInitializeWrapper returned NULL WrapperHandle\n"));

		return( NDIS_STATUS_FAILURE);
	}

	//
	// Initialize the ATM UNI module
	//
	if( !AtmUniInitialize() ) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("AtmUniInitialize failed\n"));
	  return( NDIS_STATUS_FAILURE);
	} // if

   NdisZeroMemory(&Characteristics, sizeof(Characteristics));

   //
	//	Initialize the miniport characteristics.
	//
	Characteristics.MajorNdisVersion 		    = ATM_USB_NDIS_MAJOR_VERSION;
	Characteristics.MinorNdisVersion 		    = ATM_USB_NDIS_MINOR_VERSION;
	Characteristics.CheckForHangHandler 	    = AtmUsbCheckForHang;
	Characteristics.DisableInterruptHandler   = NULL;
	Characteristics.EnableInterruptHandler 	= NULL;
	Characteristics.HaltHandler 			    = AtmUsbHalt;
	Characteristics.HandleInterruptHandler 	= NULL;
	Characteristics.InitializeHandler 		= AtmUsbInitialize;
	Characteristics.ISRHandler 				    = NULL;
	Characteristics.ResetHandler 			    = AtmUsbReset;
#if defined ( ATM_MINIPORT)
	Characteristics.ReturnPacketHandler 	    = AtmMiniportReturnPacket;
	Characteristics.CoSendPacketsHandler 	    = AtmMiniportSendPackets;
	Characteristics.CoCreateVcHandler 		= AtmMiniportCreateVc;
	Characteristics.CoDeleteVcHandler 		= AtmMiniportDeleteVc;
	Characteristics.CoActivateVcHandler 	    = AtmMiniportActivateVc;
	Characteristics.CoDeactivateVcHandler 	= AtmMiniportDeactivateVc;
	Characteristics.CoRequestHandler 		    = AtmMiniportRequest;
#elif defined (LAN_MINIPORT)
	Characteristics.TransferDataHandler		= Rfc1483TransferData;
	Characteristics.SendHandler				    = NdisRfc1577Send;
	Characteristics.QueryInformationHandler	= Rfc1483QueryInformation;
	Characteristics.SetInformationHandler	    = Rfc1483SetInformation;
#elif defined (WAN_MINIPORT)
   Characteristics.QueryInformationHandler   = Rfc2364QueryInformation;
   Characteristics.SetInformationHandler     = Rfc2364SetInformation;
   Characteristics.WanSendHandler            = (WM_SEND_HANDLER)Rfc2364Send;
   Characteristics.Reserved                  = NDIS_USE_WAN_WRAPPER;
#endif // ATM_MINIPORT LAN_MINIPORT WAN_MINIPORT

	//
	//	Register the miniport with NDIS.
	//
	NdisStatus = NdisMRegisterMiniport(
					WrapperHandle,
					&Characteristics,
					sizeof(Characteristics));

	if ( NDIS_STATUS_SUCCESS != NdisStatus)	{
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("NdisMRegisterMiniport failed with status %x\n",NdisStatus));
		
		return( NDIS_STATUS_FAILURE);
	}

   /* Needed for 9x and Me interface */
	pDriverObjectTmp = DriverObject; /* Temp save it. It will be used in AtmUsbInitialize to hook CPL*/

    // Zero out NDIS dispatch array
    NdisZeroMemory(gNdisMajorFunction, (IRP_MJ_MAXIMUM_FUNCTION + 1));

    // Save the original NDIS Dispatch Functions
	gNdisMajorFunction[IRP_MJ_CREATE]			= DriverObject->MajorFunction[IRP_MJ_CREATE];
	gNdisMajorFunction[IRP_MJ_CLOSE]			= DriverObject->MajorFunction[IRP_MJ_CLOSE];
	gNdisMajorFunction[IRP_MJ_SHUTDOWN]		    = DriverObject->MajorFunction[IRP_MJ_SHUTDOWN];
	gNdisMajorFunction[IRP_MJ_CLEANUP]			= DriverObject->MajorFunction[IRP_MJ_CLEANUP];
	gNdisMajorFunction[IRP_MJ_DEVICE_CONTROL]	= DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL];
	gNdisMajorFunction[IRP_MJ_PNP]				= DriverObject->MajorFunction[IRP_MJ_PNP];
	
   /* Save the WrapperHandle for the Win2K and WinXP */
   gNdisWrapperHandle = WrapperHandle; /* Temp save it. It will be used in AtmUsbInitialize to hook CPL*/
   
	return(NdisStatus);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbInitialize
//
// ABSTRACT:
//		Called by NDIS within the context of NdisMRegisterMiniport, this 
//		function carries out the initialization of a driver instance.  NDIS will
//		not submit any requests to the driver until this function has completed.  
//		If this function returns NDIS_STATUS_SUCCESS, then the underlying NIC
//		( or virtual NIC) has been set up in an operational state.
//
// RETURNS:
//		NDIS_STATUS
//
// ASSUMPTIONS:
//		Runs at IRQL PASSIVE_LEVEL
//
////////////////////////////////////////////////////////////////////////////////
NDIS_STATUS 
AtmUsbInitialize(
	OUT PNDIS_STATUS 	OpenErrorStatus,
	OUT PUINT		 	SelectedMediumIndex,
	IN PNDIS_MEDIUM 	MediumArray,
	IN UINT 			MediumArraySize,
	IN NDIS_HANDLE 		MiniportAdapterHandle,
	IN NDIS_HANDLE 		WrapperConfigurationContext
)
{
	NDIS_STATUS			NdisStatus = NDIS_STATUS_SUCCESS;
	PATM_USB_INSTANCE	Instance;
	NDIS_MEDIUM			AtmUsbMedium;
	BOOLEAN				Success;
   	PDEVICE_OBJECT      UsbDeviceHandle = NULL;

	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbInitialize\n"));

	// we never return NDIS_STATUS_OPEN_ERROR
    *OpenErrorStatus = 0;

	//
	// Look for our medium type (either ATM or 802.3 depeding on
	// whether we are an ATM miniport or a LAN miniport)
	//
#if defined (ATM_MINIPORT)
	AtmUsbMedium = NdisMediumAtm;
#elif defined (LAN_MINIPORT)
	AtmUsbMedium = NdisMedium802_3;
#elif defined(WAN_MINIPORT)
	AtmUsbMedium = NdisMediumWan;
#endif // ATM_MINIPORT

   while ( MediumArraySize--)
   {
       if ( MediumArray[MediumArraySize] == AtmUsbMedium)
       {
        	break;
		}
	}
   if ( MediumArray[MediumArraySize] != AtmUsbMedium)
   {
		DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL,	DBG_PRINT_LEVEL_ERR,
			("Invalid Medium (expecting %d)\n",AtmUsbMedium));
        return( NDIS_STATUS_UNSUPPORTED_MEDIA);
    }
    *SelectedMediumIndex = MediumArraySize;

	//
	// Allocate the per-instance data structure
	//
	ALLOCATE_MEMORY( &Instance,sizeof( ATM_USB_INSTANCE));
	if ( NULL == Instance) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Unable to allocate memory for AtmUsbInstance\n"));
		return( NDIS_STATUS_RESOURCES);
	}
	
	Instance->MiniportAdapterHandle = MiniportAdapterHandle;

	//
	// Set our NDIS attributes, indicating that we are a deserialized 
	// intermediate driver
	//
	NdisMSetAttributesEx(
        MiniportAdapterHandle,
        Instance,
        0,							// default check-for-hang interval
        NDIS_ATTRIBUTE_DESERIALIZE ,//| NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND ,
        0							// no AdapterType for intermediate driver
        );

    Instance->NeedQueueRcvProcess = TRUE;
	INITIALIZE_LIST_HEAD( &(Instance->PendingReceiveList));
	NdisAllocateSpinLock( &(Instance->PendingReceiveListLock));
	NdisMInitializeTimer(
		&(Instance->PendingReceiveTimer),
		MiniportAdapterHandle,
		AtmUsbPendingReceiveTimerCallback,
		Instance);

	NdisInitializeEvent( &Instance->PendingReceiveTimerEvent);
    NdisSetEvent( &Instance->PendingReceiveTimerEvent);

	NdisInitializeEvent( &Instance->RxPcktsWithNdisZeroEvent);
	NdisSetEvent( &Instance->RxPcktsWithNdisZeroEvent);

    NdisAllocateSpinLock( &Instance->IrpSpinLock);
    NdisInitializeEvent( &Instance->IrpEvent);
    NdisSetEvent( &Instance->IrpEvent);

	//
	// Parse the registry parameters (registry parameter values are stored
	//   in the instance data structure)
	//
    if ( !ParseRegistryParameters( Instance,WrapperConfigurationContext)) {
		AtmUsbHalt( Instance);
        return( NDIS_STATUS_FAILURE);
    }

    Instance->WrapperConfigurationContext = WrapperConfigurationContext;
   
	//
	// Set the DebugLevel and DebugModule based on the registry parameters
	//
	DbgSetDbgPrintLevel( Instance->DbgPrintLevel);
	DbgSetDbgPrintModule( Instance->DbgPrintModule);

	//
	// Get the device handle for the USB device
	//
	NdisMGetDeviceProperty( 
	  MiniportAdapterHandle,
	  &UsbDeviceHandle, // Physical Device Object
	  NULL,
	  NULL,
	  NULL,
	  NULL);

	ASSERT( UsbDeviceHandle);

	if ( NULL == UsbDeviceHandle) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("NdisMGetDeviceProperty returned Null\n"));
		AtmUsbHalt( Instance);
		return( NDIS_STATUS_FAILURE);
	}

#if defined (ATM_MINIPORT)
	//
	// Initialize the ATM Miniport specific portion of the driver
	//
	Success = AtmMiniportInitialize( Instance,UsbPhyBind,UsbDeviceHandle);

	if (!Success) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("AtmMiniportInitialize failed\n"));
		AtmUsbHalt( Instance);
		return( NDIS_STATUS_FAILURE);
	}
#elif defined (LAN_MINIPORT)
	//
	// Initialize the RFC1483 specific portion of the driver
	//
	Success = Rfc1483Initialize( Instance,UsbPhyBind,UsbDeviceHandle);

	if (!Success) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Rfc1483Initialize failed\n"));
		AtmUsbHalt( Instance);
		return( NDIS_STATUS_FAILURE);
	}

//Init Rfc1577
    Success = Rfc1577Initialize( Instance );

	if (!Success) {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Rfc1577Initialize failed\n"));
		AtmUsbHalt( Instance);
		return( NDIS_STATUS_FAILURE);
	}

#elif defined (WAN_MINIPORT)
   Success = Rfc2364Initialize( Instance ,	UsbPhyBind, UsbDeviceHandle );
	if (!Success)
    {
		DBGPRINT(
			DBG_PRINT_MODULE_NDIS_ALL,
			DBG_PRINT_LEVEL_ERR,
			("Rfc2364Initialize failed\n"));
		AtmUsbHalt( Instance);
		return( NDIS_STATUS_FAILURE);
	}
#endif // ATM_MINIPORT

	Instance->Status |= ATM_USB_STATUS_INITIALIZED;

	//
	// Register a shutdown handler
	//
	NdisMRegisterAdapterShutdownHandler( 
		MiniportAdapterHandle,
		Instance,
		AtmUsbShutdown);

   /* We should not keep using global variables */
   AtmUsbCreateDevice(Instance, pDriverObjectTmp, gNdisWrapperHandle);
	return(NdisStatus);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbShutdown
//
// ABSTRACT:
//		Called when the system is being shut down
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
AtmUsbShutdown( PATM_USB_INSTANCE Instance)
{
	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbShutdown\n"));

	ASSERT( Instance);

	DslShutdown( Instance->UsbPhyHandle);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbHalt
//
// ABSTRACT:
//		Releases resources allocated for an instance
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		Runs at IRQL PASSIVE_LEVEL
//
////////////////////////////////////////////////////////////////////////////////
void
AtmUsbHalt( PATM_USB_INSTANCE Instance)
{
    NDIS_STRING                 symbolicNameUniString;	

	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbHalt\n"));

	ASSERT( Instance);

	Instance->Status |= ATM_USB_STATUS_HALTING;

	if ( Instance->Status & ATM_USB_STATUS_INITIALIZED)	
    {
		// Remove our shutdown handler
		NdisMDeregisterAdapterShutdownHandler( Instance->MiniportAdapterHandle);

		// Stop allowing our IOCTLs
		AtmUsbUnHookDevice( Instance);

        // Stop allowing receive indications
        AtmUsbShutdownReceive ( Instance);

		//	Wait for any pending receive indications to complete
		if (Instance->RxPcktsWithNdisCount != 0) {
			NdisWaitEvent(&(Instance->RxPcktsWithNdisZeroEvent), 10000);
			DBGPRINT(
				DBG_PRINT_MODULE_NDIS_ALL,
				DBG_PRINT_LEVEL_ERR,
				("Timeout waiting 10s for Ndis to return Rx packets!\n"));
			// Something went wrong and Ndis did not return our memory,
			// the system is in bad shape at this point and a crash 
			// will happen if anybody accesses the unreturned packets 
			// after we complete the halt. 
			// FIXME ! 	Would it be better not to continue releasing any 
			// resources and just leak the memory ?
		}

		//
		// If we have successfully initialized the instance, call the appropriate
		// shutdown routine(s) to release resources and restore the device to
		// a known state.
		//
#if defined (ATM_MINIPORT)
		AtmMiniportShutdown( Instance);

#elif defined (LAN_MINIPORT)
        Instance->Rfc1577Config.Rfc1577Close( Instance->Rfc1577Config.Rfc1577Context );
        Instance->Rfc1577Config.Rfc1577Context = NULL;

		Rfc1483Shutdown( Instance);

#elif defined (WAN_MINIPORT)
		Rfc2364Shutdown( Instance);
#endif // ATM_MINIPORT

		AtmUsbDeleteDevice( Instance);
	}

	NdisFreeSpinLock( &Instance->PendingReceiveListLock);
    NdisFreeSpinLock( &Instance->IrpSpinLock);
    
	// Release the instance data structure
    FREE_MEMORY( Instance,sizeof( ATM_USB_INSTANCE));
} // AtmUsbHalt

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbCheckForHang
//
// ABSTRACT:
//	Called periodically by NDIS to determine if the instance is broken
//
// RETURNS:
//	TRUE
//		The instance is broken.  This causes NDIS to call AtmUsbReset
//	FALSE
//		The instance is working correctly
//
// ASSUMPTIONS:
//	None.
//
////////////////////////////////////////////////////////////////////////////////
BOOLEAN
AtmUsbCheckForHang( PATM_USB_INSTANCE	Instance )
{
	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbCheckForHang\n"));

	ASSERT( Instance);		

	//
	// FIXME!
	// TODO: Do not assume nothing will ever fail. Implement checks
	//		 here to verify the hardware & software are working ok.

   return( (Instance->Status & ATM_USB_STATUS_HUNG) ? TRUE: FALSE);
} // AtmUsbCheckForHang

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbReset
//
// ABSTRACT:
//		Called by NDIS to reset an instance
//
// RETURNS:
//		NDIS_STATUS
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
NDIS_STATUS
AtmUsbReset (
    PBOOLEAN 		AddressingReset,
    PATM_USB_INSTANCE Instance
)
{
	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbReset\n"));

	//
	// Our addressing information is maintained in SW, so we never need
	//   an addressing reset
	//
	*AddressingReset = FALSE;

	//
	// FIXME!
	// TODO: Ensure that all pending requests to this instance are cancelled
	// TODO: Stop the hardware and reinitialize it.
	// ASSERT( FALSE);

	return( NDIS_STATUS_SUCCESS);

} // AtmUsbReset


////////////////////////////////////////////////////////////////////////////////
// NAME: AtmUsbShutdownReceive
//
// ABSTRACT:
//		shutdown receive indications during halt / suspend
//
// RETURNS:
//		void
////////////////////////////////////////////////////////////////////////////////
void AtmUsbShutdownReceive ( PATM_USB_INSTANCE Instance)
{
    PATM_USB_CALLBACK_CONTEXT 	CallbackContext;


    if ( (Instance->Status & ATM_USB_STATUS_HALTING) || (Instance->Status & ATM_USB_STATUS_SUSPENDING) )
    {
        // Dequeue each pending receive and call the callback function
        while ( TRUE )
        {
            NdisAcquireSpinLock( &(Instance->PendingReceiveListLock));

   	        if ( IS_LIST_EMPTY( &Instance->PendingReceiveList) )
            {
		        NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));

                // Cancel PendingReceiveTimer
	            SafeCancelTimer(
		            &(Instance->PendingReceiveTimer),
		            &(Instance->PendingReceiveTimerEvent),
		            1000); // We may need to wait for 1 receive data indication so timeout after 1 second

                return;
            }
            else    // complete all pending receive callbacks
	        {
		        CallbackContext = (PATM_USB_CALLBACK_CONTEXT)REMOVE_TAIL_LIST( &(Instance->PendingReceiveList));
                NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));

		        // Call the callback function
		        (CallbackContext->MiniportCallback)( 
			        Instance,
			        CallbackContext->MiniportContext,
                    CallbackContext->AtmUniVc
                    );

		        // Free the callback context
		        FREE_MEMORY( CallbackContext,sizeof( ATM_USB_CALLBACK_CONTEXT));
	        }
        }
    }
}

#endif // _NTDDK_

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbScheduleReceiveCallback
//
// ABSTRACT:
//		The given MiniportCallback function is queued to run during the
//		receive timer context and the receive timer is scheduled to fire.
//		
//		This function is required to avoid a Windows 98 NDIS-USB bug caused
//		when receive indications are done outside of miniport context (i.e.
//		directly from a USB-initiated DPC call)
//
// RETURNS:
//		TRUE
//			The miniport callback function was successfully queued
//		FALSE
//			The miniport callback could not be queued
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
BOOLEAN
AtmUsbScheduleReceiveCallback(
	PATM_USB_INSTANCE			Instance,
	ATM_USB_MINIPORT_CALLBACK	MiniportCallback,
	PVOID						MiniportContext,
    ATM_UNI_HANDLE              AtmUniVc
)
{
	PATM_USB_CALLBACK_CONTEXT 	CallbackContext;


	DBGPRINT(
		DBG_PRINT_MODULE_NDIS_ALL,
		DBG_PRINT_LEVEL_TRACE,
		("AtmUsbScheduleReceiveCallback\n"));
	
	ASSERT( Instance);
	ASSERT( MiniportCallback);


    NdisAcquireSpinLock( &(Instance->PendingReceiveListLock));

    if ( (Instance->Status & ATM_USB_STATUS_HALTING) || (Instance->Status & ATM_USB_STATUS_SUSPENDING) )
    {
        NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));
        return FALSE;
    }
    else
    {
        // Allocate a callback context and push it on the pending receive list
	    ALLOCATE_MEMORY( &CallbackContext,sizeof(ATM_USB_CALLBACK_CONTEXT));
	    if( NULL == CallbackContext) 
        {
		    DBGPRINT(
			    DBG_PRINT_MODULE_NDIS_ALL,
			    DBG_PRINT_LEVEL_ERR,
			    ("Context allocation failed\n"));

            NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));
     	    return FALSE;
	    }

	    CallbackContext->MiniportCallback	= MiniportCallback;
	    CallbackContext->MiniportContext	= MiniportContext;
        CallbackContext->AtmUniVc           = AtmUniVc;

        INSERT_HEAD_LIST( &(Instance->PendingReceiveList), (PLIST_NODE)CallbackContext);	

	    // run queue processing routine if not currently running
	    if ( Instance->NeedQueueRcvProcess )
	    {
		    Instance->NeedQueueRcvProcess = FALSE;

            // Schedule a timer callback with zero delay
	        SafeSetTimer(
		        Instance,
		        &Instance->PendingReceiveTimerEvent,
		        &Instance->PendingReceiveTimer,
		        0);
        }
    }

    NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));
	return TRUE;

} // AtmUsbScheduleReceiveCallback

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbPendingReceiveTimerCallback
//
// ABSTRACT:
//		Called when the pending receive timer fires.  For each entry in the
//		pending receive list, the associated callback function is called.
//
// RETURNS:
//		void
//
// ASSUMPTIONS:
//		None.
//
////////////////////////////////////////////////////////////////////////////////
void
AtmUsbPendingReceiveTimerCallback(
	PVOID				SystemSpecific1,
	PATM_USB_INSTANCE	Instance,	   
	PVOID				SystemSpecific2,
	PVOID				SystemSpecific3
)
{
	PATM_USB_CALLBACK_CONTEXT 	CallbackContext;


	//DBGPRINT(
	//	DBG_PRINT_MODULE_NDIS_ALL,
	//	DBG_PRINT_LEVEL_TRACE,
	//	("AtmUsbPendingReceiveTimerCallback\n"));

	ASSERT( Instance);

    // Dequeue each pending receive and call the callback function
    while ( TRUE )
    {
        NdisAcquireSpinLock( &(Instance->PendingReceiveListLock));

   	    if ( (Instance->Status & ATM_USB_STATUS_HALTING) || 
             (Instance->Status & ATM_USB_STATUS_SUSPENDING) || 
             (IS_LIST_EMPTY( &Instance->PendingReceiveList)) )
        {
            Instance->NeedQueueRcvProcess = TRUE;
            NdisSetEvent( &Instance->PendingReceiveTimerEvent);
		    NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));	        
            return;
        }
        else    // process Rx queue
	    {
		    CallbackContext = (PATM_USB_CALLBACK_CONTEXT)REMOVE_TAIL_LIST( &(Instance->PendingReceiveList));
            NdisReleaseSpinLock( &(Instance->PendingReceiveListLock));

		    // Call the callback function
		    (CallbackContext->MiniportCallback)( 
			    Instance,
			    CallbackContext->MiniportContext,
                CallbackContext->AtmUniVc
                );

		    // Free the callback context
		    FREE_MEMORY( CallbackContext,sizeof( ATM_USB_CALLBACK_CONTEXT));
	    }
    }

} // AtmUsbPendingReceiveTimerCallback

#ifdef _NTDDK_
////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbCreateDevice
//
// ABSTRACT:
//
// RETURNS:
//		VOID
//
// ASSUMPTIONS:
////////////////////////////////////////////////////////////////////////////////
void AtmUsbCreateDevice(
           PATM_USB_INSTANCE   Instance,
           PDRIVER_OBJECT      DriverObject,
           NDIS_HANDLE         WrapperHandle
)
{
	NDIS_STATUS			NdisStatus = NDIS_STATUS_SUCCESS;
	GSICPL_GLOBALS		*pGlobals = NULL; 

   DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL, DBG_PRINT_LEVEL_TRACE,
           ("AtmUsbCreateDevice\n"));
 
    /*In current implementation we support just two devices*/
	if( (DeviceBitMap & 0x01) == 0 ) /* This is the first device*/
	{
       NdisInitUnicodeString(&Instance->symbolicNameUniString, PDSLPTR_SYMBOLIC_NAME0);
       NdisInitUnicodeString(&Instance->deviceNameUniString, PDSLPTR_DEVICE_NAME0);
//		RtlInitUnicodeString(&Instance->symbolicNameUniString, PDSLPTR_SYMBOLIC_NAME0);
//		RtlInitUnicodeString(&Instance->deviceNameUniString, PDSLPTR_DEVICE_NAME0);
		DeviceBitMap |= 0x01;
		Instance->DeviceIndex = 0;
        gInstance = Instance;
	}
	else if( (DeviceBitMap & 0x02) == 0)/* Second device*/
	{
       NdisInitUnicodeString(&Instance->symbolicNameUniString, PDSLPTR_SYMBOLIC_NAME1);
       NdisInitUnicodeString(&Instance->deviceNameUniString, PDSLPTR_DEVICE_NAME1);
		//RtlInitUnicodeString(&Instance->symbolicNameUniString, PDSLPTR_SYMBOLIC_NAME1);
		//RtlInitUnicodeString(&Instance->deviceNameUniString, PDSLPTR_DEVICE_NAME1);
		DeviceBitMap |= 0x02; 
		Instance->DeviceIndex = 1;
	}
	else
	{
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("ERROR:  We suport just two devices \n"));
        
		return;
   }

   /* We should not keep using global variables */
   Instance->WrapperHandle = WrapperHandle;
   
   /* 9x and Me Interface */
#ifndef NDIS_CALLS
   ALLOCATE_MEMORY(&pGlobals, sizeof( GSICPL_GLOBALS));
	if(pGlobals == NULL)
	{
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
           ("ERROR: Unable ALLOCATE_MEMORY for GSICPL_GLOBALS \n"));

	    return;
    }
	pGlobals->DriverObject = DriverObject;

	NdisStatus = IoCreateDevice(
                  DriverObject,
                  4*sizeof(ULONG),
                  &Instance->deviceNameUniString,
                  FILE_DEVICE_NETWORK,
                  FILE_DEVICE_SECURE_OPEN,
                  FALSE,
                  &pGlobals->DeviceObject
                  );
	if ( NDIS_STATUS_SUCCESS  != NdisStatus)	
	{
       DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL, DBG_PRINT_LEVEL_ERR,
			("IoCreateDevice failed with status %x\n",NdisStatus));
            pGlobals->DeviceObject = NULL;
       return;
	}
	else
	{
		ASSERT(pGlobals->DeviceObject);

		Instance->DeviceObject = pGlobals->DeviceObject;
		pGlobals->DeviceObject->Flags &= ~DO_DEVICE_INITIALIZING;
		pGlobals->DeviceObject->DeviceExtension = pGlobals;
        pGlobals->GspnSignature = GSPN_SIGNATURE;
   }
#endif /* !NDIS_CALLS */

   if ( (DeviceBitMap & 0x02) == 0 )       // Only hook if this is the first instance  
       AtmUsbHookDevice( Instance);
   else if ( gInstance )                   // Unhook first instance if this is the second                    
       AtmUsbUnHookDevice( gInstance);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbDeleteDevice
//
// ABSTRACT:
//
// RETURNS:
//		VOID
//
// ASSUMPTIONS:
////////////////////////////////////////////////////////////////////////////////
void AtmUsbDeleteDevice( PATM_USB_INSTANCE Instance)
{
PDEVICE_OBJECT DeviceObject;  
GSICPL_GLOBALS *pGlobals = NULL; 

   DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL,	DBG_PRINT_LEVEL_TRACE,
		("AtmUsbDeleteDevice\n"));
	
   /* 9x and Me Interface */
#ifndef NDIS_CALLS
	DeviceObject = Instance->DeviceObject;

   if(DeviceObject == NULL)
   {
	    DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
		    ("ERROR:  DeviceObject = NULL \n"));
   
       return;
    }
	pGlobals = DeviceObject->DeviceExtension;
	ASSERT(pGlobals);

	IoDeleteDevice(DeviceObject);
	DeviceObject = NULL;

   // Free memory for IRP handling
	FREE_MEMORY(pGlobals, sizeof(GSICPL_GLOBALS) );
#endif /* !NDIS_CALLS */

	if(Instance->DeviceIndex == 0 )
   {
       gInstance = NULL;
	    DeviceBitMap = DeviceBitMap & 0x2;
   }
	else if(Instance->DeviceIndex == 0x1)
   {
	    DeviceBitMap = DeviceBitMap & 0x1;
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbHookDevice
//
// ABSTRACT:
//
// RETURNS:
//		VOID
//
// ASSUMPTIONS:
////////////////////////////////////////////////////////////////////////////////
void AtmUsbHookDevice( PATM_USB_INSTANCE Instance)
{
NDIS_STATUS        NdisStatus = NDIS_STATUS_SUCCESS;
PDRIVER_OBJECT	    DriverObject;
PDEVICE_OBJECT	    DeviceObject;  
GSICPL_GLOBALS	    *pGlobals = NULL; 
PDRIVER_DISPATCH   MajorFunctions[IRP_MJ_MAXIMUM_FUNCTION + 1];

   DBGPRINT( DBG_PRINT_MODULE_NDIS_ALL, DBG_PRINT_LEVEL_TRACE,
       ("AtmUsbHookDevice\n"));

   /* 9x and Me Interface */
#ifndef NDIS_CALLS
	DeviceObject = Instance->DeviceObject;
    if(!DeviceObject)
    {
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("ERROR:  DeviceObject = NULL \n"));
    
       return;
    }
	else 
	{
		pGlobals = DeviceObject->DeviceExtension;
		ASSERT(pGlobals);

		DriverObject = pGlobals->DriverObject;

		// Add our Dispatch Functions
		DriverObject->MajorFunction[IRP_MJ_CREATE]			= GsiDispatch;
		DriverObject->MajorFunction[IRP_MJ_CLOSE]			= GsiDispatch;
		DriverObject->MajorFunction[IRP_MJ_SHUTDOWN]		= GsiDispatch;
		DriverObject->MajorFunction[IRP_MJ_CLEANUP]			= GsiDispatch;
		DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]	= GsiDispatch;
	    DriverObject->MajorFunction[IRP_MJ_PNP]	            = GsiDispatch;

		// Set up a symbolic name for interaction with the user-mode applications.
		NdisStatus = IoCreateSymbolicLink(&Instance->symbolicNameUniString, &Instance->deviceNameUniString);
		if ( NDIS_STATUS_SUCCESS  != NdisStatus)	
		{
		    DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL,	DBG_PRINT_LEVEL_ERR,
				("IoCreateDevice failed with status %x\n",NdisStatus));
		}
		else
			Instance->Status |= ATM_USB_SLINK_INITIALIZED;		
	}
#endif /* !NDIS_CALLS */
#ifdef NDIS_CALLS
   // init dispatch functions to zero
   NdisZeroMemory( MajorFunctions, (IRP_MJ_MAXIMUM_FUNCTION + 1));

   // set our dispatch functions
	MajorFunctions[IRP_MJ_CREATE]			= (PDRIVER_DISPATCH)GsiDispatch;
	MajorFunctions[IRP_MJ_CLOSE]			= (PDRIVER_DISPATCH)GsiDispatch;
	MajorFunctions[IRP_MJ_SHUTDOWN]		    = (PDRIVER_DISPATCH)GsiDispatch;
	MajorFunctions[IRP_MJ_CLEANUP]			= (PDRIVER_DISPATCH)GsiDispatch;
	MajorFunctions[IRP_MJ_DEVICE_CONTROL]	= (PDRIVER_DISPATCH)GsiDispatch;

   NdisStatus = NdisMRegisterDevice( Instance->WrapperHandle,
                                  &Instance->deviceNameUniString,
                                  &Instance->symbolicNameUniString,
                                  MajorFunctions,
                                  &Instance->DeviceObject,
                                  &Instance->NdisDeviceHandle);

   if ( NDIS_STATUS_SUCCESS  != NdisStatus )	
	{
	    DBGPRINT(DBG_PRINT_MODULE_NDIS_ALL,	DBG_PRINT_LEVEL_ERR,
		    ("NdisMRegisterDevice failed with status %x\n",NdisStatus));
	}
   else
   {
       Instance->Status |= ATM_USB_SLINK_INITIALIZED;
   }
   return;
#endif /* NDIS_CALLS */
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: AtmUsbUnHookDevice
//
// ABSTRACT:
//
// RETURNS:
//		VOID
//
// ASSUMPTIONS:
////////////////////////////////////////////////////////////////////////////////
void AtmUsbUnHookDevice( PATM_USB_INSTANCE Instance)
{
	PDRIVER_OBJECT DriverObject;
	PDEVICE_OBJECT DeviceObject;  
	GSICPL_GLOBALS *pGlobals = NULL; 

   DBGPRINT( DBG_PRINT_MODULE_NDIS_ALL, DBG_PRINT_LEVEL_TRACE,
           ("AtmUsbUnHookDevice\n"));

   /* 9x and Me Interface */
#ifndef NDIS_CALLS
	DeviceObject = Instance->DeviceObject;
   if(DeviceObject == NULL)
   {
		DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
			("ERROR:  DeviceObject = NULL \n"));
    
       return;
   }
#endif /* !NDIS_CALLS */
   
	// Stop allowing our IOCTLs
	if ( Instance->Status & ATM_USB_SLINK_INITIALIZED)
	{
        NdisAcquireSpinLock( &Instance->IrpSpinLock);
        if ( (Instance->Status & ATM_USB_STATUS_HALTING) || (Instance->Status & ATM_USB_STATUS_SUSPENDING) )
        {
            NdisReleaseSpinLock( &Instance->IrpSpinLock);
            NdisWaitEvent( &Instance->IrpEvent, 1000);
        }
        else
        {
            NdisReleaseSpinLock( &Instance->IrpSpinLock);
        }

		Instance->Status = Instance->Status & (~ATM_USB_SLINK_INITIALIZED);	// clear bit
   /* 9x and Me Interface */
#ifndef NDIS_CALLS
       IoDeleteSymbolicLink(&Instance->symbolicNameUniString);
#endif /* !NDIS_CALLS */
#ifdef NDIS_CALLS
       if ( NDIS_STATUS_SUCCESS != (NdisMDeregisterDevice(Instance->NdisDeviceHandle)) )
       {
           DBGPRINT( DBG_PRINT_MODULE_NDIS_ALL, DBG_PRINT_LEVEL_ERR,
                   ("NdisMDeregisterDevice failed with status\n"));
       }
       return;
#endif /* NDIS_CALLS */
   }

#ifndef NDIS_CALLS
	pGlobals = DeviceObject->DeviceExtension;
	ASSERT(pGlobals);

	DriverObject = pGlobals->DriverObject;

    // Restore the original NDIS Dispatch Functions
	DriverObject->MajorFunction[IRP_MJ_CREATE]			= gNdisMajorFunction[IRP_MJ_CREATE];
	DriverObject->MajorFunction[IRP_MJ_CLOSE]			= gNdisMajorFunction[IRP_MJ_CLOSE];
	DriverObject->MajorFunction[IRP_MJ_SHUTDOWN]		= gNdisMajorFunction[IRP_MJ_SHUTDOWN];
	DriverObject->MajorFunction[IRP_MJ_CLEANUP]			= gNdisMajorFunction[IRP_MJ_CLEANUP];
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]	= gNdisMajorFunction[IRP_MJ_DEVICE_CONTROL];
	DriverObject->MajorFunction[IRP_MJ_PNP]				= gNdisMajorFunction[IRP_MJ_PNP];
#else //NDIS_CALLS

	pDriverObjectTmp->MajorFunction[IRP_MJ_CREATE]			= gNdisMajorFunction[IRP_MJ_CREATE];
	pDriverObjectTmp->MajorFunction[IRP_MJ_CLOSE]			= gNdisMajorFunction[IRP_MJ_CLOSE];
	pDriverObjectTmp->MajorFunction[IRP_MJ_SHUTDOWN]		= gNdisMajorFunction[IRP_MJ_SHUTDOWN];
	pDriverObjectTmp->MajorFunction[IRP_MJ_CLEANUP]			= gNdisMajorFunction[IRP_MJ_CLEANUP];
	pDriverObjectTmp->MajorFunction[IRP_MJ_DEVICE_CONTROL]	= gNdisMajorFunction[IRP_MJ_DEVICE_CONTROL];
	pDriverObjectTmp->MajorFunction[IRP_MJ_PNP]				= gNdisMajorFunction[IRP_MJ_PNP];

#endif /* !NDIS_CALLS */
}

#endif // _NTDDK_

////////////////////////////////////////////////////////////////////////////////
//
// NAME: SafeCancelTimer
//
// ABSTRACT:
//		Called to synchronously cancel a timer (if possible).
//		If the current IRQL is higher than IRQL_PASSIVE_LEVEL it is not 
//		possible to sync and we will just call the default NdisMCancelTimer
//		and exit.
//
// RETURNS:
//		TRUE: if the timer was cancelled successfully, or if the associated 
//		Timer Routine was able to execute within the timeout period.
//
//		FALSE: If the associated timer routine was not able to execute
//		within the timeout period. If the assumptions are satisfied, this
//		routine was called at IRQL_PASSIVE_LEVEL, and the timeout is longer 
//		than the delay (or period) of the timer: THIS SHOULD NEVER HAPPEN !
//
// ASSUMPTIONS:
//		The timer callback routine signals the event on entry.
//		For a periodic timer: the event should be reset before calling.
//		For a episodic timer: the event should be reset before setting the 
//							  timer. 
//		
//
////////////////////////////////////////////////////////////////////////////////
BOOLEAN 
SafeCancelTimer(
	PNDIS_MINIPORT_TIMER Timer,
	PNDIS_EVENT Event,
	UINT Timeout
)
{
	BOOLEAN		TimerCancelled;

	//
	// Cancel the timer. 
	//
	NdisMCancelTimer(Timer, &TimerCancelled);

	if(!TimerCancelled) {
#ifdef _NTDDK_    
		if (KeGetCurrentIrql() > PASSIVE_LEVEL) 
#elif defined(linux)
        if (!in_interrupt())
#endif // linux        
        {
			// Since we are running @ IRQL > PASSIVE_LEVEL we cannot wait 
			// for the timer event, so bail out.
			DBGPRINT(
				DBG_PRINT_MODULE_NDIS_ALL,
				DBG_PRINT_LEVEL_WARN,
				("WARNING: Attempt to cancel timer failed! (IRQL > PASSIVE_LEVEL)\n"));

			// FIXME: verify thru code inspection that in no case EVER 
			// the resources accessed by the corresponding timer callback 
			// routine will have dissapeared after this happens.

			return (TimerCancelled);
		}
		else {
			// Since we are running @ IRQL = PASSIVE_LEVEL, wait for as 
			// much as the timeout period for the event to be signalled. 
			if (!NdisWaitEvent(Event, Timeout)) {
				DBGPRINT(
					DBG_PRINT_MODULE_NDIS_ALL,
					DBG_PRINT_LEVEL_ERR,
					("ERROR: Attempt to cancel timer failed! (IRQL = PASSIVE_LEVEL)\n"));

				// This should have never happened. If it does, PAY CLOSE 
				// ATTENTION and FIX IT ! Most likely the timeout was 
				// improperly calculated. Not much we can do in this case, 
				// but send Debug message at least.

				return(FALSE);
			};
		}
	}
	else {
        // Since the timer was cancelled, update the state of the event to 
        // signalled in order to match it.
        NdisSetEvent(Event);
	};

	return (TRUE);
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: SafeSetTimer
//
// ABSTRACT:
//		If there is no Halt request (as per Instance->Status) in progress,
//		this routine signals the event and sets the Timer .
//
// RETURNS:
//		TRUE if the timer was set, FALSE otherwise.
//
// ASSUMPTIONS:
//		None
//
////////////////////////////////////////////////////////////////////////////////

//
// FIXME! 	It would be better to convert this to an __inline function to
//			improve performance since this will be called on every receive
//			PDU. I was having weird link errors when doing it so I leave it 
//			to implement later.
//
BOOLEAN 
SafeSetTimer(
	PATM_USB_INSTANCE Instance,
	PNDIS_EVENT Event,
	PNDIS_MINIPORT_TIMER Timer,
	UINT Delay
)
{
	if ( (Instance->Status & ATM_USB_STATUS_HALTING) || (Instance->Status & ATM_USB_STATUS_SUSPENDING) )
    {
		return FALSE;
	}
	else 
    {
		NdisResetEvent(Event);
	    NdisMSetTimer(Timer, Delay);
		return TRUE;
	}
}

