//
// INTEL CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with Intel Corporation and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998 Intel Corporation. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.1 $
//     $Modtime: 8/06/01 4:02p $
//     $Author: kattre $
//
// ABSTRACT:
//     Contains debug related functions and variables
//
////////////////////////////////////////////////////////////////////////////////

#include "debug.h"
#if defined(MAC_OS) || defined(linux)
#include "GpDebug.h"
#include "common.h"
#elif defined(_NTDDK_)
#include <ntddk.h>
#endif

#if defined(linux)
#include "linux/kernel.h"
#endif

//
// Variables containing the debug print module and level settings
//
static DBG_PRINT_LEVEL DbgPrintLevel    = DBG_PRINT_LEVEL_ALL;
static DBG_PRINT_MODULE DbgPrintModule  = DBG_PRINT_MODULE_ALL;

#define GTI_SIZE_OF_TRACE 300

#ifdef GTI_TRACE
/* Trace */
LONG History[3][GTI_SIZE_OF_TRACE];
LONG HistPnt;
#endif


#ifdef GTI_TRACE

void PrintTrace(void)
{
int i , j;
int present = FALSE;

    static char *pz3dColTrans[] = {
        "Next", "Wait IPUa", "Wait IPUb", "Wait IPUx", "Wait IPUy", "Wait IPUz", "Wait IPUab", "Contex A", "Contex B"};

    static LONG w3dColTrans[]   = {
        0xffff, 0xfffd,     0xfffb,     0xfff9,     0xfff7,     0xfff5,     0xfff3,     0xfff2,     0xfff1};

    static char  *pz2dColTrans[] = {
        "NOP", "End OK", "End Fail", "Reg Read", "Reg Load", "Mem Read", "Mem Load", "Function",
        "Timeout", "Delay", "Fill", "Host Load", "IMM Load", "IMM Read", "Ind Reg", "Ind Load",
        "IF Function", "End Illegal", "Contex Cmd"};

    static LONG w2dColTrans[]   = {
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18};


	printk("Line\t\tOpcode\t\tNext\n");

    for(i = 0; i < HistPnt; i++)
    {


        printk("%04lX\t\t", History[0][i] & 0xffff);

        for(j = 0; j < sizeof(pz2dColTrans)/sizeof(UCHAR  *); j++)
        {
            if((History[2][i] & 0xffff) == w2dColTrans[j])
            {
                printk("%s\t\t",pz2dColTrans[j] );
                present = TRUE;
                break;
            }
        }

       if(!present)
       {
       printk("%04lX\t\t", History[2][i] & 0xffff);
       }

present = FALSE;

        for(j=0; j < sizeof(pz3dColTrans) / sizeof(UCHAR *); j++)
        {
            if((History[1][i] & 0xffff) == w3dColTrans[j])
            {
                printk("%s\n", pz3dColTrans[j]);
                present = TRUE;
                break;
            }
        }

        if(!present)
        {
            printk("Jump %04lX\n", History[1][i] & 0xffff);
        }



    }//end for

}

#endif




////////////////////////////////////////////////////////////////////////////////
//
// NAME: DbgGetDbgPrintLevel
//
// ABSTRACT:
//  The debug print level controls which categories of debug messages are
//  printed while the driver is running
//
// RETURNS:
//  The current debug print level
//
// ASSUMPTIONS:
//  None.
//
////////////////////////////////////////////////////////////////////////////////

DBG_PRINT_LEVEL
DbgGetDbgPrintLevel( void)
{
    return( DbgPrintLevel);

} // DbgGetDbgPrintLevel

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DbgSetDbgPrintLevel
//
// ABSTRACT:
//  The debug print level controls which categories of debug messages are
//  printed while the driver is running.
//
// RETURNS:
//  VOID
//
// ASSUMPTIONS:
//  None.
//
////////////////////////////////////////////////////////////////////////////////

void
DbgSetDbgPrintLevel( DBG_PRINT_LEVEL PrintLevel)
{
    DbgPrintLevel = PrintLevel;

} // DbgSetDbgPrintLevel

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DbgGetDbgPrintModule
//
// ABSTRACT:
//  The debug print module controls which categories of debug messages are
//  printed while the driver is running
//
// RETURNS:
//  The current debug print module
//
// ASSUMPTIONS:
//  None.
//
////////////////////////////////////////////////////////////////////////////////

DBG_PRINT_MODULE
DbgGetDbgPrintModule( void)
{
    return( DbgPrintModule);

} // DbgGetDbgPrintModule

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DbgSetDbgPrintModule
//
// ABSTRACT:
//  The debug print module controls which categories of debug messages are
//  printed while the driver is running.
//
// RETURNS:
//  VOID
//
// ASSUMPTIONS:
//  None.
//
////////////////////////////////////////////////////////////////////////////////

void
DbgSetDbgPrintModule( DBG_PRINT_MODULE PrintModule)
{
    DbgPrintModule = PrintModule;

} // DbgSetDbgPrintModule



#ifdef DBG_TIMER

/************************************************************************
**
** The following section of code is debugging code to measure the time of
** selected routines, with the intent to calculate the percent of processor
** usage for this driver.  A table of timestamps will be generated.  This
** table will later be dumped in the softice window and saved to disk.
** A windows utility has been created to parse and analyze that output.
**
************************************************************************/

// structure for each timestamp entry
typedef struct
{
   UCHAR syncString;    // known value just for sanity checking
   UCHAR id;            // which event generated timestamp
   USHORT numBytes;     // number of bytes handled by event, if used
   ULONG timestamp;     // performance counter ticks
}TIMESTAMP_STRUCT;

// table to store timestamps
#define TIMESTAMP_TABLE_SIZE 1000
static TIMESTAMP_STRUCT TimestampTable[TIMESTAMP_TABLE_SIZE];

// table location to write to next
static int TimestampIndex = 0;

// dump this out in softice to find out how long a counter tick is
static ULONG TimerFrequencyLow;
static ULONG TimerFrequencyHigh;

////////////////////////////////////////////////////////////////////////////////
//
// NAME: RecordTimestamp
//
// ABSTRACT:
//  generates a timestamp entry in the next position of the table.
//
// INPUTS:
// id - the number of the event that generated the timestamp.
//      by convention, an odd number is the entry to a routine
//      and the next consecutive number is the exit.
// numBytes - number of bytes associated with the timestamp event.
//
// RETURNS:
//  VOID
//
// ASSUMPTIONS:
//  None.
//
////////////////////////////////////////////////////////////////////////////////
void RecordTimestamp(UCHAR id, USHORT numBytes)
{
   LARGE_INTEGER timestamp;
   LARGE_INTEGER frequency;

   timestamp = KeQueryPerformanceCounter(&frequency);
   TimestampTable[TimestampIndex].timestamp = timestamp.LowPart;
   TimerFrequencyLow = frequency.LowPart;
   TimerFrequencyHigh = frequency.HighPart;
   TimestampTable[TimestampIndex].syncString = 0xad;
   TimestampTable[TimestampIndex].id = id;
   TimestampTable[TimestampIndex].numBytes = numBytes;
   TimestampIndex++;
   if (TimestampIndex == TIMESTAMP_TABLE_SIZE)
      TimestampIndex = 0;
}

#endif
