////////////////////////////////////////////////////////////////////////////////
//
// GlobeSpan Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with GlobeSpan Inc. and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  2001 GlobeSpan Inc. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.1 $
//     $Modtime: 9/17/01 11:17a $
//     $Author: kattre $
//
// ABSTRACT:
//     This file provides the interface to the GlobeSpan's EOC module.
//
////////////////////////////////////////////////////////////////////////////////

#include "Gp.h"
#include "gti.h"
#include "gtidsp.h"
#include "dsl.h"

extern GTIEOC_DATA_REG  eocDataReg;
extern void GTI_ParseDspStatus(GTI_XCVR_INFO *GTISysPtr, GTI_BYTE *dataPtr);
extern USHORT GSICallXcvr(PUSB_PHY UsbPhy, UCHAR action, UCHAR xcvrNum);
extern GTI_WORD ExecuteXCVR_USB(GTI_XCVR_INFO *,
                        GTI_WORD isIpuReady,
                        GTI_WORD getNextState);

#ifdef MESSAGE_HANDLING
extern void MsgHandling_AocSend( PVOID AtmUni, PDSL_EX pDslEx);

// Trigger an eoc send via usbphy function call
void TriggerEocSend( PUSB_PHY UsbPhy)
{
    SetDspState( UsbPhy, DSP_PHY_EMPTY, DSP_EOC_MSG);
}
#endif // MESSAGE_HANDLING


////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslEocBind
//
// ABSTRACT: Bind Dsl layer with Usb layer
//           
////////////////////////////////////////////////////////////////////////////////
void DslEocBind(PDSL_EX pDslEx, USB_PHY_HANDLE UsbPhyHandle)
{
    PUSB_PHY UsbPhy = (PUSB_PHY) UsbPhyHandle;
    
	DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("DslEocBind\n "));

	pDslEx->UsbPhyHandle						= UsbPhy;
    pDslEx->DspHandle							= (DSP_HANDLE)UsbPhy->GTISysBlock;
	UsbPhy->GTISysBlock->CustXcvrInfo.msgInfo->pDslEx	= pDslEx;
    UsbPhyBindDsl(UsbPhy, pDslEx);

	DslEocInitialize( pDslEx, UsbPhy, NULL);	// init dsl layer
}


////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslEocInitialize
//
// ABSTRACT: Initialiaze EOC_MESSAGE object
//           
////////////////////////////////////////////////////////////////////////////////
void DslEocInitialize(PDSL_EX pDslEx, USB_PHY_HANDLE UsbPhy, USHORT *actualCfg)
{
    GTI_BYTE i,j;


    DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("DslEocInitialize\n "));

    if ( (!pDslEx) || (!UsbPhy) )
	    return;

    /* init status information */
    pDslEx->status.RxEocOk      = 0;
    pDslEx->status.RxEocError   = 0;
    pDslEx->CellWriteInd        = 0;
    pDslEx->CellReadInd         = 0;
    pDslEx->EocCmd.tx.AutoLen   = 0;
    pDslEx->CellPendingCnt      = 0;

    /*
    ** Intialize the registers, which is being shared between the customer and
    ** EOC module. These registers can be read by the DSLAM.
    */
    for(j = 0; j < GTI_EOC_VENDOR_LEN; j++)
        eocDataReg.vendorId[j] = 0;

    eocDataReg.vendorId[1] = ((PUSB_PHY)UsbPhy)->VendorId;

    if ( actualCfg )
    {
        if ( (actualCfg[GTI_SELECTED_STANDARD_ACTUAL] == GTI_G_DMT_STANDARD) || 
             (actualCfg[GTI_SELECTED_STANDARD_ACTUAL] == GTI_G_LITE_STANDARD) )
        {
            eocDataReg.vendorId[0] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdHigh & 0xFF000000) >> 24);
	        eocDataReg.vendorId[1] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdHigh & 0x00FF0000) >> 16);
	        eocDataReg.vendorId[2] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdHigh & 0x0000FF00) >> 8);
	        eocDataReg.vendorId[3] = (GTI_BYTE)(((PUSB_PHY)UsbPhy)->EocVendorIdHigh & 0x000000FF);

	        eocDataReg.vendorId[4] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdLow & 0xFF000000) >> 24);
	        eocDataReg.vendorId[5] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdLow & 0x00FF0000) >> 16);
	        eocDataReg.vendorId[6] = (GTI_BYTE)((((PUSB_PHY)UsbPhy)->EocVendorIdLow & 0x0000FF00) >> 8);
	        eocDataReg.vendorId[7] = (GTI_BYTE)(((PUSB_PHY)UsbPhy)->EocVendorIdLow & 0x000000FF);
        }
    }

    /* revision number init */
    for(j = 0; j < GTI_EOC_REV_NUM_LEN; j++)
        eocDataReg.revisionNum[j] = 0;

    /* serial number init */
    for(j = 0; j < GTI_EOC_SERIAL_LEN; j++ )
    {
        if ( j < ETH_LENGTH_OF_ADDRESS )
            eocDataReg.serialNum[j] = ((PUSB_PHY)UsbPhy)->AtmEndSystemIdentifier[j];
        else
            eocDataReg.serialNum[j] = 0;
    }

    /* selfTest init: test not required for Anchor-Nova-Slade since we dont use UTOPIA */
    for(j = 0; j < GTI_EOC_SELFTEST_LEN; j++)
       eocDataReg.selfTest[j] = 0;

    /* AtuRCfg init */
    for(j = 0; j < GTI_EOC_CONFIG_LEN; j++)
       eocDataReg.aturConfig[j] = 0;

    DslSetEocAtuRCfg( actualCfg);

    /* line attn and snr margin init */
    eocDataReg.lineAtten[0] = 0x10;
    eocDataReg.snrMargin[0] = 0x40;

    /* Intialize all of the buffers being used to store the received EOC data */
    for(i = 0; i < EOC_BUFFER_SIZE; i++)
    {
        pDslEx->EocDspBuffer[i].txFlag = GTI_EOC_TXBUFF_EMPTY;
        pDslEx->EocDspBuffer[i].rxFlag = GTI_EOC_RXBUFF_EMPTY;

        for(j = 0; j < GTI_EOC_MAX_NUMOF_MSG; j++)
            pDslEx->EocDspBuffer[i].rxBuff[j] = EMPTY_PATTERN;
    }
}


////////////////////////////////////////////////////////////////////////////////
// NAME: DslSetEocAtuRCfg
//
// ABSTRACT:  Sets the EOC ATU-R configuration register.  
////////////////////////////////////////////////////////////////////////////////
void DslSetEocAtuRCfg( USHORT *actualCfg)
{
    if ( actualCfg )
    {
        if( actualCfg[GTI_AS0_LATENCY_ACTUAL] == GTI_FAST_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BF_AS0] = (GTI_BYTE)actualCfg[GTI_AS0_BYTES_ACTUAL];

        if( actualCfg[GTI_AS0_LATENCY_ACTUAL] == GTI_INTERLEAVE_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BI_AS0] = (GTI_BYTE)actualCfg[GTI_AS0_BYTES_ACTUAL];

        if( actualCfg[GTI_AS1_LATENCY_ACTUAL] == GTI_FAST_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BF_AS1] = (GTI_BYTE)actualCfg[GTI_AS1_BYTES_ACTUAL];

        if( actualCfg[GTI_AS1_LATENCY_ACTUAL] == GTI_INTERLEAVE_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BI_AS1] = (GTI_BYTE)actualCfg[GTI_AS1_BYTES_ACTUAL];


        if( actualCfg[GTI_LS0_LATENCY_ACTUAL] == GTI_FAST_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BF_LS0_UP] = (GTI_BYTE)actualCfg[GTI_LS0_BYTES_ACTUAL];

        if( actualCfg[GTI_LS0_LATENCY_ACTUAL] == GTI_INTERLEAVE_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BI_LS0_UP] = (GTI_BYTE)actualCfg[GTI_LS0_BYTES_ACTUAL];

        if( actualCfg[GTI_LS1_LATENCY_ACTUAL] == GTI_FAST_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BF_LS1_UP] = (GTI_BYTE)actualCfg[GTI_LS1_BYTES_ACTUAL];

        if( actualCfg[GTI_LS1_LATENCY_ACTUAL] == GTI_INTERLEAVE_BUFFER )
            eocDataReg.aturConfig[ATU_R_CFG_BI_LS1_UP] = (GTI_BYTE)actualCfg[GTI_LS1_BYTES_ACTUAL];


        eocDataReg.aturConfig[ATU_R_CFG_RSF_DWN] = (GTI_BYTE)actualCfg[GTI_FAST_R_DOWN_ACTUAL];
        if( actualCfg[GTI_INTERLEAVE_S_DOWN_ACTUAL] )
            eocDataReg.aturConfig[ATU_R_CFG_RSI_DWN] = 
            (GTI_BYTE)(actualCfg[GTI_INTERLEAVE_R_DOWN_ACTUAL]/actualCfg[GTI_INTERLEAVE_S_DOWN_ACTUAL]);

        eocDataReg.aturConfig[ATU_R_CFG_S_DWN] = (GTI_BYTE)actualCfg[GTI_INTERLEAVE_S_DOWN_ACTUAL];
        eocDataReg.aturConfig[ATU_R_CFG_D_DWN] = (GTI_BYTE)actualCfg[GTI_INTERLEAVE_D_DOWN_ACTUAL];


        eocDataReg.aturConfig[ATU_R_CFG_RSF_UP] = (GTI_BYTE)actualCfg[GTI_FAST_R_UP_ACTUAL];
        if( actualCfg[GTI_INTERLEAVE_S_UP_ACTUAL] )
            eocDataReg.aturConfig[ATU_R_CFG_RSI_UP] = 
            (GTI_BYTE)(actualCfg[GTI_INTERLEAVE_R_UP_ACTUAL]/actualCfg[GTI_INTERLEAVE_S_UP_ACTUAL]);

        eocDataReg.aturConfig[ATU_R_CFG_S_UP] = (GTI_BYTE)actualCfg[GTI_INTERLEAVE_S_UP_ACTUAL];
        eocDataReg.aturConfig[ATU_R_CFG_D_UP] = (GTI_BYTE)actualCfg[GTI_INTERLEAVE_D_UP_ACTUAL];
    }
}


////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslShutdown
//
// ABSTRACT:
//           
// RETURNS:
//        NONE
//
// ASSUMPTIONS:
//        USB
////////////////////////////////////////////////////////////////////////////////
void DslShutdown(USB_PHY_HANDLE UsbPhyHandle)
{
    PUSB_PHY UsbPhy = (PUSB_PHY)UsbPhyHandle;
#ifdef  _NTDDK_
    LARGE_INTEGER  longIntTimeout;
    LONG           longTimeout  = (-1) * (50 * 10000);
    KEVENT         waitEvent;


    SetUsbState (UsbPhy, USB_PHY_STATE_SHUTDOWN);    // send dying gasp and cleanup resources

    // wait on shutdown completion
    while (!UsbPhy->ShutdownAck)
    {
       // wait for 50 ms
       KeInitializeEvent( &waitEvent , NotificationEvent, FALSE);
       longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);
       KeWaitForSingleObject(
           &waitEvent,
           Executive,   
           KernelMode,
           FALSE ,
           &longIntTimeout);
    }
#elif defined(linux)
    LONG           longTimeout = (HZ * 50)/1000; //50 ms
    SetUsbState (UsbPhy, USB_PHY_STATE_SHUTDOWN);    // send dying gasp and cleanup resources
	set_current_state( TASK_INTERRUPTIBLE );
	schedule_timeout( longTimeout );
#endif //linux
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslEocParse
//
// ABSTRACT: Function copy rx cell PayLoad in to rx buffer
//           in case it is second cell GTIEocProc will be called
//           tx Buffer will be prepared. 
//           and result will be send to DSP (control chanel send) 
// RETURNS:
//        NONE
//
// ASSUMPTIONS:
//        USB
//
////////////////////////////////////////////////////////////////////////////////
void DslEocParse(PDSL_EX pDslEx, GTI_BYTE *dataPtr)
{
GTI_WORD       nIndex, nValue, rxFlag, txFlag = 0;
PUSB_PHY       UsbPhy = (PUSB_PHY)pDslEx->UsbPhyHandle;
EOC_DSP_BUFFER *pDslBuff;
      
   if( GetDspState(UsbPhy) != DSP_PHY_CONNECTED )
       return;
       
   rxFlag = (dataPtr[37] << 8) | dataPtr[36]; 
   txFlag = (dataPtr[35] << 8) | dataPtr[34];   

   if(rxFlag == GTI_EOC_RXBUFF_FULL) 
   {    
       pDslBuff = &pDslEx->EocDspBuffer[pDslEx->CellWriteInd];
       for(nIndex = 0; nIndex < GTI_EOC_FRAME_34; nIndex++)
       {
           nValue = ( (dataPtr[(nIndex << 1) + 1] ) << 8) | (dataPtr[(nIndex << 1)] );  
           pDslEx->EocDspBuffer[pDslEx->CellWriteInd].rxBuff[nIndex] = nValue;
       }

       /* Do NOT use the buffer, if the EOC data is garbage/junk */
       if( isEocValid(pDslBuff->rxBuff, GTI_EOC_FRAME_34) )
       {
           pDslBuff->rxFlag = rxFlag;
           pDslBuff->txFlag = txFlag; /*GTI_EOC_TXBUFF_EMPTY or may be not empty*/ 

           pDslEx->CellWriteInd  = (pDslEx->CellWriteInd + 1) % EOC_BUFFER_SIZE;   
           pDslEx->status.RxEocOk++;
           pDslEx->CellPendingCnt++;

           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE,
               ("EOC Superframe received = %d\n",pDslEx->status.RxEocOk ));
       }
   }
        
   if(pDslEx->CellPendingCnt > EOC_BUFFER_SIZE )
   {
       /* FIXME-KP: Provide Counter to be displayed in the debug mode of the control panel */
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
           ("Buffer Overflow CellPendingCnt = %d\n", pDslEx->CellPendingCnt ));
   }   

   if( (pDslEx->CellPendingCnt  ) && (txFlag == GTI_EOC_TXBUFF_EMPTY)   )
   {
       pDslEx->CellPendingCnt--;
       SetDspState(UsbPhy, DSP_PHY_EMPTY, DSP_EOC_MSG);
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslStatusParse
//
// ABSTRACT:
//           
// RETURNS:
//        NONE
//
// ASSUMPTIONS:
//        USB
////////////////////////////////////////////////////////////////////////////////
void DslStatusParse(PDSL_EX pDslEx, GTI_BYTE *dataPtr)
{
KIRQL      Irql;
PUSB_PHY   pUsbPhy = (PUSB_PHY)pDslEx->UsbPhyHandle;

   KeAcquireSpinLock( &(pUsbPhy->DspLock), &Irql);
   GTI_ParseDspStatus(pDslEx->DspHandle, dataPtr);
   KeReleaseSpinLock( &(pUsbPhy->DspLock),Irql);    
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslEocSendDyingGasp
//
// ABSTRACT:
//     Send Dying Gasp to the DSLAM to indicate power failure.
//     In our case, it will be send for shutdown and suspend. The USB modem
//     does not support hardware dying gasp.
//           
// RETURNS:
//        NONE
//
// ASSUMPTIONS:
//        USB
//
////////////////////////////////////////////////////////////////////////////////
BOOLEAN DslEocSendDyingGasp(PDSL_EX pDslEx)
{
#ifdef  _NTDDK_
LARGE_INTEGER  longIntTimeout;
LONG           longTimeout  = (-1) * (EOC_DYING_GASP_DELAY * 10000);
KEVENT         waitEvent;
#elif defined(linux)
LONG           longTimeout  = 1 + (EOC_DYING_GASP_DELAY * HZ) / 1000;
#endif  //linux
PUSB_PHY       pUsbPhy = (PUSB_PHY)pDslEx->UsbPhyHandle;
GTIEOC_CMD     DgaspEocCmd;
GTI_XCVR_INFO  *GTISysPtr  = (GTI_XCVR_INFO *)pDslEx->DspHandle;
UCHAR          ContinuousSendDyingGasp = 0;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("DslEocSendDyingGasp\n"));

   /* When testing dying gasp against the Alcatel DSLAM, following was observed:
   ** 1) When sending dying gasp message before the alarm (LOS and LPR) is cleared
   **      in the DSLAM. The DSLAM will not display the LPR messsage (because it is
   **      already set according to DSLAM).
   ** 2) If the EOC message in the middle of the READ/WRITE transcation and dying
   **      gasp message is send out, it would not display LPR alarm. Therefore, we
   **      send out dying gasp message in every DMT super-frame before aborting.
   */
   
   if( GetDspState(pUsbPhy) != DSP_PHY_CONNECTED )
   {
       DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_TRACE, ("SendDyingGasp(Not connected)\n"));
 
       return FALSE;
   }

   /* Send Dying Gasp message atleast 4 times before aborting the line */
   while (ContinuousSendDyingGasp <= 4)
   {
       DgaspEocCmd.tx.EocCmd  = GTI_EOC_CMD_DGASP;
       DgaspEocCmd.tx.EocLen  = 0;
       DgaspEocCmd.tx.AutoLen = 0;
       /* FIXME: Remove this flag...We should let the controller check for the flag
       ** before writting the buffer. There is no reason for the software flag to
       ** be present.
       */
       pDslEx->EocDspBuffer[pDslEx->CellReadInd].txFlag = GTI_EOC_TXBUFF_EMPTY;
       GTIEocProc( GTISysPtr,  &DgaspEocCmd);

#ifdef  _NTDDK_
       /*
       ** Wait for EOC_DYING_GASP_DELAY (7msec)
       */
       KeInitializeEvent( &waitEvent , NotificationEvent, FALSE);
       longIntTimeout =  RtlConvertLongToLargeInteger(longTimeout);
       KeWaitForSingleObject(
           &waitEvent,
           Executive,   
           KernelMode,
           FALSE ,
           &longIntTimeout);

#elif defined(linux)
	    set_current_state( TASK_INTERRUPTIBLE );
	    schedule_timeout( longTimeout );
#endif //linux
       ContinuousSendDyingGasp++;
   }

   /* ABORT_REQ to the DSP...This would cause loss of signal to the far-side */
   GSICallXcvr(pUsbPhy, GTI_ABORT_REQ, 0);
   ExecuteXCVR_USB(GTISysPtr, 0, 0);
   UsbPhyDslDisconnected(pUsbPhy);
   SetDspState(pUsbPhy, DSP_PHY_IDLE, DSP_NO_ACTION);

   return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: GetEocCount
//
// ABSTRACT:
//           
// RETURNS:
//        NONE
//
// ASSUMPTIONS:
//        USB
////////////////////////////////////////////////////////////////////////////////
GTI_WORD isEocValid(GTI_WORD *rxBuff, int nSize)
{
GTI_WORD i, nCount = 0;

   /* Is EOC Valid? */
   /* FIXME-KP: Same information is being checked by the EOC module and therefore
   ** it is really not needed here. The reason for this sanity check to be here
   ** is because we do not want to use buffer if the information is not valid.
   ** We should allocate pool of buffers to be used and use buffer instead of arrays.
   */
   for (i = 0; i < nSize; i++)
   {
       if( !((i == 0) || (rxBuff[i] == GTI_EOC_SYNCH_CONTROL) || ( i>GTI_EOC_FRAME_34 )) )
       {
           /* Check, if the overhead of the eoc is there */
           if( (rxBuff[i] & GTI_EOC_OVERHEAD_MASK) == GTI_EOC_OVERHEAD_MASK )
               nCount++;
       }
   }
   
   return nCount;
}

////////////////////////////////////////////////////////////////////////////////
//
// NAME: DslEocProc
//
// ABSTRACT:
//        This Function is called from passive thread when rx Eoc buffer of DSP is full
//        1. Get Tx/Rx Buffers
//     2. call GTIEocProc(...)
//     3. Transfer tx buffer throw control pipe
//     4. Do additional parsing of tx buffer after send  
//
// RETURNS:
//        void
//
// ASSUMPTIONS:
//        Runs at IRQL < Dispatch
//
////////////////////////////////////////////////////////////////////////////////
void DslEocProc(PDSL_EX pDslEx)
{
PUSB_PHY        pUsbPhy = (PUSB_PHY)pDslEx->UsbPhyHandle;
GTI_BYTE        nRxReadInd    =    pDslEx->CellReadInd;
GTI_BYTE        nRxWriteInd =    pDslEx->CellWriteInd;
GTI_XCVR_INFO    *GTISysPtr    =    (GTI_XCVR_INFO *)pDslEx->DspHandle;

   DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR, ("DslEocProc\n "));

   if (nRxReadInd != nRxWriteInd)
   {
       /* UpDate Current pointers*/
       if( GetDspState(pUsbPhy) != DSP_PHY_CONNECTED )
       {
           DBGPRINT(DBG_PRINT_MODULE_USB_ALL, DBG_PRINT_LEVEL_ERR,
               ("DslEocProc return not connected, skip buffer\n "));
 
           pDslEx->CellReadInd = (nRxReadInd + 1) % EOC_BUFFER_SIZE;
           return;
       }
        
       CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_NO_AI);
       GTIEocProc( GTISysPtr ,  &pDslEx->EocCmd );
       pDslEx->CellReadInd = (nRxReadInd + 1) % EOC_BUFFER_SIZE;
   }

#ifdef MESSAGE_HANDLING		
    MsgHandling_AocSend( pUsbPhy->UserHandle, pDslEx);
#endif // MESSAGE_HANDLING

}
