#ifndef OAM_C
#define OAM_C
#endif

#include "oam.h"

///////////////////////////////////////////////////////////////////////////////
// 
//	NAME: Process_Loopback
//
// ABSTRACT:
//     Detect F4/F5 ATM cells, 0/21, or 1/21 ATM cells.
//
// RETURNS:
//     Did we find the F4/F5 cells?
//     TRUE: Found the F4/F5 cell
//     FALSE: Could not find the F4/F5 cell
///////////////////////////////////////////////////////////////////////////////
BOOLEAN 
Process_Loopback (PATM_CELL_RCV atmCell, PATM_UNI_STATS Stats, PUCHAR need2Send, PATM_UNI AtmUni, PATM_UNI_VC AtmUniVc)
{
	ULONG	CellHeader	= NTOHL( (*((ULONG *)atmCell)));
	USHORT	Vpi			= (USHORT)(GET_ATM_VPI( CellHeader));
	UCHAR   Vci			= (UCHAR)(GET_ATM_VCI( CellHeader));
	USHORT  Pti			= (USHORT)(GET_ATM_PTI( CellHeader));
	UCHAR	loopChk		= atmCell->info[0];
	UCHAR	loopInd		= atmCell->info[1];
	USHORT	newCrc		= 0;
   
   /* Set the default value to TRUE - send out a loopback cell */
   *need2Send           = TRUE;

	if ((Vci == 21) && ((Vpi == 1) || (Vpi == 0)))			// 0/21 or 1/21 ATM cell?
	{
		DBGPRINT(
			DBG_PRINT_MODULE_ATMUNI_RX,
			DBG_PRINT_LEVEL_TRACE,
			("0/21 or 1/21 ATM cell detected (VCI = %d)... looping back!\n", Vci));
		Stats->OAMLoopBkSuccess++;
       
       AtmUni->AtmCounters.RxOAMCellsOk++;
//       AtmUniVc->AtmCounters.RxOAMCellsOk++;
       *need2Send = TRUE;
		return TRUE;
	}
	/* Is it F4/F5 cell??? */
	else if((Vci == OAM_F4_CELL_SEGMENT_VCI)  ||      /* F4 Cell- Segment Loopback */
            (Vci == OAM_F4_CELL_END2END_VCI)  ||      /* F4 Cell- End to End Loopback */
            (Pti == ATM_PTI_101)              ||      /* F5 Cell- End to End Loopback */
            (Pti == ATM_PTI_100))                     /* F5 Cell- Segment Loopback */
	{
		if ((loopChk == LOOPREQ) && (loopInd == 1))			// CO initiated loopback? 
		{
			if ( ChkCRC10(atmCell->info) )					// good CRC10?
			{
				atmCell->info[1] = 0;						// decrement loop indication
				newCrc = CalcCRC10 (atmCell->info);			// calculate CRC10 (cell changed)

				/* Before setting CRC10 - clear 2 LSB Bits of the 47th bytes  */
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] &= 0xFC;

				/* Write the CRC10 in bytes 47 and 48 */
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] |= (UCHAR)((newCrc >> 8) & 0x03);
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = (UCHAR) (newCrc & 0xFF);

				DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_TRACE,
					("F4 or F5 ATM cell detected (VCI = %d)... looping back!\n", Vci));
				Stats->OAMLoopBkSuccess++;

               AtmUni->AtmCounters.RxOAMCellsOk++;
//               AtmUniVc->AtmCounters.RxOAMCellsOk++;
               *need2Send = TRUE;
				return TRUE;
			}
		}
		else if ((loopChk == LOOPREQ) && (loopInd == 0))	// CP initiated loopback? 
		{
			if ( ChkCRC10(atmCell->info) )					// good CRC10?
			{
				DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_TRACE,
					("F4 or F5 ATM cell detected (VCI = %d)... initiated from CP!\n", Vci));
				Stats->OAMLoopBkSuccess++;

               AtmUni->AtmCounters.RxOAMCellsOk++;
//               AtmUniVc->AtmCounters.RxOAMCellsOk++;
               *need2Send = FALSE;
				return TRUE;
			}
		}
       else if (loopChk == OAM_AIS)
       {
			if ( ChkCRC10(atmCell->info) )					// good CRC10?
			{
				atmCell->info[0] = OAM_RDI;                 // If AIS detected, reply with RDI
				newCrc = CalcCRC10 (atmCell->info);			// calculate CRC10 (cell changed)

				/* Before setting CRC10 - clear 2 LSB Bits of the 47th bytes  */
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] &= 0xFC;

				/* Write the CRC10 in bytes 47 and 48 */
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] |= (UCHAR)((newCrc >> 8) & 0x03);
				atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = (UCHAR) (newCrc & 0xFF);

				DBGPRINT(DBG_PRINT_MODULE_ATMUNI_RX, DBG_PRINT_LEVEL_TRACE,
					("OAM_AIS ATM cell detected(VCI= %d) Looping back with RDI!\n", Vci));
				Stats->OAMLoopBkSuccess++;

               AtmUni->AtmCounters.RxOAMCellsOk++;
//               AtmUniVc->AtmCounters.RxOAMCellsOk++;
               *need2Send = TRUE;
				return TRUE;
			}
       }

		Stats->OAMLoopBkFailure++;
       AtmUni->AtmCounters.RxOAMCellsDrop++;
//       AtmUniVc->AtmCounters.RxOAMCellsDrop++;
	}

   *need2Send = FALSE;
	return FALSE;
}



// ***************************************************************
//	Function:	compares given CRC to calculated CRC, returns TRUE
//				if CRC is correct (note: ATM OAM CRC is calculated
//				with the last 2 bytes set to ZERO)
//	Input:		pointer to payload
// ***************************************************************
static BOOLEAN
ChkCRC10 (PUCHAR info)
{
	USHORT  actualCRC10 = 0;

	/* Read the actual CRC10 from the cell */
	actualCRC10 = ((info[ATM_CELL_PAYLOAD_SIZE-2] & 0x03) << 8);
	actualCRC10 |= (info[ATM_CELL_PAYLOAD_SIZE-1] & 0xFF);

	/* Inorder for the CRC10 to be calculated, last 10 bits of the
	** payload needs to be zero-ed.
	*/
	info[ATM_CELL_PAYLOAD_SIZE-2] &= 0xFC;    // Clear only CRC10 field (2 LSB)
	info[ATM_CELL_PAYLOAD_SIZE-1] = 0;        // Clear only CRC10 field

	/* Compare the actual CRC10 value with calculated CRC10 value */
	return ( (CalcCRC10 (info)) == actualCRC10);
}




// ***************************************************************
//	Function:	calculates 10-bit CRC for ATM cell payload
//				(note: table not used to save memory - also,
//				these cells should be received too often)
//	Input:		pointer to payload
// ***************************************************************
static USHORT
CalcCRC10 (PUCHAR info)
{
   register USHORT crcAccum;
   register USHORT newCrc = 0;
   register USHORT j, k;

   for (j = 0;  j < ATM_CELL_PAYLOAD_SIZE;  j++)
   {
		crcAccum = ((USHORT) ((newCrc >> 2) & 0xff) << 2);
		for (k = 0; k < 8; k++) 
			if ((crcAccum <<= 1) & 0x400)
				crcAccum ^= CRC10POLY;

		newCrc = ((newCrc << 8) & 0x3ff) ^ crcAccum ^ info[j]; 
   }

   /* Only return CRC10, meaning 10 bits */
   newCrc &= 0x03FF;

   return newCrc;
}


// ********************************************************************
//	Function:	generates a F4/F5 ATM cell which will be sent back
//	Input:		pointer to the ATM cell, VPI, VCI
// ********************************************************************
void
Generate_Loopback_Cell (PATM_CELL_TXFR atmCell, UCHAR   Vpi, USHORT  Vci)
{
	USHORT	newCrc = 0;
	int i;
	static ULONG sequenceNumber = 0;

	/* Set the ATM Header */
	atmCell->hdr[0] = (Vpi >> 4);
	atmCell->hdr[1] = (Vpi << 4) | ((UCHAR)(Vci >> 12));
	atmCell->hdr[2] = ((UCHAR)(Vci >> 4));
	atmCell->hdr[3] = ((UCHAR)(Vci << 4)) | ATM_PTI_100;
	// note:  HEC calculated in the DSP

	/* Fill in the format of the payload of the F4/F5 Cell */
	atmCell->info[0] = LOOPREQ; // set error management and loopback
	atmCell->info[1] = 1;       // set loopback indication

	/* Correlation Tag - SequenceNumber */
	sequenceNumber++;
	atmCell->info[2] = (UCHAR)((sequenceNumber & 0xFF000000)>>24);
	atmCell->info[3] = (UCHAR)((sequenceNumber & 0xFF0000)>>16);
	atmCell->info[4] = (UCHAR)((sequenceNumber & 0xFF00)>>8);
	atmCell->info[5] = (UCHAR)((sequenceNumber & 0xFF));

	/* Set loopback location ID and source ID to all ones */
	for( i=6; i < 38;i++)
       atmCell->info[i] = 0xFF;

	/* Unused default value = 0x6A */
	for (i=38; i < (ATM_CELL_PAYLOAD_SIZE-2); i++)
       atmCell->info[i] = 0x6A;

	atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] = 0;	// init CRC high byte
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = 0;	// init CRC low byte

	/* Calculate the CRC10 on the cell */
	newCrc = CalcCRC10 (atmCell->info);
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] = (UCHAR)(newCrc >> 8);
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = (UCHAR)newCrc;

   /* 54th byte is required to properly interface PCU */
	atmCell->info[ATM_CELL_PAYLOAD_SIZE] = (UCHAR)0xFF;
}

// ********************************************************************
//	Function:	Create a F4/F5 OAM cell, Add loopback indicator, and sequence
//             number. 
//	Input:		pointer to the ATM cell, VPI, VCI
// ********************************************************************
void
OAMCreateCell (PATM_CELL_TXFR atmCell,
               UCHAR   type,
               UCHAR   Vpi,
               USHORT  Vci,
               UCHAR   pti,
               UCHAR   *reservePtr)
{
	USHORT	newCrc = 0;
	int i;
	static ULONG seqNum = 0;    /* Sequence Number */

	/* Set the ATM Header */
	atmCell->hdr[0] = (Vpi >> 4);
	atmCell->hdr[1] = (Vpi << 4) | ((UCHAR)(Vci >> 12));
	atmCell->hdr[2] = ((UCHAR)(Vci >> 4));
	atmCell->hdr[3] = ((UCHAR)(Vci << 4)) | (pti << 1);
	// note:  HEC calculated in the DSP

	/* Fill in the format of the payload of the F4/F5 Cell */
	atmCell->info[0] = LOOPREQ; // set error management and loopback
	atmCell->info[1] = 1;       // set loopback indication

	/* Correlation Tag - SequenceNumber */
	seqNum++;
	atmCell->info[2] = (UCHAR)((seqNum & 0xFF000000)>>24);
	atmCell->info[3] = (UCHAR)((seqNum & 0xFF0000)>>16);
	atmCell->info[4] = (UCHAR)((seqNum & 0xFF00)>>8);
	atmCell->info[5] = (UCHAR)((seqNum & 0xFF));

	/* Set loopback location ID and source ID to all ones */
	for( i=6; i < 38;i++)
       atmCell->info[i] = 0xFF;

	/* Unused default value = 0x6A */
	for (i=38; i < (ATM_CELL_PAYLOAD_SIZE-2); i++)
       atmCell->info[i] = 0x6A;

	atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] = 0;	// init CRC high byte
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = 0;	// init CRC low byte

	/* Calculate the CRC10 on the cell */
	newCrc = CalcCRC10 (atmCell->info);
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-2] = (UCHAR)(newCrc >> 8);
	atmCell->info[ATM_CELL_PAYLOAD_SIZE-1] = (UCHAR)newCrc;

   /* 54th byte is required to properly interface PCU */
	atmCell->info[ATM_CELL_PAYLOAD_SIZE] = (UCHAR)0xFF;
}


// ********************************************************************
//	Function:	copies a received F4/F5 ATM cell to transmit buffer
//	Input:		pvc and pointers to the receive and transmit buffers
// ********************************************************************
void 
Copy_Loopback (PATM_UNI_VC AtmUniVc, PATM_CELL_TXFR atmCellTxfr, PATM_CELL_RCV atmCellRcv)
{
#if ATM_CELL_FULL_HEADER
	COPY_MEMORY((PUCHAR)atmCellTxfr, (PUCHAR)atmCellRcv, ATM_CELL_TXFR_SIZE);

    /* 54th byte is required to properly interface PCU */
	atmCellTxfr->info[ATM_CELL_PAYLOAD_SIZE] = (UCHAR)0xFF;
#else
    if ( AtmUniVc )                  
    {
        // use the found pvc header
        atmCellTxfr->hdr[0] = AtmUniVc->CellHeader[0];          // use header from lookup (gfc:4, vpi:4)
	    atmCellTxfr->hdr[1] = AtmUniVc->CellHeader[1];          // use header from lookup (vpi:4, vci:4)
	    atmCellTxfr->hdr[2] = AtmUniVc->CellHeader[2];          // use header from lookup (vci:8)
	    atmCellTxfr->hdr[3] = (AtmUniVc->CellHeader[3] & 0xF0)  // use header from lookup (vci:4)
                            | (atmCellRcv->hdr[2] & 0x0F);      // keep PTI from loopback (pti:3, cpi:1)
    }
    else
    {
	    atmCellTxfr->hdr[0] = 0;
        /* FIX_ME: Lower nibble of the first byte in the received ATM header cell
        ** is used for HEC value. Remove the HEC value. atmCellRcv->hdr[0] & 0xF0 */
	    atmCellTxfr->hdr[1] = (atmCellRcv->hdr[0] & 0xF0);
	    atmCellTxfr->hdr[2] = atmCellRcv->hdr[1];
	    atmCellTxfr->hdr[3] = atmCellRcv->hdr[2];
    }

	// note:  HEC calculated in controller
	COPY_MEMORY((PUCHAR)(atmCellTxfr->info), (PUCHAR)(atmCellRcv->info), ATM_CELL_PAYLOAD_SIZE);

   /* 54th byte is required to properly interface PCU */
	atmCellTxfr->info[ATM_CELL_PAYLOAD_SIZE] = (UCHAR)0xFF;
#endif // ATM_CELL_FULL_HEADER
}
