/*
** FileName:    rfc1577.c
** Purpose:
**      This file implements Rfc1577/Rfc2225 for PVC's.
** Only Inverse  ARP request/reply functionality is currently
** implemented.
** Notes:
**  This is the core code for Rfc1577.
**  IPAddress and all other storage operations are done in host byte
**  order.
** Edit History:
**  8th June 2000   -   New File
**  13th June 2000  -   Merged all files to one , added timer module
**                      and changed the API's for Windows driver
**  26th June 2000  -   Cleaned the code after testing the driver
*/

#if defined (LAN_MINIPORT) || defined (PPPOE_MINIPORT) || defined(GSI_LAN)

/*
** Include files
*/
#ifdef _NTDDK_
#include "ndis.h"
#else
#include "GpSnull.h"
#endif // _NTDDK_
#include "api1577.h"
#include "atmuni.h"
#include "rfc1483.h"
#include "rfc1577.h"

//extern void UsbPhyGetMacAddress( PUSB_PHY UsbPhy , PUCHAR Buffer );

//extern BOOLEAN DspNotConnected (PVOID UsbPhy);

extern u8 PVCCount;

/*****************************************************************
** Configuration API's
******************************************************************/
////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:    Rfc1577Init
** Parameters:      
**      void *NdisContext         - Context of Ndis
**      struct CONFIG *pConfig    - Basic configuration from NDIS
** ReturnValue:
**      void, Updates its context parameter in the config structure
** Description: 
**      This is a synchronous API.
**      Initializes its own pointers.
**      Updates NDIS call back function pointers and puts the
**      rfc1577 context handle back in the config structure
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
BOOLEAN Rfc1577Init (PATM_USB_INSTANCE pNdisContext, Rfc1577Config_t *pConfig)
{
struct Rfc1577Context *pContext = NULL;

   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
       ("Rfc1577Init: Initializiing Rfc1577\n"));

   /*
   ** If input parameters are NULL or If no ATMaddress is 
   ** configured return;
   */
   if (!pConfig  || !pNdisContext) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577Init: Failure: Bad Config or NDIS Context\n"));
       return FALSE;
   }
   if (!pConfig->ATMAddrType && !pConfig->ATMSubAddrType) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577Init: Failure: Need AtmAddress or Sub Address\n"));
       return FALSE;
   }
   Api1577Initialize(pNdisContext);

   ALLOCATE_MEMORY (&pContext, sizeof(struct Rfc1577Context));
   if (!pContext) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577Init: Failure: Cannot allocate memory for context\n"));
       return FALSE;
   }
   ZERO_MEMORY (pContext, sizeof(struct Rfc1577Context));

   if (pConfig->ATMAddrType == E164_ADDR) 
   {
       COPY_MEMORY( (char *)&(pContext->ATMAddress.Addr), 
                     pConfig->ATMAddress, E164_ADDRLEN);
       pContext->ATMAddress.AddrType = E164_ADDR;
       pContext->ATMAddress.AddrLen = E164_ADDRLEN;
   }
   else if (pConfig->ATMAddrType == NSAP_ADDR)
   {
       COPY_MEMORY( (char *)&(pContext->ATMAddress.Addr), 
                     pConfig->ATMAddress, NSAP_ADDRLEN);
       pContext->ATMAddress.AddrType = NSAP_ADDR;
       pContext->ATMAddress.AddrLen = NSAP_ADDRLEN;
   }

   if (pConfig->ATMSubAddrType == E164_ADDR) 
   {
       COPY_MEMORY( (char *)&(pContext->ATMAddress.SubAddr), 
                     pConfig->ATMSubAddress, E164_ADDRLEN);
       pContext->ATMAddress.SubAddrType = E164_ADDR;
       pContext->ATMAddress.SubAddrLen = E164_ADDRLEN;
   }
   else if (pConfig->ATMSubAddrType == NSAP_ADDR) 
   {
       COPY_MEMORY( (char *)&(pContext->ATMAddress.SubAddr), 
                     pConfig->ATMSubAddress, NSAP_ADDRLEN);
       pContext->ATMAddress.SubAddrType = NSAP_ADDR;
       pContext->ATMAddress.SubAddrLen = NSAP_ADDRLEN;
   }

   /* Update pointers in Rfc1577 Context */
   pContext->NdisHandle                    = pNdisContext;
   pContext->Rfc1577Rfc1483Send            = pConfig->Rfc1577Rfc1483Send;
   pContext->Rfc1577Rfc1483ReceiveComplete = pConfig->Rfc1577Rfc1483ReceiveComplete;
   /* Update pointers in the pConfig structure */
   pConfig->Rfc1483Rfc1577SendComplete     = Rfc1483Rfc1577SendComplete;
   pConfig->Rfc1483Rfc1577Receive          = Rfc1483Rfc1577Receive;
   pConfig->Rfc1577Close                   = Rfc1577Close;
   pConfig->Rfc1577Context                 = pContext;
   pContext->Rfc1577Enable                 = pConfig->Rfc1577Enable;

   pContext->MTUSize                       = pConfig->Rfc1577MTUSize;
   pContext->DefEncap                      = pConfig->DefaultEncap;
   pContext->DefVPI                        = pConfig->DefaultVPI;          
   pContext->DefVCI                        = pConfig->DefaultVCI;          
   pContext->DefHandle                     = pConfig->DefaultHandle;
   
   if (pContext->Rfc1577Enable) 
   {
       /* Initialize the Ndis timer */
       pContext->TimerSpinLock = Api1577AllocateSpinLock(pContext->NdisHandle);
       pContext->TimerHandle   = Api1577InitializeTimer (pNdisContext, 
                                                         Rfc1577ProcessTimeout);
       if (pContext->TimerHandle == RFC1577_ERROR)
       {
           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577Init: Failure: Timer Initialization failure\n"));
           FREE_MEMORY (pContext, sizeof(struct Rfc1577Context));
           pConfig->Rfc1577Context     = NULL;
           return FALSE;
       }
       if ( Api1577SetTimer (pNdisContext, pContext->TimerHandle, 
                            TIMER_PERIODIC_TICKS*1000) == RFC1577_ERROR)
       {
           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577Init: Failure: Could not set timer \n"));
           FREE_MEMORY (pContext, sizeof(struct Rfc1577Context));
           pConfig->Rfc1577Context     = NULL;
           return FALSE;
       }
       /* Internal Timer List initialization */
       Rfc1577TimerInit(pContext); 
   }
   else
   {
       /*
       ** If 1577 is not enabled, dont start any timer, use the 
       ** default VPI, VCI and the Connection handle to transfer data
       ** 1577 is a just a dummy pipe between Ndis and 1483
       */
       pContext->DefEncap = pConfig->DefaultEncap;
       pContext->DefHandle = pConfig->DefaultHandle;
       pContext->DefHandle =  Api1577OpenVC (pContext->NdisHandle, 
                                       pConfig->DefaultVPI, 
                                       pConfig->DefaultVCI, 
                                       pConfig->DefaultEncap);;
       ASSERT (pContext->DefHandle);
   }

   return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577Close
** Parameters: 
**      void *pRfc1577Context    - Rfc1577 Context
** ReturnValue:
**      RFC1577_SUCCESS     -    On success
**      RFC1577_FAILURE     -    On Failure
** Description: 
**  This is synchronous API
**  If rfc1577 is enabled, API, closes all Virtual connections, else
**  it closes the default VPI, VCI.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577Close (void *pRfc1577Context)
{
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;
int IntfIndex, PVCIndex;

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1577Close: Inside function \n"));
   if (!pRfc1577Context) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1577Close: Failure: Bad Context \n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   if (pContext->Rfc1577Enable) 
   {
       /*
       ** Go through the complete interface list. If interface
       ** is up and ARP state is valid, delete the arp entry
       */
       for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex) 
       {
           if (pContext->IntfList[IntfIndex].State == IF_UP) 
           {
               for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
               {
                   pArp = &pContext->IntfList[IntfIndex].ArpTable[PVCIndex];
                   if (pArp->State != ARP_INVALID) 
                   {
                       if (pArp->TimerHandle) 
                       {
                           DeleteTimer (pContext, pArp->TimerHandle);
                           pArp->TimerHandle = NULL;
                       }
                       if (pArp->Handle) 
                           Api1577CloseVC(pContext->NdisHandle, pArp->Handle);
                   }
               } 
           }
       }
   
       Rfc1577TimerClose (pContext);
       Api1577CancelTimer (pContext->NdisHandle, pContext->TimerHandle, \
                           TIMER_PERIODIC_TICKS*1000*2);
       Api1577FreeSpinLock (pContext->NdisHandle, pContext->TimerSpinLock);
   }

   FREE_MEMORY (pContext, sizeof (struct Rfc1577Context));
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577ConfigIntf ()
** Parameters: 
**      void *pRfc1577Context   -   Context 
**      unsigned int IPAddress  -   IPAddress of the interface
**      unsigned int IPMask     -   IPMask 
**      int MTU                 -   Max Tx Unit on that interface
** ReturnValue:
**      RFC1577_SUCCESS         -   on SUCCESS
**      RFC1577_FAILURE         -   on FAILURE
** Description: 
**  This is a synchronous API. It adds a new interface and marks
**  it IF_DOWN.
** Notes:
**  Only after Rfc1577EnableIntf () call, the interface is made
**  active and  InArp-Request/Reply is enabled.
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577ConfigIntf (void *pRfc1577Context, 
                   unsigned int IPAddress, 
                   unsigned int IPMask, 
                   int MTU)
{
struct Rfc1577Context *pContext = NULL;
struct Intf *pIntf = NULL;
int IntfIndex;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                 ("Rfc1577ConfigIntf: Inside function\n "));

   if (!pRfc1577Context)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577ConfigIntf: Failure: Bad Context\n "));
       return RFC1577_FAILURE;
   }

   pContext = (struct Rfc1577Context *)pRfc1577Context;
   if (!pContext->Rfc1577Enable) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577ConfigIntf: Failure: Rfc1577 not enabled\n "));
       return RFC1577_FAILURE;
   }
   
   if (!IPAddress || (pContext->TotalIntf == MAX_INTF))
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577ConfigIntf: Failure: Bad IPAddr/Max Intf reached \n"));
       return RFC1577_FAILURE;
   }

   if (LookupIntf(pContext, IPAddress) != RFC1577_FAILURE) 
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577ConfigIntf: Failure: Intf Already exists\n"));
           return RFC1577_FAILURE;
   } 

   /* Get an empty interface and update this new interface here */
   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex)
   {
      if (pContext->IntfList[IntfIndex].State == IF_INVALID) 
      {
          pIntf = &pContext->IntfList[IntfIndex];
          break;  
      }
   }
   if (!pIntf) 
      return RFC1577_FAILURE;

   ZERO_MEMORY (pIntf, sizeof(struct Intf));
   pIntf->IntfAddr = IPAddress;
   pIntf->IntfMask = IPMask;
   pIntf->State = IF_DOWN;
   pIntf->MTU  = MTU;
   COPY_MEMORY((char *)&pIntf->ATMAddress,(char *)&pContext->ATMAddress,sizeof(struct ATMAddr));
   pContext->TotalIntf++;
   
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:   Rfc1577EnableIntf ()
** Parameters: 
**      void *Handle            -   Context
**      unsigned int IPAddress  -   IPAddress of the interface
** ReturnValue:
**      RFC1577_SUCCESS         - On success
**      RFC1577_FAILURE         - On failure
** Description: 
**  This is a synchronous API.
**  Enables all interfaces and sends Inverse ARP request on all
**  the configured PVC's for that interface.  
** Notes:
**  The Disable/EnableIntf  feature can be used, in cases where 
**  where you want to retry the PVC validation, after Rfc1577 
**  had given up after MAX number of tries to validate.  Enabling
**  will restart all the timer, and try to re-validate.
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577EnableIntf (void *pRfc1577Context, unsigned int IPAddress)
{
int IntfIndex, PVCIndex;
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577EnableIntf: Inside function\n"));

   if (!pRfc1577Context) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577EnableIntf: Failure: Bad Context "));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   if (!pContext->Rfc1577Enable) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577EnableIntf: Failure: 1577 not enabled"));
       return RFC1577_FAILURE;
   }

   if (IPAddress) 
   {
       if ((IntfIndex = LookupIntf(pContext, IPAddress)) == RFC1577_FAILURE) 
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577EnableIntf: Failure: Unable to find interface"));
           return RFC1577_FAILURE;
       } 
   } 
   else
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1577EnableIntf: Failure: Need IPAddress"));
       return RFC1577_FAILURE;
   }

   if (pContext->IntfList[IntfIndex].State == IF_UP)
       return RFC1577_SUCCESS;

   /*Send ARP on all VC's  configured */
   pContext->IntfList[IntfIndex].State = IF_UP;
   for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
   {
       pArp = &pContext->IntfList[IntfIndex].ArpTable[PVCIndex];
       if (pArp->State == ARP_DOWN) 
       {
           pArp->Handle = Api1577OpenVC (pContext->NdisHandle, 
                                         pArp->VPI, 
                                         pArp->VCI, 
                                         pArp->Encap);
           Rfc1577TimerExpiry (pContext, pArp);
       }
   }
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577DisableIntf ()
** Parameters: 
**      void *Handle        -   Context
**      unsigned int        -   IPAddress of the interface
** ReturnValue:
**      RFC1577_SUCCESS         - On success
**      RFC1577_FAILURE         - On failure
** Description: 
**  This is a synchronous API.
**  Closes all the PVC connection
** Notes:
**  The Disable/EnableIntf  feature can be used, in cases where 
**  where you want to retry the PVC validation, after Rfc1577 
**  had given up after MAX number of tries to validate.
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577DisableIntf(void *pRfc1577Context, unsigned int IPAddress)
{
int IntfIndex, PVCIndex;
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577DisableIntf: Inside function\n"));

   if (!pRfc1577Context || !IPAddress) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1577DisableIntf: Failure: Bad Context or IPaddress\n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   if (!pContext->Rfc1577Enable) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1577DisableIntf: Failure: 1577 not enabled\n"));
       return RFC1577_FAILURE;
   }

   if ((IntfIndex = LookupIntf(pContext, IPAddress)) == RFC1577_FAILURE) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1577DisableIntf: Failure: Interface does not exist\n"));
       return RFC1577_FAILURE;
   } 

   if (pContext->IntfList[IntfIndex].State == IF_DOWN)
       return RFC1577_SUCCESS;

   for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex) 
   {
       pArp = &pContext->IntfList[IntfIndex].ArpTable[PVCIndex];
       if (pArp->State != ARP_INVALID) 
       {
           if (pArp->Handle) 
           {
               Api1577CloseVC(pContext->NdisHandle, pArp->Handle);
               pArp->Handle = 0;
           }
           if (pArp->TimerHandle) 
           {
               DeleteTimer(pContext, pArp->TimerHandle);
               pArp->TimerHandle = NULL;
           }
           pArp->State = ARP_DOWN;
       }
   }
   pContext->IntfList[IntfIndex].State = IF_DOWN;
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577DeleteIntf()
** Parameters: 
**      void *Handle            - Context
**      unsigned int            - IPaddress of the interface
** ReturnValue:
**      RFC1577_SUCCESS         - On success
**      RFC1577_FAILURE         - On failure
** Description: 
**  This is a synchronous API.
**  Deletes the Interface, closes all the PVC's, deletes configured
**  timer.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577DeleteIntf(void *pRfc1577Context, unsigned int IPAddress)
{
struct Rfc1577Context *pContext = NULL;
struct Intf *pIntf = NULL;
int IntfIndex, PVCIndex;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577DeleteIntf: Inside function\n"));

   if (!pRfc1577Context)
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577DeleteIntf: Failure: Bad Context\n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   if (!pContext->Rfc1577Enable) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577DeleteIntf: Failure: 1577 not enabled\n"));
       return RFC1577_FAILURE;
   }

   if ((IntfIndex = LookupIntf(pContext, IPAddress)) == RFC1577_FAILURE)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577DeleteIntf: Failure: Bad IPAddress\n"));
       return RFC1577_FAILURE;
   }
   else 
       pIntf = &pContext->IntfList[IntfIndex]; 

   if (pIntf->State == IF_UP) 
   {
       for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex) 
       {
           if (pIntf->ArpTable[PVCIndex].Handle) 
           {
               if (pIntf->ArpTable[PVCIndex].TimerHandle) 
               { 
                   DeleteTimer (pContext, 
                                     pIntf->ArpTable[PVCIndex].TimerHandle);
                   pIntf->ArpTable[PVCIndex].TimerHandle = NULL;
               }
               if (pIntf->ArpTable[PVCIndex].Handle) 
               {
                   Api1577CloseVC(pContext->NdisHandle, 
                           pIntf->ArpTable[PVCIndex].Handle);
                   pIntf->ArpTable[PVCIndex].Handle = 0;
               }
           } 
       }
   }
   ZERO_MEMORY (pIntf, sizeof(struct Intf));
   pContext->TotalIntf--;
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577CreatePVC
** Parameters: 
**      void *Handle            -   Context
**      unsigned int IPAddress  -   IPAddress of the interface
**      unsigned int VPI        -   VPI
**      unsigned int VCI        -   VCI
**      unsigned int EncapType  -   Encapsulation type
**      unsigned int NextHop    -   NextHopIPAddr, This is an optional
**                                  parameter
** ReturnValue:
**      RFC1577_SUCCESS         -   On success
**      RFC1577_FAILURE         -   On failure
** Description: 
**  This is synchronous API
**  Adds an Entry to the ARPtable, and sends a Inverse ARP
**  request on the created PVC. 
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577CreatePVC (void *pRfc1577Context, 
                      unsigned int IPAddress, 
                      unsigned char VPI,
                      unsigned short VCI, 
                      RFC1483_MODE EncapType, 
                      unsigned int NextHopIPAddress) 
{
int IntfIndex,PVCIndex;
struct ARPEntry *pArp = NULL;
struct Rfc1577Context *pContext = NULL;

    DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577CreatePVC: Inside Function\n"));

    if (!pRfc1577Context)
    {
        DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                 ("Rfc1577CreatePVC: Failure: Bad Context\n"));
        return RFC1577_FAILURE;
    }
    pContext = (struct Rfc1577Context *)pRfc1577Context;

    if (!pContext->Rfc1577Enable) 
    {
        DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                 ("Rfc1577CreatePVC: Failure: 1577 not enabled\n"));
        return RFC1577_FAILURE;
    }

    if ((IntfIndex = LookupIntf(pContext, IPAddress)) == RFC1577_FAILURE) 
    {
        DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                 ("Rfc1577CreatePVC: Failure: Bad Source IPAddress\n"));
        return RFC1577_FAILURE;
    }

    for (PVCIndex = 0 ; PVCIndex < MAX_PVC; ++PVCIndex)
    {
        if ((pContext->IntfList[IntfIndex].ArpTable[PVCIndex].VPI == VPI) &&
            (pContext->IntfList[IntfIndex].ArpTable[PVCIndex].VCI == VCI) &&
            (pContext->IntfList[IntfIndex].ArpTable[PVCIndex].State != 
             ARP_INVALID))
        {
            DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                     ("Rfc1577CreatePVC: Failure: PVC Entry already exits\n"));
            return RFC1577_FAILURE;
        } 
    }

    for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
    {
        if (pContext->IntfList[IntfIndex].ArpTable[PVCIndex].State == ARP_INVALID)
        {
            pArp = &(pContext->IntfList[IntfIndex].ArpTable[PVCIndex]);
            ZERO_MEMORY (pArp, sizeof(struct ARPEntry));
            pArp->IntfIndex = IntfIndex;
            pArp->VPI = VPI;
            pArp->VCI = VCI;
            pArp->Encap = EncapType;
            pArp->State = ARP_DOWN;
            pArp->NumOfTries   = 0;
            pArp->TgtIPAddr = NextHopIPAddress;

            /*
            ** Handle as normal timer expiry if interface state is up, 
            ** Send the inverse arp request
            */ 
            if (pContext->IntfList[IntfIndex].State == IF_UP)
            {
                if (!(pArp->Handle =  Api1577OpenVC (pContext->NdisHandle, 
                                        VPI,VCI,EncapType))) 
                {
                    pArp->State = ARP_INVALID;
                    DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                            ("Rfc1577CreatePVC: Failure: Could not open PVC\n"));
                    return RFC1577_FAILURE;
                }
                if (Rfc1577TimerExpiry(pContext, pArp) == RFC1577_FAILURE){
                    Api1577CloseVC (pContext->NdisHandle, pArp->Handle);
                    pArp->State = ARP_INVALID;
                    DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                     ("Rfc1577CreatePVC: Failure: Unable to send arp request\n"));
                    return RFC1577_FAILURE;
                }
            }
            return RFC1577_SUCCESS;
        }
    }

    DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
             ("Rfc1577CreatePVC: Failure: Cannot configure any more PVC's\n"));
    return RFC1577_FAILURE;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577DeletePVC
** Parameters: 
**      void *Rfc1577Context    -   Context
**      unsigned int IPaddress  -   IPAddress of interface
**      unsigned int VPI        -   VPI
**      unsigned int VCI        -   VCI
** ReturnValue:
**      RFC1577_SUCCESS         - On success
**      RFC1577_FAILURE         - On failure
** Description: 
**  This is a synchronous API.
**  Deletes the configured vpi/vci.  
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577DeletePVC (void *pRfc1577Context, 
                  unsigned int IPAddress,
                  unsigned int VPI, 
                  unsigned int VCI)
{
int IntfIndex, PVCIndex;
struct ARPEntry *pArp = NULL;
struct Rfc1577Context *pContext = NULL;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577DeletePVC: Inside Function\n"));

   if (!pRfc1577Context)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577DeletePVC: Failure: Bad 1577 context\n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;
   
   if (!pContext->Rfc1577Enable) {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577DeletePVC: Failure: 1577 not enabled\n"));
       return RFC1577_FAILURE;
   }

   if ((IntfIndex = LookupIntf(pContext, IPAddress)) == RFC1577_FAILURE){
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577DeletePVC: Failure: could not find interface\n"));
       return RFC1577_FAILURE;
   }

   for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
   {
       if ((pContext->IntfList[IntfIndex].ArpTable[PVCIndex].VPI == VPI) &&
           (pContext->IntfList[IntfIndex].ArpTable[PVCIndex].VCI == VCI))
       {
           if (pContext->IntfList[IntfIndex].ArpTable[PVCIndex].State == ARP_INVALID) 
           {
                   return RFC1577_SUCCESS;
           }
           else 
           {
               pArp = &pContext->IntfList[IntfIndex].ArpTable[PVCIndex];
               if (pArp->State != ARP_DOWN) 
                   Api1577CloseVC (pContext->NdisHandle, pArp->Handle);

               if (pArp->TimerHandle) 
               {
                   DeleteTimer (pContext, pArp->TimerHandle);
                   pArp->TimerHandle = NULL;
               }
               ZERO_MEMORY (pArp, sizeof (struct ARPEntry));
               pArp->State = ARP_INVALID;
               return RFC1577_SUCCESS;
           }
       }
   }

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577DeletePVC: Failure: could not find the PVC\n"));
   return RFC1577_FAILURE;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577ResolveARP
** Parameters: 
**      void *pRfc1577Context       -   Context
**      unsigned int NextHopIPAddr  -   IPAddress of the neighbour
**                                      whose Ether-MAC address has to
**                                      to be resolved. 
**                                      In Network byte order
**      char *pMacAddr              -   pointer to a buffer, whose length
**                                      is atleast 6 bytes (sizeof Ether
**                                      MAC Address). 
** ReturnValue:
**      RFC1577_FAILURE         - On failure
**      RFC1577_SUCCESS         - On success
** Description: 
**  This is a synchronous API.  This API returns the MAC address 
**  for the desired NextHopIPAddress, if the PVC to the NextHop is
**  already validated.  
** Notes:
**  In the returned Mac Address, we fill it with the Interface index,
**  ARPIndex and the NextHopIPAddress itself.  This way, when a
**  Normal IP Packet with this MAC address is to be sent,  we avoid
**  Interface/and PVC index lookup. We obtain both, from the MAC Address.
**  MacAddress[0]  -    First byte will contain the Interface Index
**  MacAddress[1]  -    Second byte will contain the ARP Index for the
**                      Arp table maintained per Interface
**  MacAddress[2-5] -   The last four bytes will contain the
**                      NextHopIPAddress
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577ResolveARP (void *pRfc1577Context,
                       unsigned int NHIPAddr,
                       unsigned char *pMacAddr)            
{
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;
unsigned char IntfIndex, PVCIndex;
struct Intf *pIntf = NULL;
int ArpFound = 0 ;
unsigned int NextHopIPAddr;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577ResolveARP : Inside Function\n"));
   if (!pRfc1577Context) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577ResolveARP : Failure: Bad 1577 Context\n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;
   NextHopIPAddr = NTOHL (NHIPAddr);

   if (!pContext->Rfc1577Enable) 
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577ResolveARP : Failure: 1577 not enabled\n"));
       return RFC1577_FAILURE;
   }

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1577ResolveARP : Received Resolve Arp for: %x\n", NextHopIPAddr));

   /* 
   ** Find the corresponding interface, associated with the next hop 
   ** and look for appropriate PVC configured for this nexthop
   */
   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex) 
   {
       pIntf = &pContext->IntfList[IntfIndex];
       if ((pIntf->State == IF_UP) &&
           (((pIntf->IntfAddr) & (pIntf->IntfMask)) == 
            ((pIntf->IntfMask) & NextHopIPAddr))) 
       {
           DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                   ("Rfc1577ResolveArp: Found the Interface looking for arp\n"));
           for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex) 
           {
               pArp = &(pIntf->ArpTable[PVCIndex]);
               if ((pArp->State == ARP_VALID) && 
                   (pArp->TgtIPAddr == NextHopIPAddr))
               {
                   ArpFound = 1;
                   break;
               }
           }
       }
       if (ArpFound) 
       {
           break;
       }
   }

   if (!ArpFound) {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("\n Rfc1577ResolveArp: Failure: EtherArp could not be resolved"));
       return RFC1577_FAILURE;
   }

   pMacAddr[0] = IntfIndex;
   pMacAddr[1] = PVCIndex;
   COPY_MEMORY(&pMacAddr[2], (char *)&NextHopIPAddr, IPADDR_LEN);
   COPY_MEMORY (&pArp->EthMacAddr, pMacAddr, ETH_LENGTH_OF_ADDRESS);

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("\n Rfc1577ResolveArp: EtherArp successfully resolved"));
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  NdisRfc1577Send
** Parameters: 
**      PATM_USB_INSTANCE  pInstance    - Instance associated with Ndis
**      PNDIS_PACKET       pPacket      - pointer to Ndis Packet   
**      unsigned int Flag               -   unused?
** ReturnValue:
**      NDIS_STATUS     -         
**          returns NDIS_STATUS_SUCCESS on success
**          returns NDIS_STATUS_FAILURE on failure
**          returns NDIS_STATUS_PENDING on pending
** Description: 
** Notes:
**  Mapping to the PVC is done after getting the data from the pMacAddress.
**  See Note in  Rfc1577ResolveARP
*/
////////////////////////////////////////////////////////////////////////////////
NDIS_STATUS NdisRfc1577Send (PATM_USB_INSTANCE pInstance, 
                 PNDIS_PACKET pPacket, 
                 unsigned int Flag,
				 gsi_lan_t    *plan) 
{
PNDIS_BUFFER           NdisBuffer;
UINT                   PacketLength;
UINT                   BufferCount;
UINT                   PhyBufferCount;
UINT                   BufferLength;
PMAC_HEADER            MacHeader;
struct Rfc1577Context  *pContext;
struct ARPEntry        *pArp =NULL;
struct Intf            *pIntf = NULL;
unsigned int           NextHopAddr;
char                   pMacAddr[ETH_LENGTH_OF_ADDRESS];
PETH_ARP_PACKET        EthArpPkt;
unsigned int           SrcIpAddr, IpMask;
unsigned char          broadcastMsg[ETH_LENGTH_OF_ADDRESS]= { 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF };

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("NdisRfc1577Send: Inside Function\n"));

   ASSERT( pInstance);
   ASSERT( pPacket);
   pContext = (struct Rfc1577Context *) Api1577GetContext (pInstance);
   ASSERT (pContext);

   /*If Rfc1577 is not enabled send, packet across the default handle */
   if (!pContext->Rfc1577Enable) 
	{
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
       ("NdisRfc1577Send: 1577 disabled, sending packet on default handle\n"));
        return ((pContext->Rfc1577Rfc1483Send) (pContext->NdisHandle, 
                                            pPacket, 
                                            pContext->DefHandle, 
                                            FALSE,
											plan));
   }
   NdisQueryPacket(pPacket,&PhyBufferCount,&BufferCount,&NdisBuffer,&PacketLength);
   ASSERT(BufferCount);
   ASSERT(NdisBuffer);
   
   NdisQueryBuffer( NdisBuffer, (PVOID *)&MacHeader, &BufferLength);
   ASSERT(BufferLength >= ETHERNET_HEADER_SIZE);

   /*
    * Now, if the packet is of type IP, get the destination MAc address from
    * the MAC header, and do a Arp table look up based on the destination
    * Mac address, resolve the Connection handle and send the packet across.
    */
   if (MacHeader->EtherType == ETHERTYPE_IP )
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
               ("NdisRfc1577Send: Received IP packet from Ndis\n"));

       /* All the boradcast message can pass without the ARP search */
       if (COMPARE_MEMORY(MacHeader->DstAddress, &(broadcastMsg[0]), ETH_LENGTH_OF_ADDRESS) == 0)
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
               ("NdisRfc1577Send: Broadcast Message/Packet\n"));

           return ((pContext->Rfc1577Rfc1483Send)(pContext->NdisHandle,pPacket,pContext->DefHandle,FALSE,plan));
       }

       COPY_MEMORY((void *)pMacAddr, MacHeader->DstAddress, ETH_LENGTH_OF_ADDRESS);

       pArp = GetArpEntryFromMac (pContext, pMacAddr);
       if (!pArp) 
       {
           DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("NdisRfc1577Send: Failure: Could not resolve EthMac-> ATMARP Entry\n"));
           return NDIS_STATUS_FAILURE;
       }

       pIntf = &pContext->IntfList[pArp->IntfIndex];
       if (pIntf->State != IF_UP)
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("NdisRfc1577Send: Failure: Interface down, dropping IP packet\n"));
           return NDIS_STATUS_FAILURE;
       }

       if ((pArp->State == ARP_VALID)) 
       {   
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("NdisRfc1577Send: Sending IP Packet across PVC %d %d\n",
                     pArp->VPI, pArp->VCI));
           return ((pContext->Rfc1577Rfc1483Send) (pContext->NdisHandle, 
                                            pPacket, 
                                            pArp->Handle, 
                                            FALSE,plan));
       }

       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("NdisRfc1577Send: Failure: ATMARP is not yet resolved\n"));
       return NDIS_STATUS_FAILURE;
   }
   else if ((MacHeader->EtherType == ETHERTYPE_ARP))
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                 ("NdisRfc1577Send: Received an EtherArp packet from Ndis\n"));
       EthArpPkt = Api1577GetEtherHeader(pPacket);
       COPY_MEMORY ((void *)&NextHopAddr, EthArpPkt->DestIpAddr, IP_ADDRESS_SIZE);
       NextHopAddr = NTOHL (NextHopAddr);
       
       COPY_MEMORY ((void *)&SrcIpAddr, EthArpPkt->SendIpAddr, IP_ADDRESS_SIZE);
       SrcIpAddr = NTOHL (SrcIpAddr);   

       /*
        * Now the packet is of type ETHER_ARP (could be reply or a request)
        * Get the destination IPAddress from the Ether Packet. and hence,
        * resolve the Corresponding ARP entry and send it across the
        * resolved handle.
        */
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
               ("SrcIpAddr = %X\n", SrcIpAddr));
       if(LookupIntf(pContext, SrcIpAddr) == RFC1577_FAILURE)
       {
           /* This Interface doesn't exist. Create New Interface!!*/
           IpMask = Rfc1577GetNetIDMask(SrcIpAddr );

           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                 ("IpMask = %X\n", IpMask));

           Rfc1577ConfigIntf(pContext, SrcIpAddr, IpMask, pContext->MTUSize);
           Rfc1577EnableIntf (pContext, SrcIpAddr);
           Rfc1577CreatePVC(pContext, 
                       SrcIpAddr,
                       pContext->DefVPI, 
                       pContext->DefVCI, 
                       pContext->DefEncap,
                       0);
       }
       
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
               ("NdisRfc1577Send: EthertypeArp for NextHop: %x\n", NextHopAddr));
       pArp = GetArpEntry (pContext, NextHopAddr);
       if (!pArp) 
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("NdisRfc1577Send: Failure: Unable to resolve ATMARP Entry\n"));
           return NDIS_STATUS_FAILURE;
       }

       return ((pContext->Rfc1577Rfc1483Send) (pContext->NdisHandle, 
                                            pPacket, 
                                            pArp->Handle, 
                                            FALSE, plan));
   } 

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
       ("NdisRfc1577Send: Failure: Unknown Packet type"));
   return NDIS_STATUS_FAILURE;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1483Rfc1577SendComplete
** Parameters: 
**      PATM_USB_INSTNACE  pInstance    -   Instance
**      VCCHandle_t                     -   Connection Handle
**      PNDIS_PACKET                    -   pointer to Ndis packet
**      BOOLEAN                         -   ArpFlag, True, in case
**                                          of Arp Flag
** ReturnValue:
**      void
** Description: 
  **  This is a synchronous API. Called by Rfc1577Rfc1483SendComplete
**  routine.  It frees the Memory was allocated by Rfc1577 Module,
**  else calls NdisRfc1577Send complete routine.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1483Rfc1577SendComplete (PATM_USB_INSTANCE pInstance,
            VCCHandle_t pVCCHandle,
            PNDIS_PACKET  pPacket,
            BOOLEAN ArpFlag)
{
   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1483Rfc1577SendComplete: Inside Function\n"));
   ASSERT (pInstance);
   if (ArpFlag) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1483Rfc1577SendComplete: Send complete for Arp message\n"));
       Api1577FreeNdisPacket(pInstance, pPacket);
   }
   else
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1483Rfc1577SendComplete: Send complete for Ndis message\n"));
       NdisMSendComplete(pInstance->MiniportAdapterHandle,pPacket,NDIS_STATUS_SUCCESS, NULL); /* TODO */
   }
   return;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1483Rfc1577Receive
** Parameters: 
**      void *pRfc1577Context       - 1577 Context 
**      VCCHandle_t                 - PVC connection handle
**      XFER_DATA_BUFFER * pXData   - pointer to XFER_DATA  
**      unsigned char *             - pointer to EtherMacHeader
**      unsigned long               - Size of the EtherMacHeader
**      unsigned short Flag         - ARP/IP pkt flag ETHERTYPE_ARP
**                                    ETHERTYPE_IP
** ReturnValue:
**      void
** Description: 
**  This API is a asynchronous API.
**  Calls NdisReceive, if its a IP Packet, else it handles the
**  ARP
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1483Rfc1577Receive (void * pRfc1577Context,
            VCCHandle_t VCCHandle,
            PXFER_DATA_BUFFER pXData,
            unsigned char *pEthHeader,
            unsigned long HeaderSize,
            BOOLEAN ArpFlag)
{
PATM_USB_INSTANCE      pInstance;
unsigned char          *Data       = pXData->Data;
unsigned long          Size        = pXData->Size;
struct Rfc1577Context  *pContext   = NULL;
int					   ilanindex   = 0;
gsi_lan_t    		   *plan = NULL; 


   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1483Rfc1577Receive: Inside Function\n"));
   if (!pRfc1577Context)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("\n Rfc1483Rfc1577Receive: Failure: Bad context"));
       return;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;
   pInstance = pContext->NdisHandle;
   ASSERT(pInstance);
   
   Data += HeaderSize;
   Size -= HeaderSize;

   /*
   ** If The packet is of type ATMArp, the ArpFlag is set, and we decode
   ** and reply appropriately.
   */
   if(ArpFlag)
   {
       if (!pContext->Rfc1577Enable) 
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
           ("Rfc1483Rfc1577Receive: Failure: Received ATMArp, when 1577 disabled \n"));
       }
       else
       {
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                   ("Rfc1483Rfc1577Receive: Received an ATMArp Message\n"));
           DecodeARPPkt ( pContext, VCCHandle, Data,(int) Size);
       }
   }
   else
   {
       /*
       ** We dont bother on what PVC connection handle it has arrived
       ** Give the data to NDIS. Let it figure out
       */

	   /*
	   **	Here we need to map VCHandle to the lan device
	   */

	   for(ilanindex = 0; ilanindex< PVCCount; ilanindex++)
	   {
		   if(VCCHandle == pInstance->lan[ilanindex].Rfc1483VcHandle)
		   {
			   plan = &(pInstance->lan[ilanindex]);
			   break;
		   }
	   } /* We have found the lan device */


       NdisMEthIndicateReceive(pInstance->MiniportAdapterHandle, Data,
                               pEthHeader,ETHERNET_HEADER_SIZE, Data,Size,Size,plan);
       NdisMEthIndicateReceiveComplete(pInstance->MiniportAdapterHandle);
   }
   
	/*It will free Buffer*/
   pContext->Rfc1577Rfc1483ReceiveComplete(plan,pXData);
   return;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577GetNoOfArpEntries
** Parameters: 
**      struct Rfc1577Context       -   Context
** ReturnValue:
**      RFC1577_SUCCESS         - On success
**      RFC1577_FAILURE         - On failure
** Description: 
**  Returns the no of entries in the Arp table
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577GetNoOfArpEntries (void *pRfc1577Context)
{
struct Rfc1577Context *pContext = NULL;
int NoOfEntry = 0;
int IntfIndex, PVCIndex;
struct Intf *pIntf = NULL;
struct ARPEntry *pArp = NULL;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577GetNoOfArpEntries: Inside Function\n"));
   if (!pRfc1577Context)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577Status: Failure: Bad Context \n"));
       return RFC1577_FAILURE;
   }

   pContext = (struct Rfc1577Context *)pRfc1577Context;
   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex) 
   {
       pIntf = &pContext->IntfList[IntfIndex];
       if (pIntf->State != IF_INVALID)
       {
           for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex) 
           { 
               pArp = &pIntf->ArpTable[PVCIndex];
               if (pArp->State != ARP_INVALID) 
                   NoOfEntry++;
           }
       } 
   }
   return NoOfEntry;
}

/**************************************************************************
** Internal API's
***************************************************************************/
////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  LookupIntf
** Parameters: 
**      struct Rfc1577Context       -   Context
**      unsigned int IPaddress      -   IPAddress of Interface
** ReturnValue:
**      RFC1577_FAILURE         - On failure
**      Interface Index         -   InterfaceIndex
** Description: 
**  Goes through the complete list of Interface list, and returns
**  the interface index if found.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int LookupIntf(struct Rfc1577Context *pContext, unsigned int IPAddress) 
{
int IntfIndex;
    
   if (!pContext)
       return RFC1577_FAILURE;

   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex)
   {
       /* We only have one SourceIP address and We can not handle
       ** More than ONE
       */
       if (pContext->IntfList[IntfIndex].IntfAddr != IPAddress)
          pContext->IntfList[IntfIndex].IntfAddr = IPAddress;

       if (pContext->IntfList[IntfIndex].State != IF_INVALID)
          return IntfIndex;
   }

   return RFC1577_FAILURE;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  BuildARPPkt
** Parameters: 
**      struct Rfc1577Context *Context      - Context
**      int Type                            - Type of ARP
**      char *Buffer                        - Buffer pointer
**      int BuffLen                         - Buffer Len
**      struct ARPEntry*                    - pointer to ARPEntry
** ReturnValue:
**      RFC1577_FAILURE         - On failure
**      BufferLen               - On success
** Description: 
**  Builds a (Inverse) ARP (request/reply) packet and returns the 
**  bufferlen
** Notes:
**  Only Inverse ARP -Request and reply is currently supported
*/
////////////////////////////////////////////////////////////////////////////////
int BuildARPPkt (struct Rfc1577Context *pContext,
             int Type, 
             unsigned char *pBuff,
             int BuffLen,
             struct ARPEntry *pArp)
{
struct CIPArpHdr *pHdr = NULL;
struct ATMAddr *Addr = NULL;
unsigned char *pTmpBuff = NULL;
int  PktLen;
unsigned int IPAddr;
    
   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("BuildArpPacket: Inside function\n"));
   if (!pContext || !pBuff || !pArp)
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL,  DBG_PRINT_LEVEL_ERR,
               ("BuildArpPacket: Failure: Bad context or pBuff or pArp Entry \n"));
       return RFC1577_FAILURE;
   }

   pHdr = (struct CIPArpHdr *)pBuff; 
   ZERO_MEMORY( pHdr, sizeof(struct CIPArpHdr) ); 
   
   pHdr->ArpHrd  =  HTONS(ATM_ADDR_FAMILY);
   pHdr->ArpPro  =  HTONS(IP_PROTOCOL);

   Addr = &(pContext->IntfList[pArp->IntfIndex].ATMAddress);
   pHdr->ArpShtl = (Addr->AddrType == E164_ADDR) ? (Addr->AddrLen | TL_E164) : Addr->AddrLen;
   pHdr->ArpSstl = (Addr->SubAddrType == E164_ADDR) ? (Addr->SubAddrLen | TL_E164) : Addr->SubAddrLen;

   pHdr->ArpOp   =   HTONS(Type);
   pHdr->ArpSpln =   IPADDR_LEN;
   pTmpBuff      = pHdr->Data;
   PktLen        = sizeof (struct CIPArpHdr) - 1;
   COPY_MEMORY (pTmpBuff, Addr->Addr, Addr->AddrLen);
   pTmpBuff+= Addr->AddrLen;
   PktLen  +=Addr->AddrLen;

   COPY_MEMORY (pTmpBuff, Addr->SubAddr, Addr->SubAddrLen);
   pTmpBuff+= Addr->SubAddrLen;
   PktLen  += Addr->SubAddrLen;

   IPAddr  =  HTONL(pContext->IntfList[pArp->IntfIndex].IntfAddr);
   COPY_MEMORY (pTmpBuff, (char *)&IPAddr, IPADDR_LEN);
   pTmpBuff+= IPADDR_LEN;
   PktLen  += IPADDR_LEN;

   switch (Type)
   {
       case INARP_REQUEST:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("BuildArpPkt: Inverse Arp Request  \n"));
           break;

       case INARP_REPLY:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("BuildArpPacket: Preparing Inverse Arp Reply  \n"));
           Addr = &pArp->TgtAddr;
           pHdr->ArpThtl = (Addr->AddrType == E164_ADDR) ? 
                                               (Addr->AddrLen | TL_E164) : Addr->AddrLen;
           pHdr->ArpTstl = (Addr->SubAddrType == E164_ADDR) ?
                                               (Addr->SubAddrLen | TL_E164) : Addr->SubAddrLen;

           pHdr->ArpTpln    =   IPADDR_LEN;
           COPY_MEMORY (pTmpBuff, Addr->Addr, Addr->AddrLen);
           pTmpBuff+= Addr->AddrLen;
           PktLen  += Addr->AddrLen;

           COPY_MEMORY (pTmpBuff, Addr->SubAddr, Addr->SubAddrLen);
           pTmpBuff+= Addr->SubAddrLen;
           PktLen  += Addr->SubAddrLen;

           IPAddr =  HTONL(pArp->TgtIPAddr);
           COPY_MEMORY (pTmpBuff, (char *)&IPAddr, IPADDR_LEN);
           pTmpBuff+= IPADDR_LEN;
           PktLen  += IPADDR_LEN;
           break;

       case ARP_REQUEST:
       case ARP_REPLY:
       default:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                    ("\n BuildArpPkt: FailureOnly INverseARP Req/Reply supported "));
           return RFC1577_FAILURE;
   }

   return PktLen-1;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  DecodeARPPkt
** Parameters: 
**      struct Rfc1577Context *     -   Context
**      VCCHandle_t VCCHandle       -   VC Connection Handle
**      unsigned char *pBuff        -   Data Buffer
**      int BuffLen                 -   Buffer Len
** ReturnValue:
**      RFC1577_SUCCESS             -   On success
**      RFC1577_FAILURE             -   On failure
** Description: 
**      Decodes the ARP Packet, returns back a reply if it was an
**  Inverse ATM arp.  It also updates the ARP table on receiving a
**  reply.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int DecodeARPPkt (struct Rfc1577Context *pContext, 
              VCCHandle_t VCCHandle,
              unsigned char *pBuff, 
              int BuffLen)
{
struct CIPArpHdr *pHdr = NULL;
struct ARPEntry *pArp = NULL;
char *pTmpBuff = NULL;
PXFER_DATA_BUFFER pXData = NULL;
int RetVal;
unsigned int IPAddr;
PNDIS_PACKET pPacket;
    
   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL,  DBG_PRINT_LEVEL_TRACE,
            ("DecodeArpPkt: Inside Function\n"));
   if (!pContext || !pBuff) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("DecodeArpPkt: Failure: Bad context or pBuff pointer\n"));
       return RFC1577_FAILURE;
   }

   if ((BuffLen < sizeof (struct CIPArpHdr)) || (BuffLen > MAX_ARP_PKTLEN)) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("DecodeArpPkt: Failure: Bad PacketLen\n"));
       return RFC1577_FAILURE;
   }

   if ((pArp = LookForVCC (pContext, VCCHandle)) == NULL) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("DecodeArpPkt: Failure: Unable to find VCC handle \n"));
       return RFC1577_FAILURE;
   }

   /* Get the byte ordering to the host order */
   pHdr = (struct CIPArpHdr *)pBuff;
   pHdr->ArpHrd  = NTOHS (pHdr->ArpHrd);
   pHdr->ArpPro  = NTOHS (pHdr->ArpPro);
   pHdr->ArpOp   = NTOHS (pHdr->ArpOp);
   
   if (pHdr->ArpShtl & (unsigned char)0x40)
   {
       pArp->TgtAddr.AddrLen   = pHdr->ArpShtl ^ (unsigned char)0x40;
       pArp->TgtAddr.AddrType  = E164_ADDR;
   }
   else 
   {
       pArp->TgtAddr.AddrLen   = pHdr->ArpShtl;
       pArp->TgtAddr.AddrType  = NSAP_ADDR;
   }

   if (pHdr->ArpSstl & (unsigned char) 0x40) 
   {
       pArp->TgtAddr.SubAddrLen    = pHdr->ArpSstl ^ (unsigned char)0x40;
       pArp->TgtAddr.SubAddrType   = E164_ADDR;
   }
   else 
   {
       pArp->TgtAddr.SubAddrLen    = pHdr->ArpSstl;
       pArp->TgtAddr.SubAddrType   = NSAP_ADDR;
   }

   pTmpBuff = (char *)pHdr->Data;
   COPY_MEMORY ((char *)&pArp->TgtAddr.Addr, pTmpBuff, pArp->TgtAddr.AddrLen);
   pTmpBuff += pArp->TgtAddr.AddrLen;
   COPY_MEMORY ((char *)&(pArp->TgtAddr.SubAddr), pTmpBuff, pArp->TgtAddr.SubAddrLen);
   pTmpBuff += pArp->TgtAddr.SubAddrLen;
   if (pHdr->ArpSpln == IPADDR_LEN) 
   {
       COPY_MEMORY ((char*)&IPAddr, pTmpBuff, IPADDR_LEN);

       COPY_MEMORY ((char*)&pArp->TgtIPAddr, (char *)&IPAddr, IPADDR_LEN);
       pArp->TgtIPAddr =  HTONL(pArp->TgtIPAddr);
   }
   pTmpBuff += pHdr->ArpSpln;

   switch (pHdr->ArpOp)
   {
       case INARP_REQUEST:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("DecodeArpPkt: Received an INARP request\n"));

           /* Send a InARP Reply */
           pXData = NULL;
           pXData = Api1577GetBuffer(pContext->NdisHandle);
           if (!pXData) 
           {
                DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                          ("DecodeArpPkt: Unable to allocate memory\n"));
                return RFC1577_FAILURE;
           }
 
           if ((RetVal = BuildARPPkt (pContext, INARP_REPLY, pXData->Data,
                MAX_ARP_PKTLEN, pArp)) == RFC1577_FAILURE) 
           {
               DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                        ("DecodeArpPkt Could not build the ARP reply\n"));
               Api1577FreeBuffer (pContext->NdisHandle, pXData);
               return RFC1577_FAILURE;
           }
           pXData->Size                = RetVal;
           pXData->ArpFlag             = TRUE;
           pXData->AllocatorContext    = pContext;

           pPacket = Api1577MakeNdisPacket(pContext->NdisHandle, pXData);
           if (pArp->TimerHandle)
           {
               DeleteTimer(pContext, pArp->TimerHandle); 
           }
           pArp->TimerHandle =AddTimer(pContext, pArp, ARP_TIMEOUT, Rfc1577TimerExpiry);
           pArp->State = ARP_VALID;
           pArp->NumOfTries = 0;
           if ((RetVal = pContext->Rfc1577Rfc1483Send (pContext->NdisHandle,
									pPacket, pArp->Handle, TRUE, NULL)) == NDIS_STATUS_FAILURE) /* TODO */
           {
               DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                        ("DecodeArpPkt: Failure: Sending INARP reply failed\n"));
               Api1577FreeNdisPacket(pContext->NdisHandle, pPacket);
               return NDIS_STATUS_FAILURE;
           }
         
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("DecodeArpPkt: Sent an INARP reply\n"));
           if (RetVal == NDIS_STATUS_SUCCESS) 
           {
               Api1577FreeNdisPacket(pContext->NdisHandle, pPacket);
           }
           else 
           {
               /* Dont know, flow should not come here */
               DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                        ("Received a NDIS_STATUS_PENDING, when expecting SUCCESS\n"));
           }
           break;

       case INARP_REPLY:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                    ("DecodeArpPkt: Received an INARP reply\n"));
           if (pArp->TimerHandle)
           {
               DeleteTimer(pContext, pArp->TimerHandle); 
           }
           pArp->TimerHandle   = AddTimer (pContext, pArp,
                                           ARP_TIMEOUT, 
                                           Rfc1577TimerExpiry);
           pArp->State         = ARP_VALID;
           pArp->NumOfTries    = 0;
           break;

       case ARP_REQUEST:
            DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                     ("DecodeArpPkt: Received ArpReq: Only InARP supported\n"));
            break;
       case ARP_REPLY:
            DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                     ("DecodeArpPkt: Received ArpReply: Only InARP supported\n"));
            break;

      default:
           DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                     ("DecodeArpPkt: Failure: Unknown packet type \n "));
   }
   return RFC1577_SUCCESS;
}
    
////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  Rfc1577TimerExpiry
** Parameters: 
**      void *pRfc1577Context   -     Context
**      void *pInputParam       -     Arp entry
** ReturnValue:
**      RFC1577_FAILURE         - On failure
**      RFC1577_SUCCESS         - On success
** Description: 
**      Sends an InATMARP request, if the num of tries is less than
**  Configured maximum value.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
int Rfc1577TimerExpiry (void *pRfc1577Context, void *pInputParam) 
{
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;
int RetVal;
PXFER_DATA_BUFFER pXData = NULL;
PNDIS_PACKET pPacket;
PATM_USB_INSTANCE Instance;

   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL,  DBG_PRINT_LEVEL_TRACE,
            ("Rfc1577TimerExpiry: Inside Function\n"));
     
   if ((!pRfc1577Context) || (!pInputParam)) 
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL,  DBG_PRINT_LEVEL_ERR,
               ("Rfc1577TimerExpiry: Failure: Bad context or input param\n"));
       return RFC1577_FAILURE;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;
   pArp =(struct ARPEntry *)pInputParam;
   Instance = pContext->NdisHandle;

   /* Make a sanity check whether the interface is UP */
   if (pContext->IntfList[pArp->IntfIndex].State != IF_UP) 
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL,  DBG_PRINT_LEVEL_ERR,
               ("Rfc1577TimerExpiry: Failure: Interface state is down/invalid \n"));
       return RFC1577_FAILURE;
   }

   if ( DspNotConnected ( (PUSB_PHY)(Instance->UsbPhyHandle)) )
   {
           pArp->TimerHandle = AddTimer (pContext, pArp,
                                         ARP_REQ_TIMEOUT, 
                                         Rfc1577TimerExpiry);
           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
             ("Rfc1577TimerExpiry: Failure:  Physical link not up\n"));
           return RFC1577_SUCCESS;
    }

   if (pArp->NumOfTries > MAX_TRIES) 
   {
        DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                 ("Rfc1577TimerExpiry: Max number of tries reached\n"));
        pArp->TimerHandle = NULL;
        pArp->State = ARP_MAXTRY;
        return RFC1577_FAILURE;
   }

   pXData = NULL;
   pXData = Api1577GetBuffer(pContext->NdisHandle);
   if (!pXData)
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577TimerExpiry: Unable to allocate memory\n"));
       return RFC1577_FAILURE;
   }

   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                ("Rfc1577TimerExpiry: Dumping the complete ARP table\n"));
   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                ("Rfc1577TimerExpiry: Building Arp Request\n"));

   if ((RetVal = BuildARPPkt (pContext, INARP_REQUEST, 
                              pXData->Data, 
                              MAX_ARP_PKTLEN, 
                              pArp)) < 0) 
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR, 
               ("Rfc1577TimerExpiry: Failure: Unable to build InARP packet\n"));
       Api1577FreeBuffer (pContext->NdisHandle, pXData);
       pArp->TimerHandle = AddTimer (pContext, pArp,
                                     ARP_REQ_TIMEOUT, 
                                     Rfc1577TimerExpiry);
       return RFC1577_FAILURE;
   }

   pXData->Size             = RetVal;
   pXData->AllocatorContext = pContext;
   pXData->ArpFlag          = TRUE;
   pPacket                  = Api1577MakeNdisPacket(pContext->NdisHandle, pXData);

   pArp->State = ARP_SENT;
   pArp->TimerHandle = AddTimer (pContext, pArp,
                                 ARP_REQ_TIMEOUT, 
                                 Rfc1577TimerExpiry);
   pArp->NumOfTries ++;
   if ((RetVal = pContext->Rfc1577Rfc1483Send (pContext->NdisHandle, 
               pPacket, pArp->Handle,  TRUE, NULL)) == NDIS_STATUS_FAILURE)		/* TODO */
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR, 
                ("Rfc1577TimerExpiry: Failure: 1483 send failed\n "));
       Api1577FreeNdisPacket(pContext->NdisHandle, pPacket);
       return RetVal;
   }
   
   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE, 
           (" Rfc1577TimerExpiry: Sent the ARP request packet\n"));
   if (RetVal == NDIS_STATUS_SUCCESS) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE, 
                 ("Rfc1577TimerExpiry: Sent ARP packet\n "));
       Api1577FreeNdisPacket(pContext->NdisHandle, pPacket);
   }
   else
   {
           /* Flow should not reach here */
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                 ("Rfc1577TimerExpiry: unexpected NDIS_STATUS_PENDING\n"));
   }
   
   return RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  LookForVCC
** Parameters: 
**      struct Rfc1577Context *     -   Context
**      VCCHandle_t VCCHandle       -   Handle
** ReturnValue:
**      struct ARPEntry *           -   The arp entry
** Description: 
**      Looks for the Arp entry based on connection handle 
**      on all the configured PVCs across all the interfaces 
**      and returns the ARP entry.
** Notes:
*/
////////////////////////////////////////////////////////////////////////////////
struct ARPEntry * LookForVCC (struct Rfc1577Context* pContext, VCCHandle_t VCCHandle)
{
int IntfIndex, PVCIndex;
struct Intf *pIntf = NULL;
struct ARPEntry *pArp = NULL;

   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex)
   {
       pIntf = &(pContext->IntfList[IntfIndex]);
       for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
       {
           pArp = &pIntf->ArpTable[PVCIndex];
           if (pArp->State != ARP_INVALID)
               if (pArp->Handle == VCCHandle)
                   return pArp;
       }
   }
   return NULL;
}

////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  GetArpEntry
** Parameters: 
**      struct Rfc1577Context       -   Context
**      unsigned int NHIPaddress      -   NextHopIPAddress
** ReturnValue:
**      NULL            -   on failure
**      struct ARPEntry -   pointer to valid arp entry
** Description: 
**  Goes through the complete list of Interface list, and returns
**  the valid arp entry associated with this nexthop ip address.
** Notes:
**  Need a better way of searching mechanism.
*/
////////////////////////////////////////////////////////////////////////////////
struct ARPEntry * GetArpEntry(struct Rfc1577Context *pContext, unsigned int NextHopIPAddr)
{   
struct ARPEntry *pArp = NULL;
struct Intf *pIntf= NULL;
int IntfIndex, PVCIndex;

   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex) 
   {
       pIntf = &pContext->IntfList[IntfIndex];
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("GetArpEntry: IntfIndex %d IntfAddr %x \n", IntfIndex, pIntf->IntfAddr));

       if ((pIntf->State == IF_UP) &&
           (((pIntf->IntfAddr) & (pIntf->IntfMask)) == 
            ((pIntf->IntfMask) & NextHopIPAddr))) 
       {
           for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
           {
               pArp = &(pIntf->ArpTable[PVCIndex]);
               DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                   ("GetArpEntry: ARP entry index: %d pArp->TgtIPAddr: %x \n", 
                       PVCIndex, pArp->TgtIPAddr));
               if ((pArp->State != ARP_INVALID) && 
                   (pArp->TgtIPAddr == NextHopIPAddr))
               {
                   return pArp;
               }
           }
       }
       
   }
   return NULL;
}
 
////////////////////////////////////////////////////////////////////////////////
/*
** FunctionName:  GetArpEntryFromMac
** Parameters: 
**      struct Rfc1577Context       -   Context
**      char * pMacAddr             -   Mac Address
** ReturnValue:
**      NULL            -   on failure
**      struct ARPEntry -   pointer to valid arp entry
** Description: 
**  Obtain the Interface index and ArpIndex from the MacAddress and
**  and return the Arp entry
** Notes:
**  Need a better way of searching mechanism.
*/
////////////////////////////////////////////////////////////////////////////////
struct ARPEntry * GetArpEntryFromMac (struct Rfc1577Context *pContext, unsigned char *pMacAddr)
{
struct ARPEntry *pArp = NULL;
int IntfIndex, PVCIndex;
unsigned int NextHopIPAddr;

    IntfIndex   = *pMacAddr;
    pMacAddr++;
    PVCIndex    = *pMacAddr;
    pMacAddr++;
    COPY_MEMORY ((void*)&NextHopIPAddr, pMacAddr, IP_ADDRESS_SIZE);

    if ((IntfIndex < 0) || (IntfIndex >= MAX_INTF))
        return NULL;
    if ((PVCIndex < 0) || (PVCIndex >= MAX_PVC))
        return NULL;

    pArp = &pContext->IntfList[IntfIndex].ArpTable[PVCIndex];
    if ((pArp->State == ARP_VALID) && (pArp->TgtIPAddr == NextHopIPAddr))
        return pArp;

    return NULL;
}

/**************************************************************************
** Single shot periodic tick based Timer Implementation
***************************************************************************/
////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: Rfc1577_TimerInit
** Inputs: 
**   Rfc1577Context: Pointer to the context
** Outputs: 
**   None
** ReturnValue: 
**   None
** Description: 
**   Sets timer list to NULL
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1577TimerInit(struct Rfc1577Context *pRfc1577Context)
{
DrvTimerList_t *DrvTimerList = NULL;

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("Rfc1577TimerInit: Inside Function \n"));
   DrvTimerList = &(pRfc1577Context->DrvTimerList);
   DrvTimerList->DrvTimerLeastTimer= NULL;
   DrvTimerList->CurrentTicks= 0L;
   return;
}

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: Rfc1577TimerClose
** Inputs: 
**    Rfc1577Context: Pointer to the context
** Outputs: 
**     None
** ReturnValue: 
**     None
** Description: 
**     Deletes All the current timer list
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1577TimerClose(struct Rfc1577Context *pRfc1577Context)
{
DrvTimerNode_t *LocalTimer = NULL, *pTemp= NULL;
DrvTimerList_t *DrvTimerList= NULL;
    
   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("Rfc1577TimerClose: Inside function \n"));

   /*********** Lock Here ******************/
   Api1577AcquireSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   DrvTimerList = &(pRfc1577Context->DrvTimerList);
   
   LocalTimer = DrvTimerList->DrvTimerLeastTimer;
   while (LocalTimer)
   {
       pTemp = LocalTimer;
       LocalTimer = LocalTimer->Next;
       FREE_MEMORY (pTemp, sizeof (DrvTimerNode_t));           
   }
   DrvTimerList->DrvTimerLeastTimer = NULL;

   /********** Unlock here ****************/
   Api1577ReleaseSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   return;
}


////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: Rfc1577InitTimerCallback
** Inputs:  
**  Timer - Pointer to timer structure
**  Fn    - Function pointer to callback function
**  Data  - Pointer to data for callback function
**  Arg1  - Argument1 of callback function
** Outputs: 
**  None
** ReturnValue: 
**  None
** Description: 
**  Initialization of timer structure for callback function
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1577InitTimerCallback (DrvTimerNode_t *Timer, 
                          DrvCallBack_t Fn, 
                          void *Data, 
                          void *Arg1)
{
   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("Rfc1577InitTimerCallBack : Inside function \n"));
   Timer->CbfnInfo.Cbfn = Fn;
   Timer->CbfnInfo.Data = Data;
   Timer->CbfnInfo.Arg1 = Arg1;
   Timer->Next = NULL;
   return;
} 

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: _Rfc1577SetTimer
** Inputs:  
**  Rfc1577Context  - Context
**  Timer           - Pointer to timer structure
**  Tick            - Number of seconds until timer expires
** Outputs: 
**  None
** ReturnValue: 
**  None
** Description: 
**  Starts a timer (ie inserts the timer node at the
**  proper place in the list), if it is already not running.
** Note: 
**  It is a one shot timer, so it starts only when it is already
**  not running. If you want to start it again when it is already running,
**  then you should call Rfc1577CancelTimer first and the call this function.
*/
////////////////////////////////////////////////////////////////////////////////
void _Rfc1577SetTimer(struct Rfc1577Context *pRfc1577Context, 
                 DrvTimerNode_t *Timer, 
                 unsigned int Tick)
{
DrvTimerNode_t   *LocalTimer = NULL, *PrevLocalTimer = NULL;
unsigned int Ticks = (Tick/TIMER_PERIODIC_TICKS);
DrvTimerList_t *DrvTimerList = NULL;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("_Rfc1577SetTimer: Inside Function \n"));

   /*********** Lock Here ******************/
   Api1577AcquireSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);

   DrvTimerList = &(pRfc1577Context->DrvTimerList);
   Timer->ExpiryCounter = DrvTimerList->CurrentTicks + Ticks;
   Timer->Next = NULL;
   
   if(DrvTimerList->DrvTimerLeastTimer == NULL)
   { 
       /* There is no node in the timer list */
       DrvTimerList->DrvTimerLeastTimer = Timer;
       Timer->Next = NULL;     
   }
   else 
   {
       PrevLocalTimer = NULL;
       LocalTimer = DrvTimerList->DrvTimerLeastTimer;
       while (Timer->ExpiryCounter >= LocalTimer->ExpiryCounter) 
       {
           PrevLocalTimer = LocalTimer;
           LocalTimer = LocalTimer->Next;
           if(LocalTimer == NULL)
               break;
       }
       if(LocalTimer == DrvTimerList->DrvTimerLeastTimer)
           DrvTimerList->DrvTimerLeastTimer = Timer;
       else 
       {
           PrevLocalTimer->Next = Timer;
       }    
       Timer->Next = LocalTimer;
   }

   /*************** UnLock Here **************/
   Api1577ReleaseSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   return;
}

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: _Rfc1577CancelTimer
** Inputs:  
**  pRfc1577Context -   context
**  Timer           - Pointer to timer structure
** Outputs: 
**  none
** ReturnValue: 
**  none
** Description: 
**  Cancels a timer (ie remove the timer event from list).
*/
////////////////////////////////////////////////////////////////////////////////
void _Rfc1577CancelTimer(struct Rfc1577Context *pRfc1577Context, 
                        DrvTimerNode_t *Timer)
{   
DrvTimerNode_t *LocalTimer = NULL, *PrevLocalTimer = NULL;
DrvTimerList_t *DrvTimerList= NULL;

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("_Rfc1577CancelTimer: Inside Function\n"));

   /*********** Lock Here ******************/
   Api1577AcquireSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   DrvTimerList = &(pRfc1577Context->DrvTimerList);
   if(DrvTimerList->DrvTimerLeastTimer != NULL) 
   {
       PrevLocalTimer = NULL;
       LocalTimer = DrvTimerList->DrvTimerLeastTimer;

       while (LocalTimer != Timer)
       {
           PrevLocalTimer = LocalTimer;
           LocalTimer = LocalTimer->Next;
           if(LocalTimer == NULL)
               break;
       }
       if(LocalTimer == NULL) 
       { 
           /* do nothing. it means node was not found */
           ;
       } 
       else if (LocalTimer == DrvTimerList->DrvTimerLeastTimer) 
       {
           DrvTimerList->DrvTimerLeastTimer = LocalTimer->Next;
       } 
       else 
       {
           PrevLocalTimer->Next = LocalTimer->Next;        
       }
   }
   Timer->Next = NULL;

   /*************** UnLock Here **************/
   Api1577ReleaseSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   return;
}

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: Rfc1577ProcessTimeout
** Inputs: 
**    DevPtr: Pointer to the context
** Outputs: 
**    none
** ReturnValue: 
**    none
** Description: 
**  This routine is called from the USER CODE periodically after
**  every TIMER_PERIODIC_TICKS. This function increment the global current_ticks
**  field and then processes the timeout: ie calls the call functions.
*/
////////////////////////////////////////////////////////////////////////////////
void Rfc1577ProcessTimeout (void *Notused1, void *DevPtr, void* Notused2, void* NotUsed3)
{
struct Rfc1577Context *pRfc1577Context = NULL;
DrvTimerNode_t *DrvTimerNode = NULL;
DrvTimerList_t *DrvTimerList = NULL;
PRFC1577_TIMER pRfc1577Timer = NULL;
RFC1577_HANDLE handle = RFC1577_ERROR;

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("Rfc1577ProcessTimeout: Inside Function %x \n", DevPtr));

   if (!DevPtr) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577ProcessTimeout: Failure: Bad Context\n"));
       return;
   }
   pRfc1577Context = (struct Rfc1577Context *)DevPtr;

   handle = pRfc1577Context->TimerHandle;
   if (handle  >= RFC1577_TIMER_COUNT) {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               ("Rfc1577ProcessTimeout: Failure: Bad Timer Counter handle\n"));
       return;
   }

   pRfc1577Timer = (pRfc1577Context->NdisHandle)->Rfc1577.Rfc1577Timer;

   // If we are in the process of cancelling the timer associated with 
   // this routine, the cancelling party may be waiting for this code
   // to execute. Signal it with the event. 
   NdisSetEvent( &pRfc1577Timer[handle].NdisTimerEvent );

   /*********** Lock Here ******************/
   Api1577AcquireSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);
   DrvTimerList = &(pRfc1577Context->DrvTimerList);
   DrvTimerList->CurrentTicks += TIMER_PERIODIC_TICKS;
   DrvTimerNode = DrvTimerList->DrvTimerLeastTimer;    
   
   /*************** UnLock Here **************/
   Api1577ReleaseSpinLock(pRfc1577Context->NdisHandle, pRfc1577Context->TimerSpinLock);

   while (DrvTimerNode && DrvTimerList->CurrentTicks >= DrvTimerNode->ExpiryCounter)
   {
       /* Cancel the timer */
       _Rfc1577CancelTimer(pRfc1577Context, DrvTimerNode);

       /* Call the call back function */
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                ("Rfc1577ProcessTimeout: Calling the call back function\n"));
       DrvTimerNode->CbfnInfo.Cbfn(DrvTimerNode->CbfnInfo.Data, DrvTimerNode->CbfnInfo.Arg1);

       FREE_MEMORY (DrvTimerNode, sizeof(DrvTimerNode_t));
       /* Prepare for the Next iteration */
       DrvTimerNode = DrvTimerList->DrvTimerLeastTimer;
   }
   
   if ( Api1577SetTimer (pRfc1577Context->NdisHandle, 
                        pRfc1577Context->TimerHandle, 
                        TIMER_PERIODIC_TICKS*1000) == RFC1577_ERROR)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("Rfc1577ProcessTimeout: Rfc1577SetTimer Erro\n"));
   }
   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1577ProcessTimeout: Rfc1577SetTimer Success\n"));
   return;
}

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: AddTimer
** Inputs: 
**  void *pHandle   -   Context,
**  void *pAEntry   -   pointer to the ArpEntry
**  int Timeout     -   Timeout in seconds
**  int (*CallBack)(void *, void*) - Call back function
** Outputs: 
**  void *  -   Pointer to the timerhandle
** ReturnValue: 
** Description: 
*/
////////////////////////////////////////////////////////////////////////////////
void * AddTimer (void *pHandle, 
          void *pAEntry, 
          int TimeOut, 
          int (*CallBack)(void *, void *))
{
DrvTimerNode_t  *DrvTimerNode = NULL;
struct Rfc1577Context *pRfc1577Context = NULL;
    
   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("AddTimer: Inside Function\n"));

   if (!pHandle || !pAEntry || !CallBack) 
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
                ("AddTimer: Bad context or callback function or arp entry \n"));
       return NULL;
   }
   pRfc1577Context = (struct Rfc1577Context *) pHandle;

   ALLOCATE_MEMORY (&DrvTimerNode, sizeof(DrvTimerNode_t));
   if (!DrvTimerNode) 
       return NULL;
   
   Rfc1577InitTimerCallback (DrvTimerNode, CallBack, pHandle, pAEntry);
   _Rfc1577SetTimer(pRfc1577Context, DrvTimerNode, TimeOut); 
   return DrvTimerNode;
}

////////////////////////////////////////////////////////////////////////////////
/* 
** FunctionName: DeleteTimer
** Inputs: 
**  pHandle     -   context
**  pTHandle    -   TimerHandle
** Outputs: 
**  int
** ReturnValue:
**  RFC1577_SUCCESS -   on success
**  RFC1577_FAILURE -   on failure
** Description: 
*/
////////////////////////////////////////////////////////////////////////////////
int DeleteTimer (void *pHandle, void *pTHandle)
{
DrvTimerNode_t  *pDrvTimer = NULL;
struct Rfc1577Context *pRfc1577Context = NULL;

   DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
           ("DeleteTimer: Inside Function \n"));

   if (!pTHandle || !pHandle)
   {
       DBGPRINT( DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
          ("DeleteTimer: Bad timerhandle or context \n"));
       return RFC1577_FAILURE;
   }

   pDrvTimer = (DrvTimerNode_t *)pTHandle;
   pRfc1577Context = (struct Rfc1577Context *) pHandle;

   _Rfc1577CancelTimer(pRfc1577Context, pDrvTimer);
   FREE_MEMORY (pDrvTimer, sizeof(DrvTimerNode_t));
   return  RFC1577_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
unsigned int Rfc1577GetNetIDMask(unsigned int IpAddress )
{
   if( (IpAddress & IP_CLASS_D_PATTERN) == IP_CLASS_D_PATTERN  )
       return IP_NETID_MASK_D;
   
   else if((IpAddress & IP_CLASS_C_PATTERN) == IP_CLASS_C_PATTERN  )
       return IP_NETID_MASK_C;

   else if((IpAddress & IP_CLASS_B_PATTERN) == IP_CLASS_B_PATTERN  )
       return IP_NETID_MASK_B;

   return IP_NETID_MASK_A;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
BOOLEAN Rfc1577Initialize(PATM_USB_INSTANCE Instance)
{  
   ASSERT(Instance);

   COPY_MEMORY ((void *)&Instance->Rfc1577Config.ATMAddress, 
             (void *)ATM_ADDRESS_13_BYTES_STRING, 
             ATM_ADDRESS_13_BYTES_STRING_SIZE);   

   UsbPhyGetMacAddress( (PUSB_PHY)(Instance->UsbPhyHandle), &(Instance->Rfc1577Config.ATMAddress[ATM_ADDRESS_13_BYTES_STRING_SIZE]) );
   Instance->Rfc1577Config.ATMAddress[MAX_ATMADDR_LEN - 1] = 0x0f;

   Instance->Rfc1577Config.ATMAddrType = NSAP_ADDR;
   Instance->Rfc1577Config.Rfc1577Rfc1483Send = Rfc1483Send;
   Instance->Rfc1577Config.Rfc1577Rfc1483ReceiveComplete = Rfc1483ReceiveComplete; 
   Instance->Rfc1577Config.Rfc1577Enable       =  Instance->Rfc1577Enable;
   Instance->Rfc1577Config.DefaultVPI          = Instance->Rfc1483Vpi[0];
   Instance->Rfc1577Config.DefaultVCI          = Instance->Rfc1483Vci[0]; 
   Instance->Rfc1577Config.DefaultEncap        = Instance->Rfc1483Mode;
   Instance->Rfc1577Config.DefaultHandle       = Instance->Rfc1483VcHandle;
   Instance->Rfc1577Config.Rfc1577MTUSize      = RFC1483_MAX_SDU_SIZE;

   return Rfc1577Init (Instance, &Instance->Rfc1577Config );
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
BOOLEAN Rfc1577CancelAllTimers(PATM_USB_INSTANCE Instance)
{
struct Rfc1577Context *pContext = NULL;
   pContext = (struct Rfc1577Context *) Api1577GetContext (Instance);

   if(pContext->Rfc1577Enable)
   {
       Api1577CancelTimer(Instance, pContext->TimerHandle, TIMER_PERIODIC_TICKS*1000*2);
   }
   return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
BOOLEAN Rfc1577SetAllTimers(PATM_USB_INSTANCE Instance )
{
struct Rfc1577Context *pContext = NULL;

   pContext = (struct Rfc1577Context *) Api1577GetContext (Instance);
    
   if(!pContext->Rfc1577Enable)
       return TRUE;

   pContext->TimerHandle = Api1577InitializeTimer (Instance, Rfc1577ProcessTimeout);
   if (pContext->TimerHandle == RFC1577_ERROR)
   {
       DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
               (" Rfc1577SetAllTimers ERROR\n"));
       return FALSE;
   }

   return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
int Rfc1577GetArpEntries(void *pRfc1577Context , PQUERY_ARP_BUFFER pBuffer)
{
struct Rfc1577Context *pContext= NULL;
int IntfIndex, PVCIndex;
struct Intf *pIntf = NULL;
struct ARPEntry *pArp = NULL;
int NoOfEntry = Rfc1577GetNoOfArpEntries ( pRfc1577Context);
Rfc1577Status_t *pStatus; 
int i = 0;

   pBuffer->Count = NoOfEntry;

   DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Rfc1577Status: Inside Function\n"));
   if (!pRfc1577Context || !pBuffer)
   {
       DBGPRINT (DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_ERR,
            ("Rfc1577Status: Failure: Bad Context or  NULL Buffer\n"));
       return RFC1577_FAILURE;
   }

   if (!NoOfEntry)
   {
       return pBuffer->Count;
   }
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   pStatus = &(pBuffer->ArpEntry[i]);

   
   for (IntfIndex = 0; (NoOfEntry > 0) && (IntfIndex < MAX_INTF); IntfIndex++) 
   {
       pIntf = &pContext->IntfList[IntfIndex];
       if (pIntf->State != IF_INVALID)
       {
           for (PVCIndex = 0; (NoOfEntry > 0) && (PVCIndex < MAX_PVC); PVCIndex++) 
           { 
               pArp = &pIntf->ArpTable[PVCIndex];
               switch (pArp->State) 
               {
                   
                   case ARP_INVALID:
                       break;
                   case ARP_DOWN: 
                   case ARP_SENT:
                   case ARP_VALID:
                   case ARP_MAXTRY:
                       pStatus->IntfAddr   = pIntf->IntfAddr;
                       pStatus->VPI        = pArp->VPI;
                       pStatus->VCI        = pArp->VCI;
                       pStatus->TgtIPAddr  = pArp->TgtIPAddr;
                       pStatus->State      = pArp->State;
                       pStatus->NoOfTries  = pArp->NumOfTries;
                       pStatus->TTL        = ARP_TIMEOUT;
                       i++;
                       pStatus = &(pBuffer->ArpEntry[i]);
                       NoOfEntry--;
                       break;
               }
           }
       } 
   }

   return pBuffer->Count;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
void Rfc1577DeleteArpEntry(void *pRfc1577Context , Rfc1577Status_t *pArpBuffer)
{
struct Rfc1577Context *pContext= NULL;
   ASSERT(pRfc1577Context);
   ASSERT(pArpBuffer);
   pContext = (struct Rfc1577Context *)pRfc1577Context;

   Rfc1577DeletePVC (pContext ,
                          pArpBuffer->IntfAddr ,       /* Intf IPAddr  */
                          pArpBuffer->VPI,             /* VPI          */
                          pArpBuffer->VCI);            /* VCI          */
}

#ifdef  DEBUG
/**********************************************************************************
** Temporary code for Debug/testing
**********************************************************************************/
void DbgPrintAllEntry(void *pRfc1577Context)
{
struct Rfc1577Context *pContext = NULL;
struct ARPEntry *pArp = NULL;
int IntfIndex, PVCIndex, nCount;
struct Intf *pIntf = NULL;

   pContext = (struct Rfc1577Context *)pRfc1577Context;
   ASSERT(pContext);
   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("Dumping complete arp Table\n"));
   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
            ("interface\tIndex\tTgIpAddress\tArpState\n"));
   for (IntfIndex = 0; IntfIndex < MAX_INTF; ++IntfIndex) 
   {
       pIntf = &pContext->IntfList[IntfIndex];
       if (pIntf->State != IF_INVALID)
       {
           for (PVCIndex = 0; PVCIndex < MAX_PVC; ++PVCIndex)
           {
               pArp = &(pIntf->ArpTable[PVCIndex]);
               if (pArp->State != ARP_VALID)
               {
                   DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                           ("%d\t%d\t%x", IntfIndex, PVCIndex, pArp->TgtIPAddr));
                   switch (pArp->State)
                   {
                       case ARP_VALID:
                           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                               ("\tARP_VALID\n"));
                           break;

                       case ARP_SENT:
                           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                               ("\tARP_SENT\n"));
                           break;

                       case ARP_MAXTRY:
                           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                               ("\tARP_MAXTRY\n"));
                           break;

                       case ARP_DOWN:
                           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                               ("\tARP_DOWN\n"));
                           break;

                       default:
                           DBGPRINT(DBG_PRINT_MODULE_RFC1577_ALL, DBG_PRINT_LEVEL_TRACE,
                               ("\tSTATE_N/A\n"));
                           break;
                   } 
               }
           }
       }
   }
}
#endif // DEBUG
#endif // defined GSI_LAN
