/*
   @(#)gtiinit.c	1.54.1.43
*/

/*****************************************************************/
/*  TSM         Application Code -                               */
/*  (C) Copyright 1996-2003 by GlobeSpan Inc                     */
/*****************************************************************/

#include "gti.h"
#include "gtiextn.h"

#if ( defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
#include "gti_sym.h"
#endif

#ifdef GTI_PROD_DMT
    extern void GTIEocInit(GTIEOC_INFO *eInfoPtr);
#endif

#ifdef GTI_HOSTVSC
    #include "ClkGenAddrs.h"
    #include "SysIck.h"
#endif

#ifdef GTI_DSPVIKING
void Gti_WriteVikingRegister(GTI_XCVR_INFO* pXcvr, GTI_WORD addr, GTI_DWORD value);
#endif

#if !defined(GTI_DSPQUAD) && !defined(GTI_DSPOCTANE) && !defined(GTI_DSPVIKING)
/******************************************************************************/
/* Procedure: InitXCVR_CS                                   Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure exists just for the compatibility with the old  */
/*              versions of the code. Shoild not be called from anywhere.     */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void InitXCVR_CS
(
    GTI_XCVR_INFO *pXcvrInfo,
    GTI_CS_ADDRESS dsp_a_cs,
    GTI_CS_ADDRESS dsp_b_cs,
    GTI_CS_ADDRESS framer_cs,
    GTI_CS_ADDRESS reset_cs,
#ifdef GTI_PROD_DMT
    GTI_CS_ADDRESS hostdb_cs,
    GTI_WORD  expectedRemote
#else
    GTI_CS_ADDRESS hostdb_cs
#endif
)
{
    GTI_CHIP_INFO chipInfo;
    GTI_CUST_INFO custInfo;

    custInfo.csBaseAddr = dsp_a_cs;
    custInfo.csFrmrAddr = framer_cs;
    custInfo.csReset = reset_cs;

    chipInfo.pCustInfo = &custInfo;

    Gti_BuildXCVR(&chipInfo, pXcvrInfo, 0);

#ifdef GTI_PROD_DMT
    Gti_SetExpectedStandard(pXcvrInfo, (GTI_BYTE)expectedRemote);
#endif

    dsp_b_cs = dsp_b_cs;     /* Kill the warning */
    hostdb_cs = hostdb_cs;   /* Kill the warning */
}
#endif

/******************************************************************************/
/* Procedure: Gti_BuildSystem                               Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the system data base and all    */
/*              global variables related to the system.                       */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_BuildSystem(
    GTI_SYST_INFO *pSystInfo    /* A pointer to the entire system Data Base   */
)
{
    int i, j;
#if (defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR))
    int k;
#endif

    for(j=0, i=0; i < GTI_MAX_CHIPS_PER_SYSTEM; i++)
        pSystInfo->Chip_db[i] = 0;

    (void)j;

#if (defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR))
    for(i=0; i < GTI_MAX_BUSES_PER_SYSTEM; i++)
    {
#if defined(GTI_USE_XDPXFR)
        pSystInfo->Lbus_db[i].dwMaxSize = 0x7ffff;
#endif
        pSystInfo->Lbus_db[i].wTotalChkSum = 0;
        pSystInfo->Lbus_db[i].dwCurrentAdr = 0;
        pSystInfo->Lbus_db[i].wLoadedAnnexBmp = 0;
        pSystInfo->Lbus_db[i].bLbConfiguration = GTI_LB_CONFIG_ANNEX_A;
        pSystInfo->Lbus_db[i].wStatus = GTI_LB_STATUS_IDLE;

        for(k=0; k < GTI_MAX_BIN_XDMA_TABLES; k++)
        {
            pSystInfo->Lbus_db[i].pFwImageDb[k].pTblPtr = 0;
            pSystInfo->Lbus_db[i].pFwImageDb[k].wTblLength = 0;
        }

#ifdef GTI_USE_DMAXFR
        for(k=0; k < GTI_DXLCMDTABLE_ENTRIES; k++)
        {
            for(j=0; j < GTI_MAX_SUPPORTED_XDMA_PAGES; j++)
            {
                pSystInfo->Lbus_db[i].bStartUpTable[j][k] = 0;
            }
        }
#endif
    }
#endif

#ifdef GTI_TRACE
    SASStop_line = GTI_NEXT;
    SASStop_baud = GTI_PAGE_A;
    SASStop_hip = GTI_PAGE_A;
#endif
}

/******************************************************************************/
/* Procedure: Gti_BuildCHIP                                 Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the data base of the built chip */
/*              and set all the appropriate pointers t default values.        */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_BuildCHIP(             /* ******  Parameters  **************/
    GTI_SYST_INFO *pSystInfo,   /* A pointer to the entire system Data Base   */
    GTI_CHIP_INFO *pChipInfo,   /* A pointer to the current chip Data Base    */
    GTI_CUST_INFO *pCustInfo,   /* A pointer to the Custom filled Data Base   */
    int nChipNum                /* Chip sequence number of the whole system   */
)
{
    int i;
    for(i=0; i < GTI_MAX_XCVRS_PER_CHIP; i++)
        pChipInfo->XCVR_db[i] = 0;

    pChipInfo->pSystInfo = pSystInfo;
    pChipInfo->pCustInfo = pCustInfo;
    pSystInfo->Chip_db[nChipNum] = pChipInfo;
#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    pChipInfo->wLbusID = 0;
    pChipInfo->wChipIdOnLbus = 0;
#endif
    pChipInfo->bChipVersion = 0;
    pChipInfo->bAfeVersion = 0;
}

/******************************************************************************/
/* Procedure: Gti_BuildXCVR                                 Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the data base of the built XCVR */
/*              and set all the appropriate pointers t default values.        */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_BuildXCVR(             /* ******  Parameters  **************/
    GTI_CHIP_INFO *pChipInfo,   /* A pointer to the current chip Data Base    */
    GTI_XCVR_INFO *pXcvrInfo,   /* A pointer to the built XCVR Data Base      */
    int nXcvrNum                /* XCVR sequence number of the current chip   */
)
{
    pXcvrInfo->xcvrId = (GTI_WORD)nXcvrNum; /* remember what core ID */

#if defined(GTI_USE_STD_IO_MACROS)
    /* set base address of transceiver */
    switch(nXcvrNum)
    {
    case 0:
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_0_MSK;
        break;
#if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE)
    case 1:
        #if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_1_MSK;
#endif
        #if( GTI_NUM_CORES_PER_XCVR == 2)
                pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_2_MSK;
        #endif
        #if( GTI_NUM_CORES_PER_XCVR == 3)
                pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_3_MSK;
        #endif

        break;
    case 2:
        #if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_2_MSK;
#endif
        #if( GTI_NUM_CORES_PER_XCVR == 2)
                pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_4_MSK;
        #endif
        #if( GTI_NUM_CORES_PER_XCVR == 3)
                pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_6_MSK;
        #endif

        break;
    case 3:
#if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_3_MSK;
#endif
#if( GTI_NUM_CORES_PER_XCVR == 2)
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_6_MSK;
#endif
        break;
#endif  /* GTI_DSPQUAD */
#ifdef GTI_DSPOCTANE
    case 4:
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_4_MSK;
        break;
    case 5:
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_5_MSK;
        break;
    case 6:
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_6_MSK;
        break;
    case 7:
        pXcvrInfo->pXcvrReg = pChipInfo->pChipReg | GTI_CORE_7_MSK;
        break;
#endif /* GTI_DSPOCTANE */
    default:
        /* error */
        break;
    }
#else
    /* COMPATIBILITY MODE - set the chip selects */
    switch(nXcvrNum)
    {
    case 0:
        pXcvrInfo->dsp_B_cs = GTI_CORE_0_MSK;
        break;
#if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE) || defined(GTI_DSPFRAMER_C1)

    case 1:
        #if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->dsp_B_cs = GTI_CORE_1_MSK;
#endif
        #if( GTI_NUM_CORES_PER_XCVR == 2)
                pXcvrInfo->dsp_B_cs = GTI_CORE_2_MSK;
        #endif
        #if( GTI_NUM_CORES_PER_XCVR == 3)
                pXcvrInfo->dsp_B_cs = GTI_CORE_3_MSK;
        #endif
        break;
#if !defined(GTI_DSPFRAMER_C1)
    case 2:
        #if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->dsp_B_cs = GTI_CORE_2_MSK;
        #endif
        #if( GTI_NUM_CORES_PER_XCVR == 2)
                pXcvrInfo->dsp_B_cs = GTI_CORE_4_MSK;
        #endif
        #if( GTI_NUM_CORES_PER_XCVR == 3)
                pXcvrInfo->dsp_B_cs = GTI_CORE_6_MSK;
#endif
        break;
    case 3:
        #if( GTI_NUM_CORES_PER_XCVR == 1)
        pXcvrInfo->dsp_B_cs = GTI_CORE_3_MSK;
#endif
        #if( GTI_NUM_CORES_PER_XCVR == 2)
                pXcvrInfo->dsp_B_cs = GTI_CORE_6_MSK;
#endif
        break;
#endif /* !DSPFRAMER_C1 */

#ifdef GTI_DSPOCTANE
    case 4:
        pXcvrInfo->dsp_B_cs = GTI_CORE_4_MSK;
        break;
    case 5:
        pXcvrInfo->dsp_B_cs = GTI_CORE_5_MSK;
        break;
    case 6:
        pXcvrInfo->dsp_B_cs = GTI_CORE_6_MSK;
        break;
    case 7:
        pXcvrInfo->dsp_B_cs = GTI_CORE_7_MSK;
        break;
#endif /* GTI_DSPOCTANE */
#endif /* (GTI_DSPQUAD) || (GTI_DSPOCTANE) || (GTI_DSPFRAMER_C1) */
    default:
        /* error */
        break;
    }

    pXcvrInfo->dsp_A_cs = pChipInfo->pCustInfo->csBaseAddr;

#endif /* else - defined(GTI_USE_STD_MACROS) */

    pXcvrInfo->dsp_A_cs = pChipInfo->pCustInfo->csBaseAddr;
    pXcvrInfo->pChipInfo = pChipInfo;


    /* Don't write past end of array */
    if ( nXcvrNum < GTI_MAX_XCVRS_PER_CHIP )
    {
        pChipInfo->XCVR_db[nXcvrNum] = pXcvrInfo;
    }

#ifdef GTI_USE_DMAXFR
    pXcvrInfo->paramsBA = (GTI_LBUS_RW_START_ADDR >> 0x08);
#elif !defined(GTI_USE_XDPXFR)
    pXcvrInfo->CmdInfo.ReturnLabel = GTI_EMPTY;
    pXcvrInfo->CmdInfo.enableTRACE = GTI_TRUE;
    pXcvrInfo->CmdInfo.state = GTI_COMPLETE_PASS;
    pXcvrInfo->CmdInfo.table_pnt = GTI_EMPTY;
    pXcvrInfo->CmdInfo.ProjectIDPnt = GTI_PAGE_NONE;
#ifdef GTI_HOST_SASSTATUS
    pXcvrInfo->StatusInfo.ReturnLabel = GTI_EMPTY;
    pXcvrInfo->StatusInfo.enableTRACE = GTI_FALSE;
    pXcvrInfo->StatusInfo.state = GTI_COMPLETE_PASS;
    pXcvrInfo->StatusInfo.table_pnt = GTI_EMPTY;
    pXcvrInfo->StatusInfo.ProjectIDPnt = GTI_PAGE_NONE;
    Gti_InitPointers(&pXcvrInfo->StatusInfo, GTI_PAGE_NONE);
#endif /* STATUS uses Host SAS machine */
#endif /* Host Mode */

     /* Init State Machine Structure */
    pXcvrInfo->cmd_start = GTI_EOT;
    pXcvrInfo->which_ipu = GTI_DSP_A;

#ifdef GTI_PROD_DMT
    pXcvrInfo->bExpStandard = GTI_INIT_EXP_STANDARD_MASK | 0;
#ifdef GTI_DSPQUAD
    pXcvrInfo->bAfeCalibration = GTI_INIT_AFE_CALIBRATION_MASK | 0;    
#endif
    pXcvrInfo->wAnnexSlctd  = GTI_TYPE_ANNEX_A;
#endif

    pXcvrInfo->ExecFlags.active = GTI_FALSE;
    pXcvrInfo->IrEnabled = GTI_FALSE;
    pXcvrInfo->framer_cs = pChipInfo->pCustInfo->csFrmrAddr;

#ifdef GTI_DEBUG
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    pXcvrInfo->WhichMEM = FLASH_MEM;
#endif
#endif

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    /* Init pointers to ROM table    */
    /* MUST Match table is p3debug.c */
#ifndef GTI_MULTI_PROJ
    Gti_InitPointers(&pXcvrInfo->CmdInfo, GTI_PAGE_A);
#endif

#endif /* Not USE_DMAXFR */
}

#if defined(GTI_USE_XDPXFR)
/******************************************************************************/
/* Procedure: Gti_SetLocalBusSize                           Created: 13.02.01 */
/* ------------------------                                                   */
/* Description: The procedure will assign the XDMA local bus size to a        */
/*              specified local bus number.                                   */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
int Gti_SetLocalBusSize(
    GTI_SYST_INFO *pSysInfo,    /* A pointer to the global system             */
    int nLbusNum,               /* A local bus number                         */
    GTI_DWORD dwLbusSize         /* A size in 16 bits words                   */
)
{
    if(nLbusNum < GTI_MAX_BUSES_PER_SYSTEM)
    {
        pSysInfo->Lbus_db[nLbusNum].dwMaxSize = dwLbusSize;
        return GTI_TRUE;
    }
    else
        return GTI_FALSE;
}
#endif /* GTI_USE_XDPXFR */

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
/******************************************************************************/
/* Procedure: Gti_SetLocalBusNumber                         Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will assign the XDMA local bus number to the    */
/*              current chip.                                                 */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_SetLocalBusNumber(
    GTI_CHIP_INFO *pChipInfo,   /* A pointer to the current chip Data Base    */
    GTI_WORD wLocalBusNumber    /* A local bus number the chip is connected   */
)
{
    pChipInfo->wLbusID = wLocalBusNumber;
}

/******************************************************************************/
/* Procedure: Gti_SetLbusFwImage                            Created: 09.11.00 */
/* ------------------------                                                   */
/* Description: The procedure will assign the XDMA local bus annex to the con */
/*              figuration of the local bus.                                  */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
GTI_WORD Gti_SetLbusFwImage(
    GTI_SYST_INFO *pSystInfo,                        /* Pointer to the system */
    int nLbusNum,                                    /* Local Bus number      */
    GTI_DSP_TBL_ELEMENT GTI_ADDRESS_QUAL *pTable,    /* Pointer to the table  */
    GTI_WORD wElements                               /* Length of the buffer  */
)
{
    int i;
    if(pSystInfo->Lbus_db[nLbusNum].dwCurrentAdr != 0)
    {
        pSystInfo->Lbus_db[nLbusNum].wStatus = GTI_LB_STATUS_LOAD_INITIATED;
        return GTI_FAIL;
    }

    for(i=0; i < GTI_MAX_BIN_XDMA_TABLES; i++)
    {
        pSystInfo->Lbus_db[nLbusNum].pFwImageDb[i].pTblPtr = 0;
        pSystInfo->Lbus_db[nLbusNum].pFwImageDb[i].wTblLength = 0;
    }

    for(i=0; i < wElements; i++)
    {
        pSystInfo->Lbus_db[nLbusNum].pFwImageDb[i].pTblPtr = pTable[i].pTblPtr;
        pSystInfo->Lbus_db[nLbusNum].pFwImageDb[i].wTblLength = pTable[i].wTblLength;
    }

    return GTI_OK;
}

/******************************************************************************/
/* Procedure: Gti_SetLocalBusAnnexType                      Created: 09.11.00 */
/* ------------------------                                                   */
/* Description: The procedure will assign the XDMA local bus annex to the con */
/*              figuration of the local bus.                                  */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_SetLocalBusAnnexType(
    GTI_SYST_INFO *pSystInfo,   /* A pointer to the global system             */
    int nLbusNum,               /* A local bus number the chip is connected   */
    int nAnnex                  /* An ANNEX type or a type bit map            */
)
{
    pSystInfo->Lbus_db[nLbusNum].bLbConfiguration = (GTI_BYTE)nAnnex;
}

#endif

/******************************************************************************/
/* Procedure: Gti_SetDspProjectType                         Created: 09.11.01 */
/* ------------------------                                                   */
/* Description: The procedure will assign the the project DSP type to a XCVR  */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_SetDspProjectType(
    GTI_XCVR_INFO *pXcvrInfo,   /* A pointer to the current XCVR Data Base    */
    GTI_BYTE bProjectType     /* Ann expected remote standard               */
)
{
    pXcvrInfo->bActiveProjectId = bProjectType;
}

#ifdef GTI_PROD_DMT
/******************************************************************************/
/* Procedure: Gti_SetExpectedStandard                       Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will set the expected remote standard to the    */
/*              current XCVR                                                  */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_SetExpectedStandard(
    GTI_XCVR_INFO *pXcvrInfo,   /* A pointer to the current XCVR Data Base    */
    GTI_BYTE expectedRemote     /* Ann expected remote standard               */
)
{
    switch(expectedRemote)
    {
       case GTI_ALCTL14_STANDARD:
       case GTI_ALCTL_STANDARD:
       case GTI_ADI_STANDARD:
          pXcvrInfo->bExpStandard = (GTI_BYTE)expectedRemote;
          break;
       default:
          pXcvrInfo->bExpStandard = 0;
          break;
    }

    /* Set to non initialized */
    pXcvrInfo->bExpStandard |= GTI_INIT_EXP_STANDARD_MASK;
}
#endif

/******************************************************************************/
/* Procedure: Gti_InitSystem                                Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialze the entire system. All the       */
/*              entities of the system are initialized automatically.         */
/* Returns result  of initialization                                          */
/* -------------------------------------------------------------------------- */
int Gti_InitSystem(
    GTI_SYST_INFO *pSystInfo    /* A pointer to the entire system Data Base   */
)
{
    int i;

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    int nLbus;
    GTI_WORD nBusMap;
#endif

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    int j;
#endif

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    for(nLbus=0; nLbus < GTI_MAX_BUSES_PER_SYSTEM; nLbus++)
    {
        nBusMap = 0;
        for(i = 0; i < GTI_MAX_CHIPS_PER_SYSTEM; i++)
        {
             if(nLbus == pSystInfo->Chip_db[i]->wLbusID)
                pSystInfo->Chip_db[i]->wChipIdOnLbus = nBusMap++;
        }
    }
#endif

    for(i = 0; i < GTI_MAX_CHIPS_PER_SYSTEM; i++)
    {
        Gti_InitCHIP(pSystInfo->Chip_db[i]);
    }

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)

    /* load a chip only once through the first LBUS */
    for(j = 0, i = 0; i < GTI_MAX_CHIPS_PER_SYSTEM; i++)
    {
        if(j == pSystInfo->Chip_db[i]->wLbusID)
        {
            if(Gti_InitLBUS(pSystInfo, j++) == GTI_FALSE)
            {
                return GTI_FALSE;
            }
        }
    }
#endif


    return GTI_TRUE;
}

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
/******************************************************************************/
/* Procedure: Gti_InitLBUS                                  Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialze the Local Bus of the DSP  system */
/*              according to the Local Bus number.                            */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
int Gti_InitLBUS(GTI_SYST_INFO *pSystInfo, int nLbusNum)
{
#if defined(GTI_USE_DMAXFR)
    int i, j;
    GTI_XCVR_INFO *pXcvr;
#endif

#if defined(GTI_USE_DMAXFR)

    pXcvr = 0;

    for(i = 0; i < GTI_MAX_CHIPS_PER_SYSTEM; i++)
    {
        if(pSystInfo->Chip_db[i]->wLbusID == nLbusNum)
        {
            for(j=0; j < GTI_MAX_XCVRS_PER_CHIP; j++)
            {
                pXcvr = pSystInfo->Chip_db[i]->XCVR_db[j];
                /* stop all the XCVRs LBUS actibities of the chip */
                Gti_ResetDMAEngine(pXcvr);
            }
        }
    }

#endif

#if !defined(GTI_NO_LBUS_INITIAL_LOAD) && !defined(GTI_USE_BINARY_DSP_OBJECT)
    return Gti_LocalBusLoad(pSystInfo, nLbusNum);
#else
#if defined(GTI_USE_DMAXFR)
    return GTI_TRUE;

#endif
#endif
}
#endif

/******************************************************************************/
/* Procedure: Gti_InitCHIP                                  Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialze the single chip and perform a    */
/*              hardware reset of the device.                                 */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_InitCHIP(GTI_CHIP_INFO *pChipInfo)
{
    GTI_WORD i;

#if defined(GTI_DSPQUAD) || defined(GTI_DSPOCTANE) 
    GTI_XCVR_INFO *pXcvrInfo;
    pXcvrInfo = pChipInfo->XCVR_db[0];
#endif

#if (defined(GTI_DSPOCTANE))

    WRITE_DSP_REG(pXcvrInfo, (GTI_CORE_RESETS | GTI_CORE_TOP_MSK), 0xaa);
    WRITE_DSP_REG(pXcvrInfo, (GTI_CORE_RESETS | GTI_CORE_TOP_MSK), 0x00);

#if !defined(GTI_USE_XDPXFR)
    Gti_SetVCO(pXcvrInfo, GTI_TRUE);
#endif

    WRITE_DSP_REG(pXcvrInfo, GTI_IO_CONFIG_BR, 0x0C);

    CONFIGURE_DSP(pXcvrInfo, GTI_DSP_TOP,  GTI_NO_AI);

    /* Enable Local Bus */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xbd, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x2b, 0x04);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xbf, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x08, 0x02);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xbe, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x01, 0xff);

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    /* update chip id on the LBUS */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x9d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0,  pChipInfo->wChipIdOnLbus);
#endif
#if ( !defined(GTI_USE_XDPXFR) && defined(GTI_PROD_DMT) )
    /* set PIOP0 output muxes from TCORE */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x98, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x44, 0x44);

    /* set PIOP0 output muxes from TCORE */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x99, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x44, 0x44);

    /* set PIOP1 output muxes from TCORE */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x9a, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x44, 0x44);

    /*set PIOP1 output muxes from TCORE */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x9b, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x44, 0x44);

    /*set utopia tx address for core0 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x00, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x00, 0x01);

    /*set utopia tx address for core1 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x01, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x02, 0x03);

    /*set utopia tx address for core2 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x02, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x04, 0x05);

    /*set utopia tx address for core3 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x03, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x06, 0x07);

    /*set utopia tx address for core4 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x04, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x08, 0x09);

    /*set utopia tx address for core5 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x05, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0a, 0x0b);

    /*set utopia tx address for core6 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x06, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0c, 0x0d);

    /*set utopia tx address for core7 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x07, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0e, 0x0f);

    /*tx cell length 53 byte, clav length 48 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x08, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x02, 0x34); /*0x2f changed -ywu */

    /*set utopia tx enabled, 8 bit, 5bit add enabled  */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0x09, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x1f, 0x02);

    /*set all tx channels active */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xBA, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0xff, 0xff);

    /*set utopia rx address for core0 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB0, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x00, 0x01);

    /*set utopia rx address for core1 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB1, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x02, 0x03);

    /*set utopia rx address for core2 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB2, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x04, 0x05);

    /*set utopia rx address for core3 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB3, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x06, 0x07);

    /*set utopia rx address for core4 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB4, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x08, 0x09);

    /*set utopia rx address for core5 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB5, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0a, 0x0b);

    /*set utopia rx address for core6 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB6, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0c, 0x0d);

    /*set utopia rx address for core7 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB7, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x0e, 0x0f);

    /*rx cell length 53 byte, clav length 53 */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB8, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x34, 0x34);

    /*set utopia rx enabled, 8 bit, 5bit add enabled  */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xB9, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0x1f, 0x02);

    /*set all rx channels active */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_TOP, 0xBB, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_TOP, 0xff, 0xff);

#endif

    UPDATE_DSP(pXcvrInfo);

#endif /* Octane */

#if defined(GTI_DSPQUAD)
    /* read the chip and AFE version */
    pChipInfo->bChipVersion = Gti_GetChipVersion(pXcvrInfo);
    
    if(pChipInfo->bChipVersion == 0x4)
       Gti_SetTopVCO(pXcvrInfo);

    /* Mask interrupts */
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_73, 0xff);
#endif

    for(i=0; i < GTI_MAX_XCVRS_PER_CHIP; i++)
    {
        Gti_InitXCVR(pChipInfo->XCVR_db[i]);
    }

#if defined(GTI_DSPQUAD)
    pXcvrInfo = pChipInfo->XCVR_db[0];
    /* Init the Global Register area */
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_75, 0xff);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_75, 0xfe);

#ifdef GTI_UTOPIA_ODD_ADDRESS_ENABLE
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_79, 0xaa);
#else
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_79, 0x00);
#endif

    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_70, 0x00);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_7A, 0x02);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_7B, 0x00);

    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_70, 0x0c);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_71, 0x00);

    /* Unmask interrupts */
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_73, 0x00);

    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_76, 0x30);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_77, 0x00);

    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_70, GTI_INT_POLARITY);

    WRITE_DSP_REG(pXcvrInfo, GTI_CORE_0123_MSK|0, 0);
    WRITE_DSP_REG(pXcvrInfo, GTI_CORE_0123_MSK|1, 0);
    UPDATE_DSP(pXcvrInfo);

    pChipInfo->bAfeVersion = Gti_GetAfeVersion(pXcvrInfo);

#ifdef GTI_USE_DMAXFR
    if(pChipInfo->bChipVersion >= 0x03)
    {
        /* Enable internal arbitration for every first chip on LB */
        if((pChipInfo->wChipIdOnLbus == 0) && (pChipInfo->XCVR_db[0]->xcvrId == 0))
            WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_7D, 0xC0);
    }
#endif
#endif
}

/******************************************************************************/
/* Procedure: Gti_InitXCVR                                  Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the system data base and all    */
/*              DSP, XCVR State Machine, Host RAM                             */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_InitXCVR(GTI_XCVR_INFO *pXcvrInfo)
{
    /* All Initialization of DSP is done here       */
    /* All Initialization of STATE MACHINE is done here */
#ifdef GTI_DSPQUAD
    GTI_BYTE bResetCore, bIntMaskImage;
#endif
#ifdef GTI_USE_XDPXFR
    GTI_BYTE bMsgArray[4];
#endif

#if (defined(GTI_TRACE) && defined(GTI_HMS))
    Gti_UpdateTrace( GTI_API_INITXCVR, pXcvrInfo->xcvrId);
#endif

#if defined(GTI_DSPQUAD)

#ifdef GTI_USE_DMAXFR
    Gti_StopDspActivity(pXcvrInfo);
#endif

    switch(pXcvrInfo->dsp_B_cs)
    {
    case GTI_CORE_0_MSK:
#ifdef GTI_DUAL_CORE
        bResetCore = 0x33;
#else
        bResetCore = 0x11;
#endif
        break;
    case GTI_CORE_1_MSK:
        bResetCore = 0x22;
        break;
    case GTI_CORE_2_MSK:
#ifdef GTI_DUAL_CORE
        bResetCore = 0xcc;
#else
        bResetCore = 0x44;
#endif
        break;
    case GTI_CORE_3_MSK:
        bResetCore = 0x88;
        break;
    default:
        return;
    }

    bIntMaskImage = READ_DSP_REG(pXcvrInfo, GTI_GLOBAL_73);

    /* Mask the interrupt */
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_73, bIntMaskImage | bResetCore);

    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_72, bResetCore);
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_72, (bResetCore & 0xf0));

    /* Turns on the line driver */
    CONFIGURE_DSP(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x40, 0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x7, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x28);

    /* Reset the XDMA  */
    WRITE_DSP_REG(pXcvrInfo, (GTI_GCR_DSP | pXcvrInfo->dsp_B_cs), 0x01);
    WRITE_DSP_REG(pXcvrInfo, (GTI_GCR_DSP | pXcvrInfo->dsp_B_cs), 0x6c);
    WRITE_DSP_REG(pXcvrInfo, (GTI_DMA_SUSPEND_DSP | pXcvrInfo->dsp_B_cs), 0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0xa0, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x8d, 0x05);

#elif defined(GTI_DSPOCTANE)
#ifdef GTI_USE_XDPXFR
    if(pXcvrInfo->pChipInfo->pSystInfo->Lbus_db[pXcvrInfo->pChipInfo->wLbusID].wLoadedAnnexBmp != 0)
    {
        if(Gti_BuildDspMsg(pXcvrInfo, 0, GTI_DSP_MSG_INIT_CORE, sizeof(bMsgArray), bMsgArray))
        {
            (void)Gti_SendDspMsg(pXcvrInfo, (GTI_WORD)(Gti_GetActiveProjectId(pXcvrInfo)));
        }
    }
    else
#endif /* if GTI_USE_XDPXFR */
    {
        WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_ALL, GTI_CORE_RESETS, 0xaa);
        WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_ALL, GTI_CORE_RESETS, 0x00);
    }
#endif /* if GTI_DSPOCTANE */

#ifdef GTI_DSPFRAMER_B1
    /* Power down of ILD2               */
    /* Need to turn on interface clocks */
    /* Told to remove                   */
#endif

    /* Do not initialize anything here if XDP. We've already sent a message */

#ifndef GTI_USE_XDPXFR
    Gti_InitDSP(pXcvrInfo);      /* Init DSP   */
    Gti_InitHRAM(pXcvrInfo);     /* Init Host RAM Block */
    Gti_InitProductHRAM(pXcvrInfo);  /* Product specific Init */
#endif /* GTI_USE_XDPFR */

#ifdef GTI_ESC
    /* zero out the counter images */
    pXcvrInfo->PM.adslAtucChanReceivedBlks=0;
    pXcvrInfo->PM.adslAtucChanTransmittedBlks=0;
    pXcvrInfo->PM.adslAtucChanCorrectedBlksI=0;
    pXcvrInfo->PM.adslAtucChanCorrectedBlksF=0;
    pXcvrInfo->PM.adslAtucChanUncorrectBlksI=0;
    pXcvrInfo->PM.adslAtucChanUncorrectBlksF=0;
    pXcvrInfo->PM.adslAturChanReceivedBlks=0;
    pXcvrInfo->PM.adslAturChanTransmittedBlks=0;
    pXcvrInfo->PM.adslAturChanCorrectedBlksI=0;
    pXcvrInfo->PM.adslAturChanCorrectedBlksF=0;
    pXcvrInfo->PM.adslAturChanUncorrectBlksI=0;
    pXcvrInfo->PM.adslAturChanUncorrectBlksF=0;
    pXcvrInfo->PM.adslAtucPerfStatFastR=0;
    pXcvrInfo->PM.adslAtucPerfStatFailedFastR=0;
    pXcvrInfo->PM.adslAtucPerfStatSesL=0;
    pXcvrInfo->PM.adslAtucPerfStatUasL=0;
    pXcvrInfo->PM.adslAturPerfStatSesL=0;
    pXcvrInfo->PM.adslAturPerfStatUasL=0;
    pXcvrInfo->PM.adslAtucPerfStatErrS=0;
    pXcvrInfo->PM.adslAturPerfStatErrS=0;
#endif

#ifdef GTI_PROD_DMT
    /* Initialize the EOC Module */
    GTIEocInit( &(pXcvrInfo->eocInfo) );
#endif

    /* Init State Machine */
    pXcvrInfo->cmd_start = GTI_EOT;
    pXcvrInfo->which_ipu = GTI_DSP_A;

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    pXcvrInfo->CmdInfo.ReturnLabel = GTI_EMPTY;
    pXcvrInfo->CmdInfo.state = GTI_COMPLETE_PASS;
    pXcvrInfo->CmdInfo.table_pnt = GTI_EMPTY;
    pXcvrInfo->CmdInfo.ProjectIDPnt = GTI_PAGE_NONE;

#ifdef GTI_GTI_HOST_SASSTATUS
    pXcvrInfo->StatusInfo.ReturnLabel = GTI_EMPTY;
    pXcvrInfo->StatusInfo.state = GTI_COMPLETE_PASS;
    pXcvrInfo->StatusInfo.table_pnt = GTI_EMPTY;
    pXcvrInfo->StatusInfo.ProjectIDPnt = GTI_PAGE_NONE;
    Gti_InitPointers(&pXcvrInfo->StatusInfo, GTI_PAGE_NONE);
#endif


    /* Init pointers to ROM table    */
    /* MUST Match table is p3debug.c */
#ifndef GTI_MULTI_PROJ
    Gti_InitPointers(&pXcvrInfo->CmdInfo,GTI_PAGE_A);
    pXcvrInfo->StatusInfo.ProjectIDPnt = GTI_PAGE_A;
    pXcvrInfo->CmdInfo.ProjectIDPnt = GTI_PAGE_A;
#endif
#endif /* Host Mode */

#ifdef GTI_TRACE
    SASStop_line = GTI_NEXT;
    SASStop_baud = GTI_PAGE_A;
    SASStop_hip = GTI_PAGE_A;
#endif

    pXcvrInfo->ExecFlags.active = GTI_FALSE;
    pXcvrInfo->IrEnabled = GTI_FALSE;
    pXcvrInfo->CurrentMode = 0xff;
    pXcvrInfo->bActiveProjectId = 0xff;

#ifdef GTI_DSPQUAD
    /* Restore the interrupt mask */
    WRITE_DSP_REG(pXcvrInfo, GTI_GLOBAL_73, bIntMaskImage);
#endif

#if ( defined(GTI_INSTALL_ANNEX_CO) )
    (void)GtiSetSearchStateMachine(pXcvrInfo, GTI_INSTALL_INIT_SET_STATE, (struct PARAM_XCVR_ARRAY *)NULL );
#endif /* Support Annex Fallback */

#ifdef GTI_AC_CPE
        GtiACInit(pXcvrInfo, GTI_AC_INIT);
#endif



    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_InitISRAM                                 Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the ISRAM memory to the         */
/*              default values.                                               */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_InitISRAM(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_BYTE IsRamImg[512];
    GTI_WORD i;

    for(i=0; i < sizeof(IsRamImg); i++)
        IsRamImg[i] = 0;

    CONFIGURE_DSP( pXcvrInfo, GTI_DSP_A, GTI_NO_AI);

#if( defined( GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
    /* Init PSRAM location for all HMS products */
    WRITE_DSP_ADDR(pXcvrInfo,GTI_DSP_A, GTI_HOST_INIT_BANK_A_LOC, GTI_PSRAM0W);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x00, 0x00);

    WRITE_DSP_ADDR(pXcvrInfo,GTI_DSP_A, (GTI_HOST_INIT_BANK_B_LOC & 0xff), GTI_PSRAM1W);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x00, 0x00);

    /* Init framer sync flag to out-of-sync */
#if (defined(GTI_PROD_HDSL2) || defined(GTI_PROD_MSDSL))
    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_rcvFSWsync_LOC, GTI_ISRAM);

#elif defined (GTI_PROD_IDSL)
    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x88, GTI_PSRAM0W);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x88, GTI_PSRAM1W);
#endif
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x00);
#endif

#ifdef GTI_USE_DMAXFR
    IsRamImg[GTI_LocalMemBAPR * 2] = LOBYTE(pXcvrInfo->paramsBA);
    IsRamImg[(GTI_LocalMemBAPR * 2) + 1] = HIBYTE(pXcvrInfo->paramsBA);
#endif

    CONFIGURE_DSP( pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_DMA);
    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, GTI_ISRAM);
    WRITE_DSP_BLOCK(pXcvrInfo,GTI_ALL_XCVR_CORES, GTI_DATAL_DSP, 512, IsRamImg);
}

/******************************************************************************/
/* Procedure: Gti_InitHRAM                                  Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the HRAM memory to the          */
/*              default values.                                               */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_InitHRAM(GTI_XCVR_INFO *pXcvrInfo)
{
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    GTI_WORD GTI_LOCAL_QUAL i;
#endif

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

    for(i=0; i < (GTI_RAMSize+GTI_RAM_BUFFER); i++)
    {
        SET_HOST_DB_B(pXcvrInfo, i, 0);
    }

#endif /* ! Use DMAXFR */

    pXcvrInfo->IrEnabled = GTI_FALSE;

    /* The following is not really neccessary since  */
    /* GTI_DSP_A == 0.  But in case that changes     */
    /* lets explicatly set CH A status to STARTLET A */
    /* These values if not correct should be changed */
    /* in SAS table per command                      */
#ifdef GTI_PROD_ADSL7MC
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_RS_CHA, GTI_DSP_A);
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_SQ_CHA, GTI_DSP_A);
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_OP_STATE_CHA, GTI_DSP_A);
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_GAIN_CHA, GTI_DSP_A);
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_XMIT_CHA, GTI_DSP_A);
    SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_PROG_CHA, GTI_DSP_A);
#endif     /* ADSL7MC */
}

/******************************************************************************/
/* Procedure: Gti_InitDSP                                   Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the ISRAM memory to the         */
/*              default values.                                               */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void
Gti_InitDSP(GTI_XCVR_INFO *pXcvrInfo)
{
     /*****************************************************/
     /*    Init Bus Mode                                  */
     /*****************************************************/
#ifdef GTI_DSPSTAR
#ifdef NMPX
    /* First Write in Non-Multi Mode must be 0x11 to Reg 1 */
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES ,GTI_CONFIG_DSP, 0x11);
#endif
#endif

     /*****************************************************/
     /*    Host DSP Registers                             */
     /*****************************************************/
#ifndef GTI_DSPSTAR
    /* Init Host Interface Registers */
    /* Disable PCU interrupts */

#if (defined(GTI_DSPOCTANE) )
    CONFIGURE_DSP(pXcvrInfo, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_INT_MASK, 0xff);
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_INT_CLEAR_DSP, 0xff);
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_INT_CLEAR_HOST, 0xff);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x07, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x0f);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x07, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x05);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x60);


#if defined(GTI_HMS)

    CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A,  GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x2c, GTI_IREGA);
    WRITE_DSP_WORD( pXcvrInfo, GTI_DSP_A, 0x00, 0x01);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x2a, GTI_IREGA);
    WRITE_DSP_WORD( pXcvrInfo, GTI_DSP_A, 0xff, 0x9f);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x3c, GTI_IREGA);
    WRITE_DSP_WORD( pXcvrInfo, GTI_DSP_A, 0x00, 0x0b);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x3d, GTI_IREGA);
    WRITE_DSP_WORD( pXcvrInfo, GTI_DSP_A, 0x52, 0x00);

    /* The following Reg is in "A" part" */
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_DMA,(GTI_BYTE)(GTI_CONFIG_DSP|GTI_CORE_0_MSK),
                    (GTI_NO_AI|GTI_BUS_MODE|GTI_INT_MODE) );

    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_DMA,(GTI_ADDL_DSP|GTI_CORE_0_MSK), 0x3d );
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_DMA,(GTI_ADDH_DSP|GTI_CORE_0_MSK), GTI_IREGA);
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_DMA,(GTI_DATAL_DSP|GTI_CORE_0_MSK), 0x52);
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_DMA,(GTI_DATAH_DSP|GTI_CORE_0_MSK), 0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x80, 0x07);

#endif



#elif (defined(GTI_DSPVIKING))
    Gti_SetVCO(pXcvrInfo, GTI_TRUE);

#elif (defined(GTI_DSPQUAD))

    Gti_SetVCO(pXcvrInfo, GTI_TRUE);

    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_INTERRUP_DSP, 0xff);
    /* Un-select DMA interrupts */
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_DMA_INTERRUP_DSP, 0x00);
    /* Clear PCU DMA interrupts - Only needed in NOVA according to LSI */
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_CLR_DSP, 0xff);
    /* Zero the register after setting it to FF*/
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_CLR_DSP, 0x00);

#else  /* !GTI_DSPQUAD */

#if (defined(GTI_DSPMULTI) || defined(GTI_DSPPCI))
    Gti_SetVCO(pXcvrInfo, GTI_TRUE); /*NOVA Core */
#endif

    WRITE_DSP_REG(pXcvrInfo, GTI_PCU_INTERRUP_DSP, 0xff);
    /* Un-select DMA interrupts */
    WRITE_DSP_REG(pXcvrInfo, GTI_DMA_INTERRUP_DSP, 0x00);
    /* Clear PCU DMA interrupts - Only needed in NOVA according to LSI */
    WRITE_DSP_REG(pXcvrInfo, GTI_PCU_CLR_DSP, 0xff);
    /* Zero the register after setting it to FF*/
    WRITE_DSP_REG(pXcvrInfo, GTI_PCU_CLR_DSP, 0x00);

#endif /* GTI_DSPQUAD */

#endif /* !GTI_DSPSTAR */

    /*****************************************************/
    /*    DSP Interrupt Registers                        */
    /*****************************************************/
    /* Init Interrupt to Un-MASK all */

#if ( defined(GTI_DSPOCTANE) || defined(GTI_DSPMONACO) || defined(GTI_DSPTURBO) )
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_INT_MASKS_DSP, 0xff);
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_INT_MASK, 0xff);
#else
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_INTERRUP_DSP, 0xff);
#endif

    Gti_InitDSPInterrupt(pXcvrInfo, GTI_FALSE);

#if ( defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
    WRITE_DSP_REG(pXcvrInfo,GTI_GLOBAL_INTERRUP_DSP,
        (GTI_BYTE)(GTI_OD_MODE|GTI_DSP_INT_POLARITY));
#endif

#if(defined(GTI_DSPFRAMER) || defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )

    CONFIGURE_DSP( pXcvrInfo, GTI_DSP_A, GTI_NO_AI);

#ifdef GTI_PROD_HDSL2
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_CORE_A, 0x9b, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo,  GTI_DSP_CORE_A, 0x08, 0x40);
#endif

#if( defined(GTI_PROD_SDSLE1) )

#ifdef GTI_DSPFRAMER
    WRITE_DSP_REG(pXcvrInfo,GTI_GCR_DSP,
        (GTI_BYTE)(0x63|GTI_OD_MODE|GTI_DSP_INT_POLARITY));
#endif

    /* Gemini Registers */
    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9b, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x88, 0xc4);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x0f, 0x0c);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9e, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x9f, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x18);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0xa0, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x8f);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0xa7, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x00);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0xa8, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00,0x00);
#endif
#endif /* GTI_PROD_SDSLE1 */

    Gti_InitISRAM(pXcvrInfo); /* Init ISRAM communications Block */
    Gti_InitAFE(pXcvrInfo); /* Init SLADE DA by reseting  */

    /* Init IPU PC= 0 */
    CONFIGURE_DSP( pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x1a, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0xa0); /* Write 0xa000 */

    /* Add Mode Unselect ALL */
    WRITE_DSP_SPECIAL(pXcvrInfo, GTI_ALL_XCVR_CORES, 0x00, 0x00, 0x00);


    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_TurnOnAFE                                 Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the Jupiter                     */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_TurnOnAFE(GTI_XCVR_INFO *pXcvrInfo)
{
    CONFIGURE_DSP(pXcvrInfo, GTI_DSP_ALL, GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x3c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x8d);

#if defined(GTI_JANUS_AFE)
#if defined(GTI_JUPITER_AFE)
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0xde, 0x04);       
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x40, 0x0f);    
    
    DELAY_GTI_PROCESS(240);     
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x08, 0x0b);
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x02, 0x0c); 
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x0d);
    
    DELAY_GTI_PROCESS(240);     

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x84, 0x0e);
    
    DELAY_GTI_PROCESS(240); 
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x05, 0x0a);    
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x23, 0x03);    
    
    DELAY_GTI_PROCESS(240);    
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x02, 0x01); 
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x02); 
    
    DELAY_GTI_PROCESS(240);    
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x20, 0x05);     
    
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x89, 0x06); 
    
    DELAY_GTI_PROCESS(240);
   
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x28, 0x07); 
    
    DELAY_GTI_PROCESS(240);   

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x70, 0x09);
    
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x08);
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x2b, 0x03);    
    
    DELAY_GTI_PROCESS(240);

#else

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x05);       
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x31, 0xb);    
    
    DELAY_GTI_PROCESS(240);     
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0xb1, 0xc);
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x01, 0xa); 
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x02, 0x1);
    
    DELAY_GTI_PROCESS(240);     

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x2);
    
    DELAY_GTI_PROCESS(240); 
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x3);    
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x4);    
    
    DELAY_GTI_PROCESS(240);    
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x02, 0x6); 
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x34, 0x7); 
    
    DELAY_GTI_PROCESS(240);    
        
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x8);     
    
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x02, 0x9); 
    
    DELAY_GTI_PROCESS(240);
   
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0x09); 
    
    DELAY_GTI_PROCESS(240);   

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0xd);
    
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x48, 0xe);
    
    DELAY_GTI_PROCESS(240);    
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x00, 0xf);    
    
    DELAY_GTI_PROCESS(240);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x13, 0xa);    
    
    DELAY_GTI_PROCESS(240);

#endif
#endif       

    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_TurnOffAFE                                Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will shut off the AFE                           */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_TurnOffAFE(GTI_XCVR_INFO *pXcvrInfo)
{
    CONFIGURE_DSP(pXcvrInfo, GTI_DSP_ALL, GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x3c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x8d);

#if defined(GTI_JANUS_AFE)
#if defined(GTI_JUPITER_AFE)
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x40, 0x0f);

    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x40, 0x0a);
    
    DELAY_GTI_PROCESS(240);    

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x10, 0x01);    
    
    DELAY_GTI_PROCESS(240);
 
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x02);    
    
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x03);    
    
    DELAY_GTI_PROCESS(240);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x80, 0x04);    
    
    DELAY_GTI_PROCESS(240);    

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x30, 0x06);    
        
    DELAY_GTI_PROCESS(240);
    
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_ALL, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_ALL, 0x08, 0x08);    
        
    DELAY_GTI_PROCESS(240);    

#else

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x0f, 0x09);

    DELAY_GTI_PROCESS(60);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0xff, 0x09);

    DELAY_GTI_PROCESS(60);

    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, 0x34, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, GTI_DSP_A, 0x81, 0x0b);
#endif
#endif

    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_InitAFE                                   Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will initialize the Slade                       */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void Gti_InitAFE(GTI_XCVR_INFO *pXcvrInfo)
{

    Gti_TurnOffAFE(pXcvrInfo);
    Gti_TurnOnAFE(pXcvrInfo);
  
    UPDATE_DSP(pXcvrInfo);
}

#if defined(GTI_DSPQUAD)
/*****************************************************************************/
/* Name:Gti_SetTopVCO                                                        */
/* Input:                                                                    */
/* pXcvr: System pointer                                                     */
/* nInit:    Initialization                                                  */
/* Set the VCO parameters and switch MCK either to ICK or VCO2 according to  */
/* the second parameter.                                                     */
/*****************************************************************************/
void
Gti_SetTopVCO(GTI_XCVR_INFO* pXcvr)
{

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_60, 0x18);
    DELAY_GTI_PROCESS(10);

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_61, 0xb6);
    DELAY_GTI_PROCESS(10);

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_62, 0x2b);
    DELAY_GTI_PROCESS(10);

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_63, 0x29);
    DELAY_GTI_PROCESS(10);

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_64, 0x4b);
    DELAY_GTI_PROCESS(10);

    WRITE_DSP_REG(pXcvr, GTI_GLOBAL_65, 0x3e);
    DELAY_GTI_PROCESS(10000);

}
#endif /* Quad Only */

/*****************************************************************************/
/* Name:Gti_SetVCO                                                           */
/* Input:                                                                    */
/* pXcvr: System pointer                                                     */
/* nInit:    Initialization                                                  */
/* Set the VCO parameters and switch MCK either to ICK or VCO2 according to  */
/* the second parameter.                                                     */
/*****************************************************************************/
void
Gti_SetVCO(GTI_XCVR_INFO* pXcvr, int nInit)
{

#ifdef GTI_DSPVIKING
    GTI_DWORD   SiliconVer;
#endif

    DELAY_GTI_PROCESS(1);

#if defined(GTI_DSPOCTANE)

    (void)nInit;

#if defined(GTI_HMS)

    CONFIGURE_DSP(pXcvr, GTI_DSP_TOP,  GTI_NO_AI);

            /* Set PLL 1 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x20, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x12, 0xC0);
    /* VCORNG=200-400, VCOSEL=CKI, DDLSEL=VCOCK, MCKSEL=SPCK, SPCKSEL=SPCK/2, SPCKSEL=VCOCK */
    /* DISABLE DDL */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x2d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x00);
    /* DDLSEL1=0 */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x32, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x10, 0x40);
    /* Set DDLDIV2*/

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x21, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x01, 0x00);
    /* will be configured later */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x03, 0x14);
    /* PLL1 N/D = 20/3 */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3b, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x3f, 0x04);
    /* PLL1KHREG for D=3 */

            /* Set PLL 2 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x39, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x60);
    /* SVCOSEL=DDL, SPLLPD=1, SVCORNG=200-400 */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3e, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x0a, 0x18);
    /* PLL2 N/D = 24/10 */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x34, 0x04);
    /* PLL2KHREG for D=2 */

    /* Set Top Clock 2 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x35, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x09, 0x00);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x36, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x08, 0x00);
    /* will be configured later */

    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x9d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x20);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x20, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x12, 0xC0);
    /* DISABLE DDL for NOW */

    /* Set Clock Control - Source of Clocks is Core 0 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x94, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x00);

#else /* HMS */

    CONFIGURE_DSP(pXcvr, GTI_DSP_TOP,  GTI_NO_AI);

    /* Set PLL 1 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x20, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x58, 0x87);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x21, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0xba, 0x2c);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3d, GTI_IREGA);

#if( GTI_NUM_CORES_PER_XCVR == 2 || GTI_NUM_CORES_PER_XCVR == 3) /* Dual Core DMT - VDSL */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x0f, 0x2e);
#else   /* Single Core DMT */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x2d, 0xb8);
#endif /* Single Core DMT */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3b, GTI_IREGA);
#if( GTI_NUM_CORES_PER_XCVR == 2 || GTI_NUM_CORES_PER_XCVR == 3) /* Dual Core DMT - VDSL */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0xf2, 0x04);
#else
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0xf2, 0x0f);
#endif

    /* Set PLL 2 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x39, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x45);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3e, GTI_IREGA);
    #if( GTI_NUM_CORES_PER_XCVR == 2 || GTI_NUM_CORES_PER_XCVR == 3) /* Dual Core DMT - VDSL */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x24, 0x73);
    #else   /* Single Core DMT */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x0a, 0x32);
    #endif /* Single Core DMT */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x3c, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x72, 0x04);

    /* Set Top Clock 2 */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x35, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x09, 0x00);

    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x9d, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x20);

    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x20, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x59, 0x87);

    /* Set Debugging Pin */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_TOP, 0x94, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_DSP_TOP, 0x00, 0x08);

    CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);
    WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xca, 0x08);

#endif /* defined(GTI_HMS) */

#elif (defined(GTI_DSPVIKING) )

    (void)nInit;

#if (defined(GTI_DSPTURBO))
    
    Gti_WriteVikingRegister(pXcvr, 0x08f3, 0x00410007);
    Gti_WriteVikingRegister(pXcvr, 0x08f3, 0x00410F07);
    Gti_WriteVikingRegister(pXcvr, 0x08f4, 0x00410007);
    Gti_WriteVikingRegister(pXcvr, 0x08f4, 0x00410F07);
    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x00410007);
    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x00410F07);

    Gti_WriteVikingRegister(pXcvr, 0x0880, 0x00003088);
    Gti_WriteVikingRegister(pXcvr, 0x086b, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x086c, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x086d, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x086e, 0x00000000);

    Gti_WriteVikingRegister(pXcvr, 0x0860, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x0861, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x0862, 0x00000000);
    Gti_WriteVikingRegister(pXcvr, 0x0863, 0x00000000);

    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x00840094);
    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x00790107);
    Gti_WriteVikingRegister(pXcvr, 0x08f1, 0x1000d00f);    
    Gti_WriteVikingRegister(pXcvr, 0x08f8, 0x0014ffff);

    Gti_WriteVikingRegister(pXcvr, 0x08f6, 0x00000061);
    Gti_WriteVikingRegister(pXcvr, 0x08f4, 0x00610107);
    Gti_WriteVikingRegister(pXcvr, 0x08f0, 0x70045c0c);

    Gti_WriteVikingRegister(pXcvr, 0x0927, 0x00000180);
    Gti_WriteVikingRegister(pXcvr, 0x08fd, 0x00000000);

    Gti_WriteVikingRegister(pXcvr, 0x08b0, 0x00040044);  /*UTOPIA Level 2*/
    Gti_WriteVikingRegister(pXcvr, 0x08b1, 0x1f031f03);  /*UTOPIA Level 2 */
    Gti_WriteVikingRegister(pXcvr, 0x08b8, 0x01000100);

#else

#ifdef GTI_HOSTVSC

    /* get the silicon version for the Viking Chip */
    SiliconVer = *(volatile unsigned int *)0x800020DC;

    /*
    ** Set the IREG 0x39 reg, POST Divider = 1/2
    ** This needs to be done by VSC before PLL3 clock settings
    */
    CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);
    WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
    WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x04);

    if(SiliconVer == 0x00810020)
    {
            /* Viking Bx */

            #ifndef GTI_MONACO_USB
                /* PLL3 */
                Gti_WriteVikingRegister(pXcvr, 0x08f3, 0x10007B07);
            #endif
 
            /* DSL1CK */
            Gti_WriteVikingRegister(pXcvr, 0x08f8, 0x000F0000);

            #ifdef GTI_CAFE_AFE
                /* DDLCK */
                Gti_WriteVikingRegister(pXcvr, 0x08f6, 0x00001840);
            #endif
        
            CONFIGURE_DSP(pXcvr, GTI_DSP_ALL,  GTI_NO_AI);
            WRITE_DSP_ADDR(pXcvr, GTI_DSP_ALL, 0x32, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_DSP_ALL, 0x10, 0x00); 
    }
    else
    {
            /* Viking Ax */

            #ifndef GTI_MONACO_USB
               Gti_WriteVikingRegister(pXcvr, 0x08f3, 0x00f47d09);
            #endif
 
            Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x000c8080);
 
            CONFIGURE_DSP(pXcvr, GTI_DSP_ALL,  GTI_NO_AI);
            WRITE_DSP_ADDR(pXcvr, GTI_DSP_ALL, 0x32, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_DSP_ALL, 0x10, 0x00);
 
            Gti_WriteVikingRegister(pXcvr, 0x08ff, 0x036000aa);
    }
   
#else /* !HostVSC */
#if (defined(GTI_DSPMONACO) )
    CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

#ifndef GTI_MONACO_USB
    Gti_WriteVikingRegister(pXcvr, 0x08f4, 0x00410007);  /*Reset the PLL0 charge-pump*/
    Gti_WriteVikingRegister(pXcvr, 0x08f4, 0x00410107);  /*Turn it back on*/

    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x00410007);  /*Reset the PLL1 charge-pump*/
    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x00410107);  /*Turn it back on*/
#endif
 
    Gti_WriteVikingRegister(pXcvr, 0x0861, 0x000000DB);
    Gti_WriteVikingRegister(pXcvr, 0x0862, 0x00000012);
 
    Gti_WriteVikingRegister(pXcvr, 0x08f0, 0x70041703);  /*PLL0 = 132.48 MHz + DDL function */
    Gti_WriteVikingRegister(pXcvr, 0x08f1, 0x10007d09);  /*PLL1 = 240 MHz */
    Gti_WriteVikingRegister(pXcvr, 0x08f6, 0x00000081);  /*DDLck = PLL0CK / 1 */
    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x008A0099);  /*DSL0CK = PLL1CK/2, NOVAICK=DSL0CK */
 
#ifndef GTI_MONACO_USB
    Gti_WriteVikingRegister(pXcvr, 0x08f9, 0x0039ffff);
#endif
 
#ifdef GTI_DSPMONACO
    Gti_WriteVikingRegister(pXcvr, 0x0908, 0x0017201c);  /*Map GPCK to MONTCK */
    Gti_WriteVikingRegister(pXcvr, 0x0834, 0x23232022);  /*pin out the const, map interrupt pin */
#else
    Gti_WriteVikingRegister(pXcvr, 0x0908, 0x00172020);  /*Map GPCK to MONTCK */
    Gti_WriteVikingRegister(pXcvr, 0x0834, 0x23232020);  /*pin out the const */
#endif
 
    Gti_WriteVikingRegister(pXcvr, 0x0844, 0x00acaaca);
//FIXMEKJP    Gti_WriteVikingRegister(pXcvr, 0x0850, 0x000000c0);
    Gti_WriteVikingRegister(pXcvr, 0x0851, 0x000000c0);
 
#ifndef GTI_MONACO_USB
    /*Gti_WriteVikingRegister(pXcvr, 0x08b4, 0x00000001);  UTOPIA reset */
    Gti_WriteVikingRegister(pXcvr, 0x08b0, 0x1f071f07);  /*UTOPIA Level 2 */
#else
    Gti_WriteVikingRegister(pXcvr, 0x08b4, 0x00000001);  /*UTOPIA reset */
    Gti_WriteVikingRegister(pXcvr, 0x08b0, 0x1f041f14);  /*UTOPIA Level 2 */
#endif
#if 0
    /*Gti_WriteVikingRegister(pXcvr, 0x0909, 0xdafd0000);  Enable the clocks */
#endif

#else /*  Viking and Not monaco */

    /* get the silicon version for the Viking Chip */
    SiliconVer =  Gti_ReadVikingRegister( pXcvr, 0x0837 );

    DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
 
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
 
    Gti_WriteVikingRegister(pXcvr, 0x08f2, 0x30fc1960); /* Set PLL2 to generate ?? from PLL3 ?? */

#ifndef GTI_MONACO_USB
    Gti_WriteVikingRegister(pXcvr, 0x08f3, 0x00f83209); /* Set PLL3 to generate ?? */
#endif

    Gti_WriteVikingRegister(pXcvr, 0x08f6, 0xe0048080); /* Set MPCK to select PLL1 */
 
    Gti_WriteVikingRegister(pXcvr, 0x08f5, 0x83800000);  /* Set SDRAMCK to select PLL1 */
 
#ifdef GTI_CAFE_AFE
    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x00088080);
#else
    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x04088080); /* 0x04 for div_2 */
#endif
 
    Gti_WriteVikingRegister(pXcvr, 0x08f0, 0x00f41703);
 
    Gti_WriteVikingRegister(pXcvr, 0x08f1, 0x00f47310);
 
    CONFIGURE_DSP(pXcvr, GTI_DSP_ALL,  GTI_NO_AI);
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_ADDR(pXcvr, GTI_DSP_ALL, 0x32, GTI_IREGA);
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_WORD(pXcvr, GTI_DSP_ALL, 0x10, 0x00);
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
 
 
    Gti_WriteVikingRegister(pXcvr, 0x08ff, 0x000000aa);
    Gti_WriteVikingRegister(pXcvr, 0x08ff, 0x006100aa);
    Gti_WriteVikingRegister(pXcvr, 0x08ff, 0x00e100aa);

    /* No constellation, aa-->2a constellation but no mem access */
    Gti_WriteVikingRegister(pXcvr, 0x08ff, 0x01e077aa);
 
    Gti_WriteVikingRegister(pXcvr, 0x08f6, 0xe0048080); /* Set MPCK to select from PLL1 */
   
#ifdef GTI_CAFE_AFE
    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x00088080);
#else
    Gti_WriteVikingRegister(pXcvr, 0x08f7, 0x04088080);
#endif
 
    Gti_WriteVikingRegister(pXcvr, 0x083a, 0x00014000); /* DSP Clock */
#endif /* !Monaco */
#endif /* !HOSTVSC */
 
 
    Gti_WriteVikingRegister(pXcvr, 0x086c, 0x0000ffff);
    Gti_WriteVikingRegister(pXcvr, 0x086b, 0xffffffff);
    Gti_WriteVikingRegister(pXcvr, 0x086d, 0x000000ff); /* Take IPU out of reset */
    Gti_WriteVikingRegister(pXcvr, 0x086e, 0x000000ff); /* TAke XDP out of reset */

#ifndef GTI_DSPMONACO
        if(SiliconVer == 0x00810020)
        {
		/* Viking Bx */
		Gti_WriteVikingRegister(pXcvr, 0x0861, 0x00000002); /* Enable VREG 0x63 */
		Gti_WriteVikingRegister(pXcvr, 0x0863, 0x00000084); /* Enable TDM & delta-sigma */
		Gti_WriteVikingRegister(pXcvr, 0x083e, 0x00000002);
		/*Gti_WriteVikingRegister(pXcvr, 0x0852, 0x00000dd0);  Enable ELD controls */
        }
        else
        {
                /* Viking Ax */
                Gti_WriteVikingRegister(pXcvr, 0x0861, 0x00000002); /* Enable VREG 0x63 */
                Gti_WriteVikingRegister(pXcvr, 0x0863, 0x00000010); /* Enable TDM & delta-sigma */
                Gti_WriteVikingRegister(pXcvr, 0x083e, 0x00000002);
                Gti_WriteVikingRegister(pXcvr, 0x0852, 0x00000dd0); /* Enable ELD controls */
        }
#endif  /* ! Monaco */
#endif  /*NOT TURBO*/


#elif (defined(GTI_ONEPLL) && defined(GTI_DSPQUAD))

   (void) nInit;
   CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

   switch(pXcvr->wAnnexSlctd)
   {
    case GTI_TYPE_ANNEX_C:
    
        /* Initialize MCK and External DMA clock to 80 MHz */
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x58, 0xa7);

        DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x69);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x05);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x01, 0x2c);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xa7);
        
        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xa7);
          
        break;

#ifdef GTI_PAGE_G_BUILT
    case GTI_TYPE_EREACH:
   
        /* Initialize MCK and External DMA clock to 80 MHz */
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x58, 0xa7);
 
        DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x69);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x05);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x01, 0x2c);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);
 
        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xa7);
 
        break;
#endif /* Page G - ereach */

    default:

        if(pXcvr->pChipInfo->bChipVersion == 0x4)
        {
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xe7);        
            
            DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
          
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x59);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x01);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x29, 0x35);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x20, 0x80);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 1msec */

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xe7);

        }
        else
        {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0xe0);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x00);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x5e, 0x4a);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xca, 0x48);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x29, 0x35);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1e, 0x75);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1c, 0xb6);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x20, 0x80);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x25, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xc0, 0x94);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x29, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x20, 0xb2);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x0d);

            DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xe7);
        }
        break;
    }    

#else /* !Octane, !Viking, !trinity&&1PLL */

   switch(pXcvr->wAnnexSlctd)
   {
    case GTI_TYPE_ANNEX_B:
      CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

      if(pXcvr->pChipInfo->bChipVersion == 0x4)
      {

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xe7);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x59);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x01);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x29, 0x35);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x20, 0x80);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

        DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xe7);

      }
      else
      {


          if(nInit)
          {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x58, 0xa7);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
          }

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x59);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x48);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x05);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x75);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

          DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

          WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
          WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x79, 0xa7);
        }

      break;

    case GTI_TYPE_ANNEX_C:

        CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

        if(nInit)
        {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x58, 0xa7);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
        }

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x69);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x05);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x01, 0x2c);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xa7);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xa7);

        break;

    case GTI_TYPE_ANNEX_H:

        CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

        if(nInit)
        {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xa7);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
        }

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x69);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x7e, 0x65);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xca, 0x48);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x0c, 0x90);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xa7);

        break;

#ifdef GTI_PAGE_G_BUILT
    case GTI_TYPE_EREACH:
 
        CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);
 
        if(nInit)
        {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x58, 0xa7);
 
            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
        }
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x69);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x05);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x01, 0x2c);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x00);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);
 
        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
 
        WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xa7);
 
        break;
#endif /* Page "G" - Ereach */

    default:

#if( defined(GTI_DSPMULTI) || defined(GTI_DSPPCI) ) /*Nova or PCI*/
        CONFIGURE_DSP(pXcvr, GTI_DSP_A,  GTI_NO_AI);

        if(nInit)
        {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x58, 0xa7);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
        }

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x2c, 0x59);
    
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x3a, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x5e, 0x48);
    
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x38, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0xba, 0x48);
    
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x37, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x00, 0x01);

        #if defined(GTI_DSPPCI)
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x32, 0x2c);
        #else
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x21, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x3a, 0x2c);
        #endif

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x22, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x1c, 0xb6);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x15, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x46, 0x43);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x18, 0x40);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x3d, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0xe5, 0x63);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x31, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x24, 0x42);

        DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x79, 0xe7);

#else /* ! Nova, ! PCI */
        CONFIGURE_DSP(pXcvr, GTI_ALL_XCVR_CORES,  GTI_NO_AI);

        if(pXcvr->pChipInfo->bChipVersion == 0x4)

        {

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xf9, 0xe7);        
            
            DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
          
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x59);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3e, 0x4b);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xda, 0x48);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x01);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, 0xb6);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x29, 0x35);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x10, 0x10);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x20, 0x80);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xa5, 0x63);

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 1msec */

            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xd9, 0xe7);            
        }
        else
        {

           if(nInit)
           {
            /* Initialize MCK and External DMA clock to 80 MHz */
            WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
            WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0xa0);

            DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
           }

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x39, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, 0x59);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x5e, 0x4a);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x38, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xba, 0x48);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x37, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x00, 0x01);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x21, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x3a, 0x2c);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x22, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1c, 0xb6);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2a, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x28, 0x35);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2b, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x08);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2c, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x18);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x2d, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x1a, 0x75);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x15, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x46, 0x43);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x32, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x18, 0x40);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3c, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x80, 0x8d);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x3d, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0xe5, 0x63);

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x31, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x24, 0x42);

           DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

           WRITE_DSP_ADDR(pXcvr, GTI_ALL_XCVR_CORES, 0x20, GTI_IREGA);
           WRITE_DSP_WORD(pXcvr, GTI_ALL_XCVR_CORES, 0x79, 0xe7);
        }
#endif /* !Nova */

        break;
   }

#endif /* !Octane, !Viking, !trinity&&1PLL */

    DELAY_GTI_PROCESS(10000); /* Delay in microsecs = 10msec */
}

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

/******************************************************************************/
/* Procedure: Gti_InitPointers                              Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will set all the Data Base pointers to the      */
/*              default tables according o the page number.                   */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
GTI_WORD Gti_InitPointers(GTI_SAS_INFO *pSASInfo, GTI_BYTE Page)
{

#ifndef GTI_BUILD_DSP_SEPERATE

    pSASInfo->ProjectIDPnt = Page; /* Save what Page is loaded      */
                            /* Init pointers to ROM table    */
    switch(Page)
    {
#if ( defined(GTI_PAGE_B_BUILT) || defined(GTI_PAGE_A_BUILT)  )
    case GTI_PAGE_A:
#ifdef GTI_MULTI_PROJ
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_a[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_a[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_a_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_a[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_a[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_a[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_a[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_a[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_a[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_a[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_a[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_a[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_a[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_a[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_a[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_a[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_a[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_a[0];
        pSASInfo->PntIPU = (GTI_BYTE  GTI_ADDRESS_QUAL *)&IPU_code_a[0];
        pSASInfo->PntAVM = (GTI_BYTE  GTI_ADDRESS_QUAL *)&AVM_code_a[0];
        pSASInfo->PntBE = (GTI_BYTE  GTI_ADDRESS_QUAL *)&BE_code_a[0];
        pSASInfo->PntTR= (GTI_BYTE  GTI_ADDRESS_QUAL *)&TR_code_a[0];
        pSASInfo->PntCOEF= (GTI_BYTE  GTI_ADDRESS_QUAL *)&COEF_code_a[0];
        pSASInfo->PntPCU= (GTI_BYTE  GTI_ADDRESS_QUAL *)&PCU_code_a[0];
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP= (GTI_BYTE  GTI_ADDRESS_QUAL *)&FDP_code_a[0];
#endif
#if (defined(GTI_XDP_EOC) )
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_a[0];
#endif
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_a[0];
#endif
#else /* Single Project */
 
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl[0];
        pSASInfo->PntHOSTL=(struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl[0];
        pSASInfo->PntIMML =(struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl[0];
 
        #if !defined(GTI_DSPOCTANE) && !(defined(GTI_DSPVIKING) && defined(GTI_PROD_SHDSL))
        pSASInfo->PntCmdPnt = (struct CMDPOINT GTI_ADDRESS_QUAL *)&CmdPnt_tbl[0];
        #else
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable[0];
        #endif
 
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code[0];
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code[0];
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code[0];
#endif /* Single Project */
        break;

#ifdef GTI_PAGE_B_BUILT
    case GTI_PAGE_B:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_b[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_b[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_b_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_b[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_b[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_b[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_b[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_b[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_b[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_b[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_b[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_b[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_b[0];
        pSASInfo->PntIMML =(struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_b[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_b[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_b[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_b[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_b[0];
        pSASInfo->PntIPU = (GTI_BYTE  GTI_ADDRESS_QUAL *)&IPU_code_b[0];
        pSASInfo->PntAVM = (GTI_BYTE  GTI_ADDRESS_QUAL *)&AVM_code_b[0];
        pSASInfo->PntBE = (GTI_BYTE  GTI_ADDRESS_QUAL *)&BE_code_b[0];
        pSASInfo->PntTR = (GTI_BYTE  GTI_ADDRESS_QUAL *)&TR_code_b[0];
        pSASInfo->PntCOEF = (GTI_BYTE  GTI_ADDRESS_QUAL *)&COEF_code_b[0];
#ifndef GTI_DSPSTAR
        pSASInfo->PntPCU= (GTI_BYTE  GTI_ADDRESS_QUAL *)&PCU_code_b[0];
#endif
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP= (GTI_BYTE  GTI_ADDRESS_QUAL *)&FDP_code_b[0];
#endif
        break;
#endif /* Page B */
#endif /* Page A or B Built */

    case GTI_PAGE_C:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_c[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_c[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_c_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_c[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_c[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_c[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_c[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_c[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_c[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_c[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_c[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_c[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_c[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_c[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_c[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_c[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_c[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_c[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_c[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_c[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_c[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_c[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_c[0];
#ifndef GTI_DSPSTAR
        pSASInfo->PntPCU= (GTI_BYTE  GTI_ADDRESS_QUAL *)&PCU_code_c[0];
#endif
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP= (GTI_BYTE  GTI_ADDRESS_QUAL *)&FDP_code_c[0];
#endif
#if (defined(GTI_DSPVIKING))
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_c[0];
#endif
        break;

#if ( defined(GTI_PROD_RADSLX) || defined(GTI_PAGE_D_BUILT)  )

    case GTI_PAGE_D:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_d[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_d[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_d_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_d[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_d[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_d[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_d[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_d[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_d[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_d[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_d[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_d[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_d[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_d[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_d[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_d[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_d[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_d[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_d[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_d[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_d[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_d[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_d[0];
#ifndef GTI_DSPSTAR
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_d[0];
#endif
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP= (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_d[0];
#endif
#if (defined(GTI_XDP_EOC) )
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_d[0];
#endif
        break;
#endif /* RADSLX or DMT_ANNEX_MERGE */

#if ( defined(GTI_PROD_RADSLX) || defined(GTI_PAGE_E_BUILT)  )
    case GTI_PAGE_E:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_e[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_e[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_e_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_e[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_e[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_e[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_e[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_e[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_e[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_e[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_e[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_e[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_e[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_e[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_e[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_e[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_e[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_e[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_e[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_e[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_e[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_e[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_e[0];
        pSASInfo->PntPCU = (GTI_BYTE  GTI_ADDRESS_QUAL *)&PCU_code_e[0];
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP = (GTI_BYTE  GTI_ADDRESS_QUAL *)&FDP_code_e[0];
#endif
        break;
#endif  /* RADSLX or DMT Page E (Annex H) */

#if ( defined(GTI_PROD_RADSLX) || defined(GTI_PAGE_F_BUILT)  )
    case GTI_PAGE_F:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_f[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_f[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_f_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_f[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_f[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_f[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_f[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_f[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_f[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_f[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_f[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_f[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_f[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_f[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_f[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_f[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_f[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_f[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_f[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_f[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_f[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_f[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_f[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_f[0];
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_f[0];
#endif
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_f[0];
#endif

        break;
#endif /* GTI_PROD_RADSLX or PAGE F */

#if ( defined(GTI_PAGE_G_BUILT)  )
    case GTI_PAGE_G:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_g[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_g[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_g_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_g[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_g[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_g[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_g[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_g[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_g[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_g[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_g[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_g[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_g[0];
        pSASInfo->PntIMML =(struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_g[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_g[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_g[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_g[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_g[0];
        pSASInfo->PntIPU = (GTI_BYTE  GTI_ADDRESS_QUAL *)&IPU_code_g[0];
        pSASInfo->PntAVM = (GTI_BYTE  GTI_ADDRESS_QUAL *)&AVM_code_g[0];
        pSASInfo->PntBE = (GTI_BYTE  GTI_ADDRESS_QUAL *)&BE_code_g[0];
        pSASInfo->PntTR = (GTI_BYTE  GTI_ADDRESS_QUAL *)&TR_code_g[0];
        pSASInfo->PntCOEF = (GTI_BYTE  GTI_ADDRESS_QUAL *)&COEF_code_g[0];
        pSASInfo->PntPCU= (GTI_BYTE  GTI_ADDRESS_QUAL *)&PCU_code_g[0];
#if (defined(GTI_DSP_FDP_ENG))
        pSASInfo->PntFDP= (GTI_BYTE  GTI_ADDRESS_QUAL *)&FDP_code_g[0];
#endif
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_g[0];
#endif
        break;
#endif /* Page G */

#if ( defined(GTI_PAGE_H_BUILT)  )
    case GTI_PAGE_H:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_h[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_h[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_h_tbl[0];
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_h[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_h_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_h[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_h[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_h[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_h[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_h[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_h[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_h[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_h[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_h[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_h[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_h[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_h[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_h[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_h[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_h[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_h[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_h[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_h[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_h[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_h[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_h[0];
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_h[0];
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_h[0];
#endif
        break;
#endif /* PAGE H */

#if ( defined(GTI_PAGE_I_BUILT)  )
    case GTI_PAGE_I:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_i[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_i[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_i_tbl[0];
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_i[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_i_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_i[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_i[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_i[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_i[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_i[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_i[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_i[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_i[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_i[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_i[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_i[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_i[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_i[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_i[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_i[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_i[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_i[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_i[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_i[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_i[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_i[0];
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_i[0];
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_i[0];
#endif
        break;
#endif /* PAGE I */

#if ( defined(GTI_PAGE_J_BUILT)  )
    case GTI_PAGE_J:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_j[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_j[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_j_tbl[0];
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_j[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_j_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_j[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_j[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_j[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_j[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_j[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_j[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_j[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_j[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_j[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_j[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_j[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_j[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_j[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_j[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_j[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_j[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_j[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_j[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_j[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_j[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_j[0];
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_j[0];
#if (GTI_NUM_CORES_PER_XCVR == 3)
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_j[0];
#endif
        break;
#endif /* PAGE J */
#if ( defined(GTI_PAGE_K_BUILT)  )
    case GTI_PAGE_K:
#ifdef GTI_HOST_SASSTATUS
        pSASInfo->PntStatus = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_StatusCmdTbl_k[0];
#endif
        pSASInfo->PntCmdPnt = (GTI_DWORD GTI_ADDRESS_QUAL *)&Gti_ActionLabelTable_k[0];
        pSASInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_k_tbl[0];
        pSASInfo->PntCmd = (struct BASIC_CMD GTI_ADDRESS_QUAL *)&CmdTable_k[0];
        pSASInfo->PntTime = (struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *)&Time_tbl_k[0];
        pSASInfo->PntDelay = (struct DELAY_STRUCT GTI_ADDRESS_QUAL *)&Delay_tbl_k[0];
        pSASInfo->PntFill = (struct FILL_STRUCT GTI_ADDRESS_QUAL *)&Fill_tbl_k[0];
        pSASInfo->PntFun = (struct FUN_STRUCT GTI_ADDRESS_QUAL *)&Fun_tbl_k[0];
        pSASInfo->PntMEML = (struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *)&MEML_tbl_k[0];
        pSASInfo->PntMEMR = (struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *)&MEMR_tbl_k[0];
        pSASInfo->PntREGL = (struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *)&REGL_tbl_k[0];
        pSASInfo->PntREGR = (struct REG_READ_STRUCT GTI_ADDRESS_QUAL *)&REGR_tbl_k[0];
        pSASInfo->PntHOSTL = (struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *)&HOSTL_tbl_k[0];
        pSASInfo->PntIMML = (struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&IMML_tbl_k[0];
        pSASInfo->PntIMMR = (struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *)&IMMR_tbl_k[0];
        pSASInfo->PntREGInd = (struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *)&REGLInd_tbl_k[0];
        pSASInfo->PntINDL = (struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *)&INDL_tbl_k[0];
        pSASInfo->PntIF = (struct IF_STRUCT GTI_ADDRESS_QUAL *)&IF_tbl_k[0];
        pSASInfo->PntIPU = (GTI_BYTE GTI_ADDRESS_QUAL *)&IPU_code_k[0];
        pSASInfo->PntAVM = (GTI_BYTE GTI_ADDRESS_QUAL *)&AVM_code_k[0];
        pSASInfo->PntBE = (GTI_BYTE GTI_ADDRESS_QUAL *)&BE_code_k[0];
        pSASInfo->PntTR = (GTI_BYTE GTI_ADDRESS_QUAL *)&TR_code_k[0];
        pSASInfo->PntCOEF = (GTI_BYTE GTI_ADDRESS_QUAL *)&COEF_code_k[0];
        pSASInfo->PntPCU = (GTI_BYTE GTI_ADDRESS_QUAL *)&PCU_code_k[0];
        pSASInfo->PntFDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&FDP_code_k[0];
        pSASInfo->PntXDP = (GTI_BYTE GTI_ADDRESS_QUAL *)&XDP_code_k[0];
        break;
#endif /* PAGE K */

 
    default:
        /* ERROR- Should never get here */
        return(GTI_FAIL);
    }
 
    return(GTI_OK);


#else /* Seperate Binary */
 
   GTI_BYTE btmp1;
   GTI_WORD i,itmp1;
   GTI_DWORD dtmp1;
   GTI_BYTE GTI_ADDRESS_QUAL * Ptr;
   GTI_BYTE GTI_ADDRESS_QUAL * ProjectPtr;
 
    btmp1 = 0;
    ProjectPtr = (GTI_BYTE GTI_ADDRESS_QUAL *)(NULL);

       /* Init pointers to ROM table    */
       /*printf("In InitPointers\n");*/
       /* If current project = requested project then skip loading pointers */
       /***** Always reload pointers ****/
 
       /* Search for matching project */
       /* 32 projects - 8 words       */
       /*printf("\n\nLooking for matching page:=0x%x\n", Page);*/
       /*printf("Base Address=0x%x\n", BinaryBaseAddress);*/
       ProjectPtr = (GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress);
       btmp1 = GTI_PAGE_NONE;

       for( i =0; i< 32; i++ ){
                Ptr =  (GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress+(i*GTI_PROJECT_OFFSET));
 
                /* Read the location/address of project - 32 bits */
                dtmp1 = *Ptr;
                dtmp1 |= *(Ptr+1)<<8;
                dtmp1 |= *(Ptr+2)<<16;
                dtmp1 |= *(Ptr+3)<<24;
                /*printf("Location of project=0x%x, Value=0x%x\n", Ptr, dtmp1);*/
 
                /* If address = 0x0 then no project loaded */
                if( dtmp1 != 0x000 )
                {
                      /* does it match the project requested */
                      ProjectPtr = (GTI_BYTE GTI_ADDRESS_QUAL *)(dtmp1+BinaryBaseAddress);
                      btmp1 = *(ProjectPtr);
                      /*printf("Project Start Location=0x%x\n", ProjectPtr);*/
 
                      if( btmp1 == Page || Page==GTI_PAGE_NONE ) break;
                }
 
       }

       if( i== 32 || (ProjectPtr == (GTI_BYTE GTI_ADDRESS_QUAL *)(NULL) ) )
       {
            /* No matching project!!! */
            /*printf("No matching project loaded!!\n");*/
            return(GTI_FAIL);
       }

 
       /* Use Page and Base  to get Project Address */
       /* Project Address = (((Page+1)*8) +0x50+Base); - 4 bytes */
 
       /* Check to see if Project is empty */
       /* Size = 32 bits */
       /* Length != 0 */
       dtmp1 = *(ProjectPtr+GTI_PROJECT_LENGTH_OFFSET+0);
       dtmp1 |= *(ProjectPtr+GTI_PROJECT_LENGTH_OFFSET+1)<<8;
       dtmp1 |= *(ProjectPtr+GTI_PROJECT_LENGTH_OFFSET+2)<<16;
       dtmp1 |= *(ProjectPtr+GTI_PROJECT_LENGTH_OFFSET+3)<<24;
       /*printf("Project Length 0x%x=0x%x\n", (ProjectPtr+GTI_PROJECT_LENGTH_OFFSET), dtmp1 );*/
       if( dtmp1 == 0x00)
       {
               return(GTI_FAIL);
       }
       /* Check to see if #of Table Pointers      */
       /* Length != 0 */
       itmp1 = *(ProjectPtr+GTI_TRANSLATION_LENGTH_OFFSET+0);
 
       itmp1 |= (*(ProjectPtr+GTI_TRANSLATION_LENGTH_OFFSET+1))<<8;
       if( itmp1 == 0x00)
       {
               return(GTI_FAIL);
       }

       dtmp1 = *(ProjectPtr+GTI_PROJECT_COMPATIABLITY_OFFSET+0);
       dtmp1 |= *(ProjectPtr+GTI_PROJECT_COMPATIABLITY_OFFSET+1)<<8;
       if( dtmp1 != 0 )
       {
            /* If compatability Version != 0                                        */
            /* Check that project does not exceed the RAMSize allocated by the host */
            /* Size = 16 bit */
            dtmp1 = *(ProjectPtr+GTI_PROJECT_HOSTRAM_SIZE_OFFSET+0);
            dtmp1 |= *(ProjectPtr+GTI_PROJECT_HOSTRAM_SIZE_OFFSET+1)<<8;

            /* If RAM allocated is >= RAM needed then continue */
            if( (GTI_RAMSize+GTI_RAM_BUFFER) < dtmp1 )
            {
                  return(GTI_INCOMPATIABLE_BINARY);
            }
       }

       /* Check that Parameters Version is compatiable */
       /* If Parameter version in binary == 0 || 0xffff*/
       /* Then binary does not support parameter version*/
       dtmp1 = *(ProjectPtr+GTI_PROJECT_HOSTRAMVERSION_OFFSET+0);
       dtmp1 |= *(ProjectPtr+GTI_PROJECT_HOSTRAMVERSION_OFFSET+1)<<8;
       if( dtmp1 != 0 && dtmp1 != 0xffff )
       {
             if( dtmp1 != GTI_DMT_HOSTRAM_VERSION )
             {
                  return(GTI_INCOMPATIABLE_VERSION);
             }
       }

       /* ProjectPtr = is pointing to project header for selected project */
       pSASInfo->ProjectIDPnt = btmp1; /* Save what Page is loaded      */
       /*printf("BinaryProjectID = 0x%x\n",btmp1);*/

       pSASInfo->ProjectPointer =  ProjectPtr;
 
       i=0;
       pSASInfo->PntCmdPnt =  (GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntYAT =(GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntStatus =(GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntCmd =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntTime =    (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntDelay =   (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntFill =    (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntFun =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntMEML =    (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntMEMR =    (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntREGL =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntREGR =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntHOSTL =    (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntIMML =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntIMMR =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntREGInd =   (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntINDL =     (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntIF =       (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntIPU =      (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntAVM =      (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntBE =       (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntXDP =      (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntTR=        (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntCOEF=      (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntPCU=       (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
       i+=4;
       pSASInfo->PntFDP=       (GTI_BYTE GTI_ADDRESS_QUAL *) (BinaryBaseAddress +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+0)))+
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+1))<<8) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+2))<<16) +
                                         ((*(ProjectPtr+GTI_TABLE_OFFSET+i+3))<<24)
                                         );
 
       /* Make sure not loading more or less Tables then what is in Binary */
       /*printf("#of Tables 0x%x vs #Loaded 0x%x\n", itmp1, ((i+4)/4) );*/
       if( itmp1 != ((i+4)/4) )
       {
          return(GTI_INCOMPATIABLE_BINARY);
       }
 
 
       return(GTI_OK);
 
#endif /* Sperate Binary */
 
 
}
 
#endif /* Host Mode Only */





/**/
/******************************************************************************/
/*  Procedure: Gti_InitDSPInterrupt                                           */
/* 1) Use compile flags INTERR_XXXX_ACT  and INTERR_OD                        */
/*    (to set polarity see gtidsp.h).                                         */
/* 2) To Enable/Disable Interrupt call Gti_InitDSPInterrupt                   */
/* 3) To Clear the Interrupt call Gti_ClearDSPInterrupt                       */
/*    It must be Cleared by the user .            (i.e. in the                */
/*    interrupt routine)                                                      */
/*                                                                            */
/*             !!!!!!!!!WARNING!!!!!!!!!!!!!!!                                */
/* While ExecuteXCVR is EXECUTING it is NOT allowed for any                   */
/* other routine to access the DSP     i.e. GetStatus,                        */
/* Clear Interrupt                                                            */
/******************************************************************************/
void
Gti_InitDSPInterrupt( GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE state )
{

#if ( defined(GTI_DSPMULTI)  || defined(GTI_DSPPCI) )
    /* Set Polarity and Open drain Mode  */
    WRITE_DSP_REG(pXcvrInfo, GTI_INTERRUP_DSP, GTI_INT_POLARITY);
#endif /* GTI_DSPMULTI || GTI_DSPPCI */
#if ( defined(GTI_DSPMONACO)  || defined(GTI_DSPTURBO) )
    /* Set Polarity and Open drain Mode  */
    WRITE_DSP_REG(pXcvrInfo, GTI_INT_HOST_IO_DSP, GTI_INT_POLARITY);
#endif

    /* Initialize IPU interrupt of the STARLET             */
    if(state == GTI_TRUE )
    {
        /* Clear FLAG before enabling */
        ClearSTARInterrupt(pXcvrInfo, GTI_ALL_XCVR_CORES);
        WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_MASKTEST_DSP, GTI_IRQ_MASK);

        pXcvrInfo->IrEnabled = GTI_TRUE;

#ifdef GTI_DSPSTAR
        /* Only STAR uses Interrupt config in CORE register */
        CONFIGURE_DSP(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_NO_AI);
#endif

#if ( defined(GTI_DSPQUAD) && defined(GTI_PROD_DMT) && defined(GTI_EOC_PCU) )
        /* 1 API call inits IPU and Framer Interrupts */

        /* Clear Framer Interrupt */
        WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_CLR_DSP, 0x01);

        /* Enable Framer Interrupt */
        WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_INTERRUP_DSP, 0xfe);

        /* Change so "B" interrupts come out on "A" pin */
        WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_GCR_DSP, 0x6e);

#endif /* DMT and QUAD and EOC PCU */

    }
    else
    {
        WRITE_DSP_BYTE(pXcvrInfo,GTI_ALL_XCVR_CORES, GTI_MASKTEST_DSP, 0xff);

#if ( defined(GTI_DSPQUAD) && defined(GTI_PROD_DMT) && defined(GTI_EOC_PCU) )
        /* 1 API call inits IPU and Framer Interrupts */
        /* Disable Framer Interrupt */
        WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_INTERRUP_DSP, 0xff);
#endif /* DMT and QUAD and EOC PCU */


        pXcvrInfo->IrEnabled = GTI_FALSE;
    }

    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_ClearDSPInterrupt                         Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will clear the DSP interrupts.                  */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void
Gti_ClearDSPInterrupt(GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE id)
{
    GTI_BYTE bAddress;
    GTI_BYTE bData;

#if ( defined( GTI_DSPOCTANE) || defined(GTI_DSPMONACO) || defined(GTI_DSPTURBO) )
    bAddress = GTI_INT_CLEAR_DSP;
    bData = GTI_IPUINT_CLEAR;
#else
    bAddress = GTI_INTERRUP_DSP;
    bData = GTI_IPURQST;
#endif

    /* Clear IPU Interrupt Request           */
    WRITE_DSP_BYTE(pXcvrInfo, id, bAddress, bData);


#if ( defined(GTI_DSPQUAD) && defined(GTI_PROD_DMT) && defined(GTI_EOC_PCU) )
    /* Clear Framer Interrupt */
    WRITE_DSP_BYTE(pXcvrInfo, GTI_ALL_XCVR_CORES, GTI_PCU_CLR_DSP, 0x01);
#endif /* DMT and QUAD and EOC PCU */



    UPDATE_DSP(pXcvrInfo);
}

#if(defined(GTI_DSPFRAMER) || defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE))
#if defined(GTI_HMS) || defined(GTI_PROD_MSDSL)
/**/
/******************************************************************************/
/*  Procedure: Gti_InitFRAMERInterrupt                                        */
/* 1) Use compile flags and INTERR_XXXX_ACT and INTERR_OD                     */
/*    (to set polarity see gtidsp.h).                                         */
/* 2) To Enable/Disable Interrupt call InitFRAMERInterrupt                    */
/* 3) To Clear the Interrupt call ClearFRAMERInterrupt                        */
/*                                                                            */
/*             !!!!!!!!!WARNING!!!!!!!!!!!!!!!                                */
/* While ExecuteXCVR is EXECUTING it is NOT allowed for any                   */
/* other routine to access the DSP i.e. GetStatus,                            */
/* Clear Interrupt                                                            */
/******************************************************************************/
void
Gti_InitFRAMERInterrupt( GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE state)
{
    /* Initialize PCU interrupt of the DSP                 */
    if(state == GTI_FRM_DISABLE_INT)
    {
        Gti_ClearFRAMERInterrupt(pXcvrInfo, 0xff);
    }

#if defined(GTI_DSPOCTANE)
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_A, GTI_INT_MASKS_DSP, state );
#else
    WRITE_DSP_REG(pXcvrInfo, GTI_PCU_INTERRUP_DSP, state);
#endif

    UPDATE_DSP(pXcvrInfo);
}

/******************************************************************************/
/* Procedure: Gti_ClearFRAMERInterrupt                      Created: 31.05.00 */
/* ------------------------                                                   */
/* Description: The procedure will clear the FRAMER interrupts.               */
/* Returns VOID                                                               */
/* -------------------------------------------------------------------------- */
void
Gti_ClearFRAMERInterrupt(GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE type)
{

 /* Clear Framer Interrupt Request           */

#ifndef GTI_DSPOCTANE
    WRITE_DSP_REG(pXcvrInfo, GTI_PCU_CLR_DSP, type);
#else
    WRITE_DSP_BYTE(pXcvrInfo, GTI_DSP_A, GTI_INT_CLEAR_DSP, type & 0x0f);
#endif

    UPDATE_DSP(pXcvrInfo);
}
#endif /* GTI_HMS */
#endif

#ifdef GTI_TRACE
/***************************************************************************/
/*  Name: Gti_EnableTraceOutput                                            */
/* ------------------------                                                */
/*  Usage:  Allow the user to individually enable/disable the ability      */
/*          of each XCVR to write to the Trace History buffer              */
/*  Default: On power-up BuildXCVR enables TRACE for ALL XCVRs.            */
/*  Input:                                                                 */
/*         pointer to an GTI_XCVR_INFO structure - XCVR system selected    */
/*         state: GTI_TRUE or GTI_FALSE                                    */
/* ------------------------                                                */
/*  Caveats:                                                               */
/***************************************************************************/
void
Gti_EnableTraceOutput( GTI_XCVR_INFO *pXcvrInfo, GTI_WORD state)
{
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    if(state == GTI_TRUE)
         pXcvrInfo->CmdInfo.enableTRACE = GTI_TRUE;
    else
         pXcvrInfo->CmdInfo.enableTRACE = GTI_FALSE;
#else
    (void) pXcvrInfo;
    (void) state;
#endif
}

/****************************************************************************/
/* Used by GSI to update History buffer with API information                */
/****************************************************************************/
void
Gti_UpdateTrace( GTI_WORD type, GTI_WORD value )
{
#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
    if((HistPnt < GTI_SIZE_OF_TRACE) )
    {
            History[0][HistPnt]=0xdeb0;              /* "DEB"ug */
            History[1][HistPnt]= value;
            History[2][HistPnt]= type;
            ++HistPnt;
    }
#else
    (void) type;
    (void) value;
#endif
}
#endif

#if defined(GTI_DSPQUAD)
/*****************************************************************************/
/* Name:    Gti_ShutDwnXcvr                                                  */
/* Input:                                                                    */
/* pSysPtr: System pointer                                                   */
/* Description: The procedure will allow customers to disable unused cores   */
/*              or during IDLE mode, reduce power consumption.               */
/*                                                                           */
/*****************************************************************************/
GTI_WORD Gti_ShutDwnXcvr(GTI_XCVR_INFO* pXcvr, GTI_BYTE bMask)
{
    GTI_BYTE bLnDr, bAfe, bClk;

    bLnDr = (GTI_BYTE)(bMask & GTI_LNDR_OFF_MASK);
    bAfe  = (GTI_BYTE)((bMask & GTI_AFE_OFF_MASK) >> 1);
    bClk  = (GTI_BYTE)((bMask & GTI_HDCLK_OFF_MASK) >> 2);

    if(bLnDr)
    {
        CONFIGURE_DSP(pXcvr, GTI_DSP_A,  GTI_NO_AI);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x07, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x00, 0x3c);
    }
    if(bAfe)
    {
        CONFIGURE_DSP(pXcvr, GTI_DSP_A,  GTI_NO_AI);
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x3c, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x80, 0x8d);

#if defined(GTI_JANUS_AFE)
#if defined(GTI_JUPITER_AFE)

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x10, 0x01);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x80, 0x02);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x80, 0x03);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x80, 0x04);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x30, 0x06);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x08, 0x08);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x40, 0x0a);

        DELAY_GTI_PROCESS(60);
#else
        /*do not change more than 6bits at a time*/
        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x0f, 0x09);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0xff, 0x09);

        DELAY_GTI_PROCESS(60);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x34, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x81, 0x0b);
#endif
#endif
    }
    if(bClk)
    {
        /* Move the RS clock to the first core */
#ifdef GTI_DLC_APPLICATION
        WRITE_DSP_REG(pXcvr, GTI_GLOBAL_74, 0x20);
#else
        WRITE_DSP_REG(pXcvr, GTI_GLOBAL_74, 0x30);
#endif
        CONFIGURE_DSP(pXcvr, GTI_DSP_A,  GTI_NO_AI);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0xa0, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x45, 0x80);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x20, 0x20);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x32, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x08, 0x00);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x39, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x00, 0x00);

        WRITE_DSP_ADDR(pXcvr, GTI_DSP_A, 0x20, GTI_IREGA);
        WRITE_DSP_WORD(pXcvr, GTI_DSP_A, 0x03, 0x20);
    }

    return GTI_OK;
}

#endif /* Quad */

#ifdef GTI_DSPVIKING
#if ((defined(GTI_HOSTVSC)))
void Gti_WriteVikingRegister(GTI_XCVR_INFO* pXcvr, GTI_WORD addr, GTI_DWORD value)
{
    WRITE_DSP_REG(pXcvr, 0x0c, (addr&0xff)); /*SAP Address */
    WRITE_DSP_REG(pXcvr, 0x0d, ((addr>>8)&0xff));
    WRITE_DSP_REG(pXcvr, 0x1a, (value&0xff)); /* no constellation */
    WRITE_DSP_REG(pXcvr, 0x1b, ((value>>8)&0xff));
    WRITE_DSP_REG(pXcvr, 0x1e, ((value>>16)&0xff));
    WRITE_DSP_REG(pXcvr, 0x1f, ((value>>24)&0xff));
}
#else
void Gti_WriteVikingRegister(GTI_XCVR_INFO* pXcvr, GTI_WORD addr, GTI_DWORD value)
{

    WRITE_DSP_REG(pXcvr, 0x0c, (addr&0xff)); /*SAP Address */
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x0d, ((addr>>8)&0xff));
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x1a, (value&0xff)); /* no constellation */
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x1b, ((value>>8)&0xff));
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x1e, ((value>>16)&0xff));
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x1f, ((value>>24)&0xff));
    DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
}
GTI_DWORD
Gti_ReadVikingRegister(GTI_XCVR_INFO* pXcvr, GTI_WORD addr )
{
    GTI_DWORD result;

    result =0;
    
    /* SAP */
    WRITE_DSP_REG(pXcvr, 0x0c, (addr&0xff)); /*SAP Address */
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    WRITE_DSP_REG(pXcvr, 0x0d, (((addr>>8)&0xff)|0x80));    /* Set to Read!!! */
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

    /* Read */
    result = (READ_DSP_REG(pXcvr, 0x1a)&0x000000ff); 
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    result |= (((READ_DSP_REG(pXcvr, 0x1b)) << 0x08)&0x0000ff00); 
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    result |= (((READ_DSP_REG(pXcvr, 0x1e)) << 16) & 0x00ff0000); 
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */
    result |= (((READ_DSP_REG(pXcvr, 0x1f)) << 24) & 0xff000000); 
      DELAY_GTI_PROCESS(1000); /* Delay in microsecs = 1msec */

   return(result);
}
#endif /* !HostVSC */
#endif /* Viking */

#ifdef GTI_DSPTURBO
GTI_WORD
GtiGetCoreMask(GTI_XCVR_INFO* pXcvrInfo, GTI_BYTE id)
{
  GTI_WORD bMask;

  if (GET_HOST_DB_B(pXcvrInfo, GTI_TURBO_CORE_POLARITY) == 0)
    {
      switch(id)
      {
      case GTI_DSP_TOP:
          bMask = GTI_CORE_TOP_MSK;
          break;
      case GTI_ALL_CHIP_CORES:   /* All cores associated with CHIP */
          bMask = GTI_CORE_AB_MSK;
          break;
      case GTI_DSP_DMA:
          bMask = 0;
          break;
      case GTI_DSP_B:
          bMask = GTI_CORE_B_MSK;
          break;
      case GTI_DSP_CORE_A:
          bMask = GTI_CORE_A_MSK;
          break;
      case GTI_DSP_CORE_B:
          bMask = GTI_CORE_B_MSK;
          break;
      case GTI_ALL_XCVR_CORES: /* All cores associated with XCVR */
          bMask = GTI_CORE_AB_MSK;
          break;
      case GTI_DSP_A:
      default:
          bMask = GTI_CORE_A_MSK;
          break;
      }
    }
  else
    {
      switch(id)
      {
      case GTI_DSP_TOP:
          bMask = GTI_CORE_TOP_MSK;
          break;
      case GTI_ALL_CHIP_CORES:   /* All cores associated with CHIP */
          bMask = GTI_CORE_AB_MSK;
          break;
      case GTI_DSP_DMA:
          bMask = 0;
          break;
      case GTI_DSP_B:
          bMask = GTI_CORE_A_MSK;
          break;
      case GTI_DSP_CORE_A:
          bMask = GTI_CORE_A_MSK;
          break;
      case GTI_DSP_CORE_B:
          bMask = GTI_CORE_B_MSK;
          break;
      case GTI_ALL_XCVR_CORES: /* All cores associated with XCVR */
          bMask = GTI_CORE_AB_MSK;
          break;
      case GTI_DSP_A:
      default:
          bMask = GTI_CORE_B_MSK;
          break;
      }
    }

  return bMask;

}


GTI_WORD
GtiTranslateRegister(GTI_XCVR_INFO* pXcvrInfo, GTI_WORD reg, GTI_BYTE id)
{

  #ifdef GS_INCLUDE_GROUCHO
    reg=reg<<2;
  #endif

  if (GET_HOST_DB_B(pXcvrInfo, GTI_TURBO_CORE_POLARITY) == 0)
    {
      switch(id)
      {
        case GTI_DSP_TOP:
#ifdef GS_INCLUDE_GROUCHO
          reg &=  0x7f;
#else          
          reg &=  0x1f;
#endif          
          break;
        case GTI_DSP_DMA:
          break;
        case GTI_DSP_B:      /* Relative B */
        case GTI_DSP_CORE_B: /* Physical B */
          reg = GtiTranslatePhysicalB(reg);
          break;
        case GTI_ALL_XCVR_CORES: /* All cores associated with XCVR */
        case GTI_ALL_CHIP_CORES:   /* All cores associated with CHIP */
          reg = GtiTranslatePhysicalAB(reg);
          break;
        case GTI_DSP_A:      /* Relative A */
        case GTI_DSP_CORE_A: /* Physical A */
        default:  
          reg = GtiTranslatePhysicalA(reg);
          break;
      }
    }
  else
    {
      switch(id)
      {
        case GTI_DSP_TOP:
          reg &=  0x1f;
          break;
        case GTI_DSP_DMA:
          break;
        case GTI_DSP_B:        /* Relative B */
        case GTI_DSP_CORE_A:   /* Physical A */
          reg = GtiTranslatePhysicalA(reg);
          break;
        case GTI_ALL_XCVR_CORES: /* All cores associated with XCVR */
        case GTI_ALL_CHIP_CORES:   /* All cores associated with CHIP */
          reg = GtiTranslatePhysicalAB(reg);
          break;
        case GTI_DSP_A:      /* Relative A */
        case GTI_DSP_CORE_B: /* Physical B */
        default:  
          reg = GtiTranslatePhysicalB(reg);
          break;
      }
    }

  return reg;
}
GTI_WORD GtiTranslatePhysicalA( GTI_WORD reg )
{
         /* Physical Core "A" on DSP  */
         switch(reg)
         {
                case GTI_ADDL_DSP:
                case GTI_ADDH_DSP:
                case GTI_EXT_DATA_DSP:
                case GTI_DATAL_DSP:
                case GTI_DATAH_DSP:
                 return (reg | GTI_CORE_A_MSK);
                default:
                /* All other registers are set for "A" in the #defines */
                 return reg;
         }
         
}
GTI_WORD 
GtiTranslatePhysicalB(GTI_WORD reg)
{
    /* Physical Core "B" on DSP  */
    switch(reg)
    {
          case GTI_ADDL_DSP:
          case GTI_ADDH_DSP:
          case GTI_EXT_DATA_DSP:
          case GTI_DATAL_DSP:
          case GTI_DATAH_DSP:
             return (reg | GTI_CORE_B_MSK);
          case 0x01:
          case 0x02:
          case 0x03:
          case 0x04:
#ifdef GS_INCLUDE_GROUCHO
             return(reg + (5<<2));
#else
             return(reg + 5);
#endif          
          case 0x05:
             return(0x10);
          default:
             return(reg);
    }
}
GTI_WORD 
GtiTranslatePhysicalAB(GTI_WORD reg)
{
    
   switch(reg)
   {
        case GTI_ADDL_DSP:
        case GTI_ADDH_DSP:
        case GTI_EXT_DATA_DSP:
        case GTI_DATAL_DSP:
        case GTI_DATAH_DSP:
          return (reg | GTI_CORE_AB_MSK);
        default:
          return reg;
   }
}
#endif /* Turbo */

#ifdef GTI_BUILD_DSP_SEPERATE
/******************************************************************************/
/* Procedure: Gti_SetBinaryBaseAddress                      Created: 17.04.03 */
/* ------------------------                                                   */
/* Description: Set the location of the Binary                                */
/*              Some validation of the Binary is done                         */
/* Returns:                                                                   */
/*         GTI_OK - Binary is good                                            */
/*         GTI_FAIL - Could not find any good sub-projects                    */
/*         GTI_INCOMPATIABLE_BINARY - Binary is higher version then API code  */
/*                                    knows how to translate                  */
/* -------------------------------------------------------------------------- */
GTI_WORD
Gti_SetBinaryBaseAddress(
   GTI_XCVR_INFO *pXcvrInfo,   /* A pointer to the built XCVR Data Base      */
   GTI_CONST_QUAL GTI_BYTE GTI_ADDRESS_QUAL *pntBinary
)
{
    BinaryBaseAddress = pntBinary;
    return( Gti_InitPointers(&pXcvrInfo->CmdInfo, GTI_PAGE_NONE) );
}

/******************************************************************************/
/* Procedure: Gti_ReportSupportedAnnex                      Created: 21.04.03 */
/* ------------------------                                                   */
/* Description: Report all Annexes supported by the binary                    */
/* Input:                                                                     */
/*         pResultAnnex - pointer to array of structure to store results      */
/*         maxReportedAnnexes - number of array elements                      */
/* Returns:                                                                   */
/*         # - The number of Annexes found in binary                          */
/*         0 - No annexes found in binary                                     */
/* -------------------------------------------------------------------------- */
GTI_WORD
Gti_ReportSupportedAnnexes(
                 GTI_SUPPORTED_ANNEX_STATUS *pResultAnnex,
                 GTI_WORD maxReportedAnnexes )
{

     GTI_WORD itmp,i;
     GTI_BYTE btmp1;
     GTI_DWORD dtmp1;
     GTI_BYTE GTI_ADDRESS_QUAL * Ptr;
     GTI_BYTE GTI_ADDRESS_QUAL * ProjectPtr;



     for( i =0, itmp=0; i< 32; i++ ){
           Ptr =  (GTI_BYTE GTI_ADDRESS_QUAL *)(BinaryBaseAddress+(i*GTI_PROJECT_OFFSET));

           /* Read the location/address of project - 32 bits */
           dtmp1 = *Ptr;
           dtmp1 |= *(Ptr+1)<<8;
           dtmp1 |= *(Ptr+2)<<16;
           dtmp1 |= *(Ptr+3)<<24;
           /*printf("Location of project=0x%x, Value=0x%x\n", Ptr, dtmp1);*/

           /* If address = 0x0 then no project loaded */
           if( dtmp1 != 0x000 )
           {
               /* Save the Annex and Version */
               ProjectPtr = (GTI_BYTE GTI_ADDRESS_QUAL *)(dtmp1+BinaryBaseAddress);
               btmp1 = *(ProjectPtr);

               pResultAnnex->annexType =btmp1;     /* Save Annex */
               /* Save version */
               memcpy(&(pResultAnnex->version[0]), (ProjectPtr+GTI_PROJECT_VERSION_OFFSET), GTI_FW_REL_LEN);
               ++itmp;
               ++pResultAnnex;                      /* increament to next array element */
               if( itmp >= maxReportedAnnexes ) break;
           }

     }

     return(maxReportedAnnexes);
}
#endif /* GTI_BUILD_DSP_SEPERATE */
